

import pyflow_acdc as pyf
import pandas as pd
"""
Converted to PyFlowACDC format from

%CASE_ACTIVSG2000  Synthetic Texas 2000-bus power system model.
%
%   This is an entirely synthetic 2000 bus case, geographically situated
%   in the US state of Texas. The case is designed with a 500, 230, 161
%   and 115 kV transmission network to serve a load that roughly mimics
%   the actual population of its geographic footprint. The synthetic
%   transmission system was designed by algorithms described in [1] to be
%   statistically similar to actual transmission system models but without
%   modeling any actual lines.
%
%   When publishing results based on this data, please cite:
%
%   [1] A.B. Birchfield, T. Xu, K.M. Gegner, K.S. Shetye, T.J. Overbye,
%       "Grid Structural Characteristics as Validation Criteria for
%       Synthetic Networks," IEEE Transactions on Power Systems,
%       vol. 32, no. 4, pp. 3258-3265, July 2017.
%       doi: 10.1109/TPWRS.2016.2616385
%
%   This is a synthetic power system model that does not represent the
%   actual grid. It was developed as part of the US ARPA-E GRID DATA
%   research project and contains no CEII.
%
%   One-line diagrams and other data formats available at:
%       https://electricgrids.engr.tamu.edu
%
%   August 9, 2018

%   Created from ACTIVSg2000_AUG-09-2018.pwb, saved by
%   PowerWorld Simulator, version 21, build date August 30, 2018,
%   then by MATPOWER 6.

%   MATPOWER
%   Copyright (c) 2017-2018 by A.B. Birchfield, T. Xu, K.M. Gegner, K.S. Shetye,
%   and T.J. Overbye
%   Licensed under the Creative Commons Attribution 4.0 International license,
%   https://creativecommons.org/licenses/by/4.0/


"""

def case_ACTIVSg2000():    
    
    S_base=100
    
    # DataFrame Code:
    nodes_AC_data = [
        {'type': 'PQ', 'Voltage_0': 0.9839336, 'theta_0': -0.39525316161950763, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2078, 'Reactive_load': 0.058899999999999994, 'Node_id': '1001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0229021, 'theta_0': -0.31646311742490435, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.15410000000000001, 'Reactive_load': 0.0437, 'Node_id': '1002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.012931, 'theta_0': -0.29691008909505673, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.017406, 'theta_0': -0.3289724332260158, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0101076, 'theta_0': -0.28098573990644554, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0036973, 'theta_0': -0.2433588148812305, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.32292483246127046, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0701, 'Reactive_load': 0.0199, 'Node_id': '1007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0008, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0242318, 'theta_0': -0.15272966131828158, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0142528, 'theta_0': -0.09577132987163721, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.29424980588928945, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0219, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0302222, 'theta_0': -0.2637389547435982, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168192, 'theta_0': -0.4416489524193, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.029900000000000003, 'Reactive_load': 0.0085, 'Node_id': '1012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.095, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0095823, 'theta_0': -0.26962204077305063, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2923, 'Reactive_load': 0.0828, 'Node_id': '1013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.015959, 'theta_0': -0.2088648431298079, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0222, 'Reactive_load': 0.0063, 'Node_id': '1014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0319511, 'theta_0': -0.31861678390869524, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0751, 'Reactive_load': 0.0213, 'Node_id': '1015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0189172, 'theta_0': -0.2704668324908934, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.028900000000000002, 'Reactive_load': 0.008199999999999999, 'Node_id': '1016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9904413, 'theta_0': -0.37371269196382906, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1834, 'Reactive_load': 0.052000000000000005, 'Node_id': '1017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965435, 'theta_0': -0.35179214195694364, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0007000000000000001, 'Reactive_load': 0.0002, 'Node_id': '1018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040552, 'theta_0': -0.3881103318425507, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6178, 'Reactive_load': 0.175, 'Node_id': '1019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.4298, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0259069, 'theta_0': -0.2662123131864769, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7673000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0016168, 'theta_0': -0.15128920618002564, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156825, 'theta_0': -0.2815168435978274, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.1264610249057803, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9971655, 'theta_0': -0.3339430262159406, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6304, 'Reactive_load': 0.17859999999999998, 'Node_id': '1024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.021133, 'theta_0': -0.2477859821552543, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.08215732537376844, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0139861, 'theta_0': -0.44947223371173195, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0121, 'Reactive_load': 0.2868, 'Node_id': '1027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.75, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0190224, 'theta_0': -0.3077698244069383, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1001, 'Reactive_load': 0.028399999999999998, 'Node_id': '1028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.43554949792247794, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8318000000000001, 'Reactive_load': 0.2357, 'Node_id': '1029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6541, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0182991, 'theta_0': -0.23551553211557077, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2463, 'Reactive_load': 0.0698, 'Node_id': '1030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0191923, 'theta_0': -0.3087532301739745, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0925, 'Reactive_load': 0.0262, 'Node_id': '1031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064914, 'theta_0': -0.21631809716752448, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0259109, 'theta_0': -0.29581999135084613, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064769, 'theta_0': -0.2500713511844007, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.1926625851644016, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.000559, 'theta_0': -0.21988752238065565, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.136, 'Reactive_load': 0.0385, 'Node_id': '1036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9813726, 'theta_0': -0.39238980935527074, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0224, 'Reactive_load': 0.2897, 'Node_id': '1037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200949, 'theta_0': -0.1354289105749626, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000035, 'theta_0': -0.04171232192511328, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.995842, 'theta_0': -0.3559939174113649, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4436, 'Reactive_load': 0.1257, 'Node_id': '1040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0049175, 'theta_0': -0.2639136098418453, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0049175, 'theta_0': -0.2639136098418453, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.2074107918766789, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151811, 'theta_0': -0.4516220255645834, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14679999999999999, 'Reactive_load': 0.0416, 'Node_id': '1044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9827629, 'theta_0': -0.4250270649418368, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9856, 'Reactive_load': 0.2793, 'Node_id': '1045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0281129, 'theta_0': -0.3925998946373333, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1108, 'Reactive_load': 0.031400000000000004, 'Node_id': '1046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.12, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0086095, 'theta_0': -0.2333874870651991, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0086095, 'theta_0': -0.2333874870651991, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0086095, 'theta_0': -0.2333874870651991, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.14516767326816574, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.1194265975286648, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0009878, 'theta_0': -0.1600187496865556, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0257297, 'theta_0': -0.30267552247963225, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0125102, 'theta_0': -0.3889446515848816, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1499, 'Reactive_load': 0.0425, 'Node_id': '1054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056314, 'theta_0': -0.31518941104338394, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0398, 'Reactive_load': 0.0113, 'Node_id': '1055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.3811244944318108, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5613, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -0.3811244944318108, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0257559, 'theta_0': -0.19826083345992854, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1346, 'Reactive_load': 0.0381, 'Node_id': '1058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0257559, 'theta_0': -0.19826083345992854, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0257559, 'theta_0': -0.19826083345992854, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011891, 'theta_0': -0.3008836802030722, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.011891, 'theta_0': -0.3008836802030722, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.011891, 'theta_0': -0.3008836802030722, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9768367, 'theta_0': -0.41982991076171067, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2376, 'Reactive_load': 0.3507, 'Node_id': '1064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078425, 'theta_0': -0.26102795226976794, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.12986448676033932, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0018217, 'theta_0': -0.27384659277421036, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0155, 'Reactive_load': 0.2877, 'Node_id': '1067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9876132, 'theta_0': -0.28719377133603424, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.594, 'Reactive_load': 0.16829999999999998, 'Node_id': '1068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0272175, 'theta_0': -0.15493202248820315, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400036, 'theta_0': -0.07648978750693733, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9912743, 'theta_0': -0.3733979916464019, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9146, 'Reactive_load': 0.2591, 'Node_id': '1071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.21039451949942578, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.09881240901360464, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.13448756724289945, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.18261772662698367, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9912743, 'theta_0': -0.3733979916464019, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9912743, 'theta_0': -0.3733979916464019, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.1916803138613792, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0198409, 'theta_0': -0.6690580759311804, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.18153248089809362, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300036, 'theta_0': -0.3262004664014134, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7977, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0233299, 'theta_0': -0.1056018445236177, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0227241, 'theta_0': -0.17420604248382693, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3995, 'Reactive_load': 0.11320000000000001, 'Node_id': '1083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.2615, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0283078, 'theta_0': -0.28747653212814983, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011558, 'theta_0': -0.26751288018518554, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.213, 'Reactive_load': 0.0603, 'Node_id': '1085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0186846, 'theta_0': -0.3066038921067285, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3967, 'Reactive_load': 0.1124, 'Node_id': '1086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -0.283274198168368, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0091, 'Reactive_load': 0.0026, 'Node_id': '1087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1405, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.20787196022493334, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9866587, 'theta_0': -0.37078259830899596, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.406, 'Reactive_load': 0.115, 'Node_id': '1089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -0.28037035426548484, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0327141, 'theta_0': -0.5377400440631951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1162, 'Reactive_load': 0.0329, 'Node_id': '1091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028305, 'theta_0': -1.0402556786297155, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4836, 'Reactive_load': 0.13699999999999998, 'Node_id': '2001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0127504, 'theta_0': -1.0679504584806063, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0114428, 'theta_0': -1.1072951889155118, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0127504, 'theta_0': -1.0679504584806063, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0127504, 'theta_0': -1.0679504584806063, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0127504, 'theta_0': -1.0679504584806063, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0037001, 'theta_0': -1.0362542669734232, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09539999999999998, 'Reactive_load': 0.027000000000000003, 'Node_id': '2007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0195642, 'theta_0': -0.5679095786203664, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0806, 'Reactive_load': 0.022799999999999997, 'Node_id': '2008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0240415, 'theta_0': -0.7841633778582473, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3914, 'Reactive_load': 0.1109, 'Node_id': '2009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.6318696823338213, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5681, 'Reactive_load': 0.1609, 'Node_id': '2010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3017, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0391574, 'theta_0': -0.5546714656834996, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0281418, 'theta_0': -0.5158230370746913, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0308001, 'theta_0': -0.513854113692224, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192746, 'theta_0': -0.5782001097032949, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.25429999999999997, 'Reactive_load': 0.07200000000000001, 'Node_id': '2014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0119269, 'theta_0': -1.098672023226891, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046900000000000004, 'Reactive_load': 0.013300000000000001, 'Node_id': '2015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0110365, 'theta_0': -1.21410494090785, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6927, 'Reactive_load': 0.1963, 'Node_id': '2016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0309429, 'theta_0': -0.557532618832879, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0285229, 'theta_0': -0.5437547628248329, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0309429, 'theta_0': -0.557532618832879, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9971221, 'theta_0': -0.7780236935221668, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4797, 'Reactive_load': 0.1359, 'Node_id': '2020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0147857, 'theta_0': -0.9831308427724308, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0110382, 'theta_0': -1.016833168081734, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11560000000000001, 'Reactive_load': 0.032799999999999996, 'Node_id': '2022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -0.9558629879098702, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -0.9442595503504213, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9957092, 'theta_0': -0.7647092224305252, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3962, 'Reactive_load': 0.11230000000000001, 'Node_id': '2025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0269597, 'theta_0': -0.5368796840084246, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1857, 'Reactive_load': 0.0526, 'Node_id': '2026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0033184, 'theta_0': -1.0477634218466794, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0355, 'Reactive_load': 0.0101, 'Node_id': '2027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973652, 'theta_0': -0.8193291617453475, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1424, 'Reactive_load': 0.0403, 'Node_id': '2028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0258493, 'theta_0': -0.5676143038175139, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.032, 'Reactive_load': 0.0091, 'Node_id': '2029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9899978, 'theta_0': -1.1109340956861649, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4715, 'Reactive_load': 0.1336, 'Node_id': '2030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0274656, 'theta_0': -0.5144848756838947, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.057699999999999994, 'Reactive_load': 0.0163, 'Node_id': '2031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056747, 'theta_0': -1.0257179411719362, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0335, 'Reactive_load': 0.0095, 'Node_id': '2032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9926513, 'theta_0': -1.1372642724080915, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.13570000000000002, 'Reactive_load': 0.0385, 'Node_id': '2033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0087755, 'theta_0': -1.2022980677707884, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.053399999999999996, 'Reactive_load': 0.0151, 'Node_id': '2034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0225087, 'theta_0': -0.5307682914595188, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046799999999999994, 'Reactive_load': 0.013300000000000001, 'Node_id': '2035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0010787, 'theta_0': -0.7710612435234035, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.41590000000000005, 'Reactive_load': 0.11779999999999999, 'Node_id': '2036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231762, 'theta_0': -1.104627540420301, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.035, 'Reactive_load': 0.009899999999999999, 'Node_id': '2037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0084005, 'theta_0': -1.1155186916587185, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.026000000000000002, 'Reactive_load': 0.0074, 'Node_id': '2038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9953598, 'theta_0': -1.0528522085335492, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0556, 'Reactive_load': 0.0158, 'Node_id': '2039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0085073, 'theta_0': -1.0285148836581346, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.159, 'Reactive_load': 0.0451, 'Node_id': '2040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0289976, 'theta_0': -0.4777824515634611, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0204143, 'theta_0': -0.4329213811348248, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0095614, 'theta_0': -1.2076832986311095, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0424, 'Reactive_load': 0.012, 'Node_id': '2043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.018029, 'theta_0': -1.1347523072821588, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.27399999999999997, 'Reactive_load': 0.0776, 'Node_id': '2044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965663, 'theta_0': -0.7874341597830697, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2261, 'Reactive_load': 0.064, 'Node_id': '2045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0083263, 'theta_0': -1.195521111364927, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2213, 'Reactive_load': 0.06269999999999999, 'Node_id': '2046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0102323, 'theta_0': -1.1229852800119202, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1496, 'Reactive_load': 0.0424, 'Node_id': '2047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.031688, 'theta_0': -0.5501704186222389, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0642, 'Reactive_load': 0.0182, 'Node_id': '2048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0212137, 'theta_0': -0.7097824018733246, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1157, 'Reactive_load': 0.032799999999999996, 'Node_id': '2049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9955969, 'theta_0': -1.0969185956471674, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0366, 'Reactive_load': 0.0104, 'Node_id': '2050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9932948, 'theta_0': -0.9394078840957275, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17850000000000002, 'Reactive_load': 0.0506, 'Node_id': '2051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0197355, 'theta_0': -0.5525469287449245, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2484, 'Reactive_load': 0.0704, 'Node_id': '2052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.993578, 'theta_0': -1.0298059735190048, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2913, 'Reactive_load': 0.0825, 'Node_id': '2053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9992403, 'theta_0': -0.7474446698158926, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990468, 'theta_0': -0.7618869901234654, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4962, 'Reactive_load': 0.1406, 'Node_id': '2055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9992403, 'theta_0': -0.7474446698158926, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9900036, 'theta_0': -0.6706253117863011, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0292536, 'theta_0': -0.47031050504945565, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0192676, 'theta_0': -0.4120085318781634, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987303, 'theta_0': -1.074553772079187, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.038, 'Reactive_load': 0.0108, 'Node_id': '2060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0241813, 'theta_0': -0.6484433812706372, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -0.6083595911639748, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0094256, 'theta_0': -1.1843901518872857, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6562, 'Reactive_load': 0.1859, 'Node_id': '2063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.4, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0042757, 'theta_0': -1.0454039112309081, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1005, 'Reactive_load': 0.0285, 'Node_id': '2064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0266022, 'theta_0': -1.1388096392876847, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7836, 'Reactive_load': 0.222, 'Node_id': '2065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0205541, 'theta_0': -0.47996992507486314, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0711, 'Reactive_load': 0.0202, 'Node_id': '2066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938774, 'theta_0': -0.7633551087303654, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5397, 'Reactive_load': 0.15289999999999998, 'Node_id': '2067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0000156, 'theta_0': -0.7278910131675144, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0576, 'Reactive_load': 0.0163, 'Node_id': '2068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0256589, 'theta_0': -1.199782385093549, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2192, 'Reactive_load': 0.0621, 'Node_id': '2069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0020167, 'theta_0': -0.7205890268160681, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.6564273899338852, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0193101, 'theta_0': -1.167023113538976, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0971, 'Reactive_load': 0.0275, 'Node_id': '2072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0206805, 'theta_0': -0.43624061339968256, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103706, 'theta_0': -0.4039175345317681, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103706, 'theta_0': -0.4039175345317681, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0259186, 'theta_0': -0.42057473001999923, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0234359, 'theta_0': -1.062480055552788, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.0046498448783099, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0315049, 'theta_0': -0.6009273683437447, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.044800000000000006, 'Reactive_load': 0.0127, 'Node_id': '2079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9988796, 'theta_0': -0.9978903417649686, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1693, 'Reactive_load': 0.048, 'Node_id': '2080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0029399, 'theta_0': -1.1295458504971194, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0446, 'Reactive_load': 0.0126, 'Node_id': '2081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0054662, 'theta_0': -0.7771224229497294, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5903, 'Reactive_load': 0.1673, 'Node_id': '2082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -0.7496851140700926, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0054662, 'theta_0': -0.7771224229497294, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0054662, 'theta_0': -0.7771224229497294, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966171, 'theta_0': -1.0321157771576792, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0219, 'Reactive_load': 0.0062, 'Node_id': '2086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0234535, 'theta_0': -0.477101092476775, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.079, 'Reactive_load': 0.022400000000000003, 'Node_id': '2087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0010231, 'theta_0': -0.7226542225600754, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.6598115659002096, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0037195, 'theta_0': -1.058570256228933, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0242, 'Reactive_load': 0.0069, 'Node_id': '2090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0250043, 'theta_0': -0.5898616318202502, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1705, 'Reactive_load': 0.0483, 'Node_id': '2091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0082536, 'theta_0': -1.1549906915026793, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24109999999999998, 'Reactive_load': 0.0683, 'Node_id': '2092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0287779, 'theta_0': -0.5290626684480073, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0287779, 'theta_0': -0.5290626684480073, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0287779, 'theta_0': -0.5290626684480073, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0299997, 'theta_0': -0.4825002336112344, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.6851, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0286671, 'theta_0': -0.4729576110193704, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.033775, 'theta_0': -0.5275157656786722, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0295321, 'theta_0': -0.48003696317143224, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0207998, 'theta_0': -0.4370165867851193, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123899, 'theta_0': -1.0872336240015106, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0131827, 'theta_0': -1.112433769845941, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0123899, 'theta_0': -1.0872336240015106, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0123899, 'theta_0': -1.0872336240015106, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0123899, 'theta_0': -1.0872336240015106, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9910979, 'theta_0': -1.1485197087679853, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14400000000000002, 'Reactive_load': 0.0408, 'Node_id': '2106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078222, 'theta_0': -1.127594345500587, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7415, 'Reactive_load': 0.2101, 'Node_id': '2107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0034813, 'theta_0': -0.7576093626597525, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.021400000000000002, 'Reactive_load': 0.0060999999999999995, 'Node_id': '2108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.010155, 'theta_0': -0.8039047795742627, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3381, 'Reactive_load': 0.0958, 'Node_id': '2109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0267183, 'theta_0': -1.0812186259871925, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0859, 'Reactive_load': 0.024300000000000002, 'Node_id': '2110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973075, 'theta_0': -0.8492929219837086, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.13949999999999999, 'Reactive_load': 0.0395, 'Node_id': '2111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9947264, 'theta_0': -0.856461512820425, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0454, 'Reactive_load': 0.0129, 'Node_id': '2112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0146803, 'theta_0': -0.9749467146571491, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123542, 'theta_0': -0.9973634966769616, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7078, 'Reactive_load': 0.2005, 'Node_id': '2114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0058027, 'theta_0': -0.9053693211060279, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0104495, 'theta_0': -0.938896799330866, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0030718, 'theta_0': -0.8942307868994327, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.001705, 'theta_0': -0.8853830846025427, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0093702, 'theta_0': -0.9310138801378934, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.8896806262730208, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0072293, 'theta_0': -1.0395478778048617, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0233, 'Reactive_load': 0.0066, 'Node_id': '2121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976392, 'theta_0': -1.0604972218961823, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0672, 'Reactive_load': 0.019, 'Node_id': '2122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0294908, 'theta_0': -0.46633574457084126, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9900034, 'theta_0': -1.1672379635698964, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21289999999999998, 'Reactive_load': 0.0603, 'Node_id': '2124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996358, 'theta_0': -1.168410336135046, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0406, 'Reactive_load': 0.0115, 'Node_id': '2125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0126625, 'theta_0': -1.1630303587157735, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1424, 'Reactive_load': 0.0403, 'Node_id': '2126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.5872027797844696, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -1.8486000000000002, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0303186, 'theta_0': -0.561195192268189, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9941082, 'theta_0': -1.1470637201993863, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0663, 'Reactive_load': 0.018799999999999997, 'Node_id': '2129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.005481, 'theta_0': -0.7336099858874017, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.000627, 'theta_0': -0.6888041900361761, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0167576, 'theta_0': -0.7188310962868744, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11939999999999999, 'Reactive_load': 0.0338, 'Node_id': '2132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0181866, 'theta_0': -1.050694510339186, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.056799999999999996, 'Reactive_load': 0.0161, 'Node_id': '2133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137236, 'theta_0': -0.9722302318494676, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0017833, 'theta_0': -1.014709311821567, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1286, 'Reactive_load': 0.31980000000000003, 'Node_id': '3002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0178015, 'theta_0': -0.31709454264169085, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.012923, 'theta_0': -0.2893519233160028, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200387, 'theta_0': -1.01443416066499, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07339999999999999, 'Reactive_load': 0.0208, 'Node_id': '3005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0052002, 'theta_0': -0.46701738291020767, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0735, 'Reactive_load': 0.0208, 'Node_id': '3006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0157044, 'theta_0': -0.42930015454949444, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1496, 'Reactive_load': 0.0424, 'Node_id': '3007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963778, 'theta_0': -0.644981276353211, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1981, 'Reactive_load': 0.056100000000000004, 'Node_id': '3008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0213724, 'theta_0': -0.5429448602387374, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12539999999999998, 'Reactive_load': 0.0355, 'Node_id': '3009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0025526, 'theta_0': -0.9961035260366543, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0467, 'Reactive_load': 0.0132, 'Node_id': '3010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168172, 'theta_0': -1.0656535179185191, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.27140000000000003, 'Reactive_load': 0.07690000000000001, 'Node_id': '3011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0182316, 'theta_0': -1.0586954137895934, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.005600000000000001, 'Reactive_load': 0.0016, 'Node_id': '3012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0188327, 'theta_0': -0.49616973957931665, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6456000000000001, 'Reactive_load': 0.18289999999999998, 'Node_id': '3013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.65, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0190822, 'theta_0': -0.9292465073774465, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22, 'Reactive_load': 0.0623, 'Node_id': '3014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.01533, 'theta_0': -0.2958869945408302, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0536, 'Reactive_load': 0.0152, 'Node_id': '3015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0119036, 'theta_0': -0.38156815712766773, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.29092153281890637, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0303312, 'theta_0': -0.5068182376918293, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -0.5347290893041171, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.40950000000000003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300036, 'theta_0': -0.4349100441911322, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0053529, 'theta_0': -0.6203222000301915, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0194, 'Reactive_load': 0.0055000000000000005, 'Node_id': '3021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9989041, 'theta_0': -0.3149486254197788, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2459, 'Reactive_load': 0.0697, 'Node_id': '3022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0269219, 'theta_0': -0.46942241171287086, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1041, 'Reactive_load': 0.029500000000000002, 'Node_id': '3023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0257894, 'theta_0': -0.5423813283298535, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1013, 'Reactive_load': 0.0287, 'Node_id': '3024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9989445, 'theta_0': -0.7910676734863341, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1367, 'Reactive_load': 0.0387, 'Node_id': '3025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0288856, 'theta_0': -0.6391103029087675, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.019168, 'theta_0': -0.6496114428725344, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.5940100525594381, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300528, 'theta_0': -0.9340608235861477, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0487, 'Reactive_load': 0.0138, 'Node_id': '3029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0178371, 'theta_0': -0.6963279030157957, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2972, 'Reactive_load': 0.3675, 'Node_id': '3030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0005941, 'theta_0': -0.45579728946766684, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14730000000000001, 'Reactive_load': 0.0417, 'Node_id': '3031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0022897, 'theta_0': -0.4709821601055006, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7286, 'Reactive_load': 0.2064, 'Node_id': '3032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089061, 'theta_0': -0.2612611457111269, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300036, 'theta_0': -0.09953459135149487, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021183, 'theta_0': -0.7756041261268345, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0347, 'Reactive_load': 0.0098, 'Node_id': '3035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998823, 'theta_0': -0.7266235725180085, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.057999999999999996, 'Reactive_load': 0.016399999999999998, 'Node_id': '3036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0274371, 'theta_0': -0.44343267891483823, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0075834, 'theta_0': -0.34970066155440127, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.39845652147532545, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.997184, 'theta_0': -0.5978552572476767, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1461, 'Reactive_load': 0.0414, 'Node_id': '3040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.019886, 'theta_0': -0.5085131443817335, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.006916, 'theta_0': -0.3279576115324437, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0875, 'Reactive_load': 0.0248, 'Node_id': '3042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.019886, 'theta_0': -0.5085131443817335, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.019886, 'theta_0': -0.5085131443817335, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.019886, 'theta_0': -0.5085131443817335, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.016328, 'theta_0': -0.6028779134091885, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -0.35818741230868634, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.1623999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0213904, 'theta_0': -0.7224191092565392, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0065595, 'theta_0': -0.5217314303782976, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1058, 'Reactive_load': 0.03, 'Node_id': '3049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0239623, 'theta_0': -0.5921339632398842, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.65, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0059952, 'theta_0': -0.6247574481786521, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4528999999999999, 'Reactive_load': 0.4117, 'Node_id': '3051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0062201, 'theta_0': -1.021542415469465, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0808, 'Reactive_load': 0.0229, 'Node_id': '3052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300228, 'theta_0': -0.4726928271185503, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0093892, 'theta_0': -0.4640979183105193, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.422353674313387, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.3773314925414991, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.3802042870369893, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0320922, 'theta_0': -0.4171820193936325, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0124383, 'theta_0': -0.30179610342942986, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.28511946497333146, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.3230110691796115, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.36977355365524794, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.34964966303365796, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0215851, 'theta_0': -0.3218543871244373, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0003333, 'theta_0': -0.2154463750193234, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078407, 'theta_0': -0.7447552919714945, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22030000000000002, 'Reactive_load': 0.062400000000000004, 'Node_id': '3066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0281769, 'theta_0': -0.3443983163802575, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.2939040910710544, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997036, 'theta_0': -0.5139053565590624, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0381, 'Reactive_load': 0.0108, 'Node_id': '3069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0198813, 'theta_0': -0.36696694204499336, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0054475, 'theta_0': -0.2871161739974906, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011792, 'theta_0': -0.672444939704553, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.059800000000000006, 'Reactive_load': 0.017, 'Node_id': '3072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040665, 'theta_0': -0.9463329840484982, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.000003, 'theta_0': -0.9023649636915223, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0030425, 'theta_0': -0.6998369921966855, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12179999999999999, 'Reactive_load': 0.0345, 'Node_id': '3075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0012835, 'theta_0': -0.9717275770248932, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1079, 'Reactive_load': 0.030600000000000002, 'Node_id': '3076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0031436, 'theta_0': -0.4873988143162219, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6866, 'Reactive_load': 0.1945, 'Node_id': '3077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.022309, 'theta_0': -0.44932969267172157, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0185289, 'theta_0': -0.41839437745194535, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0083586, 'theta_0': -0.3682605800865716, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.3432273051719244, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0209748, 'theta_0': -0.48746126219685826, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0143155, 'theta_0': -0.42107789099005666, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.3157143188159212, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0070584, 'theta_0': -0.32064176726673665, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.19969181146034617, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0070584, 'theta_0': -0.32064176726673665, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -0.5780175657168289, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2623, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0268531, 'theta_0': -0.6238909271116219, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0238383, 'theta_0': -0.6352282368667266, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200036, 'theta_0': -0.6506376790194145, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6579999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.4595374427882281, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040589, 'theta_0': -1.0103513119457999, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.050499999999999996, 'Reactive_load': 0.0143, 'Node_id': '3093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161091, 'theta_0': -0.40426021247710464, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.26595014727953487, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0246537, 'theta_0': -0.3730238977745295, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0203686, 'theta_0': -0.34090928103247314, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.22212737322291004, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008904, 'theta_0': -0.5011538239643593, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1208, 'Reactive_load': 0.0342, 'Node_id': '3099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0015027, 'theta_0': -0.6074460858334809, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08529999999999999, 'Reactive_load': 0.0242, 'Node_id': '3100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0063068, 'theta_0': -0.8350190434022459, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0943, 'Reactive_load': 0.026699999999999998, 'Node_id': '3101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0115922, 'theta_0': -0.6531929806706743, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7526, 'Reactive_load': 0.2132, 'Node_id': '3102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100614, 'theta_0': -0.35955007812218087, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3932, 'Reactive_load': 0.1114, 'Node_id': '3103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0133944, 'theta_0': -0.8278782905670989, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0112245, 'theta_0': -0.7306883570794407, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0044975, 'theta_0': -0.7127378202556043, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0013283, 'theta_0': -1.0236376657831916, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1487, 'Reactive_load': 0.0421, 'Node_id': '3107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089963, 'theta_0': -0.9896167306189371, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2352, 'Reactive_load': 0.0666, 'Node_id': '3108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0203797, 'theta_0': -0.3168114502370174, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -0.1860419928062265, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0114072, 'theta_0': -0.287087690224098, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.14487629054954532, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.14486162978382855, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156624, 'theta_0': -0.5993662062344208, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9831, 'Reactive_load': 0.2785, 'Node_id': '3114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0067461, 'theta_0': -0.3811652653231373, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0958, 'Reactive_load': 0.0271, 'Node_id': '3115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0059651, 'theta_0': -0.26543250007668584, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.2121490989964557, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.19521996375747136, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.029188, 'theta_0': -0.9033156270817911, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0295699, 'theta_0': -0.8778568932553853, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0177993, 'theta_0': -0.28592217680290877, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.1314776945879577, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.997916, 'theta_0': -0.9904031410733006, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6607, 'Reactive_load': 0.18719999999999998, 'Node_id': '3123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0295971, 'theta_0': -1.0404597250725662, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6454000000000001, 'Reactive_load': 0.18289999999999998, 'Node_id': '3124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.45, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0041577, 'theta_0': -0.6537664260497096, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9126000000000001, 'Reactive_load': 0.2586, 'Node_id': '3125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.998329623684403, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09960000000000001, 'Reactive_load': 0.0282, 'Node_id': '3126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.4032, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0188631, 'theta_0': -0.2866669786079048, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103131, 'theta_0': -0.31429733835610457, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0282, 'Reactive_load': 0.008, 'Node_id': '3128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.027401, 'theta_0': -0.29463363869838805, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.009018, 'theta_0': -0.33318405233741827, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9215000000000001, 'Reactive_load': 0.2611, 'Node_id': '3130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200036, 'theta_0': -0.3846816674335929, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984354, 'theta_0': -0.699824390919486, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0956, 'Reactive_load': 0.0271, 'Node_id': '3132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0209991, 'theta_0': -0.38421533291075255, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200994, 'theta_0': -0.39770967763510456, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103196, 'theta_0': -0.3358679498479651, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0030814, 'theta_0': -0.27164342875283287, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4696, 'Reactive_load': 0.133, 'Node_id': '3136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0201134, 'theta_0': -0.41018398213142104, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0187513, 'theta_0': -0.6788297729873687, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10949999999999999, 'Reactive_load': 0.031, 'Node_id': '3138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9936561, 'theta_0': -0.5151215019818521, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1492, 'Reactive_load': 0.042300000000000004, 'Node_id': '3139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9969644, 'theta_0': -0.5224612223517265, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0022, 'Reactive_load': 0.0006, 'Node_id': '3140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0197368, 'theta_0': -0.30990326252469863, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0267909, 'theta_0': -0.6993977626371285, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0707, 'Reactive_load': 0.02, 'Node_id': '3142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0061812, 'theta_0': -0.6743413272033074, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0575, 'Reactive_load': 0.0163, 'Node_id': '3143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.17, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008154, 'theta_0': -0.6902823442862752, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0556, 'Reactive_load': 0.015700000000000002, 'Node_id': '3144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.009482, 'theta_0': -0.4128331824964382, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0294, 'Reactive_load': 0.0083, 'Node_id': '3145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0263909, 'theta_0': -0.6192503235233716, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1204, 'Reactive_load': 0.0341, 'Node_id': '3146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.15, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300034, 'theta_0': -0.9740025787982828, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.34759999999999996, 'Reactive_load': 0.09849999999999999, 'Node_id': '3147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3386, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0201246, 'theta_0': -0.9219604209754859, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0101, 'Reactive_load': 0.0029, 'Node_id': '4002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055761, 'theta_0': -1.2565306312581308, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2729000000000001, 'Reactive_load': 0.3607, 'Node_id': '4003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0239652, 'theta_0': -0.9750298097828366, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0221997, 'theta_0': -1.0476850565632647, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6414, 'Reactive_load': 0.46509999999999996, 'Node_id': '4005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0280051, 'theta_0': -0.9668113859477532, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0275, 'Reactive_load': 0.0078000000000000005, 'Node_id': '4006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0090042, 'theta_0': -1.2669610330273142, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8366, 'Reactive_load': 0.5204, 'Node_id': '4007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996096, 'theta_0': -0.9546523577275169, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3249, 'Reactive_load': 0.09210000000000002, 'Node_id': '4008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0253681, 'theta_0': -0.8105823569325155, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.761004481840487, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0203187, 'theta_0': -1.0008514673739022, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1605, 'Reactive_load': 0.0455, 'Node_id': '4011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0212456, 'theta_0': -1.2027568799245527, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1374, 'Reactive_load': 0.038900000000000004, 'Node_id': '4012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0099492, 'theta_0': -1.2411550135863465, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9235, 'Reactive_load': 0.26170000000000004, 'Node_id': '4013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0260846, 'theta_0': -0.922008626969426, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300032, 'theta_0': -0.9476646702677699, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5981, 'Reactive_load': 0.45280000000000004, 'Node_id': '4015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6788, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0101817, 'theta_0': -0.873760553141077, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0625, 'Reactive_load': 0.0177, 'Node_id': '4016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0097723, 'theta_0': -0.8433227789311668, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8956000000000001, 'Reactive_load': 0.25379999999999997, 'Node_id': '4017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0024055, 'theta_0': -0.8228530338650017, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07139999999999999, 'Reactive_load': 0.0202, 'Node_id': '4018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9901916, 'theta_0': -1.1719806689193883, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3583, 'Reactive_load': 0.3848, 'Node_id': '4019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9846321, 'theta_0': -0.8758507943598506, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1781, 'Reactive_load': 0.33380000000000004, 'Node_id': '4020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976659, 'theta_0': -1.159622184472234, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3197, 'Reactive_load': 0.0906, 'Node_id': '4021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105752, 'theta_0': -1.0385317994742282, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0556, 'Reactive_load': 0.0158, 'Node_id': '4022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0104086, 'theta_0': -0.9106766230152472, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0506, 'Reactive_load': 0.0143, 'Node_id': '4023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0212305, 'theta_0': -0.7361006230898751, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0189448, 'theta_0': -0.8036809061911094, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.6066498317221359, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0217313, 'theta_0': -1.2673470998578553, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6512, 'Reactive_load': 0.1845, 'Node_id': '4027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0154938, 'theta_0': -0.588751375523179, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156665, 'theta_0': -0.6230484043218066, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100027, 'theta_0': -0.5295294567564531, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231404, 'theta_0': -1.0051971103185653, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0275, 'Reactive_load': 0.0078000000000000005, 'Node_id': '4031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9869365, 'theta_0': -0.9585778824679324, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9139, 'Reactive_load': 0.2589, 'Node_id': '4032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0086369, 'theta_0': -1.0233865652637073, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2625, 'Reactive_load': 0.3577, 'Node_id': '4033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0079148, 'theta_0': -1.2406901974999553, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3764, 'Reactive_load': 0.1066, 'Node_id': '4034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9865021, 'theta_0': -0.8487809121943436, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7916, 'Reactive_load': 0.2243, 'Node_id': '4035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0011393, 'theta_0': -0.9664629658691775, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3507, 'Reactive_load': 0.09939999999999999, 'Node_id': '4036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0113138, 'theta_0': -0.8561000027724595, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2872, 'Reactive_load': 0.0814, 'Node_id': '4037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.65, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9957176, 'theta_0': -0.845091198889165, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.833, 'Reactive_load': 0.23600000000000002, 'Node_id': '4038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0191654, 'theta_0': -1.0636026164210857, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300032, 'theta_0': -1.125917642594781, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.2566, 'Reactive_load': 0.6394, 'Node_id': '4040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.7968, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.012531, 'theta_0': -1.1421269691436577, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0190781, 'theta_0': -1.231972626952094, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1518, 'Reactive_load': 0.6097, 'Node_id': '4042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0145059, 'theta_0': -1.0079828477909658, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.024700000000000003, 'Reactive_load': 0.006999999999999999, 'Node_id': '4043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0157358, 'theta_0': -0.9856092801234329, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200031, 'theta_0': -0.839672370440743, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200031, 'theta_0': -0.9180685985429263, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200031, 'theta_0': -0.9614986211579749, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0079987, 'theta_0': -1.1516154341688325, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.131, 'Reactive_load': 0.0371, 'Node_id': '4048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0076134, 'theta_0': -1.050738353009996, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000032, 'theta_0': -0.9910155072946554, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9875562, 'theta_0': -0.9485351706854945, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8939, 'Reactive_load': 0.25329999999999997, 'Node_id': '4051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0237877, 'theta_0': -0.8757869153092275, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0159207, 'theta_0': -0.8788484322567358, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0011, 'Reactive_load': 0.0003, 'Node_id': '4053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.003428, 'theta_0': -0.8526332378259306, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.055999999999999994, 'Reactive_load': 0.0159, 'Node_id': '4054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9895083, 'theta_0': -0.8285540296935935, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1778, 'Reactive_load': 0.0504, 'Node_id': '4055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.020351, 'theta_0': -0.6787496972812872, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0019285, 'theta_0': -0.7741905316057667, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6236642437483728, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6396009498680881, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.5792516880309141, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.5975647122870973, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.561536369230369, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6095027818240311, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9944281, 'theta_0': -0.8351239725968759, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0882, 'Reactive_load': 0.025, 'Node_id': '4064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200031, 'theta_0': -0.7477449188071131, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0212963, 'theta_0': -1.2767636922244328, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9153, 'Reactive_load': 0.5427000000000001, 'Node_id': '4066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.4513, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9918237, 'theta_0': -1.1220171284225464, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5946, 'Reactive_load': 0.4518, 'Node_id': '4067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0104652, 'theta_0': -1.266478624022063, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2799, 'Reactive_load': 0.0793, 'Node_id': '4068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9951713, 'theta_0': -0.8327671672414453, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0424, 'Reactive_load': 0.012, 'Node_id': '4069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137137, 'theta_0': -0.8935459546075352, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996438, 'theta_0': -0.9716474315056417, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.8145, 'Reactive_load': 0.7975, 'Node_id': '4071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9960976, 'theta_0': -0.8273780268435997, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5171, 'Reactive_load': 0.1465, 'Node_id': '4072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0188507, 'theta_0': -1.0669114116170166, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0274918, 'theta_0': -1.1108604427917308, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0188507, 'theta_0': -1.0669114116170166, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.9666414083319014, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -1.004045402451759, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.98683, 'theta_0': -0.8364260754853262, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8406999999999999, 'Reactive_load': 0.2382, 'Node_id': '4078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0172412, 'theta_0': -1.178424284891411, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7121000000000002, 'Reactive_load': 0.4851, 'Node_id': '4079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0295117, 'theta_0': -0.7679782241195106, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0148455, 'theta_0': -0.8282430818340583, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.6535091296113805, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.6625670219100882, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.7105771375482203, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.7340618341774504, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0250677, 'theta_0': -1.0629024426850633, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.19870000000000002, 'Reactive_load': 0.056299999999999996, 'Node_id': '4086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0293, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0164653, 'theta_0': -0.8551031579701829, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0208, 'Reactive_load': 0.0059, 'Node_id': '4087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300032, 'theta_0': -1.2618184203863128, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1252, 'Reactive_load': 0.6021, 'Node_id': '4088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.8276, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0052288, 'theta_0': -1.1211588453095858, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.086, 'Reactive_load': 0.024399999999999998, 'Node_id': '4089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0052288, 'theta_0': -1.1211588453095858, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100032, 'theta_0': -1.0749967366433428, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0052288, 'theta_0': -1.1211588453095858, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100032, 'theta_0': -1.0544682088879707, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100032, 'theta_0': -1.0959262712476534, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100032, 'theta_0': -1.069965004769721, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0052288, 'theta_0': -1.1211588453095858, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100032, 'theta_0': -1.0631250419778626, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0052288, 'theta_0': -1.1211588453095858, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9944009, 'theta_0': -0.8700303656505447, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1517, 'Reactive_load': 0.043, 'Node_id': '4099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938044, 'theta_0': -0.8255995886956952, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.506, 'Reactive_load': 0.1434, 'Node_id': '4100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0065813, 'theta_0': -1.020896172407329, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2813999999999999, 'Reactive_load': 0.36310000000000003, 'Node_id': '4101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056631, 'theta_0': -1.020674690125251, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24170000000000003, 'Reactive_load': 0.06849999999999999, 'Node_id': '4102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0146255, 'theta_0': -1.2850100238742557, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8988, 'Reactive_load': 0.5379999999999999, 'Node_id': '4103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998728, 'theta_0': -0.8095290158223518, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100031, 'theta_0': -0.75058631482936, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9988978, 'theta_0': -1.009016571400337, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.37549999999999994, 'Reactive_load': 0.10640000000000001, 'Node_id': '4106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0166498, 'theta_0': -1.258078075079534, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1836, 'Reactive_load': 0.3354, 'Node_id': '4107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990433, 'theta_0': -0.7839467650447823, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2834, 'Reactive_load': 0.0803, 'Node_id': '4108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9836103, 'theta_0': -0.8663651044081865, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1086, 'Reactive_load': 0.3141, 'Node_id': '4109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074874, 'theta_0': -1.0940520843369843, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5735, 'Reactive_load': 0.1625, 'Node_id': '4110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9895912, 'theta_0': -1.1667041197115888, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0408, 'Reactive_load': 0.2949, 'Node_id': '4111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0202471, 'theta_0': -0.9483840077189792, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0559, 'Reactive_load': 0.0158, 'Node_id': '4112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0262252, 'theta_0': -0.9661728048810335, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0172849, 'theta_0': -1.00398688156194, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.8998285988626467, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.9005776243644326, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.015549, 'theta_0': -0.8338395149871631, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1889, 'Reactive_load': 0.0535, 'Node_id': '4117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9871672, 'theta_0': -0.9786656796805911, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1388, 'Reactive_load': 0.0393, 'Node_id': '4118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0116104, 'theta_0': -1.0073984941041056, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5557, 'Reactive_load': 0.1575, 'Node_id': '4119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9903374, 'theta_0': -0.8310498854305305, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.46520000000000006, 'Reactive_load': 0.1318, 'Node_id': '4120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0160068, 'theta_0': -1.1922762824860818, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.041100000000000005, 'Reactive_load': 0.0116, 'Node_id': '4121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0163884, 'theta_0': -1.2447643195728857, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4611, 'Reactive_load': 0.414, 'Node_id': '4122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9903032, 'theta_0': -0.8683503640724526, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.905, 'Reactive_load': 0.2564, 'Node_id': '4123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039254, 'theta_0': -1.0841721419605874, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2438, 'Reactive_load': 0.0691, 'Node_id': '4124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9964445, 'theta_0': -0.8029363487322085, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.045, 'Reactive_load': 0.0128, 'Node_id': '4125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0053079, 'theta_0': -0.9789863664773525, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2738, 'Reactive_load': 0.36090000000000005, 'Node_id': '4126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0034791, 'theta_0': -1.0089802685518956, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8729, 'Reactive_load': 0.2473, 'Node_id': '4127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0071846, 'theta_0': -1.2060571055538563, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2351, 'Reactive_load': 0.35, 'Node_id': '4128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0326025, 'theta_0': -0.9772872360906585, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.18510000000000001, 'Reactive_load': 0.0525, 'Node_id': '4129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -1.0030490288883807, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09480000000000001, 'Reactive_load': 0.0269, 'Node_id': '4130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.23989999999999997, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.8788037343745922, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7145, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.02802, 'theta_0': -0.8948959144240752, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3309, 'Reactive_load': 0.09380000000000001, 'Node_id': '4132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8788037343745922, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0247289, 'theta_0': -0.8644710207440447, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0251205, 'theta_0': -0.8651387139026877, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0234422, 'theta_0': -0.860857805219981, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8788037343745922, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8788037343745922, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9867455, 'theta_0': -0.9507387884924775, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10619999999999999, 'Reactive_load': 0.0301, 'Node_id': '4139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0326265, 'theta_0': -0.8456850820737412, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300032, 'theta_0': -0.9011059704355961, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8625, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.809196635319602, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011373, 'theta_0': -0.9399651153660118, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1436, 'Reactive_load': 0.0407, 'Node_id': '4143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.988789, 'theta_0': -0.9733929003839761, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1636, 'Reactive_load': 0.0463, 'Node_id': '4144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9936232, 'theta_0': -0.8101778943316582, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4986, 'Reactive_load': 0.1413, 'Node_id': '4145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0272604, 'theta_0': -0.9694276519497852, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0267965, 'theta_0': -1.0012085966454451, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9364, 'Reactive_load': 0.5487, 'Node_id': '4147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9956005, 'theta_0': -1.005717061356027, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1464, 'Reactive_load': 0.0415, 'Node_id': '4148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9960202, 'theta_0': -0.7820541125506272, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6136, 'Reactive_load': 0.17379999999999998, 'Node_id': '4149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9900746, 'theta_0': -0.8428259709595867, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1253, 'Reactive_load': 0.0355, 'Node_id': '4150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.027008, 'theta_0': -0.8668665025956994, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0218341, 'theta_0': -0.841946674082432, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0370931, 'theta_0': -0.8350546655722791, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.005071, 'theta_0': -1.1760689456125522, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2632, 'Reactive_load': 0.0746, 'Node_id': '4154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0099639, 'theta_0': -1.1399744720304659, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1945, 'Reactive_load': 0.055099999999999996, 'Node_id': '4155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9931379, 'theta_0': -0.8593841888259371, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0801, 'Reactive_load': 0.306, 'Node_id': '4156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137407, 'theta_0': -1.2179822945609102, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0467, 'Reactive_load': 0.2966, 'Node_id': '4157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0149143, 'theta_0': -0.7310232857628984, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0001984, 'theta_0': -0.8044501775122184, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3618, 'Reactive_load': 0.1025, 'Node_id': '4159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6856603633611438, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6814273814196968, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4161.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300032, 'theta_0': -0.6683606074554983, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4162.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0149143, 'theta_0': -0.7310232857628984, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4163.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0094579, 'theta_0': -1.1222891903463474, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0203, 'Reactive_load': 0.005699999999999999, 'Node_id': '4164.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.024907, 'theta_0': -0.806933903210854, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4165.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0110274, 'theta_0': -0.7396141454070649, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4166.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9991489, 'theta_0': -1.1778458827773002, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.053099999999999994, 'Reactive_load': 0.015, 'Node_id': '4167.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987165, 'theta_0': -0.8098599826084075, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08869999999999999, 'Reactive_load': 0.025099999999999997, 'Node_id': '4168.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9900028, 'theta_0': -0.9059782839353412, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2768, 'Reactive_load': 0.0784, 'Node_id': '4169.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9898578, 'theta_0': -0.8143625131995323, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3586, 'Reactive_load': 0.1016, 'Node_id': '4170.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0121991, 'theta_0': -1.08328410098388, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6763, 'Reactive_load': 0.475, 'Node_id': '4171.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100606, 'theta_0': -0.8836068804760806, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.42310000000000003, 'Reactive_load': 0.1199, 'Node_id': '4172.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.00053, 'theta_0': -0.7051377316679174, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12050000000000001, 'Reactive_load': 0.0342, 'Node_id': '4173.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204937, 'theta_0': -1.001216171374399, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1916, 'Reactive_load': 0.054299999999999994, 'Node_id': '4174.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.85, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.9760045063569053, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0316, 'Reactive_load': 0.0089, 'Node_id': '4175.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.21780000000000002, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0390062, 'theta_0': -0.916019040949017, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4176.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.8572935810880208, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4177.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0248678, 'theta_0': -0.8127606325587595, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1746, 'Reactive_load': 0.0495, 'Node_id': '4178.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.44770000000000004, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.015857, 'theta_0': -1.0141451341408598, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.19920000000000002, 'Reactive_load': 0.0564, 'Node_id': '4179.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9877546, 'theta_0': -0.8327307073133712, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1076, 'Reactive_load': 0.0305, 'Node_id': '4180.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0233176, 'theta_0': -0.6823422132004222, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4181.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0045507, 'theta_0': -0.7531537290656286, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2268, 'Reactive_load': 0.0643, 'Node_id': '4182.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0190549, 'theta_0': -0.8651359562824694, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4183.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0067657, 'theta_0': -0.9997983182499562, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4184.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0205876, 'theta_0': -1.0202934229501528, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3784, 'Reactive_load': 0.1072, 'Node_id': '4185.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.20629999999999998, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0029866, 'theta_0': -1.201426904127948, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.37060000000000004, 'Reactive_load': 0.105, 'Node_id': '4186.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5379999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0011474, 'theta_0': -1.0241995396292862, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1506, 'Reactive_load': 0.042699999999999995, 'Node_id': '4187.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0304114, 'theta_0': -0.9247699821921763, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0297, 'Reactive_load': 0.0084, 'Node_id': '4188.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976437, 'theta_0': -0.9882115660381564, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2989, 'Reactive_load': 0.08470000000000001, 'Node_id': '4189.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0115516, 'theta_0': -1.0170845129473136, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3246, 'Reactive_load': 0.3753, 'Node_id': '4190.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0038487, 'theta_0': -0.9678011970731442, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4643, 'Reactive_load': 0.1316, 'Node_id': '4191.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200033, 'theta_0': -0.878739139738976, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4192.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.013129, 'theta_0': -0.9265870095698426, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7822, 'Reactive_load': 0.505, 'Node_id': '4193.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0030258, 'theta_0': -1.067342839554817, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4194.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0352325, 'theta_0': -0.8928254652390193, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4195.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0333344, 'theta_0': -0.91868428088986, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4196.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.9531923898082236, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4197.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0412, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907152, 'theta_0': -1.2298802040717556, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1364, 'Reactive_load': 0.322, 'Node_id': '5003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.99013, 'theta_0': -1.1037093401541194, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11689999999999999, 'Reactive_load': 0.0331, 'Node_id': '5004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151133, 'theta_0': -0.9575153783791959, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0895, 'Reactive_load': 0.0253, 'Node_id': '5005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963108, 'theta_0': -1.2218198023471476, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0019, 'Reactive_load': 0.2839, 'Node_id': '5006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963546, 'theta_0': -1.1470426889818999, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7204999999999999, 'Reactive_load': 0.20420000000000002, 'Node_id': '5007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0134722, 'theta_0': -0.7561409822534646, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.076, 'Reactive_load': 0.0215, 'Node_id': '5008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984786, 'theta_0': -1.2436178826004207, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6268, 'Reactive_load': 0.1776, 'Node_id': '5009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976655, 'theta_0': -1.2082160429319881, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3387, 'Reactive_load': 0.37929999999999997, 'Node_id': '5010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0062648, 'theta_0': -0.9618623826806756, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0764, 'Reactive_load': 0.0217, 'Node_id': '5011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9942004, 'theta_0': -1.1553896912229777, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8131999999999999, 'Reactive_load': 0.2304, 'Node_id': '5012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -0.8940619437475948, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1934, 'Reactive_load': 0.0548, 'Node_id': '5013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.4489, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9996384, 'theta_0': -1.0898559986559246, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.46759999999999996, 'Reactive_load': 0.1325, 'Node_id': '5014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0045957, 'theta_0': -1.0679557642815325, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.003355, 'theta_0': -1.1484313427479567, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8115999999999999, 'Reactive_load': 0.5133, 'Node_id': '5016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9945081, 'theta_0': -1.1705457290215686, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.13849999999999998, 'Reactive_load': 0.0393, 'Node_id': '5017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0057415, 'theta_0': -0.8440246805431488, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0037418, 'theta_0': -0.8745645565143008, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.8142036707843083, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0017908, 'theta_0': -1.0312833772775256, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9994071, 'theta_0': -1.0960919379002527, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4024, 'Reactive_load': 0.3973, 'Node_id': '5022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0017908, 'theta_0': -1.0312833772775256, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0052865, 'theta_0': -0.969252019467157, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5092, 'Reactive_load': 0.14429999999999998, 'Node_id': '5024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9960384, 'theta_0': -1.08440418348464, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7445999999999999, 'Reactive_load': 0.21100000000000002, 'Node_id': '5025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.991411, 'theta_0': -1.1745302807972864, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7111, 'Reactive_load': 0.20149999999999998, 'Node_id': '5026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9948773, 'theta_0': -1.1895085568115618, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7413, 'Reactive_load': 0.21, 'Node_id': '5027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0003104, 'theta_0': -1.1913418681111492, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0935, 'Reactive_load': 0.0265, 'Node_id': '5028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0036083, 'theta_0': -0.9504895554752928, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1277, 'Reactive_load': 0.0362, 'Node_id': '5029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9924906, 'theta_0': -1.1500840997364252, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7961, 'Reactive_load': 0.2255, 'Node_id': '5030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0038071, 'theta_0': -1.157869647010429, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4236, 'Reactive_load': 0.12, 'Node_id': '5031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9941745, 'theta_0': -1.2112859898197834, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47159999999999996, 'Reactive_load': 0.1336, 'Node_id': '5032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0118899, 'theta_0': -1.1100977688151943, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0060152, 'theta_0': -1.1486661593455199, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5535, 'Reactive_load': 0.1568, 'Node_id': '5034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0118899, 'theta_0': -1.1100977688151943, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0118899, 'theta_0': -1.1100977688151943, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.0991502434419373, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.098780512893195, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.1051991182970367, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.0953214273953749, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.099869388906929, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200037, 'theta_0': -1.0979575203377094, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9871375, 'theta_0': -1.2106021174589747, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6809999999999999, 'Reactive_load': 0.1929, 'Node_id': '5043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9983642, 'theta_0': -1.0959388201649751, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.175, 'Reactive_load': 0.0496, 'Node_id': '5044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123163, 'theta_0': -0.8492640716911732, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0101609, 'theta_0': -0.8846893860380451, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7711, 'Reactive_load': 0.21850000000000003, 'Node_id': '5046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0087755, 'theta_0': -1.039085260833328, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973323, 'theta_0': -1.1147589847419104, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8454, 'Reactive_load': 0.5229, 'Node_id': '5048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0073743, 'theta_0': -1.0401809261778525, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0038591, 'theta_0': -1.1181927978721617, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6459999999999999, 'Reactive_load': 0.183, 'Node_id': '5050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0144236, 'theta_0': -1.0267838311994215, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.006318, 'theta_0': -1.095519522265476, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9906846, 'theta_0': -1.2074978224915, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5278, 'Reactive_load': 0.4329, 'Node_id': '5053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966943, 'theta_0': -0.996266888854641, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21170000000000003, 'Reactive_load': 0.06, 'Node_id': '5054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0041557, 'theta_0': -0.8704158739757252, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028279, 'theta_0': -0.8852952421812899, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.40399999999999997, 'Reactive_load': 0.11449999999999999, 'Node_id': '5056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0041557, 'theta_0': -0.8704158739757252, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.8221100821090127, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9999403, 'theta_0': -1.114503520899296, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0048472, 'theta_0': -1.1551602153329257, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -1.212913771146656, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3542, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0020469, 'theta_0': -1.2876623483786338, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4374, 'Reactive_load': 0.12390000000000001, 'Node_id': '5062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0114735, 'theta_0': -1.0787550064815399, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.003275, 'theta_0': -1.151986368994759, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7706999999999999, 'Reactive_load': 0.2184, 'Node_id': '5064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0124335, 'theta_0': -1.076957596604666, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0114735, 'theta_0': -1.0787550064815399, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -1.0337919913712947, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -1.0316129478001796, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -1.0286338802065356, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927308, 'theta_0': -1.2197931085664395, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7465, 'Reactive_load': 0.2115, 'Node_id': '5070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011603, 'theta_0': -0.9510864929860599, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7411, 'Reactive_load': 0.21, 'Node_id': '5071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961472, 'theta_0': -1.2124258119943836, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7234, 'Reactive_load': 0.205, 'Node_id': '5072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.021853, 'theta_0': -0.8096358474258664, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0883, 'Reactive_load': 0.025, 'Node_id': '5073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9989481, 'theta_0': -1.2072745076137072, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3178, 'Reactive_load': 0.09, 'Node_id': '5074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0058604, 'theta_0': -0.9876338096958688, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.039599999999999996, 'Reactive_load': 0.011200000000000002, 'Node_id': '5075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0070069, 'theta_0': -1.243720682493363, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -1.2603747014823556, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.18489999999999998, 'Reactive_load': 0.0524, 'Node_id': '5077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8178, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9920351, 'theta_0': -1.1606530806482172, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6642, 'Reactive_load': 0.1882, 'Node_id': '5078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156433, 'theta_0': -0.9764862172304557, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38189999999999996, 'Reactive_load': 0.1082, 'Node_id': '5079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973988, 'theta_0': -0.9920026655195834, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3761, 'Reactive_load': 0.1066, 'Node_id': '5080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039916, 'theta_0': -0.968640246657748, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8983, 'Reactive_load': 0.2545, 'Node_id': '5081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9967862, 'theta_0': -1.2088168027138173, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0222, 'Reactive_load': 0.2896, 'Node_id': '5082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0106265, 'theta_0': -1.1047402537833948, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963451, 'theta_0': -1.1794587764456532, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.242, 'Reactive_load': 0.6352, 'Node_id': '5084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9901891, 'theta_0': -1.2117981566887814, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.513, 'Reactive_load': 0.1454, 'Node_id': '5085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.990891, 'theta_0': -1.2014387374602764, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9204000000000001, 'Reactive_load': 0.2608, 'Node_id': '5086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923699, 'theta_0': -1.107233299540236, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.041299999999999996, 'Reactive_load': 0.011699999999999999, 'Node_id': '5087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9995355, 'theta_0': -1.0259146572319284, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9995355, 'theta_0': -1.0259146572319284, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997266, 'theta_0': -1.2331412819659369, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0874, 'Reactive_load': 0.3081, 'Node_id': '5090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.75, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9924256, 'theta_0': -1.096813317386687, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0533, 'Reactive_load': 0.0151, 'Node_id': '5091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939751, 'theta_0': -1.165012145174828, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17620000000000002, 'Reactive_load': 0.0499, 'Node_id': '5092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0079873, 'theta_0': -1.1741467970540385, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47979999999999995, 'Reactive_load': 0.136, 'Node_id': '5093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.15, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0091252, 'theta_0': -1.058779591019417, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.026600000000000002, 'Reactive_load': 0.0075, 'Node_id': '5094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0061243, 'theta_0': -1.2074258102065627, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3224, 'Reactive_load': 0.09140000000000001, 'Node_id': '5095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998145, 'theta_0': -1.1915110603288375, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6174000000000001, 'Reactive_load': 0.17489999999999997, 'Node_id': '5096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9999972, 'theta_0': -1.179996861454043, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8256999999999999, 'Reactive_load': 0.23399999999999999, 'Node_id': '5097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0133532, 'theta_0': -1.1947208604495951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5622, 'Reactive_load': 0.4426, 'Node_id': '5098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0285145, 'theta_0': -1.2129380312232587, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6762999999999999, 'Reactive_load': 0.1916, 'Node_id': '5099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9981555, 'theta_0': -1.2047215796102302, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2245, 'Reactive_load': 0.0636, 'Node_id': '5100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231385, 'theta_0': -0.9142648803310075, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2263, 'Reactive_load': 0.0641, 'Node_id': '5101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0061088, 'theta_0': -1.1228897232353736, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9970338, 'theta_0': -1.1974200295910968, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.3449, 'Reactive_load': 0.6644, 'Node_id': '5103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9906466, 'theta_0': -1.1661061699098556, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7902, 'Reactive_load': 0.22390000000000002, 'Node_id': '5104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978506, 'theta_0': -1.2271360054355525, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0527, 'Reactive_load': 0.2983, 'Node_id': '5105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9978506, 'theta_0': -1.2271360054355525, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9978506, 'theta_0': -1.2271360054355525, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0128198, 'theta_0': -1.1852023583079938, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4012, 'Reactive_load': 0.1137, 'Node_id': '5108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.3723, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9875976, 'theta_0': -1.200736347156104, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5398, 'Reactive_load': 0.15289999999999998, 'Node_id': '5109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9917234, 'theta_0': -0.9452329030209662, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0344, 'Reactive_load': 0.0097, 'Node_id': '5110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9960216, 'theta_0': -1.2339029087449223, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3724, 'Reactive_load': 0.10550000000000001, 'Node_id': '5111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0124075, 'theta_0': -0.9722960133089752, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1425, 'Reactive_load': 0.0404, 'Node_id': '5112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0034564, 'theta_0': -1.1759790960626597, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2158, 'Reactive_load': 0.0611, 'Node_id': '5113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9941422, 'theta_0': -1.20038724639912, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5695, 'Reactive_load': 0.1613, 'Node_id': '5114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.012488, 'theta_0': -1.1660035270965459, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1773, 'Reactive_load': 0.050199999999999995, 'Node_id': '5115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.995135, 'theta_0': -1.2233398968591722, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4156, 'Reactive_load': 0.4011, 'Node_id': '5116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9865518, 'theta_0': -1.2009135329817664, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2828, 'Reactive_load': 0.3635, 'Node_id': '5117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.013265, 'theta_0': -0.7791711776921829, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08650000000000001, 'Reactive_load': 0.0245, 'Node_id': '5118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0020109, 'theta_0': -0.9756856172492734, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.044000000000000004, 'Reactive_load': 0.0125, 'Node_id': '5119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0265839, 'theta_0': -0.8111791373636499, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0230557, 'theta_0': -0.8218480034685334, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7666, 'Reactive_load': 0.21719999999999998, 'Node_id': '5121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9955858, 'theta_0': -1.2288339315450625, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.65, 'Reactive_load': 0.18420000000000003, 'Node_id': '5122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100373, 'theta_0': -0.9159203600331092, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074707, 'theta_0': -0.8993089445311578, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2669, 'Reactive_load': 0.0756, 'Node_id': '5124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008332, 'theta_0': -1.1685594570663365, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6792, 'Reactive_load': 0.1924, 'Node_id': '5125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9903022, 'theta_0': -1.2382257053296766, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7321, 'Reactive_load': 0.49079999999999996, 'Node_id': '5126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0141202, 'theta_0': -0.977050901056646, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.038900000000000004, 'Reactive_load': 0.011000000000000001, 'Node_id': '5127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9933463, 'theta_0': -1.2248654367984628, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9051, 'Reactive_load': 0.2565, 'Node_id': '5128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.029375, 'theta_0': -0.8877968051448808, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1489, 'Reactive_load': 0.042199999999999994, 'Node_id': '5129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927624, 'theta_0': -1.2067997256972873, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9979, 'Reactive_load': 0.2827, 'Node_id': '5130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231168, 'theta_0': -0.8785857776576033, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0160463, 'theta_0': -0.917294248313694, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0252, 'Reactive_load': 0.2905, 'Node_id': '5132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0133566, 'theta_0': -0.9652055783161632, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1216, 'Reactive_load': 0.0345, 'Node_id': '5133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0093804, 'theta_0': -0.8281771432949179, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2698, 'Reactive_load': 0.0764, 'Node_id': '5134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204654, 'theta_0': -0.9184316095740487, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2759, 'Reactive_load': 0.0782, 'Node_id': '5135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9954279, 'theta_0': -1.0939684830658138, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11939999999999999, 'Reactive_load': 0.0338, 'Node_id': '5136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0176217, 'theta_0': -0.9401489334428318, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151038, 'theta_0': -0.9627871628381373, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0504, 'Reactive_load': 0.29760000000000003, 'Node_id': '5138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105444, 'theta_0': -1.2476088397344935, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8637999999999999, 'Reactive_load': 0.2447, 'Node_id': '5139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.003568, 'theta_0': -0.9771163509035957, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08650000000000001, 'Reactive_load': 0.0245, 'Node_id': '5140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9970567, 'theta_0': -1.1625647048713414, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.45380000000000004, 'Reactive_load': 0.1286, 'Node_id': '5141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.000532, 'theta_0': -0.9731321307404356, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1553, 'Reactive_load': 0.044000000000000004, 'Node_id': '5142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.000363, 'theta_0': -1.1656134983686026, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8745, 'Reactive_load': 0.24780000000000002, 'Node_id': '5143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055435, 'theta_0': -0.97956827670326, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7343000000000001, 'Reactive_load': 0.20800000000000002, 'Node_id': '5144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9981724, 'theta_0': -1.1943849194751714, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1786, 'Reactive_load': 0.33390000000000003, 'Node_id': '5145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9940247, 'theta_0': -1.202682668524758, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9075, 'Reactive_load': 0.2571, 'Node_id': '5146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9911209, 'theta_0': -1.2296972935661465, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0812, 'Reactive_load': 0.3063, 'Node_id': '5147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923733, 'theta_0': -1.2298390666612862, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8341, 'Reactive_load': 0.23629999999999998, 'Node_id': '5148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9905308, 'theta_0': -1.2365086678648571, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9145, 'Reactive_load': 0.2591, 'Node_id': '5149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0005802, 'theta_0': -0.9502933106541984, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.041100000000000005, 'Reactive_load': 0.011699999999999999, 'Node_id': '5150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0106406, 'theta_0': -1.0022620599286565, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0533, 'Reactive_load': 0.0151, 'Node_id': '5151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9926236, 'theta_0': -1.2136112396223382, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6098, 'Reactive_load': 0.1728, 'Node_id': '5152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987727, 'theta_0': -1.0506544550328527, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1598, 'Reactive_load': 0.0453, 'Node_id': '5153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028218, 'theta_0': -1.0155655703525954, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1558, 'Reactive_load': 0.0441, 'Node_id': '5154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0132422, 'theta_0': -0.8676896871374026, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0371, 'Reactive_load': 0.0105, 'Node_id': '5155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0253708, 'theta_0': -0.8707789199134325, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0622, 'Reactive_load': 0.0176, 'Node_id': '5156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9873718, 'theta_0': -1.2061670438434395, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4663, 'Reactive_load': 0.4155, 'Node_id': '5157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0155484, 'theta_0': -1.2103081516530614, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6819, 'Reactive_load': 0.47659999999999997, 'Node_id': '5158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0095589, 'theta_0': -0.9241674421745103, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1454, 'Reactive_load': 0.0412, 'Node_id': '5159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961502, 'theta_0': -1.146392658555287, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5935, 'Reactive_load': 0.1681, 'Node_id': '5160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151899, 'theta_0': -1.2355581441009285, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9965000000000002, 'Reactive_load': 0.5657, 'Node_id': '5161.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978615, 'theta_0': -1.014811081970251, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0669, 'Reactive_load': 0.019, 'Node_id': '5162.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0121686, 'theta_0': -0.938826706908106, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1515, 'Reactive_load': 0.0429, 'Node_id': '5163.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0047874, 'theta_0': -1.0812754190010523, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5164.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0002843, 'theta_0': -1.1341737306351225, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5553, 'Reactive_load': 0.1573, 'Node_id': '5165.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0218809, 'theta_0': -1.048138353476593, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5166.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0055327, 'theta_0': -1.079859258845984, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5167.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927381, 'theta_0': -1.2452474616164229, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2567, 'Reactive_load': 0.35609999999999997, 'Node_id': '5168.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0217423, 'theta_0': -0.9186178711118216, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5169.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0165538, 'theta_0': -0.8899735274280905, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5170.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9933353, 'theta_0': -1.0831609854584445, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0106, 'Reactive_load': 0.003, 'Node_id': '5171.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0198779, 'theta_0': -1.1322429950566888, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7424, 'Reactive_load': 0.21030000000000001, 'Node_id': '5172.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907796, 'theta_0': -1.2110573516877723, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3636000000000001, 'Reactive_load': 0.38630000000000003, 'Node_id': '5173.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9946906, 'theta_0': -1.0648702839633943, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0432, 'Reactive_load': 0.012199999999999999, 'Node_id': '5174.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984568, 'theta_0': -1.1565810704236816, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1867, 'Reactive_load': 0.3362, 'Node_id': '5175.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965165, 'theta_0': -0.9133714986467891, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.073, 'Reactive_load': 0.0207, 'Node_id': '5176.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9896107, 'theta_0': -1.197223261171227, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9138, 'Reactive_load': 0.2589, 'Node_id': '5177.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9936603, 'theta_0': -1.2178530005699226, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8395999999999999, 'Reactive_load': 0.2379, 'Node_id': '5178.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168463, 'theta_0': -1.001072443510497, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5179.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0125232, 'theta_0': -1.027302228893849, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8835999999999999, 'Reactive_load': 0.2504, 'Node_id': '5180.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927711, 'theta_0': -1.2257529890829797, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7822, 'Reactive_load': 0.505, 'Node_id': '5181.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0057523, 'theta_0': -0.9714146743965957, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0382, 'Reactive_load': 0.0108, 'Node_id': '5182.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0015418, 'theta_0': -1.2459993669114746, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3909, 'Reactive_load': 0.39409999999999995, 'Node_id': '5183.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9838705, 'theta_0': -1.2201504298242003, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8023, 'Reactive_load': 0.2273, 'Node_id': '5184.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0005536, 'theta_0': -1.1404134223373423, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2572, 'Reactive_load': 0.0729, 'Node_id': '5185.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9925876, 'theta_0': -1.159436149827264, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2498, 'Reactive_load': 0.0708, 'Node_id': '5186.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984839, 'theta_0': -0.871749008818276, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.15410000000000001, 'Reactive_load': 0.0437, 'Node_id': '5187.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9903155, 'theta_0': -1.2221809458759705, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6481999999999999, 'Reactive_load': 0.1837, 'Node_id': '5188.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9985568, 'theta_0': -1.2232179507043355, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7884, 'Reactive_load': 0.2234, 'Node_id': '5189.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0068908, 'theta_0': -1.0088605040586238, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0338, 'Reactive_load': 0.0096, 'Node_id': '5190.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0174832, 'theta_0': -0.9894984845621145, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0758, 'Reactive_load': 0.0215, 'Node_id': '5191.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192435, 'theta_0': -0.8925060699859044, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5192.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0154042, 'theta_0': -0.9151900793674896, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3477000000000001, 'Reactive_load': 0.38189999999999996, 'Node_id': '5193.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996302, 'theta_0': -1.198361669629133, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5901, 'Reactive_load': 0.1672, 'Node_id': '5194.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0087413, 'theta_0': -1.1847727280593228, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5526, 'Reactive_load': 0.1566, 'Node_id': '5195.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0035311, 'theta_0': -0.90182035115173, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5196.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021341, 'theta_0': -0.9319494289768401, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5197.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0012469, 'theta_0': -0.889286007329145, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5198.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9940136, 'theta_0': -1.18689457719085, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2433, 'Reactive_load': 0.0689, 'Node_id': '5199.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9883868, 'theta_0': -0.9741066004217016, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6635, 'Reactive_load': 0.188, 'Node_id': '5200.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0050682, 'theta_0': -0.9616560149499197, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0481, 'Reactive_load': 0.013600000000000001, 'Node_id': '5201.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9998902, 'theta_0': -1.1039887324607787, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0864, 'Reactive_load': 0.0245, 'Node_id': '5202.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9908861, 'theta_0': -1.2330475752383974, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4701, 'Reactive_load': 0.4165, 'Node_id': '5203.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0041249, 'theta_0': -0.9571834865686366, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5204.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0033086, 'theta_0': -0.987471720968236, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5205.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.9167749954079333, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5206.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.9383467064572225, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5207.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100037, 'theta_0': -0.9334601161641962, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5208.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0068628, 'theta_0': -0.9852306658487977, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0359, 'Reactive_load': 0.0102, 'Node_id': '5209.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976451, 'theta_0': -1.193548871856881, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39640000000000003, 'Reactive_load': 0.11230000000000001, 'Node_id': '5210.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978483, 'theta_0': -1.0087830463464202, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0691, 'Reactive_load': 0.0196, 'Node_id': '5211.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9853498, 'theta_0': -1.1766336992519126, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3389, 'Reactive_load': 0.096, 'Node_id': '5212.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.991153, 'theta_0': -1.2241311418755638, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9209999999999999, 'Reactive_load': 0.2609, 'Node_id': '5213.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9930973, 'theta_0': -1.2167941965792002, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.021, 'Reactive_load': 0.2893, 'Node_id': '5214.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9950843, 'theta_0': -1.1838732377227223, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1998, 'Reactive_load': 0.056600000000000004, 'Node_id': '5215.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9922295, 'theta_0': -1.194424416276144, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6658, 'Reactive_load': 0.1886, 'Node_id': '5216.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.013617, 'theta_0': -1.2019730001976046, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0033, 'Reactive_load': 0.2843, 'Node_id': '5217.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.3877, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9977031, 'theta_0': -1.1629073304568005, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39289999999999997, 'Reactive_load': 0.11130000000000001, 'Node_id': '5218.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0023259, 'theta_0': -1.1880794114068587, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1335, 'Reactive_load': 0.0378, 'Node_id': '5219.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3053, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.006319, 'theta_0': -1.1737030645450115, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.40490000000000004, 'Reactive_load': 0.11470000000000001, 'Node_id': '5220.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056901, 'theta_0': -0.9688152159152603, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9129, 'Reactive_load': 0.2587, 'Node_id': '5221.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9912211, 'theta_0': -1.1562732117969223, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33140000000000003, 'Reactive_load': 0.09390000000000001, 'Node_id': '5222.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9928155, 'theta_0': -1.014391958603677, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3479, 'Reactive_load': 0.0986, 'Node_id': '5223.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978456, 'theta_0': -1.2338128148489342, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5487, 'Reactive_load': 0.4388, 'Node_id': '5224.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0112548, 'theta_0': -1.1864421354888555, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1306, 'Reactive_load': 0.037000000000000005, 'Node_id': '5225.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0295546, 'theta_0': -0.8923031231004825, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.054900000000000004, 'Reactive_load': 0.015600000000000001, 'Node_id': '5226.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0001819, 'theta_0': -1.2360101669239028, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7088, 'Reactive_load': 0.20079999999999998, 'Node_id': '5227.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978578, 'theta_0': -1.184202860605254, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.812, 'Reactive_load': 0.23010000000000003, 'Node_id': '5228.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966448, 'theta_0': -1.2096020088909971, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.43939999999999996, 'Reactive_load': 0.1245, 'Node_id': '5229.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9979742, 'theta_0': -1.2274762050133512, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6637000000000001, 'Reactive_load': 0.1881, 'Node_id': '5230.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0144161, 'theta_0': -0.9484582540253591, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1493, 'Reactive_load': 0.042300000000000004, 'Node_id': '5231.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9970929, 'theta_0': -1.2151520162859986, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4997, 'Reactive_load': 0.1416, 'Node_id': '5232.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021182, 'theta_0': -1.2066581445883655, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9876999999999999, 'Reactive_load': 0.2799, 'Node_id': '5233.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9929776, 'theta_0': -1.184488379017588, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4265999999999999, 'Reactive_load': 0.4042, 'Node_id': '5234.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0163012, 'theta_0': -0.9401864929283348, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.20370000000000002, 'Reactive_load': 0.057699999999999994, 'Node_id': '5235.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064601, 'theta_0': -0.7924533776459551, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5236.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9995378, 'theta_0': -0.7513679430815732, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5237.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.003443, 'theta_0': -0.7760930626634881, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5238.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0302803, 'theta_0': -0.8435502651458718, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5239.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0319378, 'theta_0': -0.8750378374475641, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5240.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0340183, 'theta_0': -0.9014757533442163, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5241.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0340183, 'theta_0': -0.9014757533442163, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5242.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0248149, 'theta_0': -0.8157445521677241, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5243.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923735, 'theta_0': -1.1801405195047747, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6513, 'Reactive_load': 0.1845, 'Node_id': '5244.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0025837, 'theta_0': -1.2292036970003901, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3896000000000002, 'Reactive_load': 0.3937, 'Node_id': '5245.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0092224, 'theta_0': -0.9699199569718951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2121, 'Reactive_load': 0.0601, 'Node_id': '5246.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0047736, 'theta_0': -1.000262855083667, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0326, 'Reactive_load': 0.0092, 'Node_id': '5247.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961372, 'theta_0': -1.1963897442803522, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0102, 'Reactive_load': 0.2862, 'Node_id': '5248.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9937061, 'theta_0': -1.124896380636269, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1468, 'Reactive_load': 0.3249, 'Node_id': '5249.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.001048, 'theta_0': -1.186758633495412, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6509, 'Reactive_load': 0.4677, 'Node_id': '5250.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9962977, 'theta_0': -1.2095988323917581, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2312, 'Reactive_load': 0.3488, 'Node_id': '5251.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934338, 'theta_0': -1.2352699902414244, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8079999999999999, 'Reactive_load': 0.2289, 'Node_id': '5252.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9920041, 'theta_0': -1.199536974347426, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2668000000000001, 'Reactive_load': 0.3589, 'Node_id': '5253.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9982401, 'theta_0': -1.187543071727721, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8294, 'Reactive_load': 0.235, 'Node_id': '5254.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938103, 'theta_0': -0.9873529862192229, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0943, 'Reactive_load': 0.026699999999999998, 'Node_id': '5255.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9917716, 'theta_0': -1.1282897544877517, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.013000000000000001, 'Reactive_load': 0.0037, 'Node_id': '5256.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9916958, 'theta_0': -1.116972219313072, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.099, 'Reactive_load': 0.027999999999999997, 'Node_id': '5257.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9905157, 'theta_0': -1.2147282852501995, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0019, 'Reactive_load': 0.2839, 'Node_id': '5258.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028934, 'theta_0': -1.1953690059206157, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09210000000000002, 'Reactive_load': 0.026099999999999998, 'Node_id': '5259.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089631, 'theta_0': -0.9335012710279583, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5260.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103577, 'theta_0': -0.9554910233396853, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5261.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.8583271301644668, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5262.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.8881844951316263, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5263.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300037, 'theta_0': -0.947483749437508, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1213, 'Reactive_load': 0.0344, 'Node_id': '5264.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8093, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039999, 'theta_0': -1.1696484028932408, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0393999999999999, 'Reactive_load': 0.2945, 'Node_id': '5265.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0044384, 'theta_0': -1.1982763579352953, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3612, 'Reactive_load': 0.3857, 'Node_id': '5266.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0027853, 'theta_0': -0.9949313978175999, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3932, 'Reactive_load': 0.1114, 'Node_id': '5267.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907612, 'theta_0': -1.1146345078596582, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3223, 'Reactive_load': 0.0913, 'Node_id': '5268.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9955747, 'theta_0': -1.120251640617692, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8695999999999999, 'Reactive_load': 0.2464, 'Node_id': '5269.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0001903, 'theta_0': -1.0152158761836658, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2241, 'Reactive_load': 0.0635, 'Node_id': '5270.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123768, 'theta_0': -0.9322444245270122, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1001, 'Reactive_load': 0.028300000000000002, 'Node_id': '5271.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0001553, 'theta_0': -1.227784988664614, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1883, 'Reactive_load': 0.0533, 'Node_id': '5272.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192129, 'theta_0': -1.2299771396584114, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.16, 'Reactive_load': 0.3287, 'Node_id': '5273.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9933622, 'theta_0': -1.2002398707970816, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11869999999999999, 'Reactive_load': 0.0336, 'Node_id': '5274.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939932, 'theta_0': -1.0279075439850258, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0475, 'Reactive_load': 0.013500000000000002, 'Node_id': '5275.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0088725, 'theta_0': -0.9408309033947561, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1054, 'Reactive_load': 0.029900000000000003, 'Node_id': '5276.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996073, 'theta_0': -1.206651547243793, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.048, 'Reactive_load': 0.2969, 'Node_id': '5277.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0131096, 'theta_0': -0.9112590917465154, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5069, 'Reactive_load': 0.1436, 'Node_id': '5278.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204489, 'theta_0': -0.8761232926159643, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5279.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.016047, 'theta_0': -0.8989387252902248, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9462999999999999, 'Reactive_load': 0.2681, 'Node_id': '5280.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0138745, 'theta_0': -0.8250212389414617, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5281.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0151555, 'theta_0': -0.8329555929874906, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5282.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0204489, 'theta_0': -0.8761232926159643, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5283.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9937822, 'theta_0': -1.1772686325804955, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4465, 'Reactive_load': 0.1265, 'Node_id': '5284.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939746, 'theta_0': -1.044674311243697, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1015, 'Reactive_load': 0.0288, 'Node_id': '5285.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0085982, 'theta_0': -0.9334137602192633, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3018, 'Reactive_load': 0.0855, 'Node_id': '5286.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9915824, 'theta_0': -1.2122172451487703, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8678, 'Reactive_load': 0.5292, 'Node_id': '5287.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927968, 'theta_0': -1.1152146378497285, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1361, 'Reactive_load': 0.038599999999999995, 'Node_id': '5288.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9979111, 'theta_0': -1.2301130309939716, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6976, 'Reactive_load': 0.1977, 'Node_id': '5289.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9971602, 'theta_0': -1.0389347610919286, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3034, 'Reactive_load': 0.086, 'Node_id': '5290.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9931786, 'theta_0': -1.1813611678770346, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4417, 'Reactive_load': 0.40850000000000003, 'Node_id': '5291.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9916379, 'theta_0': -1.1474738027604352, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6204999999999999, 'Reactive_load': 0.17579999999999998, 'Node_id': '5292.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.990565, 'theta_0': -1.0781575977318747, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6297, 'Reactive_load': 0.1784, 'Node_id': '5293.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9928593, 'theta_0': -1.2403908037200682, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1116, 'Reactive_load': 0.31489999999999996, 'Node_id': '5294.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0132636, 'theta_0': -1.0956733206791618, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5295.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0048089, 'theta_0': -1.1410748148573857, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0231999999999999, 'Reactive_load': 0.2899, 'Node_id': '5296.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0132636, 'theta_0': -1.0956733381324544, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5297.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0195717, 'theta_0': -1.0837466481422438, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5298.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0260017, 'theta_0': -1.071272605445315, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5299.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0238991, 'theta_0': -1.0753848629624014, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5300.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0260985, 'theta_0': -1.0714106435358552, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5301.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400038, 'theta_0': -1.0187018397519667, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5302.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400038, 'theta_0': -1.0382834042150844, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5303.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0096946, 'theta_0': -1.100731232491564, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5304.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0007762, 'theta_0': -1.1706328733111206, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8941999999999999, 'Reactive_load': 0.5367000000000001, 'Node_id': '5305.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.002542, 'theta_0': -1.1194976932874152, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4081, 'Reactive_load': 0.11560000000000001, 'Node_id': '5306.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0086242, 'theta_0': -1.2159916418292556, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4356, 'Reactive_load': 0.4067, 'Node_id': '5307.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996695, 'theta_0': -1.1934056152318773, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1535, 'Reactive_load': 0.0435, 'Node_id': '5308.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9902391, 'theta_0': -1.233233225910932, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9884999999999999, 'Reactive_load': 0.2801, 'Node_id': '5309.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078955, 'theta_0': -1.2073600287470552, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6999, 'Reactive_load': 0.19829999999999998, 'Node_id': '5310.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996606, 'theta_0': -1.2421183655202772, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4096, 'Reactive_load': 0.1161, 'Node_id': '5311.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.987907, 'theta_0': -1.1565198093669367, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8181, 'Reactive_load': 0.2318, 'Node_id': '5312.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0023546, 'theta_0': -1.1635613925939854, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1612, 'Reactive_load': 0.045700000000000005, 'Node_id': '5313.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0079145, 'theta_0': -0.9782711130965928, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1199, 'Reactive_load': 0.034, 'Node_id': '5314.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9926555, 'theta_0': -1.18749860073838, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.31739999999999996, 'Reactive_load': 0.08990000000000001, 'Node_id': '5315.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0250721, 'theta_0': -1.2267975337337131, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7619, 'Reactive_load': 0.49920000000000003, 'Node_id': '5316.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.7117, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0090041, 'theta_0': -0.9545881645176286, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5317.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055601, 'theta_0': -1.0097126959724951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5318.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.9397627444392431, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5319.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0090041, 'theta_0': -0.9545881645176286, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5320.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -0.9404066662134738, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5321.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0114442, 'theta_0': -1.1050262085280416, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5322.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0012527, 'theta_0': -1.1799935802350494, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1944, 'Reactive_load': 0.6217, 'Node_id': '5323.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0396827, 'theta_0': -1.0497688575170985, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5324.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0392848, 'theta_0': -1.0495140568995998, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5325.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0217061, 'theta_0': -1.0857298483179922, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5326.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008092, 'theta_0': -0.9876877229164629, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0401, 'Reactive_load': 0.011399999999999999, 'Node_id': '5327.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9993733, 'theta_0': -1.00145796928897, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1364, 'Reactive_load': 0.038599999999999995, 'Node_id': '5328.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9893301, 'theta_0': -1.2178131023432217, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4825, 'Reactive_load': 0.42, 'Node_id': '5329.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998589, 'theta_0': -1.1831395187584766, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6478, 'Reactive_load': 0.18350000000000002, 'Node_id': '5330.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9951334, 'theta_0': -1.1903526678509964, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2636, 'Reactive_load': 0.0747, 'Node_id': '5331.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.994045, 'theta_0': -1.1960270474084953, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8512000000000001, 'Reactive_load': 0.2412, 'Node_id': '5332.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9858886, 'theta_0': -1.1945811293896804, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0812, 'Reactive_load': 0.3063, 'Node_id': '5333.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0170015, 'theta_0': -0.7252411146707963, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2395, 'Reactive_load': 0.0678, 'Node_id': '5334.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9905902, 'theta_0': -1.234610674663191, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6671, 'Reactive_load': 0.47229999999999994, 'Node_id': '5335.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961507, 'theta_0': -1.1892302989689163, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7414000000000001, 'Reactive_load': 0.2101, 'Node_id': '5336.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0004874, 'theta_0': -0.9526539906405684, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.18780000000000002, 'Reactive_load': 0.053200000000000004, 'Node_id': '5337.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9861986, 'theta_0': -1.1283325848675956, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8026000000000001, 'Reactive_load': 0.2274, 'Node_id': '5338.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192148, 'theta_0': -1.222823994985575, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8001, 'Reactive_load': 0.2267, 'Node_id': '5339.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9971876, 'theta_0': -1.2299892173368352, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3394, 'Reactive_load': 0.0962, 'Node_id': '5340.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0147081, 'theta_0': -1.2045711671352934, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.387, 'Reactive_load': 0.1097, 'Node_id': '5341.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0232016, 'theta_0': -1.1845739176042283, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5342.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -1.1098799517245452, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5343.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0030215, 'theta_0': -1.2468392193475342, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9470999999999999, 'Reactive_load': 0.26839999999999997, 'Node_id': '5344.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9918492, 'theta_0': -1.0803489633275087, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3312, 'Reactive_load': 0.09380000000000001, 'Node_id': '5345.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990306, 'theta_0': -1.1816720284701072, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9129999999999999, 'Reactive_load': 0.2587, 'Node_id': '5346.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.6, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011644, 'theta_0': -0.7726103977674738, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1868, 'Reactive_load': 0.0529, 'Node_id': '5347.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934822, 'theta_0': -1.1191197596911884, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0925, 'Reactive_load': 0.0262, 'Node_id': '5348.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9948937, 'theta_0': -1.197957922613269, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6083, 'Reactive_load': 0.1723, 'Node_id': '5349.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0063841, 'theta_0': -1.1158353641982004, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5350.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9996762, 'theta_0': -1.171988243648342, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7428, 'Reactive_load': 0.4938, 'Node_id': '5351.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9980317, 'theta_0': -1.211887779345871, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.41840000000000005, 'Reactive_load': 0.1185, 'Node_id': '5352.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.004548, 'theta_0': -1.1916412618910364, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5658, 'Reactive_load': 0.1603, 'Node_id': '5353.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0054482, 'theta_0': -1.2335682244075596, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7772, 'Reactive_load': 0.2202, 'Node_id': '5354.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9954015, 'theta_0': -1.131127118799426, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0565, 'Reactive_load': 0.016, 'Node_id': '5355.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987673, 'theta_0': -1.2232098000167286, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2243000000000002, 'Reactive_load': 0.3469, 'Node_id': '5356.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934368, 'theta_0': -1.1958979104971401, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9104000000000001, 'Reactive_load': 0.2579, 'Node_id': '5357.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0189112, 'theta_0': -0.8666542531053644, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5358.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0164108, 'theta_0': -0.9083308830472744, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5359.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100035, 'theta_0': -0.7569757383281085, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5360.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0027436, 'theta_0': -0.9717538093235506, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5361.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0007578, 'theta_0': -0.9886370424032076, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33909999999999996, 'Reactive_load': 0.09609999999999999, 'Node_id': '5362.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0024188, 'theta_0': -0.9683474327691409, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5363.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0005945, 'theta_0': -0.9516182773558499, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5364.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.9298501818591263, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5365.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.9240858131253945, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5366.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.9254953061227201, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5367.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.9409652239339896, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5368.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.9215793108800204, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5369.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9930941, 'theta_0': -1.1854394438335847, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0598999999999998, 'Reactive_load': 0.3003, 'Node_id': '5370.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0011586, 'theta_0': -1.1710251360605064, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.69, 'Reactive_load': 0.1955, 'Node_id': '5371.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973885, 'theta_0': -1.2401905795482795, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39799999999999996, 'Reactive_load': 0.1128, 'Node_id': '5372.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9831845, 'theta_0': -1.231828078783444, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5699, 'Reactive_load': 0.4448, 'Node_id': '5373.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0085274, 'theta_0': -1.1823843671510161, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6909000000000001, 'Reactive_load': 0.19579999999999997, 'Node_id': '5374.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9947039, 'theta_0': -1.1817489102236576, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6209, 'Reactive_load': 0.1759, 'Node_id': '5375.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0009075, 'theta_0': -1.1904183445907488, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47850000000000004, 'Reactive_load': 0.1356, 'Node_id': '5376.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0063759, 'theta_0': -0.9740776628627036, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5761999999999999, 'Reactive_load': 0.16329999999999997, 'Node_id': '5377.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.017317, 'theta_0': -0.9911225134310951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6806, 'Reactive_load': 0.1928, 'Node_id': '5378.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.1305, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9999844, 'theta_0': -1.2134779837339484, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6473, 'Reactive_load': 0.4667, 'Node_id': '5379.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161422, 'theta_0': -1.0512575186492945, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5380.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0075076, 'theta_0': -1.0784203570507624, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7979999999999999, 'Reactive_load': 0.2261, 'Node_id': '5381.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0310511, 'theta_0': -1.0427643101767772, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5382.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0330827, 'theta_0': -1.0415910300404163, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5383.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100859, 'theta_0': -1.11631637694005, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5384.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0073361, 'theta_0': -1.1610407880882552, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.4097, 'Reactive_load': 0.6828, 'Node_id': '5385.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9996083, 'theta_0': -1.2168470975088281, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6355, 'Reactive_load': 0.18, 'Node_id': '5386.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.6, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9880517, 'theta_0': -1.206744328946829, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7775, 'Reactive_load': 0.22030000000000002, 'Node_id': '5387.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0163, 'theta_0': -0.9431891399201733, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5388.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0120591, 'theta_0': -0.9659601190583854, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8747, 'Reactive_load': 0.24780000000000002, 'Node_id': '5389.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0116505, 'theta_0': -0.9276865146387214, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5390.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0109149, 'theta_0': -0.9516522414630938, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5391.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0030516, 'theta_0': -0.9741900097066544, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07490000000000001, 'Reactive_load': 0.0212, 'Node_id': '5392.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039687, 'theta_0': -1.000660266554346, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1492, 'Reactive_load': 0.042300000000000004, 'Node_id': '5393.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.023411, 'theta_0': -0.6967861392109068, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5394.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0224355, 'theta_0': -0.6602402187644995, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5395.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.016742, 'theta_0': -0.7082727618835197, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5396.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074631, 'theta_0': -0.6733620404133059, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5397.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074631, 'theta_0': -0.6733620404133059, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5398.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0271095, 'theta_0': -0.6841850714510179, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5399.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9967846, 'theta_0': -1.1691803230411484, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0827, 'Reactive_load': 0.3068, 'Node_id': '5400.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0088196, 'theta_0': -0.9672067204766224, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5401.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9998415, 'theta_0': -1.0282069377649128, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6597, 'Reactive_load': 0.1869, 'Node_id': '5402.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0218991, 'theta_0': -0.9417635898937294, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5403.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178483, 'theta_0': -0.9522897927855547, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5404.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0179518, 'theta_0': -0.9506798138170526, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5405.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0214877, 'theta_0': -0.9425145527109849, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5406.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0120751, 'theta_0': -0.999676965507065, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21280000000000002, 'Reactive_load': 0.0603, 'Node_id': '5407.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064941, 'theta_0': -1.1678433310209506, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2657, 'Reactive_load': 0.0753, 'Node_id': '5408.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9845484, 'theta_0': -1.1970952063640083, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9256, 'Reactive_load': 0.2622, 'Node_id': '5409.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0241397, 'theta_0': -0.8899490928185627, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5410.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0246621, 'theta_0': -0.9396615153426274, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2006999999999999, 'Reactive_load': 0.34020000000000006, 'Node_id': '5411.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.7461000000000002, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9954182, 'theta_0': -1.2028470436337106, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4459, 'Reactive_load': 0.4097, 'Node_id': '5412.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0054775, 'theta_0': -0.892863984655611, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5413.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.016942, 'theta_0': -0.9135157152028814, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5414.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0009421, 'theta_0': -0.8542813347852962, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5415.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0007665, 'theta_0': -0.8538510762180946, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5416.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0013355, 'theta_0': -0.8585059740529186, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5417.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0020184, 'theta_0': -0.8604682128243508, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5418.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0035218, 'theta_0': -0.8739093075532245, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5419.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0009765, 'theta_0': -1.1645217076550176, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1467, 'Reactive_load': 0.0416, 'Node_id': '5420.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0195162, 'theta_0': -1.1763236764168807, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.020099999999999996, 'Reactive_load': 0.005699999999999999, 'Node_id': '5421.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.6752, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0255171, 'theta_0': -1.2203664143191346, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8295, 'Reactive_load': 0.5183, 'Node_id': '5422.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.7928, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0244054, 'theta_0': -1.2087574091593718, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5423.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0255171, 'theta_0': -1.2203664143191346, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5424.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9943402, 'theta_0': -1.2115131793285157, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5293, 'Reactive_load': 0.15, 'Node_id': '5425.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923213, 'theta_0': -1.1331932221146446, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4386, 'Reactive_load': 0.1243, 'Node_id': '5426.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9890491, 'theta_0': -1.1549308441626285, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2531, 'Reactive_load': 0.0717, 'Node_id': '5427.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9981876, 'theta_0': -1.1817490498499976, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8526, 'Reactive_load': 0.2416, 'Node_id': '5428.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9886279, 'theta_0': -1.2020946147398834, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0617, 'Reactive_load': 0.30079999999999996, 'Node_id': '5429.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0050924, 'theta_0': -1.1245224089374442, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6059, 'Reactive_load': 0.17170000000000002, 'Node_id': '5430.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934421, 'theta_0': -1.2073223470885046, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8443, 'Reactive_load': 0.23920000000000002, 'Node_id': '5431.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990176, 'theta_0': -1.1012854443422422, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4202, 'Reactive_load': 0.1191, 'Node_id': '5432.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934057, 'theta_0': -1.215835871193515, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1698, 'Reactive_load': 0.33140000000000003, 'Node_id': '5433.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0049218, 'theta_0': -1.1616487211733098, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09960000000000001, 'Reactive_load': 0.0282, 'Node_id': '5434.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0176369, 'theta_0': -0.9478250660260281, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.045, 'Reactive_load': 0.0127, 'Node_id': '5435.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961094, 'theta_0': -1.2322458233399087, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7289, 'Reactive_load': 0.2065, 'Node_id': '5436.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9982816, 'theta_0': -1.0161863316076523, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0901, 'Reactive_load': 0.0255, 'Node_id': '5437.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9952131, 'theta_0': -1.1916512277210651, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5024000000000001, 'Reactive_load': 0.1424, 'Node_id': '5438.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9994975, 'theta_0': -1.10014789109567, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.139, 'Reactive_load': 0.32270000000000004, 'Node_id': '5439.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0016324, 'theta_0': -1.2187542187824822, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6712, 'Reactive_load': 0.47350000000000003, 'Node_id': '5440.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9958285, 'theta_0': -1.1594673912208746, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4518, 'Reactive_load': 0.4113, 'Node_id': '5441.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9931796, 'theta_0': -1.1986855852850107, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4372, 'Reactive_load': 0.12390000000000001, 'Node_id': '5442.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965091, 'theta_0': -1.1873157076860634, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5234000000000001, 'Reactive_load': 0.1483, 'Node_id': '5443.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0100036, 'theta_0': -1.0388630978728417, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5444.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997415, 'theta_0': -1.105996960658001, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5445.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0032225, 'theta_0': -1.0057513919824135, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.40659999999999996, 'Reactive_load': 0.1152, 'Node_id': '5446.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9874028, 'theta_0': -1.1843752118688886, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5446, 'Reactive_load': 0.4376, 'Node_id': '5447.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0088716, 'theta_0': -1.1076684450293435, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5448.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105325, 'theta_0': -1.1671641361425371, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.6699, 'Reactive_load': 0.7565000000000001, 'Node_id': '5449.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9914889, 'theta_0': -1.1804426359982951, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.171, 'Reactive_load': 0.0484, 'Node_id': '5450.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0250281, 'theta_0': -0.889572206419887, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5451.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200069, 'theta_0': -0.9121507979146518, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0659999999999998, 'Reactive_load': 0.302, 'Node_id': '5452.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9967829, 'theta_0': -1.1379125400521597, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6792, 'Reactive_load': 0.1924, 'Node_id': '5453.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0027612, 'theta_0': -1.144913858533365, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47759999999999997, 'Reactive_load': 0.1353, 'Node_id': '5454.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965209, 'theta_0': -1.0689105990087135, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0947, 'Reactive_load': 0.0268, 'Node_id': '5455.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0187582, 'theta_0': -0.8371492875674751, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12619999999999998, 'Reactive_load': 0.0358, 'Node_id': '5456.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9929463, 'theta_0': -1.2067099110539798, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0354, 'Reactive_load': 0.2934, 'Node_id': '5457.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105444, 'theta_0': -0.9341400266276032, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5458.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0108119, 'theta_0': -0.9593972621918663, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5459.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9928965, 'theta_0': -1.0076216868088508, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1416, 'Reactive_load': 0.0401, 'Node_id': '5460.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074313, 'theta_0': -0.9294508679795624, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.728, 'Reactive_load': 0.20629999999999998, 'Node_id': '5461.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939025, 'theta_0': -1.2086654128544994, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8604999999999999, 'Reactive_load': 0.2438, 'Node_id': '5462.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.992277, 'theta_0': -1.2286305657806202, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6033000000000002, 'Reactive_load': 0.4543, 'Node_id': '5463.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0077144, 'theta_0': -1.1112543112440283, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5464.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0110945, 'theta_0': -1.1776444717816201, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1816999999999998, 'Reactive_load': 0.6181, 'Node_id': '5465.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0031814, 'theta_0': -0.9795709121504305, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07440000000000001, 'Reactive_load': 0.021099999999999997, 'Node_id': '5466.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9986114, 'theta_0': -1.231047270835979, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2823, 'Reactive_load': 0.08, 'Node_id': '5467.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938499, 'theta_0': -1.2114640657633646, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4213, 'Reactive_load': 0.11939999999999999, 'Node_id': '5468.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927451, 'theta_0': -1.1947771996778496, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.085, 'Reactive_load': 0.0241, 'Node_id': '5469.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0090765, 'theta_0': -1.000551166022804, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7051999999999999, 'Reactive_load': 0.1998, 'Node_id': '5470.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0220386, 'theta_0': -0.9305345429121258, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0144, 'Reactive_load': 0.0040999999999999995, 'Node_id': '5471.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0004865, 'theta_0': -0.7985074884888104, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.045700000000000005, 'Reactive_load': 0.0129, 'Node_id': '5472.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.000825, 'theta_0': -1.042536719242317, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0403, 'Reactive_load': 0.011399999999999999, 'Node_id': '5473.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9972705, 'theta_0': -1.0400988258898387, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0311, 'Reactive_load': 0.0088, 'Node_id': '5474.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961808, 'theta_0': -1.1849403669339769, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5928, 'Reactive_load': 0.168, 'Node_id': '5475.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9901986, 'theta_0': -1.0854647328046143, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1097, 'Reactive_load': 0.0311, 'Node_id': '5476.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0119711, 'theta_0': -0.9638829678090018, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7903, 'Reactive_load': 0.22390000000000002, 'Node_id': '5477.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966846, 'theta_0': -1.2187468709463314, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3794, 'Reactive_load': 0.3908, 'Node_id': '5478.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0041888, 'theta_0': -1.0748653482572528, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '5479.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9941285, 'theta_0': -1.136415466432969, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7923, 'Reactive_load': 0.5078, 'Node_id': '5480.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9880417, 'theta_0': -1.1600528619184562, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5818, 'Reactive_load': 0.1649, 'Node_id': '5481.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064908, 'theta_0': -1.2438666443787074, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6918000000000001, 'Reactive_load': 0.196, 'Node_id': '5482.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.6, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0059034, 'theta_0': -1.1841403254581553, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8462999999999999, 'Reactive_load': 0.2398, 'Node_id': '5483.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0186841, 'theta_0': -0.7696155873039766, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0629, 'Reactive_load': 0.0178, 'Node_id': '5484.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105305, 'theta_0': -1.044209530063891, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046900000000000004, 'Reactive_load': 0.013300000000000001, 'Node_id': '5485.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.9916058998207276, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0381, 'Reactive_load': 0.29410000000000003, 'Node_id': '6001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.3722, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.010498, 'theta_0': -0.9910322624554745, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3248, 'Reactive_load': 0.092, 'Node_id': '6002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0317175, 'theta_0': -0.8068814735201241, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0193868, 'theta_0': -0.7445164785699442, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0225736, 'theta_0': -0.8004536178712467, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7548, 'Reactive_load': 0.2139, 'Node_id': '6005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7824632875404647, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0317175, 'theta_0': -0.8068814735201241, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7803480183002176, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7624101525667507, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7760127251580189, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0128716, 'theta_0': -0.9124149709502336, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2402, 'Reactive_load': 0.0681, 'Node_id': '6011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.8672417134782932, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1351, 'Reactive_load': 0.0383, 'Node_id': '6012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.9373, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9945611, 'theta_0': -1.0085655434150367, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9489, 'Reactive_load': 0.26890000000000003, 'Node_id': '6013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0198763, 'theta_0': -0.9021005288565528, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.055099999999999996, 'Reactive_load': 0.015600000000000001, 'Node_id': '6014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089739, 'theta_0': -0.979180132930909, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24760000000000001, 'Reactive_load': 0.0702, 'Node_id': '6015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0116521, 'theta_0': -0.9927885349218789, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7643, 'Reactive_load': 0.4999, 'Node_id': '6016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0073344, 'theta_0': -0.9624801943292965, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8331000000000001, 'Reactive_load': 0.23600000000000002, 'Node_id': '6017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9932218, 'theta_0': -0.9864772847203273, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0246, 'Reactive_load': 0.006999999999999999, 'Node_id': '6018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9925, 'theta_0': -0.9981223658357286, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5907, 'Reactive_load': 0.1674, 'Node_id': '6019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0191373, 'theta_0': -0.8988716871936557, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1653, 'Reactive_load': 0.046799999999999994, 'Node_id': '6020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273193, 'theta_0': -0.8536131005745853, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161758, 'theta_0': -0.9006638261761886, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0004502, 'theta_0': -0.9665878441771578, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3289, 'Reactive_load': 0.3765, 'Node_id': '6023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0232905, 'theta_0': -1.0287612368820538, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8499, 'Reactive_load': 0.2408, 'Node_id': '6024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.9972911527794663, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6463, 'Reactive_load': 0.18309999999999998, 'Node_id': '6025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.702, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0015763, 'theta_0': -0.9762888728519328, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2212, 'Reactive_load': 0.06269999999999999, 'Node_id': '6026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0180984, 'theta_0': -0.9143629329283846, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6498, 'Reactive_load': 0.1841, 'Node_id': '6027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9915569, 'theta_0': -0.9936724569215515, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0334999999999999, 'Reactive_load': 0.2928, 'Node_id': '6028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0211262, 'theta_0': -0.8125054130622403, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0698, 'Reactive_load': 0.019799999999999998, 'Node_id': '6029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161134, 'theta_0': -0.6247329088493689, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0120026, 'theta_0': -0.6709801895831092, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2918, 'Reactive_load': 0.0827, 'Node_id': '6031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.99853, 'theta_0': -1.0392071197217023, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.364, 'Reactive_load': 0.10310000000000001, 'Node_id': '6032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168878, 'theta_0': -0.8839140584244316, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0065201, 'theta_0': -0.9217029325509393, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9956221, 'theta_0': -0.9744848307239014, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.0637, 'Reactive_load': 0.5847, 'Node_id': '6035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0136542, 'theta_0': -0.9027584481713845, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.009548, 'theta_0': -0.9496185314587925, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4753, 'Reactive_load': 0.418, 'Node_id': '6037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0258275, 'theta_0': -0.8590160988866915, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0071441, 'theta_0': -0.9259171696562197, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9937083, 'theta_0': -0.9937915058298299, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8078, 'Reactive_load': 0.2289, 'Node_id': '6040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0298227, 'theta_0': -0.8590936438653577, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.8592031981825055, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9993195, 'theta_0': -0.9677141400500547, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.483, 'Reactive_load': 0.4202, 'Node_id': '6043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9977212, 'theta_0': -0.9900111750298878, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.183, 'Reactive_load': 0.0518, 'Node_id': '6044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0261295, 'theta_0': -0.8527859366821876, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204439, 'theta_0': -0.9071322083702972, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.014223, 'theta_0': -0.9849562477305066, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4985, 'Reactive_load': 0.42460000000000003, 'Node_id': '6047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261295, 'theta_0': -0.8527859366821876, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261295, 'theta_0': -0.8527859366821876, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261295, 'theta_0': -0.8527859366821876, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0334103, 'theta_0': -0.8391706755472574, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261295, 'theta_0': -0.8527859366821876, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0344124, 'theta_0': -0.8371042929793586, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400031, 'theta_0': -0.8230699957443172, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0256681, 'theta_0': -1.0248486101248102, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0493, 'Reactive_load': 0.013999999999999999, 'Node_id': '6055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0245988, 'theta_0': -0.8108920307016967, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192377, 'theta_0': -0.8606547012549264, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0117268, 'theta_0': -0.901217304988581, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7863999999999998, 'Reactive_load': 0.5061, 'Node_id': '6058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0261482, 'theta_0': -0.96622666574175, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0344, 'Reactive_load': 0.0097, 'Node_id': '6059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.008518, 'theta_0': -1.009138081222861, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7453, 'Reactive_load': 0.2112, 'Node_id': '6060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.990086, 'theta_0': -1.0071666969261484, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.479, 'Reactive_load': 0.4191, 'Node_id': '6061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0188419, 'theta_0': -0.5849424736833818, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.015382, 'theta_0': -0.6374095842724542, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0087123, 'theta_0': -0.6937139059417913, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8248000000000001, 'Reactive_load': 0.23370000000000002, 'Node_id': '6064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9976626, 'theta_0': -0.9898266239147818, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14859999999999998, 'Reactive_load': 0.0421, 'Node_id': '6065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0262024, 'theta_0': -0.8215309993164935, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0199111, 'theta_0': -0.8620127419459032, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0127663, 'theta_0': -0.9288342780614179, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8073639174319603, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.7850098799987572, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8119207277628222, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0262024, 'theta_0': -0.8215309993164935, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8015134341529351, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300031, 'theta_0': -0.8017073576861243, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200064, 'theta_0': -0.5584040442751073, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0208114, 'theta_0': -0.5965404658155644, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204816, 'theta_0': -0.6329448226063424, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0113648, 'theta_0': -0.5044122140514925, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.016481, 'theta_0': -0.5411477994277141, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0149714, 'theta_0': -0.5348362874267746, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231031, 'theta_0': -0.8831828178277236, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.8386528538114831, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400032, 'theta_0': -0.7278447968489216, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055868, 'theta_0': -0.9629992552488397, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0056893, 'theta_0': -0.9582960990601981, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0057247, 'theta_0': -0.915573702737078, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0055868, 'theta_0': -0.9629992552488397, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0057899, 'theta_0': -0.9555711339523518, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0058132, 'theta_0': -0.9365502041515597, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0057837, 'theta_0': -0.9330993391545165, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0091043, 'theta_0': -1.0191705479225897, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.37810000000000005, 'Reactive_load': 0.10710000000000001, 'Node_id': '6091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300029, 'theta_0': -0.8368594581918889, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3566, 'Reactive_load': 0.10099999999999999, 'Node_id': '6092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.45740000000000003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0010975, 'theta_0': -0.9771556557183506, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4054, 'Reactive_load': 0.1149, 'Node_id': '6093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.008691, 'theta_0': -0.969076194998311, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6798000000000001, 'Reactive_load': 0.19260000000000002, 'Node_id': '6094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0045511, 'theta_0': -1.0026498546351572, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6837000000000001, 'Reactive_load': 0.1937, 'Node_id': '6095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.9295274530271401, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.25780000000000003, 'Reactive_load': 0.073, 'Node_id': '6096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7820999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9988241, 'theta_0': -0.986440423366525, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0536, 'Reactive_load': 0.2985, 'Node_id': '6097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9843412, 'theta_0': -1.015995322774314, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3222, 'Reactive_load': 0.3746, 'Node_id': '6098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9989914, 'theta_0': -0.9894882743859903, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2138, 'Reactive_load': 0.060599999999999994, 'Node_id': '6099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0018955, 'theta_0': -0.9919777945777424, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3771, 'Reactive_load': 0.1069, 'Node_id': '6100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0227101, 'theta_0': -0.5758308601502977, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0211555, 'theta_0': -0.6305742989629912, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.7312677191246402, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3421, 'Reactive_load': 0.0969, 'Node_id': '6103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5934, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300374, 'theta_0': -0.5620435444577059, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0342798, 'theta_0': -0.5537029475749828, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0286262, 'theta_0': -0.5648899321216134, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0309046, 'theta_0': -0.8335425995748139, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0113783, 'theta_0': -0.833914633958169, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0061778, 'theta_0': -0.8837147941837314, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1444, 'Reactive_load': 0.0409, 'Node_id': '6109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400034, 'theta_0': -0.8033480719027541, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400034, 'theta_0': -0.8043748840082874, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0031137, 'theta_0': -0.9591872641762663, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1616, 'Reactive_load': 0.0458, 'Node_id': '6112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9896233, 'theta_0': -1.0050663153444208, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2052, 'Reactive_load': 0.0581, 'Node_id': '6113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907242, 'theta_0': -1.0168498534293828, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7054, 'Reactive_load': 0.1999, 'Node_id': '6114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9982816, 'theta_0': -0.9947387832813499, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3113, 'Reactive_load': 0.3715, 'Node_id': '6115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9977359, 'theta_0': -1.0133986742730745, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5685, 'Reactive_load': 0.1611, 'Node_id': '6116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.02423, 'theta_0': -0.7723591401683565, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.272, 'Reactive_load': 0.0771, 'Node_id': '6117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0267593, 'theta_0': -0.861281361722855, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0096663, 'theta_0': -0.9222093223801128, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984039, 'theta_0': -0.9784350693265251, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.953, 'Reactive_load': 0.5533, 'Node_id': '6120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0091454, 'theta_0': -0.9886629081827221, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.208, 'Reactive_load': 0.3423, 'Node_id': '6121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0107884, 'theta_0': -0.6852440325213731, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2271, 'Reactive_load': 0.0643, 'Node_id': '6122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9869161, 'theta_0': -1.003351756247139, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4511, 'Reactive_load': 0.1278, 'Node_id': '6123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0054779, 'theta_0': -0.9391983573190257, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5742, 'Reactive_load': 0.16269999999999998, 'Node_id': '6124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9996087, 'theta_0': -0.9436454038932296, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0259, 'Reactive_load': 0.0073, 'Node_id': '6125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938512, 'theta_0': -0.9939427560628078, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5038, 'Reactive_load': 0.1427, 'Node_id': '6126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0017935, 'theta_0': -0.9184718045067222, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.25239999999999996, 'Reactive_load': 0.07150000000000001, 'Node_id': '6127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.09380000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.994906, 'theta_0': -1.0170273883208958, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.528, 'Reactive_load': 0.1496, 'Node_id': '6128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0032534, 'theta_0': -0.9980357102383671, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0986, 'Reactive_load': 0.0279, 'Node_id': '6129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9957144, 'theta_0': -0.9884849020523112, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.134, 'Reactive_load': 0.038, 'Node_id': '6130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938605, 'theta_0': -0.9987611563419585, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4385, 'Reactive_load': 0.1243, 'Node_id': '6131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9833626, 'theta_0': -1.020640080246184, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6474, 'Reactive_load': 0.1834, 'Node_id': '6132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0173988, 'theta_0': -0.8679150789570051, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10439999999999999, 'Reactive_load': 0.0296, 'Node_id': '6133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0195404, 'theta_0': -0.9990259402427786, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0762, 'Reactive_load': 0.0216, 'Node_id': '6134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0075467, 'theta_0': -0.6580961515915945, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21710000000000002, 'Reactive_load': 0.061500000000000006, 'Node_id': '6135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0108688, 'theta_0': -1.0010805941981038, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7118000000000001, 'Reactive_load': 0.20170000000000002, 'Node_id': '6136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.984582, 'theta_0': -1.0381030244368907, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4218, 'Reactive_load': 0.1195, 'Node_id': '6137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0118154, 'theta_0': -1.0303877790251572, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2768, 'Reactive_load': 0.0784, 'Node_id': '6138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0122037, 'theta_0': -0.917762014006521, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09609999999999999, 'Reactive_load': 0.027200000000000002, 'Node_id': '6139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137125, 'theta_0': -0.9814660081055385, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11689999999999999, 'Reactive_load': 0.0331, 'Node_id': '6140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0261424, 'theta_0': -0.8052540063525169, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192444, 'theta_0': -0.867013040439697, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0171234, 'theta_0': -0.9280664030037105, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261424, 'theta_0': -0.8052540063525169, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7506736336518371, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7526873247296182, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7652845004052751, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261424, 'theta_0': -0.8052540063525169, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7954442960252202, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0185698, 'theta_0': -1.0047121531326062, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1726999999999999, 'Reactive_load': 0.3323, 'Node_id': '6150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300029, 'theta_0': -0.7514803422854015, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.16140000000000002, 'Reactive_load': 0.045700000000000005, 'Node_id': '6151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5501, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0182455, 'theta_0': -0.9213022747678514, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965087, 'theta_0': -0.9583480575120298, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0796999999999999, 'Reactive_load': 0.3059, 'Node_id': '6153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9974214, 'theta_0': -0.9776134381278563, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.431, 'Reactive_load': 0.12210000000000001, 'Node_id': '6154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9934379, 'theta_0': -0.9748430770061657, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.37020000000000003, 'Reactive_load': 0.10490000000000001, 'Node_id': '6155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0162741, 'theta_0': -0.8983636393016927, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5266, 'Reactive_load': 0.1492, 'Node_id': '6156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056316, 'theta_0': -1.0259177115581195, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.28809999999999997, 'Reactive_load': 0.0816, 'Node_id': '6157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0296053, 'theta_0': -0.752403132767516, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0564, 'Reactive_load': 0.016, 'Node_id': '6158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.669, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0066373, 'theta_0': -0.9778288815707225, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9104000000000001, 'Reactive_load': 0.2579, 'Node_id': '6159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9898856, 'theta_0': -0.9992649630838393, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9828, 'Reactive_load': 0.2785, 'Node_id': '6160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0255902, 'theta_0': -0.7898730305199291, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6161.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0216693, 'theta_0': -0.8520916097991591, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6162.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0154167, 'theta_0': -0.8960660006079048, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6163.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7787562431158138, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6164.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7774984192304866, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6165.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.7846917762892887, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6166.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.030003, 'theta_0': -0.780144879428578, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6167.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984947, 'theta_0': -1.0093178326825238, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3822999999999999, 'Reactive_load': 0.3917, 'Node_id': '6168.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0286209, 'theta_0': -0.7990897303272757, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6169.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0229032, 'theta_0': -0.8723691243015096, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6170.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0095233, 'theta_0': -0.9278611697369685, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1376, 'Reactive_load': 0.3223, 'Node_id': '6171.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0146463, 'theta_0': -0.639456366792853, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6172.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0152227, 'theta_0': -0.6776032254022372, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35479999999999995, 'Reactive_load': 0.1005, 'Node_id': '6173.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0052281, 'theta_0': -0.992303455562872, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9575, 'Reactive_load': 0.2713, 'Node_id': '6174.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9884029, 'theta_0': -1.006633463933079, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.966, 'Reactive_load': 0.2737, 'Node_id': '6175.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021531, 'theta_0': -0.9845375956028308, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4532, 'Reactive_load': 0.1284, 'Node_id': '6176.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.9057345661585927, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6177.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.093999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0155585, 'theta_0': -0.9599620332845191, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3444999999999998, 'Reactive_load': 0.3809, 'Node_id': '6178.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0104727, 'theta_0': -0.9904688178130532, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1185, 'Reactive_load': 0.0336, 'Node_id': '6179.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973509, 'theta_0': -0.9667072596045791, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3414, 'Reactive_load': 0.0967, 'Node_id': '6180.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0102369, 'theta_0': -0.9897850327187068, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4913, 'Reactive_load': 0.1392, 'Node_id': '6181.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9988885, 'theta_0': -1.0154593670676115, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9582999999999999, 'Reactive_load': 0.27149999999999996, 'Node_id': '6182.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0163648, 'theta_0': -0.7565752027180682, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2697, 'Reactive_load': 0.0764, 'Node_id': '6183.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9873133, 'theta_0': -1.0210485571043206, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4839, 'Reactive_load': 0.1371, 'Node_id': '6184.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.989699, 'theta_0': -1.0174517651285182, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2084, 'Reactive_load': 0.059000000000000004, 'Node_id': '6185.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9953878, 'theta_0': -0.9898685118168298, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3937, 'Reactive_load': 0.1115, 'Node_id': '6186.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927893, 'theta_0': -0.9997925412101321, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8044, 'Reactive_load': 0.2279, 'Node_id': '6187.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.018473, 'theta_0': -0.5863186134387017, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6188.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0163245, 'theta_0': -0.6336354493913566, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2737, 'Reactive_load': 0.0775, 'Node_id': '6189.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0195159, 'theta_0': -0.817199406272309, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.18469999999999998, 'Reactive_load': 0.052300000000000006, 'Node_id': '6190.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.00069, 'theta_0': -0.9762819438948024, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1677, 'Reactive_load': 0.0475, 'Node_id': '6191.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0090611, 'theta_0': -0.7517497513087394, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6192.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0044604, 'theta_0': -0.782956639760126, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3569, 'Reactive_load': 0.1011, 'Node_id': '6193.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.993237, 'theta_0': -0.9834937665370905, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.44420000000000004, 'Reactive_load': 0.1259, 'Node_id': '6194.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9951812, 'theta_0': -0.6845190751099721, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.026600000000000002, 'Reactive_load': 0.0075, 'Node_id': '6195.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0063642, 'theta_0': -0.9656021171222163, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21280000000000002, 'Reactive_load': 0.0603, 'Node_id': '6196.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.023295, 'theta_0': -0.844909527567325, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6197.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0150387, 'theta_0': -0.924373635372341, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6198.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.013068, 'theta_0': -0.9725292242036268, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.3974, 'Reactive_load': 0.6793, 'Node_id': '6199.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9926281, 'theta_0': -0.9950322254884877, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4628, 'Reactive_load': 0.1311, 'Node_id': '6200.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0067347, 'theta_0': -1.016940052045126, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3907, 'Reactive_load': 0.1107, 'Node_id': '6201.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0167792, 'theta_0': -0.9133687410265711, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1663, 'Reactive_load': 0.0471, 'Node_id': '6202.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963022, 'theta_0': -1.0059758587775125, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1759, 'Reactive_load': 0.3332, 'Node_id': '6203.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997981, 'theta_0': -0.9927576600474111, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0781, 'Reactive_load': 0.022099999999999998, 'Node_id': '6204.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9896711, 'theta_0': -0.9950687552297319, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2195, 'Reactive_load': 0.3455, 'Node_id': '6205.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078807, 'theta_0': -0.7755416956994906, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1883, 'Reactive_load': 0.053399999999999996, 'Node_id': '6206.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300032, 'theta_0': -1.014836267071357, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2932, 'Reactive_load': 0.08310000000000001, 'Node_id': '6207.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.5515, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9889471, 'theta_0': -1.019548795678082, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4096000000000002, 'Reactive_load': 0.3994, 'Node_id': '6208.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100542, 'theta_0': -0.9734780200915959, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07519999999999999, 'Reactive_load': 0.0213, 'Node_id': '6209.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0348101, 'theta_0': -0.8061890490459803, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6210.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0290615, 'theta_0': -0.8407185731542661, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6211.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0236419, 'theta_0': -0.8872752134979223, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6212.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7321966357124292, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6213.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.6644310335512181, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6214.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7698169633930717, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6215.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400033, 'theta_0': -0.7242875540339693, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6216.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9881199, 'theta_0': -1.0444961305803608, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5296, 'Reactive_load': 0.1501, 'Node_id': '6217.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9921182, 'theta_0': -0.9839127328240317, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5124, 'Reactive_load': 0.1452, 'Node_id': '6218.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.995836, 'theta_0': -1.017886090312877, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6393, 'Reactive_load': 0.18109999999999998, 'Node_id': '6219.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0056706, 'theta_0': -0.9707523219454638, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.965, 'Reactive_load': 0.2734, 'Node_id': '6220.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.866620533344216, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0916, 'Reactive_load': 0.026000000000000002, 'Node_id': '6221.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6201, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028922, 'theta_0': -0.9369481392210144, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6222.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939468, 'theta_0': -0.9813962472953363, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6684, 'Reactive_load': 0.4727, 'Node_id': '6223.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9868659, 'theta_0': -1.018080485084964, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1667, 'Reactive_load': 0.3306, 'Node_id': '6224.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9937059, 'theta_0': -0.9977248147387094, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8525, 'Reactive_load': 0.2415, 'Node_id': '6225.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0183211, 'theta_0': -1.050922799405347, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1802, 'Reactive_load': 0.051100000000000007, 'Node_id': '6226.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9874013, 'theta_0': -1.0050847111147367, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2634999999999998, 'Reactive_load': 0.358, 'Node_id': '6227.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0234798, 'theta_0': -0.8237410224818313, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6228.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -0.8957617897192821, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6229.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.2302999999999997, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0076853, 'theta_0': -0.9637152940277629, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9150999999999998, 'Reactive_load': 0.5426, 'Node_id': '6230.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9900394, 'theta_0': -0.9943970303605169, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7051000000000001, 'Reactive_load': 0.1998, 'Node_id': '6231.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0005415, 'theta_0': -0.9449384136162771, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6663, 'Reactive_load': 0.1888, 'Node_id': '6232.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9846305, 'theta_0': -1.0086503140568062, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6509, 'Reactive_load': 0.1844, 'Node_id': '6233.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0234382, 'theta_0': -0.8574218453347499, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6234.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0214838, 'theta_0': -0.8832978524787225, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6235.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0060055, 'theta_0': -0.9543377097699675, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6953, 'Reactive_load': 0.4803, 'Node_id': '6236.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074305, 'theta_0': -0.9245932326988269, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.018799999999999997, 'Reactive_load': 0.0053, 'Node_id': '6237.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9843392, 'theta_0': -1.0147152808476088, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.348, 'Reactive_load': 0.38189999999999996, 'Node_id': '6238.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0330154, 'theta_0': -0.7672602829317028, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6239.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0164739, 'theta_0': -0.843433223366233, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6240.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9992721, 'theta_0': -0.9312355194996043, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7572999999999999, 'Reactive_load': 0.4979, 'Node_id': '6241.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.740629002007222, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6242.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7344870487431139, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6243.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7220667098272691, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6244.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7290995140481802, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6245.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7476942868055129, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6246.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7526125373711703, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6247.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7375630170168287, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6248.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.040003, 'theta_0': -0.7243008534428694, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6249.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987051, 'theta_0': -0.9994629183276004, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8621, 'Reactive_load': 0.2443, 'Node_id': '6250.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.99651, 'theta_0': -1.0239533609382923, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5125, 'Reactive_load': 0.1452, 'Node_id': '6251.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0095241, 'theta_0': -0.9282421576593864, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1348, 'Reactive_load': 0.0382, 'Node_id': '6252.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0175154, 'theta_0': -0.731116119825812, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.132, 'Reactive_load': 0.0374, 'Node_id': '6253.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0225993, 'theta_0': -0.8500033757090255, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6254.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0149074, 'theta_0': -0.8872404116326374, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6255.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998347, 'theta_0': -0.9685440092027929, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1825, 'Reactive_load': 0.051699999999999996, 'Node_id': '6256.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0202207, 'theta_0': -0.8434982368808698, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6257.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200032, 'theta_0': -0.8275383353353954, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6258.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0225993, 'theta_0': -0.8500033757090255, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6259.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200032, 'theta_0': -0.8421843879265535, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6260.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9955252, 'theta_0': -1.0121846406986796, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3118, 'Reactive_load': 0.0883, 'Node_id': '6261.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9958387, 'theta_0': -0.9834739395967879, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.284, 'Reactive_load': 0.0805, 'Node_id': '6262.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0110056, 'theta_0': -0.7359898644955436, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6263.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0081809, 'theta_0': -0.7632897810564633, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3497, 'Reactive_load': 0.09910000000000001, 'Node_id': '6264.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0178534, 'theta_0': -0.6895293743403798, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6265.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.022154, 'theta_0': -0.6792422117429074, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6266.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.033053, 'theta_0': -0.6522370463860645, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6267.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0211182, 'theta_0': -0.6821286198065631, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6268.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0340477, 'theta_0': -0.6496866665632952, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6269.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178534, 'theta_0': -0.6895293743403798, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6270.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178534, 'theta_0': -0.6895293743403798, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6271.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178534, 'theta_0': -0.6895293743403798, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6272.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178534, 'theta_0': -0.6895293743403798, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6273.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0309912, 'theta_0': -0.6563790966734825, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6274.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9916305, 'theta_0': -0.995581306071165, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1059, 'Reactive_load': 0.3134, 'Node_id': '6275.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0020132, 'theta_0': -1.0185296979278424, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3992, 'Reactive_load': 0.1131, 'Node_id': '6276.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0192968, 'theta_0': -0.9030803392453297, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6277.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040711, 'theta_0': -0.9596051134524863, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3568, 'Reactive_load': 0.38439999999999996, 'Node_id': '6278.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0203919, 'theta_0': -0.8576041798817057, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6279.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.012131, 'theta_0': -0.9011623445704356, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3602, 'Reactive_load': 0.38539999999999996, 'Node_id': '6280.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137453, 'theta_0': -0.9865466441048015, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6045, 'Reactive_load': 0.17129999999999998, 'Node_id': '6281.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0217423, 'theta_0': -0.9705900761381984, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2619, 'Reactive_load': 0.0742, 'Node_id': '6282.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0141945, 'theta_0': -0.9914038605065166, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0688, 'Reactive_load': 0.0195, 'Node_id': '6283.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0011896, 'theta_0': -0.971917259408, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7109000000000001, 'Reactive_load': 0.2014, 'Node_id': '6284.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0210436, 'theta_0': -0.96647828986001, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6285.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300031, 'theta_0': -1.0105082694054315, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8267, 'Reactive_load': 0.23420000000000002, 'Node_id': '6286.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.907, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0122418, 'theta_0': -0.9153891341686796, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6287.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961049, 'theta_0': -0.9851981678181255, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6654, 'Reactive_load': 0.4719, 'Node_id': '6288.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9857928, 'theta_0': -1.0111952659056016, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7384000000000001, 'Reactive_load': 0.20920000000000002, 'Node_id': '6289.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9945737, 'theta_0': -1.0052649338132977, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6761, 'Reactive_load': 0.1916, 'Node_id': '6290.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0119264, 'theta_0': -0.9887665981935831, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2057, 'Reactive_load': 0.0583, 'Node_id': '6291.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0244472, 'theta_0': -0.838265006745105, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6292.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0133925, 'theta_0': -0.8905615986662576, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6293.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0046155, 'theta_0': -0.9322644434535324, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6294.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0225307, 'theta_0': -0.8328665288357614, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6295.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0228017, 'theta_0': -0.8336543704601115, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6296.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0200032, 'theta_0': -0.8256962973895481, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6297.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.023703, 'theta_0': -0.8092082417591278, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6298.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0233727, 'theta_0': -0.847448824549344, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6299.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0152724, 'theta_0': -0.8894165055973166, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6300.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.023703, 'theta_0': -0.8092082417591278, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6301.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0172276, 'theta_0': -0.7579167151410287, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6302.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0218286, 'theta_0': -0.7941971186483301, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6303.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0184923, 'theta_0': -0.7661800288444733, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6304.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0173184, 'theta_0': -0.7572837889410854, 'kV_base': 22.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6305.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9888693, 'theta_0': -0.9989461786959625, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5418999999999999, 'Reactive_load': 0.1535, 'Node_id': '6306.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9925605, 'theta_0': -0.9944962872350778, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3073, 'Reactive_load': 0.3704, 'Node_id': '6307.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0239619, 'theta_0': -0.8951375378057213, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6308.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137735, 'theta_0': -0.9568809337428035, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.41100000000000003, 'Reactive_load': 0.1165, 'Node_id': '6309.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103858, 'theta_0': -0.965656152515858, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9227, 'Reactive_load': 0.2614, 'Node_id': '6310.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0210712, 'theta_0': -0.6752876621770312, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0858, 'Reactive_load': 0.024300000000000002, 'Node_id': '6311.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0160345, 'theta_0': -0.9522712922954837, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4568, 'Reactive_load': 0.1294, 'Node_id': '6312.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9973457, 'theta_0': -1.0069271679396046, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.26289999999999997, 'Reactive_load': 0.0745, 'Node_id': '6313.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.004168, 'theta_0': -0.9971041756567002, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.29510000000000003, 'Reactive_load': 0.0836, 'Node_id': '6314.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0135155, 'theta_0': -0.9159791601756089, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6315.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0160058, 'theta_0': -0.9287773977810955, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6316.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055183, 'theta_0': -0.9538333270694336, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6317.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.994043, 'theta_0': -0.9841227308396318, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6318.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9859871, 'theta_0': -1.0064167638531514, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7021999999999999, 'Reactive_load': 0.19899999999999998, 'Node_id': '6319.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0173552, 'theta_0': -0.9686495667159536, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4789, 'Reactive_load': 0.13570000000000002, 'Node_id': '6320.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.2136, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0109006, 'theta_0': -0.8208753838362743, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6321.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0102176, 'theta_0': -0.8675885104006644, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6607, 'Reactive_load': 0.18719999999999998, 'Node_id': '6322.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.996336, 'theta_0': -0.987922836219999, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33380000000000004, 'Reactive_load': 0.0946, 'Node_id': '6323.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9979793, 'theta_0': -0.980288905698116, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5821999999999999, 'Reactive_load': 0.165, 'Node_id': '6324.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.008186, 'theta_0': -0.9415505026453533, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.047400000000000005, 'Reactive_load': 0.0134, 'Node_id': '6325.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978726, 'theta_0': -0.8021414559776803, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1543, 'Reactive_load': 0.0437, 'Node_id': '6326.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0174467, 'theta_0': -0.6921241029794422, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1761, 'Reactive_load': 0.0499, 'Node_id': '6327.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0099616, 'theta_0': -1.0252454630901289, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.27949999999999997, 'Reactive_load': 0.07919999999999999, 'Node_id': '6328.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.018426, 'theta_0': -0.7687502705141304, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0179, 'Reactive_load': 0.0051, 'Node_id': '6329.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9882031, 'theta_0': -1.0260040529962156, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3375, 'Reactive_load': 0.379, 'Node_id': '6330.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9945548, 'theta_0': -1.015546022664973, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8131999999999999, 'Reactive_load': 0.2304, 'Node_id': '6331.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.4719, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.009184, 'theta_0': -0.9822905540040582, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.73, 'Reactive_load': 0.20679999999999998, 'Node_id': '6332.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0229701, 'theta_0': -0.8489204687213332, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6333.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0226098, 'theta_0': -0.8953106744675191, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6334.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0003399, 'theta_0': -0.9645924092433527, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6471, 'Reactive_load': 0.4667, 'Node_id': '6335.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008638, 'theta_0': -1.0200462494214853, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2607, 'Reactive_load': 0.0739, 'Node_id': '6336.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0146035, 'theta_0': -0.987732874584212, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2119, 'Reactive_load': 0.34340000000000004, 'Node_id': '6337.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9941681, 'theta_0': -0.916810198698946, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3361, 'Reactive_load': 0.09519999999999999, 'Node_id': '6338.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0022333, 'theta_0': -0.9360361523269698, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.34369999999999995, 'Reactive_load': 0.0974, 'Node_id': '6339.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0190508, 'theta_0': -0.8987263885334271, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2918, 'Reactive_load': 0.0827, 'Node_id': '6340.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0097343, 'theta_0': -0.7292598225399759, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.063, 'Reactive_load': 0.0179, 'Node_id': '6341.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0159697, 'theta_0': -0.8871583113446238, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6342.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0074867, 'theta_0': -0.9284098314406254, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6343.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0027467, 'theta_0': -0.9642328888707342, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.588, 'Reactive_load': 0.4499, 'Node_id': '6344.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9891926, 'theta_0': -1.0116190143946933, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.177, 'Reactive_load': 0.050199999999999995, 'Node_id': '6345.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0022903, 'theta_0': -0.9193973002491772, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.023, 'Reactive_load': 0.006500000000000001, 'Node_id': '6346.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0293489, 'theta_0': -0.8610849423688356, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6349.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0174336, 'theta_0': -0.8992248196612118, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6350.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990794, 'theta_0': -0.9773221775822836, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2386, 'Reactive_load': 0.0676, 'Node_id': '6351.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0131142, 'theta_0': -0.9854390930680709, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2176, 'Reactive_load': 0.345, 'Node_id': '6352.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0000177, 'theta_0': -0.9851561577430301, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0796, 'Reactive_load': 0.3059, 'Node_id': '6353.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0064169, 'theta_0': -0.9335115859238377, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6354.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0030083, 'theta_0': -0.9581615690814543, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5182, 'Reactive_load': 0.4302, 'Node_id': '6355.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -1.0126354243378848, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3592, 'Reactive_load': 0.3851, 'Node_id': '6356.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.6310000000000002, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0211213, 'theta_0': -0.9430461625478499, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.30829999999999996, 'Reactive_load': 0.0874, 'Node_id': '6357.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300033, 'theta_0': -1.0183431222308044, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7798999999999999, 'Reactive_load': 0.221, 'Node_id': '6358.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.5658, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0228871, 'theta_0': -0.8594447342976889, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6359.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0018715, 'theta_0': -0.8649413346175796, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3893, 'Reactive_load': 0.1103, 'Node_id': '6360.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.002794, 'theta_0': -0.6230340053554776, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09300000000000001, 'Reactive_load': 0.0264, 'Node_id': '7001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.031626, 'theta_0': -0.437085928716301, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0173944, 'theta_0': -0.5118076976909681, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.011398, 'theta_0': -0.5931671981570649, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.2192, 'Reactive_load': 0.6288, 'Node_id': '7004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.39944085226686527, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.41568464906846897, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40486257796184544, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40322629688151573, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.3993735174643233, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907729, 'theta_0': -0.7447241727509315, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1565, 'Reactive_load': 0.0443, 'Node_id': '7010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0231097, 'theta_0': -0.7353570604686479, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3579, 'Reactive_load': 0.1014, 'Node_id': '7011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9822041, 'theta_0': -0.6760573872837458, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.009399999999999999, 'Reactive_load': 0.0027, 'Node_id': '7012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9975297, 'theta_0': -0.7057941499995924, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3805, 'Reactive_load': 0.3911, 'Node_id': '7013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9913004, 'theta_0': -0.6663548401725214, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5281, 'Reactive_load': 0.1496, 'Node_id': '7014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9913004, 'theta_0': -0.6663548401725214, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0025695, 'theta_0': -0.6162255981164954, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8556, 'Reactive_load': 0.24239999999999998, 'Node_id': '7016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987305, 'theta_0': -0.6869751547932561, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3583, 'Reactive_load': 0.3848, 'Node_id': '7017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0246775, 'theta_0': -0.5695975912597251, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0180906, 'theta_0': -0.5877314051083136, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0141452, 'theta_0': -0.6356339910112303, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7537999999999999, 'Reactive_load': 0.21359999999999998, 'Node_id': '7020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0306486, 'theta_0': -0.5584167153654767, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0378511, 'theta_0': -0.5445644908780032, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.032955, 'theta_0': -0.5541562444883108, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0302576, 'theta_0': -0.5592423084615475, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0246775, 'theta_0': -0.5695975912597251, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137233, 'theta_0': -0.6167608382382045, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4339, 'Reactive_load': 0.1229, 'Node_id': '7026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9970786, 'theta_0': -0.6932916933424412, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0922, 'Reactive_load': 0.3095, 'Node_id': '7027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.013589, 'theta_0': -0.6750460911552627, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7701, 'Reactive_load': 0.2182, 'Node_id': '7028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.8169, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0175822, 'theta_0': -0.5613888540019902, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0154034, 'theta_0': -0.6255799694952393, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0173863, 'theta_0': -0.5091955332126783, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0179529, 'theta_0': -0.5445930968244433, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178766, 'theta_0': -0.5458642375719633, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0175822, 'theta_0': -0.5613888540019902, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.014441, 'theta_0': -0.643354716756815, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0029473, 'theta_0': -0.6867468831803878, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9231, 'Reactive_load': 0.2615, 'Node_id': '7036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0228053, 'theta_0': -0.5432718127675136, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0103709, 'theta_0': -0.5697573412461603, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0032153, 'theta_0': -0.6472803811235707, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.355, 'Reactive_load': 0.6673, 'Node_id': '7039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0159299, 'theta_0': -0.5398575473248847, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0062694, 'theta_0': -0.6036487031667468, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9388, 'Reactive_load': 0.266, 'Node_id': '7041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0244234, 'theta_0': -0.5481753502075767, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0202015, 'theta_0': -0.5721349858328596, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.002184, 'theta_0': -0.6208770401994005, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2464, 'Reactive_load': 0.3532, 'Node_id': '7044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300026, 'theta_0': -0.5135014699168585, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300026, 'theta_0': -0.5322219412869198, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0214289, 'theta_0': -0.5509759055253268, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0121491, 'theta_0': -0.5885078497326484, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0080421, 'theta_0': -0.6703959580692966, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.8314999999999997, 'Reactive_load': 0.8022, 'Node_id': '7049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9902975, 'theta_0': -0.7389898236803341, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7203, 'Reactive_load': 0.2041, 'Node_id': '7050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9957598, 'theta_0': -0.6830822328032679, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4079, 'Reactive_load': 0.39890000000000003, 'Node_id': '7051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9984259, 'theta_0': -0.6532394413353625, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0666, 'Reactive_load': 0.30219999999999997, 'Node_id': '7052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0071842, 'theta_0': -0.5719499111189781, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0595999999999999, 'Reactive_load': 0.3002, 'Node_id': '7053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9855692, 'theta_0': -0.6821575050056837, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9285, 'Reactive_load': 0.2631, 'Node_id': '7054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990693, 'theta_0': -0.6467121193724138, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9808, 'Reactive_load': 0.2779, 'Node_id': '7055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0090563, 'theta_0': -0.6313918809977955, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9912754, 'theta_0': -0.7112093830697552, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0611, 'Reactive_load': 0.3006, 'Node_id': '7057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0202463, 'theta_0': -0.4953776044450065, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0204897, 'theta_0': -0.5186519016329088, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0242836, 'theta_0': -0.5568629883587664, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0172462, 'theta_0': -0.4795080411416154, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0175312, 'theta_0': -0.48086985174377644, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0173491, 'theta_0': -0.479705472786601, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0179544, 'theta_0': -0.4805313974952297, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0178142, 'theta_0': -0.4790381112405159, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0176, 'theta_0': -0.4809861953917144, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9800165, 'theta_0': -0.7179126691278899, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8269, 'Reactive_load': 0.2343, 'Node_id': '7067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.014778, 'theta_0': -0.521349273085281, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040275, 'theta_0': -0.5890467550457866, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5115999999999999, 'Reactive_load': 0.145, 'Node_id': '7069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9932183, 'theta_0': -0.6352915748652815, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0103, 'Reactive_load': 0.2863, 'Node_id': '7070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9964456, 'theta_0': -0.5547547178888197, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.037599999999999995, 'Reactive_load': 0.010700000000000001, 'Node_id': '7071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0083745, 'theta_0': -0.6085319773341943, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1642000000000001, 'Reactive_load': 0.3299, 'Node_id': '7072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0172224, 'theta_0': -0.5427661210700407, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0143681, 'theta_0': -0.5873222126651835, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6912999999999999, 'Reactive_load': 0.1959, 'Node_id': '7074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161298, 'theta_0': -0.6137227261559655, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6086, 'Reactive_load': 0.4558, 'Node_id': '7075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.425, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0344048, 'theta_0': -0.4393326561558083, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273272, 'theta_0': -0.479711651252153, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0169178, 'theta_0': -0.5584505223930878, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4145110373195504, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4063849064953125, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40607580868478427, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4129671888764063, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0160802, 'theta_0': -0.5267806155444397, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0081888, 'theta_0': -0.5858575498102024, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1474, 'Reactive_load': 0.6084, 'Node_id': '7084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9983918, 'theta_0': -0.607953365780573, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.040999999999999995, 'Reactive_load': 0.0116, 'Node_id': '7085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9916556, 'theta_0': -0.6497218174944305, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5344, 'Reactive_load': 0.1514, 'Node_id': '7086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0031112, 'theta_0': -0.6294708669032953, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7422, 'Reactive_load': 0.21030000000000001, 'Node_id': '7087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9789157, 'theta_0': -0.6978542632598348, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0105, 'Reactive_load': 0.2863, 'Node_id': '7088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.015342, 'theta_0': -0.7008958136407003, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7006999999999999, 'Reactive_load': 0.1985, 'Node_id': '7089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.1653, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9921421, 'theta_0': -0.7304372740132489, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.138, 'Reactive_load': 0.0391, 'Node_id': '7090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.026318, 'theta_0': -0.6906789354522058, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1068000000000002, 'Reactive_load': 0.5969, 'Node_id': '7091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.0742000000000003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927246, 'theta_0': -0.6720654527652918, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3923, 'Reactive_load': 0.11109999999999999, 'Node_id': '7092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9888617, 'theta_0': -0.6856224548097904, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0915000000000001, 'Reactive_load': 0.3093, 'Node_id': '7093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039242, 'theta_0': -0.6978691334650617, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7669, 'Reactive_load': 0.2173, 'Node_id': '7094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089056, 'theta_0': -0.37396497930720485, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151521, 'theta_0': -0.4128506706955432, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0152672, 'theta_0': -0.44239192162858154, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'Slack', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000018, 'theta_0': -0.22575252679905738, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9933479, 'theta_0': -0.6467300962637094, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6215999999999999, 'Reactive_load': 0.1761, 'Node_id': '7100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0014243, 'theta_0': -0.6679761812344541, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1250999999999998, 'Reactive_load': 0.6021, 'Node_id': '7101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9853947, 'theta_0': -0.7089457957496738, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1804999999999999, 'Reactive_load': 0.3345, 'Node_id': '7102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.994076, 'theta_0': -0.6527519185154028, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5864, 'Reactive_load': 0.1661, 'Node_id': '7103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0318283, 'theta_0': -0.42903908819998116, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.018642, 'theta_0': -0.5074285444246667, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0127503, 'theta_0': -0.5811325770059108, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9207, 'Reactive_load': 0.2609, 'Node_id': '7106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0393913, 'theta_0': -0.4150371493692716, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40763393392120967, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4112640267791402, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.394593235176036, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.39374310275068214, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40774223160129597, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.38779217577320474, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4091657919523926, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.409152143477642, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.982072, 'theta_0': -0.6888219400346688, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1216, 'Reactive_load': 0.0345, 'Node_id': '7116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9949012, 'theta_0': -0.6829646325182684, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8403, 'Reactive_load': 0.23809999999999998, 'Node_id': '7117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9855934, 'theta_0': -0.701755772269743, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2115, 'Reactive_load': 0.3433, 'Node_id': '7118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9938716, 'theta_0': -0.6448443029135146, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.49700000000000005, 'Reactive_load': 0.1408, 'Node_id': '7119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9850769, 'theta_0': -0.7025796200365618, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4969999999999999, 'Reactive_load': 0.4241, 'Node_id': '7120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9850769, 'theta_0': -0.7025796200365618, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 0.9850769, 'theta_0': -0.7025796200365618, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9956521, 'theta_0': -0.5462776711651758, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0663, 'Reactive_load': 0.018799999999999997, 'Node_id': '7123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9916059, 'theta_0': -0.6703385890967836, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6466, 'Reactive_load': 0.46649999999999997, 'Node_id': '7124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0278363, 'theta_0': -0.44374802500408855, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0118672, 'theta_0': -0.553339412945085, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0076727, 'theta_0': -0.5998575862320272, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.2354, 'Reactive_load': 0.6334000000000001, 'Node_id': '7127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021478, 'theta_0': -0.6932837520943448, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6394, 'Reactive_load': 0.1812, 'Node_id': '7128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0219495, 'theta_0': -0.5918187044170966, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.051500000000000004, 'Reactive_load': 0.0146, 'Node_id': '7129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0289051, 'theta_0': -0.4241799693893813, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0162005, 'theta_0': -0.5085930455548898, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0028842, 'theta_0': -0.5720580517194316, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7275, 'Reactive_load': 0.2061, 'Node_id': '7132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0279995, 'theta_0': -0.4159376694501306, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0259394, 'theta_0': -0.3978487803764885, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0250387, 'theta_0': -0.3910758731344919, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0245632, 'theta_0': -0.38777367528313356, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0247215, 'theta_0': -0.3864107127636662, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0260802, 'theta_0': -0.3990723958084767, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0261554, 'theta_0': -0.3981459052283481, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.026391, 'theta_0': -0.40221660645594454, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0273628, 'theta_0': -0.40977344578476693, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0267994, 'theta_0': -0.4051430000127634, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9994946, 'theta_0': -0.7142245615256231, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5996, 'Reactive_load': 0.1699, 'Node_id': '7143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9953186, 'theta_0': -0.6352823246202459, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9768000000000001, 'Reactive_load': 0.2768, 'Node_id': '7144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300027, 'theta_0': -0.7151059527978801, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4378, 'Reactive_load': 0.40740000000000004, 'Node_id': '7145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 3.1422000000000003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0278976, 'theta_0': -0.6946058739093154, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.004257, 'theta_0': -0.6653087421787536, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4729, 'Reactive_load': 0.134, 'Node_id': '7147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.989317, 'theta_0': -0.6982668416417137, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5946, 'Reactive_load': 0.1685, 'Node_id': '7148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0019453, 'theta_0': -0.605097326445902, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1225, 'Reactive_load': 0.0347, 'Node_id': '7149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300028, 'theta_0': -0.6688505911897032, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.6554, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0221745, 'theta_0': -0.711714062476262, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.44049999999999995, 'Reactive_load': 0.12480000000000001, 'Node_id': '7151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9950581, 'theta_0': -0.6647266574199211, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0359, 'Reactive_load': 0.0102, 'Node_id': '7152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.998063, 'theta_0': -0.49131067312859433, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2621, 'Reactive_load': 0.0742, 'Node_id': '7153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0050109, 'theta_0': -0.7106995025820776, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.586, 'Reactive_load': 0.166, 'Node_id': '7154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9870723, 'theta_0': -0.6694249092333645, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7841, 'Reactive_load': 0.22219999999999998, 'Node_id': '7155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0047581, 'theta_0': -0.6155319344585828, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.99886, 'theta_0': -0.673296991992084, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6383, 'Reactive_load': 0.4642, 'Node_id': '7157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9832623, 'theta_0': -0.6875686714586893, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7837000000000001, 'Reactive_load': 0.222, 'Node_id': '7158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273608, 'theta_0': -0.5442942790032095, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0152071, 'theta_0': -0.5841159730160999, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0060052, 'theta_0': -0.6427752325385678, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3754, 'Reactive_load': 0.3897, 'Node_id': '7161.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0361652, 'theta_0': -0.5278008652119854, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7162.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.5187653480342883, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7163.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0371569, 'theta_0': -0.525709280089688, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7164.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0362215, 'theta_0': -0.5275577059405976, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7165.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.5144318351279266, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7166.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.5079706436903361, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7167.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0374839, 'theta_0': -0.525135241298707, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7168.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0356302, 'theta_0': -0.5286073993126246, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7169.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.5056477326289793, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7170.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0167416, 'theta_0': -0.5495804100686023, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7171.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0196878, 'theta_0': -0.5870654398256301, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9112, 'Reactive_load': 0.2582, 'Node_id': '7172.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9818274, 'theta_0': -0.7028854715346813, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1254, 'Reactive_load': 0.3189, 'Node_id': '7173.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.99135, 'theta_0': -0.6234770920926814, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2467, 'Reactive_load': 0.3532, 'Node_id': '7174.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0259895, 'theta_0': -0.5642212092187118, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7175.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0117798, 'theta_0': -0.6054186415611942, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7176.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9901848, 'theta_0': -0.6887677650146871, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1163, 'Reactive_load': 0.5996, 'Node_id': '7177.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9864173, 'theta_0': -0.7056920831449358, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2248999999999999, 'Reactive_load': 0.3471, 'Node_id': '7178.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9911449, 'theta_0': -0.6520274323429001, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5829, 'Reactive_load': 0.16519999999999999, 'Node_id': '7179.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923428, 'theta_0': -0.678626564302559, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9145, 'Reactive_load': 0.2591, 'Node_id': '7180.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9863418, 'theta_0': -0.7042530242700815, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4897999999999998, 'Reactive_load': 0.42210000000000003, 'Node_id': '7181.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9881471, 'theta_0': -0.7035938832247732, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0207, 'Reactive_load': 0.0059, 'Node_id': '7182.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300027, 'theta_0': -0.6975109220893825, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1591, 'Reactive_load': 0.6117, 'Node_id': '7183.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.5229, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0184389, 'theta_0': -0.4967637274836479, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7184.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0080531, 'theta_0': -0.5400844924875216, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6893, 'Reactive_load': 0.47859999999999997, 'Node_id': '7185.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0237994, 'theta_0': -0.46605054031777293, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7186.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0105083, 'theta_0': -0.5431975839144263, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7187.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008612, 'theta_0': -0.6320028684090412, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7658, 'Reactive_load': 0.217, 'Node_id': '7188.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0252826, 'theta_0': -0.4633036015146441, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7189.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.4418472218223266, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7190.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0237994, 'theta_0': -0.46605054031777293, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7191.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.4540206490088917, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7192.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0248424, 'theta_0': -0.46413333104104215, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7193.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.43319016183963194, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7194.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.42796534419086174, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7195.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.4456013552301963, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7196.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300027, 'theta_0': -0.45293728823559387, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7197.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300027, 'theta_0': -0.6961355502789334, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8612000000000001, 'Reactive_load': 0.244, 'Node_id': '7198.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.3061000000000003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0240298, 'theta_0': -0.5455635871550149, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7199.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0138467, 'theta_0': -0.6173543025437602, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7200.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0134768, 'theta_0': -0.6810213654758054, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1679, 'Reactive_load': 0.6142, 'Node_id': '7201.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9928285, 'theta_0': -0.7462867136703645, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3593, 'Reactive_load': 0.1018, 'Node_id': '7202.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9969892, 'theta_0': -0.5924920000826384, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.20550000000000002, 'Reactive_load': 0.0582, 'Node_id': '7203.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0342474, 'theta_0': -0.4257120717666595, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7204.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0258271, 'theta_0': -0.4801536384319281, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7205.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0242257, 'theta_0': -0.5515912911662875, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7206.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0342474, 'theta_0': -0.4257120717666595, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7207.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.3602955955121703, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7208.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.380441198029655, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7209.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.38961061686756504, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7210.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.3997639127114094, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7211.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.3834709150781919, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7212.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0069205, 'theta_0': -0.621478690099148, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.685, 'Reactive_load': 0.1941, 'Node_id': '7213.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9892128, 'theta_0': -0.6558350426390508, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8389, 'Reactive_load': 0.2377, 'Node_id': '7214.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9911386, 'theta_0': -0.7156306511309073, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0488, 'Reactive_load': 0.29719999999999996, 'Node_id': '7215.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9893478, 'theta_0': -0.6790220035511834, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9256, 'Reactive_load': 0.2623, 'Node_id': '7216.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927775, 'theta_0': -0.6807228618138368, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7639, 'Reactive_load': 0.2164, 'Node_id': '7217.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9910241, 'theta_0': -0.7418866862662092, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.48109999999999997, 'Reactive_load': 0.1363, 'Node_id': '7218.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0018891, 'theta_0': -0.6387764912360311, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0125, 'Reactive_load': 0.0036, 'Node_id': '7219.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9930716, 'theta_0': -0.6801093611284683, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4127, 'Reactive_load': 0.4003, 'Node_id': '7220.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9921054, 'theta_0': -0.6745405914440075, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6958000000000002, 'Reactive_load': 0.4805, 'Node_id': '7221.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9910781, 'theta_0': -0.6558460382133384, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8698999999999999, 'Reactive_load': 0.2465, 'Node_id': '7222.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9968796, 'theta_0': -0.679075131373614, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4962, 'Reactive_load': 0.1406, 'Node_id': '7223.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9908009, 'theta_0': -0.6838025999987359, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3077, 'Reactive_load': 0.0872, 'Node_id': '7224.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9880072, 'theta_0': -0.7133240065381791, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1144, 'Reactive_load': 0.3157, 'Node_id': '7225.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9985224, 'theta_0': -0.670458074337375, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4525, 'Reactive_load': 0.1282, 'Node_id': '7226.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273034, 'theta_0': -0.5190387364083208, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7227.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0109201, 'theta_0': -0.5877990540701209, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7228.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.004637, 'theta_0': -0.6293455697162946, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.8541000000000003, 'Reactive_load': 0.8087000000000001, 'Node_id': '7229.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0075927, 'theta_0': -0.6670820141520724, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0737, 'Reactive_load': 0.0209, 'Node_id': '7230.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9978768, 'theta_0': -0.6045430971419312, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5085000000000001, 'Reactive_load': 0.1441, 'Node_id': '7231.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0230171, 'theta_0': -0.6010002707466005, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4292, 'Reactive_load': 0.1216, 'Node_id': '7232.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.990549, 'theta_0': -0.7473214321174093, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.332, 'Reactive_load': 0.0941, 'Node_id': '7233.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990666, 'theta_0': -0.7106444025375922, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7731, 'Reactive_load': 0.21899999999999997, 'Node_id': '7234.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300027, 'theta_0': -0.735810025769418, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5162, 'Reactive_load': 0.4296, 'Node_id': '7235.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.2103, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300028, 'theta_0': -0.7134675075092781, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38270000000000004, 'Reactive_load': 0.1084, 'Node_id': '7236.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.4716, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9873254, 'theta_0': -0.6743667741038014, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1509, 'Reactive_load': 0.042800000000000005, 'Node_id': '7237.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0209741, 'theta_0': -0.6336779132520577, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47850000000000004, 'Reactive_load': 0.1356, 'Node_id': '7238.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.888, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0005543, 'theta_0': -0.6154899767433648, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.424, 'Reactive_load': 0.1201, 'Node_id': '7239.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0014366, 'theta_0': -0.6829663778475203, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0769, 'Reactive_load': 0.30510000000000004, 'Node_id': '7240.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9885259, 'theta_0': -0.6930277297463696, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0432, 'Reactive_load': 0.2956, 'Node_id': '7241.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9899878, 'theta_0': -0.6939220015485066, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8014, 'Reactive_load': 0.2271, 'Node_id': '7242.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9987961, 'theta_0': -0.6605451103315303, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7870999999999999, 'Reactive_load': 0.223, 'Node_id': '7243.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927243, 'theta_0': -0.6404366658738206, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38020000000000004, 'Reactive_load': 0.10769999999999999, 'Node_id': '7244.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9991022, 'theta_0': -0.7150975054043005, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9879000000000001, 'Reactive_load': 0.2799, 'Node_id': '7245.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9825628, 'theta_0': -0.7183125764193993, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9950999999999999, 'Reactive_load': 0.5653, 'Node_id': '7246.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9965269, 'theta_0': -0.5998739923269959, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.092, 'Reactive_load': 0.026099999999999998, 'Node_id': '7247.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9867458, 'theta_0': -0.695745748443793, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.2052, 'Reactive_load': 0.6248, 'Node_id': '7248.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.989979, 'theta_0': -0.6778655134822268, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7182, 'Reactive_load': 0.20350000000000001, 'Node_id': '7249.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0009014, 'theta_0': -0.6625039282576286, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3262, 'Reactive_load': 0.0924, 'Node_id': '7250.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9974407, 'theta_0': -0.7419962231300644, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35619999999999996, 'Reactive_load': 0.1009, 'Node_id': '7251.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0191213, 'theta_0': -0.5205874368667854, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7252.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0051041, 'theta_0': -0.5768749510154259, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9093000000000001, 'Reactive_load': 0.2576, 'Node_id': '7253.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0189635, 'theta_0': -0.7298627988899549, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5013000000000001, 'Reactive_load': 0.142, 'Node_id': '7254.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9985601, 'theta_0': -0.64328687580879, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5899, 'Reactive_load': 0.1671, 'Node_id': '7255.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9935603, 'theta_0': -0.6628382111692631, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8686, 'Reactive_load': 0.24609999999999999, 'Node_id': '7256.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9897062, 'theta_0': -0.6403083318139215, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2715, 'Reactive_load': 0.3603, 'Node_id': '7257.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939123, 'theta_0': -0.6624199255607301, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4299, 'Reactive_load': 0.12179999999999999, 'Node_id': '7258.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0025731, 'theta_0': -0.6562467309030112, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4605000000000001, 'Reactive_load': 0.4138, 'Node_id': '7259.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0011852, 'theta_0': -0.6229750132267602, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2464, 'Reactive_load': 0.3531, 'Node_id': '7260.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0205503, 'theta_0': -0.45908531524549895, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7261.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0288694, 'theta_0': -0.49557733992460473, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7203, 'Reactive_load': 0.2041, 'Node_id': '7262.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7093, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0280798, 'theta_0': -0.4301304600345457, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7263.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0206883, 'theta_0': -0.4823446200551266, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7264.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0043784, 'theta_0': -0.5754851802386478, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6291, 'Reactive_load': 0.1782, 'Node_id': '7265.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0256687, 'theta_0': -0.40787648232735935, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7266.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0235243, 'theta_0': -0.39158812726995224, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7267.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0268112, 'theta_0': -0.4181319148390304, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7268.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0238371, 'theta_0': -0.3954707692706462, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7269.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0258358, 'theta_0': -0.4094713467445392, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7270.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9947876, 'theta_0': -0.7102339883639857, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2845, 'Reactive_load': 0.0806, 'Node_id': '7271.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9930027, 'theta_0': -0.6533107031287214, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.45880000000000004, 'Reactive_load': 0.13, 'Node_id': '7272.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300027, 'theta_0': -0.6137373345618047, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39390000000000003, 'Reactive_load': 0.1116, 'Node_id': '7273.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.1581, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156687, 'theta_0': -0.5943591184098368, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7274.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0135852, 'theta_0': -0.6250404533168629, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.42310000000000003, 'Reactive_load': 0.1199, 'Node_id': '7275.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0137846, 'theta_0': -0.6892416044533108, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8867, 'Reactive_load': 0.25120000000000003, 'Node_id': '7276.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 4.5762, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300026, 'theta_0': -0.6293868816596894, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.15990000000000001, 'Reactive_load': 0.0453, 'Node_id': '7277.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.37770000000000004, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0153657, 'theta_0': -0.5201239123240408, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7278.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0118517, 'theta_0': -0.5764766843334131, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4852, 'Reactive_load': 0.4208, 'Node_id': '7279.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0153657, 'theta_0': -0.5201239123240408, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7280.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0153657, 'theta_0': -0.5201239123240408, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7281.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0220341, 'theta_0': -0.4935840691990721, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7282.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0010982, 'theta_0': -0.5710853971805877, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0947, 'Reactive_load': 0.3102, 'Node_id': '7283.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0220341, 'theta_0': -0.4935840691990721, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7284.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0221494, 'theta_0': -0.4753889593871536, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7285.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0222695, 'theta_0': -0.4517102170516867, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7286.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0220436, 'theta_0': -0.4573324636244286, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7287.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0222422, 'theta_0': -0.45498180182796505, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7288.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0222545, 'theta_0': -0.46480128599907294, 'kV_base': 13.2, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7289.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9875749, 'theta_0': -0.7215230572185654, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6377, 'Reactive_load': 0.1807, 'Node_id': '7290.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.977313, 'theta_0': -0.7129776110415358, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0191, 'Reactive_load': 0.2888, 'Node_id': '7291.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0020552, 'theta_0': -0.6614385792822113, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.941, 'Reactive_load': 0.2666, 'Node_id': '7292.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0037643, 'theta_0': -0.61014509789535, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1594, 'Reactive_load': 0.0452, 'Node_id': '7293.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273797, 'theta_0': -0.5744784917793899, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0725, 'Reactive_load': 0.020499999999999997, 'Node_id': '7294.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9893437, 'theta_0': -0.6952163551750781, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0437, 'Reactive_load': 0.2957, 'Node_id': '7295.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.984002, 'theta_0': -0.7052950381933997, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9384, 'Reactive_load': 0.2659, 'Node_id': '7296.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9869762, 'theta_0': -0.6693294222699879, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9377, 'Reactive_load': 0.2657, 'Node_id': '7297.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9904072, 'theta_0': -0.6914802510183814, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0123, 'Reactive_load': 0.2868, 'Node_id': '7298.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9899927, 'theta_0': -0.6496914313121531, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.184, 'Reactive_load': 0.0521, 'Node_id': '7299.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300028, 'theta_0': -0.7308286291914236, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5435, 'Reactive_load': 0.154, 'Node_id': '7300.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.9422, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9875543, 'theta_0': -0.6364415024962505, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6918000000000001, 'Reactive_load': 0.196, 'Node_id': '7301.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0099673, 'theta_0': -0.5436061131324406, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2443, 'Reactive_load': 0.0692, 'Node_id': '7302.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156153, 'theta_0': -0.5959525342037375, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0649, 'Reactive_load': 0.0184, 'Node_id': '7303.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0226979, 'theta_0': -0.43146851670558717, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7304.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0196647, 'theta_0': -0.5024345116628877, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7305.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0086997, 'theta_0': -0.5812411364853849, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07690000000000001, 'Reactive_load': 0.0218, 'Node_id': '7306.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0331586, 'theta_0': -0.4118383099162164, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7307.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0325419, 'theta_0': -0.41273844602464, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7308.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0226979, 'theta_0': -0.43146851670558717, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7309.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400022, 'theta_0': -0.3973233315518831, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7310.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0339783, 'theta_0': -0.4102707948084153, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7311.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0244197, 'theta_0': -0.5075981555213754, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7312.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0061405, 'theta_0': -0.570637982026839, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8622, 'Reactive_load': 0.2443, 'Node_id': '7313.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9857607, 'theta_0': -0.7047985792876699, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2876, 'Reactive_load': 0.0815, 'Node_id': '7314.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0110248, 'theta_0': -0.6994104860873755, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6931, 'Reactive_load': 0.19640000000000002, 'Node_id': '7315.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9885943, 'theta_0': -0.6560129265964142, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6323, 'Reactive_load': 0.17920000000000003, 'Node_id': '7316.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9954782, 'theta_0': -0.6200253369777199, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4291, 'Reactive_load': 0.40490000000000004, 'Node_id': '7317.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9850881, 'theta_0': -0.6599740211469853, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1216, 'Reactive_load': 0.3178, 'Node_id': '7318.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9894426, 'theta_0': -0.6921844215583911, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3019, 'Reactive_load': 0.0855, 'Node_id': '7319.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0347238, 'theta_0': -0.47747745527667507, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7320.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0280999, 'theta_0': -0.500006304887758, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7321.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0161251, 'theta_0': -0.5575134376643995, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7322.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0319679, 'theta_0': -0.4698707344378306, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7323.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0309205, 'theta_0': -0.46720837429025347, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7324.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0308924, 'theta_0': -0.4671574804892653, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7325.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0328824, 'theta_0': -0.47245408353017, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7326.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0221369, 'theta_0': -0.5121888252397261, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7327.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0159226, 'theta_0': -0.541189320810619, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7328.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400025, 'theta_0': -0.4985572627295823, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7329.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9927563, 'theta_0': -0.6773212674615775, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.9605000000000001, 'Reactive_load': 0.5555, 'Node_id': '7330.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0273033, 'theta_0': -0.4667041661226448, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7331.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151778, 'theta_0': -0.5166937643852187, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7332.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997779, 'theta_0': -0.6143534881476364, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2921, 'Reactive_load': 0.0827, 'Node_id': '7333.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0273033, 'theta_0': -0.4667041661226448, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7334.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.45492947685699026, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7335.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0027689, 'theta_0': -0.648673817091778, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0769, 'Reactive_load': 0.30510000000000004, 'Node_id': '7336.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0100685, 'theta_0': -0.6480468773711691, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24559999999999998, 'Reactive_load': 0.0696, 'Node_id': '7337.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9915095, 'theta_0': -0.6580482597569197, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.092, 'Reactive_load': 0.3094, 'Node_id': '7338.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021564, 'theta_0': -0.5770468484934547, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3204, 'Reactive_load': 0.0908, 'Node_id': '7339.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9849448, 'theta_0': -0.7094320444792793, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3864, 'Reactive_load': 0.10949999999999999, 'Node_id': '7340.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0189533, 'theta_0': -0.47776281660937614, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7341.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0089995, 'theta_0': -0.5692195878503282, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7342.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9824264, 'theta_0': -0.6608624635494205, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.848, 'Reactive_load': 0.5236, 'Node_id': '7343.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9846474, 'theta_0': -0.7018135252146914, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0141, 'Reactive_load': 0.2873, 'Node_id': '7344.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9931515, 'theta_0': -0.6477456684488598, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7337, 'Reactive_load': 0.2079, 'Node_id': '7345.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0328524, 'theta_0': -0.419002345443585, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7346.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.021348, 'theta_0': -0.48428844560295026, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7347.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123994, 'theta_0': -0.5481070031140686, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7348.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.40269503611050117, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7349.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3798905815572358, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7350.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3671285595337281, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7351.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.37090946629232335, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7352.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3524588577312055, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7353.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3821348131759127, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7354.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3818039511096122, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7355.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3856210385436013, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7356.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0361943, 'theta_0': -0.41287632703554755, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7357.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.374578200739893, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7358.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.40015697831225105, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7359.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.3874399334235672, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7360.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.39558014905486877, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7361.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9877936, 'theta_0': -0.6884693660724736, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5602, 'Reactive_load': 0.15869999999999998, 'Node_id': '7362.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0032193, 'theta_0': -0.6192229044008228, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2659, 'Reactive_load': 0.0753, 'Node_id': '7363.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9855051, 'theta_0': -0.6739196905626106, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1427, 'Reactive_load': 0.0404, 'Node_id': '7364.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9952937, 'theta_0': -0.6735173223568558, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8097, 'Reactive_load': 0.22940000000000002, 'Node_id': '7365.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0305925, 'theta_0': -0.44269218807309463, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7366.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0166454, 'theta_0': -0.5201584000300602, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7367.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055832, 'theta_0': -0.6018447308518854, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7368.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.41046803446718316, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7369.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4017270416007601, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7370.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40884381361198474, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7371.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4112693849399438, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7372.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0368587, 'theta_0': -0.4308899400585511, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7373.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4173805156894618, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7374.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.3965571843701352, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7375.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40012486425401433, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7376.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.41040186903524006, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7377.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.40208342038072487, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7378.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4051858129393148, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7379.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0084931, 'theta_0': -0.5774235429059126, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5224, 'Reactive_load': 0.14800000000000002, 'Node_id': '7380.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9970172, 'theta_0': -0.6877349489765269, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9475, 'Reactive_load': 0.2685, 'Node_id': '7381.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9887354, 'theta_0': -0.6658982969467848, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7813, 'Reactive_load': 0.2214, 'Node_id': '7382.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9951655, 'theta_0': -0.5914745603951882, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8559, 'Reactive_load': 0.2425, 'Node_id': '7383.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9888157, 'theta_0': -0.6799523338556663, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6687000000000001, 'Reactive_load': 0.1895, 'Node_id': '7384.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9907025, 'theta_0': -0.7023550136151228, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3524, 'Reactive_load': 0.3832, 'Node_id': '7385.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9939336, 'theta_0': -0.6827031298364421, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1179999999999999, 'Reactive_load': 0.31679999999999997, 'Node_id': '7386.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9841147, 'theta_0': -0.6891917927564589, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6073, 'Reactive_load': 0.45539999999999997, 'Node_id': '7387.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966853, 'theta_0': -0.6942304012273339, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2597, 'Reactive_load': 0.3569, 'Node_id': '7388.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0367512, 'theta_0': -0.601185275647312, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7389.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0282107, 'theta_0': -0.6484914301849446, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7390.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0121663, 'theta_0': -0.6810283991526909, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.42840000000000006, 'Reactive_load': 0.12140000000000001, 'Node_id': '7391.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0094366, 'theta_0': -0.6074878341091886, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5203, 'Reactive_load': 0.1474, 'Node_id': '7392.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0012132, 'theta_0': -0.5979011245004191, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0731, 'Reactive_load': 0.0207, 'Node_id': '7393.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9873534, 'theta_0': -0.6692013849160616, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3741, 'Reactive_load': 0.106, 'Node_id': '7394.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9956667, 'theta_0': -0.6935775433673329, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7042, 'Reactive_load': 0.19949999999999998, 'Node_id': '7395.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9891939, 'theta_0': -0.6794175475195627, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.25489999999999996, 'Reactive_load': 0.0722, 'Node_id': '7396.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0162396, 'theta_0': -0.6946409899338656, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0572, 'Reactive_load': 0.2995, 'Node_id': '7397.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9990103, 'theta_0': -0.6253308411978097, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7359, 'Reactive_load': 0.20850000000000002, 'Node_id': '7398.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9969729, 'theta_0': -0.6550204451172675, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5683, 'Reactive_load': 0.161, 'Node_id': '7399.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0269829, 'theta_0': -0.4533225696679716, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7400.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0178573, 'theta_0': -0.5074481968320441, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7401.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0080092, 'theta_0': -0.5889601518083026, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7402.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9983346, 'theta_0': -0.6179829875936211, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6967, 'Reactive_load': 0.1974, 'Node_id': '7403.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9869937, 'theta_0': -0.7096268232238019, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.7772999999999999, 'Reactive_load': 0.5036, 'Node_id': '7404.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151963, 'theta_0': -0.5986843060956666, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5326, 'Reactive_load': 0.1509, 'Node_id': '7405.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0218191, 'theta_0': -0.5034223854728089, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7406.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.019883, 'theta_0': -0.52222804636366, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7407.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0008913, 'theta_0': -0.5766920230565243, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5366, 'Reactive_load': 0.152, 'Node_id': '7408.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0077839, 'theta_0': -0.5802150923247225, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7409.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9923129, 'theta_0': -0.6633589126983032, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.1567, 'Reactive_load': 0.6111, 'Node_id': '7410.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9948022, 'theta_0': -0.719986486798402, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6940999999999999, 'Reactive_load': 0.1967, 'Node_id': '7411.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0002475, 'theta_0': -0.5982130846509206, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0859, 'Reactive_load': 0.024300000000000002, 'Node_id': '7412.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0015679, 'theta_0': -0.616174425062827, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4291, 'Reactive_load': 0.1216, 'Node_id': '7413.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0237228, 'theta_0': -0.5571028664111604, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7414.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0182699, 'theta_0': -0.5648573119178936, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7415.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0123544, 'theta_0': -0.6027306599801977, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7416.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0047252, 'theta_0': -0.6753004030805707, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9788, 'Reactive_load': 0.2773, 'Node_id': '7417.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0051305, 'theta_0': -0.6677283793872559, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.48219999999999996, 'Reactive_load': 0.1366, 'Node_id': '7418.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000027, 'theta_0': -0.5850814542517181, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.07, 'Reactive_load': 0.3032, 'Node_id': '7419.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0093205, 'theta_0': -0.6397024756706766, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7420.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9943574, 'theta_0': -0.7213293780314716, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9064, 'Reactive_load': 0.2568, 'Node_id': '7421.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400028, 'theta_0': -0.4785167115797751, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7422.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0216486, 'theta_0': -0.5267067532104952, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7423.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0036824, 'theta_0': -0.5972945004123035, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.3988, 'Reactive_load': 0.39630000000000004, 'Node_id': '7424.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9881105, 'theta_0': -0.7128583177871619, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0598, 'Reactive_load': 0.3003, 'Node_id': '7425.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9940253, 'theta_0': -0.5771865795533694, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.418, 'Reactive_load': 0.1184, 'Node_id': '7426.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.986157, 'theta_0': -0.7077778039609466, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5644999999999998, 'Reactive_load': 0.44329999999999997, 'Node_id': '7427.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400027, 'theta_0': -0.4796248385751588, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7428.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0288833, 'theta_0': -0.495872108581974, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7429.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0092213, 'theta_0': -0.5616668674985404, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4692, 'Reactive_load': 0.1329, 'Node_id': '7430.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9969641, 'theta_0': -0.7011922578141515, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3557, 'Reactive_load': 0.1008, 'Node_id': '7431.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0113532, 'theta_0': -1.0393253483252323, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22940000000000002, 'Reactive_load': 0.065, 'Node_id': '7432.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0229121, 'theta_0': -1.0033830325473347, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1082, 'Reactive_load': 0.030699999999999998, 'Node_id': '8001.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0165544, 'theta_0': -1.2309220434621486, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11810000000000001, 'Reactive_load': 0.0335, 'Node_id': '8002.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300034, 'theta_0': -1.027211524132623, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1199, 'Reactive_load': 0.034, 'Node_id': '8003.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3797, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0256336, 'theta_0': -1.0162899692586356, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8004.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300034, 'theta_0': -1.0377119485113964, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0201, 'Reactive_load': 0.289, 'Node_id': '8005.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.4021000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.027022, 'theta_0': -1.1789287723117, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.374, 'Reactive_load': 0.106, 'Node_id': '8006.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.71, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0117128, 'theta_0': -1.1954688038472447, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3007, 'Reactive_load': 0.0852, 'Node_id': '8007.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9883101, 'theta_0': -1.2436744312681853, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3922, 'Reactive_load': 0.11109999999999999, 'Node_id': '8008.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9985149, 'theta_0': -0.7527552180375208, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0981, 'Reactive_load': 0.0278, 'Node_id': '8009.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0202252, 'theta_0': -0.9812784724774117, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.053899999999999997, 'Reactive_load': 0.015300000000000001, 'Node_id': '8010.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0043286, 'theta_0': -1.0734832220225985, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2725, 'Reactive_load': 0.07719999999999999, 'Node_id': '8011.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168369, 'theta_0': -0.7386804815237106, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.076, 'Reactive_load': 0.0215, 'Node_id': '8012.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0055883, 'theta_0': -1.2522940539384246, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6003000000000001, 'Reactive_load': 0.17010000000000003, 'Node_id': '8013.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9997178, 'theta_0': -1.1208022745434034, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1515, 'Reactive_load': 0.0429, 'Node_id': '8014.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0086374, 'theta_0': -1.1161673083686372, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.23929999999999998, 'Reactive_load': 0.0678, 'Node_id': '8015.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.8324986657893184, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6986, 'Reactive_load': 0.1979, 'Node_id': '8016.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5497, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0041348, 'theta_0': -1.1986285653783477, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1844, 'Reactive_load': 0.052199999999999996, 'Node_id': '8017.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0216998, 'theta_0': -1.1202586917478696, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8018.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -1.1949712279307938, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0312000000000001, 'Reactive_load': 0.2922, 'Node_id': '8019.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.7792, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9931819, 'theta_0': -1.2347803730263625, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17350000000000002, 'Reactive_load': 0.0492, 'Node_id': '8020.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0196725, 'theta_0': -1.1121522656908869, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1073, 'Reactive_load': 0.0304, 'Node_id': '8021.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0257431, 'theta_0': -1.126393262269242, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0689, 'Reactive_load': 0.0195, 'Node_id': '8022.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1668, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0232461, 'theta_0': -1.0610270864037954, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8023.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0250122, 'theta_0': -1.001844228105729, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8024.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0264944, 'theta_0': -1.1686612446683127, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5075999999999999, 'Reactive_load': 0.1438, 'Node_id': '8025.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5121, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9947352, 'theta_0': -1.2589676344926903, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5508, 'Reactive_load': 0.1561, 'Node_id': '8026.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0198753, 'theta_0': -1.1881180529964979, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39380000000000004, 'Reactive_load': 0.1116, 'Node_id': '8027.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0067769, 'theta_0': -1.2495903294875754, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24059999999999998, 'Reactive_load': 0.0682, 'Node_id': '8028.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9895107, 'theta_0': -1.2503603338469702, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2565, 'Reactive_load': 0.0727, 'Node_id': '8029.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0326995, 'theta_0': -0.6406602425510011, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8030.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0285487, 'theta_0': -0.6555548300276431, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8031.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0212245, 'theta_0': -0.7248738973961768, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7796, 'Reactive_load': 0.22089999999999999, 'Node_id': '8032.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0326995, 'theta_0': -0.6406602250977086, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8033.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0326995, 'theta_0': -0.6406602250977086, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8034.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0326995, 'theta_0': -0.6406602425510011, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8035.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0326995, 'theta_0': -0.6406602250977086, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8036.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0040627, 'theta_0': -1.0767963281817816, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11289999999999999, 'Reactive_load': 0.032, 'Node_id': '8037.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9951131, 'theta_0': -1.118244372351558, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7315, 'Reactive_load': 0.2073, 'Node_id': '8038.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0001988, 'theta_0': -1.248016496287882, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1261, 'Reactive_load': 0.035699999999999996, 'Node_id': '8039.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9930898, 'theta_0': -1.1905507627210976, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1809, 'Reactive_load': 0.0513, 'Node_id': '8040.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0196968, 'theta_0': -1.151819707504486, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5277000000000001, 'Reactive_load': 0.1495, 'Node_id': '8041.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.001679, 'theta_0': -1.1249510967083192, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2731, 'Reactive_load': 0.0774, 'Node_id': '8042.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0168011, 'theta_0': -0.9513853806204638, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8043.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151643, 'theta_0': -0.9894540659326512, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8044.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0132356, 'theta_0': -1.0448365570509623, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8045.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0141975, 'theta_0': -0.9287166952297111, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8046.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0168011, 'theta_0': -0.9513853806204638, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8047.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0395073, 'theta_0': -0.9079886588875434, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8048.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0152685, 'theta_0': -0.9362907958648358, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8049.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0168011, 'theta_0': -0.9513853806204638, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8050.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0168011, 'theta_0': -0.9513853806204638, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8051.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0194075, 'theta_0': -1.2125412655244028, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1519, 'Reactive_load': 0.043, 'Node_id': '8052.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9898547, 'theta_0': -1.2222264989694474, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2096, 'Reactive_load': 0.0594, 'Node_id': '8053.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9977824, 'theta_0': -1.1548495118194855, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1616, 'Reactive_load': 0.0458, 'Node_id': '8054.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0196936, 'theta_0': -1.1305109652135945, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3379, 'Reactive_load': 0.09570000000000001, 'Node_id': '8055.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0258525, 'theta_0': -1.2242239759384774, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10550000000000001, 'Reactive_load': 0.029900000000000003, 'Node_id': '8056.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0257252, 'theta_0': -1.0082149591281886, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8057.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0184844, 'theta_0': -1.0035299892703529, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8058.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0143207, 'theta_0': -1.0377832103047553, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8059.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0257252, 'theta_0': -1.0082149591281886, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8060.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0257252, 'theta_0': -1.0082149591281886, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8061.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.021633, 'theta_0': -1.0656212991405274, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10189999999999999, 'Reactive_load': 0.028900000000000002, 'Node_id': '8062.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0051362, 'theta_0': -1.1619349551706368, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6938, 'Reactive_load': 0.1966, 'Node_id': '8063.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0113515, 'theta_0': -1.0475691143410548, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21230000000000002, 'Reactive_load': 0.0601, 'Node_id': '8064.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0006446, 'theta_0': -1.157730038123562, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.06849999999999999, 'Reactive_load': 0.0194, 'Node_id': '8065.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9977296, 'theta_0': -1.091194421846109, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0323, 'Reactive_load': 0.0092, 'Node_id': '8066.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0132624, 'theta_0': -0.9055239921843395, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8067.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0193018, 'theta_0': -0.9198404917061361, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8068.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0215831, 'theta_0': -0.9437101207018936, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8069.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0132624, 'theta_0': -0.9055239921843395, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8070.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0000036, 'theta_0': -0.8097578808471658, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8071.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9905165, 'theta_0': -1.135655689702991, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7008, 'Reactive_load': 0.1986, 'Node_id': '8072.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9961717, 'theta_0': -1.2454439158770274, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1905, 'Reactive_load': 0.054000000000000006, 'Node_id': '8073.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0290596, 'theta_0': -0.8884476733295346, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8074.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0289245, 'theta_0': -0.9436964024139729, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8075.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0263568, 'theta_0': -1.01166865155862, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8076.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -0.8472518990498416, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8077.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -0.845045680702273, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8078.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -0.8546404013723091, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8079.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300037, 'theta_0': -0.8495107390743576, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8080.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0004248, 'theta_0': -1.2360717421399128, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2605, 'Reactive_load': 0.0738, 'Node_id': '8081.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0326176, 'theta_0': -0.8108290766755774, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8082.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0265714, 'theta_0': -0.8589803894501957, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8083.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0145531, 'theta_0': -0.9347967765718361, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8084.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0146683, 'theta_0': -0.943753736479901, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8085.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0155946, 'theta_0': -0.9675926651341158, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2125, 'Reactive_load': 0.0602, 'Node_id': '8086.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0326176, 'theta_0': -0.8108290766755774, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8087.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400035, 'theta_0': -0.7616468502716834, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8088.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0239769, 'theta_0': -1.2158759090465558, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2147, 'Reactive_load': 0.0608, 'Node_id': '8089.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0152228, 'theta_0': -1.1786460115195845, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1521, 'Reactive_load': 0.0431, 'Node_id': '8090.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.029654, 'theta_0': -1.1018204226645636, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11869999999999999, 'Reactive_load': 0.0336, 'Node_id': '8091.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.35, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0038364, 'theta_0': -1.1666508348095255, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0711, 'Reactive_load': 0.020099999999999996, 'Node_id': '8092.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0135561, 'theta_0': -1.0305410363867749, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.049100000000000005, 'Reactive_load': 0.0139, 'Node_id': '8093.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0084979, 'theta_0': -0.7479031852636839, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8094.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9982946, 'theta_0': -0.799522659248233, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4681, 'Reactive_load': 0.41600000000000004, 'Node_id': '8095.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9963482, 'theta_0': -1.218401278301144, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1117, 'Reactive_load': 0.0316, 'Node_id': '8096.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0276473, 'theta_0': -1.0089413128029912, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8097.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0222849, 'theta_0': -1.0692804691837963, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8098.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.026992, 'theta_0': -0.9302770021277016, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8099.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0220752, 'theta_0': -1.213713847529063, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4113, 'Reactive_load': 0.1165, 'Node_id': '8100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0049675, 'theta_0': -1.131916827926076, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12539999999999998, 'Reactive_load': 0.0355, 'Node_id': '8101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0116826, 'theta_0': -0.8917223473385889, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.047599999999999996, 'Reactive_load': 0.013500000000000002, 'Node_id': '8102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0194488, 'theta_0': -0.9648185515545336, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0815, 'Reactive_load': 0.0231, 'Node_id': '8103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0120466, 'theta_0': -0.7564771675739836, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4653, 'Reactive_load': 0.1318, 'Node_id': '8104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0200295, 'theta_0': -0.9772643024642873, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0043308, 'theta_0': -1.045922762710941, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9989, 'Reactive_load': 0.28300000000000003, 'Node_id': '8106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0271647, 'theta_0': -1.0587816156013494, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09210000000000002, 'Reactive_load': 0.026099999999999998, 'Node_id': '8107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0039714, 'theta_0': -1.2601984581344894, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4393, 'Reactive_load': 0.1245, 'Node_id': '8108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9914129, 'theta_0': -1.1751383884152666, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5176, 'Reactive_load': 0.1467, 'Node_id': '8109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.016374, 'theta_0': -1.1713741670043203, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2041, 'Reactive_load': 0.057800000000000004, 'Node_id': '8110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.4, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0140917, 'theta_0': -1.0990445637557291, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8711, 'Reactive_load': 0.2468, 'Node_id': '8111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0322691, 'theta_0': -1.023622167259434, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0289294, 'theta_0': -1.0383433039150127, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0247931, 'theta_0': -1.0726381161454903, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2441, 'Reactive_load': 0.0692, 'Node_id': '8114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0322691, 'theta_0': -1.023622167259434, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0322691, 'theta_0': -1.023622167259434, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0400037, 'theta_0': -1.0000875018537192, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0044775, 'theta_0': -1.1544903405127176, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.36619999999999997, 'Reactive_load': 0.1038, 'Node_id': '8118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0225002, 'theta_0': -1.0466567783811598, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.48, 'Reactive_load': 0.136, 'Node_id': '8119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.030003, 'theta_0': -0.8511475088468778, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.319, 'Reactive_load': 0.0904, 'Node_id': '8120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.9853000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0151324, 'theta_0': -1.1405681632288243, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7916, 'Reactive_load': 0.2243, 'Node_id': '8121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0221169, 'theta_0': -0.8522330861883258, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.29410000000000003, 'Reactive_load': 0.0833, 'Node_id': '8122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9954285, 'theta_0': -1.1651371980157335, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1753, 'Reactive_load': 0.049699999999999994, 'Node_id': '8123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0038419, 'theta_0': -1.1330189509888329, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.20199999999999999, 'Reactive_load': 0.0572, 'Node_id': '8124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0154026, 'theta_0': -1.152034505175529, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.45159999999999995, 'Reactive_load': 0.128, 'Node_id': '8125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0323198, 'theta_0': -0.9269471757142841, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0308054, 'theta_0': -0.9471295523191083, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0311164, 'theta_0': -0.98744903168796, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300038, 'theta_0': -0.8967592104802119, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300038, 'theta_0': -0.851644281911873, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0300038, 'theta_0': -0.8884628751473194, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9955454, 'theta_0': -1.129721657512673, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2193, 'Reactive_load': 0.0621, 'Node_id': '8132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0217658, 'theta_0': -1.1018438275298326, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0216485, 'theta_0': -1.154101132089523, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.115, 'Reactive_load': 0.3159, 'Node_id': '8134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0245253, 'theta_0': -1.2085966817885556, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8120999999999999, 'Reactive_load': 0.23010000000000003, 'Node_id': '8135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0021666, 'theta_0': -1.2110051837964302, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1005, 'Reactive_load': 0.0285, 'Node_id': '8136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0096317, 'theta_0': -1.09637401801396, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2432, 'Reactive_load': 0.0689, 'Node_id': '8137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300036, 'theta_0': -1.2292513793955544, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.065, 'Reactive_load': 0.0184, 'Node_id': '8138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.5877, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.005414, 'theta_0': -0.7604149445257483, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9993519, 'theta_0': -0.807783337504507, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6850999999999998, 'Reactive_load': 0.4775, 'Node_id': '8140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0149793, 'theta_0': -0.9072885026048133, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2094, 'Reactive_load': 0.0593, 'Node_id': '8141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9966649, 'theta_0': -1.0813811859537232, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2236, 'Reactive_load': 0.0634, 'Node_id': '8142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0199346, 'theta_0': -0.7973281171533602, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22940000000000002, 'Reactive_load': 0.065, 'Node_id': '8143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0073183, 'theta_0': -1.0704277788208871, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0604, 'Reactive_load': 0.0171, 'Node_id': '8144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0078965, 'theta_0': -1.156528623279659, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0616, 'Reactive_load': 0.0174, 'Node_id': '8145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0037208, 'theta_0': -1.262930753625194, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4991, 'Reactive_load': 0.1414, 'Node_id': '8146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0118935, 'theta_0': -1.038282618816921, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.007094, 'theta_0': -1.079058397065414, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0006368, 'theta_0': -1.1530767110850648, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0126974, 'theta_0': -1.1214372078719863, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0851, 'Reactive_load': 0.0241, 'Node_id': '8150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300035, 'theta_0': -1.1777347053039482, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17379999999999998, 'Reactive_load': 0.0492, 'Node_id': '8151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.46399999999999997, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0273114, 'theta_0': -0.8163874092911112, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4635, 'Reactive_load': 0.1313, 'Node_id': '8152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 0.9945546, 'theta_0': -1.2441504872749591, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38549999999999995, 'Reactive_load': 0.1092, 'Node_id': '8153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0077189, 'theta_0': -1.2364170031725426, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2631, 'Reactive_load': 0.0746, 'Node_id': '8154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0283478, 'theta_0': -0.8231257241073333, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0216632, 'theta_0': -0.8018808259604799, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0156746, 'theta_0': -0.8724807032005897, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0304051, 'theta_0': -0.6767784526109147, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0300029, 'theta_0': -0.7354016187244514, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.0863, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0267196, 'theta_0': -0.7905583864106023, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9076000000000001, 'Reactive_load': 0.2571, 'Node_id': '8160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None}
    ]
    nodes_AC = pd.DataFrame(nodes_AC_data)

    lines_AC_data = [
        {'fromNode': '1001.0', 'toNode': '1064.0', 'Resistance': 0.00524, 'Reactance': 0.0358, 'Conductance': 0, 'Susceptance': 0.00609, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1'},
        {'fromNode': '1001.0', 'toNode': '1064.0', 'Resistance': 0.00524, 'Reactance': 0.0358, 'Conductance': 0, 'Susceptance': 0.00609, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2'},
        {'fromNode': '1001.0', 'toNode': '1071.0', 'Resistance': 0.00435, 'Reactance': 0.02808, 'Conductance': 0, 'Susceptance': 0.00536, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3'},
        {'fromNode': '1001.0', 'toNode': '1071.0', 'Resistance': 0.00435, 'Reactance': 0.02808, 'Conductance': 0, 'Susceptance': 0.00536, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '4'},
        {'fromNode': '1002.0', 'toNode': '1007.0', 'Resistance': 0.07776, 'Reactance': 0.46693, 'Conductance': 0, 'Susceptance': 0.06336, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '5'},
        {'fromNode': '1010.0', 'toNode': '1002.0', 'Resistance': 0.05552, 'Reactance': 0.1357, 'Conductance': 0, 'Susceptance': 0.01777, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '6'},
        {'fromNode': '1004.0', 'toNode': '1003.0', 'Resistance': 0.0045, 'Reactance': 0.11582, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '7'},
        {'fromNode': '1022.0', 'toNode': '1003.0', 'Resistance': 0.00485, 'Reactance': 0.02012, 'Conductance': 0, 'Susceptance': 0.00341, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '8'},
        {'fromNode': '1055.0', 'toNode': '1003.0', 'Resistance': 0.01574, 'Reactance': 0.03782, 'Conductance': 0, 'Susceptance': 0.00513, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '9'},
        {'fromNode': '1004.0', 'toNode': '3133.0', 'Resistance': 0.0042, 'Reactance': 0.03, 'Conductance': 0, 'Susceptance': 0.055, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '10'},
        {'fromNode': '1006.0', 'toNode': '1005.0', 'Resistance': 0.00602, 'Reactance': 0.14707, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 42.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '11'},
        {'fromNode': '1025.0', 'toNode': '1005.0', 'Resistance': 0.01842, 'Reactance': 0.0821, 'Conductance': 0, 'Susceptance': 0.01204, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '12'},
        {'fromNode': '1005.0', 'toNode': '1061.0', 'Resistance': 0.00599, 'Reactance': 0.02484, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '13'},
        {'fromNode': '1067.0', 'toNode': '1005.0', 'Resistance': 0.01711, 'Reactance': 0.04181, 'Conductance': 0, 'Susceptance': 0.00548, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '14'},
        {'fromNode': '1007.0', 'toNode': '1015.0', 'Resistance': 0.06224, 'Reactance': 0.37375, 'Conductance': 0, 'Susceptance': 0.05072, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '15'},
        {'fromNode': '1031.0', 'toNode': '1007.0', 'Resistance': 0.04803, 'Reactance': 0.27434, 'Conductance': 0, 'Susceptance': 0.04123, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '16'},
        {'fromNode': '1009.0', 'toNode': '1008.0', 'Resistance': 0.00455, 'Reactance': 0.09471, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 120.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '17'},
        {'fromNode': '1008.0', 'toNode': '1032.0', 'Resistance': 0.04501, 'Reactance': 0.11002, 'Conductance': 0, 'Susceptance': 0.01441, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '18'},
        {'fromNode': '1008.0', 'toNode': '1069.0', 'Resistance': 0.0083, 'Reactance': 0.03628, 'Conductance': 0, 'Susceptance': 0.00553, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '19'},
        {'fromNode': '1011.0', 'toNode': '1010.0', 'Resistance': 0.00721, 'Reactance': 0.3246, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 46.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '20'},
        {'fromNode': '1010.0', 'toNode': '1015.0', 'Resistance': 0.04703, 'Reactance': 0.28243, 'Conductance': 0, 'Susceptance': 0.03833, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '21'},
        {'fromNode': '1010.0', 'toNode': '1030.0', 'Resistance': 0.0616, 'Reactance': 0.3699, 'Conductance': 0, 'Susceptance': 0.0502, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '22'},
        {'fromNode': '1012.0', 'toNode': '1087.0', 'Resistance': 0.04934, 'Reactance': 0.20469, 'Conductance': 0, 'Susceptance': 0.03473, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '23'},
        {'fromNode': '3051.0', 'toNode': '1012.0', 'Resistance': 0.0571, 'Reactance': 0.24977, 'Conductance': 0, 'Susceptance': 0.03804, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '24'},
        {'fromNode': '1013.0', 'toNode': '1031.0', 'Resistance': 0.04509, 'Reactance': 0.27078, 'Conductance': 0, 'Susceptance': 0.03675, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '25'},
        {'fromNode': '1047.0', 'toNode': '1013.0', 'Resistance': 0.00947, 'Reactance': 0.06117, 'Conductance': 0, 'Susceptance': 0.01167, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '26'},
        {'fromNode': '1086.0', 'toNode': '1013.0', 'Resistance': 0.079, 'Reactance': 0.19311, 'Conductance': 0, 'Susceptance': 0.02529, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '27'},
        {'fromNode': '1014.0', 'toNode': '1047.0', 'Resistance': 0.03566, 'Reactance': 0.14794, 'Conductance': 0, 'Susceptance': 0.0251, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '28'},
        {'fromNode': '1014.0', 'toNode': '1083.0', 'Resistance': 0.04105, 'Reactance': 0.18293, 'Conductance': 0, 'Susceptance': 0.02682, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '29'},
        {'fromNode': '1016.0', 'toNode': '1034.0', 'Resistance': 0.01828, 'Reactance': 0.10976, 'Conductance': 0, 'Susceptance': 0.01489, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '30'},
        {'fromNode': '1068.0', 'toNode': '1016.0', 'Resistance': 0.02773, 'Reactance': 0.15837, 'Conductance': 0, 'Susceptance': 0.0238, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '31'},
        {'fromNode': '1018.0', 'toNode': '1017.0', 'Resistance': 0.03227, 'Reactance': 0.07887, 'Conductance': 0, 'Susceptance': 0.01033, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '32'},
        {'fromNode': '1017.0', 'toNode': '1029.0', 'Resistance': 0.03244, 'Reactance': 0.14188, 'Conductance': 0, 'Susceptance': 0.02161, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '33'},
        {'fromNode': '1037.0', 'toNode': '1017.0', 'Resistance': 0.00752, 'Reactance': 0.03352, 'Conductance': 0, 'Susceptance': 0.00491, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '34'},
        {'fromNode': '1047.0', 'toNode': '1017.0', 'Resistance': 0.036, 'Reactance': 0.156, 'Conductance': 0, 'Susceptance': 0.024, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '35'},
        {'fromNode': '1024.0', 'toNode': '1018.0', 'Resistance': 0.01094, 'Reactance': 0.06572, 'Conductance': 0, 'Susceptance': 0.00892, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '36'},
        {'fromNode': '1019.0', 'toNode': '1040.0', 'Resistance': 0.04978, 'Reactance': 0.20651, 'Conductance': 0, 'Susceptance': 0.03504, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '37'},
        {'fromNode': '1019.0', 'toNode': '1045.0', 'Resistance': 0.01522, 'Reactance': 0.0372, 'Conductance': 0, 'Susceptance': 0.00487, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '38'},
        {'fromNode': '1019.0', 'toNode': '1071.0', 'Resistance': 0.0044, 'Reactance': 0.028, 'Conductance': 0, 'Susceptance': 0.0054, 'MVA_rating': 225.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '39'},
        {'fromNode': '1071.0', 'toNode': '1019.0', 'Resistance': 0.0044, 'Reactance': 0.028, 'Conductance': 0, 'Susceptance': 0.0054, 'MVA_rating': 225.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '40'},
        {'fromNode': '1019.0', 'toNode': '1084.0', 'Resistance': 0.03, 'Reactance': 0.16, 'Conductance': 0, 'Susceptance': 0.025, 'MVA_rating': 220.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '41'},
        {'fromNode': '1021.0', 'toNode': '1020.0', 'Resistance': 0.00327, 'Reactance': 0.07812, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '42'},
        {'fromNode': '1025.0', 'toNode': '1020.0', 'Resistance': 0.00828, 'Reactance': 0.03434, 'Conductance': 0, 'Susceptance': 0.00583, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '43'},
        {'fromNode': '1084.0', 'toNode': '1020.0', 'Resistance': 0.01056, 'Reactance': 0.06337, 'Conductance': 0, 'Susceptance': 0.0086, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '44'},
        {'fromNode': '1020.0', 'toNode': '3059.0', 'Resistance': 0.0189, 'Reactance': 0.06, 'Conductance': 0, 'Susceptance': 0.006, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '45'},
        {'fromNode': '1020.0', 'toNode': '3130.0', 'Resistance': 0.00996, 'Reactance': 0.06938, 'Conductance': 0, 'Susceptance': 0.01134, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '46'},
        {'fromNode': '1023.0', 'toNode': '1022.0', 'Resistance': 0.00535, 'Reactance': 0.11733, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '47'},
        {'fromNode': '1022.0', 'toNode': '1028.0', 'Resistance': 0.01433, 'Reactance': 0.09253, 'Conductance': 0, 'Susceptance': 0.01765, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '48'},
        {'fromNode': '1022.0', 'toNode': '1028.0', 'Resistance': 0.01433, 'Reactance': 0.09253, 'Conductance': 0, 'Susceptance': 0.01765, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '49'},
        {'fromNode': '1024.0', 'toNode': '1090.0', 'Resistance': 0.01414, 'Reactance': 0.06303, 'Conductance': 0, 'Susceptance': 0.00924, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '50'},
        {'fromNode': '1026.0', 'toNode': '1025.0', 'Resistance': 0.0042, 'Reactance': 0.18046, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '51'},
        {'fromNode': '1027.0', 'toNode': '1045.0', 'Resistance': 0.02505, 'Reactance': 0.10957, 'Conductance': 0, 'Susceptance': 0.01669, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '52'},
        {'fromNode': '1056.0', 'toNode': '1027.0', 'Resistance': 0.03121, 'Reactance': 0.0763, 'Conductance': 0, 'Susceptance': 0.00999, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '53'},
        {'fromNode': '1028.0', 'toNode': '3109.0', 'Resistance': 0.00428, 'Reactance': 0.01875, 'Conductance': 0, 'Susceptance': 0.00286, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '54'},
        {'fromNode': '1029.0', 'toNode': '1056.0', 'Resistance': 0.02054, 'Reactance': 0.11733, 'Conductance': 0, 'Susceptance': 0.01763, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '55'},
        {'fromNode': '1030.0', 'toNode': '1038.0', 'Resistance': 0.07404, 'Reactance': 0.42285, 'Conductance': 0, 'Susceptance': 0.06354, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '56'},
        {'fromNode': '1083.0', 'toNode': '1030.0', 'Resistance': 0.06537, 'Reactance': 0.37334, 'Conductance': 0, 'Susceptance': 0.0561, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '57'},
        {'fromNode': '1033.0', 'toNode': '1032.0', 'Resistance': 0.00283, 'Reactance': 0.06758, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 170.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '58'},
        {'fromNode': '1032.0', 'toNode': '1036.0', 'Resistance': 0.01956, 'Reactance': 0.11746, 'Conductance': 0, 'Susceptance': 0.01594, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '59'},
        {'fromNode': '1038.0', 'toNode': '1032.0', 'Resistance': 0.045, 'Reactance': 0.11, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 155.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '60'},
        {'fromNode': '1033.0', 'toNode': '1081.0', 'Resistance': 0.003, 'Reactance': 0.031, 'Conductance': 0, 'Susceptance': 0.059, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '61'},
        {'fromNode': '1033.0', 'toNode': '3050.0', 'Resistance': 0.0139, 'Reactance': 0.136, 'Conductance': 0, 'Susceptance': 0.262, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '62'},
        {'fromNode': '1035.0', 'toNode': '1034.0', 'Resistance': 0.00438, 'Reactance': 0.08496, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 130.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '63'},
        {'fromNode': '1067.0', 'toNode': '1034.0', 'Resistance': 0.00795, 'Reactance': 0.04771, 'Conductance': 0, 'Susceptance': 0.00647, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '64'},
        {'fromNode': '1088.0', 'toNode': '1036.0', 'Resistance': 0.02065, 'Reactance': 0.11793, 'Conductance': 0, 'Susceptance': 0.01772, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '65'},
        {'fromNode': '1037.0', 'toNode': '1071.0', 'Resistance': 0.00669, 'Reactance': 0.04319, 'Conductance': 0, 'Susceptance': 0.00824, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '66'},
        {'fromNode': '1039.0', 'toNode': '1038.0', 'Resistance': 0.00201, 'Reactance': 0.08591, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '67'},
        {'fromNode': '1082.0', 'toNode': '1038.0', 'Resistance': 0.01115, 'Reactance': 0.04969, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '68'},
        {'fromNode': '1083.0', 'toNode': '1038.0', 'Resistance': 0.01, 'Reactance': 0.05, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '69'},
        {'fromNode': '1040.0', 'toNode': '1090.0', 'Resistance': 0.02991, 'Reactance': 0.13329, 'Conductance': 0, 'Susceptance': 0.01955, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '70'},
        {'fromNode': '1042.0', 'toNode': '1041.0', 'Resistance': 0.00419, 'Reactance': 0.11123, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 145.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '71'},
        {'fromNode': '1043.0', 'toNode': '1041.0', 'Resistance': 0.003, 'Reactance': 0.12844, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 95.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '72'},
        {'fromNode': '1065.0', 'toNode': '1041.0', 'Resistance': 0.00386, 'Reactance': 0.02203, 'Conductance': 0, 'Susceptance': 0.00331, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '73'},
        {'fromNode': '1041.0', 'toNode': '1067.0', 'Resistance': 0.00393, 'Reactance': 0.01722, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '74'},
        {'fromNode': '1044.0', 'toNode': '3019.0', 'Resistance': 0.03857, 'Reactance': 0.17192, 'Conductance': 0, 'Susceptance': 0.02521, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '75'},
        {'fromNode': '3130.0', 'toNode': '1044.0', 'Resistance': 0.02731, 'Reactance': 0.19029, 'Conductance': 0, 'Susceptance': 0.03111, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '76'},
        {'fromNode': '1045.0', 'toNode': '1064.0', 'Resistance': 0.00545, 'Reactance': 0.03794, 'Conductance': 0, 'Susceptance': 0.0062, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '77'},
        {'fromNode': '1087.0', 'toNode': '1046.0', 'Resistance': 0.03262, 'Reactance': 0.14269, 'Conductance': 0, 'Susceptance': 0.02173, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '78'},
        {'fromNode': '1046.0', 'toNode': '3009.0', 'Resistance': 0.05654, 'Reactance': 0.23454, 'Conductance': 0, 'Susceptance': 0.0398, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '79'},
        {'fromNode': '1048.0', 'toNode': '1047.0', 'Resistance': 0.00335, 'Reactance': 0.06576, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 390.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '80'},
        {'fromNode': '1049.0', 'toNode': '1047.0', 'Resistance': 0.00276, 'Reactance': 0.11783, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 390.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '81'},
        {'fromNode': '1050.0', 'toNode': '1047.0', 'Resistance': 0.00313, 'Reactance': 0.09941, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 124.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '82'},
        {'fromNode': '1051.0', 'toNode': '1047.0', 'Resistance': 0.0056, 'Reactance': 0.1283, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 117.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '83'},
        {'fromNode': '1052.0', 'toNode': '1047.0', 'Resistance': 0.00368, 'Reactance': 0.0826, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 110.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '84'},
        {'fromNode': '1053.0', 'toNode': '1047.0', 'Resistance': 0.0023, 'Reactance': 0.08808, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '85'},
        {'fromNode': '1047.0', 'toNode': '1085.0', 'Resistance': 0.00947, 'Reactance': 0.06117, 'Conductance': 0, 'Susceptance': 0.01167, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '86'},
        {'fromNode': '1053.0', 'toNode': '1081.0', 'Resistance': 0.003, 'Reactance': 0.03, 'Conductance': 0, 'Susceptance': 0.06, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '87'},
        {'fromNode': '1054.0', 'toNode': '1056.0', 'Resistance': 0.02202, 'Reactance': 0.09633, 'Conductance': 0, 'Susceptance': 0.01467, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '88'},
        {'fromNode': '1089.0', 'toNode': '1054.0', 'Resistance': 0.08737, 'Reactance': 0.21357, 'Conductance': 0, 'Susceptance': 0.02796, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '89'},
        {'fromNode': '1055.0', 'toNode': '1089.0', 'Resistance': 0.04982, 'Reactance': 0.12179, 'Conductance': 0, 'Susceptance': 0.01595, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '90'},
        {'fromNode': '1057.0', 'toNode': '1056.0', 'Resistance': 0.00315, 'Reactance': 0.1312, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '91'},
        {'fromNode': '1056.0', 'toNode': '1061.0', 'Resistance': 0.02267, 'Reactance': 0.12946, 'Conductance': 0, 'Susceptance': 0.01945, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '92'},
        {'fromNode': '1056.0', 'toNode': '1065.0', 'Resistance': 0.02269, 'Reactance': 0.12962, 'Conductance': 0, 'Susceptance': 0.01948, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '93'},
        {'fromNode': '1059.0', 'toNode': '1058.0', 'Resistance': 0.02655, 'Reactance': 0.46187, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 33.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '94'},
        {'fromNode': '1060.0', 'toNode': '1058.0', 'Resistance': 0.00894, 'Reactance': 0.38854, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 27.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '95'},
        {'fromNode': '1069.0', 'toNode': '1058.0', 'Resistance': 0.0074, 'Reactance': 0.05155, 'Conductance': 0, 'Susceptance': 0.00843, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '96'},
        {'fromNode': '1069.0', 'toNode': '1058.0', 'Resistance': 0.0074, 'Reactance': 0.05155, 'Conductance': 0, 'Susceptance': 0.00843, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '97'},
        {'fromNode': '1058.0', 'toNode': '1087.0', 'Resistance': 0.01573, 'Reactance': 0.10956, 'Conductance': 0, 'Susceptance': 0.01791, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '98'},
        {'fromNode': '1058.0', 'toNode': '1087.0', 'Resistance': 0.01573, 'Reactance': 0.10956, 'Conductance': 0, 'Susceptance': 0.01791, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '99'},
        {'fromNode': '1062.0', 'toNode': '1061.0', 'Resistance': 0.0163, 'Reactance': 0.31787, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 29.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '100'},
        {'fromNode': '1063.0', 'toNode': '1061.0', 'Resistance': 0.00567, 'Reactance': 0.10409, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '101'},
        {'fromNode': '1061.0', 'toNode': '3042.0', 'Resistance': 0.054, 'Reactance': 0.18, 'Conductance': 0, 'Susceptance': 0.028, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '102'},
        {'fromNode': '1066.0', 'toNode': '1065.0', 'Resistance': 0.00508, 'Reactance': 0.12315, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '103'},
        {'fromNode': '1088.0', 'toNode': '1068.0', 'Resistance': 0.02962, 'Reactance': 0.16921, 'Conductance': 0, 'Susceptance': 0.02543, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '104'},
        {'fromNode': '1070.0', 'toNode': '1069.0', 'Resistance': 0.00297, 'Reactance': 0.07, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '105'},
        {'fromNode': '1082.0', 'toNode': '1069.0', 'Resistance': 0.02137, 'Reactance': 0.0952, 'Conductance': 0, 'Susceptance': 0.01396, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '106'},
        {'fromNode': '1072.0', 'toNode': '1071.0', 'Resistance': 0.00529, 'Reactance': 0.08551, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '107'},
        {'fromNode': '1073.0', 'toNode': '1071.0', 'Resistance': 0.00499, 'Reactance': 0.14254, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '108'},
        {'fromNode': '1074.0', 'toNode': '1071.0', 'Resistance': 0.00575, 'Reactance': 0.12453, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '109'},
        {'fromNode': '1075.0', 'toNode': '1071.0', 'Resistance': 0.00373, 'Reactance': 0.09967, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '110'},
        {'fromNode': '1076.0', 'toNode': '1071.0', 'Resistance': 0.00141, 'Reactance': 0.04937, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 230.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '111'},
        {'fromNode': '1077.0', 'toNode': '1071.0', 'Resistance': 0.00641, 'Reactance': 0.12047, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 230.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '112'},
        {'fromNode': '1078.0', 'toNode': '1071.0', 'Resistance': 0.00533, 'Reactance': 0.19087, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 137.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '113'},
        {'fromNode': '1079.0', 'toNode': '1071.0', 'Resistance': 0.00013, 'Reactance': 0.046, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2000.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '114'},
        {'fromNode': '1080.0', 'toNode': '1071.0', 'Resistance': 0.01206, 'Reactance': 0.20208, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 136.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '115'},
        {'fromNode': '1081.0', 'toNode': '1071.0', 'Resistance': 0.00167, 'Reactance': 0.0665, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 350.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '116'},
        {'fromNode': '3048.0', 'toNode': '1079.0', 'Resistance': 0.0013, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 1.02, 'MVA_rating': 2300.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '117'},
        {'fromNode': '3048.0', 'toNode': '1079.0', 'Resistance': 0.0013, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 1.02, 'MVA_rating': 2300.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '118'},
        {'fromNode': '1081.0', 'toNode': '3058.0', 'Resistance': 0.0091, 'Reactance': 0.089, 'Conductance': 0, 'Susceptance': 0.171, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '119'},
        {'fromNode': '1084.0', 'toNode': '3130.0', 'Resistance': 0.02049, 'Reactance': 0.09131, 'Conductance': 0, 'Susceptance': 0.01339, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '120'},
        {'fromNode': '1085.0', 'toNode': '1086.0', 'Resistance': 0.03392, 'Reactance': 0.14838, 'Conductance': 0, 'Susceptance': 0.0226, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '121'},
        {'fromNode': '1090.0', 'toNode': '1085.0', 'Resistance': 0.02018, 'Reactance': 0.13786, 'Conductance': 0, 'Susceptance': 0.02346, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '122'},
        {'fromNode': '2029.0', 'toNode': '1091.0', 'Resistance': 0.0389, 'Reactance': 0.17144, 'Conductance': 0, 'Susceptance': 0.0798, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '123'},
        {'fromNode': '1091.0', 'toNode': '2048.0', 'Resistance': 0.02193, 'Reactance': 0.09667, 'Conductance': 0, 'Susceptance': 0.045, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '124'},
        {'fromNode': '2098.0', 'toNode': '1091.0', 'Resistance': 0.00554, 'Reactance': 0.02443, 'Conductance': 0, 'Susceptance': 0.01137, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '125'},
        {'fromNode': '2022.0', 'toNode': '2001.0', 'Resistance': 0.00839, 'Reactance': 0.03697, 'Conductance': 0, 'Susceptance': 0.01721, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '126'},
        {'fromNode': '2027.0', 'toNode': '2001.0', 'Resistance': 0.00955, 'Reactance': 0.04062, 'Conductance': 0, 'Susceptance': 0.02032, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '127'},
        {'fromNode': '2003.0', 'toNode': '2002.0', 'Resistance': 0.0003, 'Reactance': 0.01986, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 981.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '128'},
        {'fromNode': '2004.0', 'toNode': '2002.0', 'Resistance': 0.00022, 'Reactance': 0.01466, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 635.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '129'},
        {'fromNode': '2005.0', 'toNode': '2002.0', 'Resistance': 0.00033, 'Reactance': 0.01714, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 645.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '130'},
        {'fromNode': '2006.0', 'toNode': '2002.0', 'Resistance': 0.00036, 'Reactance': 0.02679, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 636.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '131'},
        {'fromNode': '2021.0', 'toNode': '2002.0', 'Resistance': 0.00097, 'Reactance': 0.01175, 'Conductance': 0, 'Susceptance': 0.78032, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '132'},
        {'fromNode': '2002.0', 'toNode': '2101.0', 'Resistance': 0.00031, 'Reactance': 0.0037, 'Conductance': 0, 'Susceptance': 0.24573, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '133'},
        {'fromNode': '2038.0', 'toNode': '2003.0', 'Resistance': 0.00353, 'Reactance': 0.02067, 'Conductance': 0, 'Susceptance': 0.01032, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '134'},
        {'fromNode': '2003.0', 'toNode': '2047.0', 'Resistance': 0.0013, 'Reactance': 0.0099, 'Conductance': 0, 'Susceptance': 0.0052, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '135'},
        {'fromNode': '2007.0', 'toNode': '2114.0', 'Resistance': 0.00888, 'Reactance': 0.05207, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '136'},
        {'fromNode': '2007.0', 'toNode': '2122.0', 'Resistance': 0.00861, 'Reactance': 0.03661, 'Conductance': 0, 'Susceptance': 0.01831, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '137'},
        {'fromNode': '2014.0', 'toNode': '2008.0', 'Resistance': 0.02253, 'Reactance': 0.09931, 'Conductance': 0, 'Susceptance': 0.04623, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '138'},
        {'fromNode': '2008.0', 'toNode': '2052.0', 'Resistance': 0.01877, 'Reactance': 0.08271, 'Conductance': 0, 'Susceptance': 0.0385, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '139'},
        {'fromNode': '2009.0', 'toNode': '2049.0', 'Resistance': 0.01224, 'Reactance': 0.07176, 'Conductance': 0, 'Susceptance': 0.03583, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '140'},
        {'fromNode': '2009.0', 'toNode': '2082.0', 'Resistance': 0.00735, 'Reactance': 0.03126, 'Conductance': 0, 'Susceptance': 0.01563, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '141'},
        {'fromNode': '2109.0', 'toNode': '2009.0', 'Resistance': 0.00725, 'Reactance': 0.03084, 'Conductance': 0, 'Susceptance': 0.01542, 'MVA_rating': 250.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '142'},
        {'fromNode': '2061.0', 'toNode': '2010.0', 'Resistance': 0.02439, 'Reactance': 0.10748, 'Conductance': 0, 'Susceptance': 0.05003, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '143'},
        {'fromNode': '2010.0', 'toNode': '2079.0', 'Resistance': 0.02061, 'Reactance': 0.09083, 'Conductance': 0, 'Susceptance': 0.04228, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '144'},
        {'fromNode': '2010.0', 'toNode': '2091.0', 'Resistance': 0.02461, 'Reactance': 0.10467, 'Conductance': 0, 'Susceptance': 0.05235, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '145'},
        {'fromNode': '2012.0', 'toNode': '2011.0', 'Resistance': 0.00044, 'Reactance': 0.03546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 461.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '146'},
        {'fromNode': '2013.0', 'toNode': '2011.0', 'Resistance': 0.00043, 'Reactance': 0.02897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 412.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '147'},
        {'fromNode': '2127.0', 'toNode': '2011.0', 'Resistance': 0.00108, 'Reactance': 0.01307, 'Conductance': 0, 'Susceptance': 0.86827, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '148'},
        {'fromNode': '2012.0', 'toNode': '2041.0', 'Resistance': 0.01245, 'Reactance': 0.05297, 'Conductance': 0, 'Susceptance': 0.02649, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '149'},
        {'fromNode': '2012.0', 'toNode': '2052.0', 'Resistance': 0.02008, 'Reactance': 0.08851, 'Conductance': 0, 'Susceptance': 0.0412, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '150'},
        {'fromNode': '2093.0', 'toNode': '2012.0', 'Resistance': 0.01029, 'Reactance': 0.04536, 'Conductance': 0, 'Susceptance': 0.02111, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '151'},
        {'fromNode': '2012.0', 'toNode': '2123.0', 'Resistance': 0.00995, 'Reactance': 0.0423, 'Conductance': 0, 'Susceptance': 0.02115, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '152'},
        {'fromNode': '2014.0', 'toNode': '2029.0', 'Resistance': 0.01859, 'Reactance': 0.07908, 'Conductance': 0, 'Susceptance': 0.03955, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '153'},
        {'fromNode': '2015.0', 'toNode': '2037.0', 'Resistance': 0.00862, 'Reactance': 0.03799, 'Conductance': 0, 'Susceptance': 0.01769, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '154'},
        {'fromNode': '5014.0', 'toNode': '2015.0', 'Resistance': 0.00961, 'Reactance': 0.04237, 'Conductance': 0, 'Susceptance': 0.01972, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '155'},
        {'fromNode': '2043.0', 'toNode': '2016.0', 'Resistance': 0.00593, 'Reactance': 0.02614, 'Conductance': 0, 'Susceptance': 0.01217, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '156'},
        {'fromNode': '2069.0', 'toNode': '2016.0', 'Resistance': 0.0084, 'Reactance': 0.03705, 'Conductance': 0, 'Susceptance': 0.01725, 'MVA_rating': 280.5, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '157'},
        {'fromNode': '2018.0', 'toNode': '2017.0', 'Resistance': 0.00048, 'Reactance': 0.05045, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 271.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '158'},
        {'fromNode': '2019.0', 'toNode': '2017.0', 'Resistance': 0.0002, 'Reactance': 0.01182, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 454.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '159'},
        {'fromNode': '2017.0', 'toNode': '2096.0', 'Resistance': 0.00253, 'Reactance': 0.03068, 'Conductance': 0, 'Susceptance': 2.03836, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '160'},
        {'fromNode': '2017.0', 'toNode': '2127.0', 'Resistance': 0.00089, 'Reactance': 0.01081, 'Conductance': 0, 'Susceptance': 0.71846, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '161'},
        {'fromNode': '2018.0', 'toNode': '2093.0', 'Resistance': 0.01163, 'Reactance': 0.05125, 'Conductance': 0, 'Susceptance': 0.02386, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '162'},
        {'fromNode': '2045.0', 'toNode': '2020.0', 'Resistance': 0.00097, 'Reactance': 0.00548, 'Conductance': 0, 'Susceptance': 0.00295, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '163'},
        {'fromNode': '2055.0', 'toNode': '2020.0', 'Resistance': 0.00102, 'Reactance': 0.00744, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '164'},
        {'fromNode': '2022.0', 'toNode': '2021.0', 'Resistance': 0.00034, 'Reactance': 0.01888, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 867.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '165'},
        {'fromNode': '2023.0', 'toNode': '2021.0', 'Resistance': 0.00054, 'Reactance': 0.03406, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 319.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '166'},
        {'fromNode': '2024.0', 'toNode': '2021.0', 'Resistance': 0.00036, 'Reactance': 0.02425, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 391.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '167'},
        {'fromNode': '2113.0', 'toNode': '2021.0', 'Resistance': 0.0001, 'Reactance': 0.00124, 'Conductance': 0, 'Susceptance': 0.08207, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '168'},
        {'fromNode': '2022.0', 'toNode': '2040.0', 'Resistance': 0.02207, 'Reactance': 0.09391, 'Conductance': 0, 'Susceptance': 0.04697, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '169'},
        {'fromNode': '2064.0', 'toNode': '2022.0', 'Resistance': 0.00558, 'Reactance': 0.03156, 'Conductance': 0, 'Susceptance': 0.01697, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '170'},
        {'fromNode': '2025.0', 'toNode': '2055.0', 'Resistance': 0.0023, 'Reactance': 0.01016, 'Conductance': 0, 'Susceptance': 0.00473, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '171'},
        {'fromNode': '2067.0', 'toNode': '2025.0', 'Resistance': 0.00317, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.00651, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '172'},
        {'fromNode': '2058.0', 'toNode': '2026.0', 'Resistance': 0.02475, 'Reactance': 0.10526, 'Conductance': 0, 'Susceptance': 0.05264, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '173'},
        {'fromNode': '2128.0', 'toNode': '2026.0', 'Resistance': 0.01207, 'Reactance': 0.05318, 'Conductance': 0, 'Susceptance': 0.02475, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '174'},
        {'fromNode': '2027.0', 'toNode': '2090.0', 'Resistance': 0.01692, 'Reactance': 0.07455, 'Conductance': 0, 'Susceptance': 0.0347, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '175'},
        {'fromNode': '2028.0', 'toNode': '2045.0', 'Resistance': 0.00363, 'Reactance': 0.02129, 'Conductance': 0, 'Susceptance': 0.01063, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '176'},
        {'fromNode': '2028.0', 'toNode': '2109.0', 'Resistance': 0.01026, 'Reactance': 0.0452, 'Conductance': 0, 'Susceptance': 0.02104, 'MVA_rating': 250.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '177'},
        {'fromNode': '2112.0', 'toNode': '2028.0', 'Resistance': 0.00369, 'Reactance': 0.02161, 'Conductance': 0, 'Susceptance': 0.01079, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '178'},
        {'fromNode': '2030.0', 'toNode': '2106.0', 'Resistance': 0.0118, 'Reactance': 0.05199, 'Conductance': 0, 'Susceptance': 0.0242, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '179'},
        {'fromNode': '2122.0', 'toNode': '2030.0', 'Resistance': 0.00737, 'Reactance': 0.0432, 'Conductance': 0, 'Susceptance': 0.02157, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '180'},
        {'fromNode': '2031.0', 'toNode': '2048.0', 'Resistance': 0.02398, 'Reactance': 0.10567, 'Conductance': 0, 'Susceptance': 0.04919, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '181'},
        {'fromNode': '2031.0', 'toNode': '2087.0', 'Resistance': 0.02177, 'Reactance': 0.09259, 'Conductance': 0, 'Susceptance': 0.04631, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '182'},
        {'fromNode': '2032.0', 'toNode': '2114.0', 'Resistance': 0.01097, 'Reactance': 0.04834, 'Conductance': 0, 'Susceptance': 0.0225, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '183'},
        {'fromNode': '2122.0', 'toNode': '2032.0', 'Resistance': 0.01414, 'Reactance': 0.06232, 'Conductance': 0, 'Susceptance': 0.02901, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '184'},
        {'fromNode': '2129.0', 'toNode': '2033.0', 'Resistance': 0.00522, 'Reactance': 0.02298, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '185'},
        {'fromNode': '2033.0', 'toNode': '5288.0', 'Resistance': 0.00925, 'Reactance': 0.03935, 'Conductance': 0, 'Susceptance': 0.01968, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '186'},
        {'fromNode': '2034.0', 'toNode': '2043.0', 'Resistance': 0.00439, 'Reactance': 0.01867, 'Conductance': 0, 'Susceptance': 0.00934, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '187'},
        {'fromNode': '2046.0', 'toNode': '2034.0', 'Resistance': 0.00468, 'Reactance': 0.01993, 'Conductance': 0, 'Susceptance': 0.00997, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '188'},
        {'fromNode': '2035.0', 'toNode': '2066.0', 'Resistance': 0.01411, 'Reactance': 0.08272, 'Conductance': 0, 'Susceptance': 0.04131, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '189'},
        {'fromNode': '2035.0', 'toNode': '2091.0', 'Resistance': 0.01863, 'Reactance': 0.10528, 'Conductance': 0, 'Susceptance': 0.05663, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '190'},
        {'fromNode': '2036.0', 'toNode': '2055.0', 'Resistance': 0.00216, 'Reactance': 0.0092, 'Conductance': 0, 'Susceptance': 0.0046, 'MVA_rating': 300.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '191'},
        {'fromNode': '2036.0', 'toNode': '2082.0', 'Resistance': 0.00236, 'Reactance': 0.01003, 'Conductance': 0, 'Susceptance': 0.00502, 'MVA_rating': 250.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '192'},
        {'fromNode': '2037.0', 'toNode': '2065.0', 'Resistance': 0.00415, 'Reactance': 0.02434, 'Conductance': 0, 'Susceptance': 0.01216, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '193'},
        {'fromNode': '2110.0', 'toNode': '2037.0', 'Resistance': 0.00307, 'Reactance': 0.01798, 'Conductance': 0, 'Susceptance': 0.00898, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '194'},
        {'fromNode': '2081.0', 'toNode': '2038.0', 'Resistance': 0.00639, 'Reactance': 0.03748, 'Conductance': 0, 'Susceptance': 0.01871, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '195'},
        {'fromNode': '2039.0', 'toNode': '2086.0', 'Resistance': 0.00845, 'Reactance': 0.03725, 'Conductance': 0, 'Susceptance': 0.01734, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '196'},
        {'fromNode': '5285.0', 'toNode': '2039.0', 'Resistance': 0.00569, 'Reactance': 0.03334, 'Conductance': 0, 'Susceptance': 0.01665, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '197'},
        {'fromNode': '2039.0', 'toNode': '5455.0', 'Resistance': 0.00371, 'Reactance': 0.02173, 'Conductance': 0, 'Susceptance': 0.01085, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '198'},
        {'fromNode': '2040.0', 'toNode': '2114.0', 'Resistance': 0.02335, 'Reactance': 0.09932, 'Conductance': 0, 'Susceptance': 0.04967, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '199'},
        {'fromNode': '2121.0', 'toNode': '2040.0', 'Resistance': 0.00894, 'Reactance': 0.03938, 'Conductance': 0, 'Susceptance': 0.01833, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '200'},
        {'fromNode': '2042.0', 'toNode': '2041.0', 'Resistance': 0.00216, 'Reactance': 0.07846, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 95.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '201'},
        {'fromNode': '2041.0', 'toNode': '2058.0', 'Resistance': 0.0136, 'Reactance': 0.05786, 'Conductance': 0, 'Susceptance': 0.02893, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '202'},
        {'fromNode': '2065.0', 'toNode': '2044.0', 'Resistance': 0.01021, 'Reactance': 0.04497, 'Conductance': 0, 'Susceptance': 0.02093, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '203'},
        {'fromNode': '2044.0', 'toNode': '2102.0', 'Resistance': 0.01465, 'Reactance': 0.06453, 'Conductance': 0, 'Susceptance': 0.03004, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '204'},
        {'fromNode': '2063.0', 'toNode': '2046.0', 'Resistance': 0.00363, 'Reactance': 0.02051, 'Conductance': 0, 'Susceptance': 0.01103, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '205'},
        {'fromNode': '2092.0', 'toNode': '2047.0', 'Resistance': 0.00376, 'Reactance': 0.02203, 'Conductance': 0, 'Susceptance': 0.011, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '206'},
        {'fromNode': '2079.0', 'toNode': '2048.0', 'Resistance': 0.02944, 'Reactance': 0.12973, 'Conductance': 0, 'Susceptance': 0.06039, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '207'},
        {'fromNode': '2049.0', 'toNode': '2061.0', 'Resistance': 0.00912, 'Reactance': 0.05349, 'Conductance': 0, 'Susceptance': 0.02671, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '208'},
        {'fromNode': '2090.0', 'toNode': '2050.0', 'Resistance': 0.02377, 'Reactance': 0.10475, 'Conductance': 0, 'Susceptance': 0.04876, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '209'},
        {'fromNode': '2050.0', 'toNode': '5268.0', 'Resistance': 0.01273, 'Reactance': 0.05412, 'Conductance': 0, 'Susceptance': 0.02707, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '210'},
        {'fromNode': '2080.0', 'toNode': '2051.0', 'Resistance': 0.00691, 'Reactance': 0.03904, 'Conductance': 0, 'Susceptance': 0.021, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '211'},
        {'fromNode': '2051.0', 'toNode': '2112.0', 'Resistance': 0.00873, 'Reactance': 0.04934, 'Conductance': 0, 'Susceptance': 0.02654, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '212'},
        {'fromNode': '5285.0', 'toNode': '2053.0', 'Resistance': 0.00757, 'Reactance': 0.0428, 'Conductance': 0, 'Susceptance': 0.02302, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '213'},
        {'fromNode': '2053.0', 'toNode': '5362.0', 'Resistance': 0.01118, 'Reactance': 0.06552, 'Conductance': 0, 'Susceptance': 0.03272, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '214'},
        {'fromNode': '2055.0', 'toNode': '2054.0', 'Resistance': 0.00016, 'Reactance': 0.00742, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1095.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '215'},
        {'fromNode': '2055.0', 'toNode': '2054.0', 'Resistance': 0.00016, 'Reactance': 0.00742, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1095.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '216'},
        {'fromNode': '2056.0', 'toNode': '2054.0', 'Resistance': 0.00069, 'Reactance': 0.06323, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 227.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '217'},
        {'fromNode': '2057.0', 'toNode': '2054.0', 'Resistance': 0.00025, 'Reactance': 0.01321, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1001.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '218'},
        {'fromNode': '2127.0', 'toNode': '2054.0', 'Resistance': 0.033, 'Reactance': 0.043, 'Conductance': 0, 'Susceptance': 2.3, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '219'},
        {'fromNode': '2054.0', 'toNode': '5236.0', 'Resistance': 0.0008, 'Reactance': 0.00971, 'Conductance': 0, 'Susceptance': 0.64507, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '220'},
        {'fromNode': '2059.0', 'toNode': '2058.0', 'Resistance': 0.00537, 'Reactance': 0.10085, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 99.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '221'},
        {'fromNode': '2123.0', 'toNode': '2058.0', 'Resistance': 0.00524, 'Reactance': 0.0231, 'Conductance': 0, 'Susceptance': 0.01075, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '222'},
        {'fromNode': '2060.0', 'toNode': '2064.0', 'Resistance': 0.00608, 'Reactance': 0.03562, 'Conductance': 0, 'Susceptance': 0.01779, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '223'},
        {'fromNode': '5136.0', 'toNode': '2060.0', 'Resistance': 0.00421, 'Reactance': 0.02468, 'Conductance': 0, 'Susceptance': 0.01232, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '224'},
        {'fromNode': '2062.0', 'toNode': '2061.0', 'Resistance': 0.0016, 'Reactance': 0.04103, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 223.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '225'},
        {'fromNode': '2092.0', 'toNode': '2063.0', 'Resistance': 0.00409, 'Reactance': 0.02397, 'Conductance': 0, 'Susceptance': 0.01197, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '226'},
        {'fromNode': '2126.0', 'toNode': '2065.0', 'Resistance': 0.00712, 'Reactance': 0.04025, 'Conductance': 0, 'Susceptance': 0.02165, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '227'},
        {'fromNode': '2073.0', 'toNode': '2066.0', 'Resistance': 0.01087, 'Reactance': 0.06371, 'Conductance': 0, 'Susceptance': 0.03181, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '228'},
        {'fromNode': '2068.0', 'toNode': '2067.0', 'Resistance': 0.00995, 'Reactance': 0.05832, 'Conductance': 0, 'Susceptance': 0.02912, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '229'},
        {'fromNode': '2068.0', 'toNode': '2088.0', 'Resistance': 0.00135, 'Reactance': 0.00792, 'Conductance': 0, 'Susceptance': 0.00396, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '230'},
        {'fromNode': '2069.0', 'toNode': '2072.0', 'Resistance': 0.00557, 'Reactance': 0.03267, 'Conductance': 0, 'Susceptance': 0.01631, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '231'},
        {'fromNode': '2069.0', 'toNode': '5074.0', 'Resistance': 0.0102, 'Reactance': 0.04493, 'Conductance': 0, 'Susceptance': 0.02092, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '232'},
        {'fromNode': '2071.0', 'toNode': '2070.0', 'Resistance': 0.00129, 'Reactance': 0.05708, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 249.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '233'},
        {'fromNode': '2088.0', 'toNode': '2070.0', 'Resistance': 0.00235, 'Reactance': 0.01376, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '234'},
        {'fromNode': '2070.0', 'toNode': '5237.0', 'Resistance': 0.00727, 'Reactance': 0.03095, 'Conductance': 0, 'Susceptance': 0.01548, 'MVA_rating': 257.25, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '235'},
        {'fromNode': '2072.0', 'toNode': '2102.0', 'Resistance': 0.0084, 'Reactance': 0.04925, 'Conductance': 0, 'Susceptance': 0.0246, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '236'},
        {'fromNode': '2074.0', 'toNode': '2073.0', 'Resistance': 0.00225, 'Reactance': 0.04467, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 144.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '237'},
        {'fromNode': '2076.0', 'toNode': '2073.0', 'Resistance': 0.00347, 'Reactance': 0.07389, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 64.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '238'},
        {'fromNode': '2087.0', 'toNode': '2073.0', 'Resistance': 0.01908, 'Reactance': 0.08409, 'Conductance': 0, 'Susceptance': 0.03914, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '239'},
        {'fromNode': '2073.0', 'toNode': '3079.0', 'Resistance': 0.01377, 'Reactance': 0.08073, 'Conductance': 0, 'Susceptance': 0.04031, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '240'},
        {'fromNode': '2075.0', 'toNode': '2074.0', 'Resistance': 0.01303, 'Reactance': 0.29308, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 19.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '241'},
        {'fromNode': '3115.0', 'toNode': '2074.0', 'Resistance': 0.0052, 'Reactance': 0.02972, 'Conductance': 0, 'Susceptance': 0.00447, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '242'},
        {'fromNode': '2078.0', 'toNode': '2077.0', 'Resistance': 0.002, 'Reactance': 0.07139, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 189.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '243'},
        {'fromNode': '2077.0', 'toNode': '2110.0', 'Resistance': 0.00218, 'Reactance': 0.01278, 'Conductance': 0, 'Susceptance': 0.00638, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '244'},
        {'fromNode': '2077.0', 'toNode': '2133.0', 'Resistance': 0.00307, 'Reactance': 0.01798, 'Conductance': 0, 'Susceptance': 0.00898, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '245'},
        {'fromNode': '2086.0', 'toNode': '2080.0', 'Resistance': 0.01338, 'Reactance': 0.05896, 'Conductance': 0, 'Susceptance': 0.02745, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '246'},
        {'fromNode': '2133.0', 'toNode': '2080.0', 'Resistance': 0.01236, 'Reactance': 0.07244, 'Conductance': 0, 'Susceptance': 0.03618, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '247'},
        {'fromNode': '2081.0', 'toNode': '2129.0', 'Resistance': 0.01257, 'Reactance': 0.05349, 'Conductance': 0, 'Susceptance': 0.02675, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '248'},
        {'fromNode': '2083.0', 'toNode': '2082.0', 'Resistance': 0.00401, 'Reactance': 0.14309, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 91.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '249'},
        {'fromNode': '2084.0', 'toNode': '2082.0', 'Resistance': 0.00414, 'Reactance': 0.18022, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 82.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '250'},
        {'fromNode': '2085.0', 'toNode': '2082.0', 'Resistance': 0.00156, 'Reactance': 0.07238, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 98.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '251'},
        {'fromNode': '2089.0', 'toNode': '2088.0', 'Resistance': 0.00259, 'Reactance': 0.12398, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 84.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '252'},
        {'fromNode': '2090.0', 'toNode': '2121.0', 'Resistance': 0.01763, 'Reactance': 0.075, 'Conductance': 0, 'Susceptance': 0.03751, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '253'},
        {'fromNode': '2094.0', 'toNode': '2093.0', 'Resistance': 0.0017, 'Reactance': 0.10337, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 88.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '254'},
        {'fromNode': '2095.0', 'toNode': '2093.0', 'Resistance': 0.00149, 'Reactance': 0.05101, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 164.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '255'},
        {'fromNode': '2097.0', 'toNode': '2096.0', 'Resistance': 0.00012, 'Reactance': 0.00729, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 836.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '256'},
        {'fromNode': '2099.0', 'toNode': '2096.0', 'Resistance': 0.0003, 'Reactance': 0.03477, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 237.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '257'},
        {'fromNode': '2100.0', 'toNode': '2096.0', 'Resistance': 0.00075, 'Reactance': 0.04235, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 386.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '258'},
        {'fromNode': '2098.0', 'toNode': '2097.0', 'Resistance': 0.00183, 'Reactance': 0.13303, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 146.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '259'},
        {'fromNode': '2097.0', 'toNode': '3096.0', 'Resistance': 0.00546, 'Reactance': 0.05679, 'Conductance': 0, 'Susceptance': 0.19351, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '260'},
        {'fromNode': '2102.0', 'toNode': '2101.0', 'Resistance': 0.00013, 'Reactance': 0.00916, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1072.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '261'},
        {'fromNode': '2103.0', 'toNode': '2101.0', 'Resistance': 0.00019, 'Reactance': 0.01252, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 542.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '262'},
        {'fromNode': '2104.0', 'toNode': '2101.0', 'Resistance': 0.00022, 'Reactance': 0.0164, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 542.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '263'},
        {'fromNode': '2105.0', 'toNode': '2101.0', 'Resistance': 0.00055, 'Reactance': 0.02588, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 544.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '264'},
        {'fromNode': '2101.0', 'toNode': '5083.0', 'Resistance': 0.00059, 'Reactance': 0.00721, 'Conductance': 0, 'Susceptance': 0.47876, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '265'},
        {'fromNode': '2102.0', 'toNode': '2107.0', 'Resistance': 0.00156, 'Reactance': 0.01139, 'Conductance': 0, 'Susceptance': 0.00646, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '266'},
        {'fromNode': '2124.0', 'toNode': '2106.0', 'Resistance': 0.00485, 'Reactance': 0.02842, 'Conductance': 0, 'Susceptance': 0.01419, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '267'},
        {'fromNode': '2106.0', 'toNode': '2129.0', 'Resistance': 0.00424, 'Reactance': 0.01803, 'Conductance': 0, 'Susceptance': 0.00902, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '268'},
        {'fromNode': '2107.0', 'toNode': '2125.0', 'Resistance': 0.0109, 'Reactance': 0.06393, 'Conductance': 0, 'Susceptance': 0.03192, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '269'},
        {'fromNode': '2130.0', 'toNode': '2108.0', 'Resistance': 0.00945, 'Reactance': 0.04022, 'Conductance': 0, 'Susceptance': 0.02012, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '270'},
        {'fromNode': '5472.0', 'toNode': '2108.0', 'Resistance': 0.01619, 'Reactance': 0.07138, 'Conductance': 0, 'Susceptance': 0.03323, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '271'},
        {'fromNode': '2111.0', 'toNode': '5187.0', 'Resistance': 0.01319, 'Reactance': 0.05811, 'Conductance': 0, 'Susceptance': 0.02705, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '272'},
        {'fromNode': '2111.0', 'toNode': '5472.0', 'Resistance': 0.02196, 'Reactance': 0.09676, 'Conductance': 0, 'Susceptance': 0.04504, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '273'},
        {'fromNode': '2114.0', 'toNode': '2113.0', 'Resistance': 0.00013, 'Reactance': 0.00965, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 924.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '274'},
        {'fromNode': '2115.0', 'toNode': '2113.0', 'Resistance': 0.00077, 'Reactance': 0.03843, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 410.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '275'},
        {'fromNode': '2116.0', 'toNode': '2113.0', 'Resistance': 0.00027, 'Reactance': 0.02003, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 402.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '276'},
        {'fromNode': '2117.0', 'toNode': '2113.0', 'Resistance': 0.00043, 'Reactance': 0.0445, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 404.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '277'},
        {'fromNode': '2118.0', 'toNode': '2113.0', 'Resistance': 0.00064, 'Reactance': 0.04928, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 391.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '278'},
        {'fromNode': '2119.0', 'toNode': '2113.0', 'Resistance': 0.00034, 'Reactance': 0.02438, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 389.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '279'},
        {'fromNode': '2120.0', 'toNode': '2113.0', 'Resistance': 0.0007, 'Reactance': 0.04736, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 392.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '280'},
        {'fromNode': '5049.0', 'toNode': '2113.0', 'Resistance': 0.00053, 'Reactance': 0.01087, 'Conductance': 0, 'Susceptance': 0.75821, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '281'},
        {'fromNode': '2113.0', 'toNode': '8126.0', 'Resistance': 0.00058, 'Reactance': 0.01198, 'Conductance': 0, 'Susceptance': 0.83533, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '282'},
        {'fromNode': '5450.0', 'toNode': '2124.0', 'Resistance': 0.00519, 'Reactance': 0.02933, 'Conductance': 0, 'Susceptance': 0.01578, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '283'},
        {'fromNode': '2125.0', 'toNode': '5078.0', 'Resistance': 0.00981, 'Reactance': 0.04323, 'Conductance': 0, 'Susceptance': 0.02012, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '284'},
        {'fromNode': '2125.0', 'toNode': '5215.0', 'Resistance': 0.00341, 'Reactance': 0.01999, 'Conductance': 0, 'Susceptance': 0.00998, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '285'},
        {'fromNode': '2126.0', 'toNode': '5220.0', 'Resistance': 0.0038, 'Reactance': 0.02226, 'Conductance': 0, 'Susceptance': 0.01112, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '286'},
        {'fromNode': '2128.0', 'toNode': '2127.0', 'Resistance': 0.00066, 'Reactance': 0.06242, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 308.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '287'},
        {'fromNode': '2127.0', 'toNode': '5164.0', 'Resistance': 0.01, 'Reactance': 0.1, 'Conductance': 0, 'Susceptance': 3.0, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '288'},
        {'fromNode': '2129.0', 'toNode': '5141.0', 'Resistance': 0.00446, 'Reactance': 0.02617, 'Conductance': 0, 'Susceptance': 0.01307, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '289'},
        {'fromNode': '2131.0', 'toNode': '2130.0', 'Resistance': 0.00178, 'Reactance': 0.07699, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 214.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '290'},
        {'fromNode': '2132.0', 'toNode': '2131.0', 'Resistance': 0.00583, 'Reactance': 0.25242, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 24.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '291'},
        {'fromNode': '3021.0', 'toNode': '2131.0', 'Resistance': 0.02143, 'Reactance': 0.09374, 'Conductance': 0, 'Susceptance': 0.01428, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '292'},
        {'fromNode': '3002.0', 'toNode': '3001.0', 'Resistance': 0.00103, 'Reactance': 0.06934, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 171.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '293'},
        {'fromNode': '3002.0', 'toNode': '3001.0', 'Resistance': 0.00103, 'Reactance': 0.06934, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 171.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '294'},
        {'fromNode': '3001.0', 'toNode': '6177.0', 'Resistance': 0.00653, 'Reactance': 0.06378, 'Conductance': 0, 'Susceptance': 0.12272, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '295'},
        {'fromNode': '6285.0', 'toNode': '3001.0', 'Resistance': 0.00639, 'Reactance': 0.04566, 'Conductance': 0, 'Susceptance': 0.08324, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '296'},
        {'fromNode': '3002.0', 'toNode': '3107.0', 'Resistance': 0.01283, 'Reactance': 0.07327, 'Conductance': 0, 'Susceptance': 0.01101, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '297'},
        {'fromNode': '3004.0', 'toNode': '3003.0', 'Resistance': 0.00324, 'Reactance': 0.10104, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 51.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '298'},
        {'fromNode': '3003.0', 'toNode': '3103.0', 'Resistance': 0.01071, 'Reactance': 0.04441, 'Conductance': 0, 'Susceptance': 0.00754, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '299'},
        {'fromNode': '3141.0', 'toNode': '3003.0', 'Resistance': 0.00248, 'Reactance': 0.01085, 'Conductance': 0, 'Susceptance': 0.00165, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '300'},
        {'fromNode': '3005.0', 'toNode': '3052.0', 'Resistance': 0.07286, 'Reactance': 0.17811, 'Conductance': 0, 'Susceptance': 0.02332, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '301'},
        {'fromNode': '3126.0', 'toNode': '3005.0', 'Resistance': 0.0221, 'Reactance': 0.13272, 'Conductance': 0, 'Susceptance': 0.01801, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '302'},
        {'fromNode': '3006.0', 'toNode': '3049.0', 'Resistance': 0.01784, 'Reactance': 0.0795, 'Conductance': 0, 'Susceptance': 0.01166, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '303'},
        {'fromNode': '3145.0', 'toNode': '3006.0', 'Resistance': 0.01721, 'Reactance': 0.07139, 'Conductance': 0, 'Susceptance': 0.01211, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '304'},
        {'fromNode': '3007.0', 'toNode': '3013.0', 'Resistance': 0.0282, 'Reactance': 0.12334, 'Conductance': 0, 'Susceptance': 0.01878, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '305'},
        {'fromNode': '3007.0', 'toNode': '3137.0', 'Resistance': 0.00869, 'Reactance': 0.03873, 'Conductance': 0, 'Susceptance': 0.00568, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '306'},
        {'fromNode': '3007.0', 'toNode': '3145.0', 'Resistance': 0.01571, 'Reactance': 0.08971, 'Conductance': 0, 'Susceptance': 0.01348, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '307'},
        {'fromNode': '3008.0', 'toNode': '3036.0', 'Resistance': 0.03575, 'Reactance': 0.14831, 'Conductance': 0, 'Susceptance': 0.02517, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '308'},
        {'fromNode': '3140.0', 'toNode': '3008.0', 'Resistance': 0.0278, 'Reactance': 0.12389, 'Conductance': 0, 'Susceptance': 0.01817, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '309'},
        {'fromNode': '3143.0', 'toNode': '3008.0', 'Resistance': 0.04389, 'Reactance': 0.1073, 'Conductance': 0, 'Susceptance': 0.01405, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '310'},
        {'fromNode': '3009.0', 'toNode': '3024.0', 'Resistance': 0.0285, 'Reactance': 0.17117, 'Conductance': 0, 'Susceptance': 0.02323, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '311'},
        {'fromNode': '3146.0', 'toNode': '3009.0', 'Resistance': 0.03539, 'Reactance': 0.14683, 'Conductance': 0, 'Susceptance': 0.02492, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '312'},
        {'fromNode': '3010.0', 'toNode': '3076.0', 'Resistance': 0.02704, 'Reactance': 0.16239, 'Conductance': 0, 'Susceptance': 0.02204, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '313'},
        {'fromNode': '3093.0', 'toNode': '3010.0', 'Resistance': 0.02325, 'Reactance': 0.1396, 'Conductance': 0, 'Susceptance': 0.01894, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '314'},
        {'fromNode': '3011.0', 'toNode': '3012.0', 'Resistance': 0.01814, 'Reactance': 0.1036, 'Conductance': 0, 'Susceptance': 0.01557, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '315'},
        {'fromNode': '3011.0', 'toNode': '3124.0', 'Resistance': 0.02409, 'Reactance': 0.1376, 'Conductance': 0, 'Susceptance': 0.02068, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '316'},
        {'fromNode': '3012.0', 'toNode': '6226.0', 'Resistance': 0.01815, 'Reactance': 0.10369, 'Conductance': 0, 'Susceptance': 0.01558, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '317'},
        {'fromNode': '3013.0', 'toNode': '3077.0', 'Resistance': 0.00833, 'Reactance': 0.05, 'Conductance': 0, 'Susceptance': 0.00679, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '318'},
        {'fromNode': '3014.0', 'toNode': '3124.0', 'Resistance': 0.04522, 'Reactance': 0.19779, 'Conductance': 0, 'Susceptance': 0.03012, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '319'},
        {'fromNode': '6212.0', 'toNode': '3014.0', 'Resistance': 0.01229, 'Reactance': 0.05481, 'Conductance': 0, 'Susceptance': 0.00804, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '320'},
        {'fromNode': '3015.0', 'toNode': '3121.0', 'Resistance': 0.00527, 'Reactance': 0.03162, 'Conductance': 0, 'Susceptance': 0.00429, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '321'},
        {'fromNode': '3015.0', 'toNode': '3127.0', 'Resistance': 0.01148, 'Reactance': 0.0656, 'Conductance': 0, 'Susceptance': 0.00986, 'MVA_rating': 109.76, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '322'},
        {'fromNode': '3015.0', 'toNode': '3128.0', 'Resistance': 0.00752, 'Reactance': 0.04513, 'Conductance': 0, 'Susceptance': 0.00612, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '323'},
        {'fromNode': '3017.0', 'toNode': '3016.0', 'Resistance': 0.00189, 'Reactance': 0.07277, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '324'},
        {'fromNode': '3016.0', 'toNode': '3083.0', 'Resistance': 0.00919, 'Reactance': 0.05252, 'Conductance': 0, 'Susceptance': 0.00789, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '325'},
        {'fromNode': '3016.0', 'toNode': '3103.0', 'Resistance': 0.00883, 'Reactance': 0.03662, 'Conductance': 0, 'Susceptance': 0.00621, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '326'},
        {'fromNode': '3016.0', 'toNode': '3131.0', 'Resistance': 0.03765, 'Reactance': 0.09203, 'Conductance': 0, 'Susceptance': 0.01205, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '327'},
        {'fromNode': '3139.0', 'toNode': '3016.0', 'Resistance': 0.02927, 'Reactance': 0.12143, 'Conductance': 0, 'Susceptance': 0.02061, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '328'},
        {'fromNode': '3019.0', 'toNode': '3018.0', 'Resistance': 0.00052, 'Reactance': 0.03897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 142.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '329'},
        {'fromNode': '3019.0', 'toNode': '3018.0', 'Resistance': 0.00053, 'Reactance': 0.03897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 142.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '330'},
        {'fromNode': '3020.0', 'toNode': '3018.0', 'Resistance': 0.00133, 'Reactance': 0.06779, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '331'},
        {'fromNode': '3018.0', 'toNode': '3050.0', 'Resistance': 0.01395, 'Reactance': 0.13638, 'Conductance': 0, 'Susceptance': 0.2624, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '332'},
        {'fromNode': '3018.0', 'toNode': '3058.0', 'Resistance': 0.0091, 'Reactance': 0.0889, 'Conductance': 0, 'Susceptance': 0.17104, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '333'},
        {'fromNode': '3024.0', 'toNode': '3019.0', 'Resistance': 0.01378, 'Reactance': 0.07869, 'Conductance': 0, 'Susceptance': 0.01182, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '334'},
        {'fromNode': '3114.0', 'toNode': '3019.0', 'Resistance': 0.01054, 'Reactance': 0.07345, 'Conductance': 0, 'Susceptance': 0.01201, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '335'},
        {'fromNode': '3114.0', 'toNode': '3019.0', 'Resistance': 0.01054, 'Reactance': 0.07345, 'Conductance': 0, 'Susceptance': 0.01201, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '336'},
        {'fromNode': '3049.0', 'toNode': '3021.0', 'Resistance': 0.01086, 'Reactance': 0.07565, 'Conductance': 0, 'Susceptance': 0.01237, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '337'},
        {'fromNode': '5397.0', 'toNode': '3021.0', 'Resistance': 0.02082, 'Reactance': 0.09277, 'Conductance': 0, 'Susceptance': 0.0136, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '338'},
        {'fromNode': '3033.0', 'toNode': '3022.0', 'Resistance': 0.06611, 'Reactance': 0.16158, 'Conductance': 0, 'Susceptance': 0.02116, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '339'},
        {'fromNode': '3085.0', 'toNode': '3022.0', 'Resistance': 0.01233, 'Reactance': 0.07039, 'Conductance': 0, 'Susceptance': 0.01058, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '340'},
        {'fromNode': '3049.0', 'toNode': '3023.0', 'Resistance': 0.01656, 'Reactance': 0.0738, 'Conductance': 0, 'Susceptance': 0.01082, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '341'},
        {'fromNode': '3023.0', 'toNode': '3067.0', 'Resistance': 0.09726, 'Reactance': 0.23774, 'Conductance': 0, 'Susceptance': 0.03113, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '342'},
        {'fromNode': '3023.0', 'toNode': '3115.0', 'Resistance': 0.03509, 'Reactance': 0.21073, 'Conductance': 0, 'Susceptance': 0.0286, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '343'},
        {'fromNode': '3035.0', 'toNode': '3025.0', 'Resistance': 0.05582, 'Reactance': 0.13643, 'Conductance': 0, 'Susceptance': 0.01786, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '344'},
        {'fromNode': '3025.0', 'toNode': '3036.0', 'Resistance': 0.0401, 'Reactance': 0.16637, 'Conductance': 0, 'Susceptance': 0.02823, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '345'},
        {'fromNode': '3025.0', 'toNode': '3101.0', 'Resistance': 0.02809, 'Reactance': 0.12288, 'Conductance': 0, 'Susceptance': 0.01871, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '346'},
        {'fromNode': '3027.0', 'toNode': '3026.0', 'Resistance': 0.00258, 'Reactance': 0.11301, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '347'},
        {'fromNode': '3028.0', 'toNode': '3026.0', 'Resistance': 0.00048, 'Reactance': 0.03228, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 270.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '348'},
        {'fromNode': '3026.0', 'toNode': '5395.0', 'Resistance': 0.00227, 'Reactance': 0.01625, 'Conductance': 0, 'Susceptance': 0.02963, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '349'},
        {'fromNode': '3027.0', 'toNode': '3102.0', 'Resistance': 0.02467, 'Reactance': 0.05926, 'Conductance': 0, 'Susceptance': 0.00804, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '350'},
        {'fromNode': '3147.0', 'toNode': '3029.0', 'Resistance': 0.01187, 'Reactance': 0.06781, 'Conductance': 0, 'Susceptance': 0.01019, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '351'},
        {'fromNode': '3029.0', 'toNode': '5241.0', 'Resistance': 0.01232, 'Reactance': 0.05113, 'Conductance': 0, 'Susceptance': 0.00868, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '352'},
        {'fromNode': '3030.0', 'toNode': '3072.0', 'Resistance': 0.00885, 'Reactance': 0.05057, 'Conductance': 0, 'Susceptance': 0.0076, 'MVA_rating': 190.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '353'},
        {'fromNode': '3114.0', 'toNode': '3030.0', 'Resistance': 0.02646, 'Reactance': 0.10977, 'Conductance': 0, 'Susceptance': 0.01863, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '354'},
        {'fromNode': '3054.0', 'toNode': '3031.0', 'Resistance': 0.03288, 'Reactance': 0.08037, 'Conductance': 0, 'Susceptance': 0.01052, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '355'},
        {'fromNode': '3031.0', 'toNode': '3070.0', 'Resistance': 0.03731, 'Reactance': 0.09121, 'Conductance': 0, 'Susceptance': 0.01194, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '356'},
        {'fromNode': '3031.0', 'toNode': '3140.0', 'Resistance': 0.02073, 'Reactance': 0.09067, 'Conductance': 0, 'Susceptance': 0.01381, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '357'},
        {'fromNode': '3032.0', 'toNode': '3094.0', 'Resistance': 0.00583, 'Reactance': 0.04065, 'Conductance': 0, 'Susceptance': 0.00664, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '358'},
        {'fromNode': '3099.0', 'toNode': '3032.0', 'Resistance': 0.00703, 'Reactance': 0.03076, 'Conductance': 0, 'Susceptance': 0.00468, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '359'},
        {'fromNode': '3034.0', 'toNode': '3033.0', 'Resistance': 0.00593, 'Reactance': 0.17201, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 125.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '360'},
        {'fromNode': '3136.0', 'toNode': '3033.0', 'Resistance': 0.00696, 'Reactance': 0.01672, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '361'},
        {'fromNode': '3035.0', 'toNode': '3066.0', 'Resistance': 0.08317, 'Reactance': 0.2033, 'Conductance': 0, 'Susceptance': 0.02662, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '362'},
        {'fromNode': '3036.0', 'toNode': '3066.0', 'Resistance': 0.03434, 'Reactance': 0.19612, 'Conductance': 0, 'Susceptance': 0.02947, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '363'},
        {'fromNode': '3038.0', 'toNode': '3037.0', 'Resistance': 0.00411, 'Reactance': 0.1621, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 82.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '364'},
        {'fromNode': '3039.0', 'toNode': '3037.0', 'Resistance': 0.00051, 'Reactance': 0.03079, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 280.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '365'},
        {'fromNode': '3078.0', 'toNode': '3037.0', 'Resistance': 0.00125, 'Reactance': 0.01223, 'Conductance': 0, 'Susceptance': 0.02354, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '366'},
        {'fromNode': '3037.0', 'toNode': '3088.0', 'Resistance': 0.00895, 'Reactance': 0.08746, 'Conductance': 0, 'Susceptance': 0.16828, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '367'},
        {'fromNode': '3038.0', 'toNode': '3080.0', 'Resistance': 0.00768, 'Reactance': 0.04384, 'Conductance': 0, 'Susceptance': 0.00659, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '368'},
        {'fromNode': '3085.0', 'toNode': '3038.0', 'Resistance': 0.00471, 'Reactance': 0.02825, 'Conductance': 0, 'Susceptance': 0.00383, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '369'},
        {'fromNode': '3072.0', 'toNode': '3040.0', 'Resistance': 0.02206, 'Reactance': 0.09153, 'Conductance': 0, 'Susceptance': 0.01553, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '370'},
        {'fromNode': '3139.0', 'toNode': '3040.0', 'Resistance': 0.01922, 'Reactance': 0.08563, 'Conductance': 0, 'Susceptance': 0.01256, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '371'},
        {'fromNode': '3042.0', 'toNode': '3041.0', 'Resistance': 0.00322, 'Reactance': 0.18545, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '372'},
        {'fromNode': '3043.0', 'toNode': '3041.0', 'Resistance': 0.00205, 'Reactance': 0.14897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 57.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '373'},
        {'fromNode': '3044.0', 'toNode': '3041.0', 'Resistance': 0.00154, 'Reactance': 0.10487, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 112.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '374'},
        {'fromNode': '3045.0', 'toNode': '3041.0', 'Resistance': 0.00096, 'Reactance': 0.05895, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 550.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '375'},
        {'fromNode': '3041.0', 'toNode': '3046.0', 'Resistance': 0.0031, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.05832, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '376'},
        {'fromNode': '3041.0', 'toNode': '3058.0', 'Resistance': 0.00347, 'Reactance': 0.02481, 'Conductance': 0, 'Susceptance': 0.04523, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '377'},
        {'fromNode': '3042.0', 'toNode': '3064.0', 'Resistance': 0.02146, 'Reactance': 0.09562, 'Conductance': 0, 'Susceptance': 0.01402, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '378'},
        {'fromNode': '3111.0', 'toNode': '3042.0', 'Resistance': 0.01081, 'Reactance': 0.04816, 'Conductance': 0, 'Susceptance': 0.00706, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '379'},
        {'fromNode': '3047.0', 'toNode': '3046.0', 'Resistance': 0.00338, 'Reactance': 0.14142, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 290.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '380'},
        {'fromNode': '3048.0', 'toNode': '3046.0', 'Resistance': 0.00027, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '381'},
        {'fromNode': '3048.0', 'toNode': '3046.0', 'Resistance': 0.00027, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '382'},
        {'fromNode': '3046.0', 'toNode': '3078.0', 'Resistance': 0.00316, 'Reactance': 0.0309, 'Conductance': 0, 'Susceptance': 0.05946, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '383'},
        {'fromNode': '3082.0', 'toNode': '3046.0', 'Resistance': 0.00313, 'Reactance': 0.03062, 'Conductance': 0, 'Susceptance': 0.05891, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '384'},
        {'fromNode': '3047.0', 'toNode': '3064.0', 'Resistance': 0.01369, 'Reactance': 0.045, 'Conductance': 0, 'Susceptance': 0.00446, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '385'},
        {'fromNode': '3129.0', 'toNode': '3047.0', 'Resistance': 0.02116, 'Reactance': 0.05173, 'Conductance': 0, 'Susceptance': 0.00677, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '386'},
        {'fromNode': '3048.0', 'toNode': '5045.0', 'Resistance': 0.0011, 'Reactance': 0.013, 'Conductance': 0, 'Susceptance': 0.63, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '387'},
        {'fromNode': '3048.0', 'toNode': '5045.0', 'Resistance': 0.0011, 'Reactance': 0.013, 'Conductance': 0, 'Susceptance': 0.63, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '388'},
        {'fromNode': '3048.0', 'toNode': '5120.0', 'Resistance': 0.0013, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 1.02, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '389'},
        {'fromNode': '3048.0', 'toNode': '5394.0', 'Resistance': 0.0013, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 1.02, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '390'},
        {'fromNode': '3051.0', 'toNode': '3050.0', 'Resistance': 0.00147, 'Reactance': 0.09038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 122.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '391'},
        {'fromNode': '3051.0', 'toNode': '3050.0', 'Resistance': 0.00147, 'Reactance': 0.09038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 122.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '392'},
        {'fromNode': '4014.0', 'toNode': '3050.0', 'Resistance': 0.01665, 'Reactance': 0.16272, 'Conductance': 0, 'Susceptance': 0.31309, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '393'},
        {'fromNode': '3052.0', 'toNode': '3093.0', 'Resistance': 0.03727, 'Reactance': 0.22382, 'Conductance': 0, 'Susceptance': 0.03037, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '394'},
        {'fromNode': '3052.0', 'toNode': '3107.0', 'Resistance': 0.01629, 'Reactance': 0.09779, 'Conductance': 0, 'Susceptance': 0.01327, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '395'},
        {'fromNode': '3054.0', 'toNode': '3053.0', 'Resistance': 0.00422, 'Reactance': 0.16643, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 103.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '396'},
        {'fromNode': '3055.0', 'toNode': '3053.0', 'Resistance': 0.00065, 'Reactance': 0.02944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '397'},
        {'fromNode': '3056.0', 'toNode': '3053.0', 'Resistance': 0.00099, 'Reactance': 0.0556, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '398'},
        {'fromNode': '3057.0', 'toNode': '3053.0', 'Resistance': 0.00152, 'Reactance': 0.05399, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '399'},
        {'fromNode': '3058.0', 'toNode': '3053.0', 'Resistance': 0.003, 'Reactance': 0.03, 'Conductance': 0, 'Susceptance': 0.06, 'MVA_rating': 1000.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '400'},
        {'fromNode': '3082.0', 'toNode': '3053.0', 'Resistance': 0.00152, 'Reactance': 0.0149, 'Conductance': 0, 'Susceptance': 0.02867, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '401'},
        {'fromNode': '3053.0', 'toNode': '3088.0', 'Resistance': 0.00333, 'Reactance': 0.03466, 'Conductance': 0, 'Susceptance': 0.11812, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '402'},
        {'fromNode': '3053.0', 'toNode': '3088.0', 'Resistance': 0.00333, 'Reactance': 0.03466, 'Conductance': 0, 'Susceptance': 0.11812, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '403'},
        {'fromNode': '3059.0', 'toNode': '3058.0', 'Resistance': 0.00178, 'Reactance': 0.135, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '404'},
        {'fromNode': '3060.0', 'toNode': '3058.0', 'Resistance': 0.00205, 'Reactance': 0.114, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '405'},
        {'fromNode': '3061.0', 'toNode': '3058.0', 'Resistance': 0.00132, 'Reactance': 0.0814, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 155.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '406'},
        {'fromNode': '3062.0', 'toNode': '3058.0', 'Resistance': 0.00115, 'Reactance': 0.04113, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '407'},
        {'fromNode': '3063.0', 'toNode': '3058.0', 'Resistance': 0.00159, 'Reactance': 0.0585, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '408'},
        {'fromNode': '3059.0', 'toNode': '3111.0', 'Resistance': 0.01912, 'Reactance': 0.04672, 'Conductance': 0, 'Susceptance': 0.00612, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '409'},
        {'fromNode': '3065.0', 'toNode': '3064.0', 'Resistance': 0.00856, 'Reactance': 0.17814, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 110.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '410'},
        {'fromNode': '3064.0', 'toNode': '3141.0', 'Resistance': 0.01218, 'Reactance': 0.05051, 'Conductance': 0, 'Susceptance': 0.00857, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '411'},
        {'fromNode': '3066.0', 'toNode': '3142.0', 'Resistance': 0.03037, 'Reactance': 0.17347, 'Conductance': 0, 'Susceptance': 0.02607, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '412'},
        {'fromNode': '3068.0', 'toNode': '3067.0', 'Resistance': 0.00117, 'Reactance': 0.04824, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '413'},
        {'fromNode': '3067.0', 'toNode': '3070.0', 'Resistance': 0.01175, 'Reactance': 0.06712, 'Conductance': 0, 'Susceptance': 0.01009, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '414'},
        {'fromNode': '3067.0', 'toNode': '3094.0', 'Resistance': 0.01576, 'Reactance': 0.07025, 'Conductance': 0, 'Susceptance': 0.0103, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '415'},
        {'fromNode': '3067.0', 'toNode': '3141.0', 'Resistance': 0.01175, 'Reactance': 0.05234, 'Conductance': 0, 'Susceptance': 0.00768, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '416'},
        {'fromNode': '3069.0', 'toNode': '3077.0', 'Resistance': 0.01599, 'Reactance': 0.09132, 'Conductance': 0, 'Susceptance': 0.01372, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '417'},
        {'fromNode': '3069.0', 'toNode': '3140.0', 'Resistance': 0.01354, 'Reactance': 0.03311, 'Conductance': 0, 'Susceptance': 0.00434, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '418'},
        {'fromNode': '3071.0', 'toNode': '3070.0', 'Resistance': 0.00337, 'Reactance': 0.06343, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '419'},
        {'fromNode': '3070.0', 'toNode': '3131.0', 'Resistance': 0.00569, 'Reactance': 0.02535, 'Conductance': 0, 'Susceptance': 0.00372, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '420'},
        {'fromNode': '3142.0', 'toNode': '3072.0', 'Resistance': 0.01958, 'Reactance': 0.11184, 'Conductance': 0, 'Susceptance': 0.01681, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '421'},
        {'fromNode': '3074.0', 'toNode': '3073.0', 'Resistance': 0.00148, 'Reactance': 0.05882, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '422'},
        {'fromNode': '3073.0', 'toNode': '3076.0', 'Resistance': 0.01657, 'Reactance': 0.09953, 'Conductance': 0, 'Susceptance': 0.01351, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '423'},
        {'fromNode': '3123.0', 'toNode': '3073.0', 'Resistance': 0.01212, 'Reactance': 0.06921, 'Conductance': 0, 'Susceptance': 0.0104, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '424'},
        {'fromNode': '3073.0', 'toNode': '4002.0', 'Resistance': 0.03286, 'Reactance': 0.18769, 'Conductance': 0, 'Susceptance': 0.0282, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '425'},
        {'fromNode': '3132.0', 'toNode': '3075.0', 'Resistance': 0.07582, 'Reactance': 0.18532, 'Conductance': 0, 'Susceptance': 0.02427, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '426'},
        {'fromNode': '3075.0', 'toNode': '3138.0', 'Resistance': 0.0747, 'Reactance': 0.18259, 'Conductance': 0, 'Susceptance': 0.02391, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '427'},
        {'fromNode': '3077.0', 'toNode': '3099.0', 'Resistance': 0.00591, 'Reactance': 0.03377, 'Conductance': 0, 'Susceptance': 0.00508, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '428'},
        {'fromNode': '3077.0', 'toNode': '3131.0', 'Resistance': 0.01523, 'Reactance': 0.06786, 'Conductance': 0, 'Susceptance': 0.00995, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '429'},
        {'fromNode': '3079.0', 'toNode': '3078.0', 'Resistance': 0.00156, 'Reactance': 0.05913, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 166.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '430'},
        {'fromNode': '3081.0', 'toNode': '3078.0', 'Resistance': 0.00202, 'Reactance': 0.06066, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '431'},
        {'fromNode': '3078.0', 'toNode': '3133.0', 'Resistance': 0.0042, 'Reactance': 0.03003, 'Conductance': 0, 'Susceptance': 0.05475, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '432'},
        {'fromNode': '3080.0', 'toNode': '3079.0', 'Resistance': 0.00542, 'Reactance': 0.12325, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 90.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '433'},
        {'fromNode': '3134.0', 'toNode': '3079.0', 'Resistance': 0.01347, 'Reactance': 0.05936, 'Conductance': 0, 'Susceptance': 0.02763, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '434'},
        {'fromNode': '3083.0', 'toNode': '3082.0', 'Resistance': 0.00172, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 161.06, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '435'},
        {'fromNode': '3084.0', 'toNode': '3082.0', 'Resistance': 0.00171, 'Reactance': 0.09998, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '436'},
        {'fromNode': '3137.0', 'toNode': '3083.0', 'Resistance': 0.00824, 'Reactance': 0.03605, 'Conductance': 0, 'Susceptance': 0.00549, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '437'},
        {'fromNode': '3086.0', 'toNode': '3085.0', 'Resistance': 0.00703, 'Reactance': 0.2158, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 77.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '438'},
        {'fromNode': '3087.0', 'toNode': '3085.0', 'Resistance': 0.00831, 'Reactance': 0.19507, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '439'},
        {'fromNode': '3085.0', 'toNode': '3128.0', 'Resistance': 0.0069, 'Reactance': 0.01687, 'Conductance': 0, 'Susceptance': 0.00221, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '440'},
        {'fromNode': '3089.0', 'toNode': '3088.0', 'Resistance': 0.0005, 'Reactance': 0.02042, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 411.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '441'},
        {'fromNode': '3092.0', 'toNode': '3088.0', 'Resistance': 0.00152, 'Reactance': 0.0846, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '442'},
        {'fromNode': '5395.0', 'toNode': '3088.0', 'Resistance': 0.00237, 'Reactance': 0.0246, 'Conductance': 0, 'Susceptance': 0.08383, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '443'},
        {'fromNode': '5395.0', 'toNode': '3088.0', 'Resistance': 0.00237, 'Reactance': 0.0246, 'Conductance': 0, 'Susceptance': 0.08383, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '444'},
        {'fromNode': '3090.0', 'toNode': '3089.0', 'Resistance': 0.00098, 'Reactance': 0.02574, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 277.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '445'},
        {'fromNode': '3089.0', 'toNode': '5334.0', 'Resistance': 0.0094, 'Reactance': 0.05512, 'Conductance': 0, 'Susceptance': 0.02752, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '446'},
        {'fromNode': '3091.0', 'toNode': '3090.0', 'Resistance': 0.00127, 'Reactance': 0.03474, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 268.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '447'},
        {'fromNode': '3102.0', 'toNode': '3091.0', 'Resistance': 0.00402, 'Reactance': 0.02416, 'Conductance': 0, 'Susceptance': 0.00328, 'MVA_rating': 109.76, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '448'},
        {'fromNode': '3091.0', 'toNode': '3125.0', 'Resistance': 0.01164, 'Reactance': 0.02796, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '449'},
        {'fromNode': '3095.0', 'toNode': '3094.0', 'Resistance': 0.00352, 'Reactance': 0.16184, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '450'},
        {'fromNode': '3137.0', 'toNode': '3094.0', 'Resistance': 0.018, 'Reactance': 0.04399, 'Conductance': 0, 'Susceptance': 0.00576, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '451'},
        {'fromNode': '3097.0', 'toNode': '3096.0', 'Resistance': 0.00239, 'Reactance': 0.06155, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 162.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '452'},
        {'fromNode': '3097.0', 'toNode': '3096.0', 'Resistance': 0.00239, 'Reactance': 0.06155, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 162.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '453'},
        {'fromNode': '3098.0', 'toNode': '3096.0', 'Resistance': 0.0034, 'Reactance': 0.134, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '454'},
        {'fromNode': '3133.0', 'toNode': '3096.0', 'Resistance': 0.0038, 'Reactance': 0.02718, 'Conductance': 0, 'Susceptance': 0.04955, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '455'},
        {'fromNode': '3109.0', 'toNode': '3097.0', 'Resistance': 0.00325, 'Reactance': 0.02263, 'Conductance': 0, 'Susceptance': 0.0037, 'MVA_rating': 340.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '456'},
        {'fromNode': '3099.0', 'toNode': '3100.0', 'Resistance': 0.01869, 'Reactance': 0.08329, 'Conductance': 0, 'Susceptance': 0.01221, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '457'},
        {'fromNode': '3100.0', 'toNode': '3102.0', 'Resistance': 0.015, 'Reactance': 0.088, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '458'},
        {'fromNode': '3125.0', 'toNode': '3100.0', 'Resistance': 0.0162, 'Reactance': 0.07086, 'Conductance': 0, 'Susceptance': 0.01079, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '459'},
        {'fromNode': '3104.0', 'toNode': '3101.0', 'Resistance': 0.01395, 'Reactance': 0.07966, 'Conductance': 0, 'Susceptance': 0.01197, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '460'},
        {'fromNode': '6360.0', 'toNode': '3101.0', 'Resistance': 0.03353, 'Reactance': 0.08195, 'Conductance': 0, 'Susceptance': 0.01073, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '461'},
        {'fromNode': '3105.0', 'toNode': '3104.0', 'Resistance': 0.00965, 'Reactance': 0.2233, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 60.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '462'},
        {'fromNode': '3106.0', 'toNode': '3104.0', 'Resistance': 0.00703, 'Reactance': 0.22022, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '463'},
        {'fromNode': '6156.0', 'toNode': '3104.0', 'Resistance': 0.01785, 'Reactance': 0.07953, 'Conductance': 0, 'Susceptance': 0.01166, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '464'},
        {'fromNode': '3108.0', 'toNode': '3147.0', 'Resistance': 0.03432, 'Reactance': 0.08389, 'Conductance': 0, 'Susceptance': 0.01098, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '465'},
        {'fromNode': '3108.0', 'toNode': '6065.0', 'Resistance': 0.02034, 'Reactance': 0.11619, 'Conductance': 0, 'Susceptance': 0.01746, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '466'},
        {'fromNode': '3110.0', 'toNode': '3109.0', 'Resistance': 0.00505, 'Reactance': 0.2206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 114.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '467'},
        {'fromNode': '3112.0', 'toNode': '3111.0', 'Resistance': 0.01169, 'Reactance': 0.25489, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 90.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '468'},
        {'fromNode': '3113.0', 'toNode': '3111.0', 'Resistance': 0.00858, 'Reactance': 0.25519, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 90.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '469'},
        {'fromNode': '3116.0', 'toNode': '3115.0', 'Resistance': 0.02115, 'Reactance': 0.08773, 'Conductance': 0, 'Susceptance': 0.01489, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '470'},
        {'fromNode': '3117.0', 'toNode': '3116.0', 'Resistance': 0.00258, 'Reactance': 0.11311, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 80.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '471'},
        {'fromNode': '3118.0', 'toNode': '3116.0', 'Resistance': 0.00353, 'Reactance': 0.07808, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 139.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '472'},
        {'fromNode': '3116.0', 'toNode': '3136.0', 'Resistance': 0.01168, 'Reactance': 0.07015, 'Conductance': 0, 'Susceptance': 0.00952, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '473'},
        {'fromNode': '3120.0', 'toNode': '3119.0', 'Resistance': 0.00248, 'Reactance': 0.09082, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 46.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '474'},
        {'fromNode': '4002.0', 'toNode': '3119.0', 'Resistance': 0.02215, 'Reactance': 0.13302, 'Conductance': 0, 'Susceptance': 0.01805, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '475'},
        {'fromNode': '4188.0', 'toNode': '3119.0', 'Resistance': 0.027, 'Reactance': 0.1542, 'Conductance': 0, 'Susceptance': 0.02317, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '476'},
        {'fromNode': '3122.0', 'toNode': '3121.0', 'Resistance': 0.00565, 'Reactance': 0.10546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '477'},
        {'fromNode': '3121.0', 'toNode': '3129.0', 'Resistance': 0.00954, 'Reactance': 0.03956, 'Conductance': 0, 'Susceptance': 0.00671, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '478'},
        {'fromNode': '3121.0', 'toNode': '3145.0', 'Resistance': 0.02936, 'Reactance': 0.13085, 'Conductance': 0, 'Susceptance': 0.01919, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '479'},
        {'fromNode': '3126.0', 'toNode': '3123.0', 'Resistance': 0.05936, 'Reactance': 0.14511, 'Conductance': 0, 'Susceptance': 0.019, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '480'},
        {'fromNode': '6286.0', 'toNode': '3124.0', 'Resistance': 0.018, 'Reactance': 0.1, 'Conductance': 0, 'Susceptance': 0.016, 'MVA_rating': 100.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '481'},
        {'fromNode': '3126.0', 'toNode': '6059.0', 'Resistance': 0.04241, 'Reactance': 0.10187, 'Conductance': 0, 'Susceptance': 0.01382, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '482'},
        {'fromNode': '3141.0', 'toNode': '3127.0', 'Resistance': 0.00595, 'Reactance': 0.02651, 'Conductance': 0, 'Susceptance': 0.00389, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '483'},
        {'fromNode': '3132.0', 'toNode': '3144.0', 'Resistance': 0.04225, 'Reactance': 0.10327, 'Conductance': 0, 'Susceptance': 0.01352, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '484'},
        {'fromNode': '3134.0', 'toNode': '3133.0', 'Resistance': 0.00391, 'Reactance': 0.20153, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '485'},
        {'fromNode': '3135.0', 'toNode': '3134.0', 'Resistance': 0.01014, 'Reactance': 0.22418, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '486'},
        {'fromNode': '3136.0', 'toNode': '3135.0', 'Resistance': 0.00579, 'Reactance': 0.2297, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '487'},
        {'fromNode': '3138.0', 'toNode': '3142.0', 'Resistance': 0.02534, 'Reactance': 0.14471, 'Conductance': 0, 'Susceptance': 0.02175, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '488'},
        {'fromNode': '3138.0', 'toNode': '3146.0', 'Resistance': 0.03674, 'Reactance': 0.1607, 'Conductance': 0, 'Susceptance': 0.02447, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '489'},
        {'fromNode': '3144.0', 'toNode': '3143.0', 'Resistance': 0.04463, 'Reactance': 0.10909, 'Conductance': 0, 'Susceptance': 0.01428, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '490'},
        {'fromNode': '4003.0', 'toNode': '4040.0', 'Resistance': 0.01209, 'Reactance': 0.07809, 'Conductance': 0, 'Susceptance': 0.0149, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '491'},
        {'fromNode': '4103.0', 'toNode': '4003.0', 'Resistance': 0.00838, 'Reactance': 0.05414, 'Conductance': 0, 'Susceptance': 0.01033, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '492'},
        {'fromNode': '4103.0', 'toNode': '4003.0', 'Resistance': 0.00838, 'Reactance': 0.05414, 'Conductance': 0, 'Susceptance': 0.01033, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '493'},
        {'fromNode': '4128.0', 'toNode': '4003.0', 'Resistance': 0.01296, 'Reactance': 0.09028, 'Conductance': 0, 'Susceptance': 0.01476, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '494'},
        {'fromNode': '4005.0', 'toNode': '4004.0', 'Resistance': 0.00169, 'Reactance': 0.06285, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '495'},
        {'fromNode': '4014.0', 'toNode': '4004.0', 'Resistance': 0.006, 'Reactance': 0.06, 'Conductance': 0, 'Susceptance': 0.115, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '496'},
        {'fromNode': '4113.0', 'toNode': '4004.0', 'Resistance': 0.00456, 'Reactance': 0.03255, 'Conductance': 0, 'Susceptance': 0.05934, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '497'},
        {'fromNode': '4022.0', 'toNode': '4005.0', 'Resistance': 0.02734, 'Reactance': 0.11343, 'Conductance': 0, 'Susceptance': 0.01925, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '498'},
        {'fromNode': '4114.0', 'toNode': '4005.0', 'Resistance': 0.01989, 'Reactance': 0.11362, 'Conductance': 0, 'Susceptance': 0.01707, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '499'},
        {'fromNode': '4006.0', 'toNode': '4011.0', 'Resistance': 0.0269, 'Reactance': 0.11766, 'Conductance': 0, 'Susceptance': 0.01792, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '500'},
        {'fromNode': '4006.0', 'toNode': '4197.0', 'Resistance': 0.00624, 'Reactance': 0.04344, 'Conductance': 0, 'Susceptance': 0.0071, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '501'},
        {'fromNode': '4007.0', 'toNode': '4027.0', 'Resistance': 0.00348, 'Reactance': 0.02089, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '502'},
        {'fromNode': '4007.0', 'toNode': '4034.0', 'Resistance': 0.00873, 'Reactance': 0.03892, 'Conductance': 0, 'Susceptance': 0.00571, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '503'},
        {'fromNode': '4007.0', 'toNode': '4122.0', 'Resistance': 0.00469, 'Reactance': 0.02091, 'Conductance': 0, 'Susceptance': 0.00307, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '504'},
        {'fromNode': '4008.0', 'toNode': '6112.0', 'Resistance': 0.01363, 'Reactance': 0.07782, 'Conductance': 0, 'Susceptance': 0.01169, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '505'},
        {'fromNode': '6155.0', 'toNode': '4008.0', 'Resistance': 0.01482, 'Reactance': 0.06605, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '506'},
        {'fromNode': '4008.0', 'toNode': '6346.0', 'Resistance': 0.01174, 'Reactance': 0.05229, 'Conductance': 0, 'Susceptance': 0.00767, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '507'},
        {'fromNode': '4010.0', 'toNode': '4009.0', 'Resistance': 0.00242, 'Reactance': 0.06944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 196.5, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '508'},
        {'fromNode': '4010.0', 'toNode': '4009.0', 'Resistance': 0.00315, 'Reactance': 0.06944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 196.5, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '509'},
        {'fromNode': '4152.0', 'toNode': '4009.0', 'Resistance': 0.00542, 'Reactance': 0.03498, 'Conductance': 0, 'Susceptance': 0.00667, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '510'},
        {'fromNode': '4152.0', 'toNode': '4009.0', 'Resistance': 0.00542, 'Reactance': 0.03498, 'Conductance': 0, 'Susceptance': 0.00667, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '511'},
        {'fromNode': '4165.0', 'toNode': '4009.0', 'Resistance': 0.00315, 'Reactance': 0.02032, 'Conductance': 0, 'Susceptance': 0.00388, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '512'},
        {'fromNode': '4165.0', 'toNode': '4009.0', 'Resistance': 0.00315, 'Reactance': 0.02032, 'Conductance': 0, 'Susceptance': 0.00388, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '513'},
        {'fromNode': '4011.0', 'toNode': '4185.0', 'Resistance': 0.03112, 'Reactance': 0.13615, 'Conductance': 0, 'Susceptance': 0.02073, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '514'},
        {'fromNode': '4034.0', 'toNode': '4012.0', 'Resistance': 0.01625, 'Reactance': 0.09759, 'Conductance': 0, 'Susceptance': 0.01324, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '515'},
        {'fromNode': '4012.0', 'toNode': '4103.0', 'Resistance': 0.02485, 'Reactance': 0.14192, 'Conductance': 0, 'Susceptance': 0.02133, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '516'},
        {'fromNode': '4012.0', 'toNode': '4121.0', 'Resistance': 0.01296, 'Reactance': 0.07781, 'Conductance': 0, 'Susceptance': 0.01056, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '517'},
        {'fromNode': '4012.0', 'toNode': '4171.0', 'Resistance': 0.02502, 'Reactance': 0.11152, 'Conductance': 0, 'Susceptance': 0.01635, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '518'},
        {'fromNode': '4013.0', 'toNode': '4089.0', 'Resistance': 0.01437, 'Reactance': 0.09281, 'Conductance': 0, 'Susceptance': 0.01771, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '519'},
        {'fromNode': '4107.0', 'toNode': '4013.0', 'Resistance': 0.02623, 'Reactance': 0.06413, 'Conductance': 0, 'Susceptance': 0.0084, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '520'},
        {'fromNode': '4013.0', 'toNode': '4122.0', 'Resistance': 0.00312, 'Reactance': 0.01784, 'Conductance': 0, 'Susceptance': 0.00268, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '521'},
        {'fromNode': '4015.0', 'toNode': '4014.0', 'Resistance': 0.00058, 'Reactance': 0.03637, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 321.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '522'},
        {'fromNode': '4015.0', 'toNode': '4014.0', 'Resistance': 0.00058, 'Reactance': 0.03637, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 321.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '523'},
        {'fromNode': '4014.0', 'toNode': '4131.0', 'Resistance': 0.01669, 'Reactance': 0.11926, 'Conductance': 0, 'Susceptance': 0.21744, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '524'},
        {'fromNode': '4188.0', 'toNode': '4015.0', 'Resistance': 0.03538, 'Reactance': 0.21243, 'Conductance': 0, 'Susceptance': 0.02883, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '525'},
        {'fromNode': '4016.0', 'toNode': '4023.0', 'Resistance': 0.01228, 'Reactance': 0.07374, 'Conductance': 0, 'Susceptance': 0.01001, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '526'},
        {'fromNode': '4117.0', 'toNode': '4016.0', 'Resistance': 0.01709, 'Reactance': 0.0709, 'Conductance': 0, 'Susceptance': 0.01203, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '527'},
        {'fromNode': '4081.0', 'toNode': '4017.0', 'Resistance': 0.00217, 'Reactance': 0.01515, 'Conductance': 0, 'Susceptance': 0.00248, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '528'},
        {'fromNode': '4081.0', 'toNode': '4017.0', 'Resistance': 0.00217, 'Reactance': 0.01515, 'Conductance': 0, 'Susceptance': 0.00248, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '529'},
        {'fromNode': '4017.0', 'toNode': '4087.0', 'Resistance': 0.00568, 'Reactance': 0.02531, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '530'},
        {'fromNode': '4017.0', 'toNode': '4123.0', 'Resistance': 0.00492, 'Reactance': 0.03426, 'Conductance': 0, 'Susceptance': 0.0056, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '531'},
        {'fromNode': '4018.0', 'toNode': '4037.0', 'Resistance': 0.01371, 'Reactance': 0.06107, 'Conductance': 0, 'Susceptance': 0.00896, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '532'},
        {'fromNode': '4149.0', 'toNode': '4018.0', 'Resistance': 0.01472, 'Reactance': 0.06561, 'Conductance': 0, 'Susceptance': 0.00962, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '533'},
        {'fromNode': '4019.0', 'toNode': '4067.0', 'Resistance': 0.00919, 'Reactance': 0.06276, 'Conductance': 0, 'Susceptance': 0.01068, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '534'},
        {'fromNode': '4019.0', 'toNode': '4067.0', 'Resistance': 0.00919, 'Reactance': 0.06276, 'Conductance': 0, 'Susceptance': 0.01068, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '535'},
        {'fromNode': '4019.0', 'toNode': '4121.0', 'Resistance': 0.02026, 'Reactance': 0.04952, 'Conductance': 0, 'Susceptance': 0.00648, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '536'},
        {'fromNode': '4051.0', 'toNode': '4020.0', 'Resistance': 0.01602, 'Reactance': 0.10945, 'Conductance': 0, 'Susceptance': 0.01862, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '537'},
        {'fromNode': '4051.0', 'toNode': '4020.0', 'Resistance': 0.01602, 'Reactance': 0.10945, 'Conductance': 0, 'Susceptance': 0.01862, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '538'},
        {'fromNode': '4020.0', 'toNode': '4123.0', 'Resistance': 0.00803, 'Reactance': 0.03577, 'Conductance': 0, 'Susceptance': 0.00525, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '539'},
        {'fromNode': '4150.0', 'toNode': '4020.0', 'Resistance': 0.00418, 'Reactance': 0.02912, 'Conductance': 0, 'Susceptance': 0.00476, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '540'},
        {'fromNode': '4150.0', 'toNode': '4020.0', 'Resistance': 0.00418, 'Reactance': 0.02912, 'Conductance': 0, 'Susceptance': 0.00476, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '541'},
        {'fromNode': '4089.0', 'toNode': '4021.0', 'Resistance': 0.01219, 'Reactance': 0.05329, 'Conductance': 0, 'Susceptance': 0.00812, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '542'},
        {'fromNode': '4021.0', 'toNode': '4167.0', 'Resistance': 0.00965, 'Reactance': 0.04299, 'Conductance': 0, 'Susceptance': 0.0063, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '543'},
        {'fromNode': '4187.0', 'toNode': '4022.0', 'Resistance': 0.02501, 'Reactance': 0.10376, 'Conductance': 0, 'Susceptance': 0.01761, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '544'},
        {'fromNode': '6112.0', 'toNode': '4023.0', 'Resistance': 0.04152, 'Reactance': 0.10149, 'Conductance': 0, 'Susceptance': 0.01329, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '545'},
        {'fromNode': '4025.0', 'toNode': '4024.0', 'Resistance': 0.0013, 'Reactance': 0.04477, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '546'},
        {'fromNode': '4026.0', 'toNode': '4024.0', 'Resistance': 0.00148, 'Reactance': 0.06658, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '547'},
        {'fromNode': '4026.0', 'toNode': '4024.0', 'Resistance': 0.00148, 'Reactance': 0.06658, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '548'},
        {'fromNode': '4131.0', 'toNode': '4024.0', 'Resistance': 0.00611, 'Reactance': 0.05974, 'Conductance': 0, 'Susceptance': 0.11495, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '549'},
        {'fromNode': '6263.0', 'toNode': '4024.0', 'Resistance': 0.00754, 'Reactance': 0.05386, 'Conductance': 0, 'Susceptance': 0.0982, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '550'},
        {'fromNode': '4025.0', 'toNode': '4117.0', 'Resistance': 0.00307, 'Reactance': 0.01981, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 260.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '551'},
        {'fromNode': '4027.0', 'toNode': '4088.0', 'Resistance': 0.00276, 'Reactance': 0.01883, 'Conductance': 0, 'Susceptance': 0.0032, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '552'},
        {'fromNode': '4027.0', 'toNode': '4088.0', 'Resistance': 0.00276, 'Reactance': 0.01883, 'Conductance': 0, 'Susceptance': 0.0032, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '553'},
        {'fromNode': '4029.0', 'toNode': '4028.0', 'Resistance': 0.00077, 'Reactance': 0.04468, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 148.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '554'},
        {'fromNode': '4029.0', 'toNode': '4028.0', 'Resistance': 0.00077, 'Reactance': 0.04468, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 148.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '555'},
        {'fromNode': '4030.0', 'toNode': '4028.0', 'Resistance': 0.00053, 'Reactance': 0.0395, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 241.5, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '556'},
        {'fromNode': '4030.0', 'toNode': '4028.0', 'Resistance': 0.00053, 'Reactance': 0.0395, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 241.5, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '557'},
        {'fromNode': '4030.0', 'toNode': '4028.0', 'Resistance': 0.00053, 'Reactance': 0.0395, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '558'},
        {'fromNode': '4028.0', 'toNode': '7043.0', 'Resistance': 0.00286, 'Reactance': 0.02045, 'Conductance': 0, 'Susceptance': 0.03728, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '559'},
        {'fromNode': '4028.0', 'toNode': '7274.0', 'Resistance': 0.00028, 'Reactance': 0.00295, 'Conductance': 0, 'Susceptance': 0.01006, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '560'},
        {'fromNode': '4028.0', 'toNode': '7274.0', 'Resistance': 0.00028, 'Reactance': 0.00295, 'Conductance': 0, 'Susceptance': 0.01006, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '561'},
        {'fromNode': '4029.0', 'toNode': '6253.0', 'Resistance': 0.01201, 'Reactance': 0.0776, 'Conductance': 0, 'Susceptance': 0.0148, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '562'},
        {'fromNode': '7275.0', 'toNode': '4029.0', 'Resistance': 0.00321, 'Reactance': 0.01407, 'Conductance': 0, 'Susceptance': 0.00214, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '563'},
        {'fromNode': '4031.0', 'toNode': '4043.0', 'Resistance': 0.04121, 'Reactance': 0.18028, 'Conductance': 0, 'Susceptance': 0.02746, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '564'},
        {'fromNode': '4147.0', 'toNode': '4031.0', 'Resistance': 0.02053, 'Reactance': 0.0898, 'Conductance': 0, 'Susceptance': 0.01368, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '565'},
        {'fromNode': '4051.0', 'toNode': '4032.0', 'Resistance': 0.02109, 'Reactance': 0.05154, 'Conductance': 0, 'Susceptance': 0.00675, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '566'},
        {'fromNode': '4141.0', 'toNode': '4032.0', 'Resistance': 0.01931, 'Reactance': 0.08605, 'Conductance': 0, 'Susceptance': 0.01262, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '567'},
        {'fromNode': '4033.0', 'toNode': '4101.0', 'Resistance': 0.0036, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.003, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '568'},
        {'fromNode': '4033.0', 'toNode': '4147.0', 'Resistance': 0.00984, 'Reactance': 0.06854, 'Conductance': 0, 'Susceptance': 0.0112, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '569'},
        {'fromNode': '4033.0', 'toNode': '4147.0', 'Resistance': 0.00984, 'Reactance': 0.06854, 'Conductance': 0, 'Susceptance': 0.0112, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '570'},
        {'fromNode': '4033.0', 'toNode': '4190.0', 'Resistance': 0.00397, 'Reactance': 0.01648, 'Conductance': 0, 'Susceptance': 0.0028, 'MVA_rating': 223.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '571'},
        {'fromNode': '4034.0', 'toNode': '4154.0', 'Resistance': 0.01506, 'Reactance': 0.10289, 'Conductance': 0, 'Susceptance': 0.01751, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '572'},
        {'fromNode': '4064.0', 'toNode': '4035.0', 'Resistance': 0.00506, 'Reactance': 0.02099, 'Conductance': 0, 'Susceptance': 0.00356, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '573'},
        {'fromNode': '4035.0', 'toNode': '4150.0', 'Resistance': 0.02431, 'Reactance': 0.05942, 'Conductance': 0, 'Susceptance': 0.00778, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '574'},
        {'fromNode': '4132.0', 'toNode': '4036.0', 'Resistance': 0.03455, 'Reactance': 0.15396, 'Conductance': 0, 'Susceptance': 0.02258, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '575'},
        {'fromNode': '4036.0', 'toNode': '4189.0', 'Resistance': 0.05901, 'Reactance': 0.14424, 'Conductance': 0, 'Susceptance': 0.01889, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '576'},
        {'fromNode': '4037.0', 'toNode': '4054.0', 'Resistance': 0.00975, 'Reactance': 0.05854, 'Conductance': 0, 'Susceptance': 0.00794, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '577'},
        {'fromNode': '4099.0', 'toNode': '4037.0', 'Resistance': 0.01617, 'Reactance': 0.07075, 'Conductance': 0, 'Susceptance': 0.01077, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '578'},
        {'fromNode': '4054.0', 'toNode': '4038.0', 'Resistance': 0.01093, 'Reactance': 0.06561, 'Conductance': 0, 'Susceptance': 0.0089, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '579'},
        {'fromNode': '4038.0', 'toNode': '4125.0', 'Resistance': 0.01635, 'Reactance': 0.09341, 'Conductance': 0, 'Susceptance': 0.01404, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '580'},
        {'fromNode': '4038.0', 'toNode': '4178.0', 'Resistance': 0.03181, 'Reactance': 0.07775, 'Conductance': 0, 'Susceptance': 0.01018, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '581'},
        {'fromNode': '4040.0', 'toNode': '4039.0', 'Resistance': 0.00078, 'Reactance': 0.02871, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 384.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '582'},
        {'fromNode': '4040.0', 'toNode': '4039.0', 'Resistance': 0.00078, 'Reactance': 0.02871, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 384.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '583'},
        {'fromNode': '4073.0', 'toNode': '4039.0', 'Resistance': 0.00022, 'Reactance': 0.00224, 'Conductance': 0, 'Susceptance': 0.00764, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '584'},
        {'fromNode': '4073.0', 'toNode': '4039.0', 'Resistance': 0.00022, 'Reactance': 0.00224, 'Conductance': 0, 'Susceptance': 0.00764, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '585'},
        {'fromNode': '4039.0', 'toNode': '4151.0', 'Resistance': 0.00765, 'Reactance': 0.07956, 'Conductance': 0, 'Susceptance': 0.27111, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '586'},
        {'fromNode': '4039.0', 'toNode': '4151.0', 'Resistance': 0.00765, 'Reactance': 0.07956, 'Conductance': 0, 'Susceptance': 0.27111, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '587'},
        {'fromNode': '4039.0', 'toNode': '4151.0', 'Resistance': 0.00765, 'Reactance': 0.07956, 'Conductance': 0, 'Susceptance': 0.27111, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '588'},
        {'fromNode': '4074.0', 'toNode': '4040.0', 'Resistance': 0.00148, 'Reactance': 0.00957, 'Conductance': 0, 'Susceptance': 0.00183, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '589'},
        {'fromNode': '4074.0', 'toNode': '4040.0', 'Resistance': 0.00148, 'Reactance': 0.00957, 'Conductance': 0, 'Susceptance': 0.00183, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '590'},
        {'fromNode': '4074.0', 'toNode': '4040.0', 'Resistance': 0.00148, 'Reactance': 0.00957, 'Conductance': 0, 'Susceptance': 0.00183, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '591'},
        {'fromNode': '4079.0', 'toNode': '4040.0', 'Resistance': 0.00457, 'Reactance': 0.03186, 'Conductance': 0, 'Susceptance': 0.00521, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '592'},
        {'fromNode': '4079.0', 'toNode': '4040.0', 'Resistance': 0.00457, 'Reactance': 0.03186, 'Conductance': 0, 'Susceptance': 0.00521, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '593'},
        {'fromNode': '4079.0', 'toNode': '4040.0', 'Resistance': 0.00457, 'Reactance': 0.03186, 'Conductance': 0, 'Susceptance': 0.00521, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '594'},
        {'fromNode': '4042.0', 'toNode': '4041.0', 'Resistance': 0.00056, 'Reactance': 0.03546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 442.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '595'},
        {'fromNode': '4042.0', 'toNode': '4041.0', 'Resistance': 0.00056, 'Reactance': 0.03546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 442.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '596'},
        {'fromNode': '4042.0', 'toNode': '4041.0', 'Resistance': 0.00056, 'Reactance': 0.03546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 442.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '597'},
        {'fromNode': '4073.0', 'toNode': '4041.0', 'Resistance': 0.0019, 'Reactance': 0.01971, 'Conductance': 0, 'Susceptance': 0.06717, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '598'},
        {'fromNode': '4073.0', 'toNode': '4041.0', 'Resistance': 0.0019, 'Reactance': 0.01971, 'Conductance': 0, 'Susceptance': 0.06717, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '599'},
        {'fromNode': '4088.0', 'toNode': '4042.0', 'Resistance': 0.00312, 'Reactance': 0.02015, 'Conductance': 0, 'Susceptance': 0.00384, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '600'},
        {'fromNode': '4088.0', 'toNode': '4042.0', 'Resistance': 0.00312, 'Reactance': 0.02015, 'Conductance': 0, 'Susceptance': 0.00384, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '601'},
        {'fromNode': '4107.0', 'toNode': '4042.0', 'Resistance': 0.00298, 'Reactance': 0.01925, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 331.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '602'},
        {'fromNode': '4107.0', 'toNode': '4042.0', 'Resistance': 0.00298, 'Reactance': 0.01925, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 331.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '603'},
        {'fromNode': '4043.0', 'toNode': '4119.0', 'Resistance': 0.01218, 'Reactance': 0.05428, 'Conductance': 0, 'Susceptance': 0.00796, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '604'},
        {'fromNode': '4045.0', 'toNode': '4044.0', 'Resistance': 0.00353, 'Reactance': 0.16159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 120.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '605'},
        {'fromNode': '4046.0', 'toNode': '4044.0', 'Resistance': 0.00669, 'Reactance': 0.2458, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '606'},
        {'fromNode': '4047.0', 'toNode': '4044.0', 'Resistance': 0.00557, 'Reactance': 0.09157, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '607'},
        {'fromNode': '4044.0', 'toNode': '4119.0', 'Resistance': 0.00264, 'Reactance': 0.01804, 'Conductance': 0, 'Susceptance': 0.00307, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '608'},
        {'fromNode': '4185.0', 'toNode': '4044.0', 'Resistance': 0.03117, 'Reactance': 0.13891, 'Conductance': 0, 'Susceptance': 0.02037, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '609'},
        {'fromNode': '4048.0', 'toNode': '4154.0', 'Resistance': 0.00408, 'Reactance': 0.02784, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '610'},
        {'fromNode': '4048.0', 'toNode': '4155.0', 'Resistance': 0.00335, 'Reactance': 0.02332, 'Conductance': 0, 'Susceptance': 0.00381, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '611'},
        {'fromNode': '4048.0', 'toNode': '4155.0', 'Resistance': 0.00335, 'Reactance': 0.02332, 'Conductance': 0, 'Susceptance': 0.00381, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '612'},
        {'fromNode': '4050.0', 'toNode': '4049.0', 'Resistance': 0.00389, 'Reactance': 0.07962, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 195.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '613'},
        {'fromNode': '4050.0', 'toNode': '4049.0', 'Resistance': 0.00389, 'Reactance': 0.07962, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 195.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '614'},
        {'fromNode': '4086.0', 'toNode': '4049.0', 'Resistance': 0.00927, 'Reactance': 0.04053, 'Conductance': 0, 'Susceptance': 0.00617, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '615'},
        {'fromNode': '4049.0', 'toNode': '4194.0', 'Resistance': 0.00379, 'Reactance': 0.02643, 'Conductance': 0, 'Susceptance': 0.00432, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '616'},
        {'fromNode': '4049.0', 'toNode': '4194.0', 'Resistance': 0.00379, 'Reactance': 0.02643, 'Conductance': 0, 'Susceptance': 0.00432, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '617'},
        {'fromNode': '4052.0', 'toNode': '4051.0', 'Resistance': 0.00406, 'Reactance': 0.16879, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 100.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '618'},
        {'fromNode': '4127.0', 'toNode': '4051.0', 'Resistance': 0.01554, 'Reactance': 0.08876, 'Conductance': 0, 'Susceptance': 0.01334, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '619'},
        {'fromNode': '4053.0', 'toNode': '4152.0', 'Resistance': 0.0091, 'Reactance': 0.06213, 'Conductance': 0, 'Susceptance': 0.01057, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '620'},
        {'fromNode': '4053.0', 'toNode': '4152.0', 'Resistance': 0.0091, 'Reactance': 0.06213, 'Conductance': 0, 'Susceptance': 0.01057, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '621'},
        {'fromNode': '4053.0', 'toNode': '4191.0', 'Resistance': 0.02289, 'Reactance': 0.14783, 'Conductance': 0, 'Susceptance': 0.0282, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '622'},
        {'fromNode': '4053.0', 'toNode': '4191.0', 'Resistance': 0.02289, 'Reactance': 0.14783, 'Conductance': 0, 'Susceptance': 0.0282, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '623'},
        {'fromNode': '4055.0', 'toNode': '4078.0', 'Resistance': 0.00454, 'Reactance': 0.03104, 'Conductance': 0, 'Susceptance': 0.00528, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '624'},
        {'fromNode': '4055.0', 'toNode': '4078.0', 'Resistance': 0.00454, 'Reactance': 0.03104, 'Conductance': 0, 'Susceptance': 0.00528, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '625'},
        {'fromNode': '4055.0', 'toNode': '4109.0', 'Resistance': 0.01141, 'Reactance': 0.04991, 'Conductance': 0, 'Susceptance': 0.0076, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '626'},
        {'fromNode': '4055.0', 'toNode': '4145.0', 'Resistance': 0.0037, 'Reactance': 0.02577, 'Conductance': 0, 'Susceptance': 0.00421, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '627'},
        {'fromNode': '4055.0', 'toNode': '4145.0', 'Resistance': 0.0037, 'Reactance': 0.02577, 'Conductance': 0, 'Susceptance': 0.00421, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '628'},
        {'fromNode': '4057.0', 'toNode': '4056.0', 'Resistance': 0.0009, 'Reactance': 0.03263, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '629'},
        {'fromNode': '4058.0', 'toNode': '4056.0', 'Resistance': 0.00136, 'Reactance': 0.05833, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 185.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '630'},
        {'fromNode': '4059.0', 'toNode': '4056.0', 'Resistance': 0.00197, 'Reactance': 0.04935, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 185.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '631'},
        {'fromNode': '4060.0', 'toNode': '4056.0', 'Resistance': 0.00213, 'Reactance': 0.08417, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 185.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '632'},
        {'fromNode': '4061.0', 'toNode': '4056.0', 'Resistance': 0.00148, 'Reactance': 0.05744, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 259.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '633'},
        {'fromNode': '4062.0', 'toNode': '4056.0', 'Resistance': 0.00287, 'Reactance': 0.0718, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 264.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '634'},
        {'fromNode': '4063.0', 'toNode': '4056.0', 'Resistance': 0.00077, 'Reactance': 0.05841, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '635'},
        {'fromNode': '4056.0', 'toNode': '4158.0', 'Resistance': 0.00124, 'Reactance': 0.01287, 'Conductance': 0, 'Susceptance': 0.04387, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '636'},
        {'fromNode': '4056.0', 'toNode': '4181.0', 'Resistance': 0.00117, 'Reactance': 0.01146, 'Conductance': 0, 'Susceptance': 0.02204, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '637'},
        {'fromNode': '4108.0', 'toNode': '4057.0', 'Resistance': 0.00271, 'Reactance': 0.01748, 'Conductance': 0, 'Susceptance': 0.00334, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '638'},
        {'fromNode': '4108.0', 'toNode': '4057.0', 'Resistance': 0.00271, 'Reactance': 0.01748, 'Conductance': 0, 'Susceptance': 0.00334, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '639'},
        {'fromNode': '4057.0', 'toNode': '4145.0', 'Resistance': 0.00563, 'Reactance': 0.03924, 'Conductance': 0, 'Susceptance': 0.00641, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '640'},
        {'fromNode': '4057.0', 'toNode': '4145.0', 'Resistance': 0.00563, 'Reactance': 0.03924, 'Conductance': 0, 'Susceptance': 0.00641, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '641'},
        {'fromNode': '4065.0', 'toNode': '4064.0', 'Resistance': 0.00915, 'Reactance': 0.19147, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 100.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '642'},
        {'fromNode': '4069.0', 'toNode': '4064.0', 'Resistance': 0.00989, 'Reactance': 0.02376, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '643'},
        {'fromNode': '4064.0', 'toNode': '4099.0', 'Resistance': 0.00946, 'Reactance': 0.06459, 'Conductance': 0, 'Susceptance': 0.01099, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '644'},
        {'fromNode': '4064.0', 'toNode': '4099.0', 'Resistance': 0.00946, 'Reactance': 0.06459, 'Conductance': 0, 'Susceptance': 0.01099, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '645'},
        {'fromNode': '4064.0', 'toNode': '4104.0', 'Resistance': 0.00622, 'Reactance': 0.04017, 'Conductance': 0, 'Susceptance': 0.00766, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '646'},
        {'fromNode': '4064.0', 'toNode': '4104.0', 'Resistance': 0.00622, 'Reactance': 0.04017, 'Conductance': 0, 'Susceptance': 0.00766, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '647'},
        {'fromNode': '4066.0', 'toNode': '4068.0', 'Resistance': 0.013, 'Reactance': 0.05797, 'Conductance': 0, 'Susceptance': 0.0085, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '648'},
        {'fromNode': '4066.0', 'toNode': '4107.0', 'Resistance': 0.00316, 'Reactance': 0.02043, 'Conductance': 0, 'Susceptance': 0.0039, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '649'},
        {'fromNode': '4066.0', 'toNode': '4107.0', 'Resistance': 0.00316, 'Reactance': 0.02043, 'Conductance': 0, 'Susceptance': 0.0039, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '650'},
        {'fromNode': '4111.0', 'toNode': '4067.0', 'Resistance': 0.00346, 'Reactance': 0.02362, 'Conductance': 0, 'Susceptance': 0.00402, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '651'},
        {'fromNode': '4194.0', 'toNode': '4067.0', 'Resistance': 0.00473, 'Reactance': 0.03294, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '652'},
        {'fromNode': '4194.0', 'toNode': '4067.0', 'Resistance': 0.00473, 'Reactance': 0.03294, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '653'},
        {'fromNode': '4194.0', 'toNode': '4067.0', 'Resistance': 0.00473, 'Reactance': 0.03294, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '654'},
        {'fromNode': '4068.0', 'toNode': '4103.0', 'Resistance': 0.00973, 'Reactance': 0.05555, 'Conductance': 0, 'Susceptance': 0.00835, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '655'},
        {'fromNode': '4068.0', 'toNode': '4186.0', 'Resistance': 0.01855, 'Reactance': 0.08268, 'Conductance': 0, 'Susceptance': 0.01212, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '656'},
        {'fromNode': '4069.0', 'toNode': '4072.0', 'Resistance': 0.00681, 'Reactance': 0.03892, 'Conductance': 0, 'Susceptance': 0.00585, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '657'},
        {'fromNode': '4071.0', 'toNode': '4070.0', 'Resistance': 0.00121, 'Reactance': 0.04292, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 353.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '658'},
        {'fromNode': '4071.0', 'toNode': '4070.0', 'Resistance': 0.00121, 'Reactance': 0.04292, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 353.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '659'},
        {'fromNode': '4070.0', 'toNode': '4183.0', 'Resistance': 0.002, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.07, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '660'},
        {'fromNode': '4070.0', 'toNode': '4192.0', 'Resistance': 0.00137, 'Reactance': 0.01421, 'Conductance': 0, 'Susceptance': 0.04843, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '661'},
        {'fromNode': '4070.0', 'toNode': '4192.0', 'Resistance': 0.00137, 'Reactance': 0.01421, 'Conductance': 0, 'Susceptance': 0.04843, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '662'},
        {'fromNode': '4106.0', 'toNode': '4071.0', 'Resistance': 0.009, 'Reactance': 0.0401, 'Conductance': 0, 'Susceptance': 0.00588, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '663'},
        {'fromNode': '4072.0', 'toNode': '4156.0', 'Resistance': 0.00671, 'Reactance': 0.0278, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '664'},
        {'fromNode': '4072.0', 'toNode': '4168.0', 'Resistance': 0.00284, 'Reactance': 0.01981, 'Conductance': 0, 'Susceptance': 0.00324, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '665'},
        {'fromNode': '4072.0', 'toNode': '4168.0', 'Resistance': 0.00284, 'Reactance': 0.01981, 'Conductance': 0, 'Susceptance': 0.00324, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '666'},
        {'fromNode': '4074.0', 'toNode': '4073.0', 'Resistance': 0.00046, 'Reactance': 0.01717, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 416.85, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '667'},
        {'fromNode': '4074.0', 'toNode': '4073.0', 'Resistance': 0.00046, 'Reactance': 0.01717, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 416.85, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '668'},
        {'fromNode': '4075.0', 'toNode': '4073.0', 'Resistance': 0.0015, 'Reactance': 0.06318, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '669'},
        {'fromNode': '4076.0', 'toNode': '4073.0', 'Resistance': 0.0015, 'Reactance': 0.05994, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '670'},
        {'fromNode': '4077.0', 'toNode': '4073.0', 'Resistance': 0.00135, 'Reactance': 0.06817, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '671'},
        {'fromNode': '4146.0', 'toNode': '4073.0', 'Resistance': 0.006, 'Reactance': 0.06, 'Conductance': 0, 'Susceptance': 0.115, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '672'},
        {'fromNode': '4073.0', 'toNode': '4183.0', 'Resistance': 0.01015, 'Reactance': 0.07248, 'Conductance': 0, 'Susceptance': 0.13214, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '673'},
        {'fromNode': '4192.0', 'toNode': '4073.0', 'Resistance': 0.0062, 'Reactance': 0.06449, 'Conductance': 0, 'Susceptance': 0.21975, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '674'},
        {'fromNode': '4155.0', 'toNode': '4074.0', 'Resistance': 0.01013, 'Reactance': 0.05786, 'Conductance': 0, 'Susceptance': 0.00869, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '675'},
        {'fromNode': '4120.0', 'toNode': '4078.0', 'Resistance': 0.00325, 'Reactance': 0.01949, 'Conductance': 0, 'Susceptance': 0.00265, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '676'},
        {'fromNode': '4078.0', 'toNode': '4180.0', 'Resistance': 0.01569, 'Reactance': 0.08959, 'Conductance': 0, 'Susceptance': 0.01346, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '677'},
        {'fromNode': '4079.0', 'toNode': '4157.0', 'Resistance': 0.00338, 'Reactance': 0.02308, 'Conductance': 0, 'Susceptance': 0.00393, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '678'},
        {'fromNode': '4079.0', 'toNode': '4157.0', 'Resistance': 0.00338, 'Reactance': 0.02308, 'Conductance': 0, 'Susceptance': 0.00393, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '679'},
        {'fromNode': '4081.0', 'toNode': '4080.0', 'Resistance': 0.00112, 'Reactance': 0.06024, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 171.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '680'},
        {'fromNode': '4081.0', 'toNode': '4080.0', 'Resistance': 0.00112, 'Reactance': 0.06024, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 171.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '681'},
        {'fromNode': '4082.0', 'toNode': '4080.0', 'Resistance': 0.00121, 'Reactance': 0.05053, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 342.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '682'},
        {'fromNode': '4083.0', 'toNode': '4080.0', 'Resistance': 0.00111, 'Reactance': 0.04655, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 342.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '683'},
        {'fromNode': '4084.0', 'toNode': '4080.0', 'Resistance': 0.00039, 'Reactance': 0.02538, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 338.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '684'},
        {'fromNode': '4085.0', 'toNode': '4080.0', 'Resistance': 0.00054, 'Reactance': 0.02176, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '685'},
        {'fromNode': '4140.0', 'toNode': '4080.0', 'Resistance': 0.00421, 'Reactance': 0.04377, 'Conductance': 0, 'Susceptance': 0.14917, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '686'},
        {'fromNode': '4080.0', 'toNode': '4151.0', 'Resistance': 0.00484, 'Reactance': 0.05033, 'Conductance': 0, 'Susceptance': 0.17151, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '687'},
        {'fromNode': '4080.0', 'toNode': '4181.0', 'Resistance': 0.00409, 'Reactance': 0.03994, 'Conductance': 0, 'Susceptance': 0.07686, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '688'},
        {'fromNode': '4183.0', 'toNode': '4080.0', 'Resistance': 0.002, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.07, 'MVA_rating': 820.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '689'},
        {'fromNode': '4086.0', 'toNode': '4106.0', 'Resistance': 0.02192, 'Reactance': 0.09592, 'Conductance': 0, 'Susceptance': 0.01461, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '690'},
        {'fromNode': '4110.0', 'toNode': '4086.0', 'Resistance': 0.01641, 'Reactance': 0.07179, 'Conductance': 0, 'Susceptance': 0.01093, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '691'},
        {'fromNode': '4156.0', 'toNode': '4087.0', 'Resistance': 0.00563, 'Reactance': 0.02463, 'Conductance': 0, 'Susceptance': 0.00375, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '692'},
        {'fromNode': '4090.0', 'toNode': '4089.0', 'Resistance': 0.01451, 'Reactance': 0.58764, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 28.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '693'},
        {'fromNode': '4091.0', 'toNode': '4089.0', 'Resistance': 0.00427, 'Reactance': 0.20036, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 37.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '694'},
        {'fromNode': '4092.0', 'toNode': '4089.0', 'Resistance': 0.0034, 'Reactance': 0.12539, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 48.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '695'},
        {'fromNode': '4093.0', 'toNode': '4089.0', 'Resistance': 0.01405, 'Reactance': 0.24296, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '696'},
        {'fromNode': '4094.0', 'toNode': '4089.0', 'Resistance': 0.00238, 'Reactance': 0.09215, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '697'},
        {'fromNode': '4095.0', 'toNode': '4089.0', 'Resistance': 0.0077, 'Reactance': 0.18665, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '698'},
        {'fromNode': '4096.0', 'toNode': '4089.0', 'Resistance': 0.01248, 'Reactance': 0.29442, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '699'},
        {'fromNode': '4097.0', 'toNode': '4089.0', 'Resistance': 0.00925, 'Reactance': 0.21151, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '700'},
        {'fromNode': '4098.0', 'toNode': '4089.0', 'Resistance': 0.01041, 'Reactance': 0.22475, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '701'},
        {'fromNode': '4089.0', 'toNode': '4194.0', 'Resistance': 0.01637, 'Reactance': 0.06793, 'Conductance': 0, 'Susceptance': 0.01153, 'MVA_rating': 166.88, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '702'},
        {'fromNode': '4169.0', 'toNode': '4099.0', 'Resistance': 0.00915, 'Reactance': 0.0625, 'Conductance': 0, 'Susceptance': 0.01063, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '703'},
        {'fromNode': '4169.0', 'toNode': '4099.0', 'Resistance': 0.00915, 'Reactance': 0.0625, 'Conductance': 0, 'Susceptance': 0.01063, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '704'},
        {'fromNode': '4100.0', 'toNode': '4120.0', 'Resistance': 0.00186, 'Reactance': 0.00773, 'Conductance': 0, 'Susceptance': 0.00131, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '705'},
        {'fromNode': '4150.0', 'toNode': '4100.0', 'Resistance': 0.00216, 'Reactance': 0.01393, 'Conductance': 0, 'Susceptance': 0.00266, 'MVA_rating': 232.05, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '706'},
        {'fromNode': '4150.0', 'toNode': '4100.0', 'Resistance': 0.00216, 'Reactance': 0.01393, 'Conductance': 0, 'Susceptance': 0.00266, 'MVA_rating': 232.05, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '707'},
        {'fromNode': '4168.0', 'toNode': '4100.0', 'Resistance': 0.00185, 'Reactance': 0.01287, 'Conductance': 0, 'Susceptance': 0.0021, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '708'},
        {'fromNode': '4168.0', 'toNode': '4100.0', 'Resistance': 0.00185, 'Reactance': 0.01287, 'Conductance': 0, 'Susceptance': 0.0021, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '709'},
        {'fromNode': '4168.0', 'toNode': '4100.0', 'Resistance': 0.00185, 'Reactance': 0.01287, 'Conductance': 0, 'Susceptance': 0.0021, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '710'},
        {'fromNode': '4175.0', 'toNode': '4101.0', 'Resistance': 0.01931, 'Reactance': 0.08013, 'Conductance': 0, 'Susceptance': 0.0136, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '711'},
        {'fromNode': '4197.0', 'toNode': '4101.0', 'Resistance': 0.02022, 'Reactance': 0.09011, 'Conductance': 0, 'Susceptance': 0.01321, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '712'},
        {'fromNode': '4126.0', 'toNode': '4102.0', 'Resistance': 0.00411, 'Reactance': 0.02652, 'Conductance': 0, 'Susceptance': 0.00506, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '713'},
        {'fromNode': '4126.0', 'toNode': '4102.0', 'Resistance': 0.00411, 'Reactance': 0.02652, 'Conductance': 0, 'Susceptance': 0.00506, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '714'},
        {'fromNode': '4102.0', 'toNode': '4171.0', 'Resistance': 0.00634, 'Reactance': 0.04334, 'Conductance': 0, 'Susceptance': 0.00737, 'MVA_rating': 280.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '715'},
        {'fromNode': '4102.0', 'toNode': '4171.0', 'Resistance': 0.00634, 'Reactance': 0.04334, 'Conductance': 0, 'Susceptance': 0.00737, 'MVA_rating': 280.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '716'},
        {'fromNode': '4105.0', 'toNode': '4104.0', 'Resistance': 0.00385, 'Reactance': 0.08871, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.36, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '717'},
        {'fromNode': '4105.0', 'toNode': '4104.0', 'Resistance': 0.00385, 'Reactance': 0.08871, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.36, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '718'},
        {'fromNode': '4145.0', 'toNode': '4104.0', 'Resistance': 0.01294, 'Reactance': 0.03109, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '719'},
        {'fromNode': '4170.0', 'toNode': '4108.0', 'Resistance': 0.00818, 'Reactance': 0.0358, 'Conductance': 0, 'Susceptance': 0.00545, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '720'},
        {'fromNode': '4109.0', 'toNode': '4156.0', 'Resistance': 0.00373, 'Reactance': 0.02127, 'Conductance': 0, 'Susceptance': 0.0032, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '721'},
        {'fromNode': '4110.0', 'toNode': '4171.0', 'Resistance': 0.02886, 'Reactance': 0.11974, 'Conductance': 0, 'Susceptance': 0.02032, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '722'},
        {'fromNode': '4186.0', 'toNode': '4111.0', 'Resistance': 0.00861, 'Reactance': 0.0384, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '723'},
        {'fromNode': '4124.0', 'toNode': '4112.0', 'Resistance': 0.01932, 'Reactance': 0.13196, 'Conductance': 0, 'Susceptance': 0.02245, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '724'},
        {'fromNode': '4124.0', 'toNode': '4112.0', 'Resistance': 0.01932, 'Reactance': 0.13196, 'Conductance': 0, 'Susceptance': 0.02245, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '725'},
        {'fromNode': '4112.0', 'toNode': '4141.0', 'Resistance': 0.0066, 'Reactance': 0.04599, 'Conductance': 0, 'Susceptance': 0.00752, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '726'},
        {'fromNode': '4112.0', 'toNode': '4141.0', 'Resistance': 0.0066, 'Reactance': 0.04599, 'Conductance': 0, 'Susceptance': 0.00752, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '727'},
        {'fromNode': '4114.0', 'toNode': '4113.0', 'Resistance': 0.00072, 'Reactance': 0.01874, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 387.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '728'},
        {'fromNode': '4115.0', 'toNode': '4113.0', 'Resistance': 0.00101, 'Reactance': 0.05405, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 180.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '729'},
        {'fromNode': '4116.0', 'toNode': '4113.0', 'Resistance': 0.00162, 'Reactance': 0.05686, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 180.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '730'},
        {'fromNode': '4146.0', 'toNode': '4113.0', 'Resistance': 0.0025, 'Reactance': 0.02448, 'Conductance': 0, 'Susceptance': 0.04711, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '731'},
        {'fromNode': '4190.0', 'toNode': '4114.0', 'Resistance': 0.00237, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '732'},
        {'fromNode': '4190.0', 'toNode': '4114.0', 'Resistance': 0.00237, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '733'},
        {'fromNode': '4117.0', 'toNode': '4172.0', 'Resistance': 0.01489, 'Reactance': 0.06176, 'Conductance': 0, 'Susceptance': 0.01048, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '734'},
        {'fromNode': '4139.0', 'toNode': '4118.0', 'Resistance': 0.0147, 'Reactance': 0.08395, 'Conductance': 0, 'Susceptance': 0.01262, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '735'},
        {'fromNode': '4148.0', 'toNode': '4118.0', 'Resistance': 0.0246, 'Reactance': 0.14051, 'Conductance': 0, 'Susceptance': 0.02111, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '736'},
        {'fromNode': '4164.0', 'toNode': '4119.0', 'Resistance': 0.02453, 'Reactance': 0.1676, 'Conductance': 0, 'Susceptance': 0.02851, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '737'},
        {'fromNode': '4122.0', 'toNode': '4157.0', 'Resistance': 0.00313, 'Reactance': 0.02183, 'Conductance': 0, 'Susceptance': 0.00357, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '738'},
        {'fromNode': '4122.0', 'toNode': '4157.0', 'Resistance': 0.00313, 'Reactance': 0.02183, 'Conductance': 0, 'Susceptance': 0.00357, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '739'},
        {'fromNode': '4123.0', 'toNode': '4180.0', 'Resistance': 0.01637, 'Reactance': 0.09826, 'Conductance': 0, 'Susceptance': 0.01333, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '740'},
        {'fromNode': '4128.0', 'toNode': '4124.0', 'Resistance': 0.02044, 'Reactance': 0.13201, 'Conductance': 0, 'Susceptance': 0.02518, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '741'},
        {'fromNode': '4128.0', 'toNode': '4124.0', 'Resistance': 0.02044, 'Reactance': 0.13201, 'Conductance': 0, 'Susceptance': 0.02518, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '742'},
        {'fromNode': '6264.0', 'toNode': '4125.0', 'Resistance': 0.03218, 'Reactance': 0.07865, 'Conductance': 0, 'Susceptance': 0.0103, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '743'},
        {'fromNode': '4193.0', 'toNode': '4126.0', 'Resistance': 0.00522, 'Reactance': 0.0364, 'Conductance': 0, 'Susceptance': 0.00595, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '744'},
        {'fromNode': '4193.0', 'toNode': '4126.0', 'Resistance': 0.00522, 'Reactance': 0.0364, 'Conductance': 0, 'Susceptance': 0.00595, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '745'},
        {'fromNode': '4193.0', 'toNode': '4126.0', 'Resistance': 0.00522, 'Reactance': 0.0364, 'Conductance': 0, 'Susceptance': 0.00595, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '746'},
        {'fromNode': '4144.0', 'toNode': '4127.0', 'Resistance': 0.02101, 'Reactance': 0.12616, 'Conductance': 0, 'Susceptance': 0.01712, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '747'},
        {'fromNode': '4129.0', 'toNode': '4130.0', 'Resistance': 0.03748, 'Reactance': 0.09002, 'Conductance': 0, 'Susceptance': 0.01222, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '748'},
        {'fromNode': '4129.0', 'toNode': '4176.0', 'Resistance': 0.02359, 'Reactance': 0.14166, 'Conductance': 0, 'Susceptance': 0.01922, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '749'},
        {'fromNode': '4130.0', 'toNode': '4179.0', 'Resistance': 0.01386, 'Reactance': 0.07916, 'Conductance': 0, 'Susceptance': 0.0119, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '750'},
        {'fromNode': '4132.0', 'toNode': '4131.0', 'Resistance': 0.00028, 'Reactance': 0.01679, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 246.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '751'},
        {'fromNode': '4132.0', 'toNode': '4131.0', 'Resistance': 0.00028, 'Reactance': 0.01679, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 246.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '752'},
        {'fromNode': '4133.0', 'toNode': '4131.0', 'Resistance': 0.00199, 'Reactance': 0.07159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 101.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '753'},
        {'fromNode': '4134.0', 'toNode': '4131.0', 'Resistance': 0.00198, 'Reactance': 0.14741, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 103.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '754'},
        {'fromNode': '4135.0', 'toNode': '4131.0', 'Resistance': 0.00314, 'Reactance': 0.14014, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 103.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '755'},
        {'fromNode': '4136.0', 'toNode': '4131.0', 'Resistance': 0.00315, 'Reactance': 0.18409, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 105.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '756'},
        {'fromNode': '4137.0', 'toNode': '4131.0', 'Resistance': 0.00218, 'Reactance': 0.08044, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 140.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '757'},
        {'fromNode': '4138.0', 'toNode': '4131.0', 'Resistance': 0.00058, 'Reactance': 0.03762, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 135.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '758'},
        {'fromNode': '4131.0', 'toNode': '6308.0', 'Resistance': 0.00739, 'Reactance': 0.05281, 'Conductance': 0, 'Susceptance': 0.09628, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '759'},
        {'fromNode': '4132.0', 'toNode': '4143.0', 'Resistance': 0.02006, 'Reactance': 0.08324, 'Conductance': 0, 'Susceptance': 0.01413, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '760'},
        {'fromNode': '6357.0', 'toNode': '4132.0', 'Resistance': 0.01827, 'Reactance': 0.07994, 'Conductance': 0, 'Susceptance': 0.01217, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '761'},
        {'fromNode': '4139.0', 'toNode': '4144.0', 'Resistance': 0.00882, 'Reactance': 0.05038, 'Conductance': 0, 'Susceptance': 0.00757, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '762'},
        {'fromNode': '4139.0', 'toNode': '4169.0', 'Resistance': 0.01144, 'Reactance': 0.05002, 'Conductance': 0, 'Susceptance': 0.00762, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '763'},
        {'fromNode': '4141.0', 'toNode': '4140.0', 'Resistance': 0.00072, 'Reactance': 0.03223, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 428.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '764'},
        {'fromNode': '4142.0', 'toNode': '4140.0', 'Resistance': 0.00054, 'Reactance': 0.02789, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 168.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '765'},
        {'fromNode': '4140.0', 'toNode': '4151.0', 'Resistance': 0.00411, 'Reactance': 0.02936, 'Conductance': 0, 'Susceptance': 0.05353, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '766'},
        {'fromNode': '4140.0', 'toNode': '4196.0', 'Resistance': 0.01639, 'Reactance': 0.11706, 'Conductance': 0, 'Susceptance': 0.21342, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '767'},
        {'fromNode': '4141.0', 'toNode': '4165.0', 'Resistance': 0.01975, 'Reactance': 0.08194, 'Conductance': 0, 'Susceptance': 0.0139, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '768'},
        {'fromNode': '4189.0', 'toNode': '4143.0', 'Resistance': 0.02761, 'Reactance': 0.11455, 'Conductance': 0, 'Susceptance': 0.01944, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '769'},
        {'fromNode': '4147.0', 'toNode': '4146.0', 'Resistance': 0.0002, 'Reactance': 0.0123, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '770'},
        {'fromNode': '4146.0', 'toNode': '4196.0', 'Resistance': 0.00241, 'Reactance': 0.02506, 'Conductance': 0, 'Susceptance': 0.0854, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '771'},
        {'fromNode': '4146.0', 'toNode': '4196.0', 'Resistance': 0.00241, 'Reactance': 0.02506, 'Conductance': 0, 'Susceptance': 0.0854, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '772'},
        {'fromNode': '4148.0', 'toNode': '4179.0', 'Resistance': 0.03149, 'Reactance': 0.1891, 'Conductance': 0, 'Susceptance': 0.02566, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '773'},
        {'fromNode': '4173.0', 'toNode': '4149.0', 'Resistance': 0.02612, 'Reactance': 0.11641, 'Conductance': 0, 'Susceptance': 0.01707, 'MVA_rating': 170.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '774'},
        {'fromNode': '4149.0', 'toNode': '4182.0', 'Resistance': 0.00774, 'Reactance': 0.05286, 'Conductance': 0, 'Susceptance': 0.00899, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '775'},
        {'fromNode': '4152.0', 'toNode': '4151.0', 'Resistance': 0.00143, 'Reactance': 0.08362, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 81.9, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '776'},
        {'fromNode': '4152.0', 'toNode': '4151.0', 'Resistance': 0.00143, 'Reactance': 0.08362, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 81.9, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '777'},
        {'fromNode': '4153.0', 'toNode': '4151.0', 'Resistance': 0.0006, 'Reactance': 0.01612, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 381.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '778'},
        {'fromNode': '4151.0', 'toNode': '4158.0', 'Resistance': 0.00609, 'Reactance': 0.06334, 'Conductance': 0, 'Susceptance': 0.21583, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '779'},
        {'fromNode': '4164.0', 'toNode': '4155.0', 'Resistance': 0.00412, 'Reactance': 0.02661, 'Conductance': 0, 'Susceptance': 0.00508, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '780'},
        {'fromNode': '4159.0', 'toNode': '4158.0', 'Resistance': 0.0006, 'Reactance': 0.02511, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 620.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '781'},
        {'fromNode': '4159.0', 'toNode': '4158.0', 'Resistance': 0.0006, 'Reactance': 0.02511, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 620.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '782'},
        {'fromNode': '4160.0', 'toNode': '4158.0', 'Resistance': 0.00047, 'Reactance': 0.02419, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 298.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '783'},
        {'fromNode': '4161.0', 'toNode': '4158.0', 'Resistance': 0.00081, 'Reactance': 0.05928, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 296.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '784'},
        {'fromNode': '4162.0', 'toNode': '4158.0', 'Resistance': 0.00201, 'Reactance': 0.05867, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 293.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '785'},
        {'fromNode': '4163.0', 'toNode': '4158.0', 'Resistance': 0.0037, 'Reactance': 0.15066, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 74.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '786'},
        {'fromNode': '4159.0', 'toNode': '4168.0', 'Resistance': 0.00057, 'Reactance': 0.00394, 'Conductance': 0, 'Susceptance': 0.00064, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '787'},
        {'fromNode': '4159.0', 'toNode': '4168.0', 'Resistance': 0.00057, 'Reactance': 0.00394, 'Conductance': 0, 'Susceptance': 0.00064, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '788'},
        {'fromNode': '4159.0', 'toNode': '4168.0', 'Resistance': 0.00057, 'Reactance': 0.00394, 'Conductance': 0, 'Susceptance': 0.00064, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '789'},
        {'fromNode': '4159.0', 'toNode': '4168.0', 'Resistance': 0.00057, 'Reactance': 0.00394, 'Conductance': 0, 'Susceptance': 0.00064, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '790'},
        {'fromNode': '4166.0', 'toNode': '4165.0', 'Resistance': 0.00217, 'Reactance': 0.09206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 198.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '791'},
        {'fromNode': '4166.0', 'toNode': '4165.0', 'Resistance': 0.00217, 'Reactance': 0.09206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 198.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '792'},
        {'fromNode': '4186.0', 'toNode': '4167.0', 'Resistance': 0.01453, 'Reactance': 0.06359, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '793'},
        {'fromNode': '4180.0', 'toNode': '4170.0', 'Resistance': 0.00815, 'Reactance': 0.03567, 'Conductance': 0, 'Susceptance': 0.00543, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '794'},
        {'fromNode': '6221.0', 'toNode': '4172.0', 'Resistance': 0.01492, 'Reactance': 0.06529, 'Conductance': 0, 'Susceptance': 0.00994, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '795'},
        {'fromNode': '4172.0', 'toNode': '6346.0', 'Resistance': 0.01189, 'Reactance': 0.05202, 'Conductance': 0, 'Susceptance': 0.00792, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '796'},
        {'fromNode': '7030.0', 'toNode': '4173.0', 'Resistance': 0.02382, 'Reactance': 0.10423, 'Conductance': 0, 'Susceptance': 0.01587, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '797'},
        {'fromNode': '6015.0', 'toNode': '4174.0', 'Resistance': 0.00722, 'Reactance': 0.02995, 'Conductance': 0, 'Susceptance': 0.00508, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '798'},
        {'fromNode': '6276.0', 'toNode': '4174.0', 'Resistance': 0.01173, 'Reactance': 0.04865, 'Conductance': 0, 'Susceptance': 0.00826, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '799'},
        {'fromNode': '4176.0', 'toNode': '4175.0', 'Resistance': 0.02194, 'Reactance': 0.09598, 'Conductance': 0, 'Susceptance': 0.01462, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '800'},
        {'fromNode': '4177.0', 'toNode': '4176.0', 'Resistance': 0.0053, 'Reactance': 0.11274, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '801'},
        {'fromNode': '4177.0', 'toNode': '4176.0', 'Resistance': 0.0053, 'Reactance': 0.11274, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '802'},
        {'fromNode': '4178.0', 'toNode': '6253.0', 'Resistance': 0.02656, 'Reactance': 0.11618, 'Conductance': 0, 'Susceptance': 0.01769, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '803'},
        {'fromNode': '4182.0', 'toNode': '4181.0', 'Resistance': 0.00321, 'Reactance': 0.09306, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 169.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '804'},
        {'fromNode': '7274.0', 'toNode': '4181.0', 'Resistance': 0.01011, 'Reactance': 0.07225, 'Conductance': 0, 'Susceptance': 0.13172, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '805'},
        {'fromNode': '4184.0', 'toNode': '4183.0', 'Resistance': 0.00113, 'Reactance': 0.06134, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '806'},
        {'fromNode': '4191.0', 'toNode': '4184.0', 'Resistance': 0.01223, 'Reactance': 0.08358, 'Conductance': 0, 'Susceptance': 0.01422, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '807'},
        {'fromNode': '4191.0', 'toNode': '4184.0', 'Resistance': 0.01223, 'Reactance': 0.08358, 'Conductance': 0, 'Susceptance': 0.01422, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '808'},
        {'fromNode': '4184.0', 'toNode': '4194.0', 'Resistance': 0.00649, 'Reactance': 0.04522, 'Conductance': 0, 'Susceptance': 0.00739, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '809'},
        {'fromNode': '4184.0', 'toNode': '4194.0', 'Resistance': 0.00649, 'Reactance': 0.04522, 'Conductance': 0, 'Susceptance': 0.00739, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '810'},
        {'fromNode': '4187.0', 'toNode': '4189.0', 'Resistance': 0.02899, 'Reactance': 0.12682, 'Conductance': 0, 'Susceptance': 0.01931, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '811'},
        {'fromNode': '4193.0', 'toNode': '4192.0', 'Resistance': 0.00036, 'Reactance': 0.02396, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '812'},
        {'fromNode': '4193.0', 'toNode': '4192.0', 'Resistance': 0.00036, 'Reactance': 0.02396, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '813'},
        {'fromNode': '4193.0', 'toNode': '4192.0', 'Resistance': 0.00036, 'Reactance': 0.02396, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '814'},
        {'fromNode': '4196.0', 'toNode': '4195.0', 'Resistance': 8e-05, 'Reactance': 0.00576, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1233.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '815'},
        {'fromNode': '4195.0', 'toNode': '6239.0', 'Resistance': 0.00265, 'Reactance': 0.0321, 'Conductance': 0, 'Susceptance': 2.13231, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '816'},
        {'fromNode': '4197.0', 'toNode': '4196.0', 'Resistance': 0.00072, 'Reactance': 0.03236, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '817'},
        {'fromNode': '5003.0', 'toNode': '5181.0', 'Resistance': 0.00233, 'Reactance': 0.01026, 'Conductance': 0, 'Susceptance': 0.00478, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '818'},
        {'fromNode': '5213.0', 'toNode': '5003.0', 'Resistance': 0.00127, 'Reactance': 0.0072, 'Conductance': 0, 'Susceptance': 0.00387, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '819'},
        {'fromNode': '5003.0', 'toNode': '5436.0', 'Resistance': 0.00455, 'Reactance': 0.01936, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '820'},
        {'fromNode': '5004.0', 'toNode': '5426.0', 'Resistance': 0.00757, 'Reactance': 0.04276, 'Conductance': 0, 'Susceptance': 0.023, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '821'},
        {'fromNode': '5476.0', 'toNode': '5004.0', 'Resistance': 0.004, 'Reactance': 0.0226, 'Conductance': 0, 'Susceptance': 0.01216, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '822'},
        {'fromNode': '5133.0', 'toNode': '5005.0', 'Resistance': 0.00846, 'Reactance': 0.03598, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '823'},
        {'fromNode': '5005.0', 'toNode': '5435.0', 'Resistance': 0.00739, 'Reactance': 0.03256, 'Conductance': 0, 'Susceptance': 0.01515, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '824'},
        {'fromNode': '5006.0', 'toNode': '5072.0', 'Resistance': 0.00074, 'Reactance': 0.00543, 'Conductance': 0, 'Susceptance': 0.00308, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '825'},
        {'fromNode': '5006.0', 'toNode': '5072.0', 'Resistance': 0.00074, 'Reactance': 0.00543, 'Conductance': 0, 'Susceptance': 0.00308, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '826'},
        {'fromNode': '5006.0', 'toNode': '5072.0', 'Resistance': 0.00074, 'Reactance': 0.00543, 'Conductance': 0, 'Susceptance': 0.00308, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '827'},
        {'fromNode': '5289.0', 'toNode': '5006.0', 'Resistance': 0.00065, 'Reactance': 0.00498, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '828'},
        {'fromNode': '5289.0', 'toNode': '5006.0', 'Resistance': 0.00065, 'Reactance': 0.00498, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '829'},
        {'fromNode': '5006.0', 'toNode': '5294.0', 'Resistance': 0.00495, 'Reactance': 0.02107, 'Conductance': 0, 'Susceptance': 0.01054, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '830'},
        {'fromNode': '5007.0', 'toNode': '5012.0', 'Resistance': 0.0011, 'Reactance': 0.00645, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '831'},
        {'fromNode': '5165.0', 'toNode': '5007.0', 'Resistance': 0.00088, 'Reactance': 0.00648, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '832'},
        {'fromNode': '5134.0', 'toNode': '5008.0', 'Resistance': 0.01009, 'Reactance': 0.05706, 'Conductance': 0, 'Susceptance': 0.03069, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '833'},
        {'fromNode': '5347.0', 'toNode': '5008.0', 'Resistance': 0.01147, 'Reactance': 0.05055, 'Conductance': 0, 'Susceptance': 0.02353, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '834'},
        {'fromNode': '5396.0', 'toNode': '5008.0', 'Resistance': 0.00397, 'Reactance': 0.02909, 'Conductance': 0, 'Susceptance': 0.01648, 'MVA_rating': 400.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '835'},
        {'fromNode': '5294.0', 'toNode': '5009.0', 'Resistance': 0.00175, 'Reactance': 0.00769, 'Conductance': 0, 'Susceptance': 0.00358, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '836'},
        {'fromNode': '5311.0', 'toNode': '5009.0', 'Resistance': 0.0016, 'Reactance': 0.00704, 'Conductance': 0, 'Susceptance': 0.00328, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '837'},
        {'fromNode': '5482.0', 'toNode': '5009.0', 'Resistance': 0.00139, 'Reactance': 0.0061, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '838'},
        {'fromNode': '5010.0', 'toNode': '5096.0', 'Resistance': 0.00127, 'Reactance': 0.00969, 'Conductance': 0, 'Susceptance': 0.00509, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '839'},
        {'fromNode': '5010.0', 'toNode': '5096.0', 'Resistance': 0.00127, 'Reactance': 0.00969, 'Conductance': 0, 'Susceptance': 0.00509, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '840'},
        {'fromNode': '5386.0', 'toNode': '5010.0', 'Resistance': 0.00106, 'Reactance': 0.00803, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '841'},
        {'fromNode': '5386.0', 'toNode': '5010.0', 'Resistance': 0.00106, 'Reactance': 0.00803, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '842'},
        {'fromNode': '5011.0', 'toNode': '5054.0', 'Resistance': 0.00746, 'Reactance': 0.04371, 'Conductance': 0, 'Susceptance': 0.02183, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '843'},
        {'fromNode': '8084.0', 'toNode': '5011.0', 'Resistance': 0.00545, 'Reactance': 0.03193, 'Conductance': 0, 'Susceptance': 0.01595, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '844'},
        {'fromNode': '5012.0', 'toNode': '5017.0', 'Resistance': 0.00663, 'Reactance': 0.0292, 'Conductance': 0, 'Susceptance': 0.01359, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '845'},
        {'fromNode': '5013.0', 'toNode': '5101.0', 'Resistance': 0.00798, 'Reactance': 0.03394, 'Conductance': 0, 'Susceptance': 0.01697, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '846'},
        {'fromNode': '5156.0', 'toNode': '5013.0', 'Resistance': 0.00645, 'Reactance': 0.02841, 'Conductance': 0, 'Susceptance': 0.01322, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '847'},
        {'fromNode': '5013.0', 'toNode': '5226.0', 'Resistance': 0.00745, 'Reactance': 0.03283, 'Conductance': 0, 'Susceptance': 0.01528, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '848'},
        {'fromNode': '5014.0', 'toNode': '5455.0', 'Resistance': 0.00556, 'Reactance': 0.03257, 'Conductance': 0, 'Susceptance': 0.01626, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '849'},
        {'fromNode': '5016.0', 'toNode': '5015.0', 'Resistance': 0.00015, 'Reactance': 0.01417, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1206.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '850'},
        {'fromNode': '5016.0', 'toNode': '5015.0', 'Resistance': 0.00015, 'Reactance': 0.01417, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1206.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '851'},
        {'fromNode': '5063.0', 'toNode': '5015.0', 'Resistance': 0.00028, 'Reactance': 0.00336, 'Conductance': 0, 'Susceptance': 0.22327, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '852'},
        {'fromNode': '5015.0', 'toNode': '5164.0', 'Resistance': 0.00044, 'Reactance': 0.00534, 'Conductance': 0, 'Susceptance': 0.35461, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '853'},
        {'fromNode': '5361.0', 'toNode': '5015.0', 'Resistance': 0.00044, 'Reactance': 0.0053, 'Conductance': 0, 'Susceptance': 0.3546, 'MVA_rating': 4352.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '854'},
        {'fromNode': '5015.0', 'toNode': '5479.0', 'Resistance': 0.00057, 'Reactance': 0.00686, 'Conductance': 0, 'Susceptance': 0.45585, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '855'},
        {'fromNode': '5265.0', 'toNode': '5016.0', 'Resistance': 0.00128, 'Reactance': 0.00972, 'Conductance': 0, 'Susceptance': 0.00511, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '856'},
        {'fromNode': '5265.0', 'toNode': '5016.0', 'Resistance': 0.00128, 'Reactance': 0.00972, 'Conductance': 0, 'Susceptance': 0.00511, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '857'},
        {'fromNode': '5265.0', 'toNode': '5016.0', 'Resistance': 0.00128, 'Reactance': 0.00972, 'Conductance': 0, 'Susceptance': 0.00511, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '858'},
        {'fromNode': '5420.0', 'toNode': '5016.0', 'Resistance': 0.00135, 'Reactance': 0.01028, 'Conductance': 0, 'Susceptance': 0.0054, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '859'},
        {'fromNode': '5420.0', 'toNode': '5016.0', 'Resistance': 0.00135, 'Reactance': 0.01028, 'Conductance': 0, 'Susceptance': 0.0054, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '860'},
        {'fromNode': '5218.0', 'toNode': '5017.0', 'Resistance': 0.0048, 'Reactance': 0.02041, 'Conductance': 0, 'Susceptance': 0.01021, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '861'},
        {'fromNode': '5017.0', 'toNode': '5315.0', 'Resistance': 0.00523, 'Reactance': 0.02225, 'Conductance': 0, 'Susceptance': 0.01113, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '862'},
        {'fromNode': '5019.0', 'toNode': '5018.0', 'Resistance': 0.00058, 'Reactance': 0.02598, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 590.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '863'},
        {'fromNode': '5020.0', 'toNode': '5018.0', 'Resistance': 0.00032, 'Reactance': 0.03326, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 557.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '864'},
        {'fromNode': '5055.0', 'toNode': '5018.0', 'Resistance': 0.00034, 'Reactance': 0.00414, 'Conductance': 0, 'Susceptance': 0.27523, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '865'},
        {'fromNode': '5018.0', 'toNode': '5236.0', 'Resistance': 0.00037, 'Reactance': 0.00753, 'Conductance': 0, 'Susceptance': 0.52503, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '866'},
        {'fromNode': '5018.0', 'toNode': '5236.0', 'Resistance': 0.00037, 'Reactance': 0.00753, 'Conductance': 0, 'Susceptance': 0.52503, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '867'},
        {'fromNode': '5018.0', 'toNode': '5413.0', 'Resistance': 0.00034, 'Reactance': 0.00695, 'Conductance': 0, 'Susceptance': 0.48479, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '868'},
        {'fromNode': '5019.0', 'toNode': '5176.0', 'Resistance': 0.00561, 'Reactance': 0.03288, 'Conductance': 0, 'Susceptance': 0.01642, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '869'},
        {'fromNode': '5022.0', 'toNode': '5021.0', 'Resistance': 0.0001, 'Reactance': 0.01005, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1375.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '870'},
        {'fromNode': '5022.0', 'toNode': '5021.0', 'Resistance': 0.0001, 'Reactance': 0.01005, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1375.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '871'},
        {'fromNode': '5023.0', 'toNode': '5021.0', 'Resistance': 0.00028, 'Reactance': 0.02693, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 245.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '872'},
        {'fromNode': '5021.0', 'toNode': '5401.0', 'Resistance': 0.00029, 'Reactance': 0.006, 'Conductance': 0, 'Susceptance': 0.41836, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '873'},
        {'fromNode': '5021.0', 'toNode': '5413.0', 'Resistance': 0.00056, 'Reactance': 0.01139, 'Conductance': 0, 'Susceptance': 0.79467, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '874'},
        {'fromNode': '5479.0', 'toNode': '5021.0', 'Resistance': 0.00021, 'Reactance': 0.00435, 'Conductance': 0, 'Susceptance': 0.30317, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '875'},
        {'fromNode': '5249.0', 'toNode': '5022.0', 'Resistance': 0.00173, 'Reactance': 0.01314, 'Conductance': 0, 'Susceptance': 0.0069, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '876'},
        {'fromNode': '5249.0', 'toNode': '5022.0', 'Resistance': 0.00173, 'Reactance': 0.01314, 'Conductance': 0, 'Susceptance': 0.0069, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '877'},
        {'fromNode': '5306.0', 'toNode': '5022.0', 'Resistance': 0.00263, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0105, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '878'},
        {'fromNode': '5306.0', 'toNode': '5022.0', 'Resistance': 0.00263, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0105, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '879'},
        {'fromNode': '5022.0', 'toNode': '5441.0', 'Resistance': 0.00341, 'Reactance': 0.02594, 'Conductance': 0, 'Susceptance': 0.01362, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '880'},
        {'fromNode': '5022.0', 'toNode': '5441.0', 'Resistance': 0.00341, 'Reactance': 0.02594, 'Conductance': 0, 'Susceptance': 0.01362, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '881'},
        {'fromNode': '5140.0', 'toNode': '5024.0', 'Resistance': 0.00381, 'Reactance': 0.01679, 'Conductance': 0, 'Susceptance': 0.00781, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '882'},
        {'fromNode': '5024.0', 'toNode': '5182.0', 'Resistance': 0.003, 'Reactance': 0.01323, 'Conductance': 0, 'Susceptance': 0.00616, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '883'},
        {'fromNode': '5452.0', 'toNode': '5024.0', 'Resistance': 0.00923, 'Reactance': 0.05217, 'Conductance': 0, 'Susceptance': 0.02806, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '884'},
        {'fromNode': '5453.0', 'toNode': '5025.0', 'Resistance': 0.00495, 'Reactance': 0.03768, 'Conductance': 0, 'Susceptance': 0.01978, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '885'},
        {'fromNode': '5453.0', 'toNode': '5025.0', 'Resistance': 0.00495, 'Reactance': 0.03768, 'Conductance': 0, 'Susceptance': 0.01978, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '886'},
        {'fromNode': '5025.0', 'toNode': '5473.0', 'Resistance': 0.00322, 'Reactance': 0.02357, 'Conductance': 0, 'Susceptance': 0.01336, 'MVA_rating': 325.5, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '887'},
        {'fromNode': '5025.0', 'toNode': '5473.0', 'Resistance': 0.00322, 'Reactance': 0.02357, 'Conductance': 0, 'Susceptance': 0.01336, 'MVA_rating': 325.5, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '888'},
        {'fromNode': '5026.0', 'toNode': '5104.0', 'Resistance': 0.0014, 'Reactance': 0.01063, 'Conductance': 0, 'Susceptance': 0.00558, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '889'},
        {'fromNode': '5026.0', 'toNode': '5104.0', 'Resistance': 0.0014, 'Reactance': 0.01063, 'Conductance': 0, 'Susceptance': 0.00558, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '890'},
        {'fromNode': '5026.0', 'toNode': '5375.0', 'Resistance': 0.00136, 'Reactance': 0.00798, 'Conductance': 0, 'Susceptance': 0.00399, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '891'},
        {'fromNode': '5027.0', 'toNode': '5098.0', 'Resistance': 0.00294, 'Reactance': 0.01659, 'Conductance': 0, 'Susceptance': 0.00893, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '892'},
        {'fromNode': '5027.0', 'toNode': '5469.0', 'Resistance': 0.00157, 'Reactance': 0.01196, 'Conductance': 0, 'Susceptance': 0.00628, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '893'},
        {'fromNode': '5027.0', 'toNode': '5469.0', 'Resistance': 0.00157, 'Reactance': 0.01196, 'Conductance': 0, 'Susceptance': 0.00628, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '894'},
        {'fromNode': '5027.0', 'toNode': '5481.0', 'Resistance': 0.00206, 'Reactance': 0.01564, 'Conductance': 0, 'Susceptance': 0.00821, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '895'},
        {'fromNode': '5097.0', 'toNode': '5028.0', 'Resistance': 0.00096, 'Reactance': 0.00703, 'Conductance': 0, 'Susceptance': 0.00399, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '896'},
        {'fromNode': '5097.0', 'toNode': '5028.0', 'Resistance': 0.00096, 'Reactance': 0.00703, 'Conductance': 0, 'Susceptance': 0.00399, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '897'},
        {'fromNode': '5028.0', 'toNode': '5233.0', 'Resistance': 0.00165, 'Reactance': 0.00966, 'Conductance': 0, 'Susceptance': 0.00482, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '898'},
        {'fromNode': '5028.0', 'toNode': '5478.0', 'Resistance': 0.00394, 'Reactance': 0.01736, 'Conductance': 0, 'Susceptance': 0.00808, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '899'},
        {'fromNode': '5029.0', 'toNode': '5142.0', 'Resistance': 0.00477, 'Reactance': 0.02699, 'Conductance': 0, 'Susceptance': 0.01452, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '900'},
        {'fromNode': '5461.0', 'toNode': '5029.0', 'Resistance': 0.00376, 'Reactance': 0.02202, 'Conductance': 0, 'Susceptance': 0.01099, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '901'},
        {'fromNode': '5312.0', 'toNode': '5030.0', 'Resistance': 0.00423, 'Reactance': 0.02478, 'Conductance': 0, 'Susceptance': 0.01237, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '902'},
        {'fromNode': '5030.0', 'toNode': '5430.0', 'Resistance': 0.00326, 'Reactance': 0.0248, 'Conductance': 0, 'Susceptance': 0.01302, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '903'},
        {'fromNode': '5031.0', 'toNode': '5296.0', 'Resistance': 0.00345, 'Reactance': 0.0202, 'Conductance': 0, 'Susceptance': 0.01009, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '904'},
        {'fromNode': '5408.0', 'toNode': '5031.0', 'Resistance': 0.00513, 'Reactance': 0.02263, 'Conductance': 0, 'Susceptance': 0.01054, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '905'},
        {'fromNode': '5032.0', 'toNode': '5090.0', 'Resistance': 0.00304, 'Reactance': 0.01718, 'Conductance': 0, 'Susceptance': 0.00924, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '906'},
        {'fromNode': '5214.0', 'toNode': '5032.0', 'Resistance': 0.0022, 'Reactance': 0.00969, 'Conductance': 0, 'Susceptance': 0.00451, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '907'},
        {'fromNode': '5032.0', 'toNode': '5228.0', 'Resistance': 0.0023, 'Reactance': 0.01751, 'Conductance': 0, 'Susceptance': 0.00919, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '908'},
        {'fromNode': '5032.0', 'toNode': '5228.0', 'Resistance': 0.0023, 'Reactance': 0.01751, 'Conductance': 0, 'Susceptance': 0.00919, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '909'},
        {'fromNode': '5329.0', 'toNode': '5032.0', 'Resistance': 0.00188, 'Reactance': 0.00829, 'Conductance': 0, 'Susceptance': 0.00386, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '910'},
        {'fromNode': '5034.0', 'toNode': '5033.0', 'Resistance': 9e-05, 'Reactance': 0.00546, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1209.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '911'},
        {'fromNode': '5035.0', 'toNode': '5033.0', 'Resistance': 0.00084, 'Reactance': 0.07958, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 243.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '912'},
        {'fromNode': '5036.0', 'toNode': '5033.0', 'Resistance': 0.00077, 'Reactance': 0.05006, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 255.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '913'},
        {'fromNode': '5037.0', 'toNode': '5033.0', 'Resistance': 0.00015, 'Reactance': 0.01216, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 985.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '914'},
        {'fromNode': '5038.0', 'toNode': '5033.0', 'Resistance': 0.00013, 'Reactance': 0.01255, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 989.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '915'},
        {'fromNode': '5039.0', 'toNode': '5033.0', 'Resistance': 8e-05, 'Reactance': 0.00552, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 994.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '916'},
        {'fromNode': '5040.0', 'toNode': '5033.0', 'Resistance': 0.00031, 'Reactance': 0.01643, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 993.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '917'},
        {'fromNode': '5041.0', 'toNode': '5033.0', 'Resistance': 0.00021, 'Reactance': 0.01142, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1001.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '918'},
        {'fromNode': '5042.0', 'toNode': '5033.0', 'Resistance': 0.00013, 'Reactance': 0.01345, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 988.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '919'},
        {'fromNode': '5033.0', 'toNode': '5047.0', 'Resistance': 0.0004, 'Reactance': 0.00814, 'Conductance': 0, 'Susceptance': 0.5676, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '920'},
        {'fromNode': '5033.0', 'toNode': '5350.0', 'Resistance': 0.00018, 'Reactance': 0.00367, 'Conductance': 0, 'Susceptance': 0.25593, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '921'},
        {'fromNode': '5033.0', 'toNode': '5384.0', 'Resistance': 6e-05, 'Reactance': 0.00115, 'Conductance': 0, 'Susceptance': 0.08036, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '922'},
        {'fromNode': '5034.0', 'toNode': '5313.0', 'Resistance': 0.00114, 'Reactance': 0.00836, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '923'},
        {'fromNode': '5034.0', 'toNode': '5313.0', 'Resistance': 0.00114, 'Reactance': 0.00836, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '924'},
        {'fromNode': '5034.0', 'toNode': '5313.0', 'Resistance': 0.00114, 'Reactance': 0.00836, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '925'},
        {'fromNode': '5371.0', 'toNode': '5034.0', 'Resistance': 0.0033, 'Reactance': 0.01934, 'Conductance': 0, 'Susceptance': 0.00966, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '926'},
        {'fromNode': '5043.0', 'toNode': '5152.0', 'Resistance': 0.00598, 'Reactance': 0.03504, 'Conductance': 0, 'Susceptance': 0.0175, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '927'},
        {'fromNode': '5043.0', 'toNode': '5284.0', 'Resistance': 0.00758, 'Reactance': 0.04445, 'Conductance': 0, 'Susceptance': 0.0222, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '928'},
        {'fromNode': '5044.0', 'toNode': '5153.0', 'Resistance': 0.0031, 'Reactance': 0.0236, 'Conductance': 0, 'Susceptance': 0.01239, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '929'},
        {'fromNode': '5044.0', 'toNode': '5153.0', 'Resistance': 0.0031, 'Reactance': 0.0236, 'Conductance': 0, 'Susceptance': 0.01239, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '930'},
        {'fromNode': '5185.0', 'toNode': '5044.0', 'Resistance': 0.0032, 'Reactance': 0.02431, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 325.5, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '931'},
        {'fromNode': '5185.0', 'toNode': '5044.0', 'Resistance': 0.0032, 'Reactance': 0.02431, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 325.5, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '932'},
        {'fromNode': '5046.0', 'toNode': '5045.0', 'Resistance': 0.00031, 'Reactance': 0.02114, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 550.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '933'},
        {'fromNode': '5045.0', 'toNode': '5120.0', 'Resistance': 0.00127, 'Reactance': 0.01539, 'Conductance': 0, 'Susceptance': 1.02213, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '934'},
        {'fromNode': '5260.0', 'toNode': '5045.0', 'Resistance': 0.0005, 'Reactance': 0.00603, 'Conductance': 0, 'Susceptance': 0.40075, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '935'},
        {'fromNode': '5413.0', 'toNode': '5045.0', 'Resistance': 0.00055, 'Reactance': 0.00672, 'Conductance': 0, 'Susceptance': 0.4467, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '936'},
        {'fromNode': '5286.0', 'toNode': '5046.0', 'Resistance': 0.0115, 'Reactance': 0.05069, 'Conductance': 0, 'Susceptance': 0.0236, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '937'},
        {'fromNode': '5048.0', 'toNode': '5047.0', 'Resistance': 0.00013, 'Reactance': 0.01253, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1354.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '938'},
        {'fromNode': '5047.0', 'toNode': '5179.0', 'Resistance': 0.00054, 'Reactance': 0.01097, 'Conductance': 0, 'Susceptance': 0.76512, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '939'},
        {'fromNode': '5047.0', 'toNode': '5260.0', 'Resistance': 0.00078, 'Reactance': 0.00944, 'Conductance': 0, 'Susceptance': 0.62704, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '940'},
        {'fromNode': '5047.0', 'toNode': '5444.0', 'Resistance': 0.00019, 'Reactance': 0.00235, 'Conductance': 0, 'Susceptance': 0.15601, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '941'},
        {'fromNode': '5480.0', 'toNode': '5048.0', 'Resistance': 0.00268, 'Reactance': 0.02037, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '942'},
        {'fromNode': '5480.0', 'toNode': '5048.0', 'Resistance': 0.00268, 'Reactance': 0.02037, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '943'},
        {'fromNode': '5480.0', 'toNode': '5048.0', 'Resistance': 0.00268, 'Reactance': 0.02037, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '944'},
        {'fromNode': '5480.0', 'toNode': '5048.0', 'Resistance': 0.00268, 'Reactance': 0.02037, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '945'},
        {'fromNode': '5050.0', 'toNode': '5049.0', 'Resistance': 0.0003, 'Reactance': 0.025, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '946'},
        {'fromNode': '5051.0', 'toNode': '5049.0', 'Resistance': 0.00024, 'Reactance': 0.01633, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 409.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '947'},
        {'fromNode': '5052.0', 'toNode': '5049.0', 'Resistance': 0.00067, 'Reactance': 0.0499, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 308.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '948'},
        {'fromNode': '5049.0', 'toNode': '5304.0', 'Resistance': 0.00059, 'Reactance': 0.00713, 'Conductance': 0, 'Susceptance': 0.47354, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '949'},
        {'fromNode': '5049.0', 'toNode': '5350.0', 'Resistance': 0.0007, 'Reactance': 0.0085, 'Conductance': 0, 'Susceptance': 0.5646, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '950'},
        {'fromNode': '5049.0', 'toNode': '8074.0', 'Resistance': 0.00142, 'Reactance': 0.02903, 'Conductance': 0, 'Susceptance': 2.02497, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '951'},
        {'fromNode': '8126.0', 'toNode': '5049.0', 'Resistance': 0.006, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.83, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '952'},
        {'fromNode': '5454.0', 'toNode': '5050.0', 'Resistance': 0.00279, 'Reactance': 0.02125, 'Conductance': 0, 'Susceptance': 0.01116, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '953'},
        {'fromNode': '5454.0', 'toNode': '5050.0', 'Resistance': 0.00279, 'Reactance': 0.02125, 'Conductance': 0, 'Susceptance': 0.01116, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '954'},
        {'fromNode': '5052.0', 'toNode': '8038.0', 'Resistance': 0.013, 'Reactance': 0.054, 'Conductance': 0, 'Susceptance': 0.027, 'MVA_rating': 186.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '955'},
        {'fromNode': '5052.0', 'toNode': '8124.0', 'Resistance': 0.013, 'Reactance': 0.054, 'Conductance': 0, 'Susceptance': 0.027, 'MVA_rating': 186.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '956'},
        {'fromNode': '5173.0', 'toNode': '5053.0', 'Resistance': 0.00088, 'Reactance': 0.00647, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '957'},
        {'fromNode': '5173.0', 'toNode': '5053.0', 'Resistance': 0.00088, 'Reactance': 0.00647, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '958'},
        {'fromNode': '5053.0', 'toNode': '5357.0', 'Resistance': 0.00118, 'Reactance': 0.00895, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '959'},
        {'fromNode': '5053.0', 'toNode': '5357.0', 'Resistance': 0.00118, 'Reactance': 0.00895, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '960'},
        {'fromNode': '5054.0', 'toNode': '5223.0', 'Resistance': 0.00543, 'Reactance': 0.03069, 'Conductance': 0, 'Susceptance': 0.01651, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '961'},
        {'fromNode': '5056.0', 'toNode': '5055.0', 'Resistance': 0.00023, 'Reactance': 0.01209, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 913.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '962'},
        {'fromNode': '5057.0', 'toNode': '5055.0', 'Resistance': 0.00023, 'Reactance': 0.01324, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1036.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '963'},
        {'fromNode': '5058.0', 'toNode': '5055.0', 'Resistance': 0.00031, 'Reactance': 0.01928, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1023.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '964'},
        {'fromNode': '5055.0', 'toNode': '5196.0', 'Resistance': 0.00034, 'Reactance': 0.0041, 'Conductance': 0, 'Susceptance': 0.275, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '965'},
        {'fromNode': '5187.0', 'toNode': '5056.0', 'Resistance': 0.01355, 'Reactance': 0.05765, 'Conductance': 0, 'Susceptance': 0.02883, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '966'},
        {'fromNode': '5197.0', 'toNode': '5056.0', 'Resistance': 0.0075, 'Reactance': 0.04397, 'Conductance': 0, 'Susceptance': 0.02196, 'MVA_rating': 263.55, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '967'},
        {'fromNode': '5060.0', 'toNode': '5059.0', 'Resistance': 0.00378, 'Reactance': 0.07811, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 151.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '968'},
        {'fromNode': '5059.0', 'toNode': '5426.0', 'Resistance': 0.00507, 'Reactance': 0.02868, 'Conductance': 0, 'Susceptance': 0.01543, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '969'},
        {'fromNode': '8148.0', 'toNode': '5059.0', 'Resistance': 0.00396, 'Reactance': 0.03012, 'Conductance': 0, 'Susceptance': 0.01581, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '970'},
        {'fromNode': '5061.0', 'toNode': '5060.0', 'Resistance': 0.00305, 'Reactance': 0.13464, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '971'},
        {'fromNode': '5060.0', 'toNode': '8065.0', 'Resistance': 0.00719, 'Reactance': 0.04106, 'Conductance': 0, 'Susceptance': 0.00617, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '972'},
        {'fromNode': '5062.0', 'toNode': '5061.0', 'Resistance': 0.00697, 'Reactance': 0.17819, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 100.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '973'},
        {'fromNode': '5064.0', 'toNode': '5063.0', 'Resistance': 0.00013, 'Reactance': 0.00901, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1277.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '974'},
        {'fromNode': '5064.0', 'toNode': '5063.0', 'Resistance': 0.00013, 'Reactance': 0.00901, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1277.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '975'},
        {'fromNode': '5065.0', 'toNode': '5063.0', 'Resistance': 0.00025, 'Reactance': 0.01754, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 290.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '976'},
        {'fromNode': '5066.0', 'toNode': '5063.0', 'Resistance': 0.00142, 'Reactance': 0.09165, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 213.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '977'},
        {'fromNode': '5067.0', 'toNode': '5063.0', 'Resistance': 0.00011, 'Reactance': 0.01076, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1078.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '978'},
        {'fromNode': '5068.0', 'toNode': '5063.0', 'Resistance': 0.00017, 'Reactance': 0.0113, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1073.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '979'},
        {'fromNode': '5069.0', 'toNode': '5063.0', 'Resistance': 0.00021, 'Reactance': 0.01202, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1079.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '980'},
        {'fromNode': '5063.0', 'toNode': '5304.0', 'Resistance': 0.00078, 'Reactance': 0.0095, 'Conductance': 0, 'Susceptance': 0.6308, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '981'},
        {'fromNode': '5322.0', 'toNode': '5063.0', 'Resistance': 0.00026, 'Reactance': 0.00314, 'Conductance': 0, 'Susceptance': 0.20858, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '982'},
        {'fromNode': '5063.0', 'toNode': '5401.0', 'Resistance': 0.0005, 'Reactance': 0.01016, 'Conductance': 0, 'Susceptance': 0.70838, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '983'},
        {'fromNode': '5064.0', 'toNode': '5125.0', 'Resistance': 0.00095, 'Reactance': 0.00721, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '984'},
        {'fromNode': '5064.0', 'toNode': '5125.0', 'Resistance': 0.00095, 'Reactance': 0.00721, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '985'},
        {'fromNode': '5064.0', 'toNode': '5125.0', 'Resistance': 0.00095, 'Reactance': 0.00721, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '986'},
        {'fromNode': '5064.0', 'toNode': '5125.0', 'Resistance': 0.00095, 'Reactance': 0.00721, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '987'},
        {'fromNode': '5143.0', 'toNode': '5064.0', 'Resistance': 0.00084, 'Reactance': 0.00642, 'Conductance': 0, 'Susceptance': 0.00337, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '988'},
        {'fromNode': '5143.0', 'toNode': '5064.0', 'Resistance': 0.00084, 'Reactance': 0.00642, 'Conductance': 0, 'Susceptance': 0.00337, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '989'},
        {'fromNode': '5143.0', 'toNode': '5064.0', 'Resistance': 0.00084, 'Reactance': 0.00642, 'Conductance': 0, 'Susceptance': 0.00337, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '990'},
        {'fromNode': '5070.0', 'toNode': '5128.0', 'Resistance': 0.00204, 'Reactance': 0.00902, 'Conductance': 0, 'Susceptance': 0.0042, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '991'},
        {'fromNode': '5425.0', 'toNode': '5070.0', 'Resistance': 0.00111, 'Reactance': 0.00651, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '992'},
        {'fromNode': '5138.0', 'toNode': '5071.0', 'Resistance': 0.0037, 'Reactance': 0.02168, 'Conductance': 0, 'Susceptance': 0.01083, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '993'},
        {'fromNode': '5071.0', 'toNode': '5359.0', 'Resistance': 0.00459, 'Reactance': 0.03488, 'Conductance': 0, 'Susceptance': 0.01831, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '994'},
        {'fromNode': '5103.0', 'toNode': '5072.0', 'Resistance': 0.00114, 'Reactance': 0.00868, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '995'},
        {'fromNode': '5103.0', 'toNode': '5072.0', 'Resistance': 0.00114, 'Reactance': 0.00868, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '996'},
        {'fromNode': '5103.0', 'toNode': '5072.0', 'Resistance': 0.00114, 'Reactance': 0.00868, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '997'},
        {'fromNode': '5103.0', 'toNode': '5072.0', 'Resistance': 0.00114, 'Reactance': 0.00868, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '998'},
        {'fromNode': '5103.0', 'toNode': '5072.0', 'Resistance': 0.00114, 'Reactance': 0.00868, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '999'},
        {'fromNode': '5116.0', 'toNode': '5072.0', 'Resistance': 0.00108, 'Reactance': 0.0079, 'Conductance': 0, 'Susceptance': 0.00448, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1000'},
        {'fromNode': '5116.0', 'toNode': '5072.0', 'Resistance': 0.00108, 'Reactance': 0.0079, 'Conductance': 0, 'Susceptance': 0.00448, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1001'},
        {'fromNode': '5121.0', 'toNode': '5073.0', 'Resistance': 0.00516, 'Reactance': 0.02272, 'Conductance': 0, 'Susceptance': 0.01058, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1002'},
        {'fromNode': '5484.0', 'toNode': '5073.0', 'Resistance': 0.0111, 'Reactance': 0.06505, 'Conductance': 0, 'Susceptance': 0.03248, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1003'},
        {'fromNode': '5074.0', 'toNode': '5147.0', 'Resistance': 0.0043, 'Reactance': 0.03272, 'Conductance': 0, 'Susceptance': 0.01718, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1004'},
        {'fromNode': '5074.0', 'toNode': '5331.0', 'Resistance': 0.00369, 'Reactance': 0.02161, 'Conductance': 0, 'Susceptance': 0.01079, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1005'},
        {'fromNode': '5075.0', 'toNode': '5267.0', 'Resistance': 0.02194, 'Reactance': 0.09333, 'Conductance': 0, 'Susceptance': 0.04668, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1006'},
        {'fromNode': '5075.0', 'toNode': '5377.0', 'Resistance': 0.02449, 'Reactance': 0.10791, 'Conductance': 0, 'Susceptance': 0.05023, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1007'},
        {'fromNode': '5077.0', 'toNode': '5076.0', 'Resistance': 0.0015, 'Reactance': 0.06519, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1008'},
        {'fromNode': '5227.0', 'toNode': '5076.0', 'Resistance': 0.00533, 'Reactance': 0.02268, 'Conductance': 0, 'Susceptance': 0.01134, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1009'},
        {'fromNode': '8108.0', 'toNode': '5077.0', 'Resistance': 0.00971, 'Reactance': 0.05831, 'Conductance': 0, 'Susceptance': 0.00791, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1010'},
        {'fromNode': '5160.0', 'toNode': '5078.0', 'Resistance': 0.00399, 'Reactance': 0.01757, 'Conductance': 0, 'Susceptance': 0.00818, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1011'},
        {'fromNode': '5079.0', 'toNode': '5138.0', 'Resistance': 0.00181, 'Reactance': 0.01379, 'Conductance': 0, 'Susceptance': 0.00724, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1012'},
        {'fromNode': '5079.0', 'toNode': '5138.0', 'Resistance': 0.00181, 'Reactance': 0.01379, 'Conductance': 0, 'Susceptance': 0.00724, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1013'},
        {'fromNode': '5378.0', 'toNode': '5079.0', 'Resistance': 0.00152, 'Reactance': 0.00891, 'Conductance': 0, 'Susceptance': 0.00445, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1014'},
        {'fromNode': '5142.0', 'toNode': '5080.0', 'Resistance': 0.00553, 'Reactance': 0.03127, 'Conductance': 0, 'Susceptance': 0.01682, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1015'},
        {'fromNode': '5080.0', 'toNode': '5328.0', 'Resistance': 0.00866, 'Reactance': 0.03817, 'Conductance': 0, 'Susceptance': 0.01777, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1016'},
        {'fromNode': '5081.0', 'toNode': '5163.0', 'Resistance': 0.00516, 'Reactance': 0.03024, 'Conductance': 0, 'Susceptance': 0.0151, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1017'},
        {'fromNode': '5221.0', 'toNode': '5081.0', 'Resistance': 0.00517, 'Reactance': 0.0228, 'Conductance': 0, 'Susceptance': 0.01061, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1018'},
        {'fromNode': '5081.0', 'toNode': '5377.0', 'Resistance': 0.0068, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 221.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1019'},
        {'fromNode': '5082.0', 'toNode': '5086.0', 'Resistance': 0.00219, 'Reactance': 0.01283, 'Conductance': 0, 'Susceptance': 0.00641, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1020'},
        {'fromNode': '5082.0', 'toNode': '5330.0', 'Resistance': 0.00214, 'Reactance': 0.01253, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1021'},
        {'fromNode': '5082.0', 'toNode': '5440.0', 'Resistance': 0.00107, 'Reactance': 0.00605, 'Conductance': 0, 'Susceptance': 0.00326, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1022'},
        {'fromNode': '5084.0', 'toNode': '5083.0', 'Resistance': 0.00024, 'Reactance': 0.01146, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1295.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1023'},
        {'fromNode': '5083.0', 'toNode': '5295.0', 'Resistance': 7e-05, 'Reactance': 0.00134, 'Conductance': 0, 'Susceptance': 0.09371, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1024'},
        {'fromNode': '5083.0', 'toNode': '5322.0', 'Resistance': 0.0007, 'Reactance': 0.00853, 'Conductance': 0, 'Susceptance': 0.56646, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1025'},
        {'fromNode': '5083.0', 'toNode': '5464.0', 'Resistance': 0.0002, 'Reactance': 0.00237, 'Conductance': 0, 'Susceptance': 0.15714, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1026'},
        {'fromNode': '5357.0', 'toNode': '5084.0', 'Resistance': 0.00149, 'Reactance': 0.01134, 'Conductance': 0, 'Susceptance': 0.00596, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1027'},
        {'fromNode': '5357.0', 'toNode': '5084.0', 'Resistance': 0.00149, 'Reactance': 0.01134, 'Conductance': 0, 'Susceptance': 0.00596, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1028'},
        {'fromNode': '5357.0', 'toNode': '5084.0', 'Resistance': 0.00149, 'Reactance': 0.01134, 'Conductance': 0, 'Susceptance': 0.00596, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1029'},
        {'fromNode': '5157.0', 'toNode': '5085.0', 'Resistance': 0.00229, 'Reactance': 0.01293, 'Conductance': 0, 'Susceptance': 0.00695, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1030'},
        {'fromNode': '5085.0', 'toNode': '5468.0', 'Resistance': 0.00181, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1031'},
        {'fromNode': '5086.0', 'toNode': '5177.0', 'Resistance': 0.00288, 'Reactance': 0.01629, 'Conductance': 0, 'Susceptance': 0.00876, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1032'},
        {'fromNode': '5086.0', 'toNode': '5210.0', 'Resistance': 0.00527, 'Reactance': 0.02323, 'Conductance': 0, 'Susceptance': 0.01081, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1033'},
        {'fromNode': '5086.0', 'toNode': '5447.0', 'Resistance': 0.00451, 'Reactance': 0.01917, 'Conductance': 0, 'Susceptance': 0.00959, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1034'},
        {'fromNode': '5257.0', 'toNode': '5087.0', 'Resistance': 0.00596, 'Reactance': 0.02626, 'Conductance': 0, 'Susceptance': 0.01222, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1035'},
        {'fromNode': '5087.0', 'toNode': '5345.0', 'Resistance': 0.00569, 'Reactance': 0.03217, 'Conductance': 0, 'Susceptance': 0.01731, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1036'},
        {'fromNode': '5087.0', 'toNode': '5348.0', 'Resistance': 0.0048, 'Reactance': 0.02814, 'Conductance': 0, 'Susceptance': 0.01405, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1037'},
        {'fromNode': '5089.0', 'toNode': '5088.0', 'Resistance': 0.00382, 'Reactance': 0.09418, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 150.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1038'},
        {'fromNode': '5088.0', 'toNode': '5290.0', 'Resistance': 0.00328, 'Reactance': 0.01851, 'Conductance': 0, 'Susceptance': 0.00996, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1039'},
        {'fromNode': '5088.0', 'toNode': '5393.0', 'Resistance': 0.00614, 'Reactance': 0.03602, 'Conductance': 0, 'Susceptance': 0.01799, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1040'},
        {'fromNode': '5090.0', 'toNode': '5122.0', 'Resistance': 0.00277, 'Reactance': 0.0122, 'Conductance': 0, 'Susceptance': 0.00568, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1041'},
        {'fromNode': '5090.0', 'toNode': '5230.0', 'Resistance': 0.00189, 'Reactance': 0.00834, 'Conductance': 0, 'Susceptance': 0.00388, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1042'},
        {'fromNode': '5090.0', 'toNode': '5311.0', 'Resistance': 0.00167, 'Reactance': 0.0098, 'Conductance': 0, 'Susceptance': 0.00489, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1043'},
        {'fromNode': '5171.0', 'toNode': '5091.0', 'Resistance': 0.00311, 'Reactance': 0.01823, 'Conductance': 0, 'Susceptance': 0.0091, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1044'},
        {'fromNode': '5091.0', 'toNode': '5257.0', 'Resistance': 0.00494, 'Reactance': 0.02894, 'Conductance': 0, 'Susceptance': 0.01445, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1045'},
        {'fromNode': '5092.0', 'toNode': '5186.0', 'Resistance': 0.00539, 'Reactance': 0.03162, 'Conductance': 0, 'Susceptance': 0.01579, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1046'},
        {'fromNode': '5453.0', 'toNode': '5092.0', 'Resistance': 0.00849, 'Reactance': 0.03611, 'Conductance': 0, 'Susceptance': 0.01806, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1047'},
        {'fromNode': '5475.0', 'toNode': '5092.0', 'Resistance': 0.00472, 'Reactance': 0.02665, 'Conductance': 0, 'Susceptance': 0.01434, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1048'},
        {'fromNode': '5093.0', 'toNode': '5291.0', 'Resistance': 0.00299, 'Reactance': 0.01754, 'Conductance': 0, 'Susceptance': 0.00876, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1049'},
        {'fromNode': '5332.0', 'toNode': '5093.0', 'Resistance': 0.00371, 'Reactance': 0.02175, 'Conductance': 0, 'Susceptance': 0.01086, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1050'},
        {'fromNode': '5093.0', 'toNode': '5434.0', 'Resistance': 0.0015, 'Reactance': 0.01141, 'Conductance': 0, 'Susceptance': 0.00599, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1051'},
        {'fromNode': '5093.0', 'toNode': '5434.0', 'Resistance': 0.0015, 'Reactance': 0.01141, 'Conductance': 0, 'Susceptance': 0.00599, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1052'},
        {'fromNode': '5094.0', 'toNode': '5381.0', 'Resistance': 0.00329, 'Reactance': 0.025, 'Conductance': 0, 'Susceptance': 0.01312, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1053'},
        {'fromNode': '5485.0', 'toNode': '5094.0', 'Resistance': 0.00237, 'Reactance': 0.01799, 'Conductance': 0, 'Susceptance': 0.00945, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1054'},
        {'fromNode': '5277.0', 'toNode': '5095.0', 'Resistance': 0.0038, 'Reactance': 0.01676, 'Conductance': 0, 'Susceptance': 0.0078, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1055'},
        {'fromNode': '5341.0', 'toNode': '5095.0', 'Resistance': 0.00257, 'Reactance': 0.01134, 'Conductance': 0, 'Susceptance': 0.00528, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1056'},
        {'fromNode': '5305.0', 'toNode': '5096.0', 'Resistance': 0.00176, 'Reactance': 0.01289, 'Conductance': 0, 'Susceptance': 0.0073, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1057'},
        {'fromNode': '5305.0', 'toNode': '5096.0', 'Resistance': 0.00176, 'Reactance': 0.01289, 'Conductance': 0, 'Susceptance': 0.0073, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1058'},
        {'fromNode': '5305.0', 'toNode': '5096.0', 'Resistance': 0.00176, 'Reactance': 0.01289, 'Conductance': 0, 'Susceptance': 0.0073, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1059'},
        {'fromNode': '5096.0', 'toNode': '5412.0', 'Resistance': 0.00234, 'Reactance': 0.01373, 'Conductance': 0, 'Susceptance': 0.00686, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1060'},
        {'fromNode': '5097.0', 'toNode': '5125.0', 'Resistance': 0.00078, 'Reactance': 0.00569, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1061'},
        {'fromNode': '5097.0', 'toNode': '5125.0', 'Resistance': 0.00078, 'Reactance': 0.00569, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1062'},
        {'fromNode': '5098.0', 'toNode': '5453.0', 'Resistance': 0.00529, 'Reactance': 0.04026, 'Conductance': 0, 'Susceptance': 0.02114, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1063'},
        {'fromNode': '5098.0', 'toNode': '5475.0', 'Resistance': 0.01191, 'Reactance': 0.05066, 'Conductance': 0, 'Susceptance': 0.02533, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1064'},
        {'fromNode': '5099.0', 'toNode': '5195.0', 'Resistance': 0.00308, 'Reactance': 0.0174, 'Conductance': 0, 'Susceptance': 0.00936, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1065'},
        {'fromNode': '5099.0', 'toNode': '5339.0', 'Resistance': 0.00266, 'Reactance': 0.01561, 'Conductance': 0, 'Susceptance': 0.00779, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1066'},
        {'fromNode': '5229.0', 'toNode': '5100.0', 'Resistance': 0.00057, 'Reactance': 0.00414, 'Conductance': 0, 'Susceptance': 0.00235, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1067'},
        {'fromNode': '5229.0', 'toNode': '5100.0', 'Resistance': 0.00057, 'Reactance': 0.00414, 'Conductance': 0, 'Susceptance': 0.00235, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1068'},
        {'fromNode': '5250.0', 'toNode': '5100.0', 'Resistance': 0.00124, 'Reactance': 0.0091, 'Conductance': 0, 'Susceptance': 0.00516, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1069'},
        {'fromNode': '5250.0', 'toNode': '5100.0', 'Resistance': 0.00124, 'Reactance': 0.0091, 'Conductance': 0, 'Susceptance': 0.00516, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1070'},
        {'fromNode': '5250.0', 'toNode': '5100.0', 'Resistance': 0.00124, 'Reactance': 0.0091, 'Conductance': 0, 'Susceptance': 0.00516, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1071'},
        {'fromNode': '5100.0', 'toNode': '5352.0', 'Resistance': 0.00056, 'Reactance': 0.00426, 'Conductance': 0, 'Susceptance': 0.00224, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1072'},
        {'fromNode': '5100.0', 'toNode': '5352.0', 'Resistance': 0.00056, 'Reactance': 0.00426, 'Conductance': 0, 'Susceptance': 0.00224, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1073'},
        {'fromNode': '5231.0', 'toNode': '5101.0', 'Resistance': 0.01968, 'Reactance': 0.08674, 'Conductance': 0, 'Susceptance': 0.04038, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1074'},
        {'fromNode': '5103.0', 'toNode': '5102.0', 'Resistance': 0.00026, 'Reactance': 0.0114, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1345.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1075'},
        {'fromNode': '5103.0', 'toNode': '5102.0', 'Resistance': 0.00026, 'Reactance': 0.0114, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1345.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1076'},
        {'fromNode': '5102.0', 'toNode': '5304.0', 'Resistance': 0.00023, 'Reactance': 0.00277, 'Conductance': 0, 'Susceptance': 0.18414, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1077'},
        {'fromNode': '5384.0', 'toNode': '5102.0', 'Resistance': 0.00039, 'Reactance': 0.0047, 'Conductance': 0, 'Susceptance': 0.31216, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1078'},
        {'fromNode': '5464.0', 'toNode': '5102.0', 'Resistance': 0.00028, 'Reactance': 0.00339, 'Conductance': 0, 'Susceptance': 0.22498, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1079'},
        {'fromNode': '5103.0', 'toNode': '5130.0', 'Resistance': 0.00117, 'Reactance': 0.00856, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1080'},
        {'fromNode': '5103.0', 'toNode': '5130.0', 'Resistance': 0.00117, 'Reactance': 0.00856, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1081'},
        {'fromNode': '5104.0', 'toNode': '5199.0', 'Resistance': 0.00503, 'Reactance': 0.02843, 'Conductance': 0, 'Susceptance': 0.01529, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1082'},
        {'fromNode': '5447.0', 'toNode': '5104.0', 'Resistance': 0.00135, 'Reactance': 0.01026, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1083'},
        {'fromNode': '5447.0', 'toNode': '5104.0', 'Resistance': 0.00135, 'Reactance': 0.01026, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1084'},
        {'fromNode': '5104.0', 'toNode': '5480.0', 'Resistance': 0.00236, 'Reactance': 0.01795, 'Conductance': 0, 'Susceptance': 0.00942, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1085'},
        {'fromNode': '5104.0', 'toNode': '5480.0', 'Resistance': 0.00236, 'Reactance': 0.01795, 'Conductance': 0, 'Susceptance': 0.00942, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1086'},
        {'fromNode': '5104.0', 'toNode': '5480.0', 'Resistance': 0.00236, 'Reactance': 0.01795, 'Conductance': 0, 'Susceptance': 0.00942, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1087'},
        {'fromNode': '5104.0', 'toNode': '5480.0', 'Resistance': 0.00236, 'Reactance': 0.01795, 'Conductance': 0, 'Susceptance': 0.00942, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1088'},
        {'fromNode': '5106.0', 'toNode': '5105.0', 'Resistance': 0.00709, 'Reactance': 0.19751, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 82.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1089'},
        {'fromNode': '5107.0', 'toNode': '5105.0', 'Resistance': 0.00966, 'Reactance': 0.19527, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 75.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1090'},
        {'fromNode': '5105.0', 'toNode': '5148.0', 'Resistance': 0.00212, 'Reactance': 0.009, 'Conductance': 0, 'Susceptance': 0.0045, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1091'},
        {'fromNode': '5105.0', 'toNode': '5253.0', 'Resistance': 0.00604, 'Reactance': 0.02664, 'Conductance': 0, 'Susceptance': 0.0124, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1092'},
        {'fromNode': '5105.0', 'toNode': '5422.0', 'Resistance': 0.00354, 'Reactance': 0.02074, 'Conductance': 0, 'Susceptance': 0.01036, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1093'},
        {'fromNode': '5108.0', 'toNode': '5266.0', 'Resistance': 0.00144, 'Reactance': 0.01098, 'Conductance': 0, 'Susceptance': 0.00577, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1094'},
        {'fromNode': '5108.0', 'toNode': '5266.0', 'Resistance': 0.00144, 'Reactance': 0.01098, 'Conductance': 0, 'Susceptance': 0.00577, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1095'},
        {'fromNode': '5108.0', 'toNode': '5353.0', 'Resistance': 0.00106, 'Reactance': 0.00809, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1096'},
        {'fromNode': '5108.0', 'toNode': '5353.0', 'Resistance': 0.00106, 'Reactance': 0.00809, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1097'},
        {'fromNode': '5449.0', 'toNode': '5108.0', 'Resistance': 0.0015, 'Reactance': 0.01102, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1098'},
        {'fromNode': '5449.0', 'toNode': '5108.0', 'Resistance': 0.0015, 'Reactance': 0.01102, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1099'},
        {'fromNode': '5449.0', 'toNode': '5108.0', 'Resistance': 0.0015, 'Reactance': 0.01102, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1100'},
        {'fromNode': '5274.0', 'toNode': '5109.0', 'Resistance': 0.00268, 'Reactance': 0.01516, 'Conductance': 0, 'Susceptance': 0.00815, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1101'},
        {'fromNode': '5109.0', 'toNode': '5333.0', 'Resistance': 0.00213, 'Reactance': 0.01247, 'Conductance': 0, 'Susceptance': 0.00623, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1102'},
        {'fromNode': '5176.0', 'toNode': '5110.0', 'Resistance': 0.00503, 'Reactance': 0.02841, 'Conductance': 0, 'Susceptance': 0.01528, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1103'},
        {'fromNode': '5110.0', 'toNode': '5200.0', 'Resistance': 0.0045, 'Reactance': 0.02638, 'Conductance': 0, 'Susceptance': 0.01317, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1104'},
        {'fromNode': '5203.0', 'toNode': '5111.0', 'Resistance': 0.00135, 'Reactance': 0.00792, 'Conductance': 0, 'Susceptance': 0.00396, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1105'},
        {'fromNode': '5111.0', 'toNode': '5289.0', 'Resistance': 0.00089, 'Reactance': 0.00652, 'Conductance': 0, 'Susceptance': 0.0037, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1106'},
        {'fromNode': '5111.0', 'toNode': '5289.0', 'Resistance': 0.00089, 'Reactance': 0.00652, 'Conductance': 0, 'Susceptance': 0.0037, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1107'},
        {'fromNode': '5111.0', 'toNode': '5372.0', 'Resistance': 0.00119, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.0035, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1108'},
        {'fromNode': '5112.0', 'toNode': '5133.0', 'Resistance': 0.01714, 'Reactance': 0.07289, 'Conductance': 0, 'Susceptance': 0.03645, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1109'},
        {'fromNode': '5191.0', 'toNode': '5112.0', 'Resistance': 0.008, 'Reactance': 0.03402, 'Conductance': 0, 'Susceptance': 0.01701, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1110'},
        {'fromNode': '5112.0', 'toNode': '5246.0', 'Resistance': 0.01038, 'Reactance': 0.04415, 'Conductance': 0, 'Susceptance': 0.02208, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1111'},
        {'fromNode': '5286.0', 'toNode': '5112.0', 'Resistance': 0.018, 'Reactance': 0.078, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 150.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1112'},
        {'fromNode': '5113.0', 'toNode': '5195.0', 'Resistance': 0.00674, 'Reactance': 0.0395, 'Conductance': 0, 'Susceptance': 0.01972, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1113'},
        {'fromNode': '5113.0', 'toNode': '5218.0', 'Resistance': 0.00858, 'Reactance': 0.0378, 'Conductance': 0, 'Susceptance': 0.01759, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1114'},
        {'fromNode': '5220.0', 'toNode': '5113.0', 'Resistance': 0.00567, 'Reactance': 0.0241, 'Conductance': 0, 'Susceptance': 0.01205, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1115'},
        {'fromNode': '5219.0', 'toNode': '5114.0', 'Resistance': 0.00883, 'Reactance': 0.03893, 'Conductance': 0, 'Susceptance': 0.01812, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1116'},
        {'fromNode': '5114.0', 'toNode': '5222.0', 'Resistance': 0.00513, 'Reactance': 0.03755, 'Conductance': 0, 'Susceptance': 0.02128, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1117'},
        {'fromNode': '5227.0', 'toNode': '5114.0', 'Resistance': 0.005, 'Reactance': 0.03803, 'Conductance': 0, 'Susceptance': 0.01997, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1118'},
        {'fromNode': '5115.0', 'toNode': '5421.0', 'Resistance': 0.0025, 'Reactance': 0.01413, 'Conductance': 0, 'Susceptance': 0.0076, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1119'},
        {'fromNode': '5454.0', 'toNode': '5115.0', 'Resistance': 0.00397, 'Reactance': 0.02326, 'Conductance': 0, 'Susceptance': 0.01161, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1120'},
        {'fromNode': '5116.0', 'toNode': '5224.0', 'Resistance': 0.00199, 'Reactance': 0.01514, 'Conductance': 0, 'Susceptance': 0.00795, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1121'},
        {'fromNode': '5116.0', 'toNode': '5224.0', 'Resistance': 0.00199, 'Reactance': 0.01514, 'Conductance': 0, 'Susceptance': 0.00795, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1122'},
        {'fromNode': '5117.0', 'toNode': '5216.0', 'Resistance': 0.00157, 'Reactance': 0.00695, 'Conductance': 0, 'Susceptance': 0.00323, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1123'},
        {'fromNode': '5117.0', 'toNode': '5409.0', 'Resistance': 0.00292, 'Reactance': 0.01288, 'Conductance': 0, 'Susceptance': 0.00599, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1124'},
        {'fromNode': '5334.0', 'toNode': '5118.0', 'Resistance': 0.01328, 'Reactance': 0.05646, 'Conductance': 0, 'Susceptance': 0.02824, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1125'},
        {'fromNode': '5347.0', 'toNode': '5118.0', 'Resistance': 0.01023, 'Reactance': 0.04352, 'Conductance': 0, 'Susceptance': 0.02177, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1126'},
        {'fromNode': '5118.0', 'toNode': '5456.0', 'Resistance': 0.00988, 'Reactance': 0.05791, 'Conductance': 0, 'Susceptance': 0.02892, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1127'},
        {'fromNode': '5119.0', 'toNode': '5205.0', 'Resistance': 0.00604, 'Reactance': 0.02658, 'Conductance': 0, 'Susceptance': 0.01237, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1128'},
        {'fromNode': '5337.0', 'toNode': '5119.0', 'Resistance': 0.01103, 'Reactance': 0.04691, 'Conductance': 0, 'Susceptance': 0.02346, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1129'},
        {'fromNode': '5121.0', 'toNode': '5120.0', 'Resistance': 0.00064, 'Reactance': 0.04599, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 298.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1130'},
        {'fromNode': '5120.0', 'toNode': '5239.0', 'Resistance': 0.00087, 'Reactance': 0.01049, 'Conductance': 0, 'Susceptance': 0.69666, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1131'},
        {'fromNode': '5122.0', 'toNode': '5252.0', 'Resistance': 0.0046, 'Reactance': 0.01958, 'Conductance': 0, 'Susceptance': 0.00979, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1132'},
        {'fromNode': '5122.0', 'toNode': '5308.0', 'Resistance': 0.00469, 'Reactance': 0.02752, 'Conductance': 0, 'Susceptance': 0.01374, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1133'},
        {'fromNode': '5124.0', 'toNode': '5123.0', 'Resistance': 0.00266, 'Reactance': 0.07286, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 214.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1134'},
        {'fromNode': '5159.0', 'toNode': '5123.0', 'Resistance': 0.00799, 'Reactance': 0.03521, 'Conductance': 0, 'Susceptance': 0.01639, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1135'},
        {'fromNode': '8157.0', 'toNode': '5124.0', 'Resistance': 0.01256, 'Reactance': 0.05593, 'Conductance': 0, 'Susceptance': 0.0082, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1136'},
        {'fromNode': '5125.0', 'toNode': '5330.0', 'Resistance': 0.00087, 'Reactance': 0.0064, 'Conductance': 0, 'Susceptance': 0.00362, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1137'},
        {'fromNode': '5125.0', 'toNode': '5330.0', 'Resistance': 0.00087, 'Reactance': 0.0064, 'Conductance': 0, 'Susceptance': 0.00362, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1138'},
        {'fromNode': '5126.0', 'toNode': '5252.0', 'Resistance': 0.0023, 'Reactance': 0.00978, 'Conductance': 0, 'Susceptance': 0.00489, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1139'},
        {'fromNode': '5126.0', 'toNode': '5309.0', 'Resistance': 0.00399, 'Reactance': 0.01755, 'Conductance': 0, 'Susceptance': 0.00817, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1140'},
        {'fromNode': '5126.0', 'toNode': '5386.0', 'Resistance': 0.00259, 'Reactance': 0.01969, 'Conductance': 0, 'Susceptance': 0.01034, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1141'},
        {'fromNode': '5191.0', 'toNode': '5127.0', 'Resistance': 0.00791, 'Reactance': 0.03487, 'Conductance': 0, 'Susceptance': 0.01623, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1142'},
        {'fromNode': '5127.0', 'toNode': '5231.0', 'Resistance': 0.01585, 'Reactance': 0.06741, 'Conductance': 0, 'Susceptance': 0.03371, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1143'},
        {'fromNode': '5314.0', 'toNode': '5127.0', 'Resistance': 0.01301, 'Reactance': 0.05729, 'Conductance': 0, 'Susceptance': 0.02667, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1144'},
        {'fromNode': '5128.0', 'toNode': '5214.0', 'Resistance': 0.00143, 'Reactance': 0.0084, 'Conductance': 0, 'Susceptance': 0.0042, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1145'},
        {'fromNode': '5128.0', 'toNode': '5467.0', 'Resistance': 0.00166, 'Reactance': 0.00974, 'Conductance': 0, 'Susceptance': 0.00486, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1146'},
        {'fromNode': '5226.0', 'toNode': '5129.0', 'Resistance': 0.00964, 'Reactance': 0.04249, 'Conductance': 0, 'Susceptance': 0.01978, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1147'},
        {'fromNode': '5129.0', 'toNode': '5240.0', 'Resistance': 0.01196, 'Reactance': 0.05272, 'Conductance': 0, 'Susceptance': 0.02454, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1148'},
        {'fromNode': '5173.0', 'toNode': '5130.0', 'Resistance': 0.00095, 'Reactance': 0.00693, 'Conductance': 0, 'Susceptance': 0.00393, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1149'},
        {'fromNode': '5173.0', 'toNode': '5130.0', 'Resistance': 0.00095, 'Reactance': 0.00693, 'Conductance': 0, 'Susceptance': 0.00393, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1150'},
        {'fromNode': '5132.0', 'toNode': '5131.0', 'Resistance': 0.00037, 'Reactance': 0.01821, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 981.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1151'},
        {'fromNode': '5131.0', 'toNode': '5279.0', 'Resistance': 0.00022, 'Reactance': 0.00271, 'Conductance': 0, 'Susceptance': 0.1803, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1152'},
        {'fromNode': '5131.0', 'toNode': '5410.0', 'Resistance': 0.00018, 'Reactance': 0.00365, 'Conductance': 0, 'Susceptance': 0.25434, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1153'},
        {'fromNode': '5131.0', 'toNode': '6107.0', 'Resistance': 0.00051, 'Reactance': 0.01038, 'Conductance': 0, 'Susceptance': 0.72429, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1154'},
        {'fromNode': '5163.0', 'toNode': '5132.0', 'Resistance': 0.00318, 'Reactance': 0.01866, 'Conductance': 0, 'Susceptance': 0.00932, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1155'},
        {'fromNode': '5155.0', 'toNode': '5134.0', 'Resistance': 0.0067, 'Reactance': 0.0393, 'Conductance': 0, 'Susceptance': 0.01962, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1156'},
        {'fromNode': '5155.0', 'toNode': '5135.0', 'Resistance': 0.00905, 'Reactance': 0.05304, 'Conductance': 0, 'Susceptance': 0.02648, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1157'},
        {'fromNode': '5169.0', 'toNode': '5135.0', 'Resistance': 0.01055, 'Reactance': 0.04486, 'Conductance': 0, 'Susceptance': 0.02244, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1158'},
        {'fromNode': '5135.0', 'toNode': '5264.0', 'Resistance': 0.00751, 'Reactance': 0.04244, 'Conductance': 0, 'Susceptance': 0.02283, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1159'},
        {'fromNode': '5136.0', 'toNode': '5288.0', 'Resistance': 0.00558, 'Reactance': 0.03152, 'Conductance': 0, 'Susceptance': 0.01695, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1160'},
        {'fromNode': '5138.0', 'toNode': '5137.0', 'Resistance': 0.00014, 'Reactance': 0.0092, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 945.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1161'},
        {'fromNode': '5137.0', 'toNode': '5388.0', 'Resistance': 0.00021, 'Reactance': 0.00253, 'Conductance': 0, 'Susceptance': 0.16793, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1162'},
        {'fromNode': '5137.0', 'toNode': '5451.0', 'Resistance': 0.00114, 'Reactance': 0.01383, 'Conductance': 0, 'Susceptance': 0.91863, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1163'},
        {'fromNode': '5139.0', 'toNode': '5183.0', 'Resistance': 0.00187, 'Reactance': 0.00825, 'Conductance': 0, 'Susceptance': 0.00384, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1164'},
        {'fromNode': '5139.0', 'toNode': '5233.0', 'Resistance': 0.00826, 'Reactance': 0.03638, 'Conductance': 0, 'Susceptance': 0.01694, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1165'},
        {'fromNode': '5344.0', 'toNode': '5139.0', 'Resistance': 0.00178, 'Reactance': 0.00785, 'Conductance': 0, 'Susceptance': 0.00365, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1166'},
        {'fromNode': '5140.0', 'toNode': '5466.0', 'Resistance': 0.00143, 'Reactance': 0.00633, 'Conductance': 0, 'Susceptance': 0.00295, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1167'},
        {'fromNode': '5141.0', 'toNode': '5408.0', 'Resistance': 0.00702, 'Reactance': 0.03094, 'Conductance': 0, 'Susceptance': 0.0144, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1168'},
        {'fromNode': '5142.0', 'toNode': '5466.0', 'Resistance': 0.01638, 'Reactance': 0.07217, 'Conductance': 0, 'Susceptance': 0.03359, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1169'},
        {'fromNode': '5143.0', 'toNode': '5216.0', 'Resistance': 0.00311, 'Reactance': 0.01826, 'Conductance': 0, 'Susceptance': 0.00912, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1170'},
        {'fromNode': '5428.0', 'toNode': '5143.0', 'Resistance': 0.00107, 'Reactance': 0.00811, 'Conductance': 0, 'Susceptance': 0.00426, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1171'},
        {'fromNode': '5428.0', 'toNode': '5143.0', 'Resistance': 0.00107, 'Reactance': 0.00811, 'Conductance': 0, 'Susceptance': 0.00426, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1172'},
        {'fromNode': '5144.0', 'toNode': '5314.0', 'Resistance': 0.01178, 'Reactance': 0.05011, 'Conductance': 0, 'Susceptance': 0.02506, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1173'},
        {'fromNode': '5389.0', 'toNode': '5144.0', 'Resistance': 0.00473, 'Reactance': 0.02084, 'Conductance': 0, 'Susceptance': 0.0097, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1174'},
        {'fromNode': '5145.0', 'toNode': '5251.0', 'Resistance': 0.00125, 'Reactance': 0.00917, 'Conductance': 0, 'Susceptance': 0.0052, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1175'},
        {'fromNode': '5145.0', 'toNode': '5251.0', 'Resistance': 0.00125, 'Reactance': 0.00917, 'Conductance': 0, 'Susceptance': 0.0052, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1176'},
        {'fromNode': '5145.0', 'toNode': '5323.0', 'Resistance': 0.00133, 'Reactance': 0.00973, 'Conductance': 0, 'Susceptance': 0.00551, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1177'},
        {'fromNode': '5145.0', 'toNode': '5323.0', 'Resistance': 0.00133, 'Reactance': 0.00973, 'Conductance': 0, 'Susceptance': 0.00551, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1178'},
        {'fromNode': '5145.0', 'toNode': '5323.0', 'Resistance': 0.00133, 'Reactance': 0.00973, 'Conductance': 0, 'Susceptance': 0.00551, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1179'},
        {'fromNode': '5146.0', 'toNode': '5149.0', 'Resistance': 0.00482, 'Reactance': 0.02826, 'Conductance': 0, 'Susceptance': 0.01411, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1180'},
        {'fromNode': '5146.0', 'toNode': '5152.0', 'Resistance': 0.00096, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.00383, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1181'},
        {'fromNode': '5146.0', 'toNode': '5152.0', 'Resistance': 0.00096, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.00383, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1182'},
        {'fromNode': '5336.0', 'toNode': '5146.0', 'Resistance': 0.00105, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.00418, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1183'},
        {'fromNode': '5336.0', 'toNode': '5146.0', 'Resistance': 0.00105, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.00418, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1184'},
        {'fromNode': '5336.0', 'toNode': '5146.0', 'Resistance': 0.00105, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.00418, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1185'},
        {'fromNode': '5146.0', 'toNode': '5412.0', 'Resistance': 0.00262, 'Reactance': 0.01534, 'Conductance': 0, 'Susceptance': 0.00766, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1186'},
        {'fromNode': '5147.0', 'toNode': '5188.0', 'Resistance': 0.00308, 'Reactance': 0.01312, 'Conductance': 0, 'Susceptance': 0.00656, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1187'},
        {'fromNode': '5147.0', 'toNode': '5224.0', 'Resistance': 0.00362, 'Reactance': 0.02124, 'Conductance': 0, 'Susceptance': 0.01061, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1188'},
        {'fromNode': '5148.0', 'toNode': '5335.0', 'Resistance': 0.00222, 'Reactance': 0.00945, 'Conductance': 0, 'Susceptance': 0.00473, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1189'},
        {'fromNode': '5148.0', 'toNode': '5457.0', 'Resistance': 0.00612, 'Reactance': 0.02601, 'Conductance': 0, 'Susceptance': 0.01301, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1190'},
        {'fromNode': '5463.0', 'toNode': '5148.0', 'Resistance': 0.00259, 'Reactance': 0.01144, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1191'},
        {'fromNode': '5294.0', 'toNode': '5149.0', 'Resistance': 0.00116, 'Reactance': 0.00679, 'Conductance': 0, 'Susceptance': 0.00339, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1192'},
        {'fromNode': '5149.0', 'toNode': '5309.0', 'Resistance': 0.0028, 'Reactance': 0.01233, 'Conductance': 0, 'Susceptance': 0.00574, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1193'},
        {'fromNode': '5197.0', 'toNode': '5150.0', 'Resistance': 0.00186, 'Reactance': 0.01364, 'Conductance': 0, 'Susceptance': 0.00773, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1194'},
        {'fromNode': '5150.0', 'toNode': '5432.0', 'Resistance': 0.01516, 'Reactance': 0.11531, 'Conductance': 0, 'Susceptance': 0.06054, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1195'},
        {'fromNode': '5154.0', 'toNode': '5151.0', 'Resistance': 0.00778, 'Reactance': 0.0343, 'Conductance': 0, 'Susceptance': 0.01597, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1196'},
        {'fromNode': '5191.0', 'toNode': '5151.0', 'Resistance': 0.008, 'Reactance': 0.03526, 'Conductance': 0, 'Susceptance': 0.01641, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1197'},
        {'fromNode': '5407.0', 'toNode': '5151.0', 'Resistance': 0.00814, 'Reactance': 0.03465, 'Conductance': 0, 'Susceptance': 0.01733, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1198'},
        {'fromNode': '5181.0', 'toNode': '5152.0', 'Resistance': 0.00134, 'Reactance': 0.00979, 'Conductance': 0, 'Susceptance': 0.00555, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1199'},
        {'fromNode': '5181.0', 'toNode': '5152.0', 'Resistance': 0.00134, 'Reactance': 0.00979, 'Conductance': 0, 'Susceptance': 0.00555, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1200'},
        {'fromNode': '5205.0', 'toNode': '5153.0', 'Resistance': 0.00418, 'Reactance': 0.03177, 'Conductance': 0, 'Susceptance': 0.01668, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1201'},
        {'fromNode': '5205.0', 'toNode': '5153.0', 'Resistance': 0.00418, 'Reactance': 0.03177, 'Conductance': 0, 'Susceptance': 0.01668, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1202'},
        {'fromNode': '5154.0', 'toNode': '5247.0', 'Resistance': 0.0072, 'Reactance': 0.03174, 'Conductance': 0, 'Susceptance': 0.01477, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1203'},
        {'fromNode': '5290.0', 'toNode': '5154.0', 'Resistance': 0.00545, 'Reactance': 0.03196, 'Conductance': 0, 'Susceptance': 0.01596, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1204'},
        {'fromNode': '5456.0', 'toNode': '5156.0', 'Resistance': 0.00659, 'Reactance': 0.03866, 'Conductance': 0, 'Susceptance': 0.0193, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1205'},
        {'fromNode': '5157.0', 'toNode': '5194.0', 'Resistance': 0.00346, 'Reactance': 0.01472, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1206'},
        {'fromNode': '5333.0', 'toNode': '5157.0', 'Resistance': 0.00236, 'Reactance': 0.01792, 'Conductance': 0, 'Susceptance': 0.00941, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1207'},
        {'fromNode': '5333.0', 'toNode': '5157.0', 'Resistance': 0.00236, 'Reactance': 0.01792, 'Conductance': 0, 'Susceptance': 0.00941, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1208'},
        {'fromNode': '5316.0', 'toNode': '5158.0', 'Resistance': 0.00121, 'Reactance': 0.0089, 'Conductance': 0, 'Susceptance': 0.00504, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1209'},
        {'fromNode': '5316.0', 'toNode': '5158.0', 'Resistance': 0.00121, 'Reactance': 0.0089, 'Conductance': 0, 'Susceptance': 0.00504, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1210'},
        {'fromNode': '5374.0', 'toNode': '5158.0', 'Resistance': 0.00143, 'Reactance': 0.01044, 'Conductance': 0, 'Susceptance': 0.00592, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1211'},
        {'fromNode': '5374.0', 'toNode': '5158.0', 'Resistance': 0.00143, 'Reactance': 0.01044, 'Conductance': 0, 'Susceptance': 0.00592, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1212'},
        {'fromNode': '5271.0', 'toNode': '5159.0', 'Resistance': 0.01947, 'Reactance': 0.08578, 'Conductance': 0, 'Susceptance': 0.03993, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1213'},
        {'fromNode': '5165.0', 'toNode': '5160.0', 'Resistance': 0.00151, 'Reactance': 0.00888, 'Conductance': 0, 'Susceptance': 0.00443, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1214'},
        {'fromNode': '5161.0', 'toNode': '5181.0', 'Resistance': 0.00545, 'Reactance': 0.02401, 'Conductance': 0, 'Susceptance': 0.01118, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1215'},
        {'fromNode': '5161.0', 'toNode': '5272.0', 'Resistance': 0.00401, 'Reactance': 0.01705, 'Conductance': 0, 'Susceptance': 0.00853, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1216'},
        {'fromNode': '5316.0', 'toNode': '5161.0', 'Resistance': 0.00144, 'Reactance': 0.00846, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1217'},
        {'fromNode': '5161.0', 'toNode': '5356.0', 'Resistance': 0.00487, 'Reactance': 0.02856, 'Conductance': 0, 'Susceptance': 0.01426, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1218'},
        {'fromNode': '5162.0', 'toNode': '5211.0', 'Resistance': 0.01066, 'Reactance': 0.04696, 'Conductance': 0, 'Susceptance': 0.02186, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1219'},
        {'fromNode': '5162.0', 'toNode': '5437.0', 'Resistance': 0.00505, 'Reactance': 0.02226, 'Conductance': 0, 'Susceptance': 0.01036, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1220'},
        {'fromNode': '5163.0', 'toNode': '5235.0', 'Resistance': 0.00709, 'Reactance': 0.03019, 'Conductance': 0, 'Susceptance': 0.0151, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1221'},
        {'fromNode': '5165.0', 'toNode': '5164.0', 'Resistance': 8e-05, 'Reactance': 0.00485, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1379.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1222'},
        {'fromNode': '5166.0', 'toNode': '5164.0', 'Resistance': 0.00042, 'Reactance': 0.02718, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 579.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1223'},
        {'fromNode': '5167.0', 'toNode': '5164.0', 'Resistance': 0.0012, 'Reactance': 0.09028, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 215.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1224'},
        {'fromNode': '5164.0', 'toNode': '5204.0', 'Resistance': 0.00057, 'Reactance': 0.01164, 'Conductance': 0, 'Susceptance': 0.81214, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1225'},
        {'fromNode': '5164.0', 'toNode': '5464.0', 'Resistance': 0.00018, 'Reactance': 0.00364, 'Conductance': 0, 'Susceptance': 0.25416, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1226'},
        {'fromNode': '5175.0', 'toNode': '5165.0', 'Resistance': 0.00126, 'Reactance': 0.00958, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1227'},
        {'fromNode': '5175.0', 'toNode': '5165.0', 'Resistance': 0.00126, 'Reactance': 0.00958, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1228'},
        {'fromNode': '5175.0', 'toNode': '5165.0', 'Resistance': 0.00126, 'Reactance': 0.00958, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1229'},
        {'fromNode': '5168.0', 'toNode': '5203.0', 'Resistance': 0.00632, 'Reactance': 0.02787, 'Conductance': 0, 'Susceptance': 0.01297, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1230'},
        {'fromNode': '5168.0', 'toNode': '5245.0', 'Resistance': 0.00821, 'Reactance': 0.03616, 'Conductance': 0, 'Susceptance': 0.01683, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1231'},
        {'fromNode': '5168.0', 'toNode': '5311.0', 'Resistance': 0.00162, 'Reactance': 0.00948, 'Conductance': 0, 'Susceptance': 0.00473, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1232'},
        {'fromNode': '5170.0', 'toNode': '5169.0', 'Resistance': 0.00185, 'Reactance': 0.06575, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 222.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1233'},
        {'fromNode': '5471.0', 'toNode': '5169.0', 'Resistance': 0.00598, 'Reactance': 0.02635, 'Conductance': 0, 'Susceptance': 0.01226, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1234'},
        {'fromNode': '5171.0', 'toNode': '5174.0', 'Resistance': 0.00411, 'Reactance': 0.02409, 'Conductance': 0, 'Susceptance': 0.01203, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1235'},
        {'fromNode': '5306.0', 'toNode': '5172.0', 'Resistance': 0.00438, 'Reactance': 0.02476, 'Conductance': 0, 'Susceptance': 0.01332, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1236'},
        {'fromNode': '5430.0', 'toNode': '5172.0', 'Resistance': 0.00249, 'Reactance': 0.01461, 'Conductance': 0, 'Susceptance': 0.0073, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1237'},
        {'fromNode': '5173.0', 'toNode': '5203.0', 'Resistance': 0.00375, 'Reactance': 0.02199, 'Conductance': 0, 'Susceptance': 0.01098, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1238'},
        {'fromNode': '5474.0', 'toNode': '5174.0', 'Resistance': 0.00547, 'Reactance': 0.03094, 'Conductance': 0, 'Susceptance': 0.01664, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1239'},
        {'fromNode': '5195.0', 'toNode': '5175.0', 'Resistance': 0.00254, 'Reactance': 0.01433, 'Conductance': 0, 'Susceptance': 0.00771, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1240'},
        {'fromNode': '5175.0', 'toNode': '5370.0', 'Resistance': 0.00192, 'Reactance': 0.01457, 'Conductance': 0, 'Susceptance': 0.00765, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1241'},
        {'fromNode': '5175.0', 'toNode': '5370.0', 'Resistance': 0.00192, 'Reactance': 0.01457, 'Conductance': 0, 'Susceptance': 0.00765, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1242'},
        {'fromNode': '5177.0', 'toNode': '5186.0', 'Resistance': 0.00555, 'Reactance': 0.04069, 'Conductance': 0, 'Susceptance': 0.02305, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1243'},
        {'fromNode': '5177.0', 'toNode': '5186.0', 'Resistance': 0.00555, 'Reactance': 0.04069, 'Conductance': 0, 'Susceptance': 0.02305, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1244'},
        {'fromNode': '5177.0', 'toNode': '5287.0', 'Resistance': 0.00483, 'Reactance': 0.02054, 'Conductance': 0, 'Susceptance': 0.01027, 'MVA_rating': 360.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1245'},
        {'fromNode': '5258.0', 'toNode': '5178.0', 'Resistance': 0.00517, 'Reactance': 0.02281, 'Conductance': 0, 'Susceptance': 0.01062, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1246'},
        {'fromNode': '5308.0', 'toNode': '5178.0', 'Resistance': 0.00366, 'Reactance': 0.02147, 'Conductance': 0, 'Susceptance': 0.01072, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1247'},
        {'fromNode': '5178.0', 'toNode': '5354.0', 'Resistance': 0.00769, 'Reactance': 0.03272, 'Conductance': 0, 'Susceptance': 0.01636, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1248'},
        {'fromNode': '5180.0', 'toNode': '5179.0', 'Resistance': 0.00019, 'Reactance': 0.01541, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 946.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1249'},
        {'fromNode': '5358.0', 'toNode': '5179.0', 'Resistance': 0.00053, 'Reactance': 0.01093, 'Conductance': 0, 'Susceptance': 0.76232, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1250'},
        {'fromNode': '5179.0', 'toNode': '5380.0', 'Resistance': 0.00018, 'Reactance': 0.004, 'Conductance': 0, 'Susceptance': 0.34742, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1251'},
        {'fromNode': '5179.0', 'toNode': '8043.0', 'Resistance': 0.00039, 'Reactance': 0.00798, 'Conductance': 0, 'Susceptance': 0.55658, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1252'},
        {'fromNode': '5179.0', 'toNode': '8057.0', 'Resistance': 0.00045, 'Reactance': 0.00923, 'Conductance': 0, 'Susceptance': 0.64362, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1253'},
        {'fromNode': '5180.0', 'toNode': '5485.0', 'Resistance': 0.00261, 'Reactance': 0.01983, 'Conductance': 0, 'Susceptance': 0.01041, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1254'},
        {'fromNode': '5182.0', 'toNode': '5314.0', 'Resistance': 0.01253, 'Reactance': 0.05522, 'Conductance': 0, 'Susceptance': 0.0257, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1255'},
        {'fromNode': '5183.0', 'toNode': '5224.0', 'Resistance': 0.00147, 'Reactance': 0.00831, 'Conductance': 0, 'Susceptance': 0.00447, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1256'},
        {'fromNode': '5184.0', 'toNode': '5373.0', 'Resistance': 0.00681, 'Reactance': 0.02896, 'Conductance': 0, 'Susceptance': 0.01448, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1257'},
        {'fromNode': '5387.0', 'toNode': '5184.0', 'Resistance': 0.00193, 'Reactance': 0.01132, 'Conductance': 0, 'Susceptance': 0.00565, 'MVA_rating': 263.55, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1258'},
        {'fromNode': '5185.0', 'toNode': '5218.0', 'Resistance': 0.00352, 'Reactance': 0.02066, 'Conductance': 0, 'Susceptance': 0.01032, 'MVA_rating': 281.12, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1259'},
        {'fromNode': '5434.0', 'toNode': '5185.0', 'Resistance': 0.00242, 'Reactance': 0.01843, 'Conductance': 0, 'Susceptance': 0.00967, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1260'},
        {'fromNode': '5434.0', 'toNode': '5185.0', 'Resistance': 0.00242, 'Reactance': 0.01843, 'Conductance': 0, 'Susceptance': 0.00967, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1261'},
        {'fromNode': '5186.0', 'toNode': '5355.0', 'Resistance': 0.00341, 'Reactance': 0.02495, 'Conductance': 0, 'Susceptance': 0.01414, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1262'},
        {'fromNode': '5186.0', 'toNode': '5355.0', 'Resistance': 0.00341, 'Reactance': 0.02495, 'Conductance': 0, 'Susceptance': 0.01414, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1263'},
        {'fromNode': '5188.0', 'toNode': '5203.0', 'Resistance': 0.00199, 'Reactance': 0.01122, 'Conductance': 0, 'Susceptance': 0.00604, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1264'},
        {'fromNode': '5188.0', 'toNode': '5234.0', 'Resistance': 0.00344, 'Reactance': 0.02618, 'Conductance': 0, 'Susceptance': 0.01374, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1265'},
        {'fromNode': '5188.0', 'toNode': '5253.0', 'Resistance': 0.00739, 'Reactance': 0.03143, 'Conductance': 0, 'Susceptance': 0.01572, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1266'},
        {'fromNode': '5189.0', 'toNode': '5230.0', 'Resistance': 0.00077, 'Reactance': 0.00451, 'Conductance': 0, 'Susceptance': 0.00225, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1267'},
        {'fromNode': '5189.0', 'toNode': '5272.0', 'Resistance': 0.00079, 'Reactance': 0.00346, 'Conductance': 0, 'Susceptance': 0.00161, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1268'},
        {'fromNode': '5189.0', 'toNode': '5352.0', 'Resistance': 0.00101, 'Reactance': 0.00766, 'Conductance': 0, 'Susceptance': 0.00402, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1269'},
        {'fromNode': '5189.0', 'toNode': '5352.0', 'Resistance': 0.00101, 'Reactance': 0.00766, 'Conductance': 0, 'Susceptance': 0.00402, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1270'},
        {'fromNode': '5190.0', 'toNode': '5255.0', 'Resistance': 0.01063, 'Reactance': 0.06231, 'Conductance': 0, 'Susceptance': 0.03111, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1271'},
        {'fromNode': '5190.0', 'toNode': '5264.0', 'Resistance': 0.01177, 'Reactance': 0.06902, 'Conductance': 0, 'Susceptance': 0.03446, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1272'},
        {'fromNode': '5190.0', 'toNode': '5430.0', 'Resistance': 0.01256, 'Reactance': 0.09556, 'Conductance': 0, 'Susceptance': 0.05017, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1273'},
        {'fromNode': '5191.0', 'toNode': '5407.0', 'Resistance': 0.00806, 'Reactance': 0.03551, 'Conductance': 0, 'Susceptance': 0.01653, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1274'},
        {'fromNode': '5193.0', 'toNode': '5192.0', 'Resistance': 0.00022, 'Reactance': 0.01302, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 890.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1275'},
        {'fromNode': '5192.0', 'toNode': '5279.0', 'Resistance': 0.00078, 'Reactance': 0.00941, 'Conductance': 0, 'Susceptance': 0.62487, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1276'},
        {'fromNode': '6316.0', 'toNode': '5193.0', 'Resistance': 0.00674, 'Reactance': 0.02866, 'Conductance': 0, 'Susceptance': 0.01433, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1277'},
        {'fromNode': '5194.0', 'toNode': '5330.0', 'Resistance': 0.0059, 'Reactance': 0.03459, 'Conductance': 0, 'Susceptance': 0.01728, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1278'},
        {'fromNode': '5194.0', 'toNode': '5376.0', 'Resistance': 0.00067, 'Reactance': 0.0051, 'Conductance': 0, 'Susceptance': 0.00268, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1279'},
        {'fromNode': '5194.0', 'toNode': '5462.0', 'Resistance': 0.00205, 'Reactance': 0.01202, 'Conductance': 0, 'Susceptance': 0.006, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1280'},
        {'fromNode': '5197.0', 'toNode': '5196.0', 'Resistance': 0.00034, 'Reactance': 0.03164, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 305.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1281'},
        {'fromNode': '5198.0', 'toNode': '5196.0', 'Resistance': 0.00018, 'Reactance': 0.01116, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 594.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1282'},
        {'fromNode': '5196.0', 'toNode': '5204.0', 'Resistance': 0.00058, 'Reactance': 0.00708, 'Conductance': 0, 'Susceptance': 0.47032, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1283'},
        {'fromNode': '5337.0', 'toNode': '5197.0', 'Resistance': 0.00702, 'Reactance': 0.03093, 'Conductance': 0, 'Susceptance': 0.0144, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1284'},
        {'fromNode': '5199.0', 'toNode': '5375.0', 'Resistance': 0.0038, 'Reactance': 0.01619, 'Conductance': 0, 'Susceptance': 0.0081, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1285'},
        {'fromNode': '5199.0', 'toNode': '5438.0', 'Resistance': 0.00104, 'Reactance': 0.00589, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1286'},
        {'fromNode': '5255.0', 'toNode': '5200.0', 'Resistance': 0.00506, 'Reactance': 0.02964, 'Conductance': 0, 'Susceptance': 0.0148, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1287'},
        {'fromNode': '5276.0', 'toNode': '5201.0', 'Resistance': 0.00365, 'Reactance': 0.02142, 'Conductance': 0, 'Susceptance': 0.0107, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1288'},
        {'fromNode': '5327.0', 'toNode': '5201.0', 'Resistance': 0.00494, 'Reactance': 0.02792, 'Conductance': 0, 'Susceptance': 0.01502, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1289'},
        {'fromNode': '5202.0', 'toNode': '5381.0', 'Resistance': 0.00904, 'Reactance': 0.03848, 'Conductance': 0, 'Susceptance': 0.01924, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1290'},
        {'fromNode': '5426.0', 'toNode': '5202.0', 'Resistance': 0.01126, 'Reactance': 0.04962, 'Conductance': 0, 'Susceptance': 0.0231, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1291'},
        {'fromNode': '5205.0', 'toNode': '5204.0', 'Resistance': 7e-05, 'Reactance': 0.00858, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1134.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1292'},
        {'fromNode': '5206.0', 'toNode': '5204.0', 'Resistance': 0.00033, 'Reactance': 0.01983, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 903.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1293'},
        {'fromNode': '5207.0', 'toNode': '5204.0', 'Resistance': 0.00016, 'Reactance': 0.01066, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 915.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1294'},
        {'fromNode': '5208.0', 'toNode': '5204.0', 'Resistance': 0.00019, 'Reactance': 0.01341, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 913.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1295'},
        {'fromNode': '5361.0', 'toNode': '5204.0', 'Resistance': 0.00011, 'Reactance': 0.00234, 'Conductance': 0, 'Susceptance': 0.16342, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1296'},
        {'fromNode': '5204.0', 'toNode': '5413.0', 'Resistance': 0.00075, 'Reactance': 0.00906, 'Conductance': 0, 'Susceptance': 0.60178, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1297'},
        {'fromNode': '5209.0', 'toNode': '5246.0', 'Resistance': 0.00635, 'Reactance': 0.02797, 'Conductance': 0, 'Susceptance': 0.01302, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1298'},
        {'fromNode': '5247.0', 'toNode': '5209.0', 'Resistance': 0.00664, 'Reactance': 0.02927, 'Conductance': 0, 'Susceptance': 0.01362, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1299'},
        {'fromNode': '5210.0', 'toNode': '5233.0', 'Resistance': 0.00199, 'Reactance': 0.01164, 'Conductance': 0, 'Susceptance': 0.00581, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1300'},
        {'fromNode': '5210.0', 'toNode': '5428.0', 'Resistance': 0.00112, 'Reactance': 0.00634, 'Conductance': 0, 'Susceptance': 0.00341, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1301'},
        {'fromNode': '5211.0', 'toNode': '5275.0', 'Resistance': 0.00613, 'Reactance': 0.02704, 'Conductance': 0, 'Susceptance': 0.01259, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1302'},
        {'fromNode': '5211.0', 'toNode': '5327.0', 'Resistance': 0.00415, 'Reactance': 0.02348, 'Conductance': 0, 'Susceptance': 0.01263, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1303'},
        {'fromNode': '5212.0', 'toNode': '5256.0', 'Resistance': 0.00634, 'Reactance': 0.04821, 'Conductance': 0, 'Susceptance': 0.02531, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1304'},
        {'fromNode': '5333.0', 'toNode': '5212.0', 'Resistance': 0.00352, 'Reactance': 0.02679, 'Conductance': 0, 'Susceptance': 0.01407, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1305'},
        {'fromNode': '5213.0', 'toNode': '5250.0', 'Resistance': 0.00303, 'Reactance': 0.02223, 'Conductance': 0, 'Susceptance': 0.01259, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1306'},
        {'fromNode': '5214.0', 'toNode': '5229.0', 'Resistance': 0.00374, 'Reactance': 0.01647, 'Conductance': 0, 'Susceptance': 0.00767, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1307'},
        {'fromNode': '5468.0', 'toNode': '5214.0', 'Resistance': 0.00075, 'Reactance': 0.0057, 'Conductance': 0, 'Susceptance': 0.00299, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1308'},
        {'fromNode': '5215.0', 'toNode': '5277.0', 'Resistance': 0.00671, 'Reactance': 0.03931, 'Conductance': 0, 'Susceptance': 0.01963, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1309'},
        {'fromNode': '5216.0', 'toNode': '5442.0', 'Resistance': 0.00087, 'Reactance': 0.00508, 'Conductance': 0, 'Susceptance': 0.00254, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1310'},
        {'fromNode': '5216.0', 'toNode': '5443.0', 'Resistance': 0.00194, 'Reactance': 0.00852, 'Conductance': 0, 'Susceptance': 0.00397, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1311'},
        {'fromNode': '5217.0', 'toNode': '5310.0', 'Resistance': 0.00171, 'Reactance': 0.00755, 'Conductance': 0, 'Susceptance': 0.00351, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1312'},
        {'fromNode': '5217.0', 'toNode': '5310.0', 'Resistance': 0.00171, 'Reactance': 0.00755, 'Conductance': 0, 'Susceptance': 0.00351, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1313'},
        {'fromNode': '5217.0', 'toNode': '5433.0', 'Resistance': 0.00438, 'Reactance': 0.01931, 'Conductance': 0, 'Susceptance': 0.00899, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1314'},
        {'fromNode': '5483.0', 'toNode': '5217.0', 'Resistance': 0.00126, 'Reactance': 0.00922, 'Conductance': 0, 'Susceptance': 0.00522, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1315'},
        {'fromNode': '5483.0', 'toNode': '5217.0', 'Resistance': 0.00126, 'Reactance': 0.00922, 'Conductance': 0, 'Susceptance': 0.00522, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1316'},
        {'fromNode': '5219.0', 'toNode': '5371.0', 'Resistance': 0.00781, 'Reactance': 0.03325, 'Conductance': 0, 'Susceptance': 0.01663, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1317'},
        {'fromNode': '5221.0', 'toNode': '5411.0', 'Resistance': 0.00803, 'Reactance': 0.03536, 'Conductance': 0, 'Susceptance': 0.01646, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1318'},
        {'fromNode': '5222.0', 'toNode': '5426.0', 'Resistance': 0.00404, 'Reactance': 0.0307, 'Conductance': 0, 'Susceptance': 0.01612, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1319'},
        {'fromNode': '5222.0', 'toNode': '5426.0', 'Resistance': 0.00404, 'Reactance': 0.0307, 'Conductance': 0, 'Susceptance': 0.01612, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1320'},
        {'fromNode': '5223.0', 'toNode': '5275.0', 'Resistance': 0.00911, 'Reactance': 0.05341, 'Conductance': 0, 'Susceptance': 0.02667, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1321'},
        {'fromNode': '5307.0', 'toNode': '5224.0', 'Resistance': 0.00171, 'Reactance': 0.01303, 'Conductance': 0, 'Susceptance': 0.00684, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1322'},
        {'fromNode': '5225.0', 'toNode': '5259.0', 'Resistance': 0.00243, 'Reactance': 0.01424, 'Conductance': 0, 'Susceptance': 0.00711, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1323'},
        {'fromNode': '5421.0', 'toNode': '5225.0', 'Resistance': 0.00183, 'Reactance': 0.0134, 'Conductance': 0, 'Susceptance': 0.00759, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1324'},
        {'fromNode': '5227.0', 'toNode': '5354.0', 'Resistance': 0.00753, 'Reactance': 0.04412, 'Conductance': 0, 'Susceptance': 0.02203, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1325'},
        {'fromNode': '5313.0', 'toNode': '5228.0', 'Resistance': 0.00155, 'Reactance': 0.01183, 'Conductance': 0, 'Susceptance': 0.00621, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1326'},
        {'fromNode': '5313.0', 'toNode': '5228.0', 'Resistance': 0.00155, 'Reactance': 0.01183, 'Conductance': 0, 'Susceptance': 0.00621, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1327'},
        {'fromNode': '5313.0', 'toNode': '5228.0', 'Resistance': 0.00155, 'Reactance': 0.01183, 'Conductance': 0, 'Susceptance': 0.00621, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1328'},
        {'fromNode': '5349.0', 'toNode': '5228.0', 'Resistance': 0.0013, 'Reactance': 0.00988, 'Conductance': 0, 'Susceptance': 0.00519, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1329'},
        {'fromNode': '5229.0', 'toNode': '5309.0', 'Resistance': 0.0027, 'Reactance': 0.01582, 'Conductance': 0, 'Susceptance': 0.0079, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1330'},
        {'fromNode': '5230.0', 'toNode': '5478.0', 'Resistance': 0.00568, 'Reactance': 0.02417, 'Conductance': 0, 'Susceptance': 0.01209, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1331'},
        {'fromNode': '5286.0', 'toNode': '5231.0', 'Resistance': 0.01945, 'Reactance': 0.08572, 'Conductance': 0, 'Susceptance': 0.0399, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1332'},
        {'fromNode': '5232.0', 'toNode': '5251.0', 'Resistance': 0.00634, 'Reactance': 0.02792, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1333'},
        {'fromNode': '5440.0', 'toNode': '5232.0', 'Resistance': 0.00256, 'Reactance': 0.01088, 'Conductance': 0, 'Susceptance': 0.00544, 'MVA_rating': 300.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1334'},
        {'fromNode': '5232.0', 'toNode': '5442.0', 'Resistance': 0.00617, 'Reactance': 0.02716, 'Conductance': 0, 'Susceptance': 0.01264, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1335'},
        {'fromNode': '5233.0', 'toNode': '5339.0', 'Resistance': 0.00532, 'Reactance': 0.02343, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1336'},
        {'fromNode': '5234.0', 'toNode': '5296.0', 'Resistance': 0.00408, 'Reactance': 0.03102, 'Conductance': 0, 'Susceptance': 0.01628, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1337'},
        {'fromNode': '5234.0', 'toNode': '5296.0', 'Resistance': 0.00408, 'Reactance': 0.03102, 'Conductance': 0, 'Susceptance': 0.01628, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1338'},
        {'fromNode': '5235.0', 'toNode': '5271.0', 'Resistance': 0.01401, 'Reactance': 0.06175, 'Conductance': 0, 'Susceptance': 0.02875, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1339'},
        {'fromNode': '5411.0', 'toNode': '5235.0', 'Resistance': 0.00717, 'Reactance': 0.03052, 'Conductance': 0, 'Susceptance': 0.01526, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1340'},
        {'fromNode': '5237.0', 'toNode': '5236.0', 'Resistance': 0.00052, 'Reactance': 0.04304, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 287.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1341'},
        {'fromNode': '5238.0', 'toNode': '5236.0', 'Resistance': 0.00016, 'Reactance': 0.00976, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 826.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1342'},
        {'fromNode': '5236.0', 'toNode': '5394.0', 'Resistance': 0.00122, 'Reactance': 0.01485, 'Conductance': 0, 'Susceptance': 0.98624, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1343'},
        {'fromNode': '5240.0', 'toNode': '5239.0', 'Resistance': 0.00052, 'Reactance': 0.03641, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 477.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1344'},
        {'fromNode': '5243.0', 'toNode': '5239.0', 'Resistance': 0.00038, 'Reactance': 0.02602, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 584.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1345'},
        {'fromNode': '5451.0', 'toNode': '5239.0', 'Resistance': 0.00065, 'Reactance': 0.00791, 'Conductance': 0, 'Susceptance': 0.52569, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1346'},
        {'fromNode': '5239.0', 'toNode': '6210.0', 'Resistance': 0.00119, 'Reactance': 0.01447, 'Conductance': 0, 'Susceptance': 0.96116, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1347'},
        {'fromNode': '5241.0', 'toNode': '5240.0', 'Resistance': 0.00087, 'Reactance': 0.0425, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 204.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1348'},
        {'fromNode': '5242.0', 'toNode': '5241.0', 'Resistance': 0.01845, 'Reactance': 0.47482, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 24.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1349'},
        {'fromNode': '5244.0', 'toNode': '5306.0', 'Resistance': 0.00707, 'Reactance': 0.04143, 'Conductance': 0, 'Susceptance': 0.02069, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1350'},
        {'fromNode': '5244.0', 'toNode': '5442.0', 'Resistance': 0.00508, 'Reactance': 0.02158, 'Conductance': 0, 'Susceptance': 0.01079, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1351'},
        {'fromNode': '5245.0', 'toNode': '5254.0', 'Resistance': 0.00731, 'Reactance': 0.03219, 'Conductance': 0, 'Susceptance': 0.01498, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1352'},
        {'fromNode': '5245.0', 'toNode': '5339.0', 'Resistance': 0.00345, 'Reactance': 0.01469, 'Conductance': 0, 'Susceptance': 0.00734, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1353'},
        {'fromNode': '5261.0', 'toNode': '5246.0', 'Resistance': 0.0031, 'Reactance': 0.01817, 'Conductance': 0, 'Susceptance': 0.00907, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1354'},
        {'fromNode': '5248.0', 'toNode': '5428.0', 'Resistance': 0.0019, 'Reactance': 0.01112, 'Conductance': 0, 'Susceptance': 0.00555, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1355'},
        {'fromNode': '5248.0', 'toNode': '5457.0', 'Resistance': 0.00198, 'Reactance': 0.00875, 'Conductance': 0, 'Susceptance': 0.00407, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1356'},
        {'fromNode': '5248.0', 'toNode': '5483.0', 'Resistance': 0.00348, 'Reactance': 0.01531, 'Conductance': 0, 'Susceptance': 0.00713, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1357'},
        {'fromNode': '5292.0', 'toNode': '5249.0', 'Resistance': 0.0018, 'Reactance': 0.0137, 'Conductance': 0, 'Susceptance': 0.0072, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1358'},
        {'fromNode': '5292.0', 'toNode': '5249.0', 'Resistance': 0.0018, 'Reactance': 0.0137, 'Conductance': 0, 'Susceptance': 0.0072, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1359'},
        {'fromNode': '5250.0', 'toNode': '5385.0', 'Resistance': 0.00149, 'Reactance': 0.0113, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1360'},
        {'fromNode': '5250.0', 'toNode': '5385.0', 'Resistance': 0.00149, 'Reactance': 0.0113, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1361'},
        {'fromNode': '5250.0', 'toNode': '5385.0', 'Resistance': 0.00149, 'Reactance': 0.0113, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1362'},
        {'fromNode': '5250.0', 'toNode': '5385.0', 'Resistance': 0.00149, 'Reactance': 0.0113, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 500.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1363'},
        {'fromNode': '5251.0', 'toNode': '5478.0', 'Resistance': 0.00129, 'Reactance': 0.00945, 'Conductance': 0, 'Susceptance': 0.00536, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1364'},
        {'fromNode': '5251.0', 'toNode': '5478.0', 'Resistance': 0.00129, 'Reactance': 0.00945, 'Conductance': 0, 'Susceptance': 0.00536, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1365'},
        {'fromNode': '5252.0', 'toNode': '5340.0', 'Resistance': 0.00558, 'Reactance': 0.0237, 'Conductance': 0, 'Susceptance': 0.01185, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1366'},
        {'fromNode': '5252.0', 'toNode': '5436.0', 'Resistance': 0.00133, 'Reactance': 0.00585, 'Conductance': 0, 'Susceptance': 0.00272, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1367'},
        {'fromNode': '5370.0', 'toNode': '5253.0', 'Resistance': 0.00126, 'Reactance': 0.00956, 'Conductance': 0, 'Susceptance': 0.00502, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1368'},
        {'fromNode': '5370.0', 'toNode': '5253.0', 'Resistance': 0.00126, 'Reactance': 0.00956, 'Conductance': 0, 'Susceptance': 0.00502, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1369'},
        {'fromNode': '5254.0', 'toNode': '5332.0', 'Resistance': 0.00765, 'Reactance': 0.03373, 'Conductance': 0, 'Susceptance': 0.0157, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1370'},
        {'fromNode': '5254.0', 'toNode': '5420.0', 'Resistance': 0.00202, 'Reactance': 0.01533, 'Conductance': 0, 'Susceptance': 0.00805, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1371'},
        {'fromNode': '5254.0', 'toNode': '5420.0', 'Resistance': 0.00202, 'Reactance': 0.01533, 'Conductance': 0, 'Susceptance': 0.00805, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1372'},
        {'fromNode': '5254.0', 'toNode': '5457.0', 'Resistance': 0.00634, 'Reactance': 0.02792, 'Conductance': 0, 'Susceptance': 0.01299, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1373'},
        {'fromNode': '5257.0', 'toNode': '5256.0', 'Resistance': 0.00207, 'Reactance': 0.0117, 'Conductance': 0, 'Susceptance': 0.00629, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1374'},
        {'fromNode': '5256.0', 'toNode': '5427.0', 'Resistance': 0.00474, 'Reactance': 0.02777, 'Conductance': 0, 'Susceptance': 0.01387, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1375'},
        {'fromNode': '5439.0', 'toNode': '5256.0', 'Resistance': 0.0037, 'Reactance': 0.02813, 'Conductance': 0, 'Susceptance': 0.01477, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1376'},
        {'fromNode': '5308.0', 'toNode': '5258.0', 'Resistance': 0.00332, 'Reactance': 0.01947, 'Conductance': 0, 'Susceptance': 0.00972, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1377'},
        {'fromNode': '5259.0', 'toNode': '5412.0', 'Resistance': 0.00233, 'Reactance': 0.01365, 'Conductance': 0, 'Susceptance': 0.00681, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1378'},
        {'fromNode': '5261.0', 'toNode': '5260.0', 'Resistance': 0.00045, 'Reactance': 0.02804, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 375.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1379'},
        {'fromNode': '5262.0', 'toNode': '5260.0', 'Resistance': 0.0001, 'Reactance': 0.00632, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1450.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1380'},
        {'fromNode': '5263.0', 'toNode': '5260.0', 'Resistance': 6e-05, 'Reactance': 0.00451, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1450.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1381'},
        {'fromNode': '5317.0', 'toNode': '5260.0', 'Resistance': 4e-05, 'Reactance': 0.00088, 'Conductance': 0, 'Susceptance': 0.07608, 'MVA_rating': 4352.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1382'},
        {'fromNode': '5264.0', 'toNode': '5471.0', 'Resistance': 0.00664, 'Reactance': 0.03754, 'Conductance': 0, 'Susceptance': 0.02019, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1383'},
        {'fromNode': '5483.0', 'toNode': '5265.0', 'Resistance': 0.00103, 'Reactance': 0.00785, 'Conductance': 0, 'Susceptance': 0.00412, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1384'},
        {'fromNode': '5483.0', 'toNode': '5265.0', 'Resistance': 0.00103, 'Reactance': 0.00785, 'Conductance': 0, 'Susceptance': 0.00412, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1385'},
        {'fromNode': '5483.0', 'toNode': '5265.0', 'Resistance': 0.00103, 'Reactance': 0.00785, 'Conductance': 0, 'Susceptance': 0.00412, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1386'},
        {'fromNode': '5287.0', 'toNode': '5266.0', 'Resistance': 0.0022, 'Reactance': 0.01287, 'Conductance': 0, 'Susceptance': 0.00643, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1387'},
        {'fromNode': '5267.0', 'toNode': '5466.0', 'Resistance': 0.00868, 'Reactance': 0.03823, 'Conductance': 0, 'Susceptance': 0.01779, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1388'},
        {'fromNode': '5470.0', 'toNode': '5267.0', 'Resistance': 0.01229, 'Reactance': 0.05415, 'Conductance': 0, 'Susceptance': 0.02521, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1389'},
        {'fromNode': '5268.0', 'toNode': '5288.0', 'Resistance': 0.00791, 'Reactance': 0.03365, 'Conductance': 0, 'Susceptance': 0.01683, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1390'},
        {'fromNode': '5427.0', 'toNode': '5269.0', 'Resistance': 0.00325, 'Reactance': 0.02378, 'Conductance': 0, 'Susceptance': 0.01347, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1391'},
        {'fromNode': '5445.0', 'toNode': '5269.0', 'Resistance': 0.0017, 'Reactance': 0.01248, 'Conductance': 0, 'Susceptance': 0.00707, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1392'},
        {'fromNode': '5445.0', 'toNode': '5269.0', 'Resistance': 0.0017, 'Reactance': 0.01248, 'Conductance': 0, 'Susceptance': 0.00707, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1393'},
        {'fromNode': '5270.0', 'toNode': '5389.0', 'Resistance': 0.00793, 'Reactance': 0.04646, 'Conductance': 0, 'Susceptance': 0.0232, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1394'},
        {'fromNode': '5270.0', 'toNode': '5437.0', 'Resistance': 0.00885, 'Reactance': 0.03762, 'Conductance': 0, 'Susceptance': 0.01882, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1395'},
        {'fromNode': '5270.0', 'toNode': '5474.0', 'Resistance': 0.00513, 'Reactance': 0.03004, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1396'},
        {'fromNode': '5271.0', 'toNode': '5390.0', 'Resistance': 0.00759, 'Reactance': 0.03346, 'Conductance': 0, 'Susceptance': 0.01557, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1397'},
        {'fromNode': '5467.0', 'toNode': '5272.0', 'Resistance': 0.00108, 'Reactance': 0.00476, 'Conductance': 0, 'Susceptance': 0.00222, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1398'},
        {'fromNode': '5273.0', 'toNode': '5339.0', 'Resistance': 0.00649, 'Reactance': 0.02763, 'Conductance': 0, 'Susceptance': 0.01382, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1399'},
        {'fromNode': '5273.0', 'toNode': '5422.0', 'Resistance': 0.00204, 'Reactance': 0.01195, 'Conductance': 0, 'Susceptance': 0.00597, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1400'},
        {'fromNode': '5274.0', 'toNode': '5329.0', 'Resistance': 0.00478, 'Reactance': 0.02801, 'Conductance': 0, 'Susceptance': 0.01399, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1401'},
        {'fromNode': '5274.0', 'toNode': '5374.0', 'Resistance': 0.00568, 'Reactance': 0.02418, 'Conductance': 0, 'Susceptance': 0.01209, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1402'},
        {'fromNode': '5275.0', 'toNode': '5476.0', 'Resistance': 0.01075, 'Reactance': 0.063, 'Conductance': 0, 'Susceptance': 0.03146, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1403'},
        {'fromNode': '5359.0', 'toNode': '5276.0', 'Resistance': 0.00542, 'Reactance': 0.03062, 'Conductance': 0, 'Susceptance': 0.01647, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1404'},
        {'fromNode': '5277.0', 'toNode': '5315.0', 'Resistance': 0.00953, 'Reactance': 0.04199, 'Conductance': 0, 'Susceptance': 0.01955, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1405'},
        {'fromNode': '5280.0', 'toNode': '5278.0', 'Resistance': 0.00202, 'Reactance': 0.01184, 'Conductance': 0, 'Susceptance': 0.00591, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1406'},
        {'fromNode': '5390.0', 'toNode': '5278.0', 'Resistance': 0.00646, 'Reactance': 0.02848, 'Conductance': 0, 'Susceptance': 0.01326, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1407'},
        {'fromNode': '5280.0', 'toNode': '5279.0', 'Resistance': 0.0001, 'Reactance': 0.00635, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1112.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1408'},
        {'fromNode': '5281.0', 'toNode': '5279.0', 'Resistance': 0.00021, 'Reactance': 0.01977, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 942.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1409'},
        {'fromNode': '5282.0', 'toNode': '5279.0', 'Resistance': 0.00021, 'Reactance': 0.01672, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 940.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1410'},
        {'fromNode': '5283.0', 'toNode': '5279.0', 'Resistance': 0.00027, 'Reactance': 0.01279, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 935.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1411'},
        {'fromNode': '5279.0', 'toNode': '5358.0', 'Resistance': 0.00074, 'Reactance': 0.00901, 'Conductance': 0, 'Susceptance': 0.5983, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1412'},
        {'fromNode': '5280.0', 'toNode': '5461.0', 'Resistance': 0.00246, 'Reactance': 0.01871, 'Conductance': 0, 'Susceptance': 0.00982, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1413'},
        {'fromNode': '5284.0', 'toNode': '5454.0', 'Resistance': 0.00377, 'Reactance': 0.02759, 'Conductance': 0, 'Susceptance': 0.01564, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1414'},
        {'fromNode': '5289.0', 'toNode': '5436.0', 'Resistance': 0.00048, 'Reactance': 0.00367, 'Conductance': 0, 'Susceptance': 0.00192, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1415'},
        {'fromNode': '5289.0', 'toNode': '5436.0', 'Resistance': 0.00048, 'Reactance': 0.00367, 'Conductance': 0, 'Susceptance': 0.00192, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1416'},
        {'fromNode': '5290.0', 'toNode': '5345.0', 'Resistance': 0.00613, 'Reactance': 0.03593, 'Conductance': 0, 'Susceptance': 0.01794, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1417'},
        {'fromNode': '5291.0', 'toNode': '5292.0', 'Resistance': 0.00267, 'Reactance': 0.01563, 'Conductance': 0, 'Susceptance': 0.0078, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1418'},
        {'fromNode': '5429.0', 'toNode': '5291.0', 'Resistance': 0.00544, 'Reactance': 0.024, 'Conductance': 0, 'Susceptance': 0.01117, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1419'},
        {'fromNode': '5291.0', 'toNode': '5438.0', 'Resistance': 0.00699, 'Reactance': 0.03084, 'Conductance': 0, 'Susceptance': 0.01436, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1420'},
        {'fromNode': '5292.0', 'toNode': '5312.0', 'Resistance': 0.004, 'Reactance': 0.01704, 'Conductance': 0, 'Susceptance': 0.00852, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1421'},
        {'fromNode': '5293.0', 'toNode': '5338.0', 'Resistance': 0.00404, 'Reactance': 0.03072, 'Conductance': 0, 'Susceptance': 0.01613, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1422'},
        {'fromNode': '5293.0', 'toNode': '5338.0', 'Resistance': 0.00404, 'Reactance': 0.03072, 'Conductance': 0, 'Susceptance': 0.01613, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1423'},
        {'fromNode': '5293.0', 'toNode': '5402.0', 'Resistance': 0.00355, 'Reactance': 0.02603, 'Conductance': 0, 'Susceptance': 0.01475, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1424'},
        {'fromNode': '5293.0', 'toNode': '5402.0', 'Resistance': 0.00355, 'Reactance': 0.02603, 'Conductance': 0, 'Susceptance': 0.01475, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1425'},
        {'fromNode': '5296.0', 'toNode': '5295.0', 'Resistance': 0.00014, 'Reactance': 0.00709, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1233.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1426'},
        {'fromNode': '5297.0', 'toNode': '5295.0', 'Resistance': 0.00085, 'Reactance': 0.06248, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 223.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1427'},
        {'fromNode': '5298.0', 'toNode': '5295.0', 'Resistance': 0.00015, 'Reactance': 0.01082, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 525.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1428'},
        {'fromNode': '5299.0', 'toNode': '5295.0', 'Resistance': 0.0003, 'Reactance': 0.02227, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 524.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1429'},
        {'fromNode': '5300.0', 'toNode': '5295.0', 'Resistance': 0.00025, 'Reactance': 0.01848, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 519.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1430'},
        {'fromNode': '5301.0', 'toNode': '5295.0', 'Resistance': 0.00058, 'Reactance': 0.03088, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 403.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1431'},
        {'fromNode': '5302.0', 'toNode': '5295.0', 'Resistance': 0.0003, 'Reactance': 0.01763, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1123.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1432'},
        {'fromNode': '5303.0', 'toNode': '5295.0', 'Resistance': 0.00017, 'Reactance': 0.01315, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1117.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1433'},
        {'fromNode': '5304.0', 'toNode': '5295.0', 'Resistance': 0.00011, 'Reactance': 0.00134, 'Conductance': 0, 'Susceptance': 0.08928, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1434'},
        {'fromNode': '5295.0', 'toNode': '5444.0', 'Resistance': 0.00069, 'Reactance': 0.01404, 'Conductance': 0, 'Susceptance': 0.97894, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1435'},
        {'fromNode': '5296.0', 'toNode': '5400.0', 'Resistance': 0.00214, 'Reactance': 0.01566, 'Conductance': 0, 'Susceptance': 0.00887, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1436'},
        {'fromNode': '5305.0', 'toNode': '5304.0', 'Resistance': 0.00014, 'Reactance': 0.01047, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1239.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1437'},
        {'fromNode': '5307.0', 'toNode': '5341.0', 'Resistance': 0.00111, 'Reactance': 0.00847, 'Conductance': 0, 'Susceptance': 0.00445, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1438'},
        {'fromNode': '5307.0', 'toNode': '5341.0', 'Resistance': 0.00111, 'Reactance': 0.00847, 'Conductance': 0, 'Susceptance': 0.00445, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1439'},
        {'fromNode': '5351.0', 'toNode': '5308.0', 'Resistance': 0.0016, 'Reactance': 0.01174, 'Conductance': 0, 'Susceptance': 0.00665, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1440'},
        {'fromNode': '5351.0', 'toNode': '5308.0', 'Resistance': 0.0016, 'Reactance': 0.01174, 'Conductance': 0, 'Susceptance': 0.00665, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1441'},
        {'fromNode': '5310.0', 'toNode': '5339.0', 'Resistance': 0.00596, 'Reactance': 0.02626, 'Conductance': 0, 'Susceptance': 0.01222, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1442'},
        {'fromNode': '5310.0', 'toNode': '5379.0', 'Resistance': 0.00162, 'Reactance': 0.00712, 'Conductance': 0, 'Susceptance': 0.00331, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1443'},
        {'fromNode': '5310.0', 'toNode': '5438.0', 'Resistance': 0.00421, 'Reactance': 0.02468, 'Conductance': 0, 'Susceptance': 0.01232, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1444'},
        {'fromNode': '5316.0', 'toNode': '5354.0', 'Resistance': 0.00517, 'Reactance': 0.02278, 'Conductance': 0, 'Susceptance': 0.01061, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1445'},
        {'fromNode': '5318.0', 'toNode': '5317.0', 'Resistance': 0.00013, 'Reactance': 0.01555, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1106.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1446'},
        {'fromNode': '5319.0', 'toNode': '5317.0', 'Resistance': 0.00024, 'Reactance': 0.01893, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 925.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1447'},
        {'fromNode': '5320.0', 'toNode': '5317.0', 'Resistance': 0.00046, 'Reactance': 0.02092, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 917.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1448'},
        {'fromNode': '5321.0', 'toNode': '5317.0', 'Resistance': 0.00026, 'Reactance': 0.01811, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 921.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1449'},
        {'fromNode': '5388.0', 'toNode': '5317.0', 'Resistance': 0.00128, 'Reactance': 0.01548, 'Conductance': 0, 'Susceptance': 1.02824, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1450'},
        {'fromNode': '5401.0', 'toNode': '5317.0', 'Resistance': 5e-05, 'Reactance': 0.00111, 'Conductance': 0, 'Susceptance': 0.09678, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1451'},
        {'fromNode': '5401.0', 'toNode': '5317.0', 'Resistance': 5e-05, 'Reactance': 0.00111, 'Conductance': 0, 'Susceptance': 0.09678, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1452'},
        {'fromNode': '5473.0', 'toNode': '5318.0', 'Resistance': 0.00242, 'Reactance': 0.01844, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 347.2, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1453'},
        {'fromNode': '5473.0', 'toNode': '5318.0', 'Resistance': 0.00242, 'Reactance': 0.01844, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 347.2, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1454'},
        {'fromNode': '5323.0', 'toNode': '5322.0', 'Resistance': 0.00012, 'Reactance': 0.01249, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1334.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1455'},
        {'fromNode': '5323.0', 'toNode': '5322.0', 'Resistance': 0.00012, 'Reactance': 0.01249, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1334.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1456'},
        {'fromNode': '5324.0', 'toNode': '5322.0', 'Resistance': 0.00037, 'Reactance': 0.02069, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 952.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1457'},
        {'fromNode': '5325.0', 'toNode': '5322.0', 'Resistance': 0.00022, 'Reactance': 0.0207, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 945.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1458'},
        {'fromNode': '5326.0', 'toNode': '5322.0', 'Resistance': 0.00014, 'Reactance': 0.00711, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 953.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1459'},
        {'fromNode': '5322.0', 'toNode': '5384.0', 'Resistance': 0.00031, 'Reactance': 0.00376, 'Conductance': 0, 'Susceptance': 0.24969, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1460'},
        {'fromNode': '5448.0', 'toNode': '5322.0', 'Resistance': 0.00014, 'Reactance': 0.00164, 'Conductance': 0, 'Susceptance': 0.10912, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1461'},
        {'fromNode': '5376.0', 'toNode': '5323.0', 'Resistance': 0.00121, 'Reactance': 0.00887, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1462'},
        {'fromNode': '5376.0', 'toNode': '5323.0', 'Resistance': 0.00121, 'Reactance': 0.00887, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1463'},
        {'fromNode': '5323.0', 'toNode': '5425.0', 'Resistance': 0.00265, 'Reactance': 0.02018, 'Conductance': 0, 'Susceptance': 0.01059, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1464'},
        {'fromNode': '5323.0', 'toNode': '5425.0', 'Resistance': 0.00265, 'Reactance': 0.02018, 'Conductance': 0, 'Susceptance': 0.01059, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1465'},
        {'fromNode': '5446.0', 'toNode': '5328.0', 'Resistance': 0.00871, 'Reactance': 0.03704, 'Conductance': 0, 'Susceptance': 0.01852, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1466'},
        {'fromNode': '5330.0', 'toNode': '5433.0', 'Resistance': 0.00293, 'Reactance': 0.02226, 'Conductance': 0, 'Susceptance': 0.01169, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1467'},
        {'fromNode': '5400.0', 'toNode': '5331.0', 'Resistance': 0.00473, 'Reactance': 0.02774, 'Conductance': 0, 'Susceptance': 0.01385, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1468'},
        {'fromNode': '5450.0', 'toNode': '5331.0', 'Resistance': 0.00779, 'Reactance': 0.03434, 'Conductance': 0, 'Susceptance': 0.01599, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1469'},
        {'fromNode': '5429.0', 'toNode': '5332.0', 'Resistance': 0.00313, 'Reactance': 0.01379, 'Conductance': 0, 'Susceptance': 0.00642, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1470'},
        {'fromNode': '5427.0', 'toNode': '5333.0', 'Resistance': 0.00478, 'Reactance': 0.03636, 'Conductance': 0, 'Susceptance': 0.01909, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1471'},
        {'fromNode': '5427.0', 'toNode': '5333.0', 'Resistance': 0.00478, 'Reactance': 0.03636, 'Conductance': 0, 'Susceptance': 0.01909, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1472'},
        {'fromNode': '5334.0', 'toNode': '5484.0', 'Resistance': 0.01111, 'Reactance': 0.06511, 'Conductance': 0, 'Susceptance': 0.03251, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1473'},
        {'fromNode': '5335.0', 'toNode': '5372.0', 'Resistance': 0.00458, 'Reactance': 0.02019, 'Conductance': 0, 'Susceptance': 0.0094, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1474'},
        {'fromNode': '5335.0', 'toNode': '5425.0', 'Resistance': 0.00232, 'Reactance': 0.01697, 'Conductance': 0, 'Susceptance': 0.00961, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1475'},
        {'fromNode': '5351.0', 'toNode': '5336.0', 'Resistance': 0.00118, 'Reactance': 0.00899, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1476'},
        {'fromNode': '5351.0', 'toNode': '5336.0', 'Resistance': 0.00118, 'Reactance': 0.00899, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1477'},
        {'fromNode': '5351.0', 'toNode': '5336.0', 'Resistance': 0.00118, 'Reactance': 0.00899, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1478'},
        {'fromNode': '5481.0', 'toNode': '5338.0', 'Resistance': 0.00337, 'Reactance': 0.02561, 'Conductance': 0, 'Susceptance': 0.01345, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1479'},
        {'fromNode': '5481.0', 'toNode': '5338.0', 'Resistance': 0.00337, 'Reactance': 0.02561, 'Conductance': 0, 'Susceptance': 0.01345, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1480'},
        {'fromNode': '5340.0', 'toNode': '5462.0', 'Resistance': 0.00437, 'Reactance': 0.02559, 'Conductance': 0, 'Susceptance': 0.01278, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1481'},
        {'fromNode': '5340.0', 'toNode': '5467.0', 'Resistance': 0.00082, 'Reactance': 0.00348, 'Conductance': 0, 'Susceptance': 0.00174, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1482'},
        {'fromNode': '5342.0', 'toNode': '5341.0', 'Resistance': 0.01026, 'Reactance': 0.25106, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 49.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1483'},
        {'fromNode': '5343.0', 'toNode': '5341.0', 'Resistance': 0.00091, 'Reactance': 0.04835, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1484'},
        {'fromNode': '5422.0', 'toNode': '5341.0', 'Resistance': 0.00126, 'Reactance': 0.00925, 'Conductance': 0, 'Susceptance': 0.00524, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1485'},
        {'fromNode': '5422.0', 'toNode': '5341.0', 'Resistance': 0.00126, 'Reactance': 0.00925, 'Conductance': 0, 'Susceptance': 0.00524, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1486'},
        {'fromNode': '5341.0', 'toNode': '5465.0', 'Resistance': 0.00307, 'Reactance': 0.02252, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1487'},
        {'fromNode': '5341.0', 'toNode': '5465.0', 'Resistance': 0.00307, 'Reactance': 0.02252, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1488'},
        {'fromNode': '5341.0', 'toNode': '5465.0', 'Resistance': 0.00307, 'Reactance': 0.02252, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1489'},
        {'fromNode': '5341.0', 'toNode': '5465.0', 'Resistance': 0.00307, 'Reactance': 0.02252, 'Conductance': 0, 'Susceptance': 0.01276, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1490'},
        {'fromNode': '5372.0', 'toNode': '5344.0', 'Resistance': 0.00186, 'Reactance': 0.00821, 'Conductance': 0, 'Susceptance': 0.00382, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1491'},
        {'fromNode': '5344.0', 'toNode': '5482.0', 'Resistance': 0.00385, 'Reactance': 0.01695, 'Conductance': 0, 'Susceptance': 0.00789, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1492'},
        {'fromNode': '5375.0', 'toNode': '5346.0', 'Resistance': 0.00103, 'Reactance': 0.00605, 'Conductance': 0, 'Susceptance': 0.00302, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1493'},
        {'fromNode': '5346.0', 'toNode': '5441.0', 'Resistance': 0.00168, 'Reactance': 0.01277, 'Conductance': 0, 'Susceptance': 0.0067, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1494'},
        {'fromNode': '5346.0', 'toNode': '5441.0', 'Resistance': 0.00168, 'Reactance': 0.01277, 'Conductance': 0, 'Susceptance': 0.0067, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1495'},
        {'fromNode': '5443.0', 'toNode': '5346.0', 'Resistance': 0.0007, 'Reactance': 0.00512, 'Conductance': 0, 'Susceptance': 0.0029, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1496'},
        {'fromNode': '5443.0', 'toNode': '5346.0', 'Resistance': 0.0007, 'Reactance': 0.00512, 'Conductance': 0, 'Susceptance': 0.0029, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1497'},
        {'fromNode': '5348.0', 'toNode': '5355.0', 'Resistance': 0.0081, 'Reactance': 0.0357, 'Conductance': 0, 'Susceptance': 0.01662, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1498'},
        {'fromNode': '5349.0', 'toNode': '5462.0', 'Resistance': 0.00235, 'Reactance': 0.01326, 'Conductance': 0, 'Susceptance': 0.00713, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1499'},
        {'fromNode': '5351.0', 'toNode': '5350.0', 'Resistance': 8e-05, 'Reactance': 0.00906, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1258.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1500'},
        {'fromNode': '5351.0', 'toNode': '5350.0', 'Resistance': 8e-05, 'Reactance': 0.00906, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1258.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1501'},
        {'fromNode': '5350.0', 'toNode': '5448.0', 'Resistance': 0.00061, 'Reactance': 0.00734, 'Conductance': 0, 'Susceptance': 0.48762, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1502'},
        {'fromNode': '5350.0', 'toNode': '5464.0', 'Resistance': 0.00053, 'Reactance': 0.00645, 'Conductance': 0, 'Susceptance': 0.4286, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1503'},
        {'fromNode': '5351.0', 'toNode': '5431.0', 'Resistance': 0.00463, 'Reactance': 0.02714, 'Conductance': 0, 'Susceptance': 0.01355, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1504'},
        {'fromNode': '5376.0', 'toNode': '5353.0', 'Resistance': 0.001, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 350.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1505'},
        {'fromNode': '5376.0', 'toNode': '5353.0', 'Resistance': 0.001, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 350.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1506'},
        {'fromNode': '5353.0', 'toNode': '5468.0', 'Resistance': 0.00184, 'Reactance': 0.01402, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1507'},
        {'fromNode': '5355.0', 'toNode': '5445.0', 'Resistance': 0.00332, 'Reactance': 0.02526, 'Conductance': 0, 'Susceptance': 0.01326, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1508'},
        {'fromNode': '5355.0', 'toNode': '5445.0', 'Resistance': 0.00332, 'Reactance': 0.02526, 'Conductance': 0, 'Susceptance': 0.01326, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1509'},
        {'fromNode': '5386.0', 'toNode': '5356.0', 'Resistance': 0.001, 'Reactance': 0.00567, 'Conductance': 0, 'Susceptance': 0.00305, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1510'},
        {'fromNode': '5356.0', 'toNode': '5431.0', 'Resistance': 0.00571, 'Reactance': 0.0323, 'Conductance': 0, 'Susceptance': 0.01737, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1511'},
        {'fromNode': '5357.0', 'toNode': '5386.0', 'Resistance': 0.00324, 'Reactance': 0.02375, 'Conductance': 0, 'Susceptance': 0.01346, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1512'},
        {'fromNode': '5359.0', 'toNode': '5358.0', 'Resistance': 0.00026, 'Reactance': 0.01828, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1057.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1513'},
        {'fromNode': '5360.0', 'toNode': '5358.0', 'Resistance': 0.00015, 'Reactance': 0.0112, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1333.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1514'},
        {'fromNode': '8155.0', 'toNode': '5358.0', 'Resistance': 0.00037, 'Reactance': 0.0075, 'Conductance': 0, 'Susceptance': 0.52342, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1515'},
        {'fromNode': '5362.0', 'toNode': '5361.0', 'Resistance': 0.00018, 'Reactance': 0.01755, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 646.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1516'},
        {'fromNode': '5363.0', 'toNode': '5361.0', 'Resistance': 0.00057, 'Reactance': 0.03295, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 282.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1517'},
        {'fromNode': '5364.0', 'toNode': '5361.0', 'Resistance': 0.00014, 'Reactance': 0.00948, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 798.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1518'},
        {'fromNode': '5365.0', 'toNode': '5361.0', 'Resistance': 0.00026, 'Reactance': 0.01973, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 787.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1519'},
        {'fromNode': '5366.0', 'toNode': '5361.0', 'Resistance': 0.00038, 'Reactance': 0.02244, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 795.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1520'},
        {'fromNode': '5367.0', 'toNode': '5361.0', 'Resistance': 0.00031, 'Reactance': 0.02178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 809.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1521'},
        {'fromNode': '5368.0', 'toNode': '5361.0', 'Resistance': 0.00022, 'Reactance': 0.01449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 798.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1522'},
        {'fromNode': '5369.0', 'toNode': '5361.0', 'Resistance': 0.00041, 'Reactance': 0.02362, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 796.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1523'},
        {'fromNode': '5373.0', 'toNode': '5433.0', 'Resistance': 0.0024, 'Reactance': 0.01407, 'Conductance': 0, 'Susceptance': 0.00702, 'MVA_rating': 263.55, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1524'},
        {'fromNode': '5385.0', 'toNode': '5374.0', 'Resistance': 0.00128, 'Reactance': 0.00939, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1525'},
        {'fromNode': '5385.0', 'toNode': '5374.0', 'Resistance': 0.00128, 'Reactance': 0.00939, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1526'},
        {'fromNode': '5385.0', 'toNode': '5374.0', 'Resistance': 0.00128, 'Reactance': 0.00939, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1527'},
        {'fromNode': '5377.0', 'toNode': '5477.0', 'Resistance': 0.00328, 'Reactance': 0.01925, 'Conductance': 0, 'Susceptance': 0.00961, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1528'},
        {'fromNode': '5470.0', 'toNode': '5378.0', 'Resistance': 0.00267, 'Reactance': 0.01179, 'Conductance': 0, 'Susceptance': 0.00549, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1529'},
        {'fromNode': '5379.0', 'toNode': '5442.0', 'Resistance': 0.00383, 'Reactance': 0.02248, 'Conductance': 0, 'Susceptance': 0.01122, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1530'},
        {'fromNode': '5381.0', 'toNode': '5380.0', 'Resistance': 0.00015, 'Reactance': 0.00982, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1057.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1531'},
        {'fromNode': '5382.0', 'toNode': '5380.0', 'Resistance': 0.00021, 'Reactance': 0.01436, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 786.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1532'},
        {'fromNode': '5383.0', 'toNode': '5380.0', 'Resistance': 0.00023, 'Reactance': 0.01636, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 787.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1533'},
        {'fromNode': '5380.0', 'toNode': '5384.0', 'Resistance': 0.00026, 'Reactance': 0.0059, 'Conductance': 0, 'Susceptance': 0.51266, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1534'},
        {'fromNode': '5381.0', 'toNode': '5439.0', 'Resistance': 0.00275, 'Reactance': 0.02093, 'Conductance': 0, 'Susceptance': 0.01099, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1535'},
        {'fromNode': '5381.0', 'toNode': '5439.0', 'Resistance': 0.00275, 'Reactance': 0.02093, 'Conductance': 0, 'Susceptance': 0.01099, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1536'},
        {'fromNode': '5385.0', 'toNode': '5384.0', 'Resistance': 4e-05, 'Reactance': 0.00492, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2019.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1537'},
        {'fromNode': '5385.0', 'toNode': '5384.0', 'Resistance': 4e-05, 'Reactance': 0.00492, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2019.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1538'},
        {'fromNode': '5387.0', 'toNode': '5429.0', 'Resistance': 0.00362, 'Reactance': 0.01597, 'Conductance': 0, 'Susceptance': 0.00743, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1539'},
        {'fromNode': '5469.0', 'toNode': '5387.0', 'Resistance': 0.00095, 'Reactance': 0.00722, 'Conductance': 0, 'Susceptance': 0.00379, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1540'},
        {'fromNode': '5389.0', 'toNode': '5388.0', 'Resistance': 0.00011, 'Reactance': 0.00666, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1064.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1541'},
        {'fromNode': '5388.0', 'toNode': '8067.0', 'Resistance': 0.00062, 'Reactance': 0.01267, 'Conductance': 0, 'Susceptance': 0.8835, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1542'},
        {'fromNode': '5389.0', 'toNode': '5393.0', 'Resistance': 0.00709, 'Reactance': 0.04153, 'Conductance': 0, 'Susceptance': 0.02074, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1543'},
        {'fromNode': '5391.0', 'toNode': '5390.0', 'Resistance': 0.0041, 'Reactance': 0.10984, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 65.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1544'},
        {'fromNode': '5390.0', 'toNode': '5458.0', 'Resistance': 0.00669, 'Reactance': 0.02948, 'Conductance': 0, 'Susceptance': 0.01372, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1545'},
        {'fromNode': '5392.0', 'toNode': '5391.0', 'Resistance': 0.01427, 'Reactance': 0.30897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 50.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1546'},
        {'fromNode': '5459.0', 'toNode': '5391.0', 'Resistance': 0.00912, 'Reactance': 0.05206, 'Conductance': 0, 'Susceptance': 0.00782, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1547'},
        {'fromNode': '5395.0', 'toNode': '5394.0', 'Resistance': 9e-05, 'Reactance': 0.00527, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1163.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1548'},
        {'fromNode': '5399.0', 'toNode': '5394.0', 'Resistance': 9e-05, 'Reactance': 0.01069, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 645.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1549'},
        {'fromNode': '5396.0', 'toNode': '5395.0', 'Resistance': 0.00057, 'Reactance': 0.04373, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1550'},
        {'fromNode': '5397.0', 'toNode': '5396.0', 'Resistance': 0.00233, 'Reactance': 0.0652, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 168.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1551'},
        {'fromNode': '5398.0', 'toNode': '5397.0', 'Resistance': 0.01999, 'Reactance': 0.78238, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 21.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1552'},
        {'fromNode': '5402.0', 'toNode': '5401.0', 'Resistance': 0.00019, 'Reactance': 0.01369, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1174.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1553'},
        {'fromNode': '5403.0', 'toNode': '5401.0', 'Resistance': 0.00028, 'Reactance': 0.02956, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 426.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1554'},
        {'fromNode': '5404.0', 'toNode': '5401.0', 'Resistance': 0.00027, 'Reactance': 0.01999, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 435.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1555'},
        {'fromNode': '5405.0', 'toNode': '5401.0', 'Resistance': 0.0003, 'Reactance': 0.02018, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 440.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1556'},
        {'fromNode': '5406.0', 'toNode': '5401.0', 'Resistance': 0.00024, 'Reactance': 0.02866, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 433.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1557'},
        {'fromNode': '5407.0', 'toNode': '5470.0', 'Resistance': 0.01779, 'Reactance': 0.07838, 'Conductance': 0, 'Susceptance': 0.03648, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1558'},
        {'fromNode': '5408.0', 'toNode': '5421.0', 'Resistance': 0.00543, 'Reactance': 0.02392, 'Conductance': 0, 'Susceptance': 0.01113, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1559'},
        {'fromNode': '5409.0', 'toNode': '5447.0', 'Resistance': 0.00189, 'Reactance': 0.01068, 'Conductance': 0, 'Susceptance': 0.00574, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1560'},
        {'fromNode': '5411.0', 'toNode': '5410.0', 'Resistance': 0.00015, 'Reactance': 0.01564, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1041.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1561'},
        {'fromNode': '5410.0', 'toNode': '5451.0', 'Resistance': 0.0004, 'Reactance': 0.0049, 'Conductance': 0, 'Susceptance': 0.32555, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1562'},
        {'fromNode': '5411.0', 'toNode': '5477.0', 'Resistance': 0.00331, 'Reactance': 0.01942, 'Conductance': 0, 'Susceptance': 0.0097, 'MVA_rating': 251.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1563'},
        {'fromNode': '6316.0', 'toNode': '5411.0', 'Resistance': 0.00833, 'Reactance': 0.03543, 'Conductance': 0, 'Susceptance': 0.01772, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1564'},
        {'fromNode': '5414.0', 'toNode': '5413.0', 'Resistance': 0.00122, 'Reactance': 0.05952, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 315.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1565'},
        {'fromNode': '5415.0', 'toNode': '5413.0', 'Resistance': 0.00043, 'Reactance': 0.03186, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 561.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1566'},
        {'fromNode': '5416.0', 'toNode': '5413.0', 'Resistance': 0.00034, 'Reactance': 0.03222, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 559.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1567'},
        {'fromNode': '5417.0', 'toNode': '5413.0', 'Resistance': 0.00024, 'Reactance': 0.0284, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 574.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1568'},
        {'fromNode': '5418.0', 'toNode': '5413.0', 'Resistance': 0.00058, 'Reactance': 0.02748, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 551.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1569'},
        {'fromNode': '5419.0', 'toNode': '5413.0', 'Resistance': 0.00029, 'Reactance': 0.01611, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 561.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1570'},
        {'fromNode': '5414.0', 'toNode': '5471.0', 'Resistance': 0.01049, 'Reactance': 0.04462, 'Conductance': 0, 'Susceptance': 0.02231, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1571'},
        {'fromNode': '5423.0', 'toNode': '5422.0', 'Resistance': 0.00234, 'Reactance': 0.09836, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 94.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1572'},
        {'fromNode': '5424.0', 'toNode': '5422.0', 'Resistance': 0.00826, 'Reactance': 0.16541, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 77.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1573'},
        {'fromNode': '5432.0', 'toNode': '5430.0', 'Resistance': 0.00343, 'Reactance': 0.02608, 'Conductance': 0, 'Susceptance': 0.01369, 'MVA_rating': 310.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1574'},
        {'fromNode': '5435.0', 'toNode': '5471.0', 'Resistance': 0.01193, 'Reactance': 0.05076, 'Conductance': 0, 'Susceptance': 0.02538, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1575'},
        {'fromNode': '5443.0', 'toNode': '5469.0', 'Resistance': 0.00205, 'Reactance': 0.00902, 'Conductance': 0, 'Susceptance': 0.0042, 'MVA_rating': 380.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1576'},
        {'fromNode': '5445.0', 'toNode': '5444.0', 'Resistance': 0.0002, 'Reactance': 0.01571, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1266.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1577'},
        {'fromNode': '5470.0', 'toNode': '5446.0', 'Resistance': 0.00445, 'Reactance': 0.01961, 'Conductance': 0, 'Susceptance': 0.00913, 'MVA_rating': 187.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1578'},
        {'fromNode': '5449.0', 'toNode': '5448.0', 'Resistance': 7e-05, 'Reactance': 0.00805, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1267.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1579'},
        {'fromNode': '5448.0', 'toNode': '5479.0', 'Resistance': 0.00039, 'Reactance': 0.00471, 'Conductance': 0, 'Susceptance': 0.31303, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1580'},
        {'fromNode': '5452.0', 'toNode': '5451.0', 'Resistance': 0.00023, 'Reactance': 0.01073, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 960.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1581'},
        {'fromNode': '5459.0', 'toNode': '5458.0', 'Resistance': 0.00338, 'Reactance': 0.11671, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 80.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1582'},
        {'fromNode': '5460.0', 'toNode': '5459.0', 'Resistance': 0.01905, 'Reactance': 0.34714, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 35.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1583'},
        {'fromNode': '6140.0', 'toNode': '5459.0', 'Resistance': 0.01042, 'Reactance': 0.04644, 'Conductance': 0, 'Susceptance': 0.00681, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1584'},
        {'fromNode': '5459.0', 'toNode': '6325.0', 'Resistance': 0.01669, 'Reactance': 0.07435, 'Conductance': 0, 'Susceptance': 0.0109, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1585'},
        {'fromNode': '5463.0', 'toNode': '5478.0', 'Resistance': 0.00106, 'Reactance': 0.00601, 'Conductance': 0, 'Susceptance': 0.00323, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1586'},
        {'fromNode': '5465.0', 'toNode': '5464.0', 'Resistance': 8e-05, 'Reactance': 0.00976, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1284.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1587'},
        {'fromNode': '5467.0', 'toNode': '5482.0', 'Resistance': 0.00166, 'Reactance': 0.00971, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 409.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '1588'},
        {'fromNode': '5480.0', 'toNode': '5479.0', 'Resistance': 0.00031, 'Reactance': 0.01505, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 961.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1589'},
        {'fromNode': '6001.0', 'toNode': '6281.0', 'Resistance': 0.00258, 'Reactance': 0.01473, 'Conductance': 0, 'Susceptance': 0.00221, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1590'},
        {'fromNode': '6001.0', 'toNode': '6283.0', 'Resistance': 0.01462, 'Reactance': 0.03576, 'Conductance': 0, 'Susceptance': 0.00468, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1591'},
        {'fromNode': '6001.0', 'toNode': '6312.0', 'Resistance': 0.00777, 'Reactance': 0.03462, 'Conductance': 0, 'Susceptance': 0.00508, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1592'},
        {'fromNode': '6002.0', 'toNode': '6060.0', 'Resistance': 0.00991, 'Reactance': 0.05949, 'Conductance': 0, 'Susceptance': 0.00807, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1593'},
        {'fromNode': '6002.0', 'toNode': '6291.0', 'Resistance': 0.00199, 'Reactance': 0.01285, 'Conductance': 0, 'Susceptance': 0.00245, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1594'},
        {'fromNode': '6002.0', 'toNode': '6291.0', 'Resistance': 0.00199, 'Reactance': 0.01285, 'Conductance': 0, 'Susceptance': 0.00245, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1595'},
        {'fromNode': '6002.0', 'toNode': '6332.0', 'Resistance': 0.00746, 'Reactance': 0.03326, 'Conductance': 0, 'Susceptance': 0.00488, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1596'},
        {'fromNode': '6004.0', 'toNode': '6003.0', 'Resistance': 0.00026, 'Reactance': 0.01763, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1117.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1597'},
        {'fromNode': '6006.0', 'toNode': '6003.0', 'Resistance': 0.00024, 'Reactance': 0.01241, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 788.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1598'},
        {'fromNode': '6007.0', 'toNode': '6003.0', 'Resistance': 0.00012, 'Reactance': 0.00801, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1069.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1599'},
        {'fromNode': '6008.0', 'toNode': '6003.0', 'Resistance': 0.00019, 'Reactance': 0.01445, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 754.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1600'},
        {'fromNode': '6009.0', 'toNode': '6003.0', 'Resistance': 0.00033, 'Reactance': 0.02418, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 758.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1601'},
        {'fromNode': '6010.0', 'toNode': '6003.0', 'Resistance': 0.00034, 'Reactance': 0.01683, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 758.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1602'},
        {'fromNode': '6003.0', 'toNode': '6107.0', 'Resistance': 0.00065, 'Reactance': 0.00791, 'Conductance': 0, 'Susceptance': 0.5258, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1603'},
        {'fromNode': '6292.0', 'toNode': '6003.0', 'Resistance': 0.0002, 'Reactance': 0.00409, 'Conductance': 0, 'Susceptance': 0.28512, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1604'},
        {'fromNode': '6005.0', 'toNode': '6004.0', 'Resistance': 0.00134, 'Reactance': 0.05853, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1605'},
        {'fromNode': '6005.0', 'toNode': '6004.0', 'Resistance': 0.00134, 'Reactance': 0.05853, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1606'},
        {'fromNode': '6102.0', 'toNode': '6004.0', 'Resistance': 0.00197, 'Reactance': 0.0205, 'Conductance': 0, 'Susceptance': 0.06986, 'MVA_rating': 1400.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1607'},
        {'fromNode': '6012.0', 'toNode': '6005.0', 'Resistance': 0.00833, 'Reactance': 0.0569, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1608'},
        {'fromNode': '6012.0', 'toNode': '6005.0', 'Resistance': 0.00833, 'Reactance': 0.0569, 'Conductance': 0, 'Susceptance': 0.00968, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1609'},
        {'fromNode': '6005.0', 'toNode': '6117.0', 'Resistance': 0.00772, 'Reactance': 0.04984, 'Conductance': 0, 'Susceptance': 0.00951, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1610'},
        {'fromNode': '6005.0', 'toNode': '6117.0', 'Resistance': 0.00772, 'Reactance': 0.04984, 'Conductance': 0, 'Susceptance': 0.00951, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1611'},
        {'fromNode': '6011.0', 'toNode': '6068.0', 'Resistance': 0.00433, 'Reactance': 0.03013, 'Conductance': 0, 'Susceptance': 0.00493, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1612'},
        {'fromNode': '6163.0', 'toNode': '6011.0', 'Resistance': 0.00329, 'Reactance': 0.02128, 'Conductance': 0, 'Susceptance': 0.00406, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1613'},
        {'fromNode': '6027.0', 'toNode': '6012.0', 'Resistance': 0.00646, 'Reactance': 0.04503, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1614'},
        {'fromNode': '6027.0', 'toNode': '6012.0', 'Resistance': 0.00646, 'Reactance': 0.04503, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1615'},
        {'fromNode': '6026.0', 'toNode': '6013.0', 'Resistance': 0.00518, 'Reactance': 0.03346, 'Conductance': 0, 'Susceptance': 0.00638, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1616'},
        {'fromNode': '6013.0', 'toNode': '6128.0', 'Resistance': 0.0055, 'Reactance': 0.033, 'Conductance': 0, 'Susceptance': 0.00448, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1617'},
        {'fromNode': '6013.0', 'toNode': '6168.0', 'Resistance': 0.00398, 'Reactance': 0.0174, 'Conductance': 0, 'Susceptance': 0.00265, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1618'},
        {'fromNode': '6013.0', 'toNode': '6225.0', 'Resistance': 0.0082, 'Reactance': 0.04681, 'Conductance': 0, 'Susceptance': 0.00703, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1619'},
        {'fromNode': '6014.0', 'toNode': '6096.0', 'Resistance': 0.02367, 'Reactance': 0.05785, 'Conductance': 0, 'Susceptance': 0.00758, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1620'},
        {'fromNode': '6133.0', 'toNode': '6014.0', 'Resistance': 0.01059, 'Reactance': 0.04636, 'Conductance': 0, 'Susceptance': 0.00706, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1621'},
        {'fromNode': '6014.0', 'toNode': '6139.0', 'Resistance': 0.00975, 'Reactance': 0.05851, 'Conductance': 0, 'Susceptance': 0.00794, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1622'},
        {'fromNode': '6015.0', 'toNode': '6112.0', 'Resistance': 0.0199, 'Reactance': 0.04863, 'Conductance': 0, 'Susceptance': 0.00637, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1623'},
        {'fromNode': '6236.0', 'toNode': '6015.0', 'Resistance': 0.00985, 'Reactance': 0.04086, 'Conductance': 0, 'Susceptance': 0.00693, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1624'},
        {'fromNode': '6016.0', 'toNode': '6168.0', 'Resistance': 0.00378, 'Reactance': 0.0263, 'Conductance': 0, 'Susceptance': 0.0043, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1625'},
        {'fromNode': '6016.0', 'toNode': '6168.0', 'Resistance': 0.00378, 'Reactance': 0.0263, 'Conductance': 0, 'Susceptance': 0.0043, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1626'},
        {'fromNode': '6199.0', 'toNode': '6016.0', 'Resistance': 0.00208, 'Reactance': 0.01421, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1627'},
        {'fromNode': '6199.0', 'toNode': '6016.0', 'Resistance': 0.00208, 'Reactance': 0.01421, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1628'},
        {'fromNode': '6199.0', 'toNode': '6016.0', 'Resistance': 0.00208, 'Reactance': 0.01421, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1629'},
        {'fromNode': '6016.0', 'toNode': '6356.0', 'Resistance': 0.00315, 'Reactance': 0.01306, 'Conductance': 0, 'Susceptance': 0.00222, 'MVA_rating': 400.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1630'},
        {'fromNode': '6068.0', 'toNode': '6017.0', 'Resistance': 0.00427, 'Reactance': 0.02759, 'Conductance': 0, 'Susceptance': 0.00526, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1631'},
        {'fromNode': '6068.0', 'toNode': '6017.0', 'Resistance': 0.00427, 'Reactance': 0.02759, 'Conductance': 0, 'Susceptance': 0.00526, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1632'},
        {'fromNode': '6068.0', 'toNode': '6017.0', 'Resistance': 0.00427, 'Reactance': 0.02759, 'Conductance': 0, 'Susceptance': 0.00526, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1633'},
        {'fromNode': '6017.0', 'toNode': '6159.0', 'Resistance': 0.00233, 'Reactance': 0.01593, 'Conductance': 0, 'Susceptance': 0.00271, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1634'},
        {'fromNode': '6017.0', 'toNode': '6159.0', 'Resistance': 0.00233, 'Reactance': 0.01593, 'Conductance': 0, 'Susceptance': 0.00271, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1635'},
        {'fromNode': '6017.0', 'toNode': '6159.0', 'Resistance': 0.00233, 'Reactance': 0.01593, 'Conductance': 0, 'Susceptance': 0.00271, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1636'},
        {'fromNode': '6160.0', 'toNode': '6018.0', 'Resistance': 0.00176, 'Reactance': 0.01201, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1637'},
        {'fromNode': '6160.0', 'toNode': '6018.0', 'Resistance': 0.00176, 'Reactance': 0.01201, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1638'},
        {'fromNode': '6160.0', 'toNode': '6018.0', 'Resistance': 0.00176, 'Reactance': 0.01201, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1639'},
        {'fromNode': '6018.0', 'toNode': '6256.0', 'Resistance': 0.00261, 'Reactance': 0.01687, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1640'},
        {'fromNode': '6018.0', 'toNode': '6256.0', 'Resistance': 0.00261, 'Reactance': 0.01687, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1641'},
        {'fromNode': '6018.0', 'toNode': '6256.0', 'Resistance': 0.00261, 'Reactance': 0.01687, 'Conductance': 0, 'Susceptance': 0.00322, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1642'},
        {'fromNode': '6040.0', 'toNode': '6019.0', 'Resistance': 0.0008, 'Reactance': 0.00518, 'Conductance': 0, 'Susceptance': 0.00099, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1643'},
        {'fromNode': '6040.0', 'toNode': '6019.0', 'Resistance': 0.0008, 'Reactance': 0.00518, 'Conductance': 0, 'Susceptance': 0.00099, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1644'},
        {'fromNode': '6040.0', 'toNode': '6019.0', 'Resistance': 0.0008, 'Reactance': 0.00518, 'Conductance': 0, 'Susceptance': 0.00099, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1645'},
        {'fromNode': '6019.0', 'toNode': '6113.0', 'Resistance': 0.00154, 'Reactance': 0.00995, 'Conductance': 0, 'Susceptance': 0.0019, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1646'},
        {'fromNode': '6019.0', 'toNode': '6113.0', 'Resistance': 0.00154, 'Reactance': 0.00995, 'Conductance': 0, 'Susceptance': 0.0019, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1647'},
        {'fromNode': '6019.0', 'toNode': '6306.0', 'Resistance': 0.01058, 'Reactance': 0.02586, 'Conductance': 0, 'Susceptance': 0.00339, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1648'},
        {'fromNode': '6019.0', 'toNode': '6313.0', 'Resistance': 0.00538, 'Reactance': 0.03672, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1649'},
        {'fromNode': '6019.0', 'toNode': '6313.0', 'Resistance': 0.00538, 'Reactance': 0.03672, 'Conductance': 0, 'Susceptance': 0.00625, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1650'},
        {'fromNode': '6020.0', 'toNode': '6212.0', 'Resistance': 0.00352, 'Reactance': 0.0146, 'Conductance': 0, 'Susceptance': 0.00248, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1651'},
        {'fromNode': '6020.0', 'toNode': '6252.0', 'Resistance': 0.01082, 'Reactance': 0.04487, 'Conductance': 0, 'Susceptance': 0.00761, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1652'},
        {'fromNode': '6022.0', 'toNode': '6021.0', 'Resistance': 0.00033, 'Reactance': 0.01733, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 948.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1653'},
        {'fromNode': '6118.0', 'toNode': '6021.0', 'Resistance': 0.00013, 'Reactance': 0.00161, 'Conductance': 0, 'Susceptance': 0.10699, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1654'},
        {'fromNode': '6021.0', 'toNode': '6210.0', 'Resistance': 0.00053, 'Reactance': 0.00643, 'Conductance': 0, 'Susceptance': 0.42698, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1655'},
        {'fromNode': '6023.0', 'toNode': '6022.0', 'Resistance': 0.00043, 'Reactance': 0.02783, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1656'},
        {'fromNode': '6315.0', 'toNode': '6022.0', 'Resistance': 0.0043, 'Reactance': 0.03071, 'Conductance': 0, 'Susceptance': 0.05598, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1657'},
        {'fromNode': '6022.0', 'toNode': '6350.0', 'Resistance': 0.00257, 'Reactance': 0.01837, 'Conductance': 0, 'Susceptance': 0.03349, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1658'},
        {'fromNode': '6120.0', 'toNode': '6023.0', 'Resistance': 0.00776, 'Reactance': 0.03392, 'Conductance': 0, 'Susceptance': 0.00517, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1659'},
        {'fromNode': '6023.0', 'toNode': '6262.0', 'Resistance': 0.00666, 'Reactance': 0.04639, 'Conductance': 0, 'Susceptance': 0.00758, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1660'},
        {'fromNode': '6023.0', 'toNode': '6262.0', 'Resistance': 0.00666, 'Reactance': 0.04639, 'Conductance': 0, 'Susceptance': 0.00758, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1661'},
        {'fromNode': '6024.0', 'toNode': '6181.0', 'Resistance': 0.01205, 'Reactance': 0.05372, 'Conductance': 0, 'Susceptance': 0.00788, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1662'},
        {'fromNode': '6024.0', 'toNode': '6219.0', 'Resistance': 0.004, 'Reactance': 0.02403, 'Conductance': 0, 'Susceptance': 0.00326, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1663'},
        {'fromNode': '6024.0', 'toNode': '6330.0', 'Resistance': 0.00775, 'Reactance': 0.04653, 'Conductance': 0, 'Susceptance': 0.00631, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1664'},
        {'fromNode': '6025.0', 'toNode': '6060.0', 'Resistance': 0.00724, 'Reactance': 0.04134, 'Conductance': 0, 'Susceptance': 0.00621, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1665'},
        {'fromNode': '6121.0', 'toNode': '6025.0', 'Resistance': 0.00399, 'Reactance': 0.02277, 'Conductance': 0, 'Susceptance': 0.00342, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1666'},
        {'fromNode': '6025.0', 'toNode': '6337.0', 'Resistance': 0.00373, 'Reactance': 0.02598, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1667'},
        {'fromNode': '6025.0', 'toNode': '6337.0', 'Resistance': 0.00373, 'Reactance': 0.02598, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1668'},
        {'fromNode': '6352.0', 'toNode': '6025.0', 'Resistance': 0.00808, 'Reactance': 0.01942, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1669'},
        {'fromNode': '6026.0', 'toNode': '6182.0', 'Resistance': 0.00598, 'Reactance': 0.04088, 'Conductance': 0, 'Susceptance': 0.00695, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1670'},
        {'fromNode': '6236.0', 'toNode': '6026.0', 'Resistance': 0.00321, 'Reactance': 0.02074, 'Conductance': 0, 'Susceptance': 0.00396, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1671'},
        {'fromNode': '6236.0', 'toNode': '6026.0', 'Resistance': 0.00321, 'Reactance': 0.02074, 'Conductance': 0, 'Susceptance': 0.00396, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1672'},
        {'fromNode': '6027.0', 'toNode': '6181.0', 'Resistance': 0.00709, 'Reactance': 0.0494, 'Conductance': 0, 'Susceptance': 0.00808, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1673'},
        {'fromNode': '6126.0', 'toNode': '6028.0', 'Resistance': 0.00239, 'Reactance': 0.01662, 'Conductance': 0, 'Susceptance': 0.00272, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1674'},
        {'fromNode': '6126.0', 'toNode': '6028.0', 'Resistance': 0.00239, 'Reactance': 0.01662, 'Conductance': 0, 'Susceptance': 0.00272, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1675'},
        {'fromNode': '6028.0', 'toNode': '6223.0', 'Resistance': 0.00261, 'Reactance': 0.01816, 'Conductance': 0, 'Susceptance': 0.00297, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1676'},
        {'fromNode': '6028.0', 'toNode': '6223.0', 'Resistance': 0.00261, 'Reactance': 0.01816, 'Conductance': 0, 'Susceptance': 0.00297, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1677'},
        {'fromNode': '6028.0', 'toNode': '6289.0', 'Resistance': 0.0098, 'Reactance': 0.05596, 'Conductance': 0, 'Susceptance': 0.00841, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1678'},
        {'fromNode': '6029.0', 'toNode': '6221.0', 'Resistance': 0.02678, 'Reactance': 0.11107, 'Conductance': 0, 'Susceptance': 0.01885, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1679'},
        {'fromNode': '6329.0', 'toNode': '6029.0', 'Resistance': 0.01886, 'Reactance': 0.07826, 'Conductance': 0, 'Susceptance': 0.01328, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1680'},
        {'fromNode': '6031.0', 'toNode': '6030.0', 'Resistance': 0.00319, 'Reactance': 0.1363, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 124.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1681'},
        {'fromNode': '6030.0', 'toNode': '6172.0', 'Resistance': 0.00307, 'Reactance': 0.02195, 'Conductance': 0, 'Susceptance': 0.04002, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1682'},
        {'fromNode': '6030.0', 'toNode': '6188.0', 'Resistance': 0.01107, 'Reactance': 0.07911, 'Conductance': 0, 'Susceptance': 0.14424, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1683'},
        {'fromNode': '6030.0', 'toNode': '7274.0', 'Resistance': 0.0079, 'Reactance': 0.05646, 'Conductance': 0, 'Susceptance': 0.10294, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1684'},
        {'fromNode': '6173.0', 'toNode': '6031.0', 'Resistance': 0.03353, 'Reactance': 0.08197, 'Conductance': 0, 'Susceptance': 0.01073, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1685'},
        {'fromNode': '6032.0', 'toNode': '6207.0', 'Resistance': 0.01092, 'Reactance': 0.04528, 'Conductance': 0, 'Susceptance': 0.00768, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1686'},
        {'fromNode': '6217.0', 'toNode': '6032.0', 'Resistance': 0.00379, 'Reactance': 0.02164, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1687'},
        {'fromNode': '6034.0', 'toNode': '6033.0', 'Resistance': 0.00014, 'Reactance': 0.00944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1058.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1688'},
        {'fromNode': '6107.0', 'toNode': '6033.0', 'Resistance': 0.00039, 'Reactance': 0.00803, 'Conductance': 0, 'Susceptance': 0.56039, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1689'},
        {'fromNode': '6033.0', 'toNode': '6342.0', 'Resistance': 0.00011, 'Reactance': 0.00135, 'Conductance': 0, 'Susceptance': 0.0896, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1690'},
        {'fromNode': '6035.0', 'toNode': '6034.0', 'Resistance': 0.0004, 'Reactance': 0.01618, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 540.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1691'},
        {'fromNode': '6035.0', 'toNode': '6034.0', 'Resistance': 0.0004, 'Reactance': 0.01618, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 540.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1692'},
        {'fromNode': '6108.0', 'toNode': '6034.0', 'Resistance': 0.00498, 'Reactance': 0.04865, 'Conductance': 0, 'Susceptance': 0.0936, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1693'},
        {'fromNode': '6034.0', 'toNode': '6119.0', 'Resistance': 0.00307, 'Reactance': 0.02194, 'Conductance': 0, 'Susceptance': 0.04, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1694'},
        {'fromNode': '6034.0', 'toNode': '6255.0', 'Resistance': 0.00338, 'Reactance': 0.02413, 'Conductance': 0, 'Susceptance': 0.044, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1695'},
        {'fromNode': '6034.0', 'toNode': '6343.0', 'Resistance': 0.00082, 'Reactance': 0.00798, 'Conductance': 0, 'Susceptance': 0.01535, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1696'},
        {'fromNode': '6061.0', 'toNode': '6035.0', 'Resistance': 0.00446, 'Reactance': 0.03049, 'Conductance': 0, 'Susceptance': 0.00519, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1697'},
        {'fromNode': '6061.0', 'toNode': '6035.0', 'Resistance': 0.00446, 'Reactance': 0.03049, 'Conductance': 0, 'Susceptance': 0.00519, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1698'},
        {'fromNode': '6123.0', 'toNode': '6035.0', 'Resistance': 0.00473, 'Reactance': 0.03056, 'Conductance': 0, 'Susceptance': 0.00583, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1699'},
        {'fromNode': '6123.0', 'toNode': '6035.0', 'Resistance': 0.00473, 'Reactance': 0.03056, 'Conductance': 0, 'Susceptance': 0.00583, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1700'},
        {'fromNode': '6035.0', 'toNode': '6306.0', 'Resistance': 0.00918, 'Reactance': 0.05241, 'Conductance': 0, 'Susceptance': 0.00787, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1701'},
        {'fromNode': '6037.0', 'toNode': '6036.0', 'Resistance': 0.00033, 'Reactance': 0.0183, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1702'},
        {'fromNode': '6037.0', 'toNode': '6036.0', 'Resistance': 0.00033, 'Reactance': 0.0183, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1703'},
        {'fromNode': '6037.0', 'toNode': '6036.0', 'Resistance': 0.00033, 'Reactance': 0.0183, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1704'},
        {'fromNode': '6036.0', 'toNode': '6067.0', 'Resistance': 0.00172, 'Reactance': 0.01789, 'Conductance': 0, 'Susceptance': 0.06095, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1705'},
        {'fromNode': '6036.0', 'toNode': '6142.0', 'Resistance': 0.00123, 'Reactance': 0.01275, 'Conductance': 0, 'Susceptance': 0.04344, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1706'},
        {'fromNode': '6036.0', 'toNode': '6142.0', 'Resistance': 0.00123, 'Reactance': 0.01275, 'Conductance': 0, 'Susceptance': 0.04344, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1707'},
        {'fromNode': '6287.0', 'toNode': '6036.0', 'Resistance': 0.00203, 'Reactance': 0.01984, 'Conductance': 0, 'Susceptance': 0.03818, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1708'},
        {'fromNode': '6036.0', 'toNode': '6334.0', 'Resistance': 0.00352, 'Reactance': 0.02515, 'Conductance': 0, 'Susceptance': 0.04586, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1709'},
        {'fromNode': '6094.0', 'toNode': '6037.0', 'Resistance': 0.00228, 'Reactance': 0.01555, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1710'},
        {'fromNode': '6094.0', 'toNode': '6037.0', 'Resistance': 0.00228, 'Reactance': 0.01555, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1711'},
        {'fromNode': '6220.0', 'toNode': '6037.0', 'Resistance': 0.00245, 'Reactance': 0.01672, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1712'},
        {'fromNode': '6220.0', 'toNode': '6037.0', 'Resistance': 0.00245, 'Reactance': 0.01672, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1713'},
        {'fromNode': '6220.0', 'toNode': '6037.0', 'Resistance': 0.00245, 'Reactance': 0.01672, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1714'},
        {'fromNode': '6039.0', 'toNode': '6038.0', 'Resistance': 0.00027, 'Reactance': 0.0133, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1269.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1715'},
        {'fromNode': '6041.0', 'toNode': '6038.0', 'Resistance': 0.0004, 'Reactance': 0.02009, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 296.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1716'},
        {'fromNode': '6042.0', 'toNode': '6038.0', 'Resistance': 0.00042, 'Reactance': 0.03102, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 479.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1717'},
        {'fromNode': '6038.0', 'toNode': '6066.0', 'Resistance': 0.00105, 'Reactance': 0.01278, 'Conductance': 0, 'Susceptance': 0.84902, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1718'},
        {'fromNode': '6254.0', 'toNode': '6038.0', 'Resistance': 0.00024, 'Reactance': 0.00295, 'Conductance': 0, 'Susceptance': 0.19592, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1719'},
        {'fromNode': '6349.0', 'toNode': '6038.0', 'Resistance': 0.00017, 'Reactance': 0.002, 'Conductance': 0, 'Susceptance': 0.13292, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1720'},
        {'fromNode': '6040.0', 'toNode': '6039.0', 'Resistance': 0.00064, 'Reactance': 0.0344, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 285.39, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1721'},
        {'fromNode': '6040.0', 'toNode': '6039.0', 'Resistance': 0.00064, 'Reactance': 0.0344, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 285.39, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1722'},
        {'fromNode': '6222.0', 'toNode': '6039.0', 'Resistance': 0.00129, 'Reactance': 0.00919, 'Conductance': 0, 'Susceptance': 0.01675, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1723'},
        {'fromNode': '6040.0', 'toNode': '6224.0', 'Resistance': 0.00927, 'Reactance': 0.03846, 'Conductance': 0, 'Susceptance': 0.00653, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1724'},
        {'fromNode': '6043.0', 'toNode': '6236.0', 'Resistance': 0.0039, 'Reactance': 0.02665, 'Conductance': 0, 'Susceptance': 0.00453, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1725'},
        {'fromNode': '6043.0', 'toNode': '6236.0', 'Resistance': 0.0039, 'Reactance': 0.02665, 'Conductance': 0, 'Susceptance': 0.00453, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1726'},
        {'fromNode': '6335.0', 'toNode': '6043.0', 'Resistance': 0.00217, 'Reactance': 0.01509, 'Conductance': 0, 'Susceptance': 0.00247, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1727'},
        {'fromNode': '6335.0', 'toNode': '6043.0', 'Resistance': 0.00217, 'Reactance': 0.01509, 'Conductance': 0, 'Susceptance': 0.00247, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1728'},
        {'fromNode': '6044.0', 'toNode': '6203.0', 'Resistance': 0.00617, 'Reactance': 0.02696, 'Conductance': 0, 'Susceptance': 0.00411, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1729'},
        {'fromNode': '6204.0', 'toNode': '6044.0', 'Resistance': 0.00268, 'Reactance': 0.01111, 'Conductance': 0, 'Susceptance': 0.00189, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1730'},
        {'fromNode': '6044.0', 'toNode': '6284.0', 'Resistance': 0.00593, 'Reactance': 0.02596, 'Conductance': 0, 'Susceptance': 0.00395, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1731'},
        {'fromNode': '6044.0', 'toNode': '6323.0', 'Resistance': 0.0021, 'Reactance': 0.01432, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1732'},
        {'fromNode': '6044.0', 'toNode': '6323.0', 'Resistance': 0.0021, 'Reactance': 0.01432, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1733'},
        {'fromNode': '6046.0', 'toNode': '6045.0', 'Resistance': 8e-05, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1198.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1734'},
        {'fromNode': '6048.0', 'toNode': '6045.0', 'Resistance': 0.0014, 'Reactance': 0.07775, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 233.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1735'},
        {'fromNode': '6049.0', 'toNode': '6045.0', 'Resistance': 0.00046, 'Reactance': 0.04414, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 283.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1736'},
        {'fromNode': '6050.0', 'toNode': '6045.0', 'Resistance': 0.00056, 'Reactance': 0.04367, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 256.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1737'},
        {'fromNode': '6051.0', 'toNode': '6045.0', 'Resistance': 0.00035, 'Reactance': 0.0254, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 351.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1738'},
        {'fromNode': '6052.0', 'toNode': '6045.0', 'Resistance': 0.00074, 'Reactance': 0.03404, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 342.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1739'},
        {'fromNode': '6053.0', 'toNode': '6045.0', 'Resistance': 0.00028, 'Reactance': 0.02922, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 352.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1740'},
        {'fromNode': '6054.0', 'toNode': '6045.0', 'Resistance': 0.00061, 'Reactance': 0.05567, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 338.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1741'},
        {'fromNode': '6066.0', 'toNode': '6045.0', 'Resistance': 0.00059, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.47436, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1742'},
        {'fromNode': '6210.0', 'toNode': '6045.0', 'Resistance': 0.00142, 'Reactance': 0.01727, 'Conductance': 0, 'Susceptance': 1.14708, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1743'},
        {'fromNode': '6045.0', 'toNode': '6234.0', 'Resistance': 0.00031, 'Reactance': 0.0037, 'Conductance': 0, 'Susceptance': 0.24613, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1744'},
        {'fromNode': '6047.0', 'toNode': '6046.0', 'Resistance': 0.00056, 'Reactance': 0.0418, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1745'},
        {'fromNode': '6047.0', 'toNode': '6046.0', 'Resistance': 0.00056, 'Reactance': 0.0418, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1746'},
        {'fromNode': '6198.0', 'toNode': '6046.0', 'Resistance': 0.00141, 'Reactance': 0.01004, 'Conductance': 0, 'Susceptance': 0.0183, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1747'},
        {'fromNode': '6046.0', 'toNode': '6285.0', 'Resistance': 0.00296, 'Reactance': 0.02891, 'Conductance': 0, 'Susceptance': 0.05562, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1748'},
        {'fromNode': '6047.0', 'toNode': '6150.0', 'Resistance': 0.00335, 'Reactance': 0.02337, 'Conductance': 0, 'Susceptance': 0.00382, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1749'},
        {'fromNode': '6047.0', 'toNode': '6150.0', 'Resistance': 0.00335, 'Reactance': 0.02337, 'Conductance': 0, 'Susceptance': 0.00382, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1750'},
        {'fromNode': '6291.0', 'toNode': '6047.0', 'Resistance': 0.00187, 'Reactance': 0.01206, 'Conductance': 0, 'Susceptance': 0.0023, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1751'},
        {'fromNode': '6291.0', 'toNode': '6047.0', 'Resistance': 0.00187, 'Reactance': 0.01206, 'Conductance': 0, 'Susceptance': 0.0023, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1752'},
        {'fromNode': '6226.0', 'toNode': '6055.0', 'Resistance': 0.018, 'Reactance': 0.10807, 'Conductance': 0, 'Susceptance': 0.01467, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1753'},
        {'fromNode': '6055.0', 'toNode': '6286.0', 'Resistance': 0.00882, 'Reactance': 0.0504, 'Conductance': 0, 'Susceptance': 0.00757, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1754'},
        {'fromNode': '6057.0', 'toNode': '6056.0', 'Resistance': 0.00023, 'Reactance': 0.01095, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1171.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1755'},
        {'fromNode': '6056.0', 'toNode': '6066.0', 'Resistance': 0.00014, 'Reactance': 0.00165, 'Conductance': 0, 'Susceptance': 0.10973, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1756'},
        {'fromNode': '6161.0', 'toNode': '6056.0', 'Resistance': 0.00011, 'Reactance': 0.00227, 'Conductance': 0, 'Susceptance': 0.15827, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1757'},
        {'fromNode': '6298.0', 'toNode': '6056.0', 'Resistance': 8e-05, 'Reactance': 0.001, 'Conductance': 0, 'Susceptance': 0.06674, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1758'},
        {'fromNode': '6058.0', 'toNode': '6057.0', 'Resistance': 0.00061, 'Reactance': 0.02173, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 328.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1759'},
        {'fromNode': '6058.0', 'toNode': '6057.0', 'Resistance': 0.00061, 'Reactance': 0.02173, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 328.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1760'},
        {'fromNode': '6067.0', 'toNode': '6057.0', 'Resistance': 0.001, 'Reactance': 0.00977, 'Conductance': 0, 'Susceptance': 0.01879, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1761'},
        {'fromNode': '6057.0', 'toNode': '6177.0', 'Resistance': 0.00628, 'Reactance': 0.06139, 'Conductance': 0, 'Susceptance': 0.11812, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1762'},
        {'fromNode': '6058.0', 'toNode': '6124.0', 'Resistance': 0.00613, 'Reactance': 0.04269, 'Conductance': 0, 'Susceptance': 0.00698, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1763'},
        {'fromNode': '6058.0', 'toNode': '6124.0', 'Resistance': 0.00613, 'Reactance': 0.04269, 'Conductance': 0, 'Susceptance': 0.00698, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1764'},
        {'fromNode': '6058.0', 'toNode': '6124.0', 'Resistance': 0.00613, 'Reactance': 0.04269, 'Conductance': 0, 'Susceptance': 0.00698, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1765'},
        {'fromNode': '6300.0', 'toNode': '6058.0', 'Resistance': 0.00293, 'Reactance': 0.01895, 'Conductance': 0, 'Susceptance': 0.00362, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1766'},
        {'fromNode': '6357.0', 'toNode': '6059.0', 'Resistance': 0.01709, 'Reactance': 0.07089, 'Conductance': 0, 'Susceptance': 0.01203, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1767'},
        {'fromNode': '6060.0', 'toNode': '6157.0', 'Resistance': 0.00969, 'Reactance': 0.05535, 'Conductance': 0, 'Susceptance': 0.00832, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1768'},
        {'fromNode': '6314.0', 'toNode': '6060.0', 'Resistance': 0.00648, 'Reactance': 0.02889, 'Conductance': 0, 'Susceptance': 0.00424, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1769'},
        {'fromNode': '6061.0', 'toNode': '6219.0', 'Resistance': 0.00959, 'Reactance': 0.02345, 'Conductance': 0, 'Susceptance': 0.00307, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1770'},
        {'fromNode': '6061.0', 'toNode': '6224.0', 'Resistance': 0.01257, 'Reactance': 0.03074, 'Conductance': 0, 'Susceptance': 0.00403, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1771'},
        {'fromNode': '6063.0', 'toNode': '6062.0', 'Resistance': 0.00012, 'Reactance': 0.00745, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1206.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1772'},
        {'fromNode': '6075.0', 'toNode': '6062.0', 'Resistance': 0.0006, 'Reactance': 0.00726, 'Conductance': 0, 'Susceptance': 0.48242, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1773'},
        {'fromNode': '7199.0', 'toNode': '6062.0', 'Resistance': 0.00097, 'Reactance': 0.01173, 'Conductance': 0, 'Susceptance': 0.77933, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1774'},
        {'fromNode': '6064.0', 'toNode': '6063.0', 'Resistance': 0.00104, 'Reactance': 0.04096, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1775'},
        {'fromNode': '6063.0', 'toNode': '6192.0', 'Resistance': 0.00419, 'Reactance': 0.04091, 'Conductance': 0, 'Susceptance': 0.07872, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1776'},
        {'fromNode': '8094.0', 'toNode': '6063.0', 'Resistance': 0.00384, 'Reactance': 0.03753, 'Conductance': 0, 'Susceptance': 0.07222, 'MVA_rating': 800.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1777'},
        {'fromNode': '6064.0', 'toNode': '6341.0', 'Resistance': 0.01342, 'Reactance': 0.05869, 'Conductance': 0, 'Susceptance': 0.00894, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1778'},
        {'fromNode': '6065.0', 'toNode': '6262.0', 'Resistance': 0.00781, 'Reactance': 0.04461, 'Conductance': 0, 'Susceptance': 0.0067, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1779'},
        {'fromNode': '6067.0', 'toNode': '6066.0', 'Resistance': 0.00017, 'Reactance': 0.00814, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1115.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1780'},
        {'fromNode': '6069.0', 'toNode': '6066.0', 'Resistance': 0.00023, 'Reactance': 0.01215, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 667.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1781'},
        {'fromNode': '6070.0', 'toNode': '6066.0', 'Resistance': 0.00044, 'Reactance': 0.02944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 677.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1782'},
        {'fromNode': '6071.0', 'toNode': '6066.0', 'Resistance': 0.00013, 'Reactance': 0.00964, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 670.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1783'},
        {'fromNode': '6072.0', 'toNode': '6066.0', 'Resistance': 0.00026, 'Reactance': 0.02419, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 679.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1784'},
        {'fromNode': '6073.0', 'toNode': '6066.0', 'Resistance': 0.00026, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 664.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1785'},
        {'fromNode': '6074.0', 'toNode': '6066.0', 'Resistance': 0.00036, 'Reactance': 0.01602, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 666.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1786'},
        {'fromNode': '6068.0', 'toNode': '6067.0', 'Resistance': 0.00105, 'Reactance': 0.0432, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1787'},
        {'fromNode': '6068.0', 'toNode': '6067.0', 'Resistance': 0.00105, 'Reactance': 0.0432, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1788'},
        {'fromNode': '6279.0', 'toNode': '6067.0', 'Resistance': 0.00289, 'Reactance': 0.02062, 'Conductance': 0, 'Susceptance': 0.0376, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1789'},
        {'fromNode': '6076.0', 'toNode': '6075.0', 'Resistance': 0.00022, 'Reactance': 0.01507, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1058.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1790'},
        {'fromNode': '6078.0', 'toNode': '6075.0', 'Resistance': 0.00016, 'Reactance': 0.01164, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1159.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1791'},
        {'fromNode': '6079.0', 'toNode': '6075.0', 'Resistance': 9e-05, 'Reactance': 0.00542, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1165.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1792'},
        {'fromNode': '6080.0', 'toNode': '6075.0', 'Resistance': 8e-05, 'Reactance': 0.0074, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1152.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1793'},
        {'fromNode': '6075.0', 'toNode': '6101.0', 'Resistance': 0.00034, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.27429, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1794'},
        {'fromNode': '7018.0', 'toNode': '6075.0', 'Resistance': 0.00154, 'Reactance': 0.01865, 'Conductance': 0, 'Susceptance': 1.23907, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1795'},
        {'fromNode': '6077.0', 'toNode': '6076.0', 'Resistance': 0.0007, 'Reactance': 0.0251, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 185.22, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1796'},
        {'fromNode': '6102.0', 'toNode': '6076.0', 'Resistance': 0.00351, 'Reactance': 0.02507, 'Conductance': 0, 'Susceptance': 0.04571, 'MVA_rating': 850.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1797'},
        {'fromNode': '6188.0', 'toNode': '6076.0', 'Resistance': 0.00528, 'Reactance': 0.03774, 'Conductance': 0, 'Susceptance': 0.0688, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1798'},
        {'fromNode': '6077.0', 'toNode': '6135.0', 'Resistance': 0.01114, 'Reactance': 0.04621, 'Conductance': 0, 'Susceptance': 0.00784, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1799'},
        {'fromNode': '6077.0', 'toNode': '6311.0', 'Resistance': 0.01033, 'Reactance': 0.04605, 'Conductance': 0, 'Susceptance': 0.00675, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1800'},
        {'fromNode': '6082.0', 'toNode': '6081.0', 'Resistance': 0.00602, 'Reactance': 0.1165, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 80.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1801'},
        {'fromNode': '6083.0', 'toNode': '6081.0', 'Resistance': 0.00631, 'Reactance': 0.2531, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 80.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1802'},
        {'fromNode': '6081.0', 'toNode': '6156.0', 'Resistance': 0.00412, 'Reactance': 0.01838, 'Conductance': 0, 'Susceptance': 0.0027, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1803'},
        {'fromNode': '6212.0', 'toNode': '6081.0', 'Resistance': 0.00581, 'Reactance': 0.0241, 'Conductance': 0, 'Susceptance': 0.00409, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1804'},
        {'fromNode': '6085.0', 'toNode': '6084.0', 'Resistance': 0.01088, 'Reactance': 0.45298, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 31.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1805'},
        {'fromNode': '6086.0', 'toNode': '6084.0', 'Resistance': 0.00567, 'Reactance': 0.21309, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 41.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1806'},
        {'fromNode': '6087.0', 'toNode': '6084.0', 'Resistance': 0.01529, 'Reactance': 0.2834, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 43.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1807'},
        {'fromNode': '6088.0', 'toNode': '6084.0', 'Resistance': 0.00292, 'Reactance': 0.0945, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 25.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1808'},
        {'fromNode': '6089.0', 'toNode': '6084.0', 'Resistance': 0.00558, 'Reactance': 0.2567, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 25.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1809'},
        {'fromNode': '6090.0', 'toNode': '6084.0', 'Resistance': 0.00213, 'Reactance': 0.09907, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 46.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1810'},
        {'fromNode': '6084.0', 'toNode': '6136.0', 'Resistance': 0.00536, 'Reactance': 0.03659, 'Conductance': 0, 'Susceptance': 0.00622, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1811'},
        {'fromNode': '6084.0', 'toNode': '6355.0', 'Resistance': 0.00317, 'Reactance': 0.01386, 'Conductance': 0, 'Susceptance': 0.00211, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1812'},
        {'fromNode': '6091.0', 'toNode': '6176.0', 'Resistance': 0.01965, 'Reactance': 0.04804, 'Conductance': 0, 'Susceptance': 0.00629, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1813'},
        {'fromNode': '6251.0', 'toNode': '6091.0', 'Resistance': 0.00626, 'Reactance': 0.03578, 'Conductance': 0, 'Susceptance': 0.00538, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1814'},
        {'fromNode': '6265.0', 'toNode': '6092.0', 'Resistance': 0.03066, 'Reactance': 0.13415, 'Conductance': 0, 'Susceptance': 0.02043, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1815'},
        {'fromNode': '6092.0', 'toNode': '6340.0', 'Resistance': 0.02123, 'Reactance': 0.09289, 'Conductance': 0, 'Susceptance': 0.01415, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1816'},
        {'fromNode': '6094.0', 'toNode': '6093.0', 'Resistance': 0.01077, 'Reactance': 0.02588, 'Conductance': 0, 'Susceptance': 0.00351, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1817'},
        {'fromNode': '6093.0', 'toNode': '6191.0', 'Resistance': 0.00537, 'Reactance': 0.02229, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1818'},
        {'fromNode': '6094.0', 'toNode': '6352.0', 'Resistance': 0.00312, 'Reactance': 0.02171, 'Conductance': 0, 'Susceptance': 0.00355, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1819'},
        {'fromNode': '6094.0', 'toNode': '6352.0', 'Resistance': 0.00312, 'Reactance': 0.02171, 'Conductance': 0, 'Susceptance': 0.00355, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1820'},
        {'fromNode': '6116.0', 'toNode': '6095.0', 'Resistance': 0.02712, 'Reactance': 0.06628, 'Conductance': 0, 'Susceptance': 0.00868, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1821'},
        {'fromNode': '6095.0', 'toNode': '6353.0', 'Resistance': 0.00442, 'Reactance': 0.03018, 'Conductance': 0, 'Susceptance': 0.00514, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1822'},
        {'fromNode': '6095.0', 'toNode': '6353.0', 'Resistance': 0.00442, 'Reactance': 0.03018, 'Conductance': 0, 'Susceptance': 0.00514, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1823'},
        {'fromNode': '6095.0', 'toNode': '6358.0', 'Resistance': 0.00976, 'Reactance': 0.04048, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1824'},
        {'fromNode': '6196.0', 'toNode': '6096.0', 'Resistance': 0.02156, 'Reactance': 0.08942, 'Conductance': 0, 'Susceptance': 0.01517, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1825'},
        {'fromNode': '6096.0', 'toNode': '6202.0', 'Resistance': 0.01774, 'Reactance': 0.10129, 'Conductance': 0, 'Susceptance': 0.01522, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1826'},
        {'fromNode': '6096.0', 'toNode': '6340.0', 'Resistance': 0.02271, 'Reactance': 0.13634, 'Conductance': 0, 'Susceptance': 0.0185, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1827'},
        {'fromNode': '6097.0', 'toNode': '6220.0', 'Resistance': 0.00366, 'Reactance': 0.025, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1828'},
        {'fromNode': '6097.0', 'toNode': '6220.0', 'Resistance': 0.00366, 'Reactance': 0.025, 'Conductance': 0, 'Susceptance': 0.00425, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1829'},
        {'fromNode': '6097.0', 'toNode': '6225.0', 'Resistance': 0.01044, 'Reactance': 0.04569, 'Conductance': 0, 'Susceptance': 0.00696, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1830'},
        {'fromNode': '6098.0', 'toNode': '6123.0', 'Resistance': 0.00317, 'Reactance': 0.02048, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1831'},
        {'fromNode': '6098.0', 'toNode': '6123.0', 'Resistance': 0.00317, 'Reactance': 0.02048, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1832'},
        {'fromNode': '6098.0', 'toNode': '6208.0', 'Resistance': 0.00691, 'Reactance': 0.01691, 'Conductance': 0, 'Susceptance': 0.00221, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1833'},
        {'fromNode': '6098.0', 'toNode': '6289.0', 'Resistance': 0.00552, 'Reactance': 0.02415, 'Conductance': 0, 'Susceptance': 0.00368, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1834'},
        {'fromNode': '6099.0', 'toNode': '6204.0', 'Resistance': 0.00266, 'Reactance': 0.01519, 'Conductance': 0, 'Susceptance': 0.00228, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1835'},
        {'fromNode': '6099.0', 'toNode': '6225.0', 'Resistance': 0.0031, 'Reactance': 0.01769, 'Conductance': 0, 'Susceptance': 0.00266, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1836'},
        {'fromNode': '6099.0', 'toNode': '6310.0', 'Resistance': 0.00667, 'Reactance': 0.02767, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1837'},
        {'fromNode': '6100.0', 'toNode': '6128.0', 'Resistance': 0.01671, 'Reactance': 0.10034, 'Conductance': 0, 'Susceptance': 0.01362, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1838'},
        {'fromNode': '6180.0', 'toNode': '6100.0', 'Resistance': 0.00956, 'Reactance': 0.0418, 'Conductance': 0, 'Susceptance': 0.00637, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1839'},
        {'fromNode': '6283.0', 'toNode': '6100.0', 'Resistance': 0.00566, 'Reactance': 0.0323, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1840'},
        {'fromNode': '6102.0', 'toNode': '6101.0', 'Resistance': 0.00013, 'Reactance': 0.0085, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1307.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1841'},
        {'fromNode': '6104.0', 'toNode': '6101.0', 'Resistance': 0.00031, 'Reactance': 0.02423, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 378.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1842'},
        {'fromNode': '6105.0', 'toNode': '6101.0', 'Resistance': 0.00037, 'Reactance': 0.03898, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 368.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1843'},
        {'fromNode': '6106.0', 'toNode': '6101.0', 'Resistance': 0.00018, 'Reactance': 0.00963, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 560.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1844'},
        {'fromNode': '6103.0', 'toNode': '6102.0', 'Resistance': 0.00102, 'Reactance': 0.04506, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 439.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1845'},
        {'fromNode': '6103.0', 'toNode': '6117.0', 'Resistance': 0.00862, 'Reactance': 0.06005, 'Conductance': 0, 'Susceptance': 0.00982, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1846'},
        {'fromNode': '6103.0', 'toNode': '6117.0', 'Resistance': 0.00862, 'Reactance': 0.06005, 'Conductance': 0, 'Susceptance': 0.00982, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1847'},
        {'fromNode': '6103.0', 'toNode': '6183.0', 'Resistance': 0.01177, 'Reactance': 0.05149, 'Conductance': 0, 'Susceptance': 0.00784, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1848'},
        {'fromNode': '6108.0', 'toNode': '6107.0', 'Resistance': 0.00061, 'Reactance': 0.04374, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 380.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1849'},
        {'fromNode': '6110.0', 'toNode': '6107.0', 'Resistance': 0.00015, 'Reactance': 0.01079, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1181.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1850'},
        {'fromNode': '6111.0', 'toNode': '6107.0', 'Resistance': 0.00014, 'Reactance': 0.01069, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1208.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1851'},
        {'fromNode': '6107.0', 'toNode': '8082.0', 'Resistance': 0.00113, 'Reactance': 0.01368, 'Conductance': 0, 'Susceptance': 0.90909, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1852'},
        {'fromNode': '6109.0', 'toNode': '6108.0', 'Resistance': 0.00252, 'Reactance': 0.10367, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 137.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1853'},
        {'fromNode': '6108.0', 'toNode': '6192.0', 'Resistance': 0.00493, 'Reactance': 0.03524, 'Conductance': 0, 'Susceptance': 0.06426, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1854'},
        {'fromNode': '6338.0', 'toNode': '6109.0', 'Resistance': 0.03888, 'Reactance': 0.09504, 'Conductance': 0, 'Susceptance': 0.01244, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1855'},
        {'fromNode': '6113.0', 'toNode': '6175.0', 'Resistance': 0.00078, 'Reactance': 0.00501, 'Conductance': 0, 'Susceptance': 0.00096, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1856'},
        {'fromNode': '6113.0', 'toNode': '6175.0', 'Resistance': 0.00078, 'Reactance': 0.00501, 'Conductance': 0, 'Susceptance': 0.00096, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1857'},
        {'fromNode': '6113.0', 'toNode': '6345.0', 'Resistance': 0.00258, 'Reactance': 0.01129, 'Conductance': 0, 'Susceptance': 0.00172, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1858'},
        {'fromNode': '6114.0', 'toNode': '6204.0', 'Resistance': 0.01029, 'Reactance': 0.04585, 'Conductance': 0, 'Susceptance': 0.00672, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1859'},
        {'fromNode': '6114.0', 'toNode': '6261.0', 'Resistance': 0.00516, 'Reactance': 0.03097, 'Conductance': 0, 'Susceptance': 0.0042, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1860'},
        {'fromNode': '6159.0', 'toNode': '6115.0', 'Resistance': 0.00432, 'Reactance': 0.02954, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1861'},
        {'fromNode': '6159.0', 'toNode': '6115.0', 'Resistance': 0.00432, 'Reactance': 0.02954, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1862'},
        {'fromNode': '6115.0', 'toNode': '6191.0', 'Resistance': 0.0085, 'Reactance': 0.03718, 'Conductance': 0, 'Susceptance': 0.00566, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1863'},
        {'fromNode': '6115.0', 'toNode': '6225.0', 'Resistance': 0.0095, 'Reactance': 0.05707, 'Conductance': 0, 'Susceptance': 0.00774, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1864'},
        {'fromNode': '6115.0', 'toNode': '6250.0', 'Resistance': 0.00313, 'Reactance': 0.01394, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1865'},
        {'fromNode': '6181.0', 'toNode': '6116.0', 'Resistance': 0.01119, 'Reactance': 0.06392, 'Conductance': 0, 'Susceptance': 0.00961, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1866'},
        {'fromNode': '6119.0', 'toNode': '6118.0', 'Resistance': 0.00021, 'Reactance': 0.01219, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1133.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1867'},
        {'fromNode': '6118.0', 'toNode': '6349.0', 'Resistance': 9e-05, 'Reactance': 0.00182, 'Conductance': 0, 'Susceptance': 0.12727, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1868'},
        {'fromNode': '6120.0', 'toNode': '6119.0', 'Resistance': 0.00056, 'Reactance': 0.0289, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 350.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1869'},
        {'fromNode': '6120.0', 'toNode': '6119.0', 'Resistance': 0.00056, 'Reactance': 0.0289, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 350.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1870'},
        {'fromNode': '6119.0', 'toNode': '6354.0', 'Resistance': 0.00095, 'Reactance': 0.00925, 'Conductance': 0, 'Susceptance': 0.01779, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1871'},
        {'fromNode': '6290.0', 'toNode': '6120.0', 'Resistance': 0.00336, 'Reactance': 0.02297, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1872'},
        {'fromNode': '6290.0', 'toNode': '6120.0', 'Resistance': 0.00336, 'Reactance': 0.02297, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1873'},
        {'fromNode': '6121.0', 'toNode': '6174.0', 'Resistance': 0.00587, 'Reactance': 0.03351, 'Conductance': 0, 'Susceptance': 0.00504, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1874'},
        {'fromNode': '6121.0', 'toNode': '6179.0', 'Resistance': 0.0239, 'Reactance': 0.05842, 'Conductance': 0, 'Susceptance': 0.00765, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1875'},
        {'fromNode': '6220.0', 'toNode': '6121.0', 'Resistance': 0.0032, 'Reactance': 0.02187, 'Conductance': 0, 'Susceptance': 0.00372, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1876'},
        {'fromNode': '6220.0', 'toNode': '6121.0', 'Resistance': 0.0032, 'Reactance': 0.02187, 'Conductance': 0, 'Susceptance': 0.00372, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1877'},
        {'fromNode': '6122.0', 'toNode': '6327.0', 'Resistance': 0.03109, 'Reactance': 0.07602, 'Conductance': 0, 'Susceptance': 0.00995, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1878'},
        {'fromNode': '7337.0', 'toNode': '6122.0', 'Resistance': 0.02322, 'Reactance': 0.13264, 'Conductance': 0, 'Susceptance': 0.01993, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1879'},
        {'fromNode': '6233.0', 'toNode': '6123.0', 'Resistance': 0.00405, 'Reactance': 0.02315, 'Conductance': 0, 'Susceptance': 0.00348, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1880'},
        {'fromNode': '6124.0', 'toNode': '6131.0', 'Resistance': 0.01842, 'Reactance': 0.07644, 'Conductance': 0, 'Susceptance': 0.01297, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1881'},
        {'fromNode': '6191.0', 'toNode': '6124.0', 'Resistance': 0.00835, 'Reactance': 0.05393, 'Conductance': 0, 'Susceptance': 0.01029, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1882'},
        {'fromNode': '6124.0', 'toNode': '6314.0', 'Resistance': 0.01906, 'Reactance': 0.08338, 'Conductance': 0, 'Susceptance': 0.0127, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1883'},
        {'fromNode': '6180.0', 'toNode': '6125.0', 'Resistance': 0.00599, 'Reactance': 0.02482, 'Conductance': 0, 'Susceptance': 0.00421, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1884'},
        {'fromNode': '6125.0', 'toNode': '6322.0', 'Resistance': 0.01845, 'Reactance': 0.08069, 'Conductance': 0, 'Susceptance': 0.01229, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1885'},
        {'fromNode': '6126.0', 'toNode': '6130.0', 'Resistance': 0.00394, 'Reactance': 0.01755, 'Conductance': 0, 'Susceptance': 0.00257, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1886'},
        {'fromNode': '6126.0', 'toNode': '6176.0', 'Resistance': 0.00718, 'Reactance': 0.05004, 'Conductance': 0, 'Susceptance': 0.00818, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1887'},
        {'fromNode': '6126.0', 'toNode': '6176.0', 'Resistance': 0.00718, 'Reactance': 0.05004, 'Conductance': 0, 'Susceptance': 0.00818, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1888'},
        {'fromNode': '6126.0', 'toNode': '6313.0', 'Resistance': 0.00918, 'Reactance': 0.0551, 'Conductance': 0, 'Susceptance': 0.00748, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1889'},
        {'fromNode': '6127.0', 'toNode': '6237.0', 'Resistance': 0.01441, 'Reactance': 0.08233, 'Conductance': 0, 'Susceptance': 0.01237, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1890'},
        {'fromNode': '6338.0', 'toNode': '6127.0', 'Resistance': 0.01299, 'Reactance': 0.07418, 'Conductance': 0, 'Susceptance': 0.01115, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1891'},
        {'fromNode': '6127.0', 'toNode': '8102.0', 'Resistance': 0.03512, 'Reactance': 0.08585, 'Conductance': 0, 'Susceptance': 0.01124, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1892'},
        {'fromNode': '6128.0', 'toNode': '6276.0', 'Resistance': 0.01619, 'Reactance': 0.03957, 'Conductance': 0, 'Susceptance': 0.00518, 'MVA_rating': 200.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1893'},
        {'fromNode': '6129.0', 'toNode': '6309.0', 'Resistance': 0.02452, 'Reactance': 0.10724, 'Conductance': 0, 'Susceptance': 0.01633, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1894'},
        {'fromNode': '6129.0', 'toNode': '6336.0', 'Resistance': 0.01251, 'Reactance': 0.07513, 'Conductance': 0, 'Susceptance': 0.0102, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1895'},
        {'fromNode': '6130.0', 'toNode': '6230.0', 'Resistance': 0.0041, 'Reactance': 0.035, 'Conductance': 0, 'Susceptance': 0.0058, 'MVA_rating': 230.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1896'},
        {'fromNode': '6130.0', 'toNode': '6231.0', 'Resistance': 0.00525, 'Reactance': 0.02339, 'Conductance': 0, 'Susceptance': 0.00343, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1897'},
        {'fromNode': '6131.0', 'toNode': '6251.0', 'Resistance': 0.02579, 'Reactance': 0.06305, 'Conductance': 0, 'Susceptance': 0.00826, 'MVA_rating': 109.76, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1898'},
        {'fromNode': '6132.0', 'toNode': '6137.0', 'Resistance': 0.00931, 'Reactance': 0.05592, 'Conductance': 0, 'Susceptance': 0.00759, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1899'},
        {'fromNode': '6175.0', 'toNode': '6132.0', 'Resistance': 0.00341, 'Reactance': 0.01491, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1900'},
        {'fromNode': '6190.0', 'toNode': '6133.0', 'Resistance': 0.01462, 'Reactance': 0.06065, 'Conductance': 0, 'Susceptance': 0.01029, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1901'},
        {'fromNode': '6134.0', 'toNode': '6140.0', 'Resistance': 0.01102, 'Reactance': 0.04818, 'Conductance': 0, 'Susceptance': 0.00734, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1902'},
        {'fromNode': '6134.0', 'toNode': '6358.0', 'Resistance': 0.01529, 'Reactance': 0.06688, 'Conductance': 0, 'Susceptance': 0.01019, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1903'},
        {'fromNode': '6195.0', 'toNode': '6135.0', 'Resistance': 0.018, 'Reactance': 0.07467, 'Conductance': 0, 'Susceptance': 0.01267, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1904'},
        {'fromNode': '6358.0', 'toNode': '6136.0', 'Resistance': 0.00974, 'Reactance': 0.04339, 'Conductance': 0, 'Susceptance': 0.00636, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1905'},
        {'fromNode': '6137.0', 'toNode': '6184.0', 'Resistance': 0.01801, 'Reactance': 0.04326, 'Conductance': 0, 'Susceptance': 0.00587, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1906'},
        {'fromNode': '6137.0', 'toNode': '6217.0', 'Resistance': 0.00779, 'Reactance': 0.01906, 'Conductance': 0, 'Susceptance': 0.0025, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1907'},
        {'fromNode': '6157.0', 'toNode': '6138.0', 'Resistance': 0.03762, 'Reactance': 0.09196, 'Conductance': 0, 'Susceptance': 0.01204, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1908'},
        {'fromNode': '6138.0', 'toNode': '6286.0', 'Resistance': 0.02051, 'Reactance': 0.0914, 'Conductance': 0, 'Susceptance': 0.0134, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1909'},
        {'fromNode': '6139.0', 'toNode': '6339.0', 'Resistance': 0.01781, 'Reactance': 0.10172, 'Conductance': 0, 'Susceptance': 0.01528, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1910'},
        {'fromNode': '6142.0', 'toNode': '6141.0', 'Resistance': 0.0002, 'Reactance': 0.00909, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1326.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1911'},
        {'fromNode': '6142.0', 'toNode': '6141.0', 'Resistance': 0.0002, 'Reactance': 0.00909, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1326.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1912'},
        {'fromNode': '6144.0', 'toNode': '6141.0', 'Resistance': 0.00106, 'Reactance': 0.04778, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 265.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1913'},
        {'fromNode': '6145.0', 'toNode': '6141.0', 'Resistance': 0.00027, 'Reactance': 0.01788, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1093.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1914'},
        {'fromNode': '6146.0', 'toNode': '6141.0', 'Resistance': 0.00024, 'Reactance': 0.01722, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1090.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1915'},
        {'fromNode': '6147.0', 'toNode': '6141.0', 'Resistance': 0.0002, 'Reactance': 0.01493, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1332.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1916'},
        {'fromNode': '6148.0', 'toNode': '6141.0', 'Resistance': 0.00013, 'Reactance': 0.00914, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1248.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1917'},
        {'fromNode': '6149.0', 'toNode': '6141.0', 'Resistance': 4e-05, 'Reactance': 0.00474, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1255.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1918'},
        {'fromNode': '6141.0', 'toNode': '6161.0', 'Resistance': 0.00052, 'Reactance': 0.00627, 'Conductance': 0, 'Susceptance': 0.4163, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1919'},
        {'fromNode': '6141.0', 'toNode': '6197.0', 'Resistance': 0.00014, 'Reactance': 0.00318, 'Conductance': 0, 'Susceptance': 0.27573, 'MVA_rating': 4352.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1920'},
        {'fromNode': '6239.0', 'toNode': '6141.0', 'Resistance': 0.00025, 'Reactance': 0.00304, 'Conductance': 0, 'Susceptance': 0.20218, 'MVA_rating': 2400.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1921'},
        {'fromNode': '6143.0', 'toNode': '6142.0', 'Resistance': 0.0003, 'Reactance': 0.02159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1922'},
        {'fromNode': '6143.0', 'toNode': '6142.0', 'Resistance': 0.0003, 'Reactance': 0.02159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1923'},
        {'fromNode': '6143.0', 'toNode': '6142.0', 'Resistance': 0.0003, 'Reactance': 0.02159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1924'},
        {'fromNode': '6142.0', 'toNode': '6162.0', 'Resistance': 0.00533, 'Reactance': 0.03805, 'Conductance': 0, 'Susceptance': 0.06937, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1925'},
        {'fromNode': '6143.0', 'toNode': '6284.0', 'Resistance': 0.00613, 'Reactance': 0.04273, 'Conductance': 0, 'Susceptance': 0.00699, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1926'},
        {'fromNode': '6143.0', 'toNode': '6284.0', 'Resistance': 0.00613, 'Reactance': 0.04273, 'Conductance': 0, 'Susceptance': 0.00699, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1927'},
        {'fromNode': '6312.0', 'toNode': '6143.0', 'Resistance': 0.00214, 'Reactance': 0.01491, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1928'},
        {'fromNode': '6312.0', 'toNode': '6143.0', 'Resistance': 0.00214, 'Reactance': 0.01491, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1929'},
        {'fromNode': '6312.0', 'toNode': '6143.0', 'Resistance': 0.00214, 'Reactance': 0.01491, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1930'},
        {'fromNode': '6312.0', 'toNode': '6143.0', 'Resistance': 0.00214, 'Reactance': 0.01491, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1931'},
        {'fromNode': '6150.0', 'toNode': '6356.0', 'Resistance': 0.00408, 'Reactance': 0.02634, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1932'},
        {'fromNode': '6150.0', 'toNode': '6356.0', 'Resistance': 0.00408, 'Reactance': 0.02634, 'Conductance': 0, 'Susceptance': 0.00503, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1933'},
        {'fromNode': '6190.0', 'toNode': '6151.0', 'Resistance': 0.01488, 'Reactance': 0.06632, 'Conductance': 0, 'Susceptance': 0.00972, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1934'},
        {'fromNode': '6151.0', 'toNode': '6311.0', 'Resistance': 0.02053, 'Reactance': 0.09148, 'Conductance': 0, 'Susceptance': 0.01341, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1935'},
        {'fromNode': '6327.0', 'toNode': '6151.0', 'Resistance': 0.05056, 'Reactance': 0.12359, 'Conductance': 0, 'Susceptance': 0.01618, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1936'},
        {'fromNode': '6153.0', 'toNode': '6152.0', 'Resistance': 0.00286, 'Reactance': 0.10993, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 155.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1937'},
        {'fromNode': '6152.0', 'toNode': '6177.0', 'Resistance': 0.00504, 'Reactance': 0.036, 'Conductance': 0, 'Susceptance': 0.06563, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1938'},
        {'fromNode': '6152.0', 'toNode': '6198.0', 'Resistance': 0.00248, 'Reactance': 0.02429, 'Conductance': 0, 'Susceptance': 0.04673, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1939'},
        {'fromNode': '6241.0', 'toNode': '6153.0', 'Resistance': 0.0044, 'Reactance': 0.03067, 'Conductance': 0, 'Susceptance': 0.00501, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1940'},
        {'fromNode': '6241.0', 'toNode': '6153.0', 'Resistance': 0.0044, 'Reactance': 0.03067, 'Conductance': 0, 'Susceptance': 0.00501, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1941'},
        {'fromNode': '6153.0', 'toNode': '6324.0', 'Resistance': 0.00612, 'Reactance': 0.0418, 'Conductance': 0, 'Susceptance': 0.00711, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1942'},
        {'fromNode': '6153.0', 'toNode': '6324.0', 'Resistance': 0.00612, 'Reactance': 0.0418, 'Conductance': 0, 'Susceptance': 0.00711, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1943'},
        {'fromNode': '6176.0', 'toNode': '6154.0', 'Resistance': 0.01188, 'Reactance': 0.04928, 'Conductance': 0, 'Susceptance': 0.00836, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1944'},
        {'fromNode': '6252.0', 'toNode': '6154.0', 'Resistance': 0.02199, 'Reactance': 0.0912, 'Conductance': 0, 'Susceptance': 0.01548, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1945'},
        {'fromNode': '6155.0', 'toNode': '6284.0', 'Resistance': 0.00973, 'Reactance': 0.05559, 'Conductance': 0, 'Susceptance': 0.00835, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1946'},
        {'fromNode': '6156.0', 'toNode': '6282.0', 'Resistance': 0.00873, 'Reactance': 0.05961, 'Conductance': 0, 'Susceptance': 0.01014, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1947'},
        {'fromNode': '6158.0', 'toNode': '6341.0', 'Resistance': 0.02816, 'Reactance': 0.06884, 'Conductance': 0, 'Susceptance': 0.00901, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1948'},
        {'fromNode': '8104.0', 'toNode': '6158.0', 'Resistance': 0.00916, 'Reactance': 0.05232, 'Conductance': 0, 'Susceptance': 0.00786, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1949'},
        {'fromNode': '6281.0', 'toNode': '6159.0', 'Resistance': 0.00206, 'Reactance': 0.00903, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1950'},
        {'fromNode': '6160.0', 'toNode': '6175.0', 'Resistance': 0.00216, 'Reactance': 0.01479, 'Conductance': 0, 'Susceptance': 0.00252, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1951'},
        {'fromNode': '6160.0', 'toNode': '6175.0', 'Resistance': 0.00216, 'Reactance': 0.01479, 'Conductance': 0, 'Susceptance': 0.00252, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1952'},
        {'fromNode': '6160.0', 'toNode': '6233.0', 'Resistance': 0.00526, 'Reactance': 0.02342, 'Conductance': 0, 'Susceptance': 0.00343, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1953'},
        {'fromNode': '6160.0', 'toNode': '6330.0', 'Resistance': 0.00719, 'Reactance': 0.03146, 'Conductance': 0, 'Susceptance': 0.00479, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1954'},
        {'fromNode': '6162.0', 'toNode': '6161.0', 'Resistance': 0.00024, 'Reactance': 0.01685, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1132.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1955'},
        {'fromNode': '6164.0', 'toNode': '6161.0', 'Resistance': 0.00022, 'Reactance': 0.01652, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 880.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1956'},
        {'fromNode': '6165.0', 'toNode': '6161.0', 'Resistance': 0.00034, 'Reactance': 0.01841, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 879.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1957'},
        {'fromNode': '6166.0', 'toNode': '6161.0', 'Resistance': 0.00011, 'Reactance': 0.00775, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 880.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1958'},
        {'fromNode': '6167.0', 'toNode': '6161.0', 'Resistance': 0.0002, 'Reactance': 0.01447, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 883.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1959'},
        {'fromNode': '6161.0', 'toNode': '7018.0', 'Resistance': 0.00084, 'Reactance': 0.01712, 'Conductance': 0, 'Susceptance': 1.19421, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1960'},
        {'fromNode': '6163.0', 'toNode': '6162.0', 'Resistance': 0.00109, 'Reactance': 0.04275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 184.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1961'},
        {'fromNode': '6162.0', 'toNode': '6277.0', 'Resistance': 0.00506, 'Reactance': 0.03614, 'Conductance': 0, 'Susceptance': 0.0659, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1962'},
        {'fromNode': '6279.0', 'toNode': '6162.0', 'Resistance': 0.00087, 'Reactance': 0.00624, 'Conductance': 0, 'Susceptance': 0.01138, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1963'},
        {'fromNode': '6280.0', 'toNode': '6163.0', 'Resistance': 0.00495, 'Reactance': 0.02053, 'Conductance': 0, 'Susceptance': 0.00348, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1964'},
        {'fromNode': '6168.0', 'toNode': '6209.0', 'Resistance': 0.01804, 'Reactance': 0.07482, 'Conductance': 0, 'Susceptance': 0.0127, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1965'},
        {'fromNode': '6168.0', 'toNode': '6261.0', 'Resistance': 0.00295, 'Reactance': 0.00722, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1966'},
        {'fromNode': '6170.0', 'toNode': '6169.0', 'Resistance': 0.00027, 'Reactance': 0.01779, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1108.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1967'},
        {'fromNode': '6169.0', 'toNode': '6239.0', 'Resistance': 0.00065, 'Reactance': 0.00782, 'Conductance': 0, 'Susceptance': 0.51936, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '1968'},
        {'fromNode': '6171.0', 'toNode': '6170.0', 'Resistance': 0.00057, 'Reactance': 0.03437, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 297.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1969'},
        {'fromNode': '6170.0', 'toNode': '6235.0', 'Resistance': 0.0015, 'Reactance': 0.01464, 'Conductance': 0, 'Susceptance': 0.02816, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1970'},
        {'fromNode': '6308.0', 'toNode': '6170.0', 'Resistance': 0.00343, 'Reactance': 0.02448, 'Conductance': 0, 'Susceptance': 0.04463, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1971'},
        {'fromNode': '6170.0', 'toNode': '6334.0', 'Resistance': 0.00357, 'Reactance': 0.02552, 'Conductance': 0, 'Susceptance': 0.04652, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1972'},
        {'fromNode': '6236.0', 'toNode': '6171.0', 'Resistance': 0.00723, 'Reactance': 0.05035, 'Conductance': 0, 'Susceptance': 0.00823, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1973'},
        {'fromNode': '6173.0', 'toNode': '6172.0', 'Resistance': 0.0013, 'Reactance': 0.05717, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 137.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1974'},
        {'fromNode': '6173.0', 'toNode': '6265.0', 'Resistance': 0.00345, 'Reactance': 0.01429, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1975'},
        {'fromNode': '7020.0', 'toNode': '6173.0', 'Resistance': 0.03858, 'Reactance': 0.09432, 'Conductance': 0, 'Susceptance': 0.01235, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1976'},
        {'fromNode': '6174.0', 'toNode': '6203.0', 'Resistance': 0.00829, 'Reactance': 0.02025, 'Conductance': 0, 'Susceptance': 0.00265, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1977'},
        {'fromNode': '6174.0', 'toNode': '6250.0', 'Resistance': 0.00258, 'Reactance': 0.01476, 'Conductance': 0, 'Susceptance': 0.00222, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1978'},
        {'fromNode': '6174.0', 'toNode': '6320.0', 'Resistance': 0.00372, 'Reactance': 0.02404, 'Conductance': 0, 'Susceptance': 0.00459, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1979'},
        {'fromNode': '6174.0', 'toNode': '6320.0', 'Resistance': 0.00372, 'Reactance': 0.02404, 'Conductance': 0, 'Susceptance': 0.00459, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1980'},
        {'fromNode': '6175.0', 'toNode': '6194.0', 'Resistance': 0.01297, 'Reactance': 0.05381, 'Conductance': 0, 'Susceptance': 0.00913, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1981'},
        {'fromNode': '6175.0', 'toNode': '6289.0', 'Resistance': 0.01181, 'Reactance': 0.02887, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1982'},
        {'fromNode': '6230.0', 'toNode': '6176.0', 'Resistance': 0.0023, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0026, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1983'},
        {'fromNode': '6178.0', 'toNode': '6177.0', 'Resistance': 0.00082, 'Reactance': 0.04251, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 259.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1984'},
        {'fromNode': '6178.0', 'toNode': '6177.0', 'Resistance': 0.00082, 'Reactance': 0.04251, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 259.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '1985'},
        {'fromNode': '6235.0', 'toNode': '6177.0', 'Resistance': 0.00202, 'Reactance': 0.01441, 'Conductance': 0, 'Susceptance': 0.02628, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1986'},
        {'fromNode': '6177.0', 'toNode': '6287.0', 'Resistance': 0.00317, 'Reactance': 0.02266, 'Conductance': 0, 'Susceptance': 0.04132, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1987'},
        {'fromNode': '6177.0', 'toNode': '6299.0', 'Resistance': 0.00682, 'Reactance': 0.06665, 'Conductance': 0, 'Susceptance': 0.12825, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1988'},
        {'fromNode': '6177.0', 'toNode': '6308.0', 'Resistance': 0.00546, 'Reactance': 0.03901, 'Conductance': 0, 'Susceptance': 0.07112, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1989'},
        {'fromNode': '6334.0', 'toNode': '6177.0', 'Resistance': 0.00073, 'Reactance': 0.00718, 'Conductance': 0, 'Susceptance': 0.01381, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '1990'},
        {'fromNode': '6178.0', 'toNode': '6209.0', 'Resistance': 0.00915, 'Reactance': 0.04079, 'Conductance': 0, 'Susceptance': 0.00598, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1991'},
        {'fromNode': '6332.0', 'toNode': '6178.0', 'Resistance': 0.00369, 'Reactance': 0.02383, 'Conductance': 0, 'Susceptance': 0.00455, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1992'},
        {'fromNode': '6291.0', 'toNode': '6179.0', 'Resistance': 0.00323, 'Reactance': 0.0194, 'Conductance': 0, 'Susceptance': 0.00263, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1993'},
        {'fromNode': '6182.0', 'toNode': '6201.0', 'Resistance': 0.00497, 'Reactance': 0.02172, 'Conductance': 0, 'Susceptance': 0.00331, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1994'},
        {'fromNode': '6183.0', 'toNode': '6206.0', 'Resistance': 0.01289, 'Reactance': 0.07363, 'Conductance': 0, 'Susceptance': 0.01106, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1995'},
        {'fromNode': '6184.0', 'toNode': '6185.0', 'Resistance': 0.01258, 'Reactance': 0.03077, 'Conductance': 0, 'Susceptance': 0.00403, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1996'},
        {'fromNode': '6184.0', 'toNode': '6205.0', 'Resistance': 0.00904, 'Reactance': 0.04029, 'Conductance': 0, 'Susceptance': 0.00591, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1997'},
        {'fromNode': '6184.0', 'toNode': '6219.0', 'Resistance': 0.02323, 'Reactance': 0.05678, 'Conductance': 0, 'Susceptance': 0.00744, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1998'},
        {'fromNode': '6185.0', 'toNode': '6313.0', 'Resistance': 0.01416, 'Reactance': 0.03461, 'Conductance': 0, 'Susceptance': 0.00453, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '1999'},
        {'fromNode': '6186.0', 'toNode': '6187.0', 'Resistance': 0.00175, 'Reactance': 0.01131, 'Conductance': 0, 'Susceptance': 0.00216, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2000'},
        {'fromNode': '6186.0', 'toNode': '6187.0', 'Resistance': 0.00175, 'Reactance': 0.01131, 'Conductance': 0, 'Susceptance': 0.00216, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2001'},
        {'fromNode': '6186.0', 'toNode': '6351.0', 'Resistance': 0.00169, 'Reactance': 0.0118, 'Conductance': 0, 'Susceptance': 0.00193, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2002'},
        {'fromNode': '6186.0', 'toNode': '6351.0', 'Resistance': 0.00169, 'Reactance': 0.0118, 'Conductance': 0, 'Susceptance': 0.00193, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2003'},
        {'fromNode': '6187.0', 'toNode': '6219.0', 'Resistance': 0.00752, 'Reactance': 0.03293, 'Conductance': 0, 'Susceptance': 0.00502, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2004'},
        {'fromNode': '6187.0', 'toNode': '6289.0', 'Resistance': 0.00637, 'Reactance': 0.02643, 'Conductance': 0, 'Susceptance': 0.00449, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2005'},
        {'fromNode': '6189.0', 'toNode': '6188.0', 'Resistance': 0.00227, 'Reactance': 0.06067, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 178.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2006'},
        {'fromNode': '6188.0', 'toNode': '7305.0', 'Resistance': 0.00756, 'Reactance': 0.05401, 'Conductance': 0, 'Susceptance': 0.09846, 'MVA_rating': 460.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2007'},
        {'fromNode': '6327.0', 'toNode': '6189.0', 'Resistance': 0.01842, 'Reactance': 0.11058, 'Conductance': 0, 'Susceptance': 0.01501, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2008'},
        {'fromNode': '6193.0', 'toNode': '6192.0', 'Resistance': 0.00113, 'Reactance': 0.06451, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 125.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2009'},
        {'fromNode': '6193.0', 'toNode': '6206.0', 'Resistance': 0.01507, 'Reactance': 0.09046, 'Conductance': 0, 'Susceptance': 0.01228, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2010'},
        {'fromNode': '6193.0', 'toNode': '6326.0', 'Resistance': 0.03488, 'Reactance': 0.08525, 'Conductance': 0, 'Susceptance': 0.01116, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2011'},
        {'fromNode': '6196.0', 'toNode': '6194.0', 'Resistance': 0.01994, 'Reactance': 0.08268, 'Conductance': 0, 'Susceptance': 0.01403, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2012'},
        {'fromNode': '6194.0', 'toNode': '6231.0', 'Resistance': 0.00815, 'Reactance': 0.03567, 'Conductance': 0, 'Susceptance': 0.00543, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2013'},
        {'fromNode': '6232.0', 'toNode': '6194.0', 'Resistance': 0.00593, 'Reactance': 0.04134, 'Conductance': 0, 'Susceptance': 0.00676, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2014'},
        {'fromNode': '6195.0', 'toNode': '7314.0', 'Resistance': 0.01465, 'Reactance': 0.06078, 'Conductance': 0, 'Susceptance': 0.01031, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2015'},
        {'fromNode': '6198.0', 'toNode': '6197.0', 'Resistance': 0.00015, 'Reactance': 0.01413, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1372.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2016'},
        {'fromNode': '6197.0', 'toNode': '6234.0', 'Resistance': 0.00025, 'Reactance': 0.003, 'Conductance': 0, 'Susceptance': 0.19905, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2017'},
        {'fromNode': '6197.0', 'toNode': '6333.0', 'Resistance': 0.00012, 'Reactance': 0.00144, 'Conductance': 0, 'Susceptance': 0.09542, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2018'},
        {'fromNode': '6199.0', 'toNode': '6198.0', 'Resistance': 0.00048, 'Reactance': 0.01693, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 420.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2019'},
        {'fromNode': '6199.0', 'toNode': '6198.0', 'Resistance': 0.00048, 'Reactance': 0.01693, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 420.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2020'},
        {'fromNode': '6199.0', 'toNode': '6198.0', 'Resistance': 0.00048, 'Reactance': 0.01693, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 420.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2021'},
        {'fromNode': '6287.0', 'toNode': '6198.0', 'Resistance': 0.00092, 'Reactance': 0.00901, 'Conductance': 0, 'Susceptance': 0.01734, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2022'},
        {'fromNode': '6337.0', 'toNode': '6199.0', 'Resistance': 0.00218, 'Reactance': 0.01487, 'Conductance': 0, 'Susceptance': 0.00253, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2023'},
        {'fromNode': '6337.0', 'toNode': '6199.0', 'Resistance': 0.00218, 'Reactance': 0.01487, 'Conductance': 0, 'Susceptance': 0.00253, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2024'},
        {'fromNode': '6200.0', 'toNode': '6225.0', 'Resistance': 0.00252, 'Reactance': 0.01513, 'Conductance': 0, 'Susceptance': 0.00205, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2025'},
        {'fromNode': '6200.0', 'toNode': '6275.0', 'Resistance': 0.00286, 'Reactance': 0.01632, 'Conductance': 0, 'Susceptance': 0.00245, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2026'},
        {'fromNode': '6200.0', 'toNode': '6284.0', 'Resistance': 0.00797, 'Reactance': 0.03555, 'Conductance': 0, 'Susceptance': 0.00521, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2027'},
        {'fromNode': '6201.0', 'toNode': '6203.0', 'Resistance': 0.01894, 'Reactance': 0.04631, 'Conductance': 0, 'Susceptance': 0.00606, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2028'},
        {'fromNode': '6201.0', 'toNode': '6356.0', 'Resistance': 0.00518, 'Reactance': 0.02956, 'Conductance': 0, 'Susceptance': 0.00444, 'MVA_rating': 245.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2029'},
        {'fromNode': '6280.0', 'toNode': '6202.0', 'Resistance': 0.02835, 'Reactance': 0.06931, 'Conductance': 0, 'Susceptance': 0.00908, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2030'},
        {'fromNode': '6340.0', 'toNode': '6202.0', 'Resistance': 0.01509, 'Reactance': 0.08621, 'Conductance': 0, 'Susceptance': 0.01295, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2031'},
        {'fromNode': '6204.0', 'toNode': '6337.0', 'Resistance': 0.00539, 'Reactance': 0.03236, 'Conductance': 0, 'Susceptance': 0.00439, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2032'},
        {'fromNode': '6205.0', 'toNode': '6223.0', 'Resistance': 0.00232, 'Reactance': 0.01501, 'Conductance': 0, 'Susceptance': 0.00286, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2033'},
        {'fromNode': '6205.0', 'toNode': '6223.0', 'Resistance': 0.00232, 'Reactance': 0.01501, 'Conductance': 0, 'Susceptance': 0.00286, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2034'},
        {'fromNode': '6207.0', 'toNode': '6282.0', 'Resistance': 0.01027, 'Reactance': 0.04493, 'Conductance': 0, 'Susceptance': 0.00684, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2035'},
        {'fromNode': '6207.0', 'toNode': '6313.0', 'Resistance': 0.0096, 'Reactance': 0.05765, 'Conductance': 0, 'Susceptance': 0.00782, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2036'},
        {'fromNode': '6331.0', 'toNode': '6208.0', 'Resistance': 0.00218, 'Reactance': 0.0152, 'Conductance': 0, 'Susceptance': 0.00249, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2037'},
        {'fromNode': '6331.0', 'toNode': '6208.0', 'Resistance': 0.00218, 'Reactance': 0.0152, 'Conductance': 0, 'Susceptance': 0.00249, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2038'},
        {'fromNode': '6208.0', 'toNode': '6353.0', 'Resistance': 0.0108, 'Reactance': 0.04812, 'Conductance': 0, 'Susceptance': 0.00706, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2039'},
        {'fromNode': '6309.0', 'toNode': '6209.0', 'Resistance': 0.018, 'Reactance': 0.078, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2040'},
        {'fromNode': '6211.0', 'toNode': '6210.0', 'Resistance': 0.0003, 'Reactance': 0.01639, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 978.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2041'},
        {'fromNode': '6213.0', 'toNode': '6210.0', 'Resistance': 0.00066, 'Reactance': 0.04478, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 363.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2042'},
        {'fromNode': '6214.0', 'toNode': '6210.0', 'Resistance': 0.00044, 'Reactance': 0.02757, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 718.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2043'},
        {'fromNode': '6215.0', 'toNode': '6210.0', 'Resistance': 0.0001, 'Reactance': 0.0071, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 721.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2044'},
        {'fromNode': '6216.0', 'toNode': '6210.0', 'Resistance': 0.00021, 'Reactance': 0.01596, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 710.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2045'},
        {'fromNode': '6210.0', 'toNode': '6292.0', 'Resistance': 0.00114, 'Reactance': 0.01377, 'Conductance': 0, 'Susceptance': 0.91489, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2046'},
        {'fromNode': '6212.0', 'toNode': '6211.0', 'Resistance': 0.00083, 'Reactance': 0.03328, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 381.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2047'},
        {'fromNode': '6211.0', 'toNode': '6359.0', 'Resistance': 0.00367, 'Reactance': 0.02624, 'Conductance': 0, 'Susceptance': 0.04784, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2048'},
        {'fromNode': '6218.0', 'toNode': '6231.0', 'Resistance': 0.02084, 'Reactance': 0.09119, 'Conductance': 0, 'Susceptance': 0.01389, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2049'},
        {'fromNode': '6278.0', 'toNode': '6218.0', 'Resistance': 0.00985, 'Reactance': 0.04084, 'Conductance': 0, 'Susceptance': 0.00693, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2050'},
        {'fromNode': '6219.0', 'toNode': '6345.0', 'Resistance': 0.00408, 'Reactance': 0.02332, 'Conductance': 0, 'Susceptance': 0.0035, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2051'},
        {'fromNode': '6221.0', 'toNode': '6322.0', 'Resistance': 0.01605, 'Reactance': 0.09634, 'Conductance': 0, 'Susceptance': 0.01307, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2052'},
        {'fromNode': '6223.0', 'toNode': '6222.0', 'Resistance': 0.00029, 'Reactance': 0.0171, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 457.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2053'},
        {'fromNode': '6223.0', 'toNode': '6222.0', 'Resistance': 0.00029, 'Reactance': 0.0171, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 457.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2054'},
        {'fromNode': '6223.0', 'toNode': '6222.0', 'Resistance': 0.00029, 'Reactance': 0.0171, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 457.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2055'},
        {'fromNode': '6222.0', 'toNode': '6229.0', 'Resistance': 0.00375, 'Reactance': 0.03669, 'Conductance': 0, 'Susceptance': 0.07059, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2056'},
        {'fromNode': '6255.0', 'toNode': '6222.0', 'Resistance': 0.00167, 'Reactance': 0.01635, 'Conductance': 0, 'Susceptance': 0.03146, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2057'},
        {'fromNode': '6222.0', 'toNode': '6293.0', 'Resistance': 0.00233, 'Reactance': 0.0228, 'Conductance': 0, 'Susceptance': 0.04388, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2058'},
        {'fromNode': '6222.0', 'toNode': '6354.0', 'Resistance': 0.00491, 'Reactance': 0.03508, 'Conductance': 0, 'Susceptance': 0.06396, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2059'},
        {'fromNode': '6227.0', 'toNode': '6223.0', 'Resistance': 0.00231, 'Reactance': 0.01613, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2060'},
        {'fromNode': '6227.0', 'toNode': '6223.0', 'Resistance': 0.00231, 'Reactance': 0.01613, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2061'},
        {'fromNode': '6224.0', 'toNode': '6331.0', 'Resistance': 0.00467, 'Reactance': 0.01938, 'Conductance': 0, 'Susceptance': 0.00329, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2062'},
        {'fromNode': '6225.0', 'toNode': '6307.0', 'Resistance': 0.00669, 'Reactance': 0.0298, 'Conductance': 0, 'Susceptance': 0.00437, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2063'},
        {'fromNode': '6227.0', 'toNode': '6238.0', 'Resistance': 0.00232, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2064'},
        {'fromNode': '6227.0', 'toNode': '6238.0', 'Resistance': 0.00232, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2065'},
        {'fromNode': '6227.0', 'toNode': '6330.0', 'Resistance': 0.01472, 'Reactance': 0.036, 'Conductance': 0, 'Susceptance': 0.00471, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2066'},
        {'fromNode': '6229.0', 'toNode': '6228.0', 'Resistance': 0.00011, 'Reactance': 0.01313, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1208.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2067'},
        {'fromNode': '6298.0', 'toNode': '6228.0', 'Resistance': 0.00022, 'Reactance': 0.00262, 'Conductance': 0, 'Susceptance': 0.17409, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2068'},
        {'fromNode': '6230.0', 'toNode': '6229.0', 'Resistance': 0.00061, 'Reactance': 0.03602, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 350.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2069'},
        {'fromNode': '6230.0', 'toNode': '6229.0', 'Resistance': 0.00061, 'Reactance': 0.03602, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 350.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2070'},
        {'fromNode': '6277.0', 'toNode': '6229.0', 'Resistance': 0.00147, 'Reactance': 0.01434, 'Conductance': 0, 'Susceptance': 0.0276, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2071'},
        {'fromNode': '6230.0', 'toNode': '6278.0', 'Resistance': 0.009, 'Reactance': 0.05404, 'Conductance': 0, 'Susceptance': 0.00733, 'MVA_rating': 210.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2072'},
        {'fromNode': '6232.0', 'toNode': '6294.0', 'Resistance': 0.00252, 'Reactance': 0.01629, 'Conductance': 0, 'Susceptance': 0.00311, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2073'},
        {'fromNode': '6232.0', 'toNode': '6294.0', 'Resistance': 0.00252, 'Reactance': 0.01629, 'Conductance': 0, 'Susceptance': 0.00311, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2074'},
        {'fromNode': '6235.0', 'toNode': '6234.0', 'Resistance': 6e-05, 'Reactance': 0.00475, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1190.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2075'},
        {'fromNode': '6236.0', 'toNode': '6235.0', 'Resistance': 0.00075, 'Reactance': 0.02962, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 406.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2076'},
        {'fromNode': '6236.0', 'toNode': '6235.0', 'Resistance': 0.00075, 'Reactance': 0.02962, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 406.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2077'},
        {'fromNode': '6325.0', 'toNode': '6237.0', 'Resistance': 0.01418, 'Reactance': 0.05883, 'Conductance': 0, 'Susceptance': 0.00998, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2078'},
        {'fromNode': '6237.0', 'toNode': '8102.0', 'Resistance': 0.02383, 'Reactance': 0.14308, 'Conductance': 0, 'Susceptance': 0.01942, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2079'},
        {'fromNode': '6238.0', 'toNode': '6345.0', 'Resistance': 0.00383, 'Reactance': 0.02298, 'Conductance': 0, 'Susceptance': 0.00312, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2080'},
        {'fromNode': '6240.0', 'toNode': '6239.0', 'Resistance': 0.00028, 'Reactance': 0.01264, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1194.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2081'},
        {'fromNode': '6242.0', 'toNode': '6239.0', 'Resistance': 0.0004, 'Reactance': 0.04713, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 380.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2082'},
        {'fromNode': '6243.0', 'toNode': '6239.0', 'Resistance': 0.00024, 'Reactance': 0.01934, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 690.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2083'},
        {'fromNode': '6244.0', 'toNode': '6239.0', 'Resistance': 0.00024, 'Reactance': 0.01637, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 957.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2084'},
        {'fromNode': '6245.0', 'toNode': '6239.0', 'Resistance': 0.00021, 'Reactance': 0.01383, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 965.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2085'},
        {'fromNode': '6246.0', 'toNode': '6239.0', 'Resistance': 0.00011, 'Reactance': 0.00711, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 967.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2086'},
        {'fromNode': '6247.0', 'toNode': '6239.0', 'Resistance': 0.00014, 'Reactance': 0.00867, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 694.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2087'},
        {'fromNode': '6248.0', 'toNode': '6239.0', 'Resistance': 0.00018, 'Reactance': 0.01749, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 689.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2088'},
        {'fromNode': '6249.0', 'toNode': '6239.0', 'Resistance': 0.00046, 'Reactance': 0.02531, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 685.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2089'},
        {'fromNode': '6239.0', 'toNode': '7414.0', 'Resistance': 0.0015, 'Reactance': 0.019, 'Conductance': 0, 'Susceptance': 1.24, 'MVA_rating': 2000.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2090'},
        {'fromNode': '6241.0', 'toNode': '6240.0', 'Resistance': 0.00111, 'Reactance': 0.05088, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2091'},
        {'fromNode': '6241.0', 'toNode': '6240.0', 'Resistance': 0.00111, 'Reactance': 0.05088, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2092'},
        {'fromNode': '6240.0', 'toNode': '6287.0', 'Resistance': 0.00218, 'Reactance': 0.02128, 'Conductance': 0, 'Susceptance': 0.04095, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2093'},
        {'fromNode': '6240.0', 'toNode': '6321.0', 'Resistance': 0.00473, 'Reactance': 0.03381, 'Conductance': 0, 'Susceptance': 0.06164, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2094'},
        {'fromNode': '6253.0', 'toNode': '6329.0', 'Resistance': 0.01564, 'Reactance': 0.06488, 'Conductance': 0, 'Susceptance': 0.01101, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2095'},
        {'fromNode': '6255.0', 'toNode': '6254.0', 'Resistance': 4e-05, 'Reactance': 0.00481, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1350.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2096'},
        {'fromNode': '6257.0', 'toNode': '6254.0', 'Resistance': 0.00011, 'Reactance': 0.01147, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 726.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2097'},
        {'fromNode': '6258.0', 'toNode': '6254.0', 'Resistance': 0.00037, 'Reactance': 0.02186, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 731.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2098'},
        {'fromNode': '6259.0', 'toNode': '6254.0', 'Resistance': 0.00044, 'Reactance': 0.02142, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 333.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2099'},
        {'fromNode': '6260.0', 'toNode': '6254.0', 'Resistance': 0.00018, 'Reactance': 0.01038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 914.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2100'},
        {'fromNode': '6292.0', 'toNode': '6254.0', 'Resistance': 7e-05, 'Reactance': 0.00142, 'Conductance': 0, 'Susceptance': 0.09917, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2101'},
        {'fromNode': '6256.0', 'toNode': '6255.0', 'Resistance': 0.00085, 'Reactance': 0.04835, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2102'},
        {'fromNode': '6256.0', 'toNode': '6255.0', 'Resistance': 0.00085, 'Reactance': 0.04835, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2103'},
        {'fromNode': '6262.0', 'toNode': '6319.0', 'Resistance': 0.01214, 'Reactance': 0.07287, 'Conductance': 0, 'Susceptance': 0.00989, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2104'},
        {'fromNode': '6264.0', 'toNode': '6263.0', 'Resistance': 0.00081, 'Reactance': 0.03319, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 120.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2105'},
        {'fromNode': '6321.0', 'toNode': '6263.0', 'Resistance': 0.00395, 'Reactance': 0.03863, 'Conductance': 0, 'Susceptance': 0.07433, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2106'},
        {'fromNode': '6263.0', 'toNode': '7274.0', 'Resistance': 0.00452, 'Reactance': 0.04698, 'Conductance': 0, 'Susceptance': 0.16009, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2107'},
        {'fromNode': '6266.0', 'toNode': '6265.0', 'Resistance': 0.00703, 'Reactance': 0.24052, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 29.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2108'},
        {'fromNode': '6267.0', 'toNode': '6265.0', 'Resistance': 0.01927, 'Reactance': 0.55179, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 35.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2109'},
        {'fromNode': '6268.0', 'toNode': '6265.0', 'Resistance': 0.00822, 'Reactance': 0.17407, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 30.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2110'},
        {'fromNode': '6269.0', 'toNode': '6265.0', 'Resistance': 0.0207, 'Reactance': 0.59011, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 32.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2111'},
        {'fromNode': '6270.0', 'toNode': '6265.0', 'Resistance': 0.01825, 'Reactance': 0.32424, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 47.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2112'},
        {'fromNode': '6271.0', 'toNode': '6265.0', 'Resistance': 0.00336, 'Reactance': 0.1392, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 42.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2113'},
        {'fromNode': '6272.0', 'toNode': '6265.0', 'Resistance': 0.00455, 'Reactance': 0.19054, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 35.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2114'},
        {'fromNode': '6273.0', 'toNode': '6265.0', 'Resistance': 0.00863, 'Reactance': 0.26216, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 32.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2115'},
        {'fromNode': '6274.0', 'toNode': '6265.0', 'Resistance': 0.00902, 'Reactance': 0.4175, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 39.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2116'},
        {'fromNode': '6275.0', 'toNode': '6324.0', 'Resistance': 0.00458, 'Reactance': 0.02955, 'Conductance': 0, 'Susceptance': 0.00564, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2117'},
        {'fromNode': '6275.0', 'toNode': '6324.0', 'Resistance': 0.00458, 'Reactance': 0.02955, 'Conductance': 0, 'Susceptance': 0.00564, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2118'},
        {'fromNode': '6278.0', 'toNode': '6277.0', 'Resistance': 0.00046, 'Reactance': 0.02831, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 331.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2119'},
        {'fromNode': '6280.0', 'toNode': '6279.0', 'Resistance': 0.00106, 'Reactance': 0.03718, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 182.78, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2120'},
        {'fromNode': '6279.0', 'toNode': '6299.0', 'Resistance': 0.00218, 'Reactance': 0.02135, 'Conductance': 0, 'Susceptance': 0.04107, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2121'},
        {'fromNode': '6286.0', 'toNode': '6285.0', 'Resistance': 0.00032, 'Reactance': 0.0236, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 311.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2122'},
        {'fromNode': '6328.0', 'toNode': '6286.0', 'Resistance': 0.0331, 'Reactance': 0.08092, 'Conductance': 0, 'Susceptance': 0.0106, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2123'},
        {'fromNode': '6288.0', 'toNode': '6287.0', 'Resistance': 0.00058, 'Reactance': 0.03855, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2124'},
        {'fromNode': '6288.0', 'toNode': '6287.0', 'Resistance': 0.00058, 'Reactance': 0.03855, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2125'},
        {'fromNode': '6288.0', 'toNode': '6307.0', 'Resistance': 0.00199, 'Reactance': 0.01356, 'Conductance': 0, 'Susceptance': 0.00231, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2126'},
        {'fromNode': '6288.0', 'toNode': '6307.0', 'Resistance': 0.00199, 'Reactance': 0.01356, 'Conductance': 0, 'Susceptance': 0.00231, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2127'},
        {'fromNode': '6288.0', 'toNode': '6323.0', 'Resistance': 0.0013, 'Reactance': 0.00886, 'Conductance': 0, 'Susceptance': 0.00151, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2128'},
        {'fromNode': '6288.0', 'toNode': '6323.0', 'Resistance': 0.0013, 'Reactance': 0.00886, 'Conductance': 0, 'Susceptance': 0.00151, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2129'},
        {'fromNode': '6290.0', 'toNode': '6331.0', 'Resistance': 0.00175, 'Reactance': 0.01218, 'Conductance': 0, 'Susceptance': 0.00199, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2130'},
        {'fromNode': '6290.0', 'toNode': '6331.0', 'Resistance': 0.00175, 'Reactance': 0.01218, 'Conductance': 0, 'Susceptance': 0.00199, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2131'},
        {'fromNode': '6293.0', 'toNode': '6292.0', 'Resistance': 0.0002, 'Reactance': 0.01411, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1210.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2132'},
        {'fromNode': '6295.0', 'toNode': '6292.0', 'Resistance': 0.00017, 'Reactance': 0.00896, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 756.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2133'},
        {'fromNode': '6296.0', 'toNode': '6292.0', 'Resistance': 0.00013, 'Reactance': 0.00766, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 762.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2134'},
        {'fromNode': '6297.0', 'toNode': '6292.0', 'Resistance': 0.00029, 'Reactance': 0.02084, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 765.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2135'},
        {'fromNode': '6294.0', 'toNode': '6293.0', 'Resistance': 0.00058, 'Reactance': 0.02427, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 217.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2136'},
        {'fromNode': '6294.0', 'toNode': '6339.0', 'Resistance': 0.01129, 'Reactance': 0.02712, 'Conductance': 0, 'Susceptance': 0.00368, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2137'},
        {'fromNode': '6299.0', 'toNode': '6298.0', 'Resistance': 0.0002, 'Reactance': 0.01939, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 994.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2138'},
        {'fromNode': '6301.0', 'toNode': '6298.0', 'Resistance': 0.00035, 'Reactance': 0.02541, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 225.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2139'},
        {'fromNode': '6302.0', 'toNode': '6298.0', 'Resistance': 0.00029, 'Reactance': 0.02162, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 897.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2140'},
        {'fromNode': '6303.0', 'toNode': '6298.0', 'Resistance': 0.0001, 'Reactance': 0.00733, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 887.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2141'},
        {'fromNode': '6304.0', 'toNode': '6298.0', 'Resistance': 0.00026, 'Reactance': 0.01816, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 896.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2142'},
        {'fromNode': '6305.0', 'toNode': '6298.0', 'Resistance': 0.00037, 'Reactance': 0.02188, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 904.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2143'},
        {'fromNode': '6300.0', 'toNode': '6299.0', 'Resistance': 0.00189, 'Reactance': 0.06692, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 181.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2144'},
        {'fromNode': '6309.0', 'toNode': '6308.0', 'Resistance': 0.00086, 'Reactance': 0.06271, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 183.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2145'},
        {'fromNode': '6310.0', 'toNode': '6312.0', 'Resistance': 0.00222, 'Reactance': 0.01548, 'Conductance': 0, 'Susceptance': 0.00253, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2146'},
        {'fromNode': '6310.0', 'toNode': '6312.0', 'Resistance': 0.00222, 'Reactance': 0.01548, 'Conductance': 0, 'Susceptance': 0.00253, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2147'},
        {'fromNode': '6320.0', 'toNode': '6312.0', 'Resistance': 0.00148, 'Reactance': 0.01008, 'Conductance': 0, 'Susceptance': 0.00171, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2148'},
        {'fromNode': '6320.0', 'toNode': '6312.0', 'Resistance': 0.00148, 'Reactance': 0.01008, 'Conductance': 0, 'Susceptance': 0.00171, 'MVA_rating': 375.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2149'},
        {'fromNode': '6316.0', 'toNode': '6315.0', 'Resistance': 0.00143, 'Reactance': 0.0765, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 251.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2150'},
        {'fromNode': '6343.0', 'toNode': '6315.0', 'Resistance': 0.00542, 'Reactance': 0.0387, 'Conductance': 0, 'Susceptance': 0.07055, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2151'},
        {'fromNode': '6317.0', 'toNode': '6316.0', 'Resistance': 0.00181, 'Reactance': 0.06783, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 189.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2152'},
        {'fromNode': '6318.0', 'toNode': '6317.0', 'Resistance': 0.00192, 'Reactance': 0.08014, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 198.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2153'},
        {'fromNode': '6319.0', 'toNode': '6318.0', 'Resistance': 0.00221, 'Reactance': 0.05812, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 163.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2154'},
        {'fromNode': '6324.0', 'toNode': '6320.0', 'Resistance': 0.00553, 'Reactance': 0.02416, 'Conductance': 0, 'Susceptance': 0.00368, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2155'},
        {'fromNode': '6322.0', 'toNode': '6321.0', 'Resistance': 0.00114, 'Reactance': 0.03057, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 249.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2156'},
        {'fromNode': '8140.0', 'toNode': '6326.0', 'Resistance': 0.02706, 'Reactance': 0.06615, 'Conductance': 0, 'Susceptance': 0.00866, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2157'},
        {'fromNode': '6336.0', 'toNode': '6328.0', 'Resistance': 0.02009, 'Reactance': 0.12065, 'Conductance': 0, 'Susceptance': 0.01637, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2158'},
        {'fromNode': '6334.0', 'toNode': '6333.0', 'Resistance': 0.00033, 'Reactance': 0.01665, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1094.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2159'},
        {'fromNode': '6335.0', 'toNode': '6334.0', 'Resistance': 0.00063, 'Reactance': 0.03435, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 271.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2160'},
        {'fromNode': '6341.0', 'toNode': '7202.0', 'Resistance': 0.01405, 'Reactance': 0.06262, 'Conductance': 0, 'Susceptance': 0.00918, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2161'},
        {'fromNode': '6343.0', 'toNode': '6342.0', 'Resistance': 0.00025, 'Reactance': 0.01679, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1137.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2162'},
        {'fromNode': '6344.0', 'toNode': '6343.0', 'Resistance': 0.00028, 'Reactance': 0.01611, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2163'},
        {'fromNode': '6344.0', 'toNode': '6343.0', 'Resistance': 0.00028, 'Reactance': 0.01611, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2164'},
        {'fromNode': '6343.0', 'toNode': '6350.0', 'Resistance': 0.00292, 'Reactance': 0.02086, 'Conductance': 0, 'Susceptance': 0.03803, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2165'},
        {'fromNode': '6343.0', 'toNode': '6354.0', 'Resistance': 0.00111, 'Reactance': 0.0079, 'Conductance': 0, 'Susceptance': 0.01439, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2166'},
        {'fromNode': '6344.0', 'toNode': '6353.0', 'Resistance': 0.00211, 'Reactance': 0.0144, 'Conductance': 0, 'Susceptance': 0.00245, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2167'},
        {'fromNode': '6344.0', 'toNode': '6353.0', 'Resistance': 0.00211, 'Reactance': 0.0144, 'Conductance': 0, 'Susceptance': 0.00245, 'MVA_rating': 262.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2168'},
        {'fromNode': '6350.0', 'toNode': '6349.0', 'Resistance': 0.00019, 'Reactance': 0.01229, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1150.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2169'},
        {'fromNode': '6351.0', 'toNode': '6350.0', 'Resistance': 0.00179, 'Reactance': 0.06637, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 233.1, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2170'},
        {'fromNode': '6351.0', 'toNode': '6350.0', 'Resistance': 0.00179, 'Reactance': 0.06637, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 233.1, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2171'},
        {'fromNode': '6350.0', 'toNode': '6359.0', 'Resistance': 0.00816, 'Reactance': 0.05827, 'Conductance': 0, 'Susceptance': 0.10623, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2172'},
        {'fromNode': '6355.0', 'toNode': '6354.0', 'Resistance': 0.00076, 'Reactance': 0.02753, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 193.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2173'},
        {'fromNode': '6355.0', 'toNode': '6354.0', 'Resistance': 0.00076, 'Reactance': 0.02753, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 193.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2174'},
        {'fromNode': '6360.0', 'toNode': '6359.0', 'Resistance': 0.00288, 'Reactance': 0.10862, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 136.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2175'},
        {'fromNode': '7044.0', 'toNode': '7001.0', 'Resistance': 0.00753, 'Reactance': 0.043, 'Conductance': 0, 'Susceptance': 0.00646, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2176'},
        {'fromNode': '7149.0', 'toNode': '7001.0', 'Resistance': 0.02671, 'Reactance': 0.0653, 'Conductance': 0, 'Susceptance': 0.00855, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2177'},
        {'fromNode': '7001.0', 'toNode': '7337.0', 'Resistance': 0.04314, 'Reactance': 0.10362, 'Conductance': 0, 'Susceptance': 0.01406, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2178'},
        {'fromNode': '7003.0', 'toNode': '7002.0', 'Resistance': 0.00022, 'Reactance': 0.0149, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1299.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2179'},
        {'fromNode': '7005.0', 'toNode': '7002.0', 'Resistance': 0.00041, 'Reactance': 0.02493, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 630.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2180'},
        {'fromNode': '7006.0', 'toNode': '7002.0', 'Resistance': 0.00012, 'Reactance': 0.01417, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 626.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2181'},
        {'fromNode': '7007.0', 'toNode': '7002.0', 'Resistance': 0.00034, 'Reactance': 0.02135, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 639.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2182'},
        {'fromNode': '7008.0', 'toNode': '7002.0', 'Resistance': 0.00045, 'Reactance': 0.02245, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 629.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2183'},
        {'fromNode': '7009.0', 'toNode': '7002.0', 'Resistance': 0.00049, 'Reactance': 0.02499, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 625.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2184'},
        {'fromNode': '7104.0', 'toNode': '7002.0', 'Resistance': 3e-05, 'Reactance': 0.0007, 'Conductance': 0, 'Susceptance': 0.04887, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2185'},
        {'fromNode': '7002.0', 'toNode': '7227.0', 'Resistance': 0.00036, 'Reactance': 0.00731, 'Conductance': 0, 'Susceptance': 0.51017, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2186'},
        {'fromNode': '7366.0', 'toNode': '7002.0', 'Resistance': 0.00016, 'Reactance': 0.00189, 'Conductance': 0, 'Susceptance': 0.12566, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2187'},
        {'fromNode': '7004.0', 'toNode': '7003.0', 'Resistance': 0.0009, 'Reactance': 0.03512, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 386.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2188'},
        {'fromNode': '7004.0', 'toNode': '7003.0', 'Resistance': 0.0009, 'Reactance': 0.03512, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 386.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2189'},
        {'fromNode': '7004.0', 'toNode': '7003.0', 'Resistance': 0.0009, 'Reactance': 0.03512, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 386.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2190'},
        {'fromNode': '7003.0', 'toNode': '7264.0', 'Resistance': 0.00225, 'Reactance': 0.01608, 'Conductance': 0, 'Susceptance': 0.02932, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2191'},
        {'fromNode': '7075.0', 'toNode': '7004.0', 'Resistance': 0.00199, 'Reactance': 0.01386, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2192'},
        {'fromNode': '7075.0', 'toNode': '7004.0', 'Resistance': 0.00199, 'Reactance': 0.01386, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2193'},
        {'fromNode': '7075.0', 'toNode': '7004.0', 'Resistance': 0.00199, 'Reactance': 0.01386, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2194'},
        {'fromNode': '7004.0', 'toNode': '7106.0', 'Resistance': 0.0021, 'Reactance': 0.01353, 'Conductance': 0, 'Susceptance': 0.00258, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2195'},
        {'fromNode': '7004.0', 'toNode': '7106.0', 'Resistance': 0.0021, 'Reactance': 0.01353, 'Conductance': 0, 'Susceptance': 0.00258, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2196'},
        {'fromNode': '7004.0', 'toNode': '7106.0', 'Resistance': 0.0021, 'Reactance': 0.01353, 'Conductance': 0, 'Susceptance': 0.00258, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2197'},
        {'fromNode': '7004.0', 'toNode': '7106.0', 'Resistance': 0.0021, 'Reactance': 0.01353, 'Conductance': 0, 'Susceptance': 0.00258, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2198'},
        {'fromNode': '7004.0', 'toNode': '7345.0', 'Resistance': 0.00591, 'Reactance': 0.04035, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2199'},
        {'fromNode': '7004.0', 'toNode': '7345.0', 'Resistance': 0.00591, 'Reactance': 0.04035, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2200'},
        {'fromNode': '7004.0', 'toNode': '7345.0', 'Resistance': 0.00591, 'Reactance': 0.04035, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2201'},
        {'fromNode': '7010.0', 'toNode': '7182.0', 'Resistance': 0.01896, 'Reactance': 0.11385, 'Conductance': 0, 'Susceptance': 0.01545, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2202'},
        {'fromNode': '7010.0', 'toNode': '7202.0', 'Resistance': 0.00553, 'Reactance': 0.02294, 'Conductance': 0, 'Susceptance': 0.00389, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2203'},
        {'fromNode': '7233.0', 'toNode': '7010.0', 'Resistance': 0.00425, 'Reactance': 0.01763, 'Conductance': 0, 'Susceptance': 0.00299, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2204'},
        {'fromNode': '7254.0', 'toNode': '7011.0', 'Resistance': 0.00899, 'Reactance': 0.04008, 'Conductance': 0, 'Susceptance': 0.00588, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2205'},
        {'fromNode': '7011.0', 'toNode': '7300.0', 'Resistance': 0.00567, 'Reactance': 0.0248, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2206'},
        {'fromNode': '7012.0', 'toNode': '7116.0', 'Resistance': 0.00531, 'Reactance': 0.03627, 'Conductance': 0, 'Susceptance': 0.00617, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2207'},
        {'fromNode': '7343.0', 'toNode': '7012.0', 'Resistance': 0.00603, 'Reactance': 0.04198, 'Conductance': 0, 'Susceptance': 0.00686, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2208'},
        {'fromNode': '7013.0', 'toNode': '7201.0', 'Resistance': 0.00433, 'Reactance': 0.0302, 'Conductance': 0, 'Susceptance': 0.00494, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2209'},
        {'fromNode': '7013.0', 'toNode': '7201.0', 'Resistance': 0.00433, 'Reactance': 0.0302, 'Conductance': 0, 'Susceptance': 0.00494, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2210'},
        {'fromNode': '7013.0', 'toNode': '7425.0', 'Resistance': 0.003, 'Reactance': 0.02091, 'Conductance': 0, 'Susceptance': 0.00342, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2211'},
        {'fromNode': '7015.0', 'toNode': '7014.0', 'Resistance': 0.00981, 'Reactance': 0.21665, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 43.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2212'},
        {'fromNode': '7014.0', 'toNode': '7100.0', 'Resistance': 0.00476, 'Reactance': 0.03249, 'Conductance': 0, 'Susceptance': 0.00553, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2213'},
        {'fromNode': '7014.0', 'toNode': '7100.0', 'Resistance': 0.00476, 'Reactance': 0.03249, 'Conductance': 0, 'Susceptance': 0.00553, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2214'},
        {'fromNode': '7014.0', 'toNode': '7124.0', 'Resistance': 0.01606, 'Reactance': 0.03857, 'Conductance': 0, 'Susceptance': 0.00523, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2215'},
        {'fromNode': '7014.0', 'toNode': '7217.0', 'Resistance': 0.00415, 'Reactance': 0.02371, 'Conductance': 0, 'Susceptance': 0.00356, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2216'},
        {'fromNode': '7016.0', 'toNode': '7365.0', 'Resistance': 0.0068, 'Reactance': 0.04642, 'Conductance': 0, 'Susceptance': 0.0079, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2217'},
        {'fromNode': '7016.0', 'toNode': '7365.0', 'Resistance': 0.0068, 'Reactance': 0.04642, 'Conductance': 0, 'Susceptance': 0.0079, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2218'},
        {'fromNode': '7016.0', 'toNode': '7365.0', 'Resistance': 0.0068, 'Reactance': 0.04642, 'Conductance': 0, 'Susceptance': 0.0079, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2219'},
        {'fromNode': '7016.0', 'toNode': '7365.0', 'Resistance': 0.0068, 'Reactance': 0.04642, 'Conductance': 0, 'Susceptance': 0.0079, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2220'},
        {'fromNode': '7368.0', 'toNode': '7016.0', 'Resistance': 0.002, 'Reactance': 0.01369, 'Conductance': 0, 'Susceptance': 0.00233, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2221'},
        {'fromNode': '7368.0', 'toNode': '7016.0', 'Resistance': 0.002, 'Reactance': 0.01369, 'Conductance': 0, 'Susceptance': 0.00233, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2222'},
        {'fromNode': '7368.0', 'toNode': '7016.0', 'Resistance': 0.002, 'Reactance': 0.01369, 'Conductance': 0, 'Susceptance': 0.00233, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2223'},
        {'fromNode': '7016.0', 'toNode': '7402.0', 'Resistance': 0.00314, 'Reactance': 0.02144, 'Conductance': 0, 'Susceptance': 0.00365, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2224'},
        {'fromNode': '7016.0', 'toNode': '7402.0', 'Resistance': 0.00314, 'Reactance': 0.02144, 'Conductance': 0, 'Susceptance': 0.00365, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2225'},
        {'fromNode': '7089.0', 'toNode': '7017.0', 'Resistance': 0.0044, 'Reactance': 0.01826, 'Conductance': 0, 'Susceptance': 0.0031, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2226'},
        {'fromNode': '7017.0', 'toNode': '7183.0', 'Resistance': 0.00737, 'Reactance': 0.04422, 'Conductance': 0, 'Susceptance': 0.006, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2227'},
        {'fromNode': '7017.0', 'toNode': '7229.0', 'Resistance': 0.00604, 'Reactance': 0.04129, 'Conductance': 0, 'Susceptance': 0.00702, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2228'},
        {'fromNode': '7017.0', 'toNode': '7229.0', 'Resistance': 0.00604, 'Reactance': 0.04129, 'Conductance': 0, 'Susceptance': 0.00702, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2229'},
        {'fromNode': '7017.0', 'toNode': '7229.0', 'Resistance': 0.00604, 'Reactance': 0.04129, 'Conductance': 0, 'Susceptance': 0.00702, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2230'},
        {'fromNode': '7017.0', 'toNode': '7241.0', 'Resistance': 0.00382, 'Reactance': 0.01669, 'Conductance': 0, 'Susceptance': 0.00254, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2231'},
        {'fromNode': '7017.0', 'toNode': '7385.0', 'Resistance': 0.00298, 'Reactance': 0.01922, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2232'},
        {'fromNode': '7017.0', 'toNode': '7385.0', 'Resistance': 0.00298, 'Reactance': 0.01922, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2233'},
        {'fromNode': '7019.0', 'toNode': '7018.0', 'Resistance': 0.00095, 'Reactance': 0.04341, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 383.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2234'},
        {'fromNode': '7021.0', 'toNode': '7018.0', 'Resistance': 0.00069, 'Reactance': 0.05321, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 311.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2235'},
        {'fromNode': '7022.0', 'toNode': '7018.0', 'Resistance': 0.0007, 'Reactance': 0.05667, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 328.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2236'},
        {'fromNode': '7023.0', 'toNode': '7018.0', 'Resistance': 0.00056, 'Reactance': 0.03486, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 326.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2237'},
        {'fromNode': '7024.0', 'toNode': '7018.0', 'Resistance': 0.00019, 'Reactance': 0.01166, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 456.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2238'},
        {'fromNode': '7025.0', 'toNode': '7018.0', 'Resistance': 0.00094, 'Reactance': 0.07002, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 223.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2239'},
        {'fromNode': '7042.0', 'toNode': '7018.0', 'Resistance': 0.00022, 'Reactance': 0.00265, 'Conductance': 0, 'Susceptance': 0.17583, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2240'},
        {'fromNode': '7018.0', 'toNode': '7414.0', 'Resistance': 0.00039, 'Reactance': 0.00468, 'Conductance': 0, 'Susceptance': 0.3108, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2241'},
        {'fromNode': '7020.0', 'toNode': '7019.0', 'Resistance': 0.00076, 'Reactance': 0.04319, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 244.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2242'},
        {'fromNode': '7043.0', 'toNode': '7019.0', 'Resistance': 0.00225, 'Reactance': 0.01607, 'Conductance': 0, 'Susceptance': 0.0293, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2243'},
        {'fromNode': '7019.0', 'toNode': '7274.0', 'Resistance': 0.00332, 'Reactance': 0.02368, 'Conductance': 0, 'Susceptance': 0.04318, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2244'},
        {'fromNode': '7026.0', 'toNode': '7072.0', 'Resistance': 0.0032, 'Reactance': 0.02067, 'Conductance': 0, 'Susceptance': 0.00394, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2245'},
        {'fromNode': '7026.0', 'toNode': '7072.0', 'Resistance': 0.0032, 'Reactance': 0.02067, 'Conductance': 0, 'Susceptance': 0.00394, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2246'},
        {'fromNode': '7026.0', 'toNode': '7259.0', 'Resistance': 0.01125, 'Reactance': 0.07265, 'Conductance': 0, 'Susceptance': 0.01386, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2247'},
        {'fromNode': '7273.0', 'toNode': '7026.0', 'Resistance': 0.00548, 'Reactance': 0.0382, 'Conductance': 0, 'Susceptance': 0.00624, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2248'},
        {'fromNode': '7273.0', 'toNode': '7026.0', 'Resistance': 0.00548, 'Reactance': 0.0382, 'Conductance': 0, 'Susceptance': 0.00624, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2249'},
        {'fromNode': '7027.0', 'toNode': '7157.0', 'Resistance': 0.00228, 'Reactance': 0.01559, 'Conductance': 0, 'Susceptance': 0.00265, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2250'},
        {'fromNode': '7027.0', 'toNode': '7157.0', 'Resistance': 0.00228, 'Reactance': 0.01559, 'Conductance': 0, 'Susceptance': 0.00265, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2251'},
        {'fromNode': '7027.0', 'toNode': '7234.0', 'Resistance': 0.00431, 'Reactance': 0.01886, 'Conductance': 0, 'Susceptance': 0.00287, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2252'},
        {'fromNode': '7027.0', 'toNode': '7245.0', 'Resistance': 0.00974, 'Reactance': 0.04259, 'Conductance': 0, 'Susceptance': 0.00649, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2253'},
        {'fromNode': '7027.0', 'toNode': '7431.0', 'Resistance': 0.015, 'Reactance': 0.06687, 'Conductance': 0, 'Susceptance': 0.00981, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2254'},
        {'fromNode': '7147.0', 'toNode': '7028.0', 'Resistance': 0.0011, 'Reactance': 0.00768, 'Conductance': 0, 'Susceptance': 0.00126, 'MVA_rating': 315.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2255'},
        {'fromNode': '7147.0', 'toNode': '7028.0', 'Resistance': 0.0011, 'Reactance': 0.00768, 'Conductance': 0, 'Susceptance': 0.00126, 'MVA_rating': 315.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2256'},
        {'fromNode': '7028.0', 'toNode': '7220.0', 'Resistance': 0.00381, 'Reactance': 0.02289, 'Conductance': 0, 'Susceptance': 0.00311, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2257'},
        {'fromNode': '7028.0', 'toNode': '7240.0', 'Resistance': 0.00642, 'Reactance': 0.01568, 'Conductance': 0, 'Susceptance': 0.00205, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2258'},
        {'fromNode': '7028.0', 'toNode': '7330.0', 'Resistance': 0.01183, 'Reactance': 0.02841, 'Conductance': 0, 'Susceptance': 0.00385, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2259'},
        {'fromNode': '7030.0', 'toNode': '7029.0', 'Resistance': 0.00115, 'Reactance': 0.08558, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 126.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2260'},
        {'fromNode': '7031.0', 'toNode': '7029.0', 'Resistance': 0.00416, 'Reactance': 0.1856, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 107.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2261'},
        {'fromNode': '7032.0', 'toNode': '7029.0', 'Resistance': 0.00698, 'Reactance': 0.23196, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 73.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2262'},
        {'fromNode': '7033.0', 'toNode': '7029.0', 'Resistance': 0.00566, 'Reactance': 0.21439, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 84.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2263'},
        {'fromNode': '7034.0', 'toNode': '7029.0', 'Resistance': 0.00462, 'Reactance': 0.30603, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 63.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2264'},
        {'fromNode': '7274.0', 'toNode': '7029.0', 'Resistance': 0.00498, 'Reactance': 0.03556, 'Conductance': 0, 'Susceptance': 0.06482, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2265'},
        {'fromNode': '7029.0', 'toNode': '7407.0', 'Resistance': 0.00374, 'Reactance': 0.02669, 'Conductance': 0, 'Susceptance': 0.04866, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2266'},
        {'fromNode': '7415.0', 'toNode': '7029.0', 'Resistance': 0.00221, 'Reactance': 0.01579, 'Conductance': 0, 'Susceptance': 0.02879, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2267'},
        {'fromNode': '7036.0', 'toNode': '7035.0', 'Resistance': 0.00036, 'Reactance': 0.02039, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 368.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2268'},
        {'fromNode': '7040.0', 'toNode': '7035.0', 'Resistance': 0.00756, 'Reactance': 0.05402, 'Conductance': 0, 'Susceptance': 0.09848, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2269'},
        {'fromNode': '7056.0', 'toNode': '7035.0', 'Resistance': 0.00217, 'Reactance': 0.01547, 'Conductance': 0, 'Susceptance': 0.02821, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2270'},
        {'fromNode': '7035.0', 'toNode': '7390.0', 'Resistance': 0.00154, 'Reactance': 0.01505, 'Conductance': 0, 'Susceptance': 0.02896, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2271'},
        {'fromNode': '7035.0', 'toNode': '8031.0', 'Resistance': 0.00526, 'Reactance': 0.05146, 'Conductance': 0, 'Susceptance': 0.09901, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2272'},
        {'fromNode': '7036.0', 'toNode': '7128.0', 'Resistance': 0.005, 'Reactance': 0.02228, 'Conductance': 0, 'Susceptance': 0.00327, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2273'},
        {'fromNode': '7431.0', 'toNode': '7036.0', 'Resistance': 0.00448, 'Reactance': 0.03123, 'Conductance': 0, 'Susceptance': 0.00511, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2274'},
        {'fromNode': '7431.0', 'toNode': '7036.0', 'Resistance': 0.00448, 'Reactance': 0.03123, 'Conductance': 0, 'Susceptance': 0.00511, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2275'},
        {'fromNode': '7038.0', 'toNode': '7037.0', 'Resistance': 0.00016, 'Reactance': 0.00995, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1055.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2276'},
        {'fromNode': '7047.0', 'toNode': '7037.0', 'Resistance': 0.00024, 'Reactance': 0.00293, 'Conductance': 0, 'Susceptance': 0.19466, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2277'},
        {'fromNode': '7037.0', 'toNode': '7428.0', 'Resistance': 0.0006, 'Reactance': 0.01232, 'Conductance': 0, 'Susceptance': 0.85919, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2278'},
        {'fromNode': '7039.0', 'toNode': '7038.0', 'Resistance': 0.00084, 'Reactance': 0.02618, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 630.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2279'},
        {'fromNode': '7039.0', 'toNode': '7038.0', 'Resistance': 0.00084, 'Reactance': 0.02618, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 630.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2280'},
        {'fromNode': '7126.0', 'toNode': '7038.0', 'Resistance': 0.00089, 'Reactance': 0.00924, 'Conductance': 0, 'Susceptance': 0.03147, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2281'},
        {'fromNode': '7126.0', 'toNode': '7038.0', 'Resistance': 0.00089, 'Reactance': 0.00924, 'Conductance': 0, 'Susceptance': 0.03147, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2282'},
        {'fromNode': '7038.0', 'toNode': '7184.0', 'Resistance': 0.00247, 'Reactance': 0.02419, 'Conductance': 0, 'Susceptance': 0.04653, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2283'},
        {'fromNode': '7409.0', 'toNode': '7038.0', 'Resistance': 0.0006, 'Reactance': 0.00622, 'Conductance': 0, 'Susceptance': 0.02121, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2284'},
        {'fromNode': '7409.0', 'toNode': '7038.0', 'Resistance': 0.0006, 'Reactance': 0.00622, 'Conductance': 0, 'Susceptance': 0.02121, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2285'},
        {'fromNode': '7039.0', 'toNode': '7101.0', 'Resistance': 0.00285, 'Reactance': 0.01988, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2286'},
        {'fromNode': '7039.0', 'toNode': '7101.0', 'Resistance': 0.00285, 'Reactance': 0.01988, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2287'},
        {'fromNode': '7039.0', 'toNode': '7101.0', 'Resistance': 0.00285, 'Reactance': 0.01988, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2288'},
        {'fromNode': '7039.0', 'toNode': '7397.0', 'Resistance': 0.02904, 'Reactance': 0.06977, 'Conductance': 0, 'Susceptance': 0.00947, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2289'},
        {'fromNode': '7041.0', 'toNode': '7040.0', 'Resistance': 0.00089, 'Reactance': 0.03362, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 337.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2290'},
        {'fromNode': '7041.0', 'toNode': '7040.0', 'Resistance': 0.00089, 'Reactance': 0.03362, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 337.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2291'},
        {'fromNode': '7040.0', 'toNode': '7077.0', 'Resistance': 0.00208, 'Reactance': 0.02159, 'Conductance': 0, 'Susceptance': 0.07357, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2292'},
        {'fromNode': '7040.0', 'toNode': '7077.0', 'Resistance': 0.00208, 'Reactance': 0.02159, 'Conductance': 0, 'Susceptance': 0.07357, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2293'},
        {'fromNode': '7041.0', 'toNode': '7238.0', 'Resistance': 0.01373, 'Reactance': 0.06121, 'Conductance': 0, 'Susceptance': 0.00898, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2294'},
        {'fromNode': '7394.0', 'toNode': '7041.0', 'Resistance': 0.00776, 'Reactance': 0.05303, 'Conductance': 0, 'Susceptance': 0.00902, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2295'},
        {'fromNode': '7394.0', 'toNode': '7041.0', 'Resistance': 0.00776, 'Reactance': 0.05303, 'Conductance': 0, 'Susceptance': 0.00902, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2296'},
        {'fromNode': '7043.0', 'toNode': '7042.0', 'Resistance': 8e-05, 'Reactance': 0.00908, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1048.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2297'},
        {'fromNode': '7045.0', 'toNode': '7042.0', 'Resistance': 0.00029, 'Reactance': 0.01951, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 729.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2298'},
        {'fromNode': '7046.0', 'toNode': '7042.0', 'Resistance': 0.00013, 'Reactance': 0.009, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 716.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2299'},
        {'fromNode': '7058.0', 'toNode': '7042.0', 'Resistance': 0.0006, 'Reactance': 0.00731, 'Conductance': 0, 'Susceptance': 0.48578, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2300'},
        {'fromNode': '7044.0', 'toNode': '7043.0', 'Resistance': 0.00124, 'Reactance': 0.05616, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 137.4, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2301'},
        {'fromNode': '7044.0', 'toNode': '7416.0', 'Resistance': 0.01974, 'Reactance': 0.04824, 'Conductance': 0, 'Susceptance': 0.00632, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2302'},
        {'fromNode': '7048.0', 'toNode': '7047.0', 'Resistance': 0.00011, 'Reactance': 0.00819, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1125.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2303'},
        {'fromNode': '7199.0', 'toNode': '7047.0', 'Resistance': 0.00024, 'Reactance': 0.00294, 'Conductance': 0, 'Susceptance': 0.19535, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2304'},
        {'fromNode': '7049.0', 'toNode': '7048.0', 'Resistance': 0.00068, 'Reactance': 0.02653, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 521.92, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2305'},
        {'fromNode': '7049.0', 'toNode': '7048.0', 'Resistance': 0.00068, 'Reactance': 0.02653, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 521.92, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2306'},
        {'fromNode': '7228.0', 'toNode': '7048.0', 'Resistance': 0.00068, 'Reactance': 0.00665, 'Conductance': 0, 'Susceptance': 0.01279, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2307'},
        {'fromNode': '7342.0', 'toNode': '7048.0', 'Resistance': 0.00134, 'Reactance': 0.01309, 'Conductance': 0, 'Susceptance': 0.02519, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2308'},
        {'fromNode': '7049.0', 'toNode': '7089.0', 'Resistance': 0.00671, 'Reactance': 0.04677, 'Conductance': 0, 'Susceptance': 0.00764, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2309'},
        {'fromNode': '7315.0', 'toNode': '7049.0', 'Resistance': 0.00436, 'Reactance': 0.03039, 'Conductance': 0, 'Susceptance': 0.00497, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2310'},
        {'fromNode': '7315.0', 'toNode': '7049.0', 'Resistance': 0.00436, 'Reactance': 0.03039, 'Conductance': 0, 'Susceptance': 0.00497, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2311'},
        {'fromNode': '7315.0', 'toNode': '7049.0', 'Resistance': 0.00436, 'Reactance': 0.03039, 'Conductance': 0, 'Susceptance': 0.00497, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2312'},
        {'fromNode': '7050.0', 'toNode': '7090.0', 'Resistance': 0.00351, 'Reactance': 0.02269, 'Conductance': 0, 'Susceptance': 0.00433, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2313'},
        {'fromNode': '7050.0', 'toNode': '7090.0', 'Resistance': 0.00351, 'Reactance': 0.02269, 'Conductance': 0, 'Susceptance': 0.00433, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2314'},
        {'fromNode': '7050.0', 'toNode': '7251.0', 'Resistance': 0.01093, 'Reactance': 0.04781, 'Conductance': 0, 'Susceptance': 0.00728, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2315'},
        {'fromNode': '7051.0', 'toNode': '7086.0', 'Resistance': 0.01063, 'Reactance': 0.0465, 'Conductance': 0, 'Susceptance': 0.00708, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2316'},
        {'fromNode': '7051.0', 'toNode': '7181.0', 'Resistance': 0.00631, 'Reactance': 0.03605, 'Conductance': 0, 'Susceptance': 0.00542, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2317'},
        {'fromNode': '7276.0', 'toNode': '7051.0', 'Resistance': 0.00383, 'Reactance': 0.01676, 'Conductance': 0, 'Susceptance': 0.00255, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2318'},
        {'fromNode': '7051.0', 'toNode': '7365.0', 'Resistance': 0.00182, 'Reactance': 0.01243, 'Conductance': 0, 'Susceptance': 0.00211, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2319'},
        {'fromNode': '7051.0', 'toNode': '7365.0', 'Resistance': 0.00182, 'Reactance': 0.01243, 'Conductance': 0, 'Susceptance': 0.00211, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2320'},
        {'fromNode': '7052.0', 'toNode': '7053.0', 'Resistance': 0.00939, 'Reactance': 0.06413, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2321'},
        {'fromNode': '7052.0', 'toNode': '7053.0', 'Resistance': 0.00939, 'Reactance': 0.06413, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2322'},
        {'fromNode': '7052.0', 'toNode': '7053.0', 'Resistance': 0.00939, 'Reactance': 0.06413, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2323'},
        {'fromNode': '7052.0', 'toNode': '7183.0', 'Resistance': 0.00858, 'Reactance': 0.03822, 'Conductance': 0, 'Susceptance': 0.0056, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2324'},
        {'fromNode': '7052.0', 'toNode': '7221.0', 'Resistance': 0.00969, 'Reactance': 0.0402, 'Conductance': 0, 'Susceptance': 0.00682, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2325'},
        {'fromNode': '7256.0', 'toNode': '7052.0', 'Resistance': 0.00256, 'Reactance': 0.01655, 'Conductance': 0, 'Susceptance': 0.00316, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2326'},
        {'fromNode': '7256.0', 'toNode': '7052.0', 'Resistance': 0.00256, 'Reactance': 0.01655, 'Conductance': 0, 'Susceptance': 0.00316, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2327'},
        {'fromNode': '7053.0', 'toNode': '7348.0', 'Resistance': 0.00216, 'Reactance': 0.01507, 'Conductance': 0, 'Susceptance': 0.00246, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2328'},
        {'fromNode': '7053.0', 'toNode': '7348.0', 'Resistance': 0.00216, 'Reactance': 0.01507, 'Conductance': 0, 'Susceptance': 0.00246, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2329'},
        {'fromNode': '7053.0', 'toNode': '7348.0', 'Resistance': 0.00216, 'Reactance': 0.01507, 'Conductance': 0, 'Susceptance': 0.00246, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2330'},
        {'fromNode': '7054.0', 'toNode': '7086.0', 'Resistance': 0.01035, 'Reactance': 0.06217, 'Conductance': 0, 'Susceptance': 0.00844, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2331'},
        {'fromNode': '7243.0', 'toNode': '7054.0', 'Resistance': 0.00558, 'Reactance': 0.03601, 'Conductance': 0, 'Susceptance': 0.00687, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2332'},
        {'fromNode': '7054.0', 'toNode': '7387.0', 'Resistance': 0.00566, 'Reactance': 0.0323, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2333'},
        {'fromNode': '7055.0', 'toNode': '7124.0', 'Resistance': 0.00357, 'Reactance': 0.02439, 'Conductance': 0, 'Susceptance': 0.00415, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2334'},
        {'fromNode': '7055.0', 'toNode': '7124.0', 'Resistance': 0.00357, 'Reactance': 0.02439, 'Conductance': 0, 'Susceptance': 0.00415, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2335'},
        {'fromNode': '7188.0', 'toNode': '7055.0', 'Resistance': 0.0014, 'Reactance': 0.00978, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2336'},
        {'fromNode': '7188.0', 'toNode': '7055.0', 'Resistance': 0.0014, 'Reactance': 0.00978, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2337'},
        {'fromNode': '7188.0', 'toNode': '7055.0', 'Resistance': 0.0014, 'Reactance': 0.00978, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2338'},
        {'fromNode': '7188.0', 'toNode': '7055.0', 'Resistance': 0.0014, 'Reactance': 0.00978, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2339'},
        {'fromNode': '7250.0', 'toNode': '7055.0', 'Resistance': 0.00146, 'Reactance': 0.01001, 'Conductance': 0, 'Susceptance': 0.0017, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2340'},
        {'fromNode': '7250.0', 'toNode': '7055.0', 'Resistance': 0.00146, 'Reactance': 0.01001, 'Conductance': 0, 'Susceptance': 0.0017, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2341'},
        {'fromNode': '7057.0', 'toNode': '7056.0', 'Resistance': 0.00161, 'Reactance': 0.04976, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 377.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2342'},
        {'fromNode': '7056.0', 'toNode': '7156.0', 'Resistance': 0.00287, 'Reactance': 0.02047, 'Conductance': 0, 'Susceptance': 0.03732, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2343'},
        {'fromNode': '7056.0', 'toNode': '7160.0', 'Resistance': 0.00187, 'Reactance': 0.01944, 'Conductance': 0, 'Susceptance': 0.06623, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2344'},
        {'fromNode': '7056.0', 'toNode': '7420.0', 'Resistance': 0.0013, 'Reactance': 0.0093, 'Conductance': 0, 'Susceptance': 0.01695, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2345'},
        {'fromNode': '7215.0', 'toNode': '7057.0', 'Resistance': 0.00223, 'Reactance': 0.01557, 'Conductance': 0, 'Susceptance': 0.00254, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2346'},
        {'fromNode': '7215.0', 'toNode': '7057.0', 'Resistance': 0.00223, 'Reactance': 0.01557, 'Conductance': 0, 'Susceptance': 0.00254, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2347'},
        {'fromNode': '7059.0', 'toNode': '7058.0', 'Resistance': 0.0002, 'Reactance': 0.01603, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 605.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2348'},
        {'fromNode': '7061.0', 'toNode': '7058.0', 'Resistance': 0.0003, 'Reactance': 0.02449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 535.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2349'},
        {'fromNode': '7062.0', 'toNode': '7058.0', 'Resistance': 0.0003, 'Reactance': 0.02239, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 531.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2350'},
        {'fromNode': '7063.0', 'toNode': '7058.0', 'Resistance': 0.00039, 'Reactance': 0.02417, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 527.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2351'},
        {'fromNode': '7064.0', 'toNode': '7058.0', 'Resistance': 0.00017, 'Reactance': 0.01851, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 530.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2352'},
        {'fromNode': '7065.0', 'toNode': '7058.0', 'Resistance': 0.00031, 'Reactance': 0.02035, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 520.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2353'},
        {'fromNode': '7066.0', 'toNode': '7058.0', 'Resistance': 0.0003, 'Reactance': 0.02188, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 542.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2354'},
        {'fromNode': '7058.0', 'toNode': '7095.0', 'Resistance': 0.00064, 'Reactance': 0.00771, 'Conductance': 0, 'Susceptance': 0.5124, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2355'},
        {'fromNode': '7406.0', 'toNode': '7058.0', 'Resistance': 6e-05, 'Reactance': 0.00073, 'Conductance': 0, 'Susceptance': 0.04856, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2356'},
        {'fromNode': '7060.0', 'toNode': '7059.0', 'Resistance': 0.00168, 'Reactance': 0.05948, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 188.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2357'},
        {'fromNode': '7059.0', 'toNode': '7407.0', 'Resistance': 0.00062, 'Reactance': 0.00444, 'Conductance': 0, 'Susceptance': 0.00809, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2358'},
        {'fromNode': '7129.0', 'toNode': '7060.0', 'Resistance': 0.01191, 'Reactance': 0.05306, 'Conductance': 0, 'Susceptance': 0.00778, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2359'},
        {'fromNode': '7067.0', 'toNode': '7245.0', 'Resistance': 0.00829, 'Reactance': 0.04734, 'Conductance': 0, 'Susceptance': 0.00711, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2360'},
        {'fromNode': '7067.0', 'toNode': '7291.0', 'Resistance': 0.00748, 'Reactance': 0.04274, 'Conductance': 0, 'Susceptance': 0.00642, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2361'},
        {'fromNode': '7067.0', 'toNode': '7344.0', 'Resistance': 0.01002, 'Reactance': 0.02449, 'Conductance': 0, 'Susceptance': 0.00321, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2362'},
        {'fromNode': '7069.0', 'toNode': '7068.0', 'Resistance': 0.0023, 'Reactance': 0.07154, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 218.7, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2363'},
        {'fromNode': '7305.0', 'toNode': '7068.0', 'Resistance': 0.0021, 'Reactance': 0.0205, 'Conductance': 0, 'Susceptance': 0.03945, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2364'},
        {'fromNode': '7069.0', 'toNode': '7413.0', 'Resistance': 0.01312, 'Reactance': 0.05846, 'Conductance': 0, 'Susceptance': 0.00857, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2365'},
        {'fromNode': '7070.0', 'toNode': '7100.0', 'Resistance': 0.00621, 'Reactance': 0.04324, 'Conductance': 0, 'Susceptance': 0.00707, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2366'},
        {'fromNode': '7070.0', 'toNode': '7257.0', 'Resistance': 0.00619, 'Reactance': 0.03538, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2367'},
        {'fromNode': '7070.0', 'toNode': '7317.0', 'Resistance': 0.00528, 'Reactance': 0.03607, 'Conductance': 0, 'Susceptance': 0.00614, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2368'},
        {'fromNode': '7070.0', 'toNode': '7398.0', 'Resistance': 0.00238, 'Reactance': 0.01063, 'Conductance': 0, 'Susceptance': 0.00156, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2369'},
        {'fromNode': '7123.0', 'toNode': '7071.0', 'Resistance': 0.0119, 'Reactance': 0.02858, 'Conductance': 0, 'Susceptance': 0.00388, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2370'},
        {'fromNode': '7408.0', 'toNode': '7071.0', 'Resistance': 0.03482, 'Reactance': 0.08511, 'Conductance': 0, 'Susceptance': 0.01114, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2371'},
        {'fromNode': '7072.0', 'toNode': '7084.0', 'Resistance': 0.00344, 'Reactance': 0.02395, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2372'},
        {'fromNode': '7072.0', 'toNode': '7084.0', 'Resistance': 0.00344, 'Reactance': 0.02395, 'Conductance': 0, 'Susceptance': 0.00391, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2373'},
        {'fromNode': '7074.0', 'toNode': '7073.0', 'Resistance': 0.00062, 'Reactance': 0.03776, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 185.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2374'},
        {'fromNode': '7171.0', 'toNode': '7073.0', 'Resistance': 0.0022, 'Reactance': 0.01575, 'Conductance': 0, 'Susceptance': 0.02871, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2375'},
        {'fromNode': '7205.0', 'toNode': '7073.0', 'Resistance': 0.00402, 'Reactance': 0.03929, 'Conductance': 0, 'Susceptance': 0.07559, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2376'},
        {'fromNode': '7405.0', 'toNode': '7074.0', 'Resistance': 0.00499, 'Reactance': 0.02067, 'Conductance': 0, 'Susceptance': 0.00351, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2377'},
        {'fromNode': '7075.0', 'toNode': '7387.0', 'Resistance': 0.01167, 'Reactance': 0.07976, 'Conductance': 0, 'Susceptance': 0.01357, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2378'},
        {'fromNode': '7075.0', 'toNode': '7387.0', 'Resistance': 0.01167, 'Reactance': 0.07976, 'Conductance': 0, 'Susceptance': 0.01357, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2379'},
        {'fromNode': '7075.0', 'toNode': '7398.0', 'Resistance': 0.00592, 'Reactance': 0.03823, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2380'},
        {'fromNode': '7075.0', 'toNode': '7398.0', 'Resistance': 0.00592, 'Reactance': 0.03823, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2381'},
        {'fromNode': '7077.0', 'toNode': '7076.0', 'Resistance': 7e-05, 'Reactance': 0.00459, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1233.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2382'},
        {'fromNode': '7079.0', 'toNode': '7076.0', 'Resistance': 0.00024, 'Reactance': 0.01675, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 855.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2383'},
        {'fromNode': '7080.0', 'toNode': '7076.0', 'Resistance': 0.00037, 'Reactance': 0.02147, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 857.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2384'},
        {'fromNode': '7081.0', 'toNode': '7076.0', 'Resistance': 0.0003, 'Reactance': 0.02166, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 863.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2385'},
        {'fromNode': '7082.0', 'toNode': '7076.0', 'Resistance': 0.00028, 'Reactance': 0.01719, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 847.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2386'},
        {'fromNode': '7204.0', 'toNode': '7076.0', 'Resistance': 0.00014, 'Reactance': 0.00172, 'Conductance': 0, 'Susceptance': 0.1143, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2387'},
        {'fromNode': '7076.0', 'toNode': '7422.0', 'Resistance': 0.00061, 'Reactance': 0.00741, 'Conductance': 0, 'Susceptance': 0.49226, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2388'},
        {'fromNode': '7078.0', 'toNode': '7077.0', 'Resistance': 0.0011, 'Reactance': 0.04562, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 220.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2389'},
        {'fromNode': '7077.0', 'toNode': '7282.0', 'Resistance': 0.00125, 'Reactance': 0.00896, 'Conductance': 0, 'Susceptance': 0.01633, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2390'},
        {'fromNode': '7206.0', 'toNode': '7078.0', 'Resistance': 0.00828, 'Reactance': 0.03692, 'Conductance': 0, 'Susceptance': 0.00541, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2391'},
        {'fromNode': '7078.0', 'toNode': '7279.0', 'Resistance': 0.00273, 'Reactance': 0.01863, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2392'},
        {'fromNode': '7078.0', 'toNode': '7279.0', 'Resistance': 0.00273, 'Reactance': 0.01863, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2393'},
        {'fromNode': '7084.0', 'toNode': '7083.0', 'Resistance': 0.00053, 'Reactance': 0.03003, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 355.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2394'},
        {'fromNode': '7084.0', 'toNode': '7083.0', 'Resistance': 0.00053, 'Reactance': 0.03003, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 355.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2395'},
        {'fromNode': '7083.0', 'toNode': '7105.0', 'Resistance': 0.00233, 'Reactance': 0.0228, 'Conductance': 0, 'Susceptance': 0.04386, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2396'},
        {'fromNode': '7083.0', 'toNode': '7171.0', 'Resistance': 0.00193, 'Reactance': 0.01885, 'Conductance': 0, 'Susceptance': 0.03628, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2397'},
        {'fromNode': '7083.0', 'toNode': '7184.0', 'Resistance': 0.00606, 'Reactance': 0.04326, 'Conductance': 0, 'Susceptance': 0.07887, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2398'},
        {'fromNode': '7083.0', 'toNode': '7264.0', 'Resistance': 0.0013, 'Reactance': 0.01272, 'Conductance': 0, 'Susceptance': 0.02448, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2399'},
        {'fromNode': '7423.0', 'toNode': '7083.0', 'Resistance': 0.00173, 'Reactance': 0.01235, 'Conductance': 0, 'Susceptance': 0.02252, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2400'},
        {'fromNode': '7085.0', 'toNode': '7244.0', 'Resistance': 0.01057, 'Reactance': 0.07367, 'Conductance': 0, 'Susceptance': 0.01204, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2401'},
        {'fromNode': '7085.0', 'toNode': '7244.0', 'Resistance': 0.01057, 'Reactance': 0.07367, 'Conductance': 0, 'Susceptance': 0.01204, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2402'},
        {'fromNode': '7412.0', 'toNode': '7085.0', 'Resistance': 0.00305, 'Reactance': 0.02125, 'Conductance': 0, 'Susceptance': 0.00347, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2403'},
        {'fromNode': '7412.0', 'toNode': '7085.0', 'Resistance': 0.00305, 'Reactance': 0.02125, 'Conductance': 0, 'Susceptance': 0.00347, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2404'},
        {'fromNode': '7086.0', 'toNode': '7222.0', 'Resistance': 0.00549, 'Reactance': 0.02279, 'Conductance': 0, 'Susceptance': 0.00387, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2405'},
        {'fromNode': '7086.0', 'toNode': '7419.0', 'Resistance': 0.01003, 'Reactance': 0.0648, 'Conductance': 0, 'Susceptance': 0.01236, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2406'},
        {'fromNode': '7086.0', 'toNode': '7419.0', 'Resistance': 0.01003, 'Reactance': 0.0648, 'Conductance': 0, 'Susceptance': 0.01236, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2407'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2408'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2409'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2410'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2411'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2412'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2413'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2414'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2415'},
        {'fromNode': '7087.0', 'toNode': '7127.0', 'Resistance': 0.00301, 'Reactance': 0.01945, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2416'},
        {'fromNode': '7087.0', 'toNode': '7221.0', 'Resistance': 0.00412, 'Reactance': 0.02872, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2417'},
        {'fromNode': '7087.0', 'toNode': '7221.0', 'Resistance': 0.00412, 'Reactance': 0.02872, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2418'},
        {'fromNode': '7087.0', 'toNode': '7221.0', 'Resistance': 0.00412, 'Reactance': 0.02872, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2419'},
        {'fromNode': '7087.0', 'toNode': '7336.0', 'Resistance': 0.00167, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2420'},
        {'fromNode': '7087.0', 'toNode': '7336.0', 'Resistance': 0.00167, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2421'},
        {'fromNode': '7087.0', 'toNode': '7336.0', 'Resistance': 0.00167, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2422'},
        {'fromNode': '7087.0', 'toNode': '7336.0', 'Resistance': 0.00167, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2423'},
        {'fromNode': '7087.0', 'toNode': '7336.0', 'Resistance': 0.00167, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2424'},
        {'fromNode': '7088.0', 'toNode': '7116.0', 'Resistance': 0.01894, 'Reactance': 0.08439, 'Conductance': 0, 'Susceptance': 0.01238, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2425'},
        {'fromNode': '7088.0', 'toNode': '7147.0', 'Resistance': 0.01802, 'Reactance': 0.07475, 'Conductance': 0, 'Susceptance': 0.01268, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2426'},
        {'fromNode': '7237.0', 'toNode': '7088.0', 'Resistance': 0.01249, 'Reactance': 0.05466, 'Conductance': 0, 'Susceptance': 0.00832, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2427'},
        {'fromNode': '7089.0', 'toNode': '7388.0', 'Resistance': 0.00679, 'Reactance': 0.02818, 'Conductance': 0, 'Susceptance': 0.00478, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2428'},
        {'fromNode': '7089.0', 'toNode': '7427.0', 'Resistance': 0.0057, 'Reactance': 0.02492, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2429'},
        {'fromNode': '7090.0', 'toNode': '7421.0', 'Resistance': 0.00301, 'Reactance': 0.02055, 'Conductance': 0, 'Susceptance': 0.0035, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2430'},
        {'fromNode': '7090.0', 'toNode': '7421.0', 'Resistance': 0.00301, 'Reactance': 0.02055, 'Conductance': 0, 'Susceptance': 0.0035, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2431'},
        {'fromNode': '7091.0', 'toNode': '7226.0', 'Resistance': 0.0073, 'Reactance': 0.04386, 'Conductance': 0, 'Susceptance': 0.00595, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2432'},
        {'fromNode': '7091.0', 'toNode': '7259.0', 'Resistance': 0.00587, 'Reactance': 0.04091, 'Conductance': 0, 'Susceptance': 0.00669, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2433'},
        {'fromNode': '7091.0', 'toNode': '7259.0', 'Resistance': 0.00587, 'Reactance': 0.04091, 'Conductance': 0, 'Susceptance': 0.00669, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2434'},
        {'fromNode': '7091.0', 'toNode': '7418.0', 'Resistance': 0.02256, 'Reactance': 0.0542, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2435'},
        {'fromNode': '7092.0', 'toNode': '7216.0', 'Resistance': 0.00595, 'Reactance': 0.01454, 'Conductance': 0, 'Susceptance': 0.0019, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2436'},
        {'fromNode': '7092.0', 'toNode': '7250.0', 'Resistance': 0.00437, 'Reactance': 0.01813, 'Conductance': 0, 'Susceptance': 0.00308, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2437'},
        {'fromNode': '7092.0', 'toNode': '7258.0', 'Resistance': 0.00224, 'Reactance': 0.0098, 'Conductance': 0, 'Susceptance': 0.00149, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2438'},
        {'fromNode': '7092.0', 'toNode': '7330.0', 'Resistance': 0.00414, 'Reactance': 0.01846, 'Conductance': 0, 'Susceptance': 0.00271, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2439'},
        {'fromNode': '7092.0', 'toNode': '7384.0', 'Resistance': 0.00356, 'Reactance': 0.02036, 'Conductance': 0, 'Susceptance': 0.00306, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2440'},
        {'fromNode': '7221.0', 'toNode': '7093.0', 'Resistance': 0.00237, 'Reactance': 0.0153, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2441'},
        {'fromNode': '7221.0', 'toNode': '7093.0', 'Resistance': 0.00237, 'Reactance': 0.0153, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2442'},
        {'fromNode': '7093.0', 'toNode': '7225.0', 'Resistance': 0.0099, 'Reactance': 0.04331, 'Conductance': 0, 'Susceptance': 0.0066, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2443'},
        {'fromNode': '7241.0', 'toNode': '7093.0', 'Resistance': 0.00831, 'Reactance': 0.0203, 'Conductance': 0, 'Susceptance': 0.00266, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2444'},
        {'fromNode': '7093.0', 'toNode': '7258.0', 'Resistance': 0.01589, 'Reactance': 0.03818, 'Conductance': 0, 'Susceptance': 0.00518, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2445'},
        {'fromNode': '7094.0', 'toNode': '7276.0', 'Resistance': 0.00341, 'Reactance': 0.01414, 'Conductance': 0, 'Susceptance': 0.0024, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2446'},
        {'fromNode': '7094.0', 'toNode': '7388.0', 'Resistance': 0.01029, 'Reactance': 0.02471, 'Conductance': 0, 'Susceptance': 0.00335, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2447'},
        {'fromNode': '7096.0', 'toNode': '7095.0', 'Resistance': 0.00022, 'Reactance': 0.01326, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 846.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2448'},
        {'fromNode': '7098.0', 'toNode': '7095.0', 'Resistance': 0.00028, 'Reactance': 0.02986, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1600.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2449'},
        {'fromNode': '7099.0', 'toNode': '7095.0', 'Resistance': 0.00022, 'Reactance': 0.01103, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1600.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2450'},
        {'fromNode': '7304.0', 'toNode': '7095.0', 'Resistance': 0.00071, 'Reactance': 0.00855, 'Conductance': 0, 'Susceptance': 0.56788, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2451'},
        {'fromNode': '7097.0', 'toNode': '7096.0', 'Resistance': 0.00097, 'Reactance': 0.05173, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 200.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2452'},
        {'fromNode': '7261.0', 'toNode': '7096.0', 'Resistance': 0.0027, 'Reactance': 0.01928, 'Conductance': 0, 'Susceptance': 0.03515, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2453'},
        {'fromNode': '7097.0', 'toNode': '7153.0', 'Resistance': 0.03408, 'Reactance': 0.08331, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2454'},
        {'fromNode': '7100.0', 'toNode': '7103.0', 'Resistance': 0.00199, 'Reactance': 0.01384, 'Conductance': 0, 'Susceptance': 0.00226, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2455'},
        {'fromNode': '7100.0', 'toNode': '7103.0', 'Resistance': 0.00199, 'Reactance': 0.01384, 'Conductance': 0, 'Susceptance': 0.00226, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2456'},
        {'fromNode': '7100.0', 'toNode': '7144.0', 'Resistance': 0.00174, 'Reactance': 0.01212, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2457'},
        {'fromNode': '7100.0', 'toNode': '7144.0', 'Resistance': 0.00174, 'Reactance': 0.01212, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2458'},
        {'fromNode': '7100.0', 'toNode': '7144.0', 'Resistance': 0.00174, 'Reactance': 0.01212, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2459'},
        {'fromNode': '7100.0', 'toNode': '7144.0', 'Resistance': 0.00174, 'Reactance': 0.01212, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2460'},
        {'fromNode': '7100.0', 'toNode': '7222.0', 'Resistance': 0.00195, 'Reactance': 0.01331, 'Conductance': 0, 'Susceptance': 0.00226, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2461'},
        {'fromNode': '7100.0', 'toNode': '7222.0', 'Resistance': 0.00195, 'Reactance': 0.01331, 'Conductance': 0, 'Susceptance': 0.00226, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2462'},
        {'fromNode': '7101.0', 'toNode': '7183.0', 'Resistance': 0.00837, 'Reactance': 0.03729, 'Conductance': 0, 'Susceptance': 0.00547, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2463'},
        {'fromNode': '7101.0', 'toNode': '7386.0', 'Resistance': 0.00972, 'Reactance': 0.04254, 'Conductance': 0, 'Susceptance': 0.00648, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2464'},
        {'fromNode': '7102.0', 'toNode': '7177.0', 'Resistance': 0.0037, 'Reactance': 0.0239, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2465'},
        {'fromNode': '7102.0', 'toNode': '7177.0', 'Resistance': 0.0037, 'Reactance': 0.0239, 'Conductance': 0, 'Susceptance': 0.00456, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2466'},
        {'fromNode': '7290.0', 'toNode': '7102.0', 'Resistance': 0.00551, 'Reactance': 0.02285, 'Conductance': 0, 'Susceptance': 0.00388, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2467'},
        {'fromNode': '7103.0', 'toNode': '7255.0', 'Resistance': 0.00598, 'Reactance': 0.02614, 'Conductance': 0, 'Susceptance': 0.00398, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2468'},
        {'fromNode': '7103.0', 'toNode': '7381.0', 'Resistance': 0.0191, 'Reactance': 0.04669, 'Conductance': 0, 'Susceptance': 0.00611, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2469'},
        {'fromNode': '7105.0', 'toNode': '7104.0', 'Resistance': 0.00012, 'Reactance': 0.01194, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1312.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2470'},
        {'fromNode': '7107.0', 'toNode': '7104.0', 'Resistance': 0.00016, 'Reactance': 0.01012, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 592.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2471'},
        {'fromNode': '7108.0', 'toNode': '7104.0', 'Resistance': 0.00066, 'Reactance': 0.0454, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 335.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2472'},
        {'fromNode': '7109.0', 'toNode': '7104.0', 'Resistance': 0.00038, 'Reactance': 0.03767, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 339.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2473'},
        {'fromNode': '7110.0', 'toNode': '7104.0', 'Resistance': 0.00018, 'Reactance': 0.01895, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 731.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2474'},
        {'fromNode': '7111.0', 'toNode': '7104.0', 'Resistance': 0.0002, 'Reactance': 0.01942, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 743.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2475'},
        {'fromNode': '7112.0', 'toNode': '7104.0', 'Resistance': 0.00016, 'Reactance': 0.01175, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 740.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2476'},
        {'fromNode': '7113.0', 'toNode': '7104.0', 'Resistance': 0.00031, 'Reactance': 0.0227, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 748.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2477'},
        {'fromNode': '7114.0', 'toNode': '7104.0', 'Resistance': 0.00018, 'Reactance': 0.01098, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 752.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2478'},
        {'fromNode': '7115.0', 'toNode': '7104.0', 'Resistance': 0.00016, 'Reactance': 0.01098, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 744.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2479'},
        {'fromNode': '7130.0', 'toNode': '7104.0', 'Resistance': 5e-05, 'Reactance': 0.00102, 'Conductance': 0, 'Susceptance': 0.07092, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2480'},
        {'fromNode': '7106.0', 'toNode': '7105.0', 'Resistance': 0.00051, 'Reactance': 0.03345, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 354.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2481'},
        {'fromNode': '7106.0', 'toNode': '7105.0', 'Resistance': 0.00051, 'Reactance': 0.03345, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 354.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2482'},
        {'fromNode': '7105.0', 'toNode': '7187.0', 'Resistance': 0.00243, 'Reactance': 0.02373, 'Conductance': 0, 'Susceptance': 0.04566, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2483'},
        {'fromNode': '7105.0', 'toNode': '7205.0', 'Resistance': 0.00202, 'Reactance': 0.01979, 'Conductance': 0, 'Susceptance': 0.03807, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2484'},
        {'fromNode': '7105.0', 'toNode': '7332.0', 'Resistance': 0.00074, 'Reactance': 0.00725, 'Conductance': 0, 'Susceptance': 0.01395, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2485'},
        {'fromNode': '7116.0', 'toNode': '7340.0', 'Resistance': 0.01409, 'Reactance': 0.0616, 'Conductance': 0, 'Susceptance': 0.00938, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2486'},
        {'fromNode': '7116.0', 'toNode': '7364.0', 'Resistance': 0.02897, 'Reactance': 0.06959, 'Conductance': 0, 'Susceptance': 0.00944, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2487'},
        {'fromNode': '7242.0', 'toNode': '7117.0', 'Resistance': 0.00214, 'Reactance': 0.01492, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2488'},
        {'fromNode': '7242.0', 'toNode': '7117.0', 'Resistance': 0.00214, 'Reactance': 0.01492, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2489'},
        {'fromNode': '7117.0', 'toNode': '7292.0', 'Resistance': 0.00208, 'Reactance': 0.01424, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2490'},
        {'fromNode': '7117.0', 'toNode': '7292.0', 'Resistance': 0.00208, 'Reactance': 0.01424, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2491'},
        {'fromNode': '7117.0', 'toNode': '7395.0', 'Resistance': 0.00404, 'Reactance': 0.02761, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2492'},
        {'fromNode': '7117.0', 'toNode': '7395.0', 'Resistance': 0.00404, 'Reactance': 0.02761, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2493'},
        {'fromNode': '7177.0', 'toNode': '7118.0', 'Resistance': 0.00272, 'Reactance': 0.01856, 'Conductance': 0, 'Susceptance': 0.00316, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2494'},
        {'fromNode': '7177.0', 'toNode': '7118.0', 'Resistance': 0.00272, 'Reactance': 0.01856, 'Conductance': 0, 'Susceptance': 0.00316, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2495'},
        {'fromNode': '7118.0', 'toNode': '7404.0', 'Resistance': 0.0055, 'Reactance': 0.03759, 'Conductance': 0, 'Susceptance': 0.0064, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2496'},
        {'fromNode': '7144.0', 'toNode': '7119.0', 'Resistance': 0.00176, 'Reactance': 0.01229, 'Conductance': 0, 'Susceptance': 0.00201, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2497'},
        {'fromNode': '7144.0', 'toNode': '7119.0', 'Resistance': 0.00176, 'Reactance': 0.01229, 'Conductance': 0, 'Susceptance': 0.00201, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2498'},
        {'fromNode': '7119.0', 'toNode': '7259.0', 'Resistance': 0.01192, 'Reactance': 0.02865, 'Conductance': 0, 'Susceptance': 0.00389, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2499'},
        {'fromNode': '7119.0', 'toNode': '7318.0', 'Resistance': 0.00442, 'Reactance': 0.01971, 'Conductance': 0, 'Susceptance': 0.00289, 'MVA_rating': 166.88, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2500'},
        {'fromNode': '7121.0', 'toNode': '7120.0', 'Resistance': 0.0094, 'Reactance': 0.18529, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 41.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2501'},
        {'fromNode': '7122.0', 'toNode': '7120.0', 'Resistance': 0.00662, 'Reactance': 0.19298, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 46.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2502'},
        {'fromNode': '7120.0', 'toNode': '7248.0', 'Resistance': 0.00645, 'Reactance': 0.03687, 'Conductance': 0, 'Susceptance': 0.00554, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2503'},
        {'fromNode': '7120.0', 'toNode': '7396.0', 'Resistance': 0.00289, 'Reactance': 0.02016, 'Conductance': 0, 'Susceptance': 0.0033, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2504'},
        {'fromNode': '7120.0', 'toNode': '7396.0', 'Resistance': 0.00289, 'Reactance': 0.02016, 'Conductance': 0, 'Susceptance': 0.0033, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2505'},
        {'fromNode': '7427.0', 'toNode': '7120.0', 'Resistance': 0.00159, 'Reactance': 0.01024, 'Conductance': 0, 'Susceptance': 0.00195, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2506'},
        {'fromNode': '7427.0', 'toNode': '7120.0', 'Resistance': 0.00159, 'Reactance': 0.01024, 'Conductance': 0, 'Susceptance': 0.00195, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2507'},
        {'fromNode': '7123.0', 'toNode': '7153.0', 'Resistance': 0.0626, 'Reactance': 0.15302, 'Conductance': 0, 'Susceptance': 0.02004, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2508'},
        {'fromNode': '7124.0', 'toNode': '7180.0', 'Resistance': 0.0069, 'Reactance': 0.01687, 'Conductance': 0, 'Susceptance': 0.00221, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2509'},
        {'fromNode': '7126.0', 'toNode': '7125.0', 'Resistance': 0.00011, 'Reactance': 0.01326, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1358.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2510'},
        {'fromNode': '7126.0', 'toNode': '7125.0', 'Resistance': 0.00011, 'Reactance': 0.01326, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1358.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2511'},
        {'fromNode': '7125.0', 'toNode': '7331.0', 'Resistance': 0.00057, 'Reactance': 0.00696, 'Conductance': 0, 'Susceptance': 0.46259, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2512'},
        {'fromNode': '7346.0', 'toNode': '7125.0', 'Resistance': 6e-05, 'Reactance': 0.00128, 'Conductance': 0, 'Susceptance': 0.11073, 'MVA_rating': 4352.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2513'},
        {'fromNode': '7127.0', 'toNode': '7126.0', 'Resistance': 0.00019, 'Reactance': 0.01178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 512.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2514'},
        {'fromNode': '7127.0', 'toNode': '7126.0', 'Resistance': 0.00019, 'Reactance': 0.01178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 512.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2515'},
        {'fromNode': '7127.0', 'toNode': '7126.0', 'Resistance': 0.00019, 'Reactance': 0.01178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 512.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2516'},
        {'fromNode': '7127.0', 'toNode': '7126.0', 'Resistance': 0.00019, 'Reactance': 0.01178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 512.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2517'},
        {'fromNode': '7126.0', 'toNode': '7305.0', 'Resistance': 0.00691, 'Reactance': 0.04934, 'Conductance': 0, 'Susceptance': 0.08995, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2518'},
        {'fromNode': '7126.0', 'toNode': '7332.0', 'Resistance': 0.00592, 'Reactance': 0.04228, 'Conductance': 0, 'Susceptance': 0.07708, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2519'},
        {'fromNode': '7126.0', 'toNode': '7423.0', 'Resistance': 0.00612, 'Reactance': 0.04375, 'Conductance': 0, 'Susceptance': 0.07976, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2520'},
        {'fromNode': '7391.0', 'toNode': '7128.0', 'Resistance': 0.00567, 'Reactance': 0.03872, 'Conductance': 0, 'Susceptance': 0.00659, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2521'},
        {'fromNode': '7277.0', 'toNode': '7129.0', 'Resistance': 0.0122, 'Reactance': 0.07328, 'Conductance': 0, 'Susceptance': 0.00994, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2522'},
        {'fromNode': '7303.0', 'toNode': '7129.0', 'Resistance': 0.02396, 'Reactance': 0.05755, 'Conductance': 0, 'Susceptance': 0.00781, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2523'},
        {'fromNode': '7131.0', 'toNode': '7130.0', 'Resistance': 0.0003, 'Reactance': 0.01434, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1373.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2524'},
        {'fromNode': '7133.0', 'toNode': '7130.0', 'Resistance': 0.00046, 'Reactance': 0.05358, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 295.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2525'},
        {'fromNode': '7134.0', 'toNode': '7130.0', 'Resistance': 0.0002, 'Reactance': 0.01347, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 776.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2526'},
        {'fromNode': '7135.0', 'toNode': '7130.0', 'Resistance': 0.00024, 'Reactance': 0.01692, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 767.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2527'},
        {'fromNode': '7136.0', 'toNode': '7130.0', 'Resistance': 0.00025, 'Reactance': 0.0186, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 768.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2528'},
        {'fromNode': '7137.0', 'toNode': '7130.0', 'Resistance': 0.00043, 'Reactance': 0.01928, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 781.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2529'},
        {'fromNode': '7138.0', 'toNode': '7130.0', 'Resistance': 0.0001, 'Reactance': 0.00697, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1053.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2530'},
        {'fromNode': '7139.0', 'toNode': '7130.0', 'Resistance': 0.001, 'Reactance': 0.0466, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 357.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2531'},
        {'fromNode': '7140.0', 'toNode': '7130.0', 'Resistance': 0.00043, 'Reactance': 0.03937, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 360.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2532'},
        {'fromNode': '7141.0', 'toNode': '7130.0', 'Resistance': 0.00037, 'Reactance': 0.02584, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 368.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2533'},
        {'fromNode': '7142.0', 'toNode': '7130.0', 'Resistance': 0.00045, 'Reactance': 0.03413, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 363.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2534'},
        {'fromNode': '7263.0', 'toNode': '7130.0', 'Resistance': 0.00015, 'Reactance': 0.00182, 'Conductance': 0, 'Susceptance': 0.1209, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2535'},
        {'fromNode': '7132.0', 'toNode': '7131.0', 'Resistance': 0.00109, 'Reactance': 0.03352, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 446.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2536'},
        {'fromNode': '7132.0', 'toNode': '7131.0', 'Resistance': 0.00109, 'Reactance': 0.03352, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 446.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2537'},
        {'fromNode': '7131.0', 'toNode': '7160.0', 'Resistance': 0.00484, 'Reactance': 0.04731, 'Conductance': 0, 'Susceptance': 0.09103, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2538'},
        {'fromNode': '7131.0', 'toNode': '7278.0', 'Resistance': 0.0023, 'Reactance': 0.01644, 'Conductance': 0, 'Susceptance': 0.02997, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2539'},
        {'fromNode': '7131.0', 'toNode': '7401.0', 'Resistance': 0.00248, 'Reactance': 0.01771, 'Conductance': 0, 'Susceptance': 0.03228, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2540'},
        {'fromNode': '7132.0', 'toNode': '7283.0', 'Resistance': 0.00306, 'Reactance': 0.0198, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2541'},
        {'fromNode': '7419.0', 'toNode': '7132.0', 'Resistance': 0.00182, 'Reactance': 0.01246, 'Conductance': 0, 'Susceptance': 0.00212, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2542'},
        {'fromNode': '7419.0', 'toNode': '7132.0', 'Resistance': 0.00182, 'Reactance': 0.01246, 'Conductance': 0, 'Susceptance': 0.00212, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2543'},
        {'fromNode': '7419.0', 'toNode': '7132.0', 'Resistance': 0.00182, 'Reactance': 0.01246, 'Conductance': 0, 'Susceptance': 0.00212, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2544'},
        {'fromNode': '7143.0', 'toNode': '7173.0', 'Resistance': 0.00756, 'Reactance': 0.0432, 'Conductance': 0, 'Susceptance': 0.00649, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2545'},
        {'fromNode': '7234.0', 'toNode': '7143.0', 'Resistance': 0.00693, 'Reactance': 0.04159, 'Conductance': 0, 'Susceptance': 0.00564, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2546'},
        {'fromNode': '7143.0', 'toNode': '7238.0', 'Resistance': 0.03371, 'Reactance': 0.14747, 'Conductance': 0, 'Susceptance': 0.02246, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2547'},
        {'fromNode': '7300.0', 'toNode': '7143.0', 'Resistance': 0.00916, 'Reactance': 0.05231, 'Conductance': 0, 'Susceptance': 0.00786, 'MVA_rating': 245.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2548'},
        {'fromNode': '7144.0', 'toNode': '7272.0', 'Resistance': 0.00479, 'Reactance': 0.03337, 'Conductance': 0, 'Susceptance': 0.00546, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2549'},
        {'fromNode': '7144.0', 'toNode': '7272.0', 'Resistance': 0.00479, 'Reactance': 0.03337, 'Conductance': 0, 'Susceptance': 0.00546, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2550'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2551'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2552'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2553'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2554'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2555'},
        {'fromNode': '7333.0', 'toNode': '7144.0', 'Resistance': 0.00251, 'Reactance': 0.01716, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2556'},
        {'fromNode': '7146.0', 'toNode': '7145.0', 'Resistance': 0.00456, 'Reactance': 0.21285, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 25.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2557'},
        {'fromNode': '7145.0', 'toNode': '7234.0', 'Resistance': 0.0104, 'Reactance': 0.06243, 'Conductance': 0, 'Susceptance': 0.00847, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2558'},
        {'fromNode': '7145.0', 'toNode': '7243.0', 'Resistance': 0.01348, 'Reactance': 0.05591, 'Conductance': 0, 'Susceptance': 0.00949, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2559'},
        {'fromNode': '7381.0', 'toNode': '7145.0', 'Resistance': 0.00645, 'Reactance': 0.03876, 'Conductance': 0, 'Susceptance': 0.00526, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2560'},
        {'fromNode': '7145.0', 'toNode': '7404.0', 'Resistance': 0.01139, 'Reactance': 0.06841, 'Conductance': 0, 'Susceptance': 0.00928, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2561'},
        {'fromNode': '7336.0', 'toNode': '7147.0', 'Resistance': 0.00171, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2562'},
        {'fromNode': '7336.0', 'toNode': '7147.0', 'Resistance': 0.00171, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2563'},
        {'fromNode': '7336.0', 'toNode': '7147.0', 'Resistance': 0.00171, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2564'},
        {'fromNode': '7336.0', 'toNode': '7147.0', 'Resistance': 0.00171, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2565'},
        {'fromNode': '7386.0', 'toNode': '7147.0', 'Resistance': 0.00225, 'Reactance': 0.01539, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2566'},
        {'fromNode': '7386.0', 'toNode': '7147.0', 'Resistance': 0.00225, 'Reactance': 0.01539, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2567'},
        {'fromNode': '7178.0', 'toNode': '7148.0', 'Resistance': 0.00225, 'Reactance': 0.01565, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2568'},
        {'fromNode': '7178.0', 'toNode': '7148.0', 'Resistance': 0.00225, 'Reactance': 0.01565, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2569'},
        {'fromNode': '7148.0', 'toNode': '7245.0', 'Resistance': 0.00801, 'Reactance': 0.04577, 'Conductance': 0, 'Susceptance': 0.00688, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2570'},
        {'fromNode': '7148.0', 'toNode': '7298.0', 'Resistance': 0.00778, 'Reactance': 0.03402, 'Conductance': 0, 'Susceptance': 0.00518, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2571'},
        {'fromNode': '7148.0', 'toNode': '7365.0', 'Resistance': 0.0043, 'Reactance': 0.02938, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2572'},
        {'fromNode': '7148.0', 'toNode': '7365.0', 'Resistance': 0.0043, 'Reactance': 0.02938, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2573'},
        {'fromNode': '7149.0', 'toNode': '7408.0', 'Resistance': 0.0135, 'Reactance': 0.0771, 'Conductance': 0, 'Susceptance': 0.01159, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2574'},
        {'fromNode': '7151.0', 'toNode': '7150.0', 'Resistance': 0.00105, 'Reactance': 0.04281, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 151.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2575'},
        {'fromNode': '7150.0', 'toNode': '7390.0', 'Resistance': 0.00103, 'Reactance': 0.01076, 'Conductance': 0, 'Susceptance': 0.03665, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2576'},
        {'fromNode': '7150.0', 'toNode': '7390.0', 'Resistance': 0.00103, 'Reactance': 0.01076, 'Conductance': 0, 'Susceptance': 0.03665, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2577'},
        {'fromNode': '7150.0', 'toNode': '7420.0', 'Resistance': 0.0052, 'Reactance': 0.03715, 'Conductance': 0, 'Susceptance': 0.06773, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2578'},
        {'fromNode': '8004.0', 'toNode': '7150.0', 'Resistance': 0.01032, 'Reactance': 0.1009, 'Conductance': 0, 'Susceptance': 0.19415, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2579'},
        {'fromNode': '7151.0', 'toNode': '7254.0', 'Resistance': 0.00477, 'Reactance': 0.03078, 'Conductance': 0, 'Susceptance': 0.00587, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2580'},
        {'fromNode': '7330.0', 'toNode': '7152.0', 'Resistance': 0.00193, 'Reactance': 0.01249, 'Conductance': 0, 'Susceptance': 0.00238, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2581'},
        {'fromNode': '7330.0', 'toNode': '7152.0', 'Resistance': 0.00193, 'Reactance': 0.01249, 'Conductance': 0, 'Susceptance': 0.00238, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2582'},
        {'fromNode': '7152.0', 'toNode': '7399.0', 'Resistance': 0.00147, 'Reactance': 0.0095, 'Conductance': 0, 'Susceptance': 0.00181, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2583'},
        {'fromNode': '7152.0', 'toNode': '7399.0', 'Resistance': 0.00147, 'Reactance': 0.0095, 'Conductance': 0, 'Susceptance': 0.00181, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2584'},
        {'fromNode': '7154.0', 'toNode': '7157.0', 'Resistance': 0.00654, 'Reactance': 0.04553, 'Conductance': 0, 'Susceptance': 0.00744, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2585'},
        {'fromNode': '7154.0', 'toNode': '7157.0', 'Resistance': 0.00654, 'Reactance': 0.04553, 'Conductance': 0, 'Susceptance': 0.00744, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2586'},
        {'fromNode': '7300.0', 'toNode': '7154.0', 'Resistance': 0.00482, 'Reactance': 0.02753, 'Conductance': 0, 'Susceptance': 0.00414, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2587'},
        {'fromNode': '7154.0', 'toNode': '7411.0', 'Resistance': 0.00439, 'Reactance': 0.02634, 'Conductance': 0, 'Susceptance': 0.00357, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2588'},
        {'fromNode': '7155.0', 'toNode': '7258.0', 'Resistance': 0.00356, 'Reactance': 0.02032, 'Conductance': 0, 'Susceptance': 0.00305, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2589'},
        {'fromNode': '7155.0', 'toNode': '7316.0', 'Resistance': 0.00716, 'Reactance': 0.03193, 'Conductance': 0, 'Susceptance': 0.00468, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2590'},
        {'fromNode': '7157.0', 'toNode': '7156.0', 'Resistance': 0.00036, 'Reactance': 0.02246, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 424.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2591'},
        {'fromNode': '7157.0', 'toNode': '7156.0', 'Resistance': 0.00036, 'Reactance': 0.02246, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 424.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2592'},
        {'fromNode': '7157.0', 'toNode': '7156.0', 'Resistance': 0.00036, 'Reactance': 0.02246, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 424.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2593'},
        {'fromNode': '7176.0', 'toNode': '7156.0', 'Resistance': 0.00094, 'Reactance': 0.00975, 'Conductance': 0, 'Susceptance': 0.03322, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2594'},
        {'fromNode': '7156.0', 'toNode': '7367.0', 'Resistance': 0.00253, 'Reactance': 0.0263, 'Conductance': 0, 'Susceptance': 0.0896, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2595'},
        {'fromNode': '7156.0', 'toNode': '7367.0', 'Resistance': 0.00253, 'Reactance': 0.0263, 'Conductance': 0, 'Susceptance': 0.0896, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2596'},
        {'fromNode': '7157.0', 'toNode': '7295.0', 'Resistance': 0.00326, 'Reactance': 0.02272, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2597'},
        {'fromNode': '7157.0', 'toNode': '7295.0', 'Resistance': 0.00326, 'Reactance': 0.02272, 'Conductance': 0, 'Susceptance': 0.00371, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2598'},
        {'fromNode': '7173.0', 'toNode': '7158.0', 'Resistance': 0.00459, 'Reactance': 0.01903, 'Conductance': 0, 'Susceptance': 0.00323, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2599'},
        {'fromNode': '7158.0', 'toNode': '7291.0', 'Resistance': 0.00974, 'Reactance': 0.0426, 'Conductance': 0, 'Susceptance': 0.00649, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2600'},
        {'fromNode': '7158.0', 'toNode': '7394.0', 'Resistance': 0.00264, 'Reactance': 0.01704, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2601'},
        {'fromNode': '7158.0', 'toNode': '7394.0', 'Resistance': 0.00264, 'Reactance': 0.01704, 'Conductance': 0, 'Susceptance': 0.00325, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2602'},
        {'fromNode': '7160.0', 'toNode': '7159.0', 'Resistance': 0.0001, 'Reactance': 0.00504, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1310.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2603'},
        {'fromNode': '7162.0', 'toNode': '7159.0', 'Resistance': 0.00022, 'Reactance': 0.01603, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 491.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2604'},
        {'fromNode': '7163.0', 'toNode': '7159.0', 'Resistance': 0.00034, 'Reactance': 0.02489, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 498.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2605'},
        {'fromNode': '7164.0', 'toNode': '7159.0', 'Resistance': 0.00017, 'Reactance': 0.01804, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 498.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2606'},
        {'fromNode': '7165.0', 'toNode': '7159.0', 'Resistance': 0.00017, 'Reactance': 0.01624, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 512.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2607'},
        {'fromNode': '7166.0', 'toNode': '7159.0', 'Resistance': 0.00043, 'Reactance': 0.0291, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 507.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2608'},
        {'fromNode': '7167.0', 'toNode': '7159.0', 'Resistance': 0.0005, 'Reactance': 0.03535, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2609'},
        {'fromNode': '7168.0', 'toNode': '7159.0', 'Resistance': 0.00025, 'Reactance': 0.02347, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 418.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2610'},
        {'fromNode': '7169.0', 'toNode': '7159.0', 'Resistance': 0.00016, 'Reactance': 0.01916, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 416.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2611'},
        {'fromNode': '7170.0', 'toNode': '7159.0', 'Resistance': 0.00016, 'Reactance': 0.01242, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1042.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2612'},
        {'fromNode': '7159.0', 'toNode': '7175.0', 'Resistance': 8e-05, 'Reactance': 0.00168, 'Conductance': 0, 'Susceptance': 0.11713, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2613'},
        {'fromNode': '7199.0', 'toNode': '7159.0', 'Resistance': 0.00019, 'Reactance': 0.0023, 'Conductance': 0, 'Susceptance': 0.15283, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2614'},
        {'fromNode': '7159.0', 'toNode': '7227.0', 'Resistance': 0.00013, 'Reactance': 0.00271, 'Conductance': 0, 'Susceptance': 0.18901, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2615'},
        {'fromNode': '7161.0', 'toNode': '7160.0', 'Resistance': 0.00037, 'Reactance': 0.02002, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 418.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2616'},
        {'fromNode': '7161.0', 'toNode': '7160.0', 'Resistance': 0.00037, 'Reactance': 0.02002, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 418.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2617'},
        {'fromNode': '7161.0', 'toNode': '7160.0', 'Resistance': 0.00037, 'Reactance': 0.02002, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 418.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2618'},
        {'fromNode': '7187.0', 'toNode': '7160.0', 'Resistance': 0.00262, 'Reactance': 0.02562, 'Conductance': 0, 'Susceptance': 0.04929, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2619'},
        {'fromNode': '7161.0', 'toNode': '7292.0', 'Resistance': 0.00143, 'Reactance': 0.00999, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 331.65, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2620'},
        {'fromNode': '7161.0', 'toNode': '7292.0', 'Resistance': 0.00143, 'Reactance': 0.00999, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2621'},
        {'fromNode': '7161.0', 'toNode': '7292.0', 'Resistance': 0.00143, 'Reactance': 0.00999, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2622'},
        {'fromNode': '7161.0', 'toNode': '7292.0', 'Resistance': 0.00143, 'Reactance': 0.00999, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2623'},
        {'fromNode': '7172.0', 'toNode': '7171.0', 'Resistance': 0.00027, 'Reactance': 0.01742, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 352.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2624'},
        {'fromNode': '7171.0', 'toNode': '7252.0', 'Resistance': 0.00752, 'Reactance': 0.05372, 'Conductance': 0, 'Susceptance': 0.09795, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2625'},
        {'fromNode': '7232.0', 'toNode': '7172.0', 'Resistance': 0.00301, 'Reactance': 0.02095, 'Conductance': 0, 'Susceptance': 0.00342, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2626'},
        {'fromNode': '7232.0', 'toNode': '7172.0', 'Resistance': 0.00301, 'Reactance': 0.02095, 'Conductance': 0, 'Susceptance': 0.00342, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2627'},
        {'fromNode': '7173.0', 'toNode': '7387.0', 'Resistance': 0.00415, 'Reactance': 0.02368, 'Conductance': 0, 'Susceptance': 0.00356, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2628'},
        {'fromNode': '7174.0', 'toNode': '7231.0', 'Resistance': 0.00209, 'Reactance': 0.01426, 'Conductance': 0, 'Susceptance': 0.00243, 'MVA_rating': 331.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2629'},
        {'fromNode': '7174.0', 'toNode': '7231.0', 'Resistance': 0.00209, 'Reactance': 0.01426, 'Conductance': 0, 'Susceptance': 0.00243, 'MVA_rating': 331.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2630'},
        {'fromNode': '7174.0', 'toNode': '7257.0', 'Resistance': 0.00838, 'Reactance': 0.05031, 'Conductance': 0, 'Susceptance': 0.00683, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2631'},
        {'fromNode': '7301.0', 'toNode': '7174.0', 'Resistance': 0.00178, 'Reactance': 0.01152, 'Conductance': 0, 'Susceptance': 0.0022, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2632'},
        {'fromNode': '7176.0', 'toNode': '7175.0', 'Resistance': 0.00019, 'Reactance': 0.01242, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1292.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2633'},
        {'fromNode': '7176.0', 'toNode': '7175.0', 'Resistance': 0.00019, 'Reactance': 0.01242, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1292.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2634'},
        {'fromNode': '7175.0', 'toNode': '7389.0', 'Resistance': 0.00056, 'Reactance': 0.00684, 'Conductance': 0, 'Susceptance': 0.45463, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2635'},
        {'fromNode': '7177.0', 'toNode': '7176.0', 'Resistance': 0.00105, 'Reactance': 0.0449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 439.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2636'},
        {'fromNode': '7177.0', 'toNode': '7176.0', 'Resistance': 0.00105, 'Reactance': 0.0449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 439.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2637'},
        {'fromNode': '7177.0', 'toNode': '7176.0', 'Resistance': 0.00105, 'Reactance': 0.0449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 439.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2638'},
        {'fromNode': '7176.0', 'toNode': '7200.0', 'Resistance': 0.0019, 'Reactance': 0.01359, 'Conductance': 0, 'Susceptance': 0.02477, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2639'},
        {'fromNode': '7176.0', 'toNode': '7409.0', 'Resistance': 0.00513, 'Reactance': 0.03666, 'Conductance': 0, 'Susceptance': 0.06684, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2640'},
        {'fromNode': '7319.0', 'toNode': '7177.0', 'Resistance': 0.00247, 'Reactance': 0.01686, 'Conductance': 0, 'Susceptance': 0.00287, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2641'},
        {'fromNode': '7319.0', 'toNode': '7177.0', 'Resistance': 0.00247, 'Reactance': 0.01686, 'Conductance': 0, 'Susceptance': 0.00287, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2642'},
        {'fromNode': '7295.0', 'toNode': '7178.0', 'Resistance': 0.00541, 'Reactance': 0.02409, 'Conductance': 0, 'Susceptance': 0.00353, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2643'},
        {'fromNode': '7178.0', 'toNode': '7404.0', 'Resistance': 0.00777, 'Reactance': 0.01898, 'Conductance': 0, 'Susceptance': 0.00249, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2644'},
        {'fromNode': '7179.0', 'toNode': '7244.0', 'Resistance': 0.00945, 'Reactance': 0.04213, 'Conductance': 0, 'Susceptance': 0.00618, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2645'},
        {'fromNode': '7179.0', 'toNode': '7257.0', 'Resistance': 0.01429, 'Reactance': 0.0593, 'Conductance': 0, 'Susceptance': 0.01006, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2646'},
        {'fromNode': '7179.0', 'toNode': '7336.0', 'Resistance': 0.00994, 'Reactance': 0.04121, 'Conductance': 0, 'Susceptance': 0.00699, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2647'},
        {'fromNode': '7216.0', 'toNode': '7180.0', 'Resistance': 0.00437, 'Reactance': 0.01069, 'Conductance': 0, 'Susceptance': 0.0014, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2648'},
        {'fromNode': '7180.0', 'toNode': '7298.0', 'Resistance': 0.00299, 'Reactance': 0.02086, 'Conductance': 0, 'Susceptance': 0.00341, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2649'},
        {'fromNode': '7180.0', 'toNode': '7298.0', 'Resistance': 0.00299, 'Reactance': 0.02086, 'Conductance': 0, 'Susceptance': 0.00341, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2650'},
        {'fromNode': '7388.0', 'toNode': '7180.0', 'Resistance': 0.00328, 'Reactance': 0.02121, 'Conductance': 0, 'Susceptance': 0.00405, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2651'},
        {'fromNode': '7388.0', 'toNode': '7180.0', 'Resistance': 0.00328, 'Reactance': 0.02121, 'Conductance': 0, 'Susceptance': 0.00405, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2652'},
        {'fromNode': '7180.0', 'toNode': '7399.0', 'Resistance': 0.00226, 'Reactance': 0.01457, 'Conductance': 0, 'Susceptance': 0.00278, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2653'},
        {'fromNode': '7180.0', 'toNode': '7399.0', 'Resistance': 0.00226, 'Reactance': 0.01457, 'Conductance': 0, 'Susceptance': 0.00278, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2654'},
        {'fromNode': '7242.0', 'toNode': '7181.0', 'Resistance': 0.00194, 'Reactance': 0.01355, 'Conductance': 0, 'Susceptance': 0.00221, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2655'},
        {'fromNode': '7385.0', 'toNode': '7181.0', 'Resistance': 0.01045, 'Reactance': 0.02553, 'Conductance': 0, 'Susceptance': 0.00334, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2656'},
        {'fromNode': '7182.0', 'toNode': '7237.0', 'Resistance': 0.01327, 'Reactance': 0.07579, 'Conductance': 0, 'Susceptance': 0.01139, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2657'},
        {'fromNode': '7183.0', 'toNode': '7417.0', 'Resistance': 0.00881, 'Reactance': 0.02153, 'Conductance': 0, 'Susceptance': 0.00282, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2658'},
        {'fromNode': '7185.0', 'toNode': '7184.0', 'Resistance': 0.00074, 'Reactance': 0.0321, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 210.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2659'},
        {'fromNode': '7185.0', 'toNode': '7184.0', 'Resistance': 0.00074, 'Reactance': 0.0321, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 210.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2660'},
        {'fromNode': '7185.0', 'toNode': '7184.0', 'Resistance': 0.00074, 'Reactance': 0.0321, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 210.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2661'},
        {'fromNode': '7347.0', 'toNode': '7184.0', 'Resistance': 0.00047, 'Reactance': 0.00492, 'Conductance': 0, 'Susceptance': 0.01676, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2662'},
        {'fromNode': '7347.0', 'toNode': '7184.0', 'Resistance': 0.00047, 'Reactance': 0.00492, 'Conductance': 0, 'Susceptance': 0.01676, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2663'},
        {'fromNode': '7347.0', 'toNode': '7184.0', 'Resistance': 0.00047, 'Reactance': 0.00492, 'Conductance': 0, 'Susceptance': 0.01676, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2664'},
        {'fromNode': '7383.0', 'toNode': '7185.0', 'Resistance': 0.00607, 'Reactance': 0.04227, 'Conductance': 0, 'Susceptance': 0.00691, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2665'},
        {'fromNode': '7383.0', 'toNode': '7185.0', 'Resistance': 0.00607, 'Reactance': 0.04227, 'Conductance': 0, 'Susceptance': 0.00691, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2666'},
        {'fromNode': '7187.0', 'toNode': '7186.0', 'Resistance': 0.00014, 'Reactance': 0.00907, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1600.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2667'},
        {'fromNode': '7187.0', 'toNode': '7186.0', 'Resistance': 0.00014, 'Reactance': 0.00907, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1600.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2668'},
        {'fromNode': '7189.0', 'toNode': '7186.0', 'Resistance': 0.00062, 'Reactance': 0.04222, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 264.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2669'},
        {'fromNode': '7190.0', 'toNode': '7186.0', 'Resistance': 0.00046, 'Reactance': 0.05343, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 337.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2670'},
        {'fromNode': '7191.0', 'toNode': '7186.0', 'Resistance': 0.00053, 'Reactance': 0.03094, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 258.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2671'},
        {'fromNode': '7192.0', 'toNode': '7186.0', 'Resistance': 0.0005, 'Reactance': 0.03659, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 312.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2672'},
        {'fromNode': '7193.0', 'toNode': '7186.0', 'Resistance': 0.00046, 'Reactance': 0.02713, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 256.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2673'},
        {'fromNode': '7194.0', 'toNode': '7186.0', 'Resistance': 0.0002, 'Reactance': 0.01508, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 861.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2674'},
        {'fromNode': '7195.0', 'toNode': '7186.0', 'Resistance': 0.00029, 'Reactance': 0.01748, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 863.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2675'},
        {'fromNode': '7196.0', 'toNode': '7186.0', 'Resistance': 0.00013, 'Reactance': 0.0094, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 866.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2676'},
        {'fromNode': '7197.0', 'toNode': '7186.0', 'Resistance': 8e-05, 'Reactance': 0.00604, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 867.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2677'},
        {'fromNode': '7186.0', 'toNode': '7341.0', 'Resistance': 0.00031, 'Reactance': 0.00639, 'Conductance': 0, 'Susceptance': 0.44587, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2678'},
        {'fromNode': '7346.0', 'toNode': '7186.0', 'Resistance': 0.00026, 'Reactance': 0.00532, 'Conductance': 0, 'Susceptance': 0.37115, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2679'},
        {'fromNode': '7188.0', 'toNode': '7187.0', 'Resistance': 0.00025, 'Reactance': 0.01839, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 600.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2680'},
        {'fromNode': '7188.0', 'toNode': '7187.0', 'Resistance': 0.00025, 'Reactance': 0.01839, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 600.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2681'},
        {'fromNode': '7188.0', 'toNode': '7187.0', 'Resistance': 0.00027, 'Reactance': 0.01839, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 600.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2682'},
        {'fromNode': '7188.0', 'toNode': '7187.0', 'Resistance': 0.00027, 'Reactance': 0.01839, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 600.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2683'},
        {'fromNode': '7187.0', 'toNode': '7401.0', 'Resistance': 0.00192, 'Reactance': 0.01878, 'Conductance': 0, 'Susceptance': 0.03613, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2684'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2685'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2686'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2687'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2688'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2689'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2690'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2691'},
        {'fromNode': '7188.0', 'toNode': '7255.0', 'Resistance': 0.00102, 'Reactance': 0.00714, 'Conductance': 0, 'Susceptance': 0.00117, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2692'},
        {'fromNode': '7198.0', 'toNode': '7201.0', 'Resistance': 0.00328, 'Reactance': 0.02241, 'Conductance': 0, 'Susceptance': 0.00381, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2693'},
        {'fromNode': '7198.0', 'toNode': '7201.0', 'Resistance': 0.00328, 'Reactance': 0.02241, 'Conductance': 0, 'Susceptance': 0.00381, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2694'},
        {'fromNode': '7198.0', 'toNode': '7225.0', 'Resistance': 0.01119, 'Reactance': 0.07798, 'Conductance': 0, 'Susceptance': 0.01275, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2695'},
        {'fromNode': '7200.0', 'toNode': '7199.0', 'Resistance': 0.0002, 'Reactance': 0.01412, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1344.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2696'},
        {'fromNode': '7199.0', 'toNode': '7331.0', 'Resistance': 0.0004, 'Reactance': 0.00819, 'Conductance': 0, 'Susceptance': 0.57112, 'MVA_rating': 3146.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2697'},
        {'fromNode': '7201.0', 'toNode': '7200.0', 'Resistance': 0.0006, 'Reactance': 0.03834, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.4, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2698'},
        {'fromNode': '7201.0', 'toNode': '7200.0', 'Resistance': 0.0006, 'Reactance': 0.03834, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.4, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2699'},
        {'fromNode': '7201.0', 'toNode': '7200.0', 'Resistance': 0.0006, 'Reactance': 0.03834, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.4, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2700'},
        {'fromNode': '7420.0', 'toNode': '7200.0', 'Resistance': 0.00231, 'Reactance': 0.02255, 'Conductance': 0, 'Susceptance': 0.04338, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2701'},
        {'fromNode': '7203.0', 'toNode': '7247.0', 'Resistance': 0.00676, 'Reactance': 0.03864, 'Conductance': 0, 'Susceptance': 0.00581, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2702'},
        {'fromNode': '7203.0', 'toNode': '7313.0', 'Resistance': 0.018, 'Reactance': 0.078, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2703'},
        {'fromNode': '7393.0', 'toNode': '7203.0', 'Resistance': 0.03637, 'Reactance': 0.08891, 'Conductance': 0, 'Susceptance': 0.01164, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2704'},
        {'fromNode': '7426.0', 'toNode': '7203.0', 'Resistance': 0.03596, 'Reactance': 0.08792, 'Conductance': 0, 'Susceptance': 0.01151, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2705'},
        {'fromNode': '7205.0', 'toNode': '7204.0', 'Resistance': 0.00014, 'Reactance': 0.01015, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1324.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2706'},
        {'fromNode': '7207.0', 'toNode': '7204.0', 'Resistance': 0.00039, 'Reactance': 0.02463, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 243.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2707'},
        {'fromNode': '7208.0', 'toNode': '7204.0', 'Resistance': 0.00013, 'Reactance': 0.00923, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1265.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2708'},
        {'fromNode': '7209.0', 'toNode': '7204.0', 'Resistance': 7e-05, 'Reactance': 0.00639, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1260.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2709'},
        {'fromNode': '7210.0', 'toNode': '7204.0', 'Resistance': 0.00047, 'Reactance': 0.02188, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 664.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2710'},
        {'fromNode': '7211.0', 'toNode': '7204.0', 'Resistance': 0.00025, 'Reactance': 0.01573, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 656.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2711'},
        {'fromNode': '7212.0', 'toNode': '7204.0', 'Resistance': 0.00039, 'Reactance': 0.02557, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 669.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2712'},
        {'fromNode': '7263.0', 'toNode': '7204.0', 'Resistance': 0.00018, 'Reactance': 0.00218, 'Conductance': 0, 'Susceptance': 0.14491, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2713'},
        {'fromNode': '7204.0', 'toNode': '7428.0', 'Resistance': 0.00115, 'Reactance': 0.01399, 'Conductance': 0, 'Susceptance': 0.92975, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2714'},
        {'fromNode': '7206.0', 'toNode': '7205.0', 'Resistance': 0.00229, 'Reactance': 0.09203, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 181.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2715'},
        {'fromNode': '7282.0', 'toNode': '7205.0', 'Resistance': 0.00079, 'Reactance': 0.00822, 'Conductance': 0, 'Susceptance': 0.02799, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2716'},
        {'fromNode': '7206.0', 'toNode': '7294.0', 'Resistance': 0.0068, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.0058, 'MVA_rating': 100.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2717'},
        {'fromNode': '7213.0', 'toNode': '7273.0', 'Resistance': 0.01615, 'Reactance': 0.03947, 'Conductance': 0, 'Susceptance': 0.00517, 'MVA_rating': 109.76, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2718'},
        {'fromNode': '7293.0', 'toNode': '7213.0', 'Resistance': 0.0088, 'Reactance': 0.03851, 'Conductance': 0, 'Susceptance': 0.00586, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2719'},
        {'fromNode': '7213.0', 'toNode': '7363.0', 'Resistance': 0.00686, 'Reactance': 0.03916, 'Conductance': 0, 'Susceptance': 0.00588, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2720'},
        {'fromNode': '7214.0', 'toNode': '7257.0', 'Resistance': 0.00967, 'Reactance': 0.04229, 'Conductance': 0, 'Susceptance': 0.00644, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2721'},
        {'fromNode': '7214.0', 'toNode': '7297.0', 'Resistance': 0.00755, 'Reactance': 0.03365, 'Conductance': 0, 'Susceptance': 0.00493, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2722'},
        {'fromNode': '7345.0', 'toNode': '7214.0', 'Resistance': 0.00276, 'Reactance': 0.01888, 'Conductance': 0, 'Susceptance': 0.00321, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2723'},
        {'fromNode': '7345.0', 'toNode': '7214.0', 'Resistance': 0.00276, 'Reactance': 0.01888, 'Conductance': 0, 'Susceptance': 0.00321, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2724'},
        {'fromNode': '7411.0', 'toNode': '7215.0', 'Resistance': 0.00253, 'Reactance': 0.01729, 'Conductance': 0, 'Susceptance': 0.00294, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2725'},
        {'fromNode': '7215.0', 'toNode': '7431.0', 'Resistance': 0.00465, 'Reactance': 0.02075, 'Conductance': 0, 'Susceptance': 0.00304, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2726'},
        {'fromNode': '7216.0', 'toNode': '7382.0', 'Resistance': 0.00725, 'Reactance': 0.03234, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2727'},
        {'fromNode': '7216.0', 'toNode': '7386.0', 'Resistance': 0.00794, 'Reactance': 0.04769, 'Conductance': 0, 'Susceptance': 0.00647, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2728'},
        {'fromNode': '7249.0', 'toNode': '7217.0', 'Resistance': 0.00192, 'Reactance': 0.01152, 'Conductance': 0, 'Susceptance': 0.00156, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2729'},
        {'fromNode': '7217.0', 'toNode': '7276.0', 'Resistance': 0.01308, 'Reactance': 0.03197, 'Conductance': 0, 'Susceptance': 0.00419, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2730'},
        {'fromNode': '7421.0', 'toNode': '7218.0', 'Resistance': 0.00797, 'Reactance': 0.03307, 'Conductance': 0, 'Susceptance': 0.00561, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2731'},
        {'fromNode': '7218.0', 'toNode': '8009.0', 'Resistance': 0.02277, 'Reactance': 0.05567, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2732'},
        {'fromNode': '7230.0', 'toNode': '7219.0', 'Resistance': 0.00954, 'Reactance': 0.04175, 'Conductance': 0, 'Susceptance': 0.00636, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2733'},
        {'fromNode': '7219.0', 'toNode': '7237.0', 'Resistance': 0.01747, 'Reactance': 0.07641, 'Conductance': 0, 'Susceptance': 0.01164, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2734'},
        {'fromNode': '7413.0', 'toNode': '7219.0', 'Resistance': 0.0058, 'Reactance': 0.03963, 'Conductance': 0, 'Susceptance': 0.00674, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2735'},
        {'fromNode': '7413.0', 'toNode': '7219.0', 'Resistance': 0.0058, 'Reactance': 0.03963, 'Conductance': 0, 'Susceptance': 0.00674, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2736'},
        {'fromNode': '7221.0', 'toNode': '7220.0', 'Resistance': 0.00146, 'Reactance': 0.00997, 'Conductance': 0, 'Susceptance': 0.0017, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2737'},
        {'fromNode': '7221.0', 'toNode': '7220.0', 'Resistance': 0.00146, 'Reactance': 0.00997, 'Conductance': 0, 'Susceptance': 0.0017, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2738'},
        {'fromNode': '7220.0', 'toNode': '7396.0', 'Resistance': 0.00651, 'Reactance': 0.04448, 'Conductance': 0, 'Susceptance': 0.00757, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2739'},
        {'fromNode': '7221.0', 'toNode': '7296.0', 'Resistance': 0.00652, 'Reactance': 0.02849, 'Conductance': 0, 'Susceptance': 0.00434, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2740'},
        {'fromNode': '7222.0', 'toNode': '7338.0', 'Resistance': 0.00615, 'Reactance': 0.01503, 'Conductance': 0, 'Susceptance': 0.00197, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2741'},
        {'fromNode': '7222.0', 'toNode': '7362.0', 'Resistance': 0.01114, 'Reactance': 0.04873, 'Conductance': 0, 'Susceptance': 0.00742, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2742'},
        {'fromNode': '7292.0', 'toNode': '7223.0', 'Resistance': 0.002, 'Reactance': 0.01393, 'Conductance': 0, 'Susceptance': 0.00228, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2743'},
        {'fromNode': '7292.0', 'toNode': '7223.0', 'Resistance': 0.002, 'Reactance': 0.01393, 'Conductance': 0, 'Susceptance': 0.00228, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2744'},
        {'fromNode': '7223.0', 'toNode': '7404.0', 'Resistance': 0.0068, 'Reactance': 0.02978, 'Conductance': 0, 'Susceptance': 0.00454, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2745'},
        {'fromNode': '7223.0', 'toNode': '7425.0', 'Resistance': 0.00566, 'Reactance': 0.03229, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2746'},
        {'fromNode': '7224.0', 'toNode': '7248.0', 'Resistance': 0.00705, 'Reactance': 0.04027, 'Conductance': 0, 'Susceptance': 0.00605, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2747'},
        {'fromNode': '7384.0', 'toNode': '7224.0', 'Resistance': 0.00429, 'Reactance': 0.01049, 'Conductance': 0, 'Susceptance': 0.00137, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2748'},
        {'fromNode': '7224.0', 'toNode': '7417.0', 'Resistance': 0.00479, 'Reactance': 0.02874, 'Conductance': 0, 'Susceptance': 0.0039, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2749'},
        {'fromNode': '7246.0', 'toNode': '7225.0', 'Resistance': 0.00662, 'Reactance': 0.02747, 'Conductance': 0, 'Susceptance': 0.00466, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2750'},
        {'fromNode': '7225.0', 'toNode': '7395.0', 'Resistance': 0.00809, 'Reactance': 0.04618, 'Conductance': 0, 'Susceptance': 0.00694, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2751'},
        {'fromNode': '7226.0', 'toNode': '7240.0', 'Resistance': 0.00701, 'Reactance': 0.03067, 'Conductance': 0, 'Susceptance': 0.00467, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2752'},
        {'fromNode': '7226.0', 'toNode': '7250.0', 'Resistance': 0.00636, 'Reactance': 0.03633, 'Conductance': 0, 'Susceptance': 0.00546, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2753'},
        {'fromNode': '7226.0', 'toNode': '7272.0', 'Resistance': 0.00504, 'Reactance': 0.02244, 'Conductance': 0, 'Susceptance': 0.00329, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2754'},
        {'fromNode': '7226.0', 'toNode': '7338.0', 'Resistance': 0.01145, 'Reactance': 0.02798, 'Conductance': 0, 'Susceptance': 0.00366, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2755'},
        {'fromNode': '7228.0', 'toNode': '7227.0', 'Resistance': 0.00016, 'Reactance': 0.01098, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1259.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2756'},
        {'fromNode': '7227.0', 'toNode': '7422.0', 'Resistance': 0.00048, 'Reactance': 0.00974, 'Conductance': 0, 'Susceptance': 0.67927, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2757'},
        {'fromNode': '7229.0', 'toNode': '7228.0', 'Resistance': 0.00043, 'Reactance': 0.01801, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2758'},
        {'fromNode': '7229.0', 'toNode': '7228.0', 'Resistance': 0.00043, 'Reactance': 0.01801, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2759'},
        {'fromNode': '7229.0', 'toNode': '7228.0', 'Resistance': 0.00043, 'Reactance': 0.01801, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2760'},
        {'fromNode': '7409.0', 'toNode': '7228.0', 'Resistance': 0.0016, 'Reactance': 0.01143, 'Conductance': 0, 'Susceptance': 0.02083, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2761'},
        {'fromNode': '7397.0', 'toNode': '7230.0', 'Resistance': 0.01033, 'Reactance': 0.04606, 'Conductance': 0, 'Susceptance': 0.00675, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2762'},
        {'fromNode': '7265.0', 'toNode': '7231.0', 'Resistance': 0.00327, 'Reactance': 0.02233, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2763'},
        {'fromNode': '7265.0', 'toNode': '7231.0', 'Resistance': 0.00327, 'Reactance': 0.02233, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2764'},
        {'fromNode': '7265.0', 'toNode': '7231.0', 'Resistance': 0.00327, 'Reactance': 0.02233, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2765'},
        {'fromNode': '7265.0', 'toNode': '7231.0', 'Resistance': 0.00327, 'Reactance': 0.02233, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2766'},
        {'fromNode': '7231.0', 'toNode': '7403.0', 'Resistance': 0.00187, 'Reactance': 0.01275, 'Conductance': 0, 'Susceptance': 0.00217, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2767'},
        {'fromNode': '7231.0', 'toNode': '7403.0', 'Resistance': 0.00187, 'Reactance': 0.01275, 'Conductance': 0, 'Susceptance': 0.00217, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2768'},
        {'fromNode': '7273.0', 'toNode': '7232.0', 'Resistance': 0.00198, 'Reactance': 0.01376, 'Conductance': 0, 'Susceptance': 0.00225, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2769'},
        {'fromNode': '7233.0', 'toNode': '7251.0', 'Resistance': 0.01464, 'Reactance': 0.0358, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2770'},
        {'fromNode': '7251.0', 'toNode': '7235.0', 'Resistance': 0.01132, 'Reactance': 0.04952, 'Conductance': 0, 'Susceptance': 0.00754, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2771'},
        {'fromNode': '7235.0', 'toNode': '7315.0', 'Resistance': 0.00551, 'Reactance': 0.03841, 'Conductance': 0, 'Susceptance': 0.00628, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2772'},
        {'fromNode': '7235.0', 'toNode': '7315.0', 'Resistance': 0.00551, 'Reactance': 0.03841, 'Conductance': 0, 'Susceptance': 0.00628, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2773'},
        {'fromNode': '7236.0', 'toNode': '7245.0', 'Resistance': 0.00642, 'Reactance': 0.02662, 'Conductance': 0, 'Susceptance': 0.00452, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2774'},
        {'fromNode': '7236.0', 'toNode': '7292.0', 'Resistance': 0.01113, 'Reactance': 0.04618, 'Conductance': 0, 'Susceptance': 0.00784, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2775'},
        {'fromNode': '7236.0', 'toNode': '7411.0', 'Resistance': 0.01497, 'Reactance': 0.06208, 'Conductance': 0, 'Susceptance': 0.01054, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2776'},
        {'fromNode': '7237.0', 'toNode': '7247.0', 'Resistance': 0.03475, 'Reactance': 0.15202, 'Conductance': 0, 'Susceptance': 0.02315, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2777'},
        {'fromNode': '7238.0', 'toNode': '7392.0', 'Resistance': 0.0068, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2778'},
        {'fromNode': '7294.0', 'toNode': '7239.0', 'Resistance': 0.05931, 'Reactance': 0.14497, 'Conductance': 0, 'Susceptance': 0.01898, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2779'},
        {'fromNode': '7239.0', 'toNode': '7392.0', 'Resistance': 0.03658, 'Reactance': 0.08942, 'Conductance': 0, 'Susceptance': 0.01171, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2780'},
        {'fromNode': '7241.0', 'toNode': '7362.0', 'Resistance': 0.00281, 'Reactance': 0.01606, 'Conductance': 0, 'Susceptance': 0.00241, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2781'},
        {'fromNode': '7242.0', 'toNode': '7295.0', 'Resistance': 0.01417, 'Reactance': 0.03406, 'Conductance': 0, 'Susceptance': 0.00462, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2782'},
        {'fromNode': '7242.0', 'toNode': '7319.0', 'Resistance': 0.00493, 'Reactance': 0.03187, 'Conductance': 0, 'Susceptance': 0.00608, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2783'},
        {'fromNode': '7242.0', 'toNode': '7319.0', 'Resistance': 0.00493, 'Reactance': 0.03187, 'Conductance': 0, 'Susceptance': 0.00608, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2784'},
        {'fromNode': '7260.0', 'toNode': '7243.0', 'Resistance': 0.00509, 'Reactance': 0.03286, 'Conductance': 0, 'Susceptance': 0.00627, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2785'},
        {'fromNode': '7260.0', 'toNode': '7243.0', 'Resistance': 0.00509, 'Reactance': 0.03286, 'Conductance': 0, 'Susceptance': 0.00627, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2786'},
        {'fromNode': '7244.0', 'toNode': '7272.0', 'Resistance': 0.01177, 'Reactance': 0.05245, 'Conductance': 0, 'Susceptance': 0.00769, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2787'},
        {'fromNode': '7246.0', 'toNode': '7251.0', 'Resistance': 0.01351, 'Reactance': 0.07719, 'Conductance': 0, 'Susceptance': 0.0116, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2788'},
        {'fromNode': '7246.0', 'toNode': '7410.0', 'Resistance': 0.00919, 'Reactance': 0.06404, 'Conductance': 0, 'Susceptance': 0.01047, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2789'},
        {'fromNode': '7246.0', 'toNode': '7410.0', 'Resistance': 0.00919, 'Reactance': 0.06404, 'Conductance': 0, 'Susceptance': 0.01047, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2790'},
        {'fromNode': '7425.0', 'toNode': '7246.0', 'Resistance': 0.00724, 'Reactance': 0.01741, 'Conductance': 0, 'Susceptance': 0.00236, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2791'},
        {'fromNode': '7339.0', 'toNode': '7247.0', 'Resistance': 0.01441, 'Reactance': 0.05979, 'Conductance': 0, 'Susceptance': 0.01015, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2792'},
        {'fromNode': '7362.0', 'toNode': '7248.0', 'Resistance': 0.00406, 'Reactance': 0.01777, 'Conductance': 0, 'Susceptance': 0.00271, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2793'},
        {'fromNode': '7248.0', 'toNode': '7386.0', 'Resistance': 0.00247, 'Reactance': 0.01596, 'Conductance': 0, 'Susceptance': 0.00304, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2794'},
        {'fromNode': '7248.0', 'toNode': '7386.0', 'Resistance': 0.00247, 'Reactance': 0.01596, 'Conductance': 0, 'Susceptance': 0.00304, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2795'},
        {'fromNode': '7249.0', 'toNode': '7255.0', 'Resistance': 0.00406, 'Reactance': 0.02776, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2796'},
        {'fromNode': '7249.0', 'toNode': '7255.0', 'Resistance': 0.00406, 'Reactance': 0.02776, 'Conductance': 0, 'Susceptance': 0.00472, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2797'},
        {'fromNode': '7249.0', 'toNode': '7344.0', 'Resistance': 0.00383, 'Reactance': 0.02615, 'Conductance': 0, 'Susceptance': 0.00445, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2798'},
        {'fromNode': '7249.0', 'toNode': '7344.0', 'Resistance': 0.00383, 'Reactance': 0.02615, 'Conductance': 0, 'Susceptance': 0.00445, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2799'},
        {'fromNode': '7249.0', 'toNode': '7382.0', 'Resistance': 0.01682, 'Reactance': 0.04113, 'Conductance': 0, 'Susceptance': 0.00539, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2800'},
        {'fromNode': '7250.0', 'toNode': '7276.0', 'Resistance': 0.00404, 'Reactance': 0.0261, 'Conductance': 0, 'Susceptance': 0.00498, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2801'},
        {'fromNode': '7250.0', 'toNode': '7276.0', 'Resistance': 0.00404, 'Reactance': 0.0261, 'Conductance': 0, 'Susceptance': 0.00498, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2802'},
        {'fromNode': '7253.0', 'toNode': '7252.0', 'Resistance': 0.0009, 'Reactance': 0.03239, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 297.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2803'},
        {'fromNode': '7312.0', 'toNode': '7252.0', 'Resistance': 0.00133, 'Reactance': 0.01297, 'Conductance': 0, 'Susceptance': 0.02495, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2804'},
        {'fromNode': '7327.0', 'toNode': '7252.0', 'Resistance': 0.00593, 'Reactance': 0.04235, 'Conductance': 0, 'Susceptance': 0.07721, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2805'},
        {'fromNode': '7347.0', 'toNode': '7252.0', 'Resistance': 0.00363, 'Reactance': 0.03547, 'Conductance': 0, 'Susceptance': 0.06824, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2806'},
        {'fromNode': '7313.0', 'toNode': '7253.0', 'Resistance': 0.01056, 'Reactance': 0.04704, 'Conductance': 0, 'Susceptance': 0.0069, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2807'},
        {'fromNode': '7253.0', 'toNode': '7412.0', 'Resistance': 0.00665, 'Reactance': 0.04295, 'Conductance': 0, 'Susceptance': 0.00819, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2808'},
        {'fromNode': '7253.0', 'toNode': '7412.0', 'Resistance': 0.00665, 'Reactance': 0.04295, 'Conductance': 0, 'Susceptance': 0.00819, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2809'},
        {'fromNode': '7255.0', 'toNode': '7258.0', 'Resistance': 0.00242, 'Reactance': 0.01655, 'Conductance': 0, 'Susceptance': 0.00282, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2810'},
        {'fromNode': '7255.0', 'toNode': '7258.0', 'Resistance': 0.00242, 'Reactance': 0.01655, 'Conductance': 0, 'Susceptance': 0.00282, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2811'},
        {'fromNode': '7255.0', 'toNode': '7399.0', 'Resistance': 0.00104, 'Reactance': 0.00673, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2812'},
        {'fromNode': '7255.0', 'toNode': '7399.0', 'Resistance': 0.00104, 'Reactance': 0.00673, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2813'},
        {'fromNode': '7255.0', 'toNode': '7399.0', 'Resistance': 0.00104, 'Reactance': 0.00673, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2814'},
        {'fromNode': '7255.0', 'toNode': '7399.0', 'Resistance': 0.00104, 'Reactance': 0.00673, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2815'},
        {'fromNode': '7256.0', 'toNode': '7297.0', 'Resistance': 0.00926, 'Reactance': 0.02263, 'Conductance': 0, 'Susceptance': 0.00296, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2816'},
        {'fromNode': '7257.0', 'toNode': '7317.0', 'Resistance': 0.00452, 'Reactance': 0.03088, 'Conductance': 0, 'Susceptance': 0.00525, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2817'},
        {'fromNode': '7257.0', 'toNode': '7317.0', 'Resistance': 0.00452, 'Reactance': 0.03088, 'Conductance': 0, 'Susceptance': 0.00525, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2818'},
        {'fromNode': '7259.0', 'toNode': '7403.0', 'Resistance': 0.00479, 'Reactance': 0.03335, 'Conductance': 0, 'Susceptance': 0.00545, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2819'},
        {'fromNode': '7259.0', 'toNode': '7403.0', 'Resistance': 0.00479, 'Reactance': 0.03335, 'Conductance': 0, 'Susceptance': 0.00545, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2820'},
        {'fromNode': '7368.0', 'toNode': '7260.0', 'Resistance': 0.00263, 'Reactance': 0.01833, 'Conductance': 0, 'Susceptance': 0.003, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2821'},
        {'fromNode': '7368.0', 'toNode': '7260.0', 'Resistance': 0.00263, 'Reactance': 0.01833, 'Conductance': 0, 'Susceptance': 0.003, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2822'},
        {'fromNode': '7368.0', 'toNode': '7260.0', 'Resistance': 0.00263, 'Reactance': 0.01833, 'Conductance': 0, 'Susceptance': 0.003, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2823'},
        {'fromNode': '7262.0', 'toNode': '7261.0', 'Resistance': 0.00059, 'Reactance': 0.02478, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 305.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2824'},
        {'fromNode': '7261.0', 'toNode': '7429.0', 'Resistance': 0.0045, 'Reactance': 0.04398, 'Conductance': 0, 'Susceptance': 0.08462, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2825'},
        {'fromNode': '7262.0', 'toNode': '7302.0', 'Resistance': 0.01427, 'Reactance': 0.06362, 'Conductance': 0, 'Susceptance': 0.00933, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2826'},
        {'fromNode': '7264.0', 'toNode': '7263.0', 'Resistance': 0.00013, 'Reactance': 0.00819, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1362.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2827'},
        {'fromNode': '7264.0', 'toNode': '7263.0', 'Resistance': 0.00013, 'Reactance': 0.00819, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1362.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2828'},
        {'fromNode': '7266.0', 'toNode': '7263.0', 'Resistance': 0.00013, 'Reactance': 0.00771, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 983.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2829'},
        {'fromNode': '7267.0', 'toNode': '7263.0', 'Resistance': 0.00069, 'Reactance': 0.04478, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 443.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2830'},
        {'fromNode': '7268.0', 'toNode': '7263.0', 'Resistance': 0.0002, 'Reactance': 0.01399, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2831'},
        {'fromNode': '7269.0', 'toNode': '7263.0', 'Resistance': 0.00038, 'Reactance': 0.04031, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 440.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2832'},
        {'fromNode': '7270.0', 'toNode': '7263.0', 'Resistance': 0.00019, 'Reactance': 0.01203, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 701.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2833'},
        {'fromNode': '7265.0', 'toNode': '7264.0', 'Resistance': 0.00053, 'Reactance': 0.0323, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 461.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2834'},
        {'fromNode': '7265.0', 'toNode': '7264.0', 'Resistance': 0.00053, 'Reactance': 0.0323, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 461.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2835'},
        {'fromNode': '7264.0', 'toNode': '7332.0', 'Resistance': 0.00258, 'Reactance': 0.01841, 'Conductance': 0, 'Susceptance': 0.03356, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2836'},
        {'fromNode': '7315.0', 'toNode': '7271.0', 'Resistance': 0.01055, 'Reactance': 0.04377, 'Conductance': 0, 'Susceptance': 0.00743, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2837'},
        {'fromNode': '7271.0', 'toNode': '7340.0', 'Resistance': 0.01045, 'Reactance': 0.04336, 'Conductance': 0, 'Susceptance': 0.00736, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2838'},
        {'fromNode': '7316.0', 'toNode': '7272.0', 'Resistance': 0.00354, 'Reactance': 0.02023, 'Conductance': 0, 'Susceptance': 0.00304, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2839'},
        {'fromNode': '7273.0', 'toNode': '7405.0', 'Resistance': 0.01205, 'Reactance': 0.06881, 'Conductance': 0, 'Susceptance': 0.01034, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2840'},
        {'fromNode': '7275.0', 'toNode': '7274.0', 'Resistance': 0.00248, 'Reactance': 0.12538, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2841'},
        {'fromNode': '7276.0', 'toNode': '7395.0', 'Resistance': 0.00344, 'Reactance': 0.02064, 'Conductance': 0, 'Susceptance': 0.0028, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2842'},
        {'fromNode': '7277.0', 'toNode': '7337.0', 'Resistance': 0.02832, 'Reactance': 0.06923, 'Conductance': 0, 'Susceptance': 0.00906, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2843'},
        {'fromNode': '7279.0', 'toNode': '7278.0', 'Resistance': 0.00131, 'Reactance': 0.07995, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 172.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2844'},
        {'fromNode': '7280.0', 'toNode': '7278.0', 'Resistance': 0.00175, 'Reactance': 0.10484, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 87.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2845'},
        {'fromNode': '7281.0', 'toNode': '7278.0', 'Resistance': 0.00169, 'Reactance': 0.12869, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 91.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2846'},
        {'fromNode': '7367.0', 'toNode': '7278.0', 'Resistance': 0.00241, 'Reactance': 0.0172, 'Conductance': 0, 'Susceptance': 0.03135, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2847'},
        {'fromNode': '7392.0', 'toNode': '7279.0', 'Resistance': 0.00555, 'Reactance': 0.02431, 'Conductance': 0, 'Susceptance': 0.0037, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2848'},
        {'fromNode': '7283.0', 'toNode': '7282.0', 'Resistance': 0.00254, 'Reactance': 0.07063, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 145.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2849'},
        {'fromNode': '7284.0', 'toNode': '7282.0', 'Resistance': 0.00703, 'Reactance': 0.26257, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 70.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2850'},
        {'fromNode': '7285.0', 'toNode': '7282.0', 'Resistance': 0.00137, 'Reactance': 0.08957, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 93.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2851'},
        {'fromNode': '7286.0', 'toNode': '7282.0', 'Resistance': 0.00545, 'Reactance': 0.20611, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 93.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2852'},
        {'fromNode': '7287.0', 'toNode': '7282.0', 'Resistance': 0.00328, 'Reactance': 0.17841, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 91.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2853'},
        {'fromNode': '7288.0', 'toNode': '7282.0', 'Resistance': 0.00467, 'Reactance': 0.19001, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 93.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2854'},
        {'fromNode': '7289.0', 'toNode': '7282.0', 'Resistance': 0.00155, 'Reactance': 0.07083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 124.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2855'},
        {'fromNode': '7282.0', 'toNode': '7332.0', 'Resistance': 0.00276, 'Reactance': 0.01973, 'Conductance': 0, 'Susceptance': 0.03596, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2856'},
        {'fromNode': '7282.0', 'toNode': '7367.0', 'Resistance': 0.00169, 'Reactance': 0.01205, 'Conductance': 0, 'Susceptance': 0.02197, 'MVA_rating': 523.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2857'},
        {'fromNode': '7411.0', 'toNode': '7290.0', 'Resistance': 0.00285, 'Reactance': 0.01709, 'Conductance': 0, 'Susceptance': 0.00232, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2858'},
        {'fromNode': '7291.0', 'toNode': '7295.0', 'Resistance': 0.00838, 'Reactance': 0.03474, 'Conductance': 0, 'Susceptance': 0.0059, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2859'},
        {'fromNode': '7424.0', 'toNode': '7293.0', 'Resistance': 0.0065, 'Reactance': 0.02894, 'Conductance': 0, 'Susceptance': 0.00424, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2860'},
        {'fromNode': '7294.0', 'toNode': '7405.0', 'Resistance': 0.0229, 'Reactance': 0.13751, 'Conductance': 0, 'Susceptance': 0.01866, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2861'},
        {'fromNode': '7296.0', 'toNode': '7427.0', 'Resistance': 0.0045, 'Reactance': 0.011, 'Conductance': 0, 'Susceptance': 0.00144, 'MVA_rating': 102.9, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2862'},
        {'fromNode': '7316.0', 'toNode': '7297.0', 'Resistance': 0.00394, 'Reactance': 0.01758, 'Conductance': 0, 'Susceptance': 0.00258, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2863'},
        {'fromNode': '7297.0', 'toNode': '7384.0', 'Resistance': 0.00423, 'Reactance': 0.0185, 'Conductance': 0, 'Susceptance': 0.00282, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2864'},
        {'fromNode': '7298.0', 'toNode': '7362.0', 'Resistance': 0.01319, 'Reactance': 0.03222, 'Conductance': 0, 'Susceptance': 0.00422, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2865'},
        {'fromNode': '7298.0', 'toNode': '7395.0', 'Resistance': 0.00338, 'Reactance': 0.0148, 'Conductance': 0, 'Susceptance': 0.00225, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2866'},
        {'fromNode': '7364.0', 'toNode': '7299.0', 'Resistance': 0.01228, 'Reactance': 0.07011, 'Conductance': 0, 'Susceptance': 0.01053, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2867'},
        {'fromNode': '7413.0', 'toNode': '7299.0', 'Resistance': 0.01459, 'Reactance': 0.065, 'Conductance': 0, 'Susceptance': 0.00953, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2868'},
        {'fromNode': '7301.0', 'toNode': '7318.0', 'Resistance': 0.01182, 'Reactance': 0.05168, 'Conductance': 0, 'Susceptance': 0.00787, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2869'},
        {'fromNode': '7302.0', 'toNode': '7426.0', 'Resistance': 0.02525, 'Reactance': 0.06173, 'Conductance': 0, 'Susceptance': 0.00808, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2870'},
        {'fromNode': '7303.0', 'toNode': '7393.0', 'Resistance': 0.07296, 'Reactance': 0.17836, 'Conductance': 0, 'Susceptance': 0.02335, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2871'},
        {'fromNode': '7305.0', 'toNode': '7304.0', 'Resistance': 0.00017, 'Reactance': 0.01241, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1160.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2872'},
        {'fromNode': '7307.0', 'toNode': '7304.0', 'Resistance': 0.00035, 'Reactance': 0.02169, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 473.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2873'},
        {'fromNode': '7308.0', 'toNode': '7304.0', 'Resistance': 0.00021, 'Reactance': 0.02062, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2874'},
        {'fromNode': '7309.0', 'toNode': '7304.0', 'Resistance': 0.00056, 'Reactance': 0.04001, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 454.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2875'},
        {'fromNode': '7310.0', 'toNode': '7304.0', 'Resistance': 0.00063, 'Reactance': 0.03797, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 464.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2876'},
        {'fromNode': '7311.0', 'toNode': '7304.0', 'Resistance': 0.00019, 'Reactance': 0.01172, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 729.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2877'},
        {'fromNode': '7341.0', 'toNode': '7304.0', 'Resistance': 0.00071, 'Reactance': 0.00864, 'Conductance': 0, 'Susceptance': 0.57375, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2878'},
        {'fromNode': '7306.0', 'toNode': '7305.0', 'Resistance': 0.00078, 'Reactance': 0.04744, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 305.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2879'},
        {'fromNode': '7305.0', 'toNode': '7327.0', 'Resistance': 0.0022, 'Reactance': 0.01568, 'Conductance': 0, 'Susceptance': 0.02859, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2880'},
        {'fromNode': '7306.0', 'toNode': '7413.0', 'Resistance': 0.00627, 'Reactance': 0.04371, 'Conductance': 0, 'Susceptance': 0.00714, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2881'},
        {'fromNode': '7306.0', 'toNode': '7413.0', 'Resistance': 0.00627, 'Reactance': 0.04371, 'Conductance': 0, 'Susceptance': 0.00714, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2882'},
        {'fromNode': '7313.0', 'toNode': '7312.0', 'Resistance': 0.00097, 'Reactance': 0.06614, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 168.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2883'},
        {'fromNode': '7312.0', 'toNode': '7321.0', 'Resistance': 0.00143, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.02693, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2884'},
        {'fromNode': '7312.0', 'toNode': '7429.0', 'Resistance': 0.00088, 'Reactance': 0.00862, 'Conductance': 0, 'Susceptance': 0.01659, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2885'},
        {'fromNode': '7313.0', 'toNode': '7430.0', 'Resistance': 0.00541, 'Reactance': 0.0309, 'Conductance': 0, 'Susceptance': 0.00464, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2886'},
        {'fromNode': '7340.0', 'toNode': '7314.0', 'Resistance': 0.01916, 'Reactance': 0.07949, 'Conductance': 0, 'Susceptance': 0.01349, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2887'},
        {'fromNode': '7316.0', 'toNode': '7383.0', 'Resistance': 0.01217, 'Reactance': 0.07861, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2888'},
        {'fromNode': '7316.0', 'toNode': '7383.0', 'Resistance': 0.01217, 'Reactance': 0.07861, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2889'},
        {'fromNode': '7317.0', 'toNode': '7424.0', 'Resistance': 0.00457, 'Reactance': 0.02949, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2890'},
        {'fromNode': '7317.0', 'toNode': '7424.0', 'Resistance': 0.00457, 'Reactance': 0.02949, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2891'},
        {'fromNode': '7317.0', 'toNode': '7424.0', 'Resistance': 0.00457, 'Reactance': 0.02949, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2892'},
        {'fromNode': '7317.0', 'toNode': '7424.0', 'Resistance': 0.00457, 'Reactance': 0.02949, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2893'},
        {'fromNode': '7318.0', 'toNode': '7382.0', 'Resistance': 0.00912, 'Reactance': 0.04063, 'Conductance': 0, 'Susceptance': 0.00596, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2894'},
        {'fromNode': '7321.0', 'toNode': '7320.0', 'Resistance': 0.00023, 'Reactance': 0.01682, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1000.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2895'},
        {'fromNode': '7323.0', 'toNode': '7320.0', 'Resistance': 0.00036, 'Reactance': 0.02134, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 584.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2896'},
        {'fromNode': '7324.0', 'toNode': '7320.0', 'Resistance': 0.00032, 'Reactance': 0.02884, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 577.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2897'},
        {'fromNode': '7325.0', 'toNode': '7320.0', 'Resistance': 0.0003, 'Reactance': 0.02899, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 581.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2898'},
        {'fromNode': '7326.0', 'toNode': '7320.0', 'Resistance': 0.00017, 'Reactance': 0.01413, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 568.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2899'},
        {'fromNode': '7320.0', 'toNode': '7422.0', 'Resistance': 0.00066, 'Reactance': 0.00806, 'Conductance': 0, 'Susceptance': 0.53536, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2900'},
        {'fromNode': '7322.0', 'toNode': '7321.0', 'Resistance': 0.00198, 'Reactance': 0.07214, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 128.4, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2901'},
        {'fromNode': '7322.0', 'toNode': '7380.0', 'Resistance': 0.00578, 'Reactance': 0.02527, 'Conductance': 0, 'Susceptance': 0.00385, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2902'},
        {'fromNode': '7328.0', 'toNode': '7327.0', 'Resistance': 0.00161, 'Reactance': 0.04271, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 172.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2903'},
        {'fromNode': '7329.0', 'toNode': '7327.0', 'Resistance': 0.00104, 'Reactance': 0.04807, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2904'},
        {'fromNode': '7328.0', 'toNode': '7339.0', 'Resistance': 0.01199, 'Reactance': 0.05341, 'Conductance': 0, 'Susceptance': 0.00783, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2905'},
        {'fromNode': '7362.0', 'toNode': '7330.0', 'Resistance': 0.00719, 'Reactance': 0.01758, 'Conductance': 0, 'Susceptance': 0.0023, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2906'},
        {'fromNode': '7332.0', 'toNode': '7331.0', 'Resistance': 0.00024, 'Reactance': 0.01699, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1122.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2907'},
        {'fromNode': '7334.0', 'toNode': '7331.0', 'Resistance': 0.00083, 'Reactance': 0.0414, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 244.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2908'},
        {'fromNode': '7335.0', 'toNode': '7331.0', 'Resistance': 0.00017, 'Reactance': 0.0093, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1106.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2909'},
        {'fromNode': '7400.0', 'toNode': '7331.0', 'Resistance': 8e-05, 'Reactance': 0.00168, 'Conductance': 0, 'Susceptance': 0.11713, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2910'},
        {'fromNode': '7333.0', 'toNode': '7332.0', 'Resistance': 0.00093, 'Reactance': 0.03886, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 421.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2911'},
        {'fromNode': '7333.0', 'toNode': '7332.0', 'Resistance': 0.00093, 'Reactance': 0.03886, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 421.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2912'},
        {'fromNode': '7333.0', 'toNode': '7332.0', 'Resistance': 0.00093, 'Reactance': 0.03886, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 421.2, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2913'},
        {'fromNode': '7401.0', 'toNode': '7332.0', 'Resistance': 0.00104, 'Reactance': 0.01017, 'Conductance': 0, 'Susceptance': 0.01957, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2914'},
        {'fromNode': '7336.0', 'toNode': '7418.0', 'Resistance': 0.00361, 'Reactance': 0.02463, 'Conductance': 0, 'Susceptance': 0.00419, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2915'},
        {'fromNode': '7336.0', 'toNode': '7418.0', 'Resistance': 0.00361, 'Reactance': 0.02463, 'Conductance': 0, 'Susceptance': 0.00419, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2916'},
        {'fromNode': '7345.0', 'toNode': '7338.0', 'Resistance': 0.0023, 'Reactance': 0.01602, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2917'},
        {'fromNode': '7345.0', 'toNode': '7338.0', 'Resistance': 0.0023, 'Reactance': 0.01602, 'Conductance': 0, 'Susceptance': 0.00262, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2918'},
        {'fromNode': '7342.0', 'toNode': '7341.0', 'Resistance': 0.00017, 'Reactance': 0.01252, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1311.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2919'},
        {'fromNode': '7343.0', 'toNode': '7342.0', 'Resistance': 0.00185, 'Reactance': 0.0832, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2920'},
        {'fromNode': '7343.0', 'toNode': '7342.0', 'Resistance': 0.00185, 'Reactance': 0.0832, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2921'},
        {'fromNode': '7342.0', 'toNode': '7409.0', 'Resistance': 0.00056, 'Reactance': 0.00583, 'Conductance': 0, 'Susceptance': 0.01986, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2922'},
        {'fromNode': '7342.0', 'toNode': '7409.0', 'Resistance': 0.00056, 'Reactance': 0.00583, 'Conductance': 0, 'Susceptance': 0.01986, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2923'},
        {'fromNode': '7344.0', 'toNode': '7404.0', 'Resistance': 0.00903, 'Reactance': 0.03745, 'Conductance': 0, 'Susceptance': 0.00635, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2924'},
        {'fromNode': '7345.0', 'toNode': '7382.0', 'Resistance': 0.00439, 'Reactance': 0.02833, 'Conductance': 0, 'Susceptance': 0.00541, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2925'},
        {'fromNode': '7345.0', 'toNode': '7382.0', 'Resistance': 0.00439, 'Reactance': 0.02833, 'Conductance': 0, 'Susceptance': 0.00541, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2926'},
        {'fromNode': '7347.0', 'toNode': '7346.0', 'Resistance': 0.00014, 'Reactance': 0.00986, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1271.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2927'},
        {'fromNode': '7347.0', 'toNode': '7346.0', 'Resistance': 0.00014, 'Reactance': 0.00986, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1271.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2928'},
        {'fromNode': '7349.0', 'toNode': '7346.0', 'Resistance': 8e-05, 'Reactance': 0.00563, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1019.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2929'},
        {'fromNode': '7350.0', 'toNode': '7346.0', 'Resistance': 0.00022, 'Reactance': 0.01346, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1021.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2930'},
        {'fromNode': '7351.0', 'toNode': '7346.0', 'Resistance': 0.00031, 'Reactance': 0.01784, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1018.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2931'},
        {'fromNode': '7352.0', 'toNode': '7346.0', 'Resistance': 0.00034, 'Reactance': 0.01655, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1025.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2932'},
        {'fromNode': '7353.0', 'toNode': '7346.0', 'Resistance': 0.00019, 'Reactance': 0.01203, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1225.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2933'},
        {'fromNode': '7354.0', 'toNode': '7346.0', 'Resistance': 6e-05, 'Reactance': 0.00671, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1222.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2934'},
        {'fromNode': '7355.0', 'toNode': '7346.0', 'Resistance': 6e-05, 'Reactance': 0.00677, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1218.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2935'},
        {'fromNode': '7356.0', 'toNode': '7346.0', 'Resistance': 0.00012, 'Reactance': 0.00609, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1219.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2936'},
        {'fromNode': '7357.0', 'toNode': '7346.0', 'Resistance': 0.00066, 'Reactance': 0.04067, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 287.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2937'},
        {'fromNode': '7358.0', 'toNode': '7346.0', 'Resistance': 0.0003, 'Reactance': 0.02129, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 835.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2938'},
        {'fromNode': '7359.0', 'toNode': '7346.0', 'Resistance': 0.00013, 'Reactance': 0.00906, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 846.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2939'},
        {'fromNode': '7360.0', 'toNode': '7346.0', 'Resistance': 0.00021, 'Reactance': 0.01514, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 845.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2940'},
        {'fromNode': '7361.0', 'toNode': '7346.0', 'Resistance': 0.00026, 'Reactance': 0.02464, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 471.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2941'},
        {'fromNode': '7348.0', 'toNode': '7347.0', 'Resistance': 0.00062, 'Reactance': 0.02698, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 407.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2942'},
        {'fromNode': '7348.0', 'toNode': '7347.0', 'Resistance': 0.00062, 'Reactance': 0.02698, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 407.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2943'},
        {'fromNode': '7380.0', 'toNode': '7363.0', 'Resistance': 0.03224, 'Reactance': 0.13375, 'Conductance': 0, 'Susceptance': 0.0227, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2944'},
        {'fromNode': '7365.0', 'toNode': '7381.0', 'Resistance': 0.00343, 'Reactance': 0.01423, 'Conductance': 0, 'Susceptance': 0.00241, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2945'},
        {'fromNode': '7367.0', 'toNode': '7366.0', 'Resistance': 0.00021, 'Reactance': 0.01366, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1249.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2946'},
        {'fromNode': '7367.0', 'toNode': '7366.0', 'Resistance': 0.00021, 'Reactance': 0.01366, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1249.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2947'},
        {'fromNode': '7369.0', 'toNode': '7366.0', 'Resistance': 0.00053, 'Reactance': 0.03243, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 494.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2948'},
        {'fromNode': '7370.0', 'toNode': '7366.0', 'Resistance': 0.00065, 'Reactance': 0.04118, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 480.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2949'},
        {'fromNode': '7371.0', 'toNode': '7366.0', 'Resistance': 0.00049, 'Reactance': 0.03404, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 469.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2950'},
        {'fromNode': '7372.0', 'toNode': '7366.0', 'Resistance': 0.00049, 'Reactance': 0.03162, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 485.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2951'},
        {'fromNode': '7373.0', 'toNode': '7366.0', 'Resistance': 0.0001, 'Reactance': 0.01184, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 491.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2952'},
        {'fromNode': '7374.0', 'toNode': '7366.0', 'Resistance': 0.0004, 'Reactance': 0.0255, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 487.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2953'},
        {'fromNode': '7375.0', 'toNode': '7366.0', 'Resistance': 0.00045, 'Reactance': 0.02708, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 712.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2954'},
        {'fromNode': '7376.0', 'toNode': '7366.0', 'Resistance': 0.00035, 'Reactance': 0.02498, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 713.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2955'},
        {'fromNode': '7377.0', 'toNode': '7366.0', 'Resistance': 0.00031, 'Reactance': 0.01898, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 705.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2956'},
        {'fromNode': '7378.0', 'toNode': '7366.0', 'Resistance': 0.00036, 'Reactance': 0.02384, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 694.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2957'},
        {'fromNode': '7379.0', 'toNode': '7366.0', 'Resistance': 0.00031, 'Reactance': 0.02202, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 700.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2958'},
        {'fromNode': '7366.0', 'toNode': '7400.0', 'Resistance': 0.00014, 'Reactance': 0.00168, 'Conductance': 0, 'Susceptance': 0.11182, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2959'},
        {'fromNode': '7368.0', 'toNode': '7367.0', 'Resistance': 0.00086, 'Reactance': 0.03726, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 392.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2960'},
        {'fromNode': '7368.0', 'toNode': '7367.0', 'Resistance': 0.00086, 'Reactance': 0.03726, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 392.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2961'},
        {'fromNode': '7368.0', 'toNode': '7367.0', 'Resistance': 0.00086, 'Reactance': 0.03726, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 392.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2962'},
        {'fromNode': '7385.0', 'toNode': '7404.0', 'Resistance': 0.00674, 'Reactance': 0.03846, 'Conductance': 0, 'Susceptance': 0.00578, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2963'},
        {'fromNode': '7390.0', 'toNode': '7389.0', 'Resistance': 0.00019, 'Reactance': 0.01131, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1278.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2964'},
        {'fromNode': '8030.0', 'toNode': '7389.0', 'Resistance': 0.00054, 'Reactance': 0.00651, 'Conductance': 0, 'Susceptance': 0.43275, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2965'},
        {'fromNode': '7391.0', 'toNode': '7390.0', 'Resistance': 0.00148, 'Reactance': 0.044, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2966'},
        {'fromNode': '7396.0', 'toNode': '7410.0', 'Resistance': 0.00293, 'Reactance': 0.01892, 'Conductance': 0, 'Susceptance': 0.00361, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2967'},
        {'fromNode': '7396.0', 'toNode': '7410.0', 'Resistance': 0.00293, 'Reactance': 0.01892, 'Conductance': 0, 'Susceptance': 0.00361, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2968'},
        {'fromNode': '7396.0', 'toNode': '7410.0', 'Resistance': 0.00293, 'Reactance': 0.01892, 'Conductance': 0, 'Susceptance': 0.00361, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2969'},
        {'fromNode': '7403.0', 'toNode': '7398.0', 'Resistance': 0.00218, 'Reactance': 0.01408, 'Conductance': 0, 'Susceptance': 0.00269, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2970'},
        {'fromNode': '7403.0', 'toNode': '7398.0', 'Resistance': 0.00218, 'Reactance': 0.01408, 'Conductance': 0, 'Susceptance': 0.00269, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2971'},
        {'fromNode': '7399.0', 'toNode': '7417.0', 'Resistance': 0.00504, 'Reactance': 0.03508, 'Conductance': 0, 'Susceptance': 0.00573, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2972'},
        {'fromNode': '7399.0', 'toNode': '7417.0', 'Resistance': 0.00504, 'Reactance': 0.03508, 'Conductance': 0, 'Susceptance': 0.00573, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2973'},
        {'fromNode': '7401.0', 'toNode': '7400.0', 'Resistance': 0.00011, 'Reactance': 0.00786, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1297.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2974'},
        {'fromNode': '7402.0', 'toNode': '7401.0', 'Resistance': 0.00049, 'Reactance': 0.03223, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 423.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2975'},
        {'fromNode': '7401.0', 'toNode': '7409.0', 'Resistance': 0.00483, 'Reactance': 0.04717, 'Conductance': 0, 'Susceptance': 0.09075, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2976'},
        {'fromNode': '7403.0', 'toNode': '7419.0', 'Resistance': 0.00504, 'Reactance': 0.03445, 'Conductance': 0, 'Susceptance': 0.00586, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2977'},
        {'fromNode': '7403.0', 'toNode': '7419.0', 'Resistance': 0.00504, 'Reactance': 0.03445, 'Conductance': 0, 'Susceptance': 0.00586, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2978'},
        {'fromNode': '7407.0', 'toNode': '7406.0', 'Resistance': 0.0002, 'Reactance': 0.01441, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1026.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2979'},
        {'fromNode': '7414.0', 'toNode': '7406.0', 'Resistance': 0.00034, 'Reactance': 0.0041, 'Conductance': 0, 'Susceptance': 0.275, 'MVA_rating': 2295.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '2980'},
        {'fromNode': '7408.0', 'toNode': '7407.0', 'Resistance': 0.00118, 'Reactance': 0.08057, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2981'},
        {'fromNode': '7410.0', 'toNode': '7409.0', 'Resistance': 0.00054, 'Reactance': 0.03923, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 372.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2982'},
        {'fromNode': '7410.0', 'toNode': '7409.0', 'Resistance': 0.00054, 'Reactance': 0.03923, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 372.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2983'},
        {'fromNode': '7410.0', 'toNode': '7409.0', 'Resistance': 0.00054, 'Reactance': 0.03923, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 372.6, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2984'},
        {'fromNode': '7409.0', 'toNode': '7420.0', 'Resistance': 0.00502, 'Reactance': 0.04904, 'Conductance': 0, 'Susceptance': 0.09435, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '2985'},
        {'fromNode': '7415.0', 'toNode': '7414.0', 'Resistance': 0.0004, 'Reactance': 0.04372, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 239.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2986'},
        {'fromNode': '7416.0', 'toNode': '7415.0', 'Resistance': 0.00128, 'Reactance': 0.09703, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 109.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2987'},
        {'fromNode': '7417.0', 'toNode': '7418.0', 'Resistance': 0.00278, 'Reactance': 0.01937, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2988'},
        {'fromNode': '7417.0', 'toNode': '7418.0', 'Resistance': 0.00278, 'Reactance': 0.01937, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2989'},
        {'fromNode': '7421.0', 'toNode': '7420.0', 'Resistance': 0.00106, 'Reactance': 0.06841, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 242.1, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2990'},
        {'fromNode': '7421.0', 'toNode': '7420.0', 'Resistance': 0.00106, 'Reactance': 0.06841, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 242.1, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2991'},
        {'fromNode': '7423.0', 'toNode': '7422.0', 'Resistance': 0.00014, 'Reactance': 0.00897, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1350.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2992'},
        {'fromNode': '7424.0', 'toNode': '7423.0', 'Resistance': 0.00105, 'Reactance': 0.04353, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 238.56, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2993'},
        {'fromNode': '7424.0', 'toNode': '7423.0', 'Resistance': 0.00105, 'Reactance': 0.04353, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 238.56, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2994'},
        {'fromNode': '7424.0', 'toNode': '7423.0', 'Resistance': 0.00105, 'Reactance': 0.04353, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 238.56, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2995'},
        {'fromNode': '7429.0', 'toNode': '7428.0', 'Resistance': 0.00026, 'Reactance': 0.01258, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 927.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2996'},
        {'fromNode': '7430.0', 'toNode': '7429.0', 'Resistance': 0.00214, 'Reactance': 0.08887, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 133.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '2997'},
        {'fromNode': '7432.0', 'toNode': '8003.0', 'Resistance': 0.02966, 'Reactance': 0.07251, 'Conductance': 0, 'Susceptance': 0.00949, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2998'},
        {'fromNode': '7432.0', 'toNode': '8037.0', 'Resistance': 0.0136, 'Reactance': 0.05948, 'Conductance': 0, 'Susceptance': 0.00906, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2999'},
        {'fromNode': '7432.0', 'toNode': '8141.0', 'Resistance': 0.03471, 'Reactance': 0.2084, 'Conductance': 0, 'Susceptance': 0.02828, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3000'},
        {'fromNode': '8003.0', 'toNode': '8001.0', 'Resistance': 0.01464, 'Reactance': 0.06073, 'Conductance': 0, 'Susceptance': 0.01031, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3001'},
        {'fromNode': '8010.0', 'toNode': '8001.0', 'Resistance': 0.01076, 'Reactance': 0.04467, 'Conductance': 0, 'Susceptance': 0.00758, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3002'},
        {'fromNode': '8028.0', 'toNode': '8002.0', 'Resistance': 0.01939, 'Reactance': 0.04659, 'Conductance': 0, 'Susceptance': 0.00632, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3003'},
        {'fromNode': '8002.0', 'toNode': '8100.0', 'Resistance': 0.00588, 'Reactance': 0.03355, 'Conductance': 0, 'Susceptance': 0.00504, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3004'},
        {'fromNode': '8005.0', 'toNode': '8004.0', 'Resistance': 0.00029, 'Reactance': 0.01244, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3005'},
        {'fromNode': '8005.0', 'toNode': '8004.0', 'Resistance': 0.00029, 'Reactance': 0.01244, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3006'},
        {'fromNode': '8097.0', 'toNode': '8004.0', 'Resistance': 0.0073, 'Reactance': 0.05215, 'Conductance': 0, 'Susceptance': 0.09507, 'MVA_rating': 443.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3007'},
        {'fromNode': '8005.0', 'toNode': '8023.0', 'Resistance': 0.00379, 'Reactance': 0.02588, 'Conductance': 0, 'Susceptance': 0.0044, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3008'},
        {'fromNode': '8005.0', 'toNode': '8023.0', 'Resistance': 0.00379, 'Reactance': 0.02588, 'Conductance': 0, 'Susceptance': 0.0044, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3009'},
        {'fromNode': '8137.0', 'toNode': '8005.0', 'Resistance': 0.02182, 'Reactance': 0.09544, 'Conductance': 0, 'Susceptance': 0.01454, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3010'},
        {'fromNode': '8006.0', 'toNode': '8007.0', 'Resistance': 0.00768, 'Reactance': 0.04613, 'Conductance': 0, 'Susceptance': 0.00626, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3011'},
        {'fromNode': '8006.0', 'toNode': '8134.0', 'Resistance': 0.00667, 'Reactance': 0.02973, 'Conductance': 0, 'Susceptance': 0.00436, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3012'},
        {'fromNode': '8007.0', 'toNode': '8017.0', 'Resistance': 0.00629, 'Reactance': 0.03774, 'Conductance': 0, 'Susceptance': 0.00512, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3013'},
        {'fromNode': '8008.0', 'toNode': '8053.0', 'Resistance': 0.01567, 'Reactance': 0.09407, 'Conductance': 0, 'Susceptance': 0.01277, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3014'},
        {'fromNode': '8081.0', 'toNode': '8008.0', 'Resistance': 0.02514, 'Reactance': 0.06143, 'Conductance': 0, 'Susceptance': 0.00804, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3015'},
        {'fromNode': '8009.0', 'toNode': '8104.0', 'Resistance': 0.01187, 'Reactance': 0.06777, 'Conductance': 0, 'Susceptance': 0.01018, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3016'},
        {'fromNode': '8086.0', 'toNode': '8010.0', 'Resistance': 0.0146, 'Reactance': 0.0834, 'Conductance': 0, 'Susceptance': 0.01253, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3017'},
        {'fromNode': '8010.0', 'toNode': '8103.0', 'Resistance': 0.00735, 'Reactance': 0.04411, 'Conductance': 0, 'Susceptance': 0.00599, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3018'},
        {'fromNode': '8045.0', 'toNode': '8011.0', 'Resistance': 0.00606, 'Reactance': 0.03914, 'Conductance': 0, 'Susceptance': 0.00747, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3019'},
        {'fromNode': '8045.0', 'toNode': '8011.0', 'Resistance': 0.00606, 'Reactance': 0.03914, 'Conductance': 0, 'Susceptance': 0.00747, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3020'},
        {'fromNode': '8072.0', 'toNode': '8011.0', 'Resistance': 0.00722, 'Reactance': 0.05031, 'Conductance': 0, 'Susceptance': 0.00822, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3021'},
        {'fromNode': '8012.0', 'toNode': '8032.0', 'Resistance': 0.00861, 'Reactance': 0.03768, 'Conductance': 0, 'Susceptance': 0.00574, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3022'},
        {'fromNode': '8104.0', 'toNode': '8012.0', 'Resistance': 0.01361, 'Reactance': 0.05953, 'Conductance': 0, 'Susceptance': 0.00907, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3023'},
        {'fromNode': '8026.0', 'toNode': '8013.0', 'Resistance': 0.01793, 'Reactance': 0.10765, 'Conductance': 0, 'Susceptance': 0.01461, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3024'},
        {'fromNode': '8013.0', 'toNode': '8039.0', 'Resistance': 0.02164, 'Reactance': 0.12994, 'Conductance': 0, 'Susceptance': 0.01763, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3025'},
        {'fromNode': '8013.0', 'toNode': '8136.0', 'Resistance': 0.01354, 'Reactance': 0.05923, 'Conductance': 0, 'Susceptance': 0.00902, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3026'},
        {'fromNode': '8014.0', 'toNode': '8037.0', 'Resistance': 0.01899, 'Reactance': 0.08308, 'Conductance': 0, 'Susceptance': 0.01265, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3027'},
        {'fromNode': '8118.0', 'toNode': '8014.0', 'Resistance': 0.01481, 'Reactance': 0.08893, 'Conductance': 0, 'Susceptance': 0.01207, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3028'},
        {'fromNode': '8111.0', 'toNode': '8015.0', 'Resistance': 0.02139, 'Reactance': 0.05228, 'Conductance': 0, 'Susceptance': 0.00685, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3029'},
        {'fromNode': '8015.0', 'toNode': '8150.0', 'Resistance': 0.00949, 'Reactance': 0.05422, 'Conductance': 0, 'Susceptance': 0.00815, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3030'},
        {'fromNode': '8016.0', 'toNode': '8122.0', 'Resistance': 0.03388, 'Reactance': 0.08282, 'Conductance': 0, 'Susceptance': 0.01084, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3031'},
        {'fromNode': '8016.0', 'toNode': '8152.0', 'Resistance': 0.00375, 'Reactance': 0.01641, 'Conductance': 0, 'Susceptance': 0.0025, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3032'},
        {'fromNode': '8017.0', 'toNode': '8040.0', 'Resistance': 0.01458, 'Reactance': 0.08752, 'Conductance': 0, 'Susceptance': 0.01188, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3033'},
        {'fromNode': '8019.0', 'toNode': '8018.0', 'Resistance': 0.00115, 'Reactance': 0.03658, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 378.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3034'},
        {'fromNode': '8019.0', 'toNode': '8018.0', 'Resistance': 0.00115, 'Reactance': 0.03658, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 378.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3035'},
        {'fromNode': '8127.0', 'toNode': '8018.0', 'Resistance': 0.00839, 'Reactance': 0.08205, 'Conductance': 0, 'Susceptance': 0.15788, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3036'},
        {'fromNode': '8018.0', 'toNode': '8133.0', 'Resistance': 0.00093, 'Reactance': 0.00905, 'Conductance': 0, 'Susceptance': 0.01742, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3037'},
        {'fromNode': '8019.0', 'toNode': '8052.0', 'Resistance': 0.00643, 'Reactance': 0.02664, 'Conductance': 0, 'Susceptance': 0.00452, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3038'},
        {'fromNode': '8019.0', 'toNode': '8100.0', 'Resistance': 0.0068, 'Reactance': 0.043, 'Conductance': 0, 'Susceptance': 0.0058, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3039'},
        {'fromNode': '8019.0', 'toNode': '8135.0', 'Resistance': 0.00212, 'Reactance': 0.0148, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3040'},
        {'fromNode': '8019.0', 'toNode': '8135.0', 'Resistance': 0.00212, 'Reactance': 0.0148, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3041'},
        {'fromNode': '8020.0', 'toNode': '8029.0', 'Resistance': 0.01833, 'Reactance': 0.0817, 'Conductance': 0, 'Susceptance': 0.01198, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3042'},
        {'fromNode': '8020.0', 'toNode': '8040.0', 'Resistance': 0.02253, 'Reactance': 0.09344, 'Conductance': 0, 'Susceptance': 0.01586, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3043'},
        {'fromNode': '8020.0', 'toNode': '8153.0', 'Resistance': 0.01361, 'Reactance': 0.08173, 'Conductance': 0, 'Susceptance': 0.01109, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3044'},
        {'fromNode': '8020.0', 'toNode': '8154.0', 'Resistance': 0.04852, 'Reactance': 0.1186, 'Conductance': 0, 'Susceptance': 0.01553, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3045'},
        {'fromNode': '8022.0', 'toNode': '8021.0', 'Resistance': 0.06068, 'Reactance': 0.14832, 'Conductance': 0, 'Susceptance': 0.01942, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3046'},
        {'fromNode': '8021.0', 'toNode': '8041.0', 'Resistance': 0.01109, 'Reactance': 0.0485, 'Conductance': 0, 'Susceptance': 0.00739, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3047'},
        {'fromNode': '8021.0', 'toNode': '8119.0', 'Resistance': 0.01573, 'Reactance': 0.06526, 'Conductance': 0, 'Susceptance': 0.01107, 'MVA_rating': 156.45, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3048'},
        {'fromNode': '8022.0', 'toNode': '8150.0', 'Resistance': 0.05259, 'Reactance': 0.12855, 'Conductance': 0, 'Susceptance': 0.01683, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3049'},
        {'fromNode': '8024.0', 'toNode': '8023.0', 'Resistance': 0.01105, 'Reactance': 0.18799, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 47.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3050'},
        {'fromNode': '8023.0', 'toNode': '8111.0', 'Resistance': 0.00548, 'Reactance': 0.03542, 'Conductance': 0, 'Susceptance': 0.00676, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3051'},
        {'fromNode': '8023.0', 'toNode': '8111.0', 'Resistance': 0.00548, 'Reactance': 0.03542, 'Conductance': 0, 'Susceptance': 0.00676, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3052'},
        {'fromNode': '8025.0', 'toNode': '8055.0', 'Resistance': 0.04211, 'Reactance': 0.10294, 'Conductance': 0, 'Susceptance': 0.01348, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3053'},
        {'fromNode': '8025.0', 'toNode': '8121.0', 'Resistance': 0.03375, 'Reactance': 0.08251, 'Conductance': 0, 'Susceptance': 0.0108, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3054'},
        {'fromNode': '8025.0', 'toNode': '8151.0', 'Resistance': 0.02868, 'Reactance': 0.1722, 'Conductance': 0, 'Susceptance': 0.02337, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3055'},
        {'fromNode': '8026.0', 'toNode': '8154.0', 'Resistance': 0.02053, 'Reactance': 0.05018, 'Conductance': 0, 'Susceptance': 0.00657, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3056'},
        {'fromNode': '8100.0', 'toNode': '8027.0', 'Resistance': 0.00732, 'Reactance': 0.03203, 'Conductance': 0, 'Susceptance': 0.00488, 'MVA_rating': 223.5, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3057'},
        {'fromNode': '8027.0', 'toNode': '8134.0', 'Resistance': 0.00435, 'Reactance': 0.02974, 'Conductance': 0, 'Susceptance': 0.00506, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3058'},
        {'fromNode': '8146.0', 'toNode': '8028.0', 'Resistance': 0.02838, 'Reactance': 0.06936, 'Conductance': 0, 'Susceptance': 0.00908, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3059'},
        {'fromNode': '8073.0', 'toNode': '8029.0', 'Resistance': 0.02002, 'Reactance': 0.08756, 'Conductance': 0, 'Susceptance': 0.01333, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3060'},
        {'fromNode': '8031.0', 'toNode': '8030.0', 'Resistance': 7e-05, 'Reactance': 0.00828, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1036.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3061'},
        {'fromNode': '8033.0', 'toNode': '8030.0', 'Resistance': 0.00029, 'Reactance': 0.02168, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 826.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3062'},
        {'fromNode': '8034.0', 'toNode': '8030.0', 'Resistance': 0.00021, 'Reactance': 0.01721, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 818.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3063'},
        {'fromNode': '8035.0', 'toNode': '8030.0', 'Resistance': 0.00021, 'Reactance': 0.01536, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 830.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3064'},
        {'fromNode': '8036.0', 'toNode': '8030.0', 'Resistance': 0.00021, 'Reactance': 0.01615, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 820.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3065'},
        {'fromNode': '8158.0', 'toNode': '8030.0', 'Resistance': 0.00069, 'Reactance': 0.0084, 'Conductance': 0, 'Susceptance': 0.55816, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '3066'},
        {'fromNode': '8032.0', 'toNode': '8031.0', 'Resistance': 0.00099, 'Reactance': 0.03425, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3067'},
        {'fromNode': '8143.0', 'toNode': '8032.0', 'Resistance': 0.017, 'Reactance': 0.07576, 'Conductance': 0, 'Susceptance': 0.01111, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3068'},
        {'fromNode': '8038.0', 'toNode': '8066.0', 'Resistance': 0.01227, 'Reactance': 0.05467, 'Conductance': 0, 'Susceptance': 0.00802, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3069'},
        {'fromNode': '8038.0', 'toNode': '8124.0', 'Resistance': 0.01143, 'Reactance': 0.06861, 'Conductance': 0, 'Susceptance': 0.00931, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3070'},
        {'fromNode': '8039.0', 'toNode': '8081.0', 'Resistance': 0.01342, 'Reactance': 0.07667, 'Conductance': 0, 'Susceptance': 0.01152, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3071'},
        {'fromNode': '8040.0', 'toNode': '8072.0', 'Resistance': 0.02142, 'Reactance': 0.09547, 'Conductance': 0, 'Susceptance': 0.014, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3072'},
        {'fromNode': '8109.0', 'toNode': '8040.0', 'Resistance': 0.015, 'Reactance': 0.088, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3073'},
        {'fromNode': '8041.0', 'toNode': '8063.0', 'Resistance': 0.02127, 'Reactance': 0.05109, 'Conductance': 0, 'Susceptance': 0.00693, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3074'},
        {'fromNode': '8111.0', 'toNode': '8042.0', 'Resistance': 0.02452, 'Reactance': 0.05992, 'Conductance': 0, 'Susceptance': 0.00785, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3075'},
        {'fromNode': '8118.0', 'toNode': '8042.0', 'Resistance': 0.02735, 'Reactance': 0.16423, 'Conductance': 0, 'Susceptance': 0.02229, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3076'},
        {'fromNode': '8044.0', 'toNode': '8043.0', 'Resistance': 9e-05, 'Reactance': 0.00881, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1072.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3077'},
        {'fromNode': '8046.0', 'toNode': '8043.0', 'Resistance': 0.00013, 'Reactance': 0.01356, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 606.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3078'},
        {'fromNode': '8047.0', 'toNode': '8043.0', 'Resistance': 0.00021, 'Reactance': 0.01156, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 613.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3079'},
        {'fromNode': '8048.0', 'toNode': '8043.0', 'Resistance': 0.00055, 'Reactance': 0.02691, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 598.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3080'},
        {'fromNode': '8049.0', 'toNode': '8043.0', 'Resistance': 0.00017, 'Reactance': 0.00903, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 608.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3081'},
        {'fromNode': '8050.0', 'toNode': '8043.0', 'Resistance': 0.00032, 'Reactance': 0.0182, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 618.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3082'},
        {'fromNode': '8051.0', 'toNode': '8043.0', 'Resistance': 0.00013, 'Reactance': 0.00831, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 608.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3083'},
        {'fromNode': '8043.0', 'toNode': '8067.0', 'Resistance': 0.0004, 'Reactance': 0.00819, 'Conductance': 0, 'Susceptance': 0.57108, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '3084'},
        {'fromNode': '8045.0', 'toNode': '8044.0', 'Resistance': 0.00042, 'Reactance': 0.02402, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 388.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3085'},
        {'fromNode': '8147.0', 'toNode': '8044.0', 'Resistance': 0.00245, 'Reactance': 0.02399, 'Conductance': 0, 'Susceptance': 0.04616, 'MVA_rating': 647.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3086'},
        {'fromNode': '8045.0', 'toNode': '8091.0', 'Resistance': 0.02272, 'Reactance': 0.09941, 'Conductance': 0, 'Susceptance': 0.01514, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3087'},
        {'fromNode': '8045.0', 'toNode': '8144.0', 'Resistance': 0.01434, 'Reactance': 0.08609, 'Conductance': 0, 'Susceptance': 0.01168, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3088'},
        {'fromNode': '8081.0', 'toNode': '8052.0', 'Resistance': 0.01891, 'Reactance': 0.04622, 'Conductance': 0, 'Susceptance': 0.00605, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3089'},
        {'fromNode': '8053.0', 'toNode': '8065.0', 'Resistance': 0.0229, 'Reactance': 0.10022, 'Conductance': 0, 'Susceptance': 0.01526, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3090'},
        {'fromNode': '8053.0', 'toNode': '8153.0', 'Resistance': 0.01818, 'Reactance': 0.10383, 'Conductance': 0, 'Susceptance': 0.0156, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3091'},
        {'fromNode': '8054.0', 'toNode': '8092.0', 'Resistance': 0.00962, 'Reactance': 0.05775, 'Conductance': 0, 'Susceptance': 0.00784, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3092'},
        {'fromNode': '8132.0', 'toNode': '8054.0', 'Resistance': 0.01211, 'Reactance': 0.06914, 'Conductance': 0, 'Susceptance': 0.01039, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3093'},
        {'fromNode': '8055.0', 'toNode': '8098.0', 'Resistance': 0.011, 'Reactance': 0.078, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 220.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3094'},
        {'fromNode': '8121.0', 'toNode': '8055.0', 'Resistance': 0.02807, 'Reactance': 0.06863, 'Conductance': 0, 'Susceptance': 0.00899, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3095'},
        {'fromNode': '8056.0', 'toNode': '8089.0', 'Resistance': 0.0071, 'Reactance': 0.04055, 'Conductance': 0, 'Susceptance': 0.00609, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3096'},
        {'fromNode': '8056.0', 'toNode': '8138.0', 'Resistance': 0.00777, 'Reactance': 0.04664, 'Conductance': 0, 'Susceptance': 0.00633, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3097'},
        {'fromNode': '8058.0', 'toNode': '8057.0', 'Resistance': 0.00019, 'Reactance': 0.02064, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 491.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3098'},
        {'fromNode': '8060.0', 'toNode': '8057.0', 'Resistance': 0.00013, 'Reactance': 0.00977, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1157.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3099'},
        {'fromNode': '8061.0', 'toNode': '8057.0', 'Resistance': 0.00019, 'Reactance': 0.01179, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1152.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3100'},
        {'fromNode': '8057.0', 'toNode': '8112.0', 'Resistance': 0.0013, 'Reactance': 0.01581, 'Conductance': 0, 'Susceptance': 1.05006, 'MVA_rating': 2020.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '3101'},
        {'fromNode': '8059.0', 'toNode': '8058.0', 'Resistance': 0.00191, 'Reactance': 0.06531, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3102'},
        {'fromNode': '8058.0', 'toNode': '8068.0', 'Resistance': 0.00454, 'Reactance': 0.04438, 'Conductance': 0, 'Susceptance': 0.0854, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3103'},
        {'fromNode': '8058.0', 'toNode': '8147.0', 'Resistance': 0.00321, 'Reactance': 0.03142, 'Conductance': 0, 'Susceptance': 0.06046, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3104'},
        {'fromNode': '8064.0', 'toNode': '8059.0', 'Resistance': 0.00422, 'Reactance': 0.01881, 'Conductance': 0, 'Susceptance': 0.00276, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3105'},
        {'fromNode': '8062.0', 'toNode': '8098.0', 'Resistance': 0.00445, 'Reactance': 0.01944, 'Conductance': 0, 'Susceptance': 0.00296, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3106'},
        {'fromNode': '8119.0', 'toNode': '8062.0', 'Resistance': 0.01516, 'Reactance': 0.06756, 'Conductance': 0, 'Susceptance': 0.00991, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3107'},
        {'fromNode': '8101.0', 'toNode': '8063.0', 'Resistance': 0.01764, 'Reactance': 0.10595, 'Conductance': 0, 'Susceptance': 0.01438, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3108'},
        {'fromNode': '8063.0', 'toNode': '8125.0', 'Resistance': 0.03529, 'Reactance': 0.08625, 'Conductance': 0, 'Susceptance': 0.01129, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3109'},
        {'fromNode': '8144.0', 'toNode': '8064.0', 'Resistance': 0.01683, 'Reactance': 0.06982, 'Conductance': 0, 'Susceptance': 0.01185, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3110'},
        {'fromNode': '8065.0', 'toNode': '8149.0', 'Resistance': 0.00226, 'Reactance': 0.01459, 'Conductance': 0, 'Susceptance': 0.00278, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3111'},
        {'fromNode': '8065.0', 'toNode': '8149.0', 'Resistance': 0.00226, 'Reactance': 0.01459, 'Conductance': 0, 'Susceptance': 0.00278, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3112'},
        {'fromNode': '8066.0', 'toNode': '8093.0', 'Resistance': 0.04293, 'Reactance': 0.19131, 'Conductance': 0, 'Susceptance': 0.02805, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3113'},
        {'fromNode': '8142.0', 'toNode': '8066.0', 'Resistance': 0.01064, 'Reactance': 0.0474, 'Conductance': 0, 'Susceptance': 0.00695, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3114'},
        {'fromNode': '8068.0', 'toNode': '8067.0', 'Resistance': 0.00035, 'Reactance': 0.03434, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 285.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3115'},
        {'fromNode': '8070.0', 'toNode': '8067.0', 'Resistance': 0.00016, 'Reactance': 0.01452, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1325.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3116'},
        {'fromNode': '8071.0', 'toNode': '8067.0', 'Resistance': 0.00014, 'Reactance': 0.01049, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1314.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3117'},
        {'fromNode': '8069.0', 'toNode': '8068.0', 'Resistance': 0.00074, 'Reactance': 0.05407, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 143.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3118'},
        {'fromNode': '8068.0', 'toNode': '8083.0', 'Resistance': 0.00329, 'Reactance': 0.03216, 'Conductance': 0, 'Susceptance': 0.06187, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3119'},
        {'fromNode': '8103.0', 'toNode': '8069.0', 'Resistance': 0.01048, 'Reactance': 0.04673, 'Conductance': 0, 'Susceptance': 0.00685, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3120'},
        {'fromNode': '8073.0', 'toNode': '8108.0', 'Resistance': 0.01751, 'Reactance': 0.10003, 'Conductance': 0, 'Susceptance': 0.01503, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3121'},
        {'fromNode': '8136.0', 'toNode': '8073.0', 'Resistance': 0.015, 'Reactance': 0.088, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3122'},
        {'fromNode': '8075.0', 'toNode': '8074.0', 'Resistance': 0.00028, 'Reactance': 0.02981, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 550.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3123'},
        {'fromNode': '8075.0', 'toNode': '8074.0', 'Resistance': 0.00028, 'Reactance': 0.02981, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 550.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3124'},
        {'fromNode': '8077.0', 'toNode': '8074.0', 'Resistance': 0.00016, 'Reactance': 0.01864, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 806.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3125'},
        {'fromNode': '8078.0', 'toNode': '8074.0', 'Resistance': 0.00029, 'Reactance': 0.01964, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 818.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3126'},
        {'fromNode': '8079.0', 'toNode': '8074.0', 'Resistance': 0.0002, 'Reactance': 0.0153, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 818.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3127'},
        {'fromNode': '8080.0', 'toNode': '8074.0', 'Resistance': 0.00022, 'Reactance': 0.01762, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 814.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3128'},
        {'fromNode': '8076.0', 'toNode': '8075.0', 'Resistance': 0.00099, 'Reactance': 0.04033, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3129'},
        {'fromNode': '8075.0', 'toNode': '8097.0', 'Resistance': 0.00445, 'Reactance': 0.03181, 'Conductance': 0, 'Susceptance': 0.05799, 'MVA_rating': 740.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3130'},
        {'fromNode': '8119.0', 'toNode': '8076.0', 'Resistance': 0.00591, 'Reactance': 0.0412, 'Conductance': 0, 'Susceptance': 0.00674, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3131'},
        {'fromNode': '8119.0', 'toNode': '8076.0', 'Resistance': 0.00591, 'Reactance': 0.0412, 'Conductance': 0, 'Susceptance': 0.00674, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3132'},
        {'fromNode': '8083.0', 'toNode': '8082.0', 'Resistance': 0.00031, 'Reactance': 0.01574, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1134.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3133'},
        {'fromNode': '8087.0', 'toNode': '8082.0', 'Resistance': 8e-05, 'Reactance': 0.00888, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1315.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3134'},
        {'fromNode': '8088.0', 'toNode': '8082.0', 'Resistance': 8e-05, 'Reactance': 0.00591, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1307.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3135'},
        {'fromNode': '8155.0', 'toNode': '8082.0', 'Resistance': 0.00016, 'Reactance': 0.00337, 'Conductance': 0, 'Susceptance': 0.23512, 'MVA_rating': 2768.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '3136'},
        {'fromNode': '8084.0', 'toNode': '8083.0', 'Resistance': 0.00236, 'Reactance': 0.06305, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 184.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3137'},
        {'fromNode': '8085.0', 'toNode': '8084.0', 'Resistance': 0.00092, 'Reactance': 0.02498, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 231.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3138'},
        {'fromNode': '8086.0', 'toNode': '8085.0', 'Resistance': 0.00184, 'Reactance': 0.06658, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 205.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3139'},
        {'fromNode': '8089.0', 'toNode': '8135.0', 'Resistance': 0.00521, 'Reactance': 0.03628, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3140'},
        {'fromNode': '8089.0', 'toNode': '8135.0', 'Resistance': 0.00521, 'Reactance': 0.03628, 'Conductance': 0, 'Susceptance': 0.00593, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3141'},
        {'fromNode': '8090.0', 'toNode': '8092.0', 'Resistance': 0.01569, 'Reactance': 0.08961, 'Conductance': 0, 'Susceptance': 0.01347, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3142'},
        {'fromNode': '8151.0', 'toNode': '8090.0', 'Resistance': 0.01339, 'Reactance': 0.08041, 'Conductance': 0, 'Susceptance': 0.01091, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3143'},
        {'fromNode': '8091.0', 'toNode': '8109.0', 'Resistance': 0.07888, 'Reactance': 0.19284, 'Conductance': 0, 'Susceptance': 0.02525, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3144'},
        {'fromNode': '8106.0', 'toNode': '8093.0', 'Resistance': 0.018, 'Reactance': 0.078, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3145'},
        {'fromNode': '8093.0', 'toNode': '8128.0', 'Resistance': 0.01795, 'Reactance': 0.07854, 'Conductance': 0, 'Susceptance': 0.01196, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3146'},
        {'fromNode': '8095.0', 'toNode': '8094.0', 'Resistance': 0.00041, 'Reactance': 0.03017, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 317.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3147'},
        {'fromNode': '8094.0', 'toNode': '8139.0', 'Resistance': 0.00069, 'Reactance': 0.00679, 'Conductance': 0, 'Susceptance': 0.01307, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3148'},
        {'fromNode': '8094.0', 'toNode': '8159.0', 'Resistance': 0.00239, 'Reactance': 0.02336, 'Conductance': 0, 'Susceptance': 0.04495, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3149'},
        {'fromNode': '8095.0', 'toNode': '8140.0', 'Resistance': 0.01066, 'Reactance': 0.02604, 'Conductance': 0, 'Susceptance': 0.00341, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3150'},
        {'fromNode': '8096.0', 'toNode': '8110.0', 'Resistance': 0.0448, 'Reactance': 0.10762, 'Conductance': 0, 'Susceptance': 0.0146, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3151'},
        {'fromNode': '8096.0', 'toNode': '8153.0', 'Resistance': 0.0128, 'Reactance': 0.07686, 'Conductance': 0, 'Susceptance': 0.01043, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3152'},
        {'fromNode': '8098.0', 'toNode': '8097.0', 'Resistance': 0.00373, 'Reactance': 0.10051, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 175.5, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3153'},
        {'fromNode': '8099.0', 'toNode': '8097.0', 'Resistance': 0.00303, 'Reactance': 0.097, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 184.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3154'},
        {'fromNode': '8097.0', 'toNode': '8113.0', 'Resistance': 0.00194, 'Reactance': 0.01386, 'Conductance': 0, 'Susceptance': 0.02527, 'MVA_rating': 800.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3155'},
        {'fromNode': '8138.0', 'toNode': '8100.0', 'Resistance': 0.01582, 'Reactance': 0.03801, 'Conductance': 0, 'Susceptance': 0.00516, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3156'},
        {'fromNode': '8101.0', 'toNode': '8137.0', 'Resistance': 0.01472, 'Reactance': 0.08841, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3157'},
        {'fromNode': '8102.0', 'toNode': '8157.0', 'Resistance': 0.00742, 'Reactance': 0.03309, 'Conductance': 0, 'Susceptance': 0.00485, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3158'},
        {'fromNode': '8106.0', 'toNode': '8105.0', 'Resistance': 0.00098, 'Reactance': 0.05869, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 212.8, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3159'},
        {'fromNode': '8105.0', 'toNode': '8127.0', 'Resistance': 0.00276, 'Reactance': 0.02696, 'Conductance': 0, 'Susceptance': 0.05188, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3160'},
        {'fromNode': '8142.0', 'toNode': '8106.0', 'Resistance': 0.01907, 'Reactance': 0.08501, 'Conductance': 0, 'Susceptance': 0.01247, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3161'},
        {'fromNode': '8128.0', 'toNode': '8107.0', 'Resistance': 0.01223, 'Reactance': 0.05352, 'Conductance': 0, 'Susceptance': 0.00815, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3162'},
        {'fromNode': '8145.0', 'toNode': '8107.0', 'Resistance': 0.01849, 'Reactance': 0.08086, 'Conductance': 0, 'Susceptance': 0.01231, 'MVA_rating': 250.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3163'},
        {'fromNode': '8153.0', 'toNode': '8108.0', 'Resistance': 0.013, 'Reactance': 0.054, 'Conductance': 0, 'Susceptance': 0.027, 'MVA_rating': 186.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3164'},
        {'fromNode': '8109.0', 'toNode': '8123.0', 'Resistance': 0.00627, 'Reactance': 0.0358, 'Conductance': 0, 'Susceptance': 0.00538, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3165'},
        {'fromNode': '8124.0', 'toNode': '8110.0', 'Resistance': 0.00917, 'Reactance': 0.05504, 'Conductance': 0, 'Susceptance': 0.00747, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3166'},
        {'fromNode': '8111.0', 'toNode': '8125.0', 'Resistance': 0.01474, 'Reactance': 0.0885, 'Conductance': 0, 'Susceptance': 0.01201, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3167'},
        {'fromNode': '8113.0', 'toNode': '8112.0', 'Resistance': 9e-05, 'Reactance': 0.00481, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1290.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3168'},
        {'fromNode': '8115.0', 'toNode': '8112.0', 'Resistance': 0.00071, 'Reactance': 0.04514, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 273.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3169'},
        {'fromNode': '8116.0', 'toNode': '8112.0', 'Resistance': 0.00024, 'Reactance': 0.01298, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1026.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3170'},
        {'fromNode': '8117.0', 'toNode': '8112.0', 'Resistance': 0.00025, 'Reactance': 0.01118, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1034.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3171'},
        {'fromNode': '8114.0', 'toNode': '8113.0', 'Resistance': 0.00077, 'Reactance': 0.03155, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3172'},
        {'fromNode': '8113.0', 'toNode': '8133.0', 'Resistance': 0.003, 'Reactance': 0.0312, 'Conductance': 0, 'Susceptance': 0.10631, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3173'},
        {'fromNode': '8113.0', 'toNode': '8133.0', 'Resistance': 0.003, 'Reactance': 0.0312, 'Conductance': 0, 'Susceptance': 0.10631, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3174'},
        {'fromNode': '8121.0', 'toNode': '8114.0', 'Resistance': 0.01148, 'Reactance': 0.07843, 'Conductance': 0, 'Susceptance': 0.01334, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3175'},
        {'fromNode': '8118.0', 'toNode': '8151.0', 'Resistance': 0.01924, 'Reactance': 0.11555, 'Conductance': 0, 'Susceptance': 0.01568, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3176'},
        {'fromNode': '8120.0', 'toNode': '8122.0', 'Resistance': 0.03382, 'Reactance': 0.08268, 'Conductance': 0, 'Susceptance': 0.01083, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3177'},
        {'fromNode': '8120.0', 'toNode': '8140.0', 'Resistance': 0.01216, 'Reactance': 0.07301, 'Conductance': 0, 'Susceptance': 0.00991, 'MVA_rating': 170.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3178'},
        {'fromNode': '8120.0', 'toNode': '8141.0', 'Resistance': 0.0166, 'Reactance': 0.06885, 'Conductance': 0, 'Susceptance': 0.01168, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3179'},
        {'fromNode': '8143.0', 'toNode': '8120.0', 'Resistance': 0.01659, 'Reactance': 0.07254, 'Conductance': 0, 'Susceptance': 0.01105, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3180'},
        {'fromNode': '8149.0', 'toNode': '8123.0', 'Resistance': 0.00474, 'Reactance': 0.02711, 'Conductance': 0, 'Susceptance': 0.00407, 'MVA_rating': 147.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3181'},
        {'fromNode': '8127.0', 'toNode': '8126.0', 'Resistance': 0.00011, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1175.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3182'},
        {'fromNode': '8127.0', 'toNode': '8126.0', 'Resistance': 0.00011, 'Reactance': 0.00797, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1175.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3183'},
        {'fromNode': '8129.0', 'toNode': '8126.0', 'Resistance': 8e-05, 'Reactance': 0.00487, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1190.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3184'},
        {'fromNode': '8130.0', 'toNode': '8126.0', 'Resistance': 0.00026, 'Reactance': 0.01215, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1192.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3185'},
        {'fromNode': '8131.0', 'toNode': '8126.0', 'Resistance': 0.0001, 'Reactance': 0.00621, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1173.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3186'},
        {'fromNode': '8128.0', 'toNode': '8127.0', 'Resistance': 0.0004, 'Reactance': 0.02178, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 286.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3187'},
        {'fromNode': '8132.0', 'toNode': '8144.0', 'Resistance': 0.02494, 'Reactance': 0.10345, 'Conductance': 0, 'Susceptance': 0.01756, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3188'},
        {'fromNode': '8134.0', 'toNode': '8133.0', 'Resistance': 0.00067, 'Reactance': 0.03526, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3189'},
        {'fromNode': '8134.0', 'toNode': '8133.0', 'Resistance': 0.00067, 'Reactance': 0.03526, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3190'},
        {'fromNode': '8133.0', 'toNode': '8147.0', 'Resistance': 0.00702, 'Reactance': 0.06865, 'Conductance': 0, 'Susceptance': 0.13208, 'MVA_rating': 548.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3191'},
        {'fromNode': '8154.0', 'toNode': '8135.0', 'Resistance': 0.00895, 'Reactance': 0.03991, 'Conductance': 0, 'Susceptance': 0.00585, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3192'},
        {'fromNode': '8136.0', 'toNode': '8145.0', 'Resistance': 0.01057, 'Reactance': 0.04623, 'Conductance': 0, 'Susceptance': 0.00704, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3193'},
        {'fromNode': '8138.0', 'toNode': '8146.0', 'Resistance': 0.02103, 'Reactance': 0.12014, 'Conductance': 0, 'Susceptance': 0.01805, 'MVA_rating': 98.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3194'},
        {'fromNode': '8140.0', 'toNode': '8139.0', 'Resistance': 0.00037, 'Reactance': 0.02517, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 365.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3195'},
        {'fromNode': '8148.0', 'toNode': '8147.0', 'Resistance': 0.00049, 'Reactance': 0.03635, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 201.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3196'},
        {'fromNode': '8148.0', 'toNode': '8147.0', 'Resistance': 0.00049, 'Reactance': 0.03635, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 201.3, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3197'},
        {'fromNode': '8149.0', 'toNode': '8148.0', 'Resistance': 0.00259, 'Reactance': 0.0686, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 254.0, 'kV_base': 161.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3198'},
        {'fromNode': '8152.0', 'toNode': '8160.0', 'Resistance': 0.00538, 'Reactance': 0.03751, 'Conductance': 0, 'Susceptance': 0.00613, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3199'},
        {'fromNode': '8152.0', 'toNode': '8160.0', 'Resistance': 0.00538, 'Reactance': 0.03751, 'Conductance': 0, 'Susceptance': 0.00613, 'MVA_rating': 221.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '3200'},
        {'fromNode': '8156.0', 'toNode': '8155.0', 'Resistance': 0.00077, 'Reactance': 0.04753, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3201'},
        {'fromNode': '8157.0', 'toNode': '8156.0', 'Resistance': 0.0013, 'Reactance': 0.06678, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 240.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3202'},
        {'fromNode': '8156.0', 'toNode': '8159.0', 'Resistance': 0.00621, 'Reactance': 0.04434, 'Conductance': 0, 'Susceptance': 0.08084, 'MVA_rating': 600.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '3203'},
        {'fromNode': '8159.0', 'toNode': '8158.0', 'Resistance': 0.00013, 'Reactance': 0.01364, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1204.0, 'kV_base': 500.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3204'},
        {'fromNode': '8160.0', 'toNode': '8159.0', 'Resistance': 0.0008, 'Reactance': 0.05008, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 377.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3205'},
        {'fromNode': '8160.0', 'toNode': '8159.0', 'Resistance': 0.0008, 'Reactance': 0.05008, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 377.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '3206'}
    ]
    lines_AC = pd.DataFrame(lines_AC_data)

    nodes_DC = None

    lines_DC = None

    Converters_ACDC = None

    
    # Create the grid
    [grid, res] = pyf.Create_grid_from_data(S_base, nodes_AC, lines_AC, nodes_DC, lines_DC, Converters_ACDC, data_in_pu=True)
    
    # Assign Price Zones to Nodes
    for index, row in nodes_AC.iterrows():
        node_name = nodes_AC.at[index, 'Node_id']
        price_zone = nodes_AC.at[index, 'PZ']
        ACDC = 'AC'
        if price_zone is not None:
            pyf.assign_nodeToPrice_Zone(grid, node_name, ACDC, price_zone)
    
    # Add Generators
    pyf.add_gen(grid, '1004.0', '1', price_zone_link=False, lf=0.0, qf=0.0, MWmax=158.25, MWmin=158.25, MVArmax=44.94, MVArmin=-30.380000000000003, PsetMW=158.25, QsetMVA=-30.380000000000003)
    pyf.add_gen(grid, '1006.0', '2', price_zone_link=False, lf=0.0, qf=0.0, MWmax=25.730000000000004, MWmin=25.730000000000004, MVArmax=7.31, MVArmin=-4.94, PsetMW=25.730000000000004, QsetMVA=-4.94)
    pyf.add_gen(grid, '1009.0', '3', price_zone_link=False, lf=0.0, qf=0.0, MWmax=61.870000000000005, MWmin=61.870000000000005, MVArmax=17.57, MVArmin=-11.88, PsetMW=61.870000000000005, QsetMVA=-11.88)
    pyf.add_gen(grid, '1011.0', '4', price_zone_link=False, lf=0.0, qf=0.0, MWmax=10.0, MWmin=3.0, MVArmax=0.0, MVArmin=0.0, PsetMW=9.97, QsetMVA=0.0)
    pyf.add_gen(grid, '1021.0', '5', price_zone_link=False, lf=0.0, qf=0.0, MWmax=149.63, MWmin=149.63, MVArmax=42.49, MVArmin=-28.73, PsetMW=149.63, QsetMVA=-28.73)
    pyf.add_gen(grid, '1023.0', '6', price_zone_link=False, lf=0.0, qf=0.0, MWmax=135.0, MWmin=135.0, MVArmax=38.34, MVArmin=-25.920000000000005, PsetMW=135.0, QsetMVA=-0.55)
    pyf.add_gen(grid, '1026.0', '7', price_zone_link=False, lf=0.0, qf=0.0, MWmax=93.15, MWmin=93.15, MVArmax=26.450000000000003, MVArmin=-17.88, PsetMW=93.15, QsetMVA=-6.13)
    pyf.add_gen(grid, '1033.0', '8', price_zone_link=False, lf=0.0, qf=0.0, MWmax=208.5, MWmin=208.5, MVArmax=59.209999999999994, MVArmin=-40.03, PsetMW=208.5, QsetMVA=-40.03)
    pyf.add_gen(grid, '1035.0', '9', price_zone_link=False, lf=0.0, qf=0.0, MWmax=67.5, MWmin=67.5, MVArmax=19.17, MVArmin=-12.960000000000003, PsetMW=67.5, QsetMVA=-9.15)
    pyf.add_gen(grid, '1039.0', '10', price_zone_link=False, lf=0.0, qf=0.0, MWmax=110.63000000000001, MWmin=110.63000000000001, MVArmax=31.42, MVArmin=-21.24, PsetMW=110.63000000000001, QsetMVA=-20.76)
    pyf.add_gen(grid, '1043.0', '12', price_zone_link=False, lf=0.0, qf=0.0, MWmax=44.1, MWmin=44.1, MVArmax=12.520000000000001, MVArmin=-8.47, PsetMW=44.1, QsetMVA=-3.61)
    pyf.add_gen(grid, '1050.0', '15', price_zone_link=False, lf=17.464, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=89.15, QsetMVA=-7.519999999999999)
    pyf.add_gen(grid, '1051.0', '16', price_zone_link=False, lf=17.464, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=89.15, QsetMVA=-5.5)
    pyf.add_gen(grid, '1052.0', '17', price_zone_link=False, lf=17.464, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=89.15, QsetMVA=-9.92)
    pyf.add_gen(grid, '1066.0', '24', price_zone_link=False, lf=0.0, qf=0.0, MWmax=106.88, MWmin=106.88, MVArmax=30.349999999999998, MVArmin=-20.52, PsetMW=106.88, QsetMVA=-3.74)
    pyf.add_gen(grid, '1070.0', '25', price_zone_link=False, lf=0.0, qf=0.0, MWmax=120.38, MWmin=120.38, MVArmax=34.19, MVArmin=-23.11, PsetMW=120.38, QsetMVA=18.58)
    pyf.add_gen(grid, '1072.0', '26', price_zone_link=False, lf=17.977, qf=0.001, MWmax=192.13, MWmin=57.64, MVArmax=97.8, MVArmin=-21.33, PsetMW=191.6, QsetMVA=25.790000000000003)
    pyf.add_gen(grid, '1073.0', '27', price_zone_link=False, lf=17.977, qf=0.001, MWmax=192.13, MWmin=57.64, MVArmax=97.8, MVArmin=-21.33, PsetMW=191.6, QsetMVA=32.88)
    pyf.add_gen(grid, '1074.0', '28', price_zone_link=False, lf=17.977, qf=0.001, MWmax=192.13, MWmin=57.64, MVArmax=97.8, MVArmin=-21.33, PsetMW=191.6, QsetMVA=29.18)
    pyf.add_gen(grid, '1075.0', '29', price_zone_link=False, lf=17.977, qf=0.001, MWmax=192.13, MWmin=57.64, MVArmax=97.8, MVArmin=-21.33, PsetMW=191.6, QsetMVA=30.03)
    pyf.add_gen(grid, '1078.0', '32', price_zone_link=False, lf=17.495, qf=0.001, MWmax=95.5, MWmin=28.65, MVArmax=48.61, MVArmin=-10.6, PsetMW=95.24, QsetMVA=15.89)
    pyf.add_gen(grid, '1079.0', '33', price_zone_link=False, lf=17.495, qf=0.001, MWmax=95.5, MWmin=28.65, MVArmax=48.61, MVArmin=-10.6, PsetMW=95.24, QsetMVA=-10.6)
    pyf.add_gen(grid, '1080.0', '34', price_zone_link=False, lf=17.495, qf=0.001, MWmax=95.5, MWmin=28.65, MVArmax=48.61, MVArmin=-10.6, PsetMW=95.24, QsetMVA=12.770000000000001)
    pyf.add_gen(grid, '1082.0', '36', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-21.6)
    pyf.add_gen(grid, '1084.0', '37', price_zone_link=False, lf=0.0, qf=0.0, MWmax=86.62, MWmin=86.62, MVArmax=24.6, MVArmin=-16.63, PsetMW=86.62, QsetMVA=24.6)
    pyf.add_gen(grid, '1088.0', '38', price_zone_link=False, lf=0.0, qf=0.0, MWmax=56.25, MWmin=56.25, MVArmax=15.97, MVArmin=-10.8, PsetMW=56.25, QsetMVA=-3.26)
    pyf.add_gen(grid, '1090.0', '39', price_zone_link=False, lf=0.0, qf=0.0, MWmax=114.36999999999999, MWmin=114.36999999999999, MVArmax=32.48, MVArmin=-21.96, PsetMW=114.36999999999999, QsetMVA=23.48)
    pyf.add_gen(grid, '1090.0', '40', price_zone_link=False, lf=0.0, qf=0.0, MWmax=27.0, MWmin=27.0, MVArmax=7.670000000000001, MVArmin=-5.18, PsetMW=27.0, QsetMVA=7.670000000000001)
    pyf.add_gen(grid, '2013.0', '44', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.45, MWmin=150.45, MVArmax=42.73, MVArmin=-28.89, PsetMW=150.45, QsetMVA=-28.89)
    pyf.add_gen(grid, '2023.0', '46', price_zone_link=False, lf=16.989, qf=0.001, MWmax=83.33, MWmin=25.0, MVArmax=42.42, MVArmin=-9.25, PsetMW=83.1, QsetMVA=15.439999999999998)
    pyf.add_gen(grid, '2024.0', '47', price_zone_link=False, lf=17.394, qf=0.001, MWmax=166.67, MWmin=50.0, MVArmax=84.83, MVArmin=-18.5, PsetMW=166.21, QsetMVA=22.7)
    pyf.add_gen(grid, '2042.0', '48', price_zone_link=False, lf=0.0, qf=0.0, MWmax=59.699999999999996, MWmin=59.699999999999996, MVArmax=16.95, MVArmin=-11.46, PsetMW=59.699999999999996, QsetMVA=-11.46)
    pyf.add_gen(grid, '2057.0', '50', price_zone_link=False, lf=17.268, qf=0.002, MWmax=720.0, MWmin=216.0, MVArmax=335.52, MVArmin=-87.84, PsetMW=573.59, QsetMVA=-57.989999999999995)
    pyf.add_gen(grid, '2059.0', '51', price_zone_link=False, lf=0.0, qf=0.0, MWmax=60.0, MWmin=60.0, MVArmax=17.04, MVArmin=-11.52, PsetMW=60.0, QsetMVA=-11.52)
    pyf.add_gen(grid, '2062.0', '52', price_zone_link=False, lf=0.0, qf=0.0, MWmax=101.55000000000001, MWmin=101.55000000000001, MVArmax=28.84, MVArmin=-19.5, PsetMW=101.55000000000001, QsetMVA=-12.3)
    pyf.add_gen(grid, '2071.0', '53', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-2.46)
    pyf.add_gen(grid, '2076.0', '54', price_zone_link=False, lf=0.0, qf=0.0, MWmax=22.5, MWmin=22.5, MVArmax=6.39, MVArmin=-4.32, PsetMW=22.5, QsetMVA=6.39)
    pyf.add_gen(grid, '2078.0', '55', price_zone_link=False, lf=0.0, qf=0.0, MWmax=84.38, MWmin=84.38, MVArmax=23.96, MVArmin=-16.2, PsetMW=84.38, QsetMVA=-4.82)
    pyf.add_gen(grid, '2083.0', '56', price_zone_link=False, lf=16.033, qf=0.001, MWmax=20.0, MWmin=6.0, MVArmax=10.18, MVArmin=-2.22, PsetMW=19.94, QsetMVA=10.07)
    pyf.add_gen(grid, '2089.0', '59', price_zone_link=False, lf=0.0, qf=0.0, MWmax=50.7, MWmin=50.7, MVArmax=14.400000000000002, MVArmin=-9.73, PsetMW=50.7, QsetMVA=-0.29)
    pyf.add_gen(grid, '2099.0', '62', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.5, MWmin=7.5, MVArmax=2.13, MVArmin=-1.44, PsetMW=7.5, QsetMVA=-1.44)
    pyf.add_gen(grid, '2100.0', '63', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-21.6)
    pyf.add_gen(grid, '2115.0', '67', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-20.5)
    pyf.add_gen(grid, '2116.0', '68', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-20.5)
    pyf.add_gen(grid, '2117.0', '69', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-20.5)
    pyf.add_gen(grid, '2118.0', '70', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-20.5)
    pyf.add_gen(grid, '2119.0', '71', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-20.5)
    pyf.add_gen(grid, '2120.0', '72', price_zone_link=False, lf=17.482, qf=0.001, MWmax=184.72, MWmin=55.40999999999999, MVArmax=94.02, MVArmin=-20.5, PsetMW=184.21, QsetMVA=-4.83)
    pyf.add_gen(grid, '2123.0', '73', price_zone_link=False, lf=0.0, qf=0.0, MWmax=136.28, MWmin=136.28, MVArmax=38.7, MVArmin=-26.16, PsetMW=136.28, QsetMVA=-26.16)
    pyf.add_gen(grid, '2127.0', '74', price_zone_link=False, lf=0.0, qf=0.0, MWmax=216.45, MWmin=216.45, MVArmax=61.47, MVArmin=-41.56, PsetMW=216.45, QsetMVA=-1.0)
    pyf.add_gen(grid, '3004.0', '75', price_zone_link=False, lf=0.0, qf=0.0, MWmax=28.13, MWmin=28.13, MVArmax=7.99, MVArmin=-5.4, PsetMW=28.13, QsetMVA=-5.4)
    pyf.add_gen(grid, '3017.0', '76', price_zone_link=False, lf=0.0, qf=0.0, MWmax=127.12999999999998, MWmin=127.12999999999998, MVArmax=36.1, MVArmin=-24.41, PsetMW=127.12999999999998, QsetMVA=-0.17)
    pyf.add_gen(grid, '3020.0', '77', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=1.34)
    pyf.add_gen(grid, '3028.0', '78', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.0, MWmin=150.0, MVArmax=42.6, MVArmin=-28.800000000000004, PsetMW=150.0, QsetMVA=36.96)
    pyf.add_gen(grid, '3034.0', '79', price_zone_link=False, lf=0.0, qf=0.0, MWmax=97.87, MWmin=97.87, MVArmax=27.800000000000004, MVArmin=-18.79, PsetMW=97.87, QsetMVA=17.14)
    pyf.add_gen(grid, '3039.0', '80', price_zone_link=False, lf=0.0, qf=0.0, MWmax=156.75, MWmin=156.75, MVArmax=44.52, MVArmin=-30.099999999999998, PsetMW=156.75, QsetMVA=43.36)
    pyf.add_gen(grid, '3048.0', '84', price_zone_link=False, lf=0.0, qf=0.0, MWmax=180.75, MWmin=180.75, MVArmax=51.33, MVArmin=-34.7, PsetMW=180.75, QsetMVA=51.33)
    pyf.add_gen(grid, '3055.0', '85', price_zone_link=False, lf=0.0, qf=0.0, MWmax=183.88, MWmin=183.88, MVArmax=52.22, MVArmin=-35.3, PsetMW=183.88, QsetMVA=35.81)
    pyf.add_gen(grid, '3056.0', '86', price_zone_link=False, lf=0.0, qf=0.0, MWmax=183.88, MWmin=183.88, MVArmax=52.22, MVArmin=-35.3, PsetMW=183.88, QsetMVA=24.15)
    pyf.add_gen(grid, '3057.0', '87', price_zone_link=False, lf=0.0, qf=0.0, MWmax=183.88, MWmin=183.88, MVArmax=52.22, MVArmin=-35.3, PsetMW=183.88, QsetMVA=22.53)
    pyf.add_gen(grid, '3060.0', '88', price_zone_link=False, lf=0.0, qf=0.0, MWmax=124.22, MWmin=124.22, MVArmax=35.28, MVArmin=-23.85, PsetMW=124.22, QsetMVA=13.18)
    pyf.add_gen(grid, '3061.0', '89', price_zone_link=False, lf=0.0, qf=0.0, MWmax=124.22, MWmin=124.22, MVArmax=35.28, MVArmin=-23.85, PsetMW=124.22, QsetMVA=13.94)
    pyf.add_gen(grid, '3062.0', '90', price_zone_link=False, lf=0.0, qf=0.0, MWmax=124.22, MWmin=124.22, MVArmax=35.28, MVArmin=-23.85, PsetMW=124.22, QsetMVA=19.46)
    pyf.add_gen(grid, '3063.0', '91', price_zone_link=False, lf=0.0, qf=0.0, MWmax=124.22, MWmin=124.22, MVArmax=35.28, MVArmin=-23.85, PsetMW=124.22, QsetMVA=14.87)
    pyf.add_gen(grid, '3065.0', '92', price_zone_link=False, lf=0.0, qf=0.0, MWmax=60.370000000000005, MWmin=60.370000000000005, MVArmax=17.15, MVArmin=-11.59, PsetMW=60.370000000000005, QsetMVA=-11.59)
    pyf.add_gen(grid, '3068.0', '93', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=25.590000000000003)
    pyf.add_gen(grid, '3071.0', '94', price_zone_link=False, lf=0.0, qf=0.0, MWmax=127.64999999999999, MWmin=127.64999999999999, MVArmax=36.25, MVArmin=-24.51, PsetMW=127.64999999999999, QsetMVA=-24.51)
    pyf.add_gen(grid, '3074.0', '95', price_zone_link=False, lf=0.0, qf=0.0, MWmax=74.85, MWmin=74.85, MVArmax=21.26, MVArmin=-14.37, PsetMW=74.85, QsetMVA=-7.139999999999999)
    pyf.add_gen(grid, '3081.0', '96', price_zone_link=False, lf=0.0, qf=0.0, MWmax=186.75, MWmin=186.75, MVArmax=53.04, MVArmin=-35.86, PsetMW=186.75, QsetMVA=33.97)
    pyf.add_gen(grid, '3084.0', '97', price_zone_link=False, lf=0.0, qf=0.0, MWmax=174.37, MWmin=174.37, MVArmax=49.52, MVArmin=-33.48, PsetMW=174.37, QsetMVA=-8.93)
    pyf.add_gen(grid, '3086.0', '98', price_zone_link=False, lf=0.0, qf=0.0, MWmax=56.25, MWmin=56.25, MVArmax=15.97, MVArmin=-10.8, PsetMW=56.25, QsetMVA=-1.69)
    pyf.add_gen(grid, '3092.0', '100', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.0, MWmin=150.0, MVArmax=42.6, MVArmin=-28.800000000000004, PsetMW=150.0, QsetMVA=18.47)
    pyf.add_gen(grid, '3095.0', '101', price_zone_link=False, lf=0.0, qf=0.0, MWmax=90.45, MWmin=90.45, MVArmax=25.69, MVArmin=-17.37, PsetMW=90.45, QsetMVA=19.62)
    pyf.add_gen(grid, '3098.0', '102', price_zone_link=False, lf=0.0, qf=0.0, MWmax=120.0, MWmin=120.0, MVArmax=34.08, MVArmin=-23.04, PsetMW=120.0, QsetMVA=17.91)
    pyf.add_gen(grid, '3105.0', '103', price_zone_link=False, lf=0.0, qf=0.0, MWmax=44.5, MWmin=44.5, MVArmax=17.07, MVArmin=-0.74, PsetMW=44.5, QsetMVA=-0.74)
    pyf.add_gen(grid, '3106.0', '104', price_zone_link=False, lf=0.0, qf=0.0, MWmax=53.02, MWmin=53.02, MVArmax=62.480000000000004, MVArmin=-2.69, PsetMW=53.02, QsetMVA=-2.69)
    pyf.add_gen(grid, '3110.0', '105', price_zone_link=False, lf=0.0, qf=0.0, MWmax=63.0, MWmin=63.0, MVArmax=17.89, MVArmin=-12.1, PsetMW=63.0, QsetMVA=11.92)
    pyf.add_gen(grid, '3112.0', '106', price_zone_link=False, lf=0.0, qf=0.0, MWmax=56.10000000000001, MWmin=56.10000000000001, MVArmax=15.93, MVArmin=-10.77, PsetMW=56.10000000000001, QsetMVA=-3.04)
    pyf.add_gen(grid, '3113.0', '107', price_zone_link=False, lf=0.0, qf=0.0, MWmax=56.10000000000001, MWmin=56.10000000000001, MVArmax=15.93, MVArmin=-10.77, PsetMW=56.10000000000001, QsetMVA=-2.35)
    pyf.add_gen(grid, '3117.0', '108', price_zone_link=False, lf=0.0, qf=0.0, MWmax=47.25, MWmin=47.25, MVArmax=13.419999999999998, MVArmin=-9.07, PsetMW=47.25, QsetMVA=-5.09)
    pyf.add_gen(grid, '3118.0', '109', price_zone_link=False, lf=0.0, qf=0.0, MWmax=90.0, MWmin=90.0, MVArmax=25.56, MVArmin=-17.28, PsetMW=90.0, QsetMVA=-8.53)
    pyf.add_gen(grid, '3120.0', '110', price_zone_link=False, lf=0.0, qf=0.0, MWmax=29.7, MWmin=29.7, MVArmax=0.0, MVArmin=0.0, PsetMW=29.7, QsetMVA=0.0)
    pyf.add_gen(grid, '3122.0', '111', price_zone_link=False, lf=0.0, qf=0.0, MWmax=147.75, MWmin=147.75, MVArmax=41.96, MVArmin=-28.37, PsetMW=147.75, QsetMVA=-13.3)
    pyf.add_gen(grid, '3127.0', '112', price_zone_link=False, lf=0.0, qf=0.0, MWmax=101.25, MWmin=101.25, MVArmax=28.76, MVArmin=-19.44, PsetMW=101.25, QsetMVA=-19.44)
    pyf.add_gen(grid, '3129.0', '113', price_zone_link=False, lf=0.0, qf=0.0, MWmax=94.88, MWmin=94.88, MVArmax=26.94, MVArmin=-18.22, PsetMW=94.88, QsetMVA=-18.22)
    pyf.add_gen(grid, '3131.0', '114', price_zone_link=False, lf=0.0, qf=0.0, MWmax=85.5, MWmin=85.5, MVArmax=24.28, MVArmin=-16.42, PsetMW=85.5, QsetMVA=22.85)
    pyf.add_gen(grid, '3137.0', '118', price_zone_link=False, lf=0.0, qf=0.0, MWmax=75.9, MWmin=75.9, MVArmax=21.56, MVArmin=-14.57, PsetMW=75.9, QsetMVA=21.56)
    pyf.add_gen(grid, '3141.0', '119', price_zone_link=False, lf=0.0, qf=0.0, MWmax=68.62, MWmin=68.62, MVArmax=19.49, MVArmin=-13.18, PsetMW=68.62, QsetMVA=-13.18)
    pyf.add_gen(grid, '4010.0', '120', price_zone_link=False, lf=0.0, qf=0.0, MWmax=151.2, MWmin=151.2, MVArmax=42.94, MVArmin=-29.03, PsetMW=151.2, QsetMVA=11.42)
    pyf.add_gen(grid, '4026.0', '121', price_zone_link=False, lf=17.172, qf=0.002, MWmax=409.99999999999994, MWmin=123.0, MVArmax=191.06, MVArmin=-50.019999999999996, PsetMW=408.86000000000007, QsetMVA=44.49)
    pyf.add_gen(grid, '4030.0', '122', price_zone_link=False, lf=17.172, qf=0.002, MWmax=622.4, MWmin=186.72, MVArmax=290.04, MVArmin=-75.93, PsetMW=460.57, QsetMVA=-34.65)
    pyf.add_gen(grid, '4045.0', '123', price_zone_link=False, lf=0.0, qf=0.0, MWmax=93.4, MWmin=93.4, MVArmax=35.86, MVArmin=-1.54, PsetMW=93.4, QsetMVA=7.470000000000001)
    pyf.add_gen(grid, '4046.0', '124', price_zone_link=False, lf=0.0, qf=0.0, MWmax=28.499999999999996, MWmin=28.499999999999996, MVArmax=10.94, MVArmin=-0.4699999999999999, PsetMW=28.499999999999996, QsetMVA=1.96)
    pyf.add_gen(grid, '4047.0', '125', price_zone_link=False, lf=0.0, qf=0.0, MWmax=27.48, MWmin=27.48, MVArmax=75.1, MVArmin=-3.2300000000000004, PsetMW=27.48, QsetMVA=3.4099999999999997)
    pyf.add_gen(grid, '4050.0', '126', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.0, MWmin=150.0, MVArmax=42.6, MVArmin=-28.800000000000004, PsetMW=150.0, QsetMVA=-21.93)
    pyf.add_gen(grid, '4052.0', '127', price_zone_link=False, lf=16.73, qf=0.001, MWmax=44.2, MWmin=13.26, MVArmax=22.5, MVArmin=-4.91, PsetMW=44.08, QsetMVA=22.5)
    pyf.add_gen(grid, '4058.0', '128', price_zone_link=False, lf=17.213, qf=0.001, MWmax=144.0, MWmin=43.2, MVArmax=73.3, MVArmin=-15.98, PsetMW=99.6, QsetMVA=17.45)
    pyf.add_gen(grid, '4059.0', '129', price_zone_link=False, lf=17.213, qf=0.001, MWmax=144.0, MWmin=43.2, MVArmax=73.3, MVArmin=-15.98, PsetMW=84.08, QsetMVA=18.42)
    pyf.add_gen(grid, '4060.0', '130', price_zone_link=False, lf=17.213, qf=0.001, MWmax=144.0, MWmin=43.2, MVArmax=73.3, MVArmin=-15.98, PsetMW=124.4, QsetMVA=14.84)
    pyf.add_gen(grid, '4061.0', '131', price_zone_link=False, lf=17.35, qf=0.001, MWmax=172.33, MWmin=51.7, MVArmax=87.72, MVArmin=-19.13, PsetMW=148.88, QsetMVA=19.5)
    pyf.add_gen(grid, '4062.0', '132', price_zone_link=False, lf=17.35, qf=0.001, MWmax=172.33, MWmin=51.7, MVArmax=87.72, MVArmin=-19.13, PsetMW=171.85, QsetMVA=17.02)
    pyf.add_gen(grid, '4063.0', '133', price_zone_link=False, lf=17.35, qf=0.001, MWmax=172.33, MWmin=51.7, MVArmax=87.72, MVArmin=-19.13, PsetMW=124.75, QsetMVA=19.69)
    pyf.add_gen(grid, '4065.0', '134', price_zone_link=False, lf=16.743, qf=0.001, MWmax=47.0, MWmin=14.099999999999998, MVArmax=23.92, MVArmin=-5.22, PsetMW=46.87, QsetMVA=13.41)
    pyf.add_gen(grid, '4076.0', '136', price_zone_link=False, lf=29.085, qf=0.002, MWmax=176.33, MWmin=52.900000000000006, MVArmax=89.75, MVArmin=-19.57, PsetMW=175.84, QsetMVA=23.56)
    pyf.add_gen(grid, '4077.0', '137', price_zone_link=False, lf=29.085, qf=0.002, MWmax=176.33, MWmin=52.900000000000006, MVArmax=89.75, MVArmin=-19.57, PsetMW=97.06, QsetMVA=17.97)
    pyf.add_gen(grid, '4082.0', '138', price_zone_link=False, lf=17.694, qf=0.001, MWmax=243.4, MWmin=73.02, MVArmax=123.89000000000001, MVArmin=-27.02, PsetMW=242.73, QsetMVA=29.65)
    pyf.add_gen(grid, '4083.0', '139', price_zone_link=False, lf=17.694, qf=0.001, MWmax=243.4, MWmin=73.02, MVArmax=123.89000000000001, MVArmin=-27.02, PsetMW=242.73, QsetMVA=30.42)
    pyf.add_gen(grid, '4084.0', '140', price_zone_link=False, lf=17.694, qf=0.001, MWmax=243.4, MWmin=73.02, MVArmax=123.89000000000001, MVArmin=-27.02, PsetMW=242.73, QsetMVA=46.21)
    pyf.add_gen(grid, '4085.0', '141', price_zone_link=False, lf=18.22, qf=0.001, MWmax=352.0, MWmin=105.60000000000001, MVArmax=179.17, MVArmin=-39.07, PsetMW=168.05, QsetMVA=48.8)
    pyf.add_gen(grid, '4091.0', '143', price_zone_link=False, lf=17.172, qf=0.002, MWmax=23.5, MWmin=7.049999999999999, MVArmax=10.95, MVArmin=-2.87, PsetMW=23.43, QsetMVA=2.45)
    pyf.add_gen(grid, '4093.0', '145', price_zone_link=False, lf=17.172, qf=0.002, MWmax=28.000000000000004, MWmin=8.4, MVArmax=13.05, MVArmin=-3.42, PsetMW=27.92, QsetMVA=1.3)
    pyf.add_gen(grid, '4094.0', '146', price_zone_link=False, lf=17.172, qf=0.002, MWmax=28.000000000000004, MWmin=8.4, MVArmax=13.05, MVArmin=-3.42, PsetMW=27.92, QsetMVA=4.86)
    pyf.add_gen(grid, '4095.0', '147', price_zone_link=False, lf=17.172, qf=0.002, MWmax=28.000000000000004, MWmin=8.4, MVArmax=13.05, MVArmin=-3.42, PsetMW=27.92, QsetMVA=2.14)
    pyf.add_gen(grid, '4097.0', '149', price_zone_link=False, lf=17.172, qf=0.002, MWmax=28.000000000000004, MWmin=8.4, MVArmax=13.05, MVArmin=-3.42, PsetMW=27.92, QsetMVA=1.87)
    pyf.add_gen(grid, '4105.0', '151', price_zone_link=False, lf=0.0, qf=0.0, MWmax=135.0, MWmin=135.0, MVArmax=38.34, MVArmin=-25.920000000000005, PsetMW=135.0, QsetMVA=23.76)
    pyf.add_gen(grid, '4115.0', '152', price_zone_link=False, lf=28.724, qf=0.002, MWmax=131.8, MWmin=39.54, MVArmax=67.09, MVArmin=-14.63, PsetMW=131.43, QsetMVA=28.4)
    pyf.add_gen(grid, '4116.0', '153', price_zone_link=False, lf=28.724, qf=0.002, MWmax=131.8, MWmin=39.54, MVArmax=67.09, MVArmin=-14.63, PsetMW=123.76, QsetMVA=25.71)
    pyf.add_gen(grid, '4134.0', '155', price_zone_link=False, lf=27.928, qf=0.002, MWmax=33.6, MWmin=10.08, MVArmax=17.1, MVArmin=-3.73, PsetMW=10.21, QsetMVA=-3.73)
    pyf.add_gen(grid, '4135.0', '156', price_zone_link=False, lf=27.928, qf=0.002, MWmax=33.6, MWmin=10.08, MVArmax=17.1, MVArmin=-3.73, PsetMW=10.21, QsetMVA=-3.73)
    pyf.add_gen(grid, '4136.0', '157', price_zone_link=False, lf=27.928, qf=0.002, MWmax=33.6, MWmin=10.08, MVArmax=17.1, MVArmin=-3.73, PsetMW=10.21, QsetMVA=-3.73)
    pyf.add_gen(grid, '4142.0', '160', price_zone_link=False, lf=0.0, qf=0.0, MWmax=141.0, MWmin=141.0, MVArmax=40.04, MVArmin=-27.07, PsetMW=141.0, QsetMVA=27.339999999999996)
    pyf.add_gen(grid, '4153.0', '161', price_zone_link=False, lf=0.0, qf=0.0, MWmax=212.4, MWmin=212.4, MVArmax=60.31999999999999, MVArmin=-40.78, PsetMW=212.4, QsetMVA=60.31999999999999)
    pyf.add_gen(grid, '4160.0', '162', price_zone_link=False, lf=17.473, qf=0.001, MWmax=197.77, MWmin=59.32999999999999, MVArmax=100.66, MVArmin=-21.95, PsetMW=197.22, QsetMVA=64.86)
    pyf.add_gen(grid, '4161.0', '163', price_zone_link=False, lf=17.473, qf=0.001, MWmax=197.77, MWmin=59.32999999999999, MVArmax=100.66, MVArmin=-21.95, PsetMW=87.79, QsetMVA=27.190000000000005)
    pyf.add_gen(grid, '4162.0', '164', price_zone_link=False, lf=17.473, qf=0.001, MWmax=197.77, MWmin=59.32999999999999, MVArmax=100.66, MVArmin=-21.95, PsetMW=112.47, QsetMVA=26.13)
    pyf.add_gen(grid, '4166.0', '166', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.75, MWmin=150.75, MVArmax=42.81, MVArmin=-28.939999999999998, PsetMW=150.75, QsetMVA=-28.939999999999998)
    pyf.add_gen(grid, '4177.0', '167', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-0.14)
    pyf.add_gen(grid, '4181.0', '168', price_zone_link=False, lf=0.0, qf=0.0, MWmax=150.07, MWmin=150.07, MVArmax=42.62, MVArmin=-28.809999999999995, PsetMW=150.07, QsetMVA=42.62)
    pyf.add_gen(grid, '4183.0', '169', price_zone_link=False, lf=0.0, qf=0.0, MWmax=152.25, MWmin=152.25, MVArmax=43.24, MVArmin=-29.23, PsetMW=152.25, QsetMVA=43.24)
    pyf.add_gen(grid, '4192.0', '170', price_zone_link=False, lf=16.879, qf=0.001, MWmax=75.0, MWmin=22.5, MVArmax=38.17, MVArmin=-8.33, PsetMW=74.79, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '171', price_zone_link=False, lf=17.405, qf=0.001, MWmax=183.77, MWmin=55.13, MVArmax=93.54, MVArmin=-20.4, PsetMW=183.26, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '173', price_zone_link=False, lf=16.811, qf=0.001, MWmax=61.0, MWmin=18.3, MVArmax=31.05, MVArmin=-6.77, PsetMW=60.83, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '174', price_zone_link=False, lf=16.566, qf=0.001, MWmax=10.4, MWmin=3.12, MVArmax=5.29, MVArmin=-1.15, PsetMW=10.37, QsetMVA=5.29)
    pyf.add_gen(grid, '4192.0', '175', price_zone_link=False, lf=17.808, qf=0.001, MWmax=267.0, MWmin=80.1, MVArmax=135.9, MVArmin=-29.64, PsetMW=136.48, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '176', price_zone_link=False, lf=17.808, qf=0.001, MWmax=267.0, MWmin=80.1, MVArmax=135.9, MVArmin=-29.64, PsetMW=155.9, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '177', price_zone_link=False, lf=17.808, qf=0.001, MWmax=267.0, MWmin=80.1, MVArmax=135.9, MVArmin=-29.64, PsetMW=155.9, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '178', price_zone_link=False, lf=17.405, qf=0.001, MWmax=183.77, MWmin=55.13, MVArmax=93.54, MVArmin=-20.4, PsetMW=183.26, QsetMVA=17.91)
    pyf.add_gen(grid, '4192.0', '179', price_zone_link=False, lf=17.405, qf=0.001, MWmax=183.77, MWmin=55.13, MVArmax=93.54, MVArmin=-20.4, PsetMW=183.26, QsetMVA=17.91)
    pyf.add_gen(grid, '4194.0', '180', price_zone_link=False, lf=0.0, qf=0.0, MWmax=151.2, MWmin=151.2, MVArmax=42.94, MVArmin=-29.03, PsetMW=151.2, QsetMVA=42.94)
    pyf.add_gen(grid, '4195.0', '181', price_zone_link=False, lf=0.0, qf=0.0, MWmax=69.0, MWmin=69.0, MVArmax=19.6, MVArmin=-13.25, PsetMW=69.0, QsetMVA=-13.25)
    pyf.add_gen(grid, '5020.0', '182', price_zone_link=False, lf=0.0, qf=0.0, MWmax=90.0, MWmin=90.0, MVArmax=25.56, MVArmin=-17.28, PsetMW=90.0, QsetMVA=-16.77)
    pyf.add_gen(grid, '5037.0', '186', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=67.41)
    pyf.add_gen(grid, '5038.0', '187', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=65.5)
    pyf.add_gen(grid, '5039.0', '188', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=148.79)
    pyf.add_gen(grid, '5040.0', '189', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=49.29)
    pyf.add_gen(grid, '5041.0', '190', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=71.22)
    pyf.add_gen(grid, '5042.0', '191', price_zone_link=False, lf=16.529, qf=0.001, MWmax=308.47, MWmin=92.54, MVArmax=157.01, MVArmin=-34.24, PsetMW=93.74, QsetMVA=61.19)
    pyf.add_gen(grid, '5051.0', '192', price_zone_link=False, lf=16.996, qf=0.001, MWmax=84.7, MWmin=25.41, MVArmax=43.11, MVArmin=-9.4, PsetMW=84.47, QsetMVA=43.11)
    pyf.add_gen(grid, '5058.0', '194', price_zone_link=False, lf=17.422, qf=0.001, MWmax=317.35, MWmin=95.2, MVArmax=161.53, MVArmin=-35.23, PsetMW=251.17, QsetMVA=-19.5)
    pyf.add_gen(grid, '5065.0', '195', price_zone_link=False, lf=15.989, qf=0.001, MWmax=10.6, MWmin=3.18, MVArmax=5.4, MVArmin=-1.18, PsetMW=10.57, QsetMVA=5.4)
    pyf.add_gen(grid, '5067.0', '197', price_zone_link=False, lf=17.987, qf=0.001, MWmax=438.27, MWmin=131.48, MVArmax=223.08000000000004, MVArmin=-48.65, PsetMW=437.05999999999995, QsetMVA=182.7)
    pyf.add_gen(grid, '5068.0', '198', price_zone_link=False, lf=17.987, qf=0.001, MWmax=438.27, MWmin=131.48, MVArmax=223.08000000000004, MVArmin=-48.65, PsetMW=437.05999999999995, QsetMVA=172.57)
    pyf.add_gen(grid, '5069.0', '199', price_zone_link=False, lf=17.987, qf=0.001, MWmax=438.27, MWmin=131.48, MVArmax=223.08000000000004, MVArmin=-48.65, PsetMW=437.05999999999995, QsetMVA=162.04)
    pyf.add_gen(grid, '5166.0', '203', price_zone_link=False, lf=16.531, qf=0.001, MWmax=126.49999999999999, MWmin=37.95, MVArmax=64.39, MVArmin=-14.04, PsetMW=126.15, QsetMVA=64.39)
    pyf.add_gen(grid, '5167.0', '204', price_zone_link=False, lf=15.947, qf=0.001, MWmax=1.6, MWmin=0.48, MVArmax=0.8100000000000002, MVArmin=-0.18, PsetMW=1.6, QsetMVA=0.8100000000000002)
    pyf.add_gen(grid, '5170.0', '205', price_zone_link=False, lf=0.0, qf=0.0, MWmax=45.0, MWmin=45.0, MVArmax=12.78, MVArmin=-8.64, PsetMW=45.0, QsetMVA=-8.64)
    pyf.add_gen(grid, '5198.0', '206', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-21.6)
    pyf.add_gen(grid, '5206.0', '207', price_zone_link=False, lf=17.101, qf=0.001, MWmax=248.67, MWmin=74.6, MVArmax=126.56999999999998, MVArmin=-27.6, PsetMW=207.10999999999999, QsetMVA=30.67)
    pyf.add_gen(grid, '5207.0', '208', price_zone_link=False, lf=17.101, qf=0.001, MWmax=248.67, MWmin=74.6, MVArmax=126.56999999999998, MVArmin=-27.6, PsetMW=180.01, QsetMVA=54.69)
    pyf.add_gen(grid, '5208.0', '209', price_zone_link=False, lf=17.101, qf=0.001, MWmax=248.67, MWmin=74.6, MVArmax=126.56999999999998, MVArmin=-27.6, PsetMW=180.01, QsetMVA=43.85)
    pyf.add_gen(grid, '5238.0', '210', price_zone_link=False, lf=0.0, qf=0.0, MWmax=168.75, MWmin=168.75, MVArmax=47.93, MVArmin=-32.4, PsetMW=168.75, QsetMVA=-32.4)
    pyf.add_gen(grid, '5243.0', '211', price_zone_link=False, lf=0.0, qf=0.0, MWmax=112.5, MWmin=112.5, MVArmax=31.95, MVArmin=-21.6, PsetMW=112.5, QsetMVA=-21.6)
    pyf.add_gen(grid, '5262.0', '212', price_zone_link=False, lf=6.975, qf=0.0, MWmax=1215.0, MWmin=364.5, MVArmax=447.11999999999995, MVArmin=-99.63, PsetMW=1211.63, QsetMVA=43.0)
    pyf.add_gen(grid, '5263.0', '213', price_zone_link=False, lf=8.11, qf=0.0, MWmax=1215.0, MWmin=364.5, MVArmax=447.11999999999995, MVArmin=-99.63, PsetMW=1024.01, QsetMVA=32.88)
    pyf.add_gen(grid, '5281.0', '214', price_zone_link=False, lf=19.596, qf=0.002, MWmax=267.73, MWmin=80.32, MVArmax=136.28, MVArmin=-29.719999999999995, PsetMW=266.99, QsetMVA=-29.719999999999995)
    pyf.add_gen(grid, '5282.0', '215', price_zone_link=False, lf=19.596, qf=0.002, MWmax=267.73, MWmin=80.32, MVArmax=136.28, MVArmin=-29.719999999999995, PsetMW=266.99, QsetMVA=-29.719999999999995)
    pyf.add_gen(grid, '5298.0', '218', price_zone_link=False, lf=17.143, qf=0.001, MWmax=114.99999999999999, MWmin=34.5, MVArmax=58.53, MVArmin=-12.76, PsetMW=114.68, QsetMVA=58.53)
    pyf.add_gen(grid, '5299.0', '219', price_zone_link=False, lf=17.143, qf=0.001, MWmax=114.99999999999999, MWmin=34.5, MVArmax=58.53, MVArmin=-12.76, PsetMW=114.68, QsetMVA=58.53)
    pyf.add_gen(grid, '5300.0', '220', price_zone_link=False, lf=17.143, qf=0.001, MWmax=114.99999999999999, MWmin=34.5, MVArmax=58.53, MVArmin=-12.76, PsetMW=114.68, QsetMVA=58.53)
    pyf.add_gen(grid, '5301.0', '221', price_zone_link=False, lf=16.986, qf=0.001, MWmax=82.7, MWmin=24.81, MVArmax=42.09, MVArmin=-9.18, PsetMW=82.47, QsetMVA=42.09)
    pyf.add_gen(grid, '5302.0', '222', price_zone_link=False, lf=18.839, qf=0.001, MWmax=463.75, MWmin=139.13, MVArmax=236.05, MVArmin=-51.47999999999999, PsetMW=462.46, QsetMVA=167.57)
    pyf.add_gen(grid, '5303.0', '223', price_zone_link=False, lf=18.839, qf=0.001, MWmax=463.75, MWmin=139.13, MVArmax=236.05, MVArmin=-51.47999999999999, PsetMW=462.46, QsetMVA=218.7)
    pyf.add_gen(grid, '5319.0', '224', price_zone_link=False, lf=19.57, qf=0.002, MWmax=262.8, MWmin=78.84, MVArmax=133.77, MVArmin=-29.17, PsetMW=79.87, QsetMVA=4.91)
    pyf.add_gen(grid, '5321.0', '226', price_zone_link=False, lf=19.57, qf=0.002, MWmax=262.8, MWmin=78.84, MVArmax=133.77, MVArmin=-29.17, PsetMW=79.87, QsetMVA=4.99)
    pyf.add_gen(grid, '5324.0', '227', price_zone_link=False, lf=16.42, qf=0.001, MWmax=284.07, MWmin=85.22, MVArmax=144.59, MVArmin=-31.53, PsetMW=283.28, QsetMVA=144.59)
    pyf.add_gen(grid, '5325.0', '228', price_zone_link=False, lf=16.42, qf=0.001, MWmax=284.07, MWmin=85.22, MVArmax=144.59, MVArmin=-31.53, PsetMW=283.28, QsetMVA=144.59)
    pyf.add_gen(grid, '5326.0', '229', price_zone_link=False, lf=16.42, qf=0.001, MWmax=284.07, MWmin=85.22, MVArmax=144.59, MVArmin=-31.53, PsetMW=283.28, QsetMVA=144.59)
    pyf.add_gen(grid, '5342.0', '230', price_zone_link=False, lf=0.0, qf=0.0, MWmax=8.4, MWmin=8.4, MVArmax=3.2, MVArmin=-0.14, PsetMW=8.4, QsetMVA=3.2)
    pyf.add_gen(grid, '5343.0', '231', price_zone_link=False, lf=0.0, qf=0.0, MWmax=207.50000000000003, MWmin=207.50000000000003, MVArmax=79.67, MVArmin=-3.4300000000000006, PsetMW=207.50000000000003, QsetMVA=60.28)
    pyf.add_gen(grid, '5360.0', '232', price_zone_link=False, lf=18.671, qf=0.002, MWmax=1008.0, MWmin=302.4, MVArmax=469.73, MVArmin=-122.98, PsetMW=1005.2100000000002, QsetMVA=-38.58)
    pyf.add_gen(grid, '5363.0', '233', price_zone_link=False, lf=15.988, qf=0.001, MWmax=10.4, MWmin=3.12, MVArmax=5.29, MVArmin=-1.15, PsetMW=10.37, QsetMVA=-1.15)
    pyf.add_gen(grid, '5364.0', '234', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-23.68)
    pyf.add_gen(grid, '5365.0', '235', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-12.23)
    pyf.add_gen(grid, '5366.0', '236', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-10.74)
    pyf.add_gen(grid, '5367.0', '237', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-10.68)
    pyf.add_gen(grid, '5368.0', '238', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-18.86)
    pyf.add_gen(grid, '5369.0', '239', price_zone_link=False, lf=16.936, qf=0.001, MWmax=213.33, MWmin=64.0, MVArmax=108.59, MVArmin=-23.68, PsetMW=212.74, QsetMVA=-9.95)
    pyf.add_gen(grid, '5382.0', '240', price_zone_link=False, lf=16.087, qf=0.001, MWmax=209.0, MWmin=62.7, MVArmax=106.37999999999998, MVArmin=-23.2, PsetMW=63.519999999999996, QsetMVA=106.37999999999998)
    pyf.add_gen(grid, '5383.0', '241', price_zone_link=False, lf=16.087, qf=0.001, MWmax=209.0, MWmin=62.7, MVArmax=106.37999999999998, MVArmin=-23.2, PsetMW=63.519999999999996, QsetMVA=106.37999999999998)
    pyf.add_gen(grid, '5399.0', '242', price_zone_link=False, lf=0.0, qf=0.0, MWmax=124.2, MWmin=124.2, MVArmax=35.27, MVArmin=-23.85, PsetMW=124.2, QsetMVA=35.27)
    pyf.add_gen(grid, '5403.0', '243', price_zone_link=False, lf=16.357, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=89.15, QsetMVA=45.5)
    pyf.add_gen(grid, '5404.0', '244', price_zone_link=False, lf=16.357, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=77.23, QsetMVA=45.5)
    pyf.add_gen(grid, '5405.0', '245', price_zone_link=False, lf=16.357, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=84.77, QsetMVA=45.5)
    pyf.add_gen(grid, '5406.0', '246', price_zone_link=False, lf=16.357, qf=0.001, MWmax=89.4, MWmin=26.82, MVArmax=45.5, MVArmin=-9.92, PsetMW=89.15, QsetMVA=45.5)
    pyf.add_gen(grid, '5415.0', '247', price_zone_link=False, lf=16.51, qf=0.001, MWmax=122.0, MWmin=36.6, MVArmax=62.1, MVArmin=-13.54, PsetMW=121.66, QsetMVA=-13.54)
    pyf.add_gen(grid, '5416.0', '248', price_zone_link=False, lf=16.51, qf=0.001, MWmax=122.0, MWmin=36.6, MVArmax=62.1, MVArmin=-13.54, PsetMW=121.66, QsetMVA=-13.54)
    pyf.add_gen(grid, '5417.0', '249', price_zone_link=False, lf=16.51, qf=0.001, MWmax=122.0, MWmin=36.6, MVArmax=62.1, MVArmin=-13.54, PsetMW=121.66, QsetMVA=-13.54)
    pyf.add_gen(grid, '5418.0', '250', price_zone_link=False, lf=16.495, qf=0.001, MWmax=118.8, MWmin=35.64, MVArmax=60.47, MVArmin=-13.19, PsetMW=118.47000000000001, QsetMVA=-13.19)
    pyf.add_gen(grid, '5419.0', '251', price_zone_link=False, lf=16.495, qf=0.001, MWmax=118.8, MWmin=35.64, MVArmax=60.47, MVArmin=-13.19, PsetMW=118.47000000000001, QsetMVA=-13.19)
    pyf.add_gen(grid, '5423.0', '252', price_zone_link=False, lf=15.998, qf=0.001, MWmax=12.4, MWmin=3.72, MVArmax=6.309999999999999, MVArmin=-1.38, PsetMW=12.37, QsetMVA=-1.38)
    pyf.add_gen(grid, '5444.0', '257', price_zone_link=False, lf=19.71, qf=0.002, MWmax=289.0, MWmin=86.7, MVArmax=147.1, MVArmin=-32.08, PsetMW=288.2, QsetMVA=9.19)
    pyf.add_gen(grid, '5444.0', '258', price_zone_link=False, lf=19.71, qf=0.002, MWmax=289.0, MWmin=86.7, MVArmax=147.1, MVArmin=-32.08, PsetMW=288.2, QsetMVA=9.19)
    pyf.add_gen(grid, '5444.0', '260', price_zone_link=False, lf=18.923, qf=0.002, MWmax=141.3, MWmin=42.39, MVArmax=71.92, MVArmin=-15.68, PsetMW=140.91, QsetMVA=9.19)
    pyf.add_gen(grid, '5444.0', '261', price_zone_link=False, lf=18.923, qf=0.002, MWmax=141.3, MWmin=42.39, MVArmax=71.92, MVArmin=-15.68, PsetMW=140.91, QsetMVA=9.19)
    pyf.add_gen(grid, '6006.0', '262', price_zone_link=False, lf=17.595, qf=0.001, MWmax=213.0, MWmin=63.9, MVArmax=108.42, MVArmin=-23.64, PsetMW=212.41, QsetMVA=67.91)
    pyf.add_gen(grid, '6008.0', '264', price_zone_link=False, lf=17.524, qf=0.001, MWmax=198.33, MWmin=59.5, MVArmax=100.95, MVArmin=-22.01, PsetMW=197.78, QsetMVA=59.650000000000006)
    pyf.add_gen(grid, '6009.0', '265', price_zone_link=False, lf=17.524, qf=0.001, MWmax=198.33, MWmin=59.5, MVArmax=100.95, MVArmin=-22.01, PsetMW=197.78, QsetMVA=37.33)
    pyf.add_gen(grid, '6010.0', '266', price_zone_link=False, lf=17.524, qf=0.001, MWmax=198.33, MWmin=59.5, MVArmax=100.95, MVArmin=-22.01, PsetMW=197.78, QsetMVA=50.24000000000001)
    pyf.add_gen(grid, '6041.0', '267', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=20.48, MVArmin=-0.88, PsetMW=0.0, QsetMVA=20.48)
    pyf.add_gen(grid, '6042.0', '268', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=122.90999999999998, MVArmin=-5.29, PsetMW=0.0, QsetMVA=47.53)
    pyf.add_gen(grid, '6051.0', '272', price_zone_link=False, lf=15.631, qf=0.001, MWmax=57.4, MWmin=17.22, MVArmax=29.220000000000002, MVArmin=-6.370000000000001, PsetMW=57.24, QsetMVA=29.220000000000002)
    pyf.add_gen(grid, '6053.0', '274', price_zone_link=False, lf=15.631, qf=0.001, MWmax=57.4, MWmin=17.22, MVArmax=29.220000000000002, MVArmin=-6.370000000000001, PsetMW=57.24, QsetMVA=29.220000000000002)
    pyf.add_gen(grid, '6054.0', '275', price_zone_link=False, lf=15.631, qf=0.001, MWmax=57.4, MWmin=17.22, MVArmax=29.220000000000002, MVArmin=-6.370000000000001, PsetMW=57.24, QsetMVA=26.14)
    pyf.add_gen(grid, '6069.0', '276', price_zone_link=False, lf=17.442, qf=0.001, MWmax=181.37, MWmin=54.40999999999999, MVArmax=92.32, MVArmin=-20.13, PsetMW=123.82, QsetMVA=30.75)
    pyf.add_gen(grid, '6070.0', '277', price_zone_link=False, lf=17.442, qf=0.001, MWmax=181.37, MWmin=54.40999999999999, MVArmax=92.32, MVArmin=-20.13, PsetMW=131.29, QsetMVA=13.73)
    pyf.add_gen(grid, '6071.0', '278', price_zone_link=False, lf=17.442, qf=0.001, MWmax=181.37, MWmin=54.40999999999999, MVArmax=92.32, MVArmin=-20.13, PsetMW=105.90000000000002, QsetMVA=39.69)
    pyf.add_gen(grid, '6073.0', '280', price_zone_link=False, lf=17.442, qf=0.001, MWmax=181.37, MWmin=54.40999999999999, MVArmax=92.32, MVArmin=-20.13, PsetMW=131.29, QsetMVA=23.42)
    pyf.add_gen(grid, '6074.0', '281', price_zone_link=False, lf=17.442, qf=0.001, MWmax=181.37, MWmin=54.40999999999999, MVArmax=92.32, MVArmin=-20.13, PsetMW=131.29, QsetMVA=22.78)
    pyf.add_gen(grid, '6078.0', '282', price_zone_link=False, lf=17.597, qf=0.002, MWmax=563.33, MWmin=169.0, MVArmax=262.51, MVArmin=-68.73, PsetMW=477.31000000000006, QsetMVA=-68.73)
    pyf.add_gen(grid, '6079.0', '283', price_zone_link=False, lf=17.597, qf=0.002, MWmax=563.33, MWmin=169.0, MVArmax=262.51, MVArmin=-68.73, PsetMW=328.93, QsetMVA=-68.73)
    pyf.add_gen(grid, '6080.0', '284', price_zone_link=False, lf=17.597, qf=0.002, MWmax=563.33, MWmin=169.0, MVArmax=262.51, MVArmin=-68.73, PsetMW=328.93, QsetMVA=-68.73)
    pyf.add_gen(grid, '6082.0', '285', price_zone_link=False, lf=0.0, qf=0.0, MWmax=41.37, MWmin=41.37, MVArmax=25.040000000000003, MVArmin=-1.08, PsetMW=41.37, QsetMVA=13.849999999999998)
    pyf.add_gen(grid, '6083.0', '286', price_zone_link=False, lf=0.0, qf=0.0, MWmax=65.3, MWmin=65.3, MVArmax=25.040000000000003, MVArmin=-1.08, PsetMW=65.3, QsetMVA=10.38)
    pyf.add_gen(grid, '6085.0', '287', price_zone_link=False, lf=0.0, qf=0.0, MWmax=1.05, MWmin=1.05, MVArmax=0.0, MVArmin=0.0, PsetMW=1.05, QsetMVA=0.0)
    pyf.add_gen(grid, '6086.0', '288', price_zone_link=False, lf=0.0, qf=0.0, MWmax=22.5, MWmin=22.5, MVArmax=0.0, MVArmin=0.0, PsetMW=22.5, QsetMVA=0.0)
    pyf.add_gen(grid, '6088.0', '290', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.95, MWmin=7.95, MVArmax=0.0, MVArmin=0.0, PsetMW=7.95, QsetMVA=0.0)
    pyf.add_gen(grid, '6089.0', '291', price_zone_link=False, lf=0.0, qf=0.0, MWmax=10.42, MWmin=10.42, MVArmax=0.0, MVArmin=0.0, PsetMW=10.42, QsetMVA=0.0)
    pyf.add_gen(grid, '6090.0', '292', price_zone_link=False, lf=0.0, qf=0.0, MWmax=30.519999999999996, MWmin=30.519999999999996, MVArmax=0.0, MVArmin=0.0, PsetMW=30.519999999999996, QsetMVA=0.0)
    pyf.add_gen(grid, '6104.0', '293', price_zone_link=False, lf=16.855, qf=0.001, MWmax=60.5, MWmin=18.15, MVArmax=30.79, MVArmin=-6.72, PsetMW=60.33, QsetMVA=30.79)
    pyf.add_gen(grid, '6105.0', '294', price_zone_link=False, lf=16.855, qf=0.001, MWmax=60.5, MWmin=18.15, MVArmax=30.79, MVArmin=-6.72, PsetMW=60.33, QsetMVA=30.79)
    pyf.add_gen(grid, '6106.0', '295', price_zone_link=False, lf=17.149, qf=0.001, MWmax=121.0, MWmin=36.3, MVArmax=61.59, MVArmin=-13.43, PsetMW=120.66, QsetMVA=61.59)
    pyf.add_gen(grid, '6110.0', '296', price_zone_link=False, lf=17.597, qf=0.002, MWmax=590.6, MWmin=177.18, MVArmax=275.22, MVArmin=-72.05, PsetMW=301.19, QsetMVA=88.04)
    pyf.add_gen(grid, '6111.0', '297', price_zone_link=False, lf=17.597, qf=0.002, MWmax=661.5, MWmin=198.45, MVArmax=308.26, MVArmin=-80.7, PsetMW=293.64, QsetMVA=88.94)
    pyf.add_gen(grid, '6145.0', '299', price_zone_link=False, lf=17.383, qf=0.001, MWmax=446.0, MWmin=133.8, MVArmax=227.01, MVArmin=-49.51, PsetMW=322.86, QsetMVA=26.170000000000005)
    pyf.add_gen(grid, '6146.0', '300', price_zone_link=False, lf=17.383, qf=0.001, MWmax=446.0, MWmin=133.8, MVArmax=227.01, MVArmin=-49.51, PsetMW=322.86, QsetMVA=27.07)
    pyf.add_gen(grid, '6147.0', '301', price_zone_link=False, lf=19.572, qf=0.001, MWmax=932.0, MWmin=279.6, MVArmax=474.39, MVArmin=-103.45, PsetMW=283.24, QsetMVA=28.49)
    pyf.add_gen(grid, '6149.0', '303', price_zone_link=False, lf=18.626, qf=0.001, MWmax=722.0, MWmin=216.6, MVArmax=367.5, MVArmin=-80.14, PsetMW=219.42, QsetMVA=83.11)
    pyf.add_gen(grid, '6164.0', '304', price_zone_link=False, lf=17.702, qf=0.001, MWmax=235.05, MWmin=70.52, MVArmax=119.63999999999999, MVArmin=-26.090000000000003, PsetMW=71.44, QsetMVA=26.96)
    pyf.add_gen(grid, '6165.0', '305', price_zone_link=False, lf=17.702, qf=0.001, MWmax=235.05, MWmin=70.52, MVArmax=119.63999999999999, MVArmin=-26.090000000000003, PsetMW=71.44, QsetMVA=23.81)
    pyf.add_gen(grid, '6166.0', '306', price_zone_link=False, lf=17.702, qf=0.001, MWmax=235.05, MWmin=70.52, MVArmax=119.63999999999999, MVArmin=-26.090000000000003, PsetMW=71.44, QsetMVA=57.82000000000001)
    pyf.add_gen(grid, '6167.0', '307', price_zone_link=False, lf=17.702, qf=0.001, MWmax=235.05, MWmin=70.52, MVArmax=119.63999999999999, MVArmin=-26.090000000000003, PsetMW=71.44, QsetMVA=30.769999999999996)
    pyf.add_gen(grid, '6213.0', '308', price_zone_link=False, lf=0.0, qf=0.0, MWmax=177.9, MWmin=177.9, MVArmax=68.29, MVArmin=-2.94, PsetMW=177.9, QsetMVA=16.01)
    pyf.add_gen(grid, '6214.0', '309', price_zone_link=False, lf=0.0, qf=0.0, MWmax=552.3, MWmin=552.3, MVArmax=212.06, MVArmin=-9.13, PsetMW=552.3, QsetMVA=49.93)
    pyf.add_gen(grid, '6215.0', '310', price_zone_link=False, lf=0.0, qf=0.0, MWmax=552.3, MWmin=552.3, MVArmax=212.06, MVArmin=-9.13, PsetMW=552.3, QsetMVA=78.32)
    pyf.add_gen(grid, '6216.0', '311', price_zone_link=False, lf=0.0, qf=0.0, MWmax=552.3, MWmin=552.3, MVArmax=212.06, MVArmin=-9.13, PsetMW=552.3, QsetMVA=49.18)
    pyf.add_gen(grid, '6242.0', '312', price_zone_link=False, lf=15.648, qf=0.001, MWmax=61.0, MWmin=18.3, MVArmax=31.05, MVArmin=-6.77, PsetMW=60.83, QsetMVA=15.71)
    pyf.add_gen(grid, '6243.0', '313', price_zone_link=False, lf=16.197, qf=0.001, MWmax=183.0, MWmin=54.89999999999999, MVArmax=93.15, MVArmin=-20.31, PsetMW=182.49, QsetMVA=38.29)
    pyf.add_gen(grid, '6244.0', '314', price_zone_link=False, lf=16.716, qf=0.001, MWmax=298.0, MWmin=89.4, MVArmax=151.68, MVArmin=-33.08, PsetMW=297.17, QsetMVA=46.74)
    pyf.add_gen(grid, '6245.0', '315', price_zone_link=False, lf=16.716, qf=0.001, MWmax=298.0, MWmin=89.4, MVArmax=151.68, MVArmin=-33.08, PsetMW=297.17, QsetMVA=53.68999999999999)
    pyf.add_gen(grid, '6246.0', '316', price_zone_link=False, lf=16.716, qf=0.001, MWmax=298.0, MWmin=89.4, MVArmax=151.68, MVArmin=-33.08, PsetMW=297.17, QsetMVA=100.49999999999999)
    pyf.add_gen(grid, '6247.0', '317', price_zone_link=False, lf=16.199, qf=0.001, MWmax=183.33, MWmin=55.00000000000001, MVArmax=93.32, MVArmin=-20.35, PsetMW=182.82, QsetMVA=82.2)
    pyf.add_gen(grid, '6248.0', '318', price_zone_link=False, lf=16.199, qf=0.001, MWmax=183.33, MWmin=55.00000000000001, MVArmax=93.32, MVArmin=-20.35, PsetMW=182.82, QsetMVA=42.38)
    pyf.add_gen(grid, '6249.0', '319', price_zone_link=False, lf=16.199, qf=0.001, MWmax=183.33, MWmin=55.00000000000001, MVArmax=93.32, MVArmin=-20.35, PsetMW=182.82, QsetMVA=29.31)
    pyf.add_gen(grid, '6257.0', '320', price_zone_link=False, lf=17.503, qf=0.001, MWmax=194.0, MWmin=58.20000000000001, MVArmax=98.75, MVArmin=-21.53, PsetMW=58.96, QsetMVA=-21.53)
    pyf.add_gen(grid, '6258.0', '321', price_zone_link=False, lf=17.503, qf=0.001, MWmax=194.0, MWmin=58.20000000000001, MVArmax=98.75, MVArmin=-21.53, PsetMW=106.96, QsetMVA=-12.72)
    pyf.add_gen(grid, '6260.0', '323', price_zone_link=False, lf=17.809, qf=0.001, MWmax=257.0, MWmin=77.1, MVArmax=130.81, MVArmin=-28.53, PsetMW=78.1, QsetMVA=-26.56)
    pyf.add_gen(grid, '6266.0', '324', price_zone_link=False, lf=0.0, qf=0.0, MWmax=4.5, MWmin=4.5, MVArmax=1.72, MVArmin=-0.07, PsetMW=4.5, QsetMVA=1.72)
    pyf.add_gen(grid, '6267.0', '325', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.200000000000001, MWmin=7.200000000000001, MVArmax=2.73, MVArmin=-0.12, PsetMW=7.200000000000001, QsetMVA=2.73)
    pyf.add_gen(grid, '6268.0', '326', price_zone_link=False, lf=0.0, qf=0.0, MWmax=4.5, MWmin=4.5, MVArmax=1.72, MVArmin=-0.07, PsetMW=4.5, QsetMVA=1.72)
    pyf.add_gen(grid, '6269.0', '327', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.200000000000001, MWmin=7.200000000000001, MVArmax=2.73, MVArmin=-0.12, PsetMW=7.200000000000001, QsetMVA=2.73)
    pyf.add_gen(grid, '6274.0', '332', price_zone_link=False, lf=0.0, qf=0.0, MWmax=8.4, MWmin=8.4, MVArmax=3.2, MVArmin=-0.14, PsetMW=8.4, QsetMVA=3.2)
    pyf.add_gen(grid, '6295.0', '333', price_zone_link=False, lf=17.562, qf=0.001, MWmax=206.23, MWmin=61.870000000000005, MVArmax=104.97000000000001, MVArmin=-22.89, PsetMW=62.67, QsetMVA=-22.89)
    pyf.add_gen(grid, '6296.0', '334', price_zone_link=False, lf=17.562, qf=0.001, MWmax=206.23, MWmin=61.870000000000005, MVArmax=104.97000000000001, MVArmin=-22.89, PsetMW=62.67, QsetMVA=-22.89)
    pyf.add_gen(grid, '6297.0', '335', price_zone_link=False, lf=17.562, qf=0.001, MWmax=206.23, MWmin=61.9, MVArmax=104.97000000000001, MVArmin=-22.89, PsetMW=62.7, QsetMVA=-22.23)
    pyf.add_gen(grid, '6302.0', '337', price_zone_link=False, lf=17.762, qf=0.001, MWmax=247.25, MWmin=74.18, MVArmax=125.85, MVArmin=-27.440000000000005, PsetMW=246.56000000000003, QsetMVA=-27.440000000000005)
    pyf.add_gen(grid, '6303.0', '338', price_zone_link=False, lf=17.762, qf=0.001, MWmax=247.25, MWmin=74.18, MVArmax=125.85, MVArmin=-27.440000000000005, PsetMW=213.83, QsetMVA=-27.440000000000005)
    pyf.add_gen(grid, '6304.0', '339', price_zone_link=False, lf=17.762, qf=0.001, MWmax=247.25, MWmin=74.18, MVArmax=125.85, MVArmin=-27.440000000000005, PsetMW=246.56000000000003, QsetMVA=-27.440000000000005)
    pyf.add_gen(grid, '6305.0', '340', price_zone_link=False, lf=17.762, qf=0.001, MWmax=247.25, MWmin=74.18, MVArmax=125.85, MVArmin=-27.440000000000005, PsetMW=246.56000000000003, QsetMVA=-27.440000000000005)
    pyf.add_gen(grid, '6349.0', '343', price_zone_link=False, lf=17.061, qf=0.001, MWmax=103.0, MWmin=30.9, MVArmax=52.43, MVArmin=-11.43, PsetMW=31.3, QsetMVA=52.43)
    pyf.add_gen(grid, '6349.0', '349', price_zone_link=False, lf=18.323, qf=0.001, MWmax=363.0, MWmin=108.89999999999999, MVArmax=184.77, MVArmin=-40.29, PsetMW=110.32, QsetMVA=184.77)
    pyf.add_gen(grid, '6349.0', '350', price_zone_link=False, lf=18.323, qf=0.001, MWmax=363.0, MWmin=108.89999999999999, MVArmax=184.77, MVArmin=-40.29, PsetMW=110.32, QsetMVA=184.77)
    pyf.add_gen(grid, '7005.0', '352', price_zone_link=False, lf=15.681, qf=0.001, MWmax=163.0, MWmin=48.9, MVArmax=82.97, MVArmin=-18.09, PsetMW=162.55, QsetMVA=35.32)
    pyf.add_gen(grid, '7006.0', '353', price_zone_link=False, lf=15.681, qf=0.001, MWmax=163.0, MWmin=48.9, MVArmax=82.97, MVArmin=-18.09, PsetMW=162.55, QsetMVA=61.84)
    pyf.add_gen(grid, '7007.0', '354', price_zone_link=False, lf=15.681, qf=0.001, MWmax=163.0, MWmin=48.9, MVArmax=82.97, MVArmin=-18.09, PsetMW=162.55, QsetMVA=40.83)
    pyf.add_gen(grid, '7008.0', '355', price_zone_link=False, lf=15.681, qf=0.001, MWmax=163.0, MWmin=48.9, MVArmax=82.97, MVArmin=-18.09, PsetMW=162.55, QsetMVA=38.29)
    pyf.add_gen(grid, '7009.0', '356', price_zone_link=False, lf=15.681, qf=0.001, MWmax=163.0, MWmin=48.9, MVArmax=82.97, MVArmin=-18.09, PsetMW=162.55, QsetMVA=34.73)
    pyf.add_gen(grid, '7021.0', '358', price_zone_link=False, lf=16.624, qf=0.001, MWmax=22.4, MWmin=6.72, MVArmax=11.4, MVArmin=-2.49, PsetMW=22.34, QsetMVA=11.4)
    pyf.add_gen(grid, '7022.0', '359', price_zone_link=False, lf=16.745, qf=0.001, MWmax=47.4, MWmin=14.219999999999999, MVArmax=24.13, MVArmin=-5.26, PsetMW=47.27, QsetMVA=24.13)
    pyf.add_gen(grid, '7023.0', '360', price_zone_link=False, lf=16.745, qf=0.001, MWmax=47.4, MWmin=14.219999999999999, MVArmax=24.13, MVArmin=-5.26, PsetMW=47.27, QsetMVA=24.13)
    pyf.add_gen(grid, '7024.0', '361', price_zone_link=False, lf=16.974, qf=0.001, MWmax=94.8, MWmin=28.439999999999998, MVArmax=48.25, MVArmin=-10.52, PsetMW=94.54, QsetMVA=48.25)
    pyf.add_gen(grid, '7031.0', '363', price_zone_link=False, lf=0.0, qf=0.0, MWmax=29.100000000000005, MWmin=29.100000000000005, MVArmax=0.0, MVArmin=0.0, PsetMW=29.100000000000005, QsetMVA=0.0)
    pyf.add_gen(grid, '7032.0', '364', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.5, MWmin=7.5, MVArmax=0.0, MVArmin=0.0, PsetMW=7.5, QsetMVA=0.0)
    pyf.add_gen(grid, '7033.0', '365', price_zone_link=False, lf=0.0, qf=0.0, MWmax=7.5, MWmin=7.5, MVArmax=0.0, MVArmin=0.0, PsetMW=7.5, QsetMVA=0.0)
    pyf.add_gen(grid, '7045.0', '367', price_zone_link=False, lf=17.428, qf=0.001, MWmax=188.45, MWmin=56.53, MVArmax=95.92, MVArmin=-20.92, PsetMW=187.93, QsetMVA=29.909999999999997)
    pyf.add_gen(grid, '7046.0', '368', price_zone_link=False, lf=17.428, qf=0.001, MWmax=188.45, MWmin=56.53, MVArmax=95.92, MVArmin=-20.92, PsetMW=187.93, QsetMVA=62.63000000000001)
    pyf.add_gen(grid, '7061.0', '369', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=67.09, QsetMVA=-12.75)
    pyf.add_gen(grid, '7062.0', '370', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=67.09, QsetMVA=-12.75)
    pyf.add_gen(grid, '7063.0', '371', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=67.09, QsetMVA=-12.75)
    pyf.add_gen(grid, '7064.0', '372', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=83.18, QsetMVA=-12.75)
    pyf.add_gen(grid, '7065.0', '373', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=83.18, QsetMVA=-12.75)
    pyf.add_gen(grid, '7066.0', '374', price_zone_link=False, lf=17.072, qf=0.001, MWmax=114.9, MWmin=34.47, MVArmax=58.48, MVArmin=-12.75, PsetMW=68.11, QsetMVA=-12.75)
    pyf.add_gen(grid, '7079.0', '375', price_zone_link=False, lf=15.969, qf=0.001, MWmax=228.65000000000003, MWmin=68.59, MVArmax=116.38, MVArmin=-25.379999999999995, PsetMW=159.89, QsetMVA=34.45)
    pyf.add_gen(grid, '7080.0', '376', price_zone_link=False, lf=15.969, qf=0.001, MWmax=228.65000000000003, MWmin=68.59, MVArmax=116.38, MVArmin=-25.379999999999995, PsetMW=165.52, QsetMVA=26.979999999999997)
    pyf.add_gen(grid, '7081.0', '377', price_zone_link=False, lf=15.969, qf=0.001, MWmax=228.65000000000003, MWmin=68.59, MVArmax=116.38, MVArmin=-25.379999999999995, PsetMW=165.52, QsetMVA=27.33)
    pyf.add_gen(grid, '7082.0', '378', price_zone_link=False, lf=15.969, qf=0.001, MWmax=228.65000000000003, MWmin=68.59, MVArmax=116.38, MVArmin=-25.379999999999995, PsetMW=165.52, QsetMVA=33.35)
    pyf.add_gen(grid, '7098.0', '379', price_zone_link=False, lf=7.063, qf=0.0, MWmax=1354.3, MWmin=406.29, MVArmax=498.37999999999994, MVArmin=-111.05000000000001, PsetMW=1236.1, QsetMVA=192.11)
    pyf.add_gen(grid, '7099.0', '380', price_zone_link=False, lf=7.445, qf=0.0, MWmax=1354.3, MWmin=406.29, MVArmax=498.37999999999994, MVArmin=-111.05000000000001, PsetMW=1350.55, QsetMVA=-7.380000000000001)
    pyf.add_gen(grid, '7107.0', '381', price_zone_link=False, lf=15.624, qf=0.001, MWmax=150.0, MWmin=45.0, MVArmax=76.35, MVArmin=-16.65, PsetMW=149.58, QsetMVA=76.35)
    pyf.add_gen(grid, '7108.0', '382', price_zone_link=False, lf=15.19, qf=0.001, MWmax=51.0, MWmin=15.299999999999999, MVArmax=25.96, MVArmin=-5.66, PsetMW=50.85999999999999, QsetMVA=18.53)
    pyf.add_gen(grid, '7109.0', '383', price_zone_link=False, lf=15.19, qf=0.001, MWmax=51.0, MWmin=15.299999999999999, MVArmax=25.96, MVArmin=-5.66, PsetMW=50.85999999999999, QsetMVA=22.51)
    pyf.add_gen(grid, '7110.0', '384', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=46.37)
    pyf.add_gen(grid, '7111.0', '385', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=45.21)
    pyf.add_gen(grid, '7112.0', '386', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=71.76)
    pyf.add_gen(grid, '7113.0', '387', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=38.8)
    pyf.add_gen(grid, '7114.0', '388', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=76.15)
    pyf.add_gen(grid, '7115.0', '389', price_zone_link=False, lf=15.826, qf=0.001, MWmax=196.0, MWmin=58.8, MVArmax=99.76, MVArmin=-21.76, PsetMW=195.46, QsetMVA=76.51)
    pyf.add_gen(grid, '7133.0', '392', price_zone_link=False, lf=15.037, qf=0.001, MWmax=16.3, MWmin=4.89, MVArmax=8.3, MVArmin=-1.81, PsetMW=16.25, QsetMVA=-1.81)
    pyf.add_gen(grid, '7134.0', '393', price_zone_link=False, lf=15.872, qf=0.001, MWmax=206.55, MWmin=61.970000000000006, MVArmax=105.13, MVArmin=-22.93, PsetMW=205.98000000000002, QsetMVA=-22.93)
    pyf.add_gen(grid, '7135.0', '394', price_zone_link=False, lf=15.872, qf=0.001, MWmax=206.55, MWmin=61.970000000000006, MVArmax=105.13, MVArmin=-22.93, PsetMW=205.98000000000002, QsetMVA=-22.93)
    pyf.add_gen(grid, '7136.0', '395', price_zone_link=False, lf=15.872, qf=0.001, MWmax=206.55, MWmin=61.970000000000006, MVArmax=105.13, MVArmin=-22.93, PsetMW=205.98000000000002, QsetMVA=-22.93)
    pyf.add_gen(grid, '7137.0', '396', price_zone_link=False, lf=15.872, qf=0.001, MWmax=206.55, MWmin=61.970000000000006, MVArmax=105.13, MVArmin=-22.93, PsetMW=205.98000000000002, QsetMVA=-22.93)
    pyf.add_gen(grid, '7138.0', '397', price_zone_link=False, lf=16.636, qf=0.001, MWmax=380.7, MWmin=114.21, MVArmax=193.78, MVArmin=-42.26, PsetMW=379.64, QsetMVA=-42.26)
    pyf.add_gen(grid, '7139.0', '398', price_zone_link=False, lf=15.225, qf=0.001, MWmax=59.0, MWmin=17.7, MVArmax=30.03, MVArmin=-6.550000000000001, PsetMW=58.84, QsetMVA=-6.550000000000001)
    pyf.add_gen(grid, '7140.0', '399', price_zone_link=False, lf=15.225, qf=0.001, MWmax=59.0, MWmin=17.7, MVArmax=30.03, MVArmin=-6.550000000000001, PsetMW=58.84, QsetMVA=-6.550000000000001)
    pyf.add_gen(grid, '7141.0', '400', price_zone_link=False, lf=15.225, qf=0.001, MWmax=59.0, MWmin=17.7, MVArmax=30.03, MVArmin=-6.550000000000001, PsetMW=58.84, QsetMVA=-6.550000000000001)
    pyf.add_gen(grid, '7142.0', '401', price_zone_link=False, lf=15.225, qf=0.001, MWmax=59.0, MWmin=17.7, MVArmax=30.03, MVArmin=-6.550000000000001, PsetMW=58.84, QsetMVA=-6.550000000000001)
    pyf.add_gen(grid, '7146.0', '402', price_zone_link=False, lf=15.011, qf=0.001, MWmax=10.2, MWmin=3.06, MVArmax=5.19, MVArmin=-1.13, PsetMW=10.17, QsetMVA=-1.13)
    pyf.add_gen(grid, '7162.0', '403', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.18, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=56.3)
    pyf.add_gen(grid, '7163.0', '404', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.18, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=52.72)
    pyf.add_gen(grid, '7164.0', '405', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.18, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=56.3)
    pyf.add_gen(grid, '7165.0', '406', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.18, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=56.3)
    pyf.add_gen(grid, '7166.0', '407', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.18, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=45.19)
    pyf.add_gen(grid, '7167.0', '408', price_zone_link=False, lf=15.451, qf=0.001, MWmax=110.6, MWmin=33.2, MVArmax=56.3, MVArmin=-12.28, PsetMW=110.28999999999999, QsetMVA=37.63)
    pyf.add_gen(grid, '7168.0', '409', price_zone_link=False, lf=15.351, qf=0.001, MWmax=87.72, MWmin=26.32, MVArmax=44.65, MVArmin=-9.74, PsetMW=87.48, QsetMVA=44.65)
    pyf.add_gen(grid, '7169.0', '410', price_zone_link=False, lf=15.351, qf=0.001, MWmax=87.72, MWmin=26.3, MVArmax=44.65, MVArmin=-9.74, PsetMW=87.48, QsetMVA=44.65)
    pyf.add_gen(grid, '7170.0', '411', price_zone_link=False, lf=16.505, qf=0.001, MWmax=350.87, MWmin=105.25999999999999, MVArmax=178.59, MVArmin=-38.95, PsetMW=333.76, QsetMVA=107.98)
    pyf.add_gen(grid, '7189.0', '412', price_zone_link=False, lf=14.996, qf=0.001, MWmax=6.9, MWmin=2.07, MVArmax=3.51, MVArmin=-0.77, PsetMW=6.88, QsetMVA=3.51)
    pyf.add_gen(grid, '7190.0', '413', price_zone_link=False, lf=15.177, qf=0.001, MWmax=48.0, MWmin=14.400000000000002, MVArmax=24.43, MVArmin=-5.33, PsetMW=47.87, QsetMVA=12.12)
    pyf.add_gen(grid, '7192.0', '415', price_zone_link=False, lf=15.119, qf=0.001, MWmax=35.0, MWmin=10.5, MVArmax=17.81, MVArmin=-3.88, PsetMW=34.9, QsetMVA=17.19)
    pyf.add_gen(grid, '7193.0', '416', price_zone_link=False, lf=14.999, qf=0.001, MWmax=7.5, MWmin=2.25, MVArmax=3.82, MVArmin=-0.83, PsetMW=7.48, QsetMVA=3.82)
    pyf.add_gen(grid, '7194.0', '417', price_zone_link=False, lf=15.979, qf=0.001, MWmax=230.95, MWmin=69.28, MVArmax=117.55, MVArmin=-25.64, PsetMW=230.31, QsetMVA=43.09)
    pyf.add_gen(grid, '7195.0', '418', price_zone_link=False, lf=15.979, qf=0.001, MWmax=230.95, MWmin=69.28, MVArmax=117.55, MVArmin=-25.64, PsetMW=230.31, QsetMVA=37.11)
    pyf.add_gen(grid, '7196.0', '419', price_zone_link=False, lf=15.979, qf=0.001, MWmax=230.95, MWmin=69.28, MVArmax=117.55, MVArmin=-25.64, PsetMW=230.31, QsetMVA=67.13)
    pyf.add_gen(grid, '7197.0', '420', price_zone_link=False, lf=15.979, qf=0.001, MWmax=230.95, MWmin=69.28, MVArmax=117.55, MVArmin=-25.64, PsetMW=230.31, QsetMVA=104.23)
    pyf.add_gen(grid, '7208.0', '422', price_zone_link=False, lf=18.322, qf=0.001, MWmax=765.0, MWmin=229.5, MVArmax=389.39, MVArmin=-84.92, PsetMW=762.88, QsetMVA=79.03)
    pyf.add_gen(grid, '7209.0', '423', price_zone_link=False, lf=18.322, qf=0.001, MWmax=765.0, MWmin=229.5, MVArmax=389.39, MVArmin=-84.92, PsetMW=762.88, QsetMVA=102.56)
    pyf.add_gen(grid, '7210.0', '424', price_zone_link=False, lf=15.749, qf=0.001, MWmax=178.5, MWmin=53.55, MVArmax=90.86, MVArmin=-19.81, PsetMW=178.01, QsetMVA=26.74)
    pyf.add_gen(grid, '7211.0', '425', price_zone_link=False, lf=15.749, qf=0.001, MWmax=178.5, MWmin=53.55, MVArmax=90.86, MVArmin=-19.81, PsetMW=178.01, QsetMVA=37.53)
    pyf.add_gen(grid, '7212.0', '426', price_zone_link=False, lf=15.749, qf=0.001, MWmax=178.5, MWmin=53.55, MVArmax=90.86, MVArmin=-19.81, PsetMW=178.01, QsetMVA=24.45)
    pyf.add_gen(grid, '7266.0', '427', price_zone_link=False, lf=16.302, qf=0.001, MWmax=304.6, MWmin=91.38, MVArmax=155.04, MVArmin=-33.81, PsetMW=303.76, QsetMVA=-33.81)
    pyf.add_gen(grid, '7267.0', '428', price_zone_link=False, lf=15.364, qf=0.001, MWmax=90.64, MWmin=27.190000000000005, MVArmax=46.14, MVArmin=-10.06, PsetMW=90.39, QsetMVA=-10.06)
    pyf.add_gen(grid, '7268.0', '429', price_zone_link=False, lf=15.364, qf=0.001, MWmax=90.64, MWmin=27.190000000000005, MVArmax=46.14, MVArmin=-10.06, PsetMW=90.39, QsetMVA=-10.06)
    pyf.add_gen(grid, '7269.0', '430', price_zone_link=False, lf=15.364, qf=0.001, MWmax=90.64, MWmin=27.190000000000005, MVArmax=46.14, MVArmin=-10.06, PsetMW=90.39, QsetMVA=-10.06)
    pyf.add_gen(grid, '7270.0', '431', price_zone_link=False, lf=15.761, qf=0.001, MWmax=181.28, MWmin=54.38000000000001, MVArmax=92.27, MVArmin=-20.12, PsetMW=180.78, QsetMVA=-20.12)
    pyf.add_gen(grid, '7285.0', '435', price_zone_link=False, lf=0.0, qf=0.0, MWmax=21.22, MWmin=21.22, MVArmax=0.0, MVArmin=0.0, PsetMW=21.22, QsetMVA=0.0)
    pyf.add_gen(grid, '7286.0', '436', price_zone_link=False, lf=0.0, qf=0.0, MWmax=21.22, MWmin=21.22, MVArmax=0.0, MVArmin=0.0, PsetMW=21.22, QsetMVA=0.0)
    pyf.add_gen(grid, '7287.0', '437', price_zone_link=False, lf=0.0, qf=0.0, MWmax=21.22, MWmin=21.22, MVArmax=0.0, MVArmin=0.0, PsetMW=21.22, QsetMVA=0.0)
    pyf.add_gen(grid, '7288.0', '438', price_zone_link=False, lf=0.0, qf=0.0, MWmax=21.22, MWmin=21.22, MVArmax=0.0, MVArmin=0.0, PsetMW=21.22, QsetMVA=0.0)
    pyf.add_gen(grid, '7289.0', '439', price_zone_link=False, lf=0.0, qf=0.0, MWmax=42.45, MWmin=42.45, MVArmax=0.0, MVArmin=0.0, PsetMW=42.45, QsetMVA=0.0)
    pyf.add_gen(grid, '7307.0', '440', price_zone_link=False, lf=16.984, qf=0.001, MWmax=96.68, MWmin=28.999999999999996, MVArmax=49.21, MVArmin=-10.73, PsetMW=96.41, QsetMVA=49.21)
    pyf.add_gen(grid, '7308.0', '441', price_zone_link=False, lf=16.984, qf=0.001, MWmax=96.68, MWmin=28.999999999999996, MVArmax=49.21, MVArmin=-10.73, PsetMW=96.41, QsetMVA=49.21)
    pyf.add_gen(grid, '7310.0', '443', price_zone_link=False, lf=16.984, qf=0.001, MWmax=96.68, MWmin=28.999999999999996, MVArmax=49.21, MVArmin=-10.73, PsetMW=96.41, QsetMVA=47.43)
    pyf.add_gen(grid, '7311.0', '444', price_zone_link=False, lf=17.452, qf=0.001, MWmax=193.37, MWmin=58.01, MVArmax=98.42, MVArmin=-21.46, PsetMW=192.83, QsetMVA=98.42)
    pyf.add_gen(grid, '7323.0', '445', price_zone_link=False, lf=15.512, qf=0.001, MWmax=124.48000000000002, MWmin=37.34, MVArmax=63.35999999999999, MVArmin=-13.819999999999999, PsetMW=37.83, QsetMVA=-13.819999999999999)
    pyf.add_gen(grid, '7324.0', '446', price_zone_link=False, lf=15.512, qf=0.001, MWmax=124.48000000000002, MWmin=37.34, MVArmax=63.35999999999999, MVArmin=-13.819999999999999, PsetMW=37.83, QsetMVA=-13.819999999999999)
    pyf.add_gen(grid, '7325.0', '447', price_zone_link=False, lf=15.512, qf=0.001, MWmax=124.48000000000002, MWmin=37.34, MVArmax=63.35999999999999, MVArmin=-13.819999999999999, PsetMW=37.83, QsetMVA=-13.819999999999999)
    pyf.add_gen(grid, '7326.0', '448', price_zone_link=False, lf=15.512, qf=0.001, MWmax=124.48000000000002, MWmin=37.34, MVArmax=63.35999999999999, MVArmin=-13.819999999999999, PsetMW=37.83, QsetMVA=-13.819999999999999)
    pyf.add_gen(grid, '7329.0', '449', price_zone_link=False, lf=17.009, qf=0.001, MWmax=101.9, MWmin=30.570000000000004, MVArmax=51.86999999999999, MVArmin=-11.31, PsetMW=30.97, QsetMVA=38.19)
    pyf.add_gen(grid, '7335.0', '451', price_zone_link=False, lf=18.671, qf=0.002, MWmax=453.5, MWmin=136.05, MVArmax=211.33, MVArmin=-55.33, PsetMW=137.82, QsetMVA=140.29)
    pyf.add_gen(grid, '7349.0', '452', price_zone_link=False, lf=16.343, qf=0.001, MWmax=313.83, MWmin=94.15, MVArmax=159.74, MVArmin=-34.83, PsetMW=312.96, QsetMVA=130.17)
    pyf.add_gen(grid, '7350.0', '453', price_zone_link=False, lf=16.343, qf=0.001, MWmax=313.83, MWmin=94.15, MVArmax=159.74, MVArmin=-34.83, PsetMW=312.96, QsetMVA=56.24)
    pyf.add_gen(grid, '7351.0', '454', price_zone_link=False, lf=16.343, qf=0.001, MWmax=313.83, MWmin=94.15, MVArmax=159.74, MVArmin=-34.83, PsetMW=312.96, QsetMVA=44.34)
    pyf.add_gen(grid, '7352.0', '455', price_zone_link=False, lf=16.343, qf=0.001, MWmax=313.83, MWmin=94.15, MVArmax=159.74, MVArmin=-34.83, PsetMW=312.96, QsetMVA=46.01)
    pyf.add_gen(grid, '7353.0', '456', price_zone_link=False, lf=17.967, qf=0.001, MWmax=684.2, MWmin=205.26, MVArmax=348.26, MVArmin=-75.95, PsetMW=594.86, QsetMVA=72.18)
    pyf.add_gen(grid, '7354.0', '457', price_zone_link=False, lf=17.967, qf=0.001, MWmax=684.2, MWmin=205.26, MVArmax=348.26, MVArmin=-75.95, PsetMW=591.08, QsetMVA=116.42000000000002)
    pyf.add_gen(grid, '7355.0', '458', price_zone_link=False, lf=17.967, qf=0.001, MWmax=684.2, MWmin=205.26, MVArmax=348.26, MVArmin=-75.95, PsetMW=591.08, QsetMVA=115.58)
    pyf.add_gen(grid, '7356.0', '459', price_zone_link=False, lf=17.967, qf=0.001, MWmax=684.2, MWmin=204.99999999999997, MVArmax=348.26, MVArmin=-75.95, PsetMW=591.03, QsetMVA=120.29)
    pyf.add_gen(grid, '7357.0', '460', price_zone_link=False, lf=15.037, qf=0.001, MWmax=16.3, MWmin=4.89, MVArmax=8.3, MVArmin=-1.81, PsetMW=16.25, QsetMVA=8.3)
    pyf.add_gen(grid, '7358.0', '461', price_zone_link=False, lf=15.954, qf=0.001, MWmax=225.2, MWmin=67.56, MVArmax=114.62999999999998, MVArmin=-25.0, PsetMW=224.58, QsetMVA=36.74)
    pyf.add_gen(grid, '7359.0', '462', price_zone_link=False, lf=15.954, qf=0.001, MWmax=225.2, MWmin=67.56, MVArmax=114.62999999999998, MVArmin=-25.0, PsetMW=224.58, QsetMVA=80.96)
    pyf.add_gen(grid, '7360.0', '463', price_zone_link=False, lf=15.954, qf=0.001, MWmax=225.2, MWmin=67.56, MVArmax=114.62999999999998, MVArmin=-25.0, PsetMW=224.58, QsetMVA=49.54)
    pyf.add_gen(grid, '7361.0', '464', price_zone_link=False, lf=15.416, qf=0.001, MWmax=102.70000000000002, MWmin=30.81, MVArmax=52.27, MVArmin=-11.4, PsetMW=102.42, QsetMVA=30.29)
    pyf.add_gen(grid, '7369.0', '465', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=30.15)
    pyf.add_gen(grid, '7370.0', '466', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=24.26)
    pyf.add_gen(grid, '7371.0', '467', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=29.01)
    pyf.add_gen(grid, '7372.0', '468', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=30.97)
    pyf.add_gen(grid, '7373.0', '469', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=54.6)
    pyf.add_gen(grid, '7374.0', '470', price_zone_link=False, lf=15.437, qf=0.001, MWmax=107.27, MWmin=32.18, MVArmax=54.6, MVArmin=-11.91, PsetMW=106.97000000000001, QsetMVA=38.05)
    pyf.add_gen(grid, '7375.0', '471', price_zone_link=False, lf=15.772, qf=0.001, MWmax=183.66, MWmin=55.1, MVArmax=93.48, MVArmin=-20.39, PsetMW=183.15, QsetMVA=37.31)
    pyf.add_gen(grid, '7376.0', '472', price_zone_link=False, lf=15.772, qf=0.001, MWmax=183.66, MWmin=55.1, MVArmax=93.48, MVArmin=-20.39, PsetMW=183.15, QsetMVA=40.5)
    pyf.add_gen(grid, '7377.0', '473', price_zone_link=False, lf=15.772, qf=0.001, MWmax=183.66, MWmin=55.1, MVArmax=93.48, MVArmin=-20.39, PsetMW=183.15, QsetMVA=51.519999999999996)
    pyf.add_gen(grid, '7378.0', '474', price_zone_link=False, lf=15.772, qf=0.001, MWmax=183.66, MWmin=55.1, MVArmax=93.48, MVArmin=-20.39, PsetMW=183.15, QsetMVA=41.99)
    pyf.add_gen(grid, '7379.0', '475', price_zone_link=False, lf=15.772, qf=0.001, MWmax=183.66, MWmin=55.1, MVArmax=93.48, MVArmin=-20.39, PsetMW=183.15, QsetMVA=45.29)
    pyf.add_gen(grid, '7389.0', '476', price_zone_link=False, lf=17.347, qf=0.001, MWmax=542.8, MWmin=162.84, MVArmax=276.29, MVArmin=-60.25, PsetMW=541.3, QsetMVA=276.29)
    pyf.add_gen(grid, '7400.0', '477', price_zone_link=False, lf=15.282, qf=0.001, MWmax=72.0, MWmin=21.6, MVArmax=36.65, MVArmin=-7.99, PsetMW=71.8, QsetMVA=-7.99)
    pyf.add_gen(grid, '7400.0', '478', price_zone_link=False, lf=15.282, qf=0.001, MWmax=72.0, MWmin=21.6, MVArmax=36.65, MVArmin=-7.99, PsetMW=71.8, QsetMVA=-7.99)
    pyf.add_gen(grid, '7400.0', '479', price_zone_link=False, lf=15.282, qf=0.001, MWmax=72.0, MWmin=21.6, MVArmax=36.65, MVArmin=-7.99, PsetMW=71.8, QsetMVA=-7.99)
    pyf.add_gen(grid, '7400.0', '480', price_zone_link=False, lf=15.282, qf=0.001, MWmax=72.0, MWmin=21.6, MVArmax=36.65, MVArmin=-7.99, PsetMW=71.8, QsetMVA=-7.99)
    pyf.add_gen(grid, '7400.0', '481', price_zone_link=False, lf=15.598, qf=0.001, MWmax=144.0, MWmin=43.2, MVArmax=73.3, MVArmin=-15.98, PsetMW=143.6, QsetMVA=-15.98)
    pyf.add_gen(grid, '7400.0', '482', price_zone_link=False, lf=16.924, qf=0.001, MWmax=446.4, MWmin=133.92, MVArmax=227.21999999999997, MVArmin=-49.55, PsetMW=445.16, QsetMVA=-49.55)
    pyf.add_gen(grid, '7406.0', '483', price_zone_link=False, lf=16.821, qf=0.001, MWmax=63.1, MWmin=18.93, MVArmax=32.12, MVArmin=-7.000000000000001, PsetMW=62.93, QsetMVA=32.12)
    pyf.add_gen(grid, '7406.0', '484', price_zone_link=False, lf=18.016, qf=0.001, MWmax=310.0, MWmin=93.0, MVArmax=157.79, MVArmin=-34.41, PsetMW=309.14, QsetMVA=157.79)
    pyf.add_gen(grid, '7414.0', '485', price_zone_link=False, lf=17.011, qf=0.001, MWmax=102.4, MWmin=30.72, MVArmax=52.12, MVArmin=-11.37, PsetMW=102.12, QsetMVA=52.12)
    pyf.add_gen(grid, '7419.0', '486', price_zone_link=False, lf=15.353, qf=0.001, MWmax=88.2, MWmin=26.46, MVArmax=44.89, MVArmin=-9.79, PsetMW=87.96, QsetMVA=-3.01)
    pyf.add_gen(grid, '7419.0', '487', price_zone_link=False, lf=15.353, qf=0.001, MWmax=88.2, MWmin=26.46, MVArmax=44.89, MVArmin=-9.79, PsetMW=87.96, QsetMVA=-3.01)
    pyf.add_gen(grid, '7422.0', '488', price_zone_link=False, lf=14.996, qf=0.001, MWmax=6.800000000000001, MWmin=2.04, MVArmax=3.46, MVArmin=-0.75, PsetMW=2.07, QsetMVA=3.46)
    pyf.add_gen(grid, '7422.0', '489', price_zone_link=False, lf=15.721, qf=0.001, MWmax=172.2, MWmin=51.66, MVArmax=87.65, MVArmin=-19.11, PsetMW=52.33, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '490', price_zone_link=False, lf=15.721, qf=0.001, MWmax=172.2, MWmin=51.66, MVArmax=87.65, MVArmin=-19.11, PsetMW=52.33, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '491', price_zone_link=False, lf=15.459, qf=0.001, MWmax=112.5, MWmin=33.75, MVArmax=57.26, MVArmin=-12.49, PsetMW=34.19, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '492', price_zone_link=False, lf=15.459, qf=0.001, MWmax=112.5, MWmin=33.75, MVArmax=57.26, MVArmin=-12.49, PsetMW=34.19, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '493', price_zone_link=False, lf=15.459, qf=0.001, MWmax=112.5, MWmin=33.75, MVArmax=57.26, MVArmin=-12.49, PsetMW=34.19, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '494', price_zone_link=False, lf=15.459, qf=0.001, MWmax=112.5, MWmin=33.75, MVArmax=57.26, MVArmin=-12.49, PsetMW=34.19, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '495', price_zone_link=False, lf=15.804, qf=0.001, MWmax=191.1, MWmin=57.330000000000005, MVArmax=97.27, MVArmin=-21.21, PsetMW=58.08, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '496', price_zone_link=False, lf=15.721, qf=0.001, MWmax=172.2, MWmin=51.66, MVArmax=87.65, MVArmin=-19.11, PsetMW=52.33, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '497', price_zone_link=False, lf=15.721, qf=0.001, MWmax=172.2, MWmin=51.66, MVArmax=87.65, MVArmin=-19.11, PsetMW=52.33, QsetMVA=44.86)
    pyf.add_gen(grid, '7422.0', '498', price_zone_link=False, lf=15.721, qf=0.001, MWmax=172.2, MWmin=51.66, MVArmax=87.65, MVArmin=-19.11, PsetMW=52.33, QsetMVA=44.86)
    pyf.add_gen(grid, '7428.0', '499', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '500', price_zone_link=False, lf=15.536, qf=0.001, MWmax=130.0, MWmin=39.0, MVArmax=66.17, MVArmin=-14.429999999999998, PsetMW=39.51, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '501', price_zone_link=False, lf=15.536, qf=0.001, MWmax=130.0, MWmin=39.0, MVArmax=66.17, MVArmin=-14.429999999999998, PsetMW=39.51, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '502', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '503', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '504', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '505', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '506', price_zone_link=False, lf=15.407, qf=0.001, MWmax=100.51999999999998, MWmin=30.159999999999997, MVArmax=51.16, MVArmin=-11.16, PsetMW=30.55, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '507', price_zone_link=False, lf=15.295, qf=0.001, MWmax=75.0, MWmin=22.5, MVArmax=38.17, MVArmin=-8.33, PsetMW=22.79, QsetMVA=23.38)
    pyf.add_gen(grid, '7428.0', '509', price_zone_link=False, lf=15.017, qf=0.001, MWmax=11.7, MWmin=3.51, MVArmax=5.96, MVArmin=-1.3, PsetMW=3.56, QsetMVA=5.96)
    pyf.add_gen(grid, '8024.0', '510', price_zone_link=False, lf=0.0, qf=0.0, MWmax=33.0, MWmin=33.0, MVArmax=0.0, MVArmin=0.0, PsetMW=33.0, QsetMVA=0.0)
    pyf.add_gen(grid, '8046.0', '515', price_zone_link=False, lf=19.09, qf=0.002, MWmax=172.67, MWmin=51.800000000000004, MVArmax=87.89, MVArmin=-19.17, PsetMW=172.19, QsetMVA=-19.17)
    pyf.add_gen(grid, '8048.0', '517', price_zone_link=False, lf=19.09, qf=0.002, MWmax=172.67, MWmin=51.800000000000004, MVArmax=87.89, MVArmin=-19.17, PsetMW=172.19, QsetMVA=87.89)
    pyf.add_gen(grid, '8049.0', '518', price_zone_link=False, lf=19.09, qf=0.002, MWmax=172.67, MWmin=51.800000000000004, MVArmax=87.89, MVArmin=-19.17, PsetMW=172.19, QsetMVA=-19.17)
    pyf.add_gen(grid, '8071.0', '524', price_zone_link=False, lf=18.671, qf=0.002, MWmax=924.9000000000001, MWmin=277.47, MVArmax=430.99999999999994, MVArmin=-112.84, PsetMW=922.34, QsetMVA=-94.45)
    pyf.add_gen(grid, '8077.0', '525', price_zone_link=False, lf=16.202, qf=0.001, MWmax=234.90000000000003, MWmin=70.47, MVArmax=119.56, MVArmin=-26.07, PsetMW=234.24999999999997, QsetMVA=8.03)
    pyf.add_gen(grid, '8078.0', '526', price_zone_link=False, lf=16.202, qf=0.001, MWmax=234.90000000000003, MWmin=70.47, MVArmax=119.56, MVArmin=-26.07, PsetMW=234.24999999999997, QsetMVA=6.570000000000001)
    pyf.add_gen(grid, '8079.0', '527', price_zone_link=False, lf=16.202, qf=0.001, MWmax=234.90000000000003, MWmin=70.47, MVArmax=119.56, MVArmin=-26.07, PsetMW=234.24999999999997, QsetMVA=7.249999999999999)
    pyf.add_gen(grid, '8080.0', '528', price_zone_link=False, lf=16.202, qf=0.001, MWmax=234.90000000000003, MWmin=70.47, MVArmax=119.56, MVArmin=-26.07, PsetMW=234.24999999999997, QsetMVA=7.15)
    pyf.add_gen(grid, '8088.0', '530', price_zone_link=False, lf=18.671, qf=0.002, MWmax=897.7, MWmin=269.31, MVArmax=418.33, MVArmin=-109.52, PsetMW=895.2099999999999, QsetMVA=139.83)
    pyf.add_gen(grid, '8099.0', '531', price_zone_link=False, lf=0.0, qf=0.0, MWmax=85.5, MWmin=85.5, MVArmax=0.0, MVArmin=0.0, PsetMW=85.5, QsetMVA=0.0)
    pyf.add_gen(grid, '8117.0', '534', price_zone_link=False, lf=21.838, qf=0.002, MWmax=351.7, MWmin=105.51000000000002, MVArmax=163.89, MVArmin=-42.91, PsetMW=227.51999999999998, QsetMVA=69.52)
    pyf.add_gen(grid, '8129.0', '535', price_zone_link=False, lf=16.843, qf=0.002, MWmax=660.0, MWmin=198.0, MVArmax=307.56, MVArmin=-80.52, PsetMW=658.17, QsetMVA=-49.85)
    pyf.add_gen(grid, '8130.0', '536', price_zone_link=False, lf=16.843, qf=0.002, MWmax=660.0, MWmin=198.0, MVArmax=307.56, MVArmin=-80.52, PsetMW=658.17, QsetMVA=-8.92)
    pyf.add_gen(grid, '8131.0', '537', price_zone_link=False, lf=16.843, qf=0.002, MWmax=660.0, MWmin=198.0, MVArmax=307.56, MVArmin=-80.52, PsetMW=658.17, QsetMVA=-36.34)
    pyf.add_gen(grid, '8155.0', '542', price_zone_link=False, lf=18.671, qf=0.002, MWmax=174.6, MWmin=52.38, MVArmax=81.36, MVArmin=-21.3, PsetMW=174.12, QsetMVA=-21.3)
    
    
    # Add Renewable Source Zones

    
    # Add Renewable Sources

    
    # Return the grid
    return grid,res
