

import pyflow_acdc as pyf
import pandas as pd
"""
Converted to PyFlowACDC format from

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%                                                                  %%%%%
%%%%    IEEE PES Power Grid Library - Optimal Power Flow - v18.08     %%%%%
%%%%          (https://github.com/power-grid-lib/pglib-opf)           %%%%%
%%%%               Benchmark Group - Typical Operations               %%%%%
%%%%                        08 - Augustmonth - 2018                        %%%%%
%%%%                                                                  %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   A 500 bus model created by the Sustainable Data Evolution Technology (SDET)
%   tool.
%
%   https://egriddata.org/group/sustainable-data-evolution-technology-sdet
%
%   Developed by Ruisheng Diao
%   Provided in the public domain. March, 2018
%
%   HVDC grid added by Hakan Ergun, KU Leuven - EnergyVille, September, 2018
%

"""

def pglib_opf_case588_sdet_acdc():    
    
    S_base=100
    
    # DataFrame Code:
    nodes_AC_data = [
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8979, 'Reactive_load': 0.28550000000000003, 'Node_id': '1.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '3.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6235, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3247, 'Reactive_load': 0.8589, 'Node_id': '5.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '6.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '8.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0824, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '10.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '11.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3141, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '12.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8361, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '13.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '14.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11, 'Reactive_load': 0.4, 'Node_id': '15.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.05, 'Reactive_load': 0.1, 'Node_id': '16.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5271, 'Reactive_load': 0.6023, 'Node_id': '17.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0693, 'Reactive_load': 0.0311, 'Node_id': '18.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1465, 'Reactive_load': 0.01, 'Node_id': '19.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0814, 'Reactive_load': 0.0132, 'Node_id': '20.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1417, 'Reactive_load': 0.042199999999999994, 'Node_id': '21.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042199999999999994, 'Reactive_load': 0.01, 'Node_id': '22.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '23.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.2667, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 32.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '24.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.486, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '25.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3197, 'Reactive_load': 0.20829999999999999, 'Node_id': '26.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.052199999999999996, 'Reactive_load': 0.021099999999999997, 'Node_id': '27.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.026099999999999998, 'Reactive_load': 0.01, 'Node_id': '28.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '29.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '30.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1567, 'Reactive_load': 0.052199999999999996, 'Node_id': '31.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1678, 'Reactive_load': 0.0472, 'Node_id': '32.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 16.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '33.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.45, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042199999999999994, 'Reactive_load': 0.005, 'Node_id': '34.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09949999999999999, 'Reactive_load': 0.021099999999999997, 'Node_id': '35.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1085, 'Reactive_load': 0.01, 'Node_id': '36.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1467, 'Reactive_load': 0.0311, 'Node_id': '37.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0754, 'Reactive_load': 0.01, 'Node_id': '38.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.021099999999999997, 'Reactive_load': 0.005, 'Node_id': '39.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.021099999999999997, 'Reactive_load': 0.005, 'Node_id': '40.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0874, 'Reactive_load': 0.0311, 'Node_id': '41.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.057300000000000004, 'Reactive_load': 0.01, 'Node_id': '42.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4338, 'Reactive_load': 0.0625, 'Node_id': '43.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.026099999999999998, 'Node_id': '44.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4203, 'Reactive_load': 0.1276, 'Node_id': '45.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1567, 'Reactive_load': 0.026099999999999998, 'Node_id': '46.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1989, 'Reactive_load': 0.0683, 'Node_id': '47.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0834, 'Reactive_load': 0.0271, 'Node_id': '48.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1678, 'Reactive_load': 0.0633, 'Node_id': '49.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.13140000000000002, 'Reactive_load': 0.0126, 'Node_id': '50.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.052199999999999996, 'Reactive_load': 0.005, 'Node_id': '51.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '52.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1607, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6866, 'Reactive_load': 0.049699999999999994, 'Node_id': '53.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.020099999999999996, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.01, 'Node_id': '54.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.009300000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0834, 'Reactive_load': 0.0311, 'Node_id': '55.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '56.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1916, 'Reactive_load': 0.020099999999999996, 'Node_id': '57.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3044, 'Reactive_load': 0.042199999999999994, 'Node_id': '58.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1889, 'Reactive_load': 0.0472, 'Node_id': '59.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1155, 'Reactive_load': 0.01, 'Node_id': '60.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1778, 'Reactive_load': 0.042199999999999994, 'Node_id': '61.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '62.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '63.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1519, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '64.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1411, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '65.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '66.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.30469999999999997, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '67.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '68.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6248, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '69.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3247, 'Reactive_load': 0.8589, 'Node_id': '70.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '71.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '72.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '73.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '74.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.8468000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '75.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.42840000000000006, 'Reactive_load': 0.2961, 'Node_id': '76.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '77.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8362, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '78.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.7, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '79.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11, 'Reactive_load': 0.4, 'Node_id': '80.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.05, 'Reactive_load': 0.1, 'Node_id': '81.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5271, 'Reactive_load': 0.6023, 'Node_id': '82.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0693, 'Reactive_load': 0.0311, 'Node_id': '83.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1465, 'Reactive_load': 0.01, 'Node_id': '84.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0814, 'Reactive_load': 0.0132, 'Node_id': '85.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1417, 'Reactive_load': 0.042199999999999994, 'Node_id': '86.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042199999999999994, 'Reactive_load': 0.01, 'Node_id': '87.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6149, 'Reactive_load': 0.2847, 'Node_id': '88.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 32.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '89.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.486, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '90.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0395, 'Reactive_load': 0.20829999999999999, 'Node_id': '91.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.052199999999999996, 'Reactive_load': 0.021099999999999997, 'Node_id': '92.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.026099999999999998, 'Reactive_load': 0.01, 'Node_id': '93.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '94.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '95.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1567, 'Reactive_load': 0.052199999999999996, 'Node_id': '96.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1678, 'Reactive_load': 0.0472, 'Node_id': '97.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 16.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '98.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.45, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042199999999999994, 'Reactive_load': 0.005, 'Node_id': '99.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09949999999999999, 'Reactive_load': 0.021099999999999997, 'Node_id': '100.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1085, 'Reactive_load': 0.01, 'Node_id': '101.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1467, 'Reactive_load': 0.0311, 'Node_id': '102.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0754, 'Reactive_load': 0.01, 'Node_id': '103.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.021099999999999997, 'Reactive_load': 0.005, 'Node_id': '104.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.021099999999999997, 'Reactive_load': 0.005, 'Node_id': '105.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0874, 'Reactive_load': 0.0311, 'Node_id': '106.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.057300000000000004, 'Reactive_load': 0.01, 'Node_id': '107.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4311, 'Reactive_load': 0.0625, 'Node_id': '108.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.026099999999999998, 'Node_id': '109.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4203, 'Reactive_load': 0.1276, 'Node_id': '110.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1567, 'Reactive_load': 0.026099999999999998, 'Node_id': '111.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1989, 'Reactive_load': 0.0683, 'Node_id': '112.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0834, 'Reactive_load': 0.0271, 'Node_id': '113.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1678, 'Reactive_load': 0.0633, 'Node_id': '114.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.13140000000000002, 'Reactive_load': 0.0126, 'Node_id': '115.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.052199999999999996, 'Reactive_load': 0.005, 'Node_id': '116.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '117.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1607, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6866, 'Reactive_load': 0.049699999999999994, 'Node_id': '118.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.020099999999999996, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.01, 'Node_id': '119.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0192, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0834, 'Reactive_load': 0.0311, 'Node_id': '120.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '121.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1916, 'Reactive_load': 0.020099999999999996, 'Node_id': '122.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3044, 'Reactive_load': 0.042199999999999994, 'Node_id': '123.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1889, 'Reactive_load': 0.0472, 'Node_id': '124.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1155, 'Reactive_load': 0.01, 'Node_id': '125.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1778, 'Reactive_load': 0.042199999999999994, 'Node_id': '126.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '127.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '128.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1519, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '129.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1411, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '130.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.40590000000000004, 'Reactive_load': 0.7073, 'Node_id': '131.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '132.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -1.7697999999999998, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '133.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '134.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '135.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1728, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08380000000000001, 'Reactive_load': 0.012, 'Node_id': '136.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0483, 'Reactive_load': 0.011000000000000001, 'Node_id': '137.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '138.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1155, 'Reactive_load': 0.0582, 'Node_id': '139.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1316, 'Reactive_load': 0.0492, 'Node_id': '140.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.26039999999999996, 'Reactive_load': 0.0335, 'Node_id': '141.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12560000000000002, 'Reactive_load': 0.021099999999999997, 'Node_id': '142.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1351, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0472, 'Reactive_load': 0.006999999999999999, 'Node_id': '143.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': -0.0327, 'Node_id': '144.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1015, 'Reactive_load': 0.042199999999999994, 'Node_id': '145.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '146.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '147.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '148.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 16.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '149.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.6, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1778, 'Reactive_load': 0.042199999999999994, 'Node_id': '150.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08039999999999999, 'Reactive_load': 0.0412, 'Node_id': '151.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '152.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7333, 'Reactive_load': 0.3139, 'Node_id': '153.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.629, 'Reactive_load': 0.2621, 'Node_id': '154.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '155.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0834, 'Reactive_load': 0.026099999999999998, 'Node_id': '156.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0512, 'Reactive_load': 0.0068000000000000005, 'Node_id': '157.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1517, 'Reactive_load': 0.0472, 'Node_id': '158.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1366, 'Reactive_load': 0.0633, 'Node_id': '159.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0271, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0894, 'Reactive_load': 0.0232, 'Node_id': '160.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0395, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1399, 'Reactive_load': 0.11800000000000001, 'Node_id': '161.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.022099999999999998, 'Node_id': '162.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1155, 'Reactive_load': 0.026099999999999998, 'Node_id': '163.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0392, 'Reactive_load': 0.0161, 'Node_id': '164.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1628, 'Reactive_load': 0.052199999999999996, 'Node_id': '165.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0333, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1246, 'Reactive_load': 0.0582, 'Node_id': '166.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '167.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.15410000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12560000000000002, 'Reactive_load': 0.042199999999999994, 'Node_id': '168.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.3387, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2361, 'Reactive_load': 0.0472, 'Node_id': '169.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.001, 'Reactive_load': 0.001, 'Node_id': '170.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.001, 'Reactive_load': 0.001, 'Node_id': '171.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09449999999999999, 'Reactive_load': 0.0231, 'Node_id': '172.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1356, 'Reactive_load': 0.0623, 'Node_id': '173.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0683, 'Reactive_load': 0.026099999999999998, 'Node_id': '174.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0472, 'Reactive_load': 0.026099999999999998, 'Node_id': '175.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0633, 'Reactive_load': 0.009000000000000001, 'Node_id': '176.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0412, 'Reactive_load': 0.025099999999999997, 'Node_id': '177.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0001, 'Node_id': '178.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.44310000000000005, 'Reactive_load': -0.03, 'Node_id': '179.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0633, 'Reactive_load': 0.015700000000000002, 'Node_id': '180.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0175, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6144, 'Reactive_load': 0.45530000000000004, 'Node_id': '181.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '182.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33509999999999995, 'Reactive_load': 1.594, 'Node_id': '183.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1991, 'Reactive_load': 0.3657, 'Node_id': '184.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12, 'Reactive_load': 0.3, 'Node_id': '185.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12, 'Reactive_load': 0.3, 'Node_id': '186.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '187.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '188.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.9307, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33409999999999995, 'Reactive_load': 0.22469999999999998, 'Node_id': '189.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.18510000000000001, 'Reactive_load': 0.6888, 'Node_id': '190.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1474, 'Reactive_load': 0.0403, 'Node_id': '191.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '192.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 1.6891999999999998, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3223, 'Reactive_load': 0.5302, 'Node_id': '193.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.815, 'Reactive_load': 0.8590000000000001, 'Node_id': '194.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '195.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '196.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '197.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.59, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1084, 'Reactive_load': 0.025099999999999997, 'Node_id': '198.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2948, 'Reactive_load': 0.36939999999999995, 'Node_id': '199.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '200.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.06559999999999999, 'Reactive_load': 0.0209, 'Node_id': '201.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09380000000000001, 'Reactive_load': 0.0209, 'Node_id': '202.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.125, 'Reactive_load': 0.022000000000000002, 'Node_id': '203.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1573, 'Reactive_load': 0.0575, 'Node_id': '204.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1877, 'Reactive_load': 0.0681, 'Node_id': '205.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.2503, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17190000000000003, 'Reactive_load': 0.0774, 'Node_id': '206.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0013, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '207.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0042, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8304, 'Reactive_load': 0.0934, 'Node_id': '208.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1459, 'Reactive_load': 0.053399999999999996, 'Node_id': '209.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5024000000000001, 'Reactive_load': 0.12269999999999999, 'Node_id': '210.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 16.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '211.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.41490000000000005, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '212.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.019799999999999998, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '213.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5498, 'Reactive_load': -0.1005, 'Node_id': '214.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '215.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1106, 'Reactive_load': -0.10279999999999999, 'Node_id': '216.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11460000000000001, 'Reactive_load': 0.0377, 'Node_id': '217.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11460000000000001, 'Reactive_load': 0.0408, 'Node_id': '218.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8062999999999999, 'Reactive_load': -0.030600000000000002, 'Node_id': '219.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08130000000000001, 'Reactive_load': 0.045, 'Node_id': '220.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6127, 'Reactive_load': 0.1016, 'Node_id': '221.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11560000000000001, 'Reactive_load': 0.0001, 'Node_id': '222.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.08529999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '223.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4879, 'Reactive_load': 0.005, 'Node_id': '224.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.16899999999999998, 'Reactive_load': 0.0209, 'Node_id': '225.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1306, 'Reactive_load': 0.0311, 'Node_id': '226.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0001, 'Reactive_load': 0.0, 'Node_id': '227.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2771, 'Reactive_load': 0.0212, 'Node_id': '228.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '229.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 6.34, 'Reactive_load': 0.0, 'Node_id': '230.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '231.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '232.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '233.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2169, 'Reactive_load': -0.011399999999999999, 'Node_id': '234.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0525, 'Reactive_load': -0.0287, 'Node_id': '235.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0158, 'Reactive_load': 0.0005, 'Node_id': '236.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '237.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '238.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.012199999999999999, 'Reactive_load': -0.018600000000000002, 'Node_id': '239.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.0001, 'Reactive_load': 0.0048, 'Node_id': '240.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0471, 'Reactive_load': -0.0269, 'Node_id': '241.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.016399999999999998, 'Reactive_load': 0.0007000000000000001, 'Node_id': '242.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0241, 'Reactive_load': 0.009899999999999999, 'Node_id': '243.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -1.014, 'Reactive_load': 0.3837, 'Node_id': '244.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '245.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '246.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '247.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '248.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.11259999999999999, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1226, 'Reactive_load': 0.0159, 'Node_id': '249.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0507, 'Reactive_load': -0.0242, 'Node_id': '250.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0406, 'Reactive_load': -0.017, 'Node_id': '251.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '252.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0403, 'Reactive_load': -0.0159, 'Node_id': '253.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.06480000000000001, 'Reactive_load': -0.0458, 'Node_id': '254.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '255.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0482, 'Reactive_load': -0.0217, 'Node_id': '256.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.018500000000000003, 'Reactive_load': -0.0021, 'Node_id': '257.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0898, 'Reactive_load': -0.0101, 'Node_id': '258.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '259.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2095, 'Reactive_load': -0.0015, 'Node_id': '260.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '261.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0901, 'Reactive_load': -0.0097, 'Node_id': '262.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '263.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0095, 'Reactive_load': -0.025, 'Node_id': '264.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '265.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0143, 'Reactive_load': -0.0003, 'Node_id': '266.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '267.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6707, 'Reactive_load': -0.1151, 'Node_id': '268.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0332, 'Reactive_load': -0.0115, 'Node_id': '269.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0008, 'Reactive_load': 0.005, 'Node_id': '270.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '271.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0227, 'Reactive_load': -0.0046, 'Node_id': '272.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '273.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.051500000000000004, 'Reactive_load': -0.0227, 'Node_id': '274.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '275.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0176, 'Reactive_load': -0.015700000000000002, 'Node_id': '276.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.024300000000000002, 'Reactive_load': 0.01, 'Node_id': '277.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '278.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0233, 'Reactive_load': -0.002, 'Node_id': '279.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '280.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '281.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '282.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '283.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.017, 'Reactive_load': -0.0158, 'Node_id': '284.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '285.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0297, 'Reactive_load': 0.0004, 'Node_id': '286.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0438, 'Reactive_load': -0.015, 'Node_id': '287.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.022000000000000002, 'Reactive_load': -0.0025, 'Node_id': '288.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '289.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10439999999999999, 'Reactive_load': 0.0039000000000000003, 'Node_id': '290.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '291.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '292.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.8769, 'Reactive_load': -0.0986, 'Node_id': '293.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.212, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '294.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.0023, 'Reactive_load': -0.0722, 'Node_id': '295.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8575, 'Reactive_load': -0.14029999999999998, 'Node_id': '296.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0592, 'Reactive_load': -0.0369, 'Node_id': '297.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0538, 'Reactive_load': -0.0241, 'Node_id': '298.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0604, 'Reactive_load': -0.0239, 'Node_id': '299.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0191, 'Reactive_load': -0.0045000000000000005, 'Node_id': '300.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '301.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '302.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2146, 'Reactive_load': -0.002, 'Node_id': '303.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.069, 'Reactive_load': -0.050300000000000004, 'Node_id': '304.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.0003, 'Reactive_load': 0.0048, 'Node_id': '305.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0313, 'Reactive_load': -0.0067, 'Node_id': '306.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.076, 'Reactive_load': 0.0463, 'Node_id': '307.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0369, 'Reactive_load': -0.013500000000000002, 'Node_id': '308.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '309.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '310.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '311.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '312.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0842, 'Reactive_load': 0.0512, 'Node_id': '313.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.015700000000000002, 'Reactive_load': 0.0005, 'Node_id': '314.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '315.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.037000000000000005, 'Reactive_load': -0.013300000000000001, 'Node_id': '316.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.044800000000000006, 'Reactive_load': -0.0183, 'Node_id': '317.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0704, 'Reactive_load': -0.0067, 'Node_id': '318.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10279999999999999, 'Reactive_load': 0.0039000000000000003, 'Node_id': '319.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '320.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '321.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0353, 'Reactive_load': -0.0137, 'Node_id': '322.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '323.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 161.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '324.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0239, 'Reactive_load': -0.0053, 'Node_id': '325.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '326.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.2, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '327.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '328.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.083, 'Reactive_load': 0.0127, 'Node_id': '329.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '330.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.005600000000000001, 'Reactive_load': -0.031, 'Node_id': '331.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.0013, 'Reactive_load': -0.0704, 'Node_id': '332.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0242, 'Reactive_load': 0.0034000000000000002, 'Node_id': '333.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0091, 'Reactive_load': 0.0060999999999999995, 'Node_id': '334.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '335.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.4013999999999998, 'Reactive_load': -0.073, 'Node_id': '336.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '337.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0641, 'Reactive_load': 0.0351, 'Node_id': '338.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '339.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '340.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0384, 'Reactive_load': -0.0128, 'Node_id': '341.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '342.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0103, 'Reactive_load': -0.057699999999999994, 'Node_id': '343.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.017, 'Reactive_load': -0.015700000000000002, 'Node_id': '344.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '345.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08070000000000001, 'Reactive_load': 0.0493, 'Node_id': '346.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0373, 'Reactive_load': 0.01, 'Node_id': '347.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0371, 'Reactive_load': -0.0131, 'Node_id': '348.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2202, 'Reactive_load': 0.0039000000000000003, 'Node_id': '349.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '350.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 69.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '351.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '352.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '353.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 2.3356, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 35.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.474, 'Reactive_load': -0.027000000000000003, 'Node_id': '354.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '355.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 14.4, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '356.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.0275, 'Reactive_load': 0.0296, 'Node_id': '357.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '358.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '359.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0374, 'Reactive_load': 0.030600000000000002, 'Node_id': '360.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3358, 'Reactive_load': 0.0348, 'Node_id': '361.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.36479999999999996, 'Reactive_load': 0.0359, 'Node_id': '362.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 1.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '363.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '364.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.5, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47659999999999997, 'Reactive_load': 0.019, 'Node_id': '365.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4669, 'Reactive_load': 0.0209, 'Node_id': '366.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0507, 'Reactive_load': -0.0151, 'Node_id': '367.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '368.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '369.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 1.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '370.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '371.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 1.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '372.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '373.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6069, 'Reactive_load': -0.23190000000000002, 'Node_id': '374.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.48350000000000004, 'Reactive_load': 0.0143, 'Node_id': '375.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0742, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 35.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0490000000000002, 'Reactive_load': 0.042800000000000005, 'Node_id': '376.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1084, 'Reactive_load': 0.0575, 'Node_id': '377.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '378.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35950000000000004, 'Reactive_load': 0.0229, 'Node_id': '379.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1788, 'Reactive_load': 0.0169, 'Node_id': '380.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 1.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '381.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 14.4, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '382.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '383.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '384.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '385.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 35.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4454, 'Reactive_load': 0.023799999999999998, 'Node_id': '386.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '387.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '388.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.5, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3249, 'Reactive_load': 0.0429, 'Node_id': '389.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '390.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '391.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2257, 'Reactive_load': 0.0315, 'Node_id': '392.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.5967, 'Reactive_load': -0.2544, 'Node_id': '393.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '394.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '395.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '396.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '397.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '398.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1177, 'Reactive_load': 0.07200000000000001, 'Node_id': '399.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 18.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '400.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 14.4, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '401.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 1.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '402.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '403.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '404.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09140000000000001, 'Reactive_load': 0.0084, 'Node_id': '405.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8791, 'Reactive_load': 0.059500000000000004, 'Node_id': '406.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '407.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7748, 'Reactive_load': -0.047400000000000005, 'Node_id': '408.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7946, 'Reactive_load': -0.0086, 'Node_id': '409.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2349, 'Reactive_load': -0.07200000000000001, 'Node_id': '410.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.5, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9459000000000001, 'Reactive_load': 0.2271, 'Node_id': '411.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.10800000000000001, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 34.5, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.40869999999999995, 'Reactive_load': -0.0969, 'Node_id': '412.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0789, 'Reactive_load': 0.1072, 'Node_id': '413.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7103, 'Reactive_load': 0.0639, 'Node_id': '414.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0873, 'Reactive_load': -0.018600000000000002, 'Node_id': '415.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.659, 'Reactive_load': -1.4398, 'Node_id': '416.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '417.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09380000000000001, 'Reactive_load': 0.0058, 'Node_id': '418.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '419.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2658, 'Reactive_load': 0.3936, 'Node_id': '420.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2325, 'Reactive_load': 0.2531, 'Node_id': '421.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '422.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07, 'Reactive_load': 0.18, 'Node_id': '423.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08, 'Reactive_load': 0.21, 'Node_id': '424.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0593, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.1589, 'Reactive_load': 0.1959, 'Node_id': '425.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0116, 'Reactive_load': -0.45890000000000003, 'Node_id': '426.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '427.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '428.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042800000000000005, 'Reactive_load': 0.0158, 'Node_id': '429.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0548, 'Reactive_load': 0.023, 'Node_id': '430.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '431.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09759999999999999, 'Reactive_load': -0.1359, 'Node_id': '432.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0823, 'Reactive_load': 0.0216, 'Node_id': '433.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0187, 'Reactive_load': 0.008, 'Node_id': '434.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0334, 'Reactive_load': 0.014199999999999999, 'Node_id': '435.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1352, 'Reactive_load': 0.050300000000000004, 'Node_id': '436.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '437.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '438.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0481, 'Reactive_load': 0.0173, 'Node_id': '439.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2361, 'Reactive_load': 0.21100000000000002, 'Node_id': '440.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '441.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0546, 'Reactive_load': 0.0231, 'Node_id': '442.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '443.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0962, 'Reactive_load': 0.031400000000000004, 'Node_id': '444.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.001, 'Node_id': '445.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '446.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '447.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '448.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '449.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '450.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '451.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.01, 'Reactive_load': 0.0038, 'Node_id': '452.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0449, 'Reactive_load': 0.0311, 'Node_id': '453.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '454.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.8, 'Reactive_load': 0.39, 'Node_id': '455.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.052300000000000006, 'Reactive_load': 0.0181, 'Node_id': '456.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0192, 'Reactive_load': 0.006, 'Node_id': '457.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0275, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.53, 'Reactive_load': 0.2502, 'Node_id': '458.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0023, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.06, 'Reactive_load': 0.0213, 'Node_id': '459.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0278, 'Reactive_load': 0.009000000000000001, 'Node_id': '460.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3875, 'Reactive_load': -0.0925, 'Node_id': '461.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0043, 'Reactive_load': 0.001, 'Node_id': '462.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '463.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0278, 'Reactive_load': 0.0166, 'Node_id': '464.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0583, 'Reactive_load': 0.0067, 'Node_id': '465.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0812, 'Reactive_load': 0.0276, 'Node_id': '466.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0043, 'Reactive_load': 0.0125, 'Node_id': '467.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '468.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0029, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4489, 'Reactive_load': 0.1697, 'Node_id': '469.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.016, 'Reactive_load': 0.004, 'Node_id': '470.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.010700000000000001, 'Reactive_load': 0.0049, 'Node_id': '471.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0653, 'Reactive_load': 0.0279, 'Node_id': '472.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.09230000000000001, 'Reactive_load': -0.08410000000000001, 'Node_id': '473.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0481, 'Reactive_load': 0.0151, 'Node_id': '474.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1891, 'Reactive_load': 0.30210000000000004, 'Node_id': '475.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0034000000000000002, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07, 'Reactive_load': 0.14, 'Node_id': '476.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.003, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0909, 'Reactive_load': 0.030600000000000002, 'Node_id': '477.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '478.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '479.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '480.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.3787, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0294, 'Reactive_load': 0.0125, 'Node_id': '481.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '482.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14550000000000002, 'Reactive_load': 0.0055000000000000005, 'Node_id': '483.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.052199999999999996, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.11230000000000001, 'Reactive_load': 0.026600000000000002, 'Node_id': '484.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0638, 'Reactive_load': 0.0277, 'Node_id': '485.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08310000000000001, 'Reactive_load': 0.0368, 'Node_id': '486.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0085, 'Reactive_load': 0.0008, 'Node_id': '487.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '488.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0066, 'Reactive_load': 0.0039000000000000003, 'Node_id': '489.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0053, 'Reactive_load': 0.0031, 'Node_id': '490.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '491.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.10369999999999999, 'Reactive_load': 0.0346, 'Node_id': '492.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.025699999999999997, 'Reactive_load': 0.009300000000000001, 'Node_id': '493.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '494.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0535, 'Reactive_load': 0.0076, 'Node_id': '495.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '496.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0726, 'Reactive_load': 0.0271, 'Node_id': '497.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '498.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.029900000000000003, 'Reactive_load': 0.011399999999999999, 'Node_id': '499.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0642, 'Reactive_load': 0.022799999999999997, 'Node_id': '500.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '501.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '502.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0599, 'Reactive_load': 0.0206, 'Node_id': '503.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '504.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0454, 'Reactive_load': 0.0189, 'Node_id': '505.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042800000000000005, 'Reactive_load': 0.0179, 'Node_id': '506.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '507.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0364, 'Reactive_load': 0.0123, 'Node_id': '508.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042699999999999995, 'Reactive_load': 0.0155, 'Node_id': '509.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '510.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '511.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0075, 'Reactive_load': 0.0037, 'Node_id': '512.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0128, 'Reactive_load': 0.0058, 'Node_id': '513.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '514.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046, 'Reactive_load': 0.0173, 'Node_id': '515.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '516.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046, 'Reactive_load': 0.0163, 'Node_id': '517.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1219, 'Reactive_load': 0.039599999999999996, 'Node_id': '518.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1816, 'Reactive_load': 0.0579, 'Node_id': '519.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.001, 'Node_id': '520.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '521.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0385, 'Reactive_load': 0.0138, 'Node_id': '522.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '523.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.053099999999999994, 'Reactive_load': 0.0148, 'Node_id': '524.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0742, 'Reactive_load': 0.020099999999999996, 'Node_id': '525.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0215, 'Reactive_load': 0.005600000000000001, 'Node_id': '526.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '527.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1182, 'Reactive_load': 0.0466, 'Node_id': '528.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1121, 'Reactive_load': 0.0484, 'Node_id': '529.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.041299999999999996, 'Reactive_load': 0.017, 'Node_id': '530.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.025699999999999997, 'Reactive_load': 0.0104, 'Node_id': '531.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.34369999999999995, 'Reactive_load': -0.0169, 'Node_id': '532.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0053, 'Reactive_load': 0.003, 'Node_id': '533.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '534.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '535.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '536.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47759999999999997, 'Reactive_load': 0.16620000000000001, 'Node_id': '537.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.7304000000000004, 'Reactive_load': 0.2217, 'Node_id': '538.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '539.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0443, 'Reactive_load': 0.45049999999999996, 'Node_id': '540.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '541.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '542.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '543.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.7299, 'Reactive_load': -0.0101, 'Node_id': '544.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.32159999999999994, 'Reactive_load': 0.5207, 'Node_id': '545.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4636, 'Reactive_load': 0.9207, 'Node_id': '546.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'Slack', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6, 'Reactive_load': 1.2, 'Node_id': '547.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 12.535, 'Reactive_load': 2.8475, 'Node_id': '548.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '549.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.8514, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7131000000000001, 'Reactive_load': 0.5875, 'Node_id': '550.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6806, 'Reactive_load': -0.5434, 'Node_id': '551.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38799999999999996, 'Reactive_load': -0.027000000000000003, 'Node_id': '552.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1405, 'Reactive_load': 0.2554, 'Node_id': '553.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '554.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0083, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.5009000000000001, 'Reactive_load': 0.556, 'Node_id': '555.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '556.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '557.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '558.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.37079999999999996, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 500.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0947, 'Reactive_load': 0.40409999999999996, 'Node_id': '559.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.1849000000000003, 'Reactive_load': 0.0721, 'Node_id': '560.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0385, 'Reactive_load': 0.01, 'Node_id': '561.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0227, 'Reactive_load': 0.0111, 'Node_id': '562.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0158, 'Reactive_load': 0.004, 'Node_id': '563.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0444, 'Reactive_load': 0.011000000000000001, 'Node_id': '564.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0464, 'Reactive_load': 0.004, 'Node_id': '565.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': -0.0028000000000000004, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1389, 'Reactive_load': 0.0776, 'Node_id': '566.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.19519999999999998, 'Reactive_load': 0.11230000000000001, 'Node_id': '567.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '568.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.67, 'Reactive_load': 0.0919, 'Node_id': '569.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 24.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2, 'Reactive_load': 0.4, 'Node_id': '570.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2043, 'Reactive_load': 0.20620000000000002, 'Node_id': '571.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.365, 'Reactive_load': 0.1201, 'Node_id': '572.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.6335999999999999, 'Reactive_load': 0.10830000000000001, 'Node_id': '573.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0015, 'Reactive_load': 0.0067, 'Node_id': '574.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1852, 'Reactive_load': 0.1225, 'Node_id': '575.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1409, 'Reactive_load': 0.0613, 'Node_id': '576.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2072, 'Reactive_load': -0.042199999999999994, 'Node_id': '577.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0397, 'Reactive_load': 0.20489999999999997, 'Node_id': '578.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '579.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0562, 'Reactive_load': 0.012, 'Node_id': '580.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1462, 'Reactive_load': 0.1559, 'Node_id': '581.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0355, 'Reactive_load': 0.003, 'Node_id': '582.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.023700000000000002, 'Reactive_load': 0.006, 'Node_id': '583.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0355, 'Reactive_load': 0.006, 'Node_id': '584.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0207, 'Reactive_load': 0.004, 'Node_id': '585.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.025699999999999997, 'Reactive_load': -0.0849, 'Node_id': '586.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0118, 'Reactive_load': 0.003, 'Node_id': '587.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.004, 'Reactive_load': 0.001, 'Node_id': '588.0', 'Umin': 0.9, 'Umax': 1.1, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None}
    ]
    nodes_AC = pd.DataFrame(nodes_AC_data)

    lines_AC_data = [
        {'fromNode': '1.0', 'toNode': '10.0', 'Resistance': 0.00186, 'Reactance': 0.02155, 'Conductance': 0, 'Susceptance': 0.05317, 'MVA_rating': 461.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '1'},
        {'fromNode': '2.0', 'toNode': '3.0', 'Resistance': 0.00051, 'Reactance': 0.0275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 500.0, 'm': 1.07702, 'shift': 0.0, 'Line_id': '2'},
        {'fromNode': '2.0', 'toNode': '17.0', 'Resistance': 0.00568, 'Reactance': 0.04215, 'Conductance': 0, 'Susceptance': 0.0716, 'MVA_rating': 317.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '3'},
        {'fromNode': '3.0', 'toNode': '14.0', 'Resistance': 0.00131, 'Reactance': 0.01547, 'Conductance': 0, 'Susceptance': 0.41152, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '4'},
        {'fromNode': '3.0', 'toNode': '70.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 158.96, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '5'},
        {'fromNode': '4.0', 'toNode': '5.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '6'},
        {'fromNode': '6.0', 'toNode': '4.0', 'Resistance': 0.00084, 'Reactance': 0.06858, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '7'},
        {'fromNode': '4.0', 'toNode': '65.0', 'Resistance': 0.00033, 'Reactance': 0.00388, 'Conductance': 0, 'Susceptance': 0.10475, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '8'},
        {'fromNode': '4.0', 'toNode': '77.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 126.64, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '9'},
        {'fromNode': '5.0', 'toNode': '12.0', 'Resistance': 0.0024, 'Reactance': 0.02795, 'Conductance': 0, 'Susceptance': 0.75408, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '10'},
        {'fromNode': '5.0', 'toNode': '16.0', 'Resistance': 0.00133, 'Reactance': 0.01553, 'Conductance': 0, 'Susceptance': 0.41898, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '11'},
        {'fromNode': '5.0', 'toNode': '68.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 149.53, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '12'},
        {'fromNode': '23.0', 'toNode': '6.0', 'Resistance': 0.00044, 'Reactance': -0.00422, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.06772, 'shift': 0.0, 'Line_id': '13'},
        {'fromNode': '24.0', 'toNode': '6.0', 'Resistance': 0.00496, 'Reactance': 0.06982, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 64.99, 'kV_base': 500.0, 'm': 0.98438, 'shift': 0.0, 'Line_id': '14'},
        {'fromNode': '7.0', 'toNode': '8.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '15'},
        {'fromNode': '7.0', 'toNode': '10.0', 'Resistance': 0.02738, 'Reactance': 0.14254, 'Conductance': 0, 'Susceptance': 0.20047, 'MVA_rating': 222.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '16'},
        {'fromNode': '25.0', 'toNode': '7.0', 'Resistance': 0.00015, 'Reactance': 0.06524, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.07325, 'shift': 0.0, 'Line_id': '17'},
        {'fromNode': '8.0', 'toNode': '17.0', 'Resistance': 0.01909, 'Reactance': 0.11277, 'Conductance': 0, 'Susceptance': 0.16673, 'MVA_rating': 197.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '18'},
        {'fromNode': '26.0', 'toNode': '8.0', 'Resistance': 0.00027, 'Reactance': 0.06715, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.08282, 'shift': 0.0, 'Line_id': '19'},
        {'fromNode': '9.0', 'toNode': '10.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '20'},
        {'fromNode': '9.0', 'toNode': '76.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '21'},
        {'fromNode': '11.0', 'toNode': '10.0', 'Resistance': 0.00076, 'Reactance': 0.0789, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 243.53, 'kV_base': 345.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '22'},
        {'fromNode': '10.0', 'toNode': '66.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '23'},
        {'fromNode': '33.0', 'toNode': '11.0', 'Resistance': -0.00017, 'Reactance': 0.16709, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 345.0, 'm': 0.94157, 'shift': 0.0, 'Line_id': '24'},
        {'fromNode': '11.0', 'toNode': '75.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '25'},
        {'fromNode': '12.0', 'toNode': '69.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 129.32, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '26'},
        {'fromNode': '13.0', 'toNode': '14.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '27'},
        {'fromNode': '13.0', 'toNode': '15.0', 'Resistance': 0.0015, 'Reactance': 0.01768, 'Conductance': 0, 'Susceptance': 0.46944, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '28'},
        {'fromNode': '52.0', 'toNode': '13.0', 'Resistance': 0.00059, 'Reactance': 0.04206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 1.04877, 'shift': 0.0, 'Line_id': '29'},
        {'fromNode': '15.0', 'toNode': '16.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '30'},
        {'fromNode': '63.0', 'toNode': '15.0', 'Resistance': 0.00137, 'Reactance': 0.05969, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.05449, 'shift': 0.0, 'Line_id': '31'},
        {'fromNode': '64.0', 'toNode': '15.0', 'Resistance': 0.00137, 'Reactance': 0.05969, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.05312, 'shift': 0.0, 'Line_id': '32'},
        {'fromNode': '15.0', 'toNode': '65.0', 'Resistance': 0.001, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.31426, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '33'},
        {'fromNode': '18.0', 'toNode': '37.0', 'Resistance': 0.021, 'Reactance': 0.0679, 'Conductance': 0, 'Susceptance': 0.00696, 'MVA_rating': 101.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '34'},
        {'fromNode': '18.0', 'toNode': '63.0', 'Resistance': 0.02139, 'Reactance': 0.06954, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '35'},
        {'fromNode': '19.0', 'toNode': '20.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '36'},
        {'fromNode': '19.0', 'toNode': '25.0', 'Resistance': 0.03025, 'Reactance': 0.09752, 'Conductance': 0, 'Susceptance': 0.01002, 'MVA_rating': 86.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '37'},
        {'fromNode': '19.0', 'toNode': '28.0', 'Resistance': 0.00983, 'Reactance': 0.01736, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '38'},
        {'fromNode': '19.0', 'toNode': '63.0', 'Resistance': 0.03669, 'Reactance': 0.11818, 'Conductance': 0, 'Susceptance': 0.01217, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '39'},
        {'fromNode': '20.0', 'toNode': '42.0', 'Resistance': 0.00711, 'Reactance': 0.02314, 'Conductance': 0, 'Susceptance': 0.00234, 'MVA_rating': 124.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '40'},
        {'fromNode': '20.0', 'toNode': '48.0', 'Resistance': 0.00331, 'Reactance': 0.01074, 'Conductance': 0, 'Susceptance': 0.00109, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '41'},
        {'fromNode': '21.0', 'toNode': '25.0', 'Resistance': 0.01116, 'Reactance': 0.01529, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '42'},
        {'fromNode': '21.0', 'toNode': '40.0', 'Resistance': 0.01116, 'Reactance': 0.01529, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '43'},
        {'fromNode': '22.0', 'toNode': '37.0', 'Resistance': 0.0405, 'Reactance': 0.07107, 'Conductance': 0, 'Susceptance': 0.00673, 'MVA_rating': 59.55, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '44'},
        {'fromNode': '22.0', 'toNode': '51.0', 'Resistance': 0.04628, 'Reactance': 0.07107, 'Conductance': 0, 'Susceptance': 0.0076, 'MVA_rating': 67.92, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '45'},
        {'fromNode': '23.0', 'toNode': '39.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '46'},
        {'fromNode': '23.0', 'toNode': '43.0', 'Resistance': 0.00793, 'Reactance': 0.02231, 'Conductance': 0, 'Susceptance': 0.00215, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '47'},
        {'fromNode': '23.0', 'toNode': '45.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00041, 'MVA_rating': 170.12, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '48'},
        {'fromNode': '23.0', 'toNode': '514.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 229.86, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '49'},
        {'fromNode': '25.0', 'toNode': '49.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '50'},
        {'fromNode': '25.0', 'toNode': '61.0', 'Resistance': 0.01977, 'Reactance': 0.06457, 'Conductance': 0, 'Susceptance': 0.00658, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '51'},
        {'fromNode': '25.0', 'toNode': '62.0', 'Resistance': 0.00218, 'Reactance': 0.00712, 'Conductance': 0, 'Susceptance': 0.00073, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '52'},
        {'fromNode': '26.0', 'toNode': '35.0', 'Resistance': 0.0065, 'Reactance': 0.02121, 'Conductance': 0, 'Susceptance': 0.00216, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '53'},
        {'fromNode': '26.0', 'toNode': '119.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 170.58, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '54'},
        {'fromNode': '27.0', 'toNode': '28.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '55'},
        {'fromNode': '27.0', 'toNode': '32.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '56'},
        {'fromNode': '27.0', 'toNode': '56.0', 'Resistance': 0.00099, 'Reactance': 0.00347, 'Conductance': 0, 'Susceptance': 0.00036, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '57'},
        {'fromNode': '27.0', 'toNode': '61.0', 'Resistance': 0.01083, 'Reactance': 0.03471, 'Conductance': 0, 'Susceptance': 0.00358, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '58'},
        {'fromNode': '28.0', 'toNode': '31.0', 'Resistance': 0.00496, 'Reactance': 0.01273, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '59'},
        {'fromNode': '28.0', 'toNode': '55.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '60'},
        {'fromNode': '29.0', 'toNode': '30.0', 'Resistance': 0.00142, 'Reactance': 0.00465, 'Conductance': 0, 'Susceptance': 0.00047, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '61'},
        {'fromNode': '29.0', 'toNode': '58.0', 'Resistance': 0.00408, 'Reactance': 0.01332, 'Conductance': 0, 'Susceptance': 0.00136, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '62'},
        {'fromNode': '30.0', 'toNode': '64.0', 'Resistance': 0.01705, 'Reactance': 0.05552, 'Conductance': 0, 'Susceptance': 0.00567, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '63'},
        {'fromNode': '31.0', 'toNode': '46.0', 'Resistance': 0.00372, 'Reactance': 0.00934, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '64'},
        {'fromNode': '32.0', 'toNode': '55.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '65'},
        {'fromNode': '34.0', 'toNode': '62.0', 'Resistance': 0.00133, 'Reactance': 0.00434, 'Conductance': 0, 'Susceptance': 0.00044, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '66'},
        {'fromNode': '36.0', 'toNode': '42.0', 'Resistance': 0.00289, 'Reactance': 0.00926, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '67'},
        {'fromNode': '36.0', 'toNode': '56.0', 'Resistance': 0.00207, 'Reactance': 0.00694, 'Conductance': 0, 'Susceptance': 0.0007, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '68'},
        {'fromNode': '37.0', 'toNode': '38.0', 'Resistance': 0.00413, 'Reactance': 0.01372, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '69'},
        {'fromNode': '38.0', 'toNode': '60.0', 'Resistance': 0.03438, 'Reactance': 0.06099, 'Conductance': 0, 'Susceptance': 0.00569, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '70'},
        {'fromNode': '39.0', 'toNode': '40.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '71'},
        {'fromNode': '39.0', 'toNode': '41.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '72'},
        {'fromNode': '40.0', 'toNode': '41.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '73'},
        {'fromNode': '44.0', 'toNode': '60.0', 'Resistance': 0.0395, 'Reactance': 0.1281, 'Conductance': 0, 'Susceptance': 0.01314, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '74'},
        {'fromNode': '44.0', 'toNode': '64.0', 'Resistance': 0.00165, 'Reactance': 0.00496, 'Conductance': 0, 'Susceptance': 0.00053, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '75'},
        {'fromNode': '46.0', 'toNode': '62.0', 'Resistance': 0.0235, 'Reactance': 0.07683, 'Conductance': 0, 'Susceptance': 0.00783, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '76'},
        {'fromNode': '47.0', 'toNode': '48.0', 'Resistance': 0.00645, 'Reactance': 0.02066, 'Conductance': 0, 'Susceptance': 0.00213, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '77'},
        {'fromNode': '47.0', 'toNode': '58.0', 'Resistance': 0.04174, 'Reactance': 0.07322, 'Conductance': 0, 'Susceptance': 0.00692, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '78'},
        {'fromNode': '47.0', 'toNode': '59.0', 'Resistance': 0.01132, 'Reactance': 0.03636, 'Conductance': 0, 'Susceptance': 0.00375, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '79'},
        {'fromNode': '50.0', 'toNode': '52.0', 'Resistance': 0.04351, 'Reactance': 0.09356, 'Conductance': 0, 'Susceptance': 0.00962, 'MVA_rating': 54.76, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '80'},
        {'fromNode': '51.0', 'toNode': '54.0', 'Resistance': 0.01653, 'Reactance': 0.03306, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 62.58, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '81'},
        {'fromNode': '52.0', 'toNode': '53.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '82'},
        {'fromNode': '52.0', 'toNode': '57.0', 'Resistance': 0.02394, 'Reactance': 0.04718, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 58.83, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '83'},
        {'fromNode': '54.0', 'toNode': '432.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 193.33, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '84'},
        {'fromNode': '59.0', 'toNode': '63.0', 'Resistance': 0.01721, 'Reactance': 0.05623, 'Conductance': 0, 'Susceptance': 0.00573, 'MVA_rating': 140.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '85'},
        {'fromNode': '63.0', 'toNode': '64.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '86'},
        {'fromNode': '66.0', 'toNode': '75.0', 'Resistance': 0.00186, 'Reactance': 0.02155, 'Conductance': 0, 'Susceptance': 0.05317, 'MVA_rating': 461.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '87'},
        {'fromNode': '67.0', 'toNode': '68.0', 'Resistance': 0.00051, 'Reactance': 0.0275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 500.0, 'm': 1.07702, 'shift': 0.0, 'Line_id': '88'},
        {'fromNode': '67.0', 'toNode': '82.0', 'Resistance': 0.00568, 'Reactance': 0.04215, 'Conductance': 0, 'Susceptance': 0.0716, 'MVA_rating': 317.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '89'},
        {'fromNode': '68.0', 'toNode': '79.0', 'Resistance': 0.00131, 'Reactance': 0.01547, 'Conductance': 0, 'Susceptance': 0.41152, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '90'},
        {'fromNode': '69.0', 'toNode': '70.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '91'},
        {'fromNode': '71.0', 'toNode': '69.0', 'Resistance': 0.00084, 'Reactance': 0.06858, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '92'},
        {'fromNode': '69.0', 'toNode': '130.0', 'Resistance': 0.00033, 'Reactance': 0.00388, 'Conductance': 0, 'Susceptance': 0.10475, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '93'},
        {'fromNode': '70.0', 'toNode': '77.0', 'Resistance': 0.0024, 'Reactance': 0.02795, 'Conductance': 0, 'Susceptance': 0.75408, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '94'},
        {'fromNode': '70.0', 'toNode': '81.0', 'Resistance': 0.00133, 'Reactance': 0.01553, 'Conductance': 0, 'Susceptance': 0.41898, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '95'},
        {'fromNode': '88.0', 'toNode': '71.0', 'Resistance': 0.00044, 'Reactance': -0.00422, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.06772, 'shift': 0.0, 'Line_id': '96'},
        {'fromNode': '89.0', 'toNode': '71.0', 'Resistance': 0.00496, 'Reactance': 0.06982, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 64.99, 'kV_base': 500.0, 'm': 0.98438, 'shift': 0.0, 'Line_id': '97'},
        {'fromNode': '72.0', 'toNode': '73.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '98'},
        {'fromNode': '72.0', 'toNode': '75.0', 'Resistance': 0.02738, 'Reactance': 0.14254, 'Conductance': 0, 'Susceptance': 0.20047, 'MVA_rating': 222.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '99'},
        {'fromNode': '90.0', 'toNode': '72.0', 'Resistance': 0.00015, 'Reactance': 0.06524, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.07325, 'shift': 0.0, 'Line_id': '100'},
        {'fromNode': '73.0', 'toNode': '82.0', 'Resistance': 0.01909, 'Reactance': 0.11277, 'Conductance': 0, 'Susceptance': 0.16673, 'MVA_rating': 197.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '101'},
        {'fromNode': '91.0', 'toNode': '73.0', 'Resistance': 0.00027, 'Reactance': 0.06715, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.08282, 'shift': 0.0, 'Line_id': '102'},
        {'fromNode': '74.0', 'toNode': '75.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '103'},
        {'fromNode': '74.0', 'toNode': '193.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 269.8, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '104'},
        {'fromNode': '76.0', 'toNode': '75.0', 'Resistance': 0.00076, 'Reactance': 0.0789, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 243.53, 'kV_base': 345.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '105'},
        {'fromNode': '98.0', 'toNode': '76.0', 'Resistance': -0.00017, 'Reactance': 0.16709, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 345.0, 'm': 0.94157, 'shift': 0.0, 'Line_id': '106'},
        {'fromNode': '78.0', 'toNode': '79.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '107'},
        {'fromNode': '78.0', 'toNode': '80.0', 'Resistance': 0.0015, 'Reactance': 0.01768, 'Conductance': 0, 'Susceptance': 0.46944, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '108'},
        {'fromNode': '117.0', 'toNode': '78.0', 'Resistance': 0.00059, 'Reactance': 0.04206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 1.04877, 'shift': 0.0, 'Line_id': '109'},
        {'fromNode': '80.0', 'toNode': '81.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '110'},
        {'fromNode': '128.0', 'toNode': '80.0', 'Resistance': 0.00137, 'Reactance': 0.05969, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.05449, 'shift': 0.0, 'Line_id': '111'},
        {'fromNode': '129.0', 'toNode': '80.0', 'Resistance': 0.00137, 'Reactance': 0.05969, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.05312, 'shift': 0.0, 'Line_id': '112'},
        {'fromNode': '80.0', 'toNode': '130.0', 'Resistance': 0.001, 'Reactance': 0.01165, 'Conductance': 0, 'Susceptance': 0.31426, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '113'},
        {'fromNode': '83.0', 'toNode': '102.0', 'Resistance': 0.021, 'Reactance': 0.0679, 'Conductance': 0, 'Susceptance': 0.00696, 'MVA_rating': 101.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '114'},
        {'fromNode': '83.0', 'toNode': '128.0', 'Resistance': 0.02139, 'Reactance': 0.06954, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '115'},
        {'fromNode': '84.0', 'toNode': '85.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '116'},
        {'fromNode': '84.0', 'toNode': '90.0', 'Resistance': 0.03025, 'Reactance': 0.09752, 'Conductance': 0, 'Susceptance': 0.01002, 'MVA_rating': 86.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '117'},
        {'fromNode': '84.0', 'toNode': '93.0', 'Resistance': 0.00983, 'Reactance': 0.01736, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '118'},
        {'fromNode': '84.0', 'toNode': '128.0', 'Resistance': 0.03669, 'Reactance': 0.11818, 'Conductance': 0, 'Susceptance': 0.01217, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '119'},
        {'fromNode': '85.0', 'toNode': '107.0', 'Resistance': 0.00711, 'Reactance': 0.02314, 'Conductance': 0, 'Susceptance': 0.00234, 'MVA_rating': 124.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '120'},
        {'fromNode': '85.0', 'toNode': '113.0', 'Resistance': 0.00331, 'Reactance': 0.01074, 'Conductance': 0, 'Susceptance': 0.00109, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '121'},
        {'fromNode': '86.0', 'toNode': '90.0', 'Resistance': 0.01116, 'Reactance': 0.01529, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '122'},
        {'fromNode': '86.0', 'toNode': '105.0', 'Resistance': 0.01116, 'Reactance': 0.01529, 'Conductance': 0, 'Susceptance': 0.00275, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '123'},
        {'fromNode': '87.0', 'toNode': '102.0', 'Resistance': 0.0405, 'Reactance': 0.07107, 'Conductance': 0, 'Susceptance': 0.00673, 'MVA_rating': 88.34, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '124'},
        {'fromNode': '87.0', 'toNode': '116.0', 'Resistance': 0.04628, 'Reactance': 0.07107, 'Conductance': 0, 'Susceptance': 0.0076, 'MVA_rating': 82.01, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '125'},
        {'fromNode': '88.0', 'toNode': '104.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '126'},
        {'fromNode': '88.0', 'toNode': '108.0', 'Resistance': 0.00793, 'Reactance': 0.02231, 'Conductance': 0, 'Susceptance': 0.00215, 'MVA_rating': 176.21, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '127'},
        {'fromNode': '88.0', 'toNode': '110.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00041, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '128'},
        {'fromNode': '90.0', 'toNode': '114.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '129'},
        {'fromNode': '90.0', 'toNode': '126.0', 'Resistance': 0.01977, 'Reactance': 0.06457, 'Conductance': 0, 'Susceptance': 0.00658, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '130'},
        {'fromNode': '90.0', 'toNode': '127.0', 'Resistance': 0.00218, 'Reactance': 0.00712, 'Conductance': 0, 'Susceptance': 0.00073, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '131'},
        {'fromNode': '91.0', 'toNode': '100.0', 'Resistance': 0.0065, 'Reactance': 0.02121, 'Conductance': 0, 'Susceptance': 0.00216, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '132'},
        {'fromNode': '91.0', 'toNode': '480.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '133'},
        {'fromNode': '92.0', 'toNode': '93.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '134'},
        {'fromNode': '92.0', 'toNode': '97.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '135'},
        {'fromNode': '92.0', 'toNode': '121.0', 'Resistance': 0.00099, 'Reactance': 0.00347, 'Conductance': 0, 'Susceptance': 0.00036, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '136'},
        {'fromNode': '92.0', 'toNode': '126.0', 'Resistance': 0.01083, 'Reactance': 0.03471, 'Conductance': 0, 'Susceptance': 0.00358, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '137'},
        {'fromNode': '93.0', 'toNode': '96.0', 'Resistance': 0.00496, 'Reactance': 0.01273, 'Conductance': 0, 'Susceptance': 0.00128, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '138'},
        {'fromNode': '93.0', 'toNode': '120.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '139'},
        {'fromNode': '94.0', 'toNode': '95.0', 'Resistance': 0.00142, 'Reactance': 0.00465, 'Conductance': 0, 'Susceptance': 0.00047, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '140'},
        {'fromNode': '94.0', 'toNode': '123.0', 'Resistance': 0.00408, 'Reactance': 0.01332, 'Conductance': 0, 'Susceptance': 0.00136, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '141'},
        {'fromNode': '95.0', 'toNode': '129.0', 'Resistance': 0.01705, 'Reactance': 0.05552, 'Conductance': 0, 'Susceptance': 0.00567, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '142'},
        {'fromNode': '96.0', 'toNode': '111.0', 'Resistance': 0.00372, 'Reactance': 0.00934, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '143'},
        {'fromNode': '97.0', 'toNode': '120.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '144'},
        {'fromNode': '99.0', 'toNode': '127.0', 'Resistance': 0.00133, 'Reactance': 0.00434, 'Conductance': 0, 'Susceptance': 0.00044, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '145'},
        {'fromNode': '101.0', 'toNode': '107.0', 'Resistance': 0.00289, 'Reactance': 0.00926, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '146'},
        {'fromNode': '101.0', 'toNode': '121.0', 'Resistance': 0.00207, 'Reactance': 0.00694, 'Conductance': 0, 'Susceptance': 0.0007, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '147'},
        {'fromNode': '102.0', 'toNode': '103.0', 'Resistance': 0.00413, 'Reactance': 0.01372, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '148'},
        {'fromNode': '103.0', 'toNode': '125.0', 'Resistance': 0.03438, 'Reactance': 0.06099, 'Conductance': 0, 'Susceptance': 0.00569, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '149'},
        {'fromNode': '104.0', 'toNode': '105.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '150'},
        {'fromNode': '104.0', 'toNode': '106.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '151'},
        {'fromNode': '105.0', 'toNode': '106.0', 'Resistance': 0.00124, 'Reactance': 0.00413, 'Conductance': 0, 'Susceptance': 0.00039, 'MVA_rating': 128.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '152'},
        {'fromNode': '108.0', 'toNode': '222.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 131.42, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '153'},
        {'fromNode': '109.0', 'toNode': '125.0', 'Resistance': 0.0395, 'Reactance': 0.1281, 'Conductance': 0, 'Susceptance': 0.01314, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '154'},
        {'fromNode': '109.0', 'toNode': '129.0', 'Resistance': 0.00165, 'Reactance': 0.00496, 'Conductance': 0, 'Susceptance': 0.00053, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '155'},
        {'fromNode': '111.0', 'toNode': '127.0', 'Resistance': 0.0235, 'Reactance': 0.07683, 'Conductance': 0, 'Susceptance': 0.00783, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '156'},
        {'fromNode': '112.0', 'toNode': '113.0', 'Resistance': 0.00645, 'Reactance': 0.02066, 'Conductance': 0, 'Susceptance': 0.00213, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '157'},
        {'fromNode': '112.0', 'toNode': '123.0', 'Resistance': 0.04174, 'Reactance': 0.07322, 'Conductance': 0, 'Susceptance': 0.00692, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '158'},
        {'fromNode': '112.0', 'toNode': '124.0', 'Resistance': 0.01132, 'Reactance': 0.03636, 'Conductance': 0, 'Susceptance': 0.00375, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '159'},
        {'fromNode': '115.0', 'toNode': '117.0', 'Resistance': 0.04351, 'Reactance': 0.09356, 'Conductance': 0, 'Susceptance': 0.00962, 'MVA_rating': 54.76, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '160'},
        {'fromNode': '116.0', 'toNode': '119.0', 'Resistance': 0.01653, 'Reactance': 0.03306, 'Conductance': 0, 'Susceptance': 0.00242, 'MVA_rating': 76.66, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '161'},
        {'fromNode': '117.0', 'toNode': '118.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '162'},
        {'fromNode': '117.0', 'toNode': '122.0', 'Resistance': 0.02394, 'Reactance': 0.04718, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 58.83, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '163'},
        {'fromNode': '124.0', 'toNode': '128.0', 'Resistance': 0.01721, 'Reactance': 0.05623, 'Conductance': 0, 'Susceptance': 0.00573, 'MVA_rating': 140.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '164'},
        {'fromNode': '128.0', 'toNode': '129.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '165'},
        {'fromNode': '131.0', 'toNode': '132.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '166'},
        {'fromNode': '131.0', 'toNode': '133.0', 'Resistance': 0.00033, 'Reactance': 0.02711, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.08613, 'shift': 0.0, 'Line_id': '167'},
        {'fromNode': '146.0', 'toNode': '131.0', 'Resistance': 0.00163, 'Reactance': 0.06671, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.06463, 'shift': 0.0, 'Line_id': '168'},
        {'fromNode': '148.0', 'toNode': '132.0', 'Resistance': 0.00163, 'Reactance': 0.06671, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.05908, 'shift': 0.0, 'Line_id': '169'},
        {'fromNode': '132.0', 'toNode': '190.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 229.39, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '170'},
        {'fromNode': '134.0', 'toNode': '133.0', 'Resistance': 0.00032, 'Reactance': 0.0464, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '171'},
        {'fromNode': '133.0', 'toNode': '188.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 379.94, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '172'},
        {'fromNode': '147.0', 'toNode': '134.0', 'Resistance': 0.00032, 'Reactance': -0.00435, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 1.06892, 'shift': 0.0, 'Line_id': '173'},
        {'fromNode': '149.0', 'toNode': '134.0', 'Resistance': -0.00022, 'Reactance': 0.1019, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 500.0, 'm': 0.96037, 'shift': 0.0, 'Line_id': '174'},
        {'fromNode': '167.0', 'toNode': '135.0', 'Resistance': -0.00013, 'Reactance': -0.00407, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 688.8, 'kV_base': 500.0, 'm': 1.05737, 'shift': 0.0, 'Line_id': '175'},
        {'fromNode': '135.0', 'toNode': '544.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 689.73, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '176'},
        {'fromNode': '136.0', 'toNode': '137.0', 'Resistance': 0.02165, 'Reactance': 0.0724, 'Conductance': 0, 'Susceptance': 0.00706, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '177'},
        {'fromNode': '136.0', 'toNode': '139.0', 'Resistance': 0.07884, 'Reactance': 0.10231, 'Conductance': 0, 'Susceptance': 0.01021, 'MVA_rating': 49.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '178'},
        {'fromNode': '137.0', 'toNode': '138.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '179'},
        {'fromNode': '138.0', 'toNode': '565.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '180'},
        {'fromNode': '139.0', 'toNode': '140.0', 'Resistance': 0.01322, 'Reactance': 0.01777, 'Conductance': 0, 'Susceptance': 0.00191, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '181'},
        {'fromNode': '139.0', 'toNode': '166.0', 'Resistance': 0.03636, 'Reactance': 0.12066, 'Conductance': 0, 'Susceptance': 0.01181, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '182'},
        {'fromNode': '139.0', 'toNode': '173.0', 'Resistance': 0.02331, 'Reactance': 0.07893, 'Conductance': 0, 'Susceptance': 0.00745, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '183'},
        {'fromNode': '140.0', 'toNode': '151.0', 'Resistance': 0.02231, 'Reactance': 0.07231, 'Conductance': 0, 'Susceptance': 0.01493, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '184'},
        {'fromNode': '141.0', 'toNode': '148.0', 'Resistance': 0.02868, 'Reactance': 0.09256, 'Conductance': 0, 'Susceptance': 0.00951, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '185'},
        {'fromNode': '141.0', 'toNode': '457.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '186'},
        {'fromNode': '142.0', 'toNode': '172.0', 'Resistance': 0.02975, 'Reactance': 0.09587, 'Conductance': 0, 'Susceptance': 0.01007, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '187'},
        {'fromNode': '142.0', 'toNode': '577.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '188'},
        {'fromNode': '143.0', 'toNode': '167.0', 'Resistance': 0.01039, 'Reactance': 0.05083, 'Conductance': 0, 'Susceptance': 0.00494, 'MVA_rating': 133.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '189'},
        {'fromNode': '143.0', 'toNode': '169.0', 'Resistance': 0.02446, 'Reactance': 0.04347, 'Conductance': 0, 'Susceptance': 0.00404, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '190'},
        {'fromNode': '144.0', 'toNode': '145.0', 'Resistance': 0.00149, 'Reactance': 0.00496, 'Conductance': 0, 'Susceptance': 0.00048, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '191'},
        {'fromNode': '144.0', 'toNode': '177.0', 'Resistance': 0.00688, 'Reactance': 0.02158, 'Conductance': 0, 'Susceptance': 0.0024, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '192'},
        {'fromNode': '145.0', 'toNode': '175.0', 'Resistance': 0.00273, 'Reactance': 0.00826, 'Conductance': 0, 'Susceptance': 0.00085, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '193'},
        {'fromNode': '146.0', 'toNode': '147.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '194'},
        {'fromNode': '146.0', 'toNode': '151.0', 'Resistance': 0.0314, 'Reactance': 0.09884, 'Conductance': 0, 'Susceptance': 0.01074, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '195'},
        {'fromNode': '146.0', 'toNode': '154.0', 'Resistance': 0.04, 'Reactance': 0.12959, 'Conductance': 0, 'Susceptance': 0.01329, 'MVA_rating': 151.36, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '196'},
        {'fromNode': '146.0', 'toNode': '174.0', 'Resistance': 0.00736, 'Reactance': 0.0238, 'Conductance': 0, 'Susceptance': 0.00251, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '197'},
        {'fromNode': '147.0', 'toNode': '148.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '198'},
        {'fromNode': '147.0', 'toNode': '155.0', 'Resistance': 0.04025, 'Reactance': 0.13025, 'Conductance': 0, 'Susceptance': 0.01336, 'MVA_rating': 114.35, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '199'},
        {'fromNode': '147.0', 'toNode': '157.0', 'Resistance': 0.02124, 'Reactance': 0.0686, 'Conductance': 0, 'Susceptance': 0.00704, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '200'},
        {'fromNode': '147.0', 'toNode': '175.0', 'Resistance': 0.0095, 'Reactance': 0.02455, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '201'},
        {'fromNode': '148.0', 'toNode': '161.0', 'Resistance': 0.02781, 'Reactance': 0.07968, 'Conductance': 0, 'Susceptance': 0.00807, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '202'},
        {'fromNode': '148.0', 'toNode': '177.0', 'Resistance': 0.00883, 'Reactance': 0.02792, 'Conductance': 0, 'Susceptance': 0.00305, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '203'},
        {'fromNode': '150.0', 'toNode': '158.0', 'Resistance': 0.00983, 'Reactance': 0.01727, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '204'},
        {'fromNode': '150.0', 'toNode': '166.0', 'Resistance': 0.04669, 'Reactance': 0.08182, 'Conductance': 0, 'Susceptance': 0.00774, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '205'},
        {'fromNode': '150.0', 'toNode': '170.0', 'Resistance': 0.00992, 'Reactance': 0.03322, 'Conductance': 0, 'Susceptance': 0.00419, 'MVA_rating': 107.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '206'},
        {'fromNode': '150.0', 'toNode': '176.0', 'Resistance': 0.02455, 'Reactance': 0.07992, 'Conductance': 0, 'Susceptance': 0.00823, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '207'},
        {'fromNode': '152.0', 'toNode': '176.0', 'Resistance': 0.01338, 'Reactance': 0.04326, 'Conductance': 0, 'Susceptance': 0.00444, 'MVA_rating': 140.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '208'},
        {'fromNode': '153.0', 'toNode': '154.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '209'},
        {'fromNode': '153.0', 'toNode': '155.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '210'},
        {'fromNode': '153.0', 'toNode': '156.0', 'Resistance': 0.00405, 'Reactance': 0.01331, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '211'},
        {'fromNode': '153.0', 'toNode': '178.0', 'Resistance': 0.03017, 'Reactance': 0.09764, 'Conductance': 0, 'Susceptance': 0.01004, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '212'},
        {'fromNode': '156.0', 'toNode': '163.0', 'Resistance': 0.02074, 'Reactance': 0.06628, 'Conductance': 0, 'Susceptance': 0.00699, 'MVA_rating': 126.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '213'},
        {'fromNode': '157.0', 'toNode': '170.0', 'Resistance': 0.03364, 'Reactance': 0.10744, 'Conductance': 0, 'Susceptance': 0.01016, 'MVA_rating': 107.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '214'},
        {'fromNode': '158.0', 'toNode': '169.0', 'Resistance': 0.05322, 'Reactance': 0.09347, 'Conductance': 0, 'Susceptance': 0.00882, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '215'},
        {'fromNode': '159.0', 'toNode': '171.0', 'Resistance': 0.05042, 'Reactance': 0.10487, 'Conductance': 0, 'Susceptance': 0.0102, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '216'},
        {'fromNode': '159.0', 'toNode': '260.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '217'},
        {'fromNode': '160.0', 'toNode': '167.0', 'Resistance': 0.03603, 'Reactance': 0.12893, 'Conductance': 0, 'Susceptance': 0.01251, 'MVA_rating': 90.54, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '218'},
        {'fromNode': '160.0', 'toNode': '271.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '219'},
        {'fromNode': '162.0', 'toNode': '172.0', 'Resistance': 0.00512, 'Reactance': 0.01653, 'Conductance': 0, 'Susceptance': 0.00174, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '220'},
        {'fromNode': '162.0', 'toNode': '178.0', 'Resistance': 0.01103, 'Reactance': 0.03572, 'Conductance': 0, 'Susceptance': 0.00367, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '221'},
        {'fromNode': '163.0', 'toNode': '176.0', 'Resistance': 0.02835, 'Reactance': 0.09223, 'Conductance': 0, 'Susceptance': 0.00951, 'MVA_rating': 137.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '222'},
        {'fromNode': '164.0', 'toNode': '174.0', 'Resistance': 0.00826, 'Reactance': 0.01653, 'Conductance': 0, 'Susceptance': 0.00194, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '223'},
        {'fromNode': '164.0', 'toNode': '175.0', 'Resistance': 0.00421, 'Reactance': 0.01074, 'Conductance': 0, 'Susceptance': 0.00121, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '224'},
        {'fromNode': '165.0', 'toNode': '167.0', 'Resistance': 0.03041, 'Reactance': 0.0986, 'Conductance': 0, 'Susceptance': 0.01013, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '225'},
        {'fromNode': '165.0', 'toNode': '275.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 133.86, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '226'},
        {'fromNode': '167.0', 'toNode': '168.0', 'Resistance': 0.03587, 'Reactance': 0.1157, 'Conductance': 0, 'Susceptance': 0.0122, 'MVA_rating': 140.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '227'},
        {'fromNode': '167.0', 'toNode': '569.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 627.77, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '228'},
        {'fromNode': '168.0', 'toNode': '214.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '229'},
        {'fromNode': '171.0', 'toNode': '173.0', 'Resistance': 0.01976, 'Reactance': 0.0514, 'Conductance': 0, 'Susceptance': 0.00515, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '230'},
        {'fromNode': '175.0', 'toNode': '179.0', 'Resistance': 0.02017, 'Reactance': 0.05223, 'Conductance': 0, 'Susceptance': 0.00545, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '231'},
        {'fromNode': '175.0', 'toNode': '180.0', 'Resistance': 0.03562, 'Reactance': 0.10942, 'Conductance': 0, 'Susceptance': 0.01146, 'MVA_rating': 69.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '232'},
        {'fromNode': '180.0', 'toNode': '281.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '233'},
        {'fromNode': '181.0', 'toNode': '182.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '234'},
        {'fromNode': '181.0', 'toNode': '184.0', 'Resistance': 0.00568, 'Reactance': 0.04012, 'Conductance': 0, 'Susceptance': 0.07033, 'MVA_rating': 317.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '235'},
        {'fromNode': '181.0', 'toNode': '189.0', 'Resistance': 0.00105, 'Reactance': 0.00733, 'Conductance': 0, 'Susceptance': 0.01258, 'MVA_rating': 231.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '236'},
        {'fromNode': '183.0', 'toNode': '184.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '237'},
        {'fromNode': '183.0', 'toNode': '230.0', 'Resistance': 0.0018, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.0484, 'MVA_rating': 754.02, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '238'},
        {'fromNode': '183.0', 'toNode': '424.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 487.52, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '239'},
        {'fromNode': '184.0', 'toNode': '185.0', 'Resistance': 0.00064, 'Reactance': 0.0305, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 709.38, 'kV_base': 500.0, 'm': 1.07408, 'shift': 0.0, 'Line_id': '240'},
        {'fromNode': '207.0', 'toNode': '184.0', 'Resistance': 0.00163, 'Reactance': 0.06136, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.0215, 'shift': 0.0, 'Line_id': '241'},
        {'fromNode': '184.0', 'toNode': '230.0', 'Resistance': 0.0018, 'Reactance': 0.01616, 'Conductance': 0, 'Susceptance': 0.0484, 'MVA_rating': 734.76, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '242'},
        {'fromNode': '185.0', 'toNode': '186.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '243'},
        {'fromNode': '185.0', 'toNode': '196.0', 'Resistance': 0.00296, 'Reactance': 0.03484, 'Conductance': 0, 'Susceptance': 0.9296, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '244'},
        {'fromNode': '186.0', 'toNode': '192.0', 'Resistance': 0.00373, 'Reactance': 0.04344, 'Conductance': 0, 'Susceptance': 1.172, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '245'},
        {'fromNode': '188.0', 'toNode': '187.0', 'Resistance': 0.00032, 'Reactance': 0.04392, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 562.44, 'kV_base': 500.0, 'm': 0.9999, 'shift': 0.0, 'Line_id': '246'},
        {'fromNode': '187.0', 'toNode': '192.0', 'Resistance': 0.00228, 'Reactance': 0.02296, 'Conductance': 0, 'Susceptance': 0.8129, 'MVA_rating': 1386.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '247'},
        {'fromNode': '187.0', 'toNode': '549.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 494.5, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '248'},
        {'fromNode': '211.0', 'toNode': '188.0', 'Resistance': -0.00023, 'Reactance': 0.09951, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 500.0, 'm': 0.96037, 'shift': 0.0, 'Line_id': '249'},
        {'fromNode': '190.0', 'toNode': '191.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '250'},
        {'fromNode': '190.0', 'toNode': '192.0', 'Resistance': 0.00035, 'Reactance': 0.02375, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.08671, 'shift': 0.0, 'Line_id': '251'},
        {'fromNode': '190.0', 'toNode': '193.0', 'Resistance': 0.00236, 'Reactance': 0.01411, 'Conductance': 0, 'Susceptance': 0.02031, 'MVA_rating': 229.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '252'},
        {'fromNode': '213.0', 'toNode': '191.0', 'Resistance': 0.00165, 'Reactance': 0.0593, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 211.32, 'kV_base': 345.0, 'm': 1.10254, 'shift': 0.0, 'Line_id': '253'},
        {'fromNode': '191.0', 'toNode': '353.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 677.01, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '254'},
        {'fromNode': '212.0', 'toNode': '192.0', 'Resistance': 0.00101, 'Reactance': 0.06188, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 1.1306, 'shift': 0.0, 'Line_id': '255'},
        {'fromNode': '213.0', 'toNode': '192.0', 'Resistance': 0.00153, 'Reactance': 0.06206, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.10126, 'shift': 0.0, 'Line_id': '256'},
        {'fromNode': '192.0', 'toNode': '229.0', 'Resistance': 0.00019, 'Reactance': 0.00196, 'Conductance': 0, 'Susceptance': 0.06877, 'MVA_rating': 831.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '257'},
        {'fromNode': '194.0', 'toNode': '195.0', 'Resistance': 0.00051, 'Reactance': 0.0275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 400.0, 'kV_base': 500.0, 'm': 1.07702, 'shift': 0.0, 'Line_id': '258'},
        {'fromNode': '195.0', 'toNode': '196.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '259'},
        {'fromNode': '195.0', 'toNode': '197.0', 'Resistance': 0.00131, 'Reactance': 0.01547, 'Conductance': 0, 'Susceptance': 0.41152, 'MVA_rating': 838.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '260'},
        {'fromNode': '223.0', 'toNode': '196.0', 'Resistance': 0.00137, 'Reactance': 0.05968, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 500.0, 'm': 1.05095, 'shift': 0.0, 'Line_id': '261'},
        {'fromNode': '197.0', 'toNode': '557.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.15, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '262'},
        {'fromNode': '198.0', 'toNode': '210.0', 'Resistance': 0.01452, 'Reactance': 0.04917, 'Conductance': 0, 'Susceptance': 0.00487, 'MVA_rating': 120.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '263'},
        {'fromNode': '198.0', 'toNode': '213.0', 'Resistance': 0.02276, 'Reactance': 0.0771, 'Conductance': 0, 'Susceptance': 0.00763, 'MVA_rating': 153.09, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '264'},
        {'fromNode': '199.0', 'toNode': '200.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '265'},
        {'fromNode': '199.0', 'toNode': '207.0', 'Resistance': 0.00064, 'Reactance': 0.00483, 'Conductance': 0, 'Susceptance': 0.00047, 'MVA_rating': 161.57, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '266'},
        {'fromNode': '201.0', 'toNode': '213.0', 'Resistance': 0.00731, 'Reactance': 0.01279, 'Conductance': 0, 'Susceptance': 0.00122, 'MVA_rating': 78.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '267'},
        {'fromNode': '202.0', 'toNode': '216.0', 'Resistance': 0.02035, 'Reactance': 0.03558, 'Conductance': 0, 'Susceptance': 0.0034, 'MVA_rating': 78.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '268'},
        {'fromNode': '203.0', 'toNode': '217.0', 'Resistance': 0.00202, 'Reactance': 0.00325, 'Conductance': 0, 'Susceptance': 0.02903, 'MVA_rating': 80.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '269'},
        {'fromNode': '204.0', 'toNode': '208.0', 'Resistance': 0.00405, 'Reactance': 0.01331, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '270'},
        {'fromNode': '204.0', 'toNode': '213.0', 'Resistance': 0.00574, 'Reactance': 0.01944, 'Conductance': 0, 'Susceptance': 0.00192, 'MVA_rating': 160.2, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '271'},
        {'fromNode': '205.0', 'toNode': '220.0', 'Resistance': 0.04645, 'Reactance': 0.08174, 'Conductance': 0, 'Susceptance': 0.00801, 'MVA_rating': 93.92, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '272'},
        {'fromNode': '205.0', 'toNode': '581.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '273'},
        {'fromNode': '206.0', 'toNode': '212.0', 'Resistance': 0.01612, 'Reactance': 0.05405, 'Conductance': 0, 'Susceptance': 0.00532, 'MVA_rating': 170.4, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '274'},
        {'fromNode': '206.0', 'toNode': '219.0', 'Resistance': 0.00615, 'Reactance': 0.02083, 'Conductance': 0, 'Susceptance': 0.00206, 'MVA_rating': 209.66, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '275'},
        {'fromNode': '206.0', 'toNode': '573.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 119.22, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '276'},
        {'fromNode': '209.0', 'toNode': '212.0', 'Resistance': 0.01344, 'Reactance': 0.02383, 'Conductance': 0, 'Susceptance': 0.00227, 'MVA_rating': 116.43, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '277'},
        {'fromNode': '209.0', 'toNode': '216.0', 'Resistance': 0.00698, 'Reactance': 0.01279, 'Conductance': 0, 'Susceptance': 0.00098, 'MVA_rating': 78.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '278'},
        {'fromNode': '212.0', 'toNode': '218.0', 'Resistance': 0.00959, 'Reactance': 0.02474, 'Conductance': 0, 'Susceptance': 0.00251, 'MVA_rating': 160.94, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '279'},
        {'fromNode': '212.0', 'toNode': '220.0', 'Resistance': 0.02397, 'Reactance': 0.04223, 'Conductance': 0, 'Susceptance': 0.00414, 'MVA_rating': 78.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '280'},
        {'fromNode': '213.0', 'toNode': '214.0', 'Resistance': 0.08066, 'Reactance': 0.14207, 'Conductance': 0, 'Susceptance': 0.01348, 'MVA_rating': 180.21, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '281'},
        {'fromNode': '213.0', 'toNode': '217.0', 'Resistance': 0.00959, 'Reactance': 0.02474, 'Conductance': 0, 'Susceptance': 0.00251, 'MVA_rating': 102.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '282'},
        {'fromNode': '214.0', 'toNode': '215.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '283'},
        {'fromNode': '218.0', 'toNode': '221.0', 'Resistance': 0.00413, 'Reactance': 0.01405, 'Conductance': 0, 'Susceptance': 0.00138, 'MVA_rating': 173.24, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '284'},
        {'fromNode': '221.0', 'toNode': '483.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 144.6, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '285'},
        {'fromNode': '222.0', 'toNode': '223.0', 'Resistance': 0.05504, 'Reactance': 0.1643, 'Conductance': 0, 'Susceptance': 0.01682, 'MVA_rating': 114.16, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '286'},
        {'fromNode': '223.0', 'toNode': '224.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '287'},
        {'fromNode': '223.0', 'toNode': '226.0', 'Resistance': 0.00926, 'Reactance': 0.03008, 'Conductance': 0, 'Susceptance': 0.00307, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '288'},
        {'fromNode': '223.0', 'toNode': '227.0', 'Resistance': 0.01463, 'Reactance': 0.05661, 'Conductance': 0, 'Susceptance': 0.00653, 'MVA_rating': 140.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '289'},
        {'fromNode': '223.0', 'toNode': '228.0', 'Resistance': 0.02688, 'Reactance': 0.08734, 'Conductance': 0, 'Susceptance': 0.00915, 'MVA_rating': 123.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '290'},
        {'fromNode': '225.0', 'toNode': '226.0', 'Resistance': 0.00132, 'Reactance': 0.00421, 'Conductance': 0, 'Susceptance': 0.00044, 'MVA_rating': 119.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '291'},
        {'fromNode': '231.0', 'toNode': '238.0', 'Resistance': 0.098, 'Reactance': 0.1715, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '292'},
        {'fromNode': '231.0', 'toNode': '269.0', 'Resistance': 0.00346, 'Reactance': 0.00605, 'Conductance': 0, 'Susceptance': 0.00011, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '293'},
        {'fromNode': '231.0', 'toNode': '339.0', 'Resistance': 0.14867, 'Reactance': 0.26016, 'Conductance': 0, 'Susceptance': 0.00432, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '294'},
        {'fromNode': '232.0', 'toNode': '284.0', 'Resistance': 0.06171, 'Reactance': 0.10799, 'Conductance': 0, 'Susceptance': 0.0018, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '295'},
        {'fromNode': '232.0', 'toNode': '344.0', 'Resistance': 0.01711, 'Reactance': 0.02994, 'Conductance': 0, 'Susceptance': 0.0005, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '296'},
        {'fromNode': '233.0', 'toNode': '285.0', 'Resistance': 0.07, 'Reactance': 0.1472, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '297'},
        {'fromNode': '233.0', 'toNode': '301.0', 'Resistance': 0.009, 'Reactance': 0.0284, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '298'},
        {'fromNode': '234.0', 'toNode': '300.0', 'Resistance': 0.0355, 'Reactance': 0.06201, 'Conductance': 0, 'Susceptance': 0.00104, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '299'},
        {'fromNode': '234.0', 'toNode': '333.0', 'Resistance': 0.01907, 'Reactance': 0.03338, 'Conductance': 0, 'Susceptance': 0.00054, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '300'},
        {'fromNode': '235.0', 'toNode': '246.0', 'Resistance': 0.01216, 'Reactance': 0.02127, 'Conductance': 0, 'Susceptance': 0.00034, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '301'},
        {'fromNode': '236.0', 'toNode': '242.0', 'Resistance': 0.09892, 'Reactance': 0.17311, 'Conductance': 0, 'Susceptance': 0.00286, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '302'},
        {'fromNode': '236.0', 'toNode': '312.0', 'Resistance': 0.0268, 'Reactance': 0.047, 'Conductance': 0, 'Susceptance': 0.0008, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '303'},
        {'fromNode': '310.0', 'toNode': '237.0', 'Resistance': 0.1522, 'Reactance': 0.1534, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 13.8, 'm': 1.0, 'shift': 0.0, 'Line_id': '304'},
        {'fromNode': '238.0', 'toNode': '253.0', 'Resistance': 0.0056, 'Reactance': 0.0098, 'Conductance': 0, 'Susceptance': 0.00015, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '305'},
        {'fromNode': '239.0', 'toNode': '247.0', 'Resistance': 0.0287, 'Reactance': 0.05023, 'Conductance': 0, 'Susceptance': 0.00082, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '306'},
        {'fromNode': '240.0', 'toNode': '283.0', 'Resistance': 0.01814, 'Reactance': 0.03174, 'Conductance': 0, 'Susceptance': 0.00054, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '307'},
        {'fromNode': '241.0', 'toNode': '282.0', 'Resistance': 0.0486, 'Reactance': 0.0851, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '308'},
        {'fromNode': '241.0', 'toNode': '337.0', 'Resistance': 0.0178, 'Reactance': 0.0311, 'Conductance': 0, 'Susceptance': 0.0005, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '309'},
        {'fromNode': '242.0', 'toNode': '285.0', 'Resistance': 0.0561, 'Reactance': 0.09817, 'Conductance': 0, 'Susceptance': 0.00162, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '310'},
        {'fromNode': '243.0', 'toNode': '277.0', 'Resistance': 0.01494, 'Reactance': 0.02212, 'Conductance': 0, 'Susceptance': 0.00034, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '311'},
        {'fromNode': '244.0', 'toNode': '302.0', 'Resistance': 0.00617, 'Reactance': 0.04003, 'Conductance': 0, 'Susceptance': 0.00693, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '312'},
        {'fromNode': '244.0', 'toNode': '309.0', 'Resistance': 0.0066, 'Reactance': 0.0428, 'Conductance': 0, 'Susceptance': 0.021, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '313'},
        {'fromNode': '245.0', 'toNode': '257.0', 'Resistance': 0.01169, 'Reactance': 0.02045, 'Conductance': 0, 'Susceptance': 0.00034, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '314'},
        {'fromNode': '245.0', 'toNode': '339.0', 'Resistance': 0.08041, 'Reactance': 0.14071, 'Conductance': 0, 'Susceptance': 0.00234, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '315'},
        {'fromNode': '245.0', 'toNode': '340.0', 'Resistance': 0.0561, 'Reactance': 0.09817, 'Conductance': 0, 'Susceptance': 0.00162, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '316'},
        {'fromNode': '246.0', 'toNode': '337.0', 'Resistance': 0.0094, 'Reactance': 0.0164, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '317'},
        {'fromNode': '247.0', 'toNode': '264.0', 'Resistance': 0.0275, 'Reactance': 0.0482, 'Conductance': 0, 'Susceptance': 0.0008, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '318'},
        {'fromNode': '247.0', 'toNode': '300.0', 'Resistance': 0.0662, 'Reactance': 0.11584, 'Conductance': 0, 'Susceptance': 0.00192, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '319'},
        {'fromNode': '267.0', 'toNode': '248.0', 'Resistance': 0.0, 'Reactance': 0.03225, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 138.0, 'm': 1.0224, 'shift': 0.0, 'Line_id': '320'},
        {'fromNode': '248.0', 'toNode': '469.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 83.7, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '321'},
        {'fromNode': '249.0', 'toNode': '278.0', 'Resistance': 0.00958, 'Reactance': 0.0622, 'Conductance': 0, 'Susceptance': 0.03069, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '322'},
        {'fromNode': '249.0', 'toNode': '309.0', 'Resistance': 0.00843, 'Reactance': 0.0548, 'Conductance': 0, 'Susceptance': 0.02704, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '323'},
        {'fromNode': '250.0', 'toNode': '270.0', 'Resistance': 0.1184, 'Reactance': 0.2053, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '324'},
        {'fromNode': '250.0', 'toNode': '334.0', 'Resistance': 0.0421, 'Reactance': 0.0755, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '325'},
        {'fromNode': '251.0', 'toNode': '282.0', 'Resistance': 0.0103, 'Reactance': 0.018, 'Conductance': 0, 'Susceptance': 0.0003, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '326'},
        {'fromNode': '251.0', 'toNode': '320.0', 'Resistance': 0.02534, 'Reactance': 0.04434, 'Conductance': 0, 'Susceptance': 0.00074, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '327'},
        {'fromNode': '252.0', 'toNode': '272.0', 'Resistance': 0.03165, 'Reactance': 0.08038, 'Conductance': 0, 'Susceptance': 0.00154, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '328'},
        {'fromNode': '309.0', 'toNode': '252.0', 'Resistance': 0.0, 'Reactance': 0.18689, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0058, 'shift': 0.0, 'Line_id': '329'},
        {'fromNode': '252.0', 'toNode': '340.0', 'Resistance': 0.0001, 'Reactance': 0.001, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '330'},
        {'fromNode': '253.0', 'toNode': '328.0', 'Resistance': 0.06199, 'Reactance': 0.10848, 'Conductance': 0, 'Susceptance': 0.0018, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '331'},
        {'fromNode': '254.0', 'toNode': '297.0', 'Resistance': 0.05115, 'Reactance': 0.08951, 'Conductance': 0, 'Susceptance': 0.0015, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '332'},
        {'fromNode': '254.0', 'toNode': '304.0', 'Resistance': 0.02805, 'Reactance': 0.04909, 'Conductance': 0, 'Susceptance': 0.0008, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '333'},
        {'fromNode': '255.0', 'toNode': '308.0', 'Resistance': 0.01777, 'Reactance': 0.03109, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '334'},
        {'fromNode': '255.0', 'toNode': '316.0', 'Resistance': 0.05386, 'Reactance': 0.09425, 'Conductance': 0, 'Susceptance': 0.00156, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '335'},
        {'fromNode': '256.0', 'toNode': '345.0', 'Resistance': 0.03703, 'Reactance': 0.06479, 'Conductance': 0, 'Susceptance': 0.00106, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '336'},
        {'fromNode': '258.0', 'toNode': '262.0', 'Resistance': 0.09799, 'Reactance': 0.17147, 'Conductance': 0, 'Susceptance': 0.00284, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '337'},
        {'fromNode': '259.0', 'toNode': '276.0', 'Resistance': 0.00458, 'Reactance': 0.00802, 'Conductance': 0, 'Susceptance': 0.00014, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '338'},
        {'fromNode': '259.0', 'toNode': '286.0', 'Resistance': 0.0869, 'Reactance': 0.1522, 'Conductance': 0, 'Susceptance': 0.0025, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '339'},
        {'fromNode': '259.0', 'toNode': '331.0', 'Resistance': 0.08499, 'Reactance': 0.14873, 'Conductance': 0, 'Susceptance': 0.00246, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '340'},
        {'fromNode': '293.0', 'toNode': '260.0', 'Resistance': 0.0, 'Reactance': 0.08294, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 138.0, 'm': 1.0074, 'shift': 0.0, 'Line_id': '341'},
        {'fromNode': '261.0', 'toNode': '320.0', 'Resistance': 0.0047, 'Reactance': 0.0082, 'Conductance': 0, 'Susceptance': 0.00014, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '342'},
        {'fromNode': '261.0', 'toNode': '322.0', 'Resistance': 0.0412, 'Reactance': 0.1047, 'Conductance': 0, 'Susceptance': 0.002, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '343'},
        {'fromNode': '261.0', 'toNode': '348.0', 'Resistance': 0.0346, 'Reactance': 0.0605, 'Conductance': 0, 'Susceptance': 0.0001, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '344'},
        {'fromNode': '262.0', 'toNode': '273.0', 'Resistance': 0.05256, 'Reactance': 0.1335, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '345'},
        {'fromNode': '262.0', 'toNode': '299.0', 'Resistance': 0.10089, 'Reactance': 0.17655, 'Conductance': 0, 'Susceptance': 0.00292, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '346'},
        {'fromNode': '263.0', 'toNode': '312.0', 'Resistance': 0.02683, 'Reactance': 0.04696, 'Conductance': 0, 'Susceptance': 0.00078, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '347'},
        {'fromNode': '263.0', 'toNode': '314.0', 'Resistance': 0.03142, 'Reactance': 0.05498, 'Conductance': 0, 'Susceptance': 0.00092, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '348'},
        {'fromNode': '264.0', 'toNode': '331.0', 'Resistance': 0.0275, 'Reactance': 0.0482, 'Conductance': 0, 'Susceptance': 0.0008, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '349'},
        {'fromNode': '265.0', 'toNode': '268.0', 'Resistance': 0.0, 'Reactance': 0.03774, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 483.0, 'kV_base': 138.0, 'm': 0.9997, 'shift': 0.0, 'Line_id': '350'},
        {'fromNode': '265.0', 'toNode': '294.0', 'Resistance': 0.0013, 'Reactance': 0.0159, 'Conductance': 0, 'Susceptance': 0.2748, 'MVA_rating': 684.22, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '351'},
        {'fromNode': '265.0', 'toNode': '296.0', 'Resistance': 0.0012, 'Reactance': 0.01423, 'Conductance': 0, 'Susceptance': 0.2354, 'MVA_rating': 851.53, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '352'},
        {'fromNode': '266.0', 'toNode': '340.0', 'Resistance': 0.05483, 'Reactance': 0.1128, 'Conductance': 0, 'Susceptance': 0.00206, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '353'},
        {'fromNode': '267.0', 'toNode': '324.0', 'Resistance': 0.02901, 'Reactance': 0.18849, 'Conductance': 0, 'Susceptance': 0.09302, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '354'},
        {'fromNode': '267.0', 'toNode': '350.0', 'Resistance': 0.0, 'Reactance': 0.14449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.02, 'shift': 0.0, 'Line_id': '355'},
        {'fromNode': '267.0', 'toNode': '350.0', 'Resistance': 0.0, 'Reactance': 0.14449, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.02, 'shift': 0.0, 'Line_id': '356'},
        {'fromNode': '311.0', 'toNode': '268.0', 'Resistance': 0.0, 'Reactance': 0.01641, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 193.0, 'kV_base': 138.0, 'm': 1.0006, 'shift': 0.0, 'Line_id': '357'},
        {'fromNode': '268.0', 'toNode': '327.0', 'Resistance': 0.0, 'Reactance': 0.14478, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 81.0, 'kV_base': 69.0, 'm': 1.0054, 'shift': 0.0, 'Line_id': '358'},
        {'fromNode': '270.0', 'toNode': '289.0', 'Resistance': 0.06826, 'Reactance': 0.11944, 'Conductance': 0, 'Susceptance': 0.00198, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '359'},
        {'fromNode': '271.0', 'toNode': '321.0', 'Resistance': 0.0, 'Reactance': 0.10324, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 0.9825, 'shift': 0.0, 'Line_id': '360'},
        {'fromNode': '272.0', 'toNode': '320.0', 'Resistance': 0.05478, 'Reactance': 0.13911, 'Conductance': 0, 'Susceptance': 0.00264, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '361'},
        {'fromNode': '281.0', 'toNode': '273.0', 'Resistance': 0.0, 'Reactance': 0.13708, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 0.9795, 'shift': 0.0, 'Line_id': '362'},
        {'fromNode': '273.0', 'toNode': '319.0', 'Resistance': 0.01752, 'Reactance': 0.04449, 'Conductance': 0, 'Susceptance': 0.00086, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '363'},
        {'fromNode': '274.0', 'toNode': '327.0', 'Resistance': 0.04264, 'Reactance': 0.07461, 'Conductance': 0, 'Susceptance': 0.00124, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '364'},
        {'fromNode': '274.0', 'toNode': '345.0', 'Resistance': 0.02609, 'Reactance': 0.04565, 'Conductance': 0, 'Susceptance': 0.00076, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '365'},
        {'fromNode': '309.0', 'toNode': '275.0', 'Resistance': 0.0, 'Reactance': 0.01583, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 138.0, 'm': 1.0077, 'shift': 0.0, 'Line_id': '366'},
        {'fromNode': '276.0', 'toNode': '344.0', 'Resistance': 0.07022, 'Reactance': 0.12288, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '367'},
        {'fromNode': '277.0', 'toNode': '326.0', 'Resistance': 0.01051, 'Reactance': 0.02669, 'Conductance': 0, 'Susceptance': 0.0005, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '368'},
        {'fromNode': '278.0', 'toNode': '303.0', 'Resistance': 0.0001, 'Reactance': 0.14409, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0041, 'shift': 0.0, 'Line_id': '369'},
        {'fromNode': '278.0', 'toNode': '310.0', 'Resistance': 0.01785, 'Reactance': 0.11602, 'Conductance': 0, 'Susceptance': 0.0572, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '370'},
        {'fromNode': '278.0', 'toNode': '349.0', 'Resistance': 0.0001, 'Reactance': 0.14409, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0041, 'shift': 0.0, 'Line_id': '371'},
        {'fromNode': '279.0', 'toNode': '342.0', 'Resistance': 0.06078, 'Reactance': 0.10635, 'Conductance': 0, 'Susceptance': 0.00176, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '372'},
        {'fromNode': '310.0', 'toNode': '280.0', 'Resistance': 0.1522, 'Reactance': 0.1534, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 13.8, 'm': 1.0, 'shift': 0.0, 'Line_id': '373'},
        {'fromNode': '283.0', 'toNode': '289.0', 'Resistance': 0.04273, 'Reactance': 0.07477, 'Conductance': 0, 'Susceptance': 0.00124, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '374'},
        {'fromNode': '296.0', 'toNode': '285.0', 'Resistance': 0.0, 'Reactance': 0.08528, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0191, 'shift': 0.0, 'Line_id': '375'},
        {'fromNode': '296.0', 'toNode': '285.0', 'Resistance': 0.0, 'Reactance': 0.08528, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0191, 'shift': 0.0, 'Line_id': '376'},
        {'fromNode': '286.0', 'toNode': '335.0', 'Resistance': 0.04834, 'Reactance': 0.08459, 'Conductance': 0, 'Susceptance': 0.0014, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '377'},
        {'fromNode': '287.0', 'toNode': '306.0', 'Resistance': 0.09687, 'Reactance': 0.16951, 'Conductance': 0, 'Susceptance': 0.0028, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '378'},
        {'fromNode': '287.0', 'toNode': '327.0', 'Resistance': 0.0897, 'Reactance': 0.1571, 'Conductance': 0, 'Susceptance': 0.0026, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '379'},
        {'fromNode': '288.0', 'toNode': '326.0', 'Resistance': 0.08798, 'Reactance': 0.15397, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '380'},
        {'fromNode': '288.0', 'toNode': '342.0', 'Resistance': 0.01917, 'Reactance': 0.03354, 'Conductance': 0, 'Susceptance': 0.00056, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '381'},
        {'fromNode': '289.0', 'toNode': '305.0', 'Resistance': 0.02122, 'Reactance': 0.03714, 'Conductance': 0, 'Susceptance': 0.00062, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '382'},
        {'fromNode': '290.0', 'toNode': '319.0', 'Resistance': 0.08574, 'Reactance': 0.15004, 'Conductance': 0, 'Susceptance': 0.0025, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '383'},
        {'fromNode': '291.0', 'toNode': '298.0', 'Resistance': 0.11678, 'Reactance': 0.20436, 'Conductance': 0, 'Susceptance': 0.00338, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '384'},
        {'fromNode': '291.0', 'toNode': '327.0', 'Resistance': 0.058, 'Reactance': 0.1014, 'Conductance': 0, 'Susceptance': 0.0017, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '385'},
        {'fromNode': '292.0', 'toNode': '334.0', 'Resistance': 0.0612, 'Reactance': 0.1072, 'Conductance': 0, 'Susceptance': 0.00178, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '386'},
        {'fromNode': '292.0', 'toNode': '335.0', 'Resistance': 0.0719, 'Reactance': 0.1258, 'Conductance': 0, 'Susceptance': 0.00208, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '387'},
        {'fromNode': '294.0', 'toNode': '293.0', 'Resistance': 0.0, 'Reactance': 0.01234, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 564.89, 'kV_base': 230.0, 'm': 1.0058, 'shift': 0.0, 'Line_id': '388'},
        {'fromNode': '294.0', 'toNode': '293.0', 'Resistance': 0.0, 'Reactance': 0.01234, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 564.89, 'kV_base': 230.0, 'm': 1.0058, 'shift': 0.0, 'Line_id': '389'},
        {'fromNode': '293.0', 'toNode': '375.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 224.1, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '390'},
        {'fromNode': '295.0', 'toNode': '321.0', 'Resistance': 0.08415, 'Reactance': 0.14726, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '391'},
        {'fromNode': '295.0', 'toNode': '332.0', 'Resistance': 0.0234, 'Reactance': 0.0593, 'Conductance': 0, 'Susceptance': 0.00112, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '392'},
        {'fromNode': '297.0', 'toNode': '337.0', 'Resistance': 0.0528, 'Reactance': 0.0924, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '393'},
        {'fromNode': '299.0', 'toNode': '318.0', 'Resistance': 0.0785, 'Reactance': 0.1374, 'Conductance': 0, 'Susceptance': 0.0028, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '394'},
        {'fromNode': '299.0', 'toNode': '321.0', 'Resistance': 0.14137, 'Reactance': 0.24739, 'Conductance': 0, 'Susceptance': 0.0041, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '395'},
        {'fromNode': '301.0', 'toNode': '306.0', 'Resistance': 0.0175, 'Reactance': 0.0368, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '396'},
        {'fromNode': '302.0', 'toNode': '311.0', 'Resistance': 0.00703, 'Reactance': 0.04557, 'Conductance': 0, 'Susceptance': 0.00693, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '397'},
        {'fromNode': '302.0', 'toNode': '336.0', 'Resistance': 0.0, 'Reactance': 0.0157, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0231, 'shift': 0.0, 'Line_id': '398'},
        {'fromNode': '304.0', 'toNode': '326.0', 'Resistance': 0.09874, 'Reactance': 0.17278, 'Conductance': 0, 'Susceptance': 0.00286, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '399'},
        {'fromNode': '307.0', 'toNode': '351.0', 'Resistance': 0.00944, 'Reactance': 0.01653, 'Conductance': 0, 'Susceptance': 0.00026, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '400'},
        {'fromNode': '309.0', 'toNode': '340.0', 'Resistance': 0.0, 'Reactance': 0.18626, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1.0075, 'shift': 0.0, 'Line_id': '401'},
        {'fromNode': '310.0', 'toNode': '315.0', 'Resistance': 0.1522, 'Reactance': 0.1534, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 13.8, 'm': 1.0, 'shift': 0.0, 'Line_id': '402'},
        {'fromNode': '310.0', 'toNode': '324.0', 'Resistance': 0.00019, 'Reactance': 0.00118, 'Conductance': 0, 'Susceptance': 0.00061, 'MVA_rating': 260.0, 'kV_base': 161.0, 'm': 1, 'shift': 0, 'Line_id': '403'},
        {'fromNode': '313.0', 'toNode': '330.0', 'Resistance': 0.09182, 'Reactance': 0.16067, 'Conductance': 0, 'Susceptance': 0.00266, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '404'},
        {'fromNode': '316.0', 'toNode': '323.0', 'Resistance': 0.04675, 'Reactance': 0.08181, 'Conductance': 0, 'Susceptance': 0.00136, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '405'},
        {'fromNode': '317.0', 'toNode': '328.0', 'Resistance': 0.00234, 'Reactance': 0.00409, 'Conductance': 0, 'Susceptance': 9e-05, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '406'},
        {'fromNode': '318.0', 'toNode': '329.0', 'Resistance': 0.07948, 'Reactance': 0.13908, 'Conductance': 0, 'Susceptance': 0.0023, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '407'},
        {'fromNode': '319.0', 'toNode': '329.0', 'Resistance': 0.12286, 'Reactance': 0.215, 'Conductance': 0, 'Susceptance': 0.00356, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '408'},
        {'fromNode': '321.0', 'toNode': '331.0', 'Resistance': 0.052, 'Reactance': 0.1837, 'Conductance': 0, 'Susceptance': 0.00368, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '409'},
        {'fromNode': '321.0', 'toNode': '343.0', 'Resistance': 0.01661, 'Reactance': 0.03417, 'Conductance': 0, 'Susceptance': 0.00064, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '410'},
        {'fromNode': '323.0', 'toNode': '341.0', 'Resistance': 0.01608, 'Reactance': 0.02814, 'Conductance': 0, 'Susceptance': 0.00046, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '411'},
        {'fromNode': '323.0', 'toNode': '348.0', 'Resistance': 0.08798, 'Reactance': 0.15397, 'Conductance': 0, 'Susceptance': 0.00256, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '412'},
        {'fromNode': '325.0', 'toNode': '339.0', 'Resistance': 0.01178, 'Reactance': 0.02062, 'Conductance': 0, 'Susceptance': 0.00034, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '413'},
        {'fromNode': '328.0', 'toNode': '345.0', 'Resistance': 0.07761, 'Reactance': 0.1358, 'Conductance': 0, 'Susceptance': 0.00224, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '414'},
        {'fromNode': '329.0', 'toNode': '350.0', 'Resistance': 0.1004, 'Reactance': 0.20654, 'Conductance': 0, 'Susceptance': 0.00378, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '415'},
        {'fromNode': '330.0', 'toNode': '346.0', 'Resistance': 0.0187, 'Reactance': 0.03272, 'Conductance': 0, 'Susceptance': 0.00054, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '416'},
        {'fromNode': '330.0', 'toNode': '351.0', 'Resistance': 0.07536, 'Reactance': 0.13188, 'Conductance': 0, 'Susceptance': 0.00218, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '417'},
        {'fromNode': '335.0', 'toNode': '338.0', 'Resistance': 0.06558, 'Reactance': 0.16654, 'Conductance': 0, 'Susceptance': 0.00318, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '418'},
        {'fromNode': '335.0', 'toNode': '347.0', 'Resistance': 0.02749, 'Reactance': 0.0481, 'Conductance': 0, 'Susceptance': 0.0008, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '419'},
        {'fromNode': '338.0', 'toNode': '350.0', 'Resistance': 0.02155, 'Reactance': 0.05472, 'Conductance': 0, 'Susceptance': 0.00104, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '420'},
        {'fromNode': '343.0', 'toNode': '350.0', 'Resistance': 0.10084, 'Reactance': 0.20745, 'Conductance': 0, 'Susceptance': 0.0038, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '421'},
        {'fromNode': '350.0', 'toNode': '351.0', 'Resistance': 0.06246, 'Reactance': 0.1093, 'Conductance': 0, 'Susceptance': 0.00182, 'MVA_rating': 260.0, 'kV_base': 69.0, 'm': 1, 'shift': 0, 'Line_id': '422'},
        {'fromNode': '352.0', 'toNode': '381.0', 'Resistance': 0.0, 'Reactance': 0.1926, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 115.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '423'},
        {'fromNode': '353.0', 'toNode': '367.0', 'Resistance': 0.00293, 'Reactance': 0.03245, 'Conductance': 0, 'Susceptance': 0.0678, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '424'},
        {'fromNode': '353.0', 'toNode': '372.0', 'Resistance': 0.0, 'Reactance': -0.038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 125.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '425'},
        {'fromNode': '353.0', 'toNode': '381.0', 'Resistance': 0.0, 'Reactance': -0.032, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '426'},
        {'fromNode': '353.0', 'toNode': '383.0', 'Resistance': 0.00198, 'Reactance': 0.02267, 'Conductance': 0, 'Susceptance': 0.0543, 'MVA_rating': 832.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '427'},
        {'fromNode': '353.0', 'toNode': '390.0', 'Resistance': 0.00094, 'Reactance': 0.00899, 'Conductance': 0, 'Susceptance': 0.03215, 'MVA_rating': 1226.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '428'},
        {'fromNode': '353.0', 'toNode': '400.0', 'Resistance': 0.0, 'Reactance': 0.057, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 210.0, 'kV_base': 18.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '429'},
        {'fromNode': '353.0', 'toNode': '402.0', 'Resistance': 0.0, 'Reactance': -0.033, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 190.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '430'},
        {'fromNode': '354.0', 'toNode': '390.0', 'Resistance': 0.0028, 'Reactance': 0.1393, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '431'},
        {'fromNode': '354.0', 'toNode': '404.0', 'Resistance': 0.011, 'Reactance': 0.2744, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '432'},
        {'fromNode': '355.0', 'toNode': '363.0', 'Resistance': 0.0, 'Reactance': 0.318, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '433'},
        {'fromNode': '378.0', 'toNode': '356.0', 'Resistance': 0.0, 'Reactance': 0.0989, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 120.0, 'kV_base': 14.4, 'm': 1.0, 'shift': 0.0, 'Line_id': '434'},
        {'fromNode': '357.0', 'toNode': '366.0', 'Resistance': 0.00064, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.0152, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '435'},
        {'fromNode': '357.0', 'toNode': '409.0', 'Resistance': 0.0001, 'Reactance': 0.0012, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '436'},
        {'fromNode': '358.0', 'toNode': '383.0', 'Resistance': 0.002, 'Reactance': 0.0221, 'Conductance': 0, 'Susceptance': 0.0461, 'MVA_rating': 671.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '437'},
        {'fromNode': '386.0', 'toNode': '358.0', 'Resistance': 0.0, 'Reactance': 0.1652, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 0.9857, 'shift': 0.0, 'Line_id': '438'},
        {'fromNode': '386.0', 'toNode': '358.0', 'Resistance': 0.0, 'Reactance': 0.1059, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 0.9857, 'shift': 0.0, 'Line_id': '439'},
        {'fromNode': '358.0', 'toNode': '406.0', 'Resistance': 0.0013, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 0.03001, 'MVA_rating': 671.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '440'},
        {'fromNode': '359.0', 'toNode': '370.0', 'Resistance': 0.0, 'Reactance': 0.318, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '441'},
        {'fromNode': '360.0', 'toNode': '375.0', 'Resistance': 0.00064, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.0152, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '442'},
        {'fromNode': '360.0', 'toNode': '408.0', 'Resistance': 0.0001, 'Reactance': 0.0012, 'Conductance': 0, 'Susceptance': 0.00244, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '443'},
        {'fromNode': '368.0', 'toNode': '361.0', 'Resistance': 0.0, 'Reactance': 0.27654, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '444'},
        {'fromNode': '368.0', 'toNode': '361.0', 'Resistance': 0.0, 'Reactance': 0.41078, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '445'},
        {'fromNode': '376.0', 'toNode': '362.0', 'Resistance': 0.0, 'Reactance': 0.1059, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 0.9857, 'shift': 0.0, 'Line_id': '446'},
        {'fromNode': '376.0', 'toNode': '362.0', 'Resistance': 0.0, 'Reactance': 0.1059, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 230.0, 'm': 0.9857, 'shift': 0.0, 'Line_id': '447'},
        {'fromNode': '362.0', 'toNode': '377.0', 'Resistance': 0.00285, 'Reactance': 0.03463, 'Conductance': 0, 'Susceptance': 0.13764, 'MVA_rating': 738.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '448'},
        {'fromNode': '398.0', 'toNode': '363.0', 'Resistance': 0.0, 'Reactance': 0.316, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '449'},
        {'fromNode': '411.0', 'toNode': '363.0', 'Resistance': 0.0, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 0.9971, 'shift': 0.0, 'Line_id': '450'},
        {'fromNode': '364.0', 'toNode': '402.0', 'Resistance': 0.0, 'Reactance': 0.1935, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 115.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '451'},
        {'fromNode': '391.0', 'toNode': '365.0', 'Resistance': 0.00296, 'Reactance': 0.1677, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0, 'shift': 0.0, 'Line_id': '452'},
        {'fromNode': '397.0', 'toNode': '365.0', 'Resistance': 0.00296, 'Reactance': 0.1677, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0, 'shift': 0.0, 'Line_id': '453'},
        {'fromNode': '367.0', 'toNode': '413.0', 'Resistance': 0.00373, 'Reactance': 0.04092, 'Conductance': 0, 'Susceptance': 0.08703, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '454'},
        {'fromNode': '368.0', 'toNode': '384.0', 'Resistance': 0.0013, 'Reactance': 0.0073, 'Conductance': 0, 'Susceptance': 0.00101, 'MVA_rating': 239.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '455'},
        {'fromNode': '369.0', 'toNode': '372.0', 'Resistance': 0.0, 'Reactance': 0.2525, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 63.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '456'},
        {'fromNode': '371.0', 'toNode': '370.0', 'Resistance': 0.0, 'Reactance': 0.316, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '457'},
        {'fromNode': '411.0', 'toNode': '370.0', 'Resistance': 0.0, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 1.0, 'm': 0.9971, 'shift': 0.0, 'Line_id': '458'},
        {'fromNode': '388.0', 'toNode': '372.0', 'Resistance': 0.0, 'Reactance': 0.2495, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 63.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '459'},
        {'fromNode': '373.0', 'toNode': '387.0', 'Resistance': 0.00079, 'Reactance': 0.00902, 'Conductance': 0, 'Susceptance': 0.01865, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '460'},
        {'fromNode': '373.0', 'toNode': '389.0', 'Resistance': 0.0, 'Reactance': 0.1512, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0, 'shift': 0.0, 'Line_id': '461'},
        {'fromNode': '373.0', 'toNode': '395.0', 'Resistance': 0.00225, 'Reactance': 0.02555, 'Conductance': 0, 'Susceptance': 0.05269, 'MVA_rating': 671.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '462'},
        {'fromNode': '374.0', 'toNode': '377.0', 'Resistance': 0.00045, 'Reactance': 0.00537, 'Conductance': 0, 'Susceptance': 0.02136, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '463'},
        {'fromNode': '374.0', 'toNode': '395.0', 'Resistance': 0.0025, 'Reactance': 0.0288, 'Conductance': 0, 'Susceptance': 0.0583, 'MVA_rating': 750.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '464'},
        {'fromNode': '378.0', 'toNode': '382.0', 'Resistance': 0.0, 'Reactance': 0.0986, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 140.0, 'kV_base': 14.4, 'm': 1.0, 'shift': 0.0, 'Line_id': '465'},
        {'fromNode': '378.0', 'toNode': '395.0', 'Resistance': 0.0001, 'Reactance': 0.001, 'Conductance': 0, 'Susceptance': 0.00024, 'MVA_rating': 648.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '466'},
        {'fromNode': '378.0', 'toNode': '401.0', 'Resistance': 0.0, 'Reactance': 0.0995, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 140.0, 'kV_base': 14.4, 'm': 1.0, 'shift': 0.0, 'Line_id': '467'},
        {'fromNode': '406.0', 'toNode': '379.0', 'Resistance': 0.00305, 'Reactance': 0.17267, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.0, 'm': 0.9855, 'shift': 0.0, 'Line_id': '468'},
        {'fromNode': '380.0', 'toNode': '394.0', 'Resistance': 0.0067, 'Reactance': 0.0742, 'Conductance': 0, 'Susceptance': 0.0097, 'MVA_rating': 239.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '469'},
        {'fromNode': '380.0', 'toNode': '417.0', 'Resistance': 0.0015, 'Reactance': 0.0172, 'Conductance': 0, 'Susceptance': 0.0022, 'MVA_rating': 167.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '470'},
        {'fromNode': '407.0', 'toNode': '381.0', 'Resistance': 0.0, 'Reactance': 0.2782, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 75.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '471'},
        {'fromNode': '383.0', 'toNode': '411.0', 'Resistance': 0.00532, 'Reactance': 0.13802, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0147, 'shift': 0.0, 'Line_id': '472'},
        {'fromNode': '383.0', 'toNode': '411.0', 'Resistance': 0.0, 'Reactance': 0.13432, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 0.9893, 'shift': 0.0, 'Line_id': '473'},
        {'fromNode': '383.0', 'toNode': '411.0', 'Resistance': 0.0, 'Reactance': 0.13461, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 0.9893, 'shift': 0.0, 'Line_id': '474'},
        {'fromNode': '384.0', 'toNode': '403.0', 'Resistance': 0.0087, 'Reactance': 0.0354, 'Conductance': 0, 'Susceptance': 0.0047, 'MVA_rating': 192.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '475'},
        {'fromNode': '384.0', 'toNode': '415.0', 'Resistance': 0.01861, 'Reactance': 0.07926, 'Conductance': 0, 'Susceptance': 0.0106, 'MVA_rating': 211.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '476'},
        {'fromNode': '385.0', 'toNode': '404.0', 'Resistance': 0.0148, 'Reactance': 0.0608, 'Conductance': 0, 'Susceptance': 0.0081, 'MVA_rating': 141.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '477'},
        {'fromNode': '387.0', 'toNode': '405.0', 'Resistance': 0.00079, 'Reactance': 0.00902, 'Conductance': 0, 'Susceptance': 0.01865, 'MVA_rating': 738.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '478'},
        {'fromNode': '391.0', 'toNode': '394.0', 'Resistance': 0.0073, 'Reactance': 0.07877, 'Conductance': 0, 'Susceptance': 0.01026, 'MVA_rating': 146.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '479'},
        {'fromNode': '391.0', 'toNode': '418.0', 'Resistance': 0.0335, 'Reactance': 0.0509, 'Conductance': 0, 'Susceptance': 0.0055, 'MVA_rating': 92.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '480'},
        {'fromNode': '392.0', 'toNode': '413.0', 'Resistance': 0.00044, 'Reactance': 0.0047, 'Conductance': 0, 'Susceptance': 0.01059, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '481'},
        {'fromNode': '392.0', 'toNode': '416.0', 'Resistance': 0.0004, 'Reactance': 0.0046, 'Conductance': 0, 'Susceptance': 0.0094, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '482'},
        {'fromNode': '393.0', 'toNode': '406.0', 'Resistance': 0.001, 'Reactance': 0.0118, 'Conductance': 0, 'Susceptance': 0.02431, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '483'},
        {'fromNode': '393.0', 'toNode': '416.0', 'Resistance': 0.001, 'Reactance': 0.0112, 'Conductance': 0, 'Susceptance': 0.0231, 'MVA_rating': 738.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '484'},
        {'fromNode': '394.0', 'toNode': '396.0', 'Resistance': 0.00875, 'Reactance': 0.03684, 'Conductance': 0, 'Susceptance': 0.00482, 'MVA_rating': 182.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '485'},
        {'fromNode': '394.0', 'toNode': '397.0', 'Resistance': 0.0072, 'Reactance': 0.0779, 'Conductance': 0, 'Susceptance': 0.0102, 'MVA_rating': 146.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '486'},
        {'fromNode': '406.0', 'toNode': '394.0', 'Resistance': 0.001, 'Reactance': 0.05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 225.0, 'kV_base': 115.0, 'm': 0.9812, 'shift': 0.0, 'Line_id': '487'},
        {'fromNode': '395.0', 'toNode': '410.0', 'Resistance': 0.00588, 'Reactance': 0.17061, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.0, 'm': 0.9855, 'shift': 0.0, 'Line_id': '488'},
        {'fromNode': '396.0', 'toNode': '412.0', 'Resistance': 0.0, 'Reactance': 0.358, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0, 'shift': 0.0, 'Line_id': '489'},
        {'fromNode': '396.0', 'toNode': '412.0', 'Resistance': 0.0, 'Reactance': 0.273, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 260.0, 'kV_base': 34.5, 'm': 1.0, 'shift': 0.0, 'Line_id': '490'},
        {'fromNode': '396.0', 'toNode': '415.0', 'Resistance': 0.00632, 'Reactance': 0.02601, 'Conductance': 0, 'Susceptance': 0.0034, 'MVA_rating': 211.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '491'},
        {'fromNode': '397.0', 'toNode': '399.0', 'Resistance': 0.03308, 'Reactance': 0.05098, 'Conductance': 0, 'Susceptance': 0.0056, 'MVA_rating': 92.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '492'},
        {'fromNode': '407.0', 'toNode': '402.0', 'Resistance': 0.0, 'Reactance': 0.2802, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 75.0, 'kV_base': 1.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '493'},
        {'fromNode': '403.0', 'toNode': '404.0', 'Resistance': 0.0065, 'Reactance': 0.0284, 'Conductance': 0, 'Susceptance': 0.0037, 'MVA_rating': 192.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '494'},
        {'fromNode': '405.0', 'toNode': '406.0', 'Resistance': 0.00079, 'Reactance': 0.00902, 'Conductance': 0, 'Susceptance': 0.01865, 'MVA_rating': 738.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '495'},
        {'fromNode': '408.0', 'toNode': '416.0', 'Resistance': 0.0012, 'Reactance': 0.0135, 'Conductance': 0, 'Susceptance': 0.02789, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '496'},
        {'fromNode': '409.0', 'toNode': '416.0', 'Resistance': 0.0012, 'Reactance': 0.0135, 'Conductance': 0, 'Susceptance': 0.02789, 'MVA_rating': 703.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '497'},
        {'fromNode': '414.0', 'toNode': '416.0', 'Resistance': 0.001, 'Reactance': 0.0112, 'Conductance': 0, 'Susceptance': 0.0231, 'MVA_rating': 540.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '498'},
        {'fromNode': '417.0', 'toNode': '416.0', 'Resistance': 0.0, 'Reactance': 0.0465, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 239.0, 'kV_base': 230.0, 'm': 0.9957, 'shift': 0.0, 'Line_id': '499'},
        {'fromNode': '419.0', 'toNode': '420.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '500'},
        {'fromNode': '419.0', 'toNode': '423.0', 'Resistance': 0.00171, 'Reactance': 0.00961, 'Conductance': 0, 'Susceptance': 0.01424, 'MVA_rating': 250.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '501'},
        {'fromNode': '458.0', 'toNode': '419.0', 'Resistance': 0.00141, 'Reactance': 0.06975, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.02312, 'shift': 0.0, 'Line_id': '502'},
        {'fromNode': '421.0', 'toNode': '422.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '503'},
        {'fromNode': '421.0', 'toNode': '424.0', 'Resistance': 0.00194, 'Reactance': 0.01415, 'Conductance': 0, 'Susceptance': 0.0223, 'MVA_rating': 426.3, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '504'},
        {'fromNode': '468.0', 'toNode': '422.0', 'Resistance': 0.00143, 'Reactance': 0.06701, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.02256, 'shift': 0.0, 'Line_id': '505'},
        {'fromNode': '422.0', 'toNode': '558.0', 'Resistance': 0.0001, 'Reactance': 0.01, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 455.96, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '506'},
        {'fromNode': '423.0', 'toNode': '424.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '507'},
        {'fromNode': '423.0', 'toNode': '426.0', 'Resistance': 0.00537, 'Reactance': 0.04039, 'Conductance': 0, 'Susceptance': 0.06005, 'MVA_rating': 319.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '508'},
        {'fromNode': '475.0', 'toNode': '423.0', 'Resistance': 0.00174, 'Reactance': 0.09356, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 0.98025, 'shift': 0.0, 'Line_id': '509'},
        {'fromNode': '476.0', 'toNode': '424.0', 'Resistance': 0.00167, 'Reactance': 0.09275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 0.99807, 'shift': 0.0, 'Line_id': '510'},
        {'fromNode': '425.0', 'toNode': '426.0', 'Resistance': 0.00066, 'Reactance': 0.00496, 'Conductance': 0, 'Susceptance': 0.00758, 'MVA_rating': 319.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '511'},
        {'fromNode': '427.0', 'toNode': '476.0', 'Resistance': 0.01248, 'Reactance': 0.04595, 'Conductance': 0, 'Susceptance': 0.00524, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '512'},
        {'fromNode': '428.0', 'toNode': '429.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '513'},
        {'fromNode': '428.0', 'toNode': '494.0', 'Resistance': 0.00364, 'Reactance': 0.01149, 'Conductance': 0, 'Susceptance': 0.00123, 'MVA_rating': 175.28, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '514'},
        {'fromNode': '429.0', 'toNode': '504.0', 'Resistance': 0.00653, 'Reactance': 0.02058, 'Conductance': 0, 'Susceptance': 0.00222, 'MVA_rating': 170.42, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '515'},
        {'fromNode': '430.0', 'toNode': '431.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '516'},
        {'fromNode': '430.0', 'toNode': '464.0', 'Resistance': 0.00488, 'Reactance': 0.01231, 'Conductance': 0, 'Susceptance': 0.00189, 'MVA_rating': 74.62, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '517'},
        {'fromNode': '431.0', 'toNode': '527.0', 'Resistance': 0.00074, 'Reactance': 0.00223, 'Conductance': 0, 'Susceptance': 0.0003, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '518'},
        {'fromNode': '432.0', 'toNode': '494.0', 'Resistance': 0.00157, 'Reactance': 0.00463, 'Conductance': 0, 'Susceptance': 0.00057, 'MVA_rating': 178.28, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '519'},
        {'fromNode': '433.0', 'toNode': '468.0', 'Resistance': 0.00041, 'Reactance': 0.00124, 'Conductance': 0, 'Susceptance': 0.00013, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '520'},
        {'fromNode': '434.0', 'toNode': '435.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '521'},
        {'fromNode': '434.0', 'toNode': '464.0', 'Resistance': 0.00455, 'Reactance': 0.01289, 'Conductance': 0, 'Susceptance': 0.00174, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '522'},
        {'fromNode': '435.0', 'toNode': '531.0', 'Resistance': 0.00603, 'Reactance': 0.01711, 'Conductance': 0, 'Susceptance': 0.00231, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '523'},
        {'fromNode': '436.0', 'toNode': '437.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '524'},
        {'fromNode': '436.0', 'toNode': '438.0', 'Resistance': 0.0005, 'Reactance': 0.00149, 'Conductance': 0, 'Susceptance': 0.0096, 'MVA_rating': 98.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '525'},
        {'fromNode': '438.0', 'toNode': '455.0', 'Resistance': 8e-05, 'Reactance': 0.00066, 'Conductance': 0, 'Susceptance': 7e-05, 'MVA_rating': 98.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '526'},
        {'fromNode': '438.0', 'toNode': '514.0', 'Resistance': 0.00207, 'Reactance': 0.01413, 'Conductance': 0, 'Susceptance': 0.00162, 'MVA_rating': 98.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '527'},
        {'fromNode': '439.0', 'toNode': '450.0', 'Resistance': 0.00339, 'Reactance': 0.01116, 'Conductance': 0, 'Susceptance': 0.00113, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '528'},
        {'fromNode': '439.0', 'toNode': '474.0', 'Resistance': 0.00132, 'Reactance': 0.00231, 'Conductance': 0, 'Susceptance': 0.00023, 'MVA_rating': 47.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '529'},
        {'fromNode': '439.0', 'toNode': '499.0', 'Resistance': 0.00636, 'Reactance': 0.0205, 'Conductance': 0, 'Susceptance': 0.00213, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '530'},
        {'fromNode': '439.0', 'toNode': '502.0', 'Resistance': 0.00264, 'Reactance': 0.00818, 'Conductance': 0, 'Susceptance': 0.00094, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '531'},
        {'fromNode': '439.0', 'toNode': '516.0', 'Resistance': 0.0081, 'Reactance': 0.02298, 'Conductance': 0, 'Susceptance': 0.00239, 'MVA_rating': 63.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '532'},
        {'fromNode': '440.0', 'toNode': '441.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '533'},
        {'fromNode': '440.0', 'toNode': '456.0', 'Resistance': 0.00174, 'Reactance': 0.00496, 'Conductance': 0, 'Susceptance': 0.00065, 'MVA_rating': 57.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '534'},
        {'fromNode': '440.0', 'toNode': '477.0', 'Resistance': 0.00455, 'Reactance': 0.01405, 'Conductance': 0, 'Susceptance': 0.0016, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '535'},
        {'fromNode': '440.0', 'toNode': '510.0', 'Resistance': 0.00504, 'Reactance': 0.01562, 'Conductance': 0, 'Susceptance': 0.00173, 'MVA_rating': 108.47, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '536'},
        {'fromNode': '440.0', 'toNode': '519.0', 'Resistance': 0.00711, 'Reactance': 0.02322, 'Conductance': 0, 'Susceptance': 0.00234, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '537'},
        {'fromNode': '441.0', 'toNode': '469.0', 'Resistance': 0.00496, 'Reactance': 0.01628, 'Conductance': 0, 'Susceptance': 0.00164, 'MVA_rating': 142.49, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '538'},
        {'fromNode': '441.0', 'toNode': '492.0', 'Resistance': 0.00496, 'Reactance': 0.01496, 'Conductance': 0, 'Susceptance': 0.00179, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '539'},
        {'fromNode': '441.0', 'toNode': '511.0', 'Resistance': 0.00504, 'Reactance': 0.01562, 'Conductance': 0, 'Susceptance': 0.00173, 'MVA_rating': 131.91, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '540'},
        {'fromNode': '442.0', 'toNode': '443.0', 'Resistance': 0.00124, 'Reactance': 0.00421, 'Conductance': 0, 'Susceptance': 0.00059, 'MVA_rating': 70.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '541'},
        {'fromNode': '443.0', 'toNode': '446.0', 'Resistance': 0.00413, 'Reactance': 0.01587, 'Conductance': 0, 'Susceptance': 0.00183, 'MVA_rating': 70.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '542'},
        {'fromNode': '443.0', 'toNode': '476.0', 'Resistance': 0.00488, 'Reactance': 0.01678, 'Conductance': 0, 'Susceptance': 0.00193, 'MVA_rating': 78.62, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '543'},
        {'fromNode': '444.0', 'toNode': '468.0', 'Resistance': 0.00355, 'Reactance': 0.01149, 'Conductance': 0, 'Susceptance': 0.00116, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '544'},
        {'fromNode': '444.0', 'toNode': '523.0', 'Resistance': 0.00182, 'Reactance': 0.00587, 'Conductance': 0, 'Susceptance': 0.00587, 'MVA_rating': 114.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '545'},
        {'fromNode': '445.0', 'toNode': '446.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '546'},
        {'fromNode': '445.0', 'toNode': '448.0', 'Resistance': 0.00066, 'Reactance': 0.00207, 'Conductance': 0, 'Susceptance': 0.00021, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '547'},
        {'fromNode': '445.0', 'toNode': '460.0', 'Resistance': 0.00066, 'Reactance': 0.0024, 'Conductance': 0, 'Susceptance': 0.00031, 'MVA_rating': 80.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '548'},
        {'fromNode': '445.0', 'toNode': '468.0', 'Resistance': 0.00314, 'Reactance': 0.01355, 'Conductance': 0, 'Susceptance': 0.00135, 'MVA_rating': 80.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '549'},
        {'fromNode': '445.0', 'toNode': '486.0', 'Resistance': 0.00223, 'Reactance': 0.00826, 'Conductance': 0, 'Susceptance': 0.00099, 'MVA_rating': 70.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '550'},
        {'fromNode': '446.0', 'toNode': '447.0', 'Resistance': 0.00066, 'Reactance': 0.00215, 'Conductance': 0, 'Susceptance': 0.00022, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '551'},
        {'fromNode': '446.0', 'toNode': '468.0', 'Resistance': 0.00314, 'Reactance': 0.01355, 'Conductance': 0, 'Susceptance': 0.00135, 'MVA_rating': 80.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '552'},
        {'fromNode': '446.0', 'toNode': '521.0', 'Resistance': 0.00091, 'Reactance': 0.00273, 'Conductance': 0, 'Susceptance': 0.00033, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '553'},
        {'fromNode': '449.0', 'toNode': '468.0', 'Resistance': 0.0014, 'Reactance': 0.00455, 'Conductance': 0, 'Susceptance': 0.00046, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '554'},
        {'fromNode': '451.0', 'toNode': '452.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '555'},
        {'fromNode': '451.0', 'toNode': '464.0', 'Resistance': 0.00405, 'Reactance': 0.01331, 'Conductance': 0, 'Susceptance': 0.00135, 'MVA_rating': 77.01, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '556'},
        {'fromNode': '452.0', 'toNode': '506.0', 'Resistance': 0.01025, 'Reactance': 0.02926, 'Conductance': 0, 'Susceptance': 0.0039, 'MVA_rating': 77.98, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '557'},
        {'fromNode': '453.0', 'toNode': '454.0', 'Resistance': 0.00231, 'Reactance': 0.00273, 'Conductance': 0, 'Susceptance': 0.00058, 'MVA_rating': 39.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '558'},
        {'fromNode': '454.0', 'toNode': '475.0', 'Resistance': 0.00579, 'Reactance': 0.01785, 'Conductance': 0, 'Susceptance': 0.00204, 'MVA_rating': 105.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '559'},
        {'fromNode': '454.0', 'toNode': '498.0', 'Resistance': 0.0043, 'Reactance': 0.01347, 'Conductance': 0, 'Susceptance': 0.00147, 'MVA_rating': 105.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '560'},
        {'fromNode': '457.0', 'toNode': '491.0', 'Resistance': 0.01198, 'Reactance': 0.03702, 'Conductance': 0, 'Susceptance': 0.00418, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '561'},
        {'fromNode': '459.0', 'toNode': '468.0', 'Resistance': 0.00372, 'Reactance': 0.01207, 'Conductance': 0, 'Susceptance': 0.00122, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '562'},
        {'fromNode': '461.0', 'toNode': '498.0', 'Resistance': 0.00686, 'Reactance': 0.02215, 'Conductance': 0, 'Susceptance': 0.00232, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '563'},
        {'fromNode': '462.0', 'toNode': '463.0', 'Resistance': 0.00033, 'Reactance': 0.00041, 'Conductance': 0, 'Susceptance': 9e-05, 'MVA_rating': 39.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '564'},
        {'fromNode': '463.0', 'toNode': '475.0', 'Resistance': 0.0076, 'Reactance': 0.02355, 'Conductance': 0, 'Susceptance': 0.00263, 'MVA_rating': 163.37, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '565'},
        {'fromNode': '463.0', 'toNode': '503.0', 'Resistance': 0.00273, 'Reactance': 0.00876, 'Conductance': 0, 'Susceptance': 0.0009, 'MVA_rating': 163.87, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '566'},
        {'fromNode': '464.0', 'toNode': '514.0', 'Resistance': 0.01694, 'Reactance': 0.05562, 'Conductance': 0, 'Susceptance': 0.0056, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '567'},
        {'fromNode': '465.0', 'toNode': '475.0', 'Resistance': 0.03413, 'Reactance': 0.08545, 'Conductance': 0, 'Susceptance': 0.00909, 'MVA_rating': 74.04, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '568'},
        {'fromNode': '465.0', 'toNode': '482.0', 'Resistance': 0.00992, 'Reactance': 0.01661, 'Conductance': 0, 'Susceptance': 0.00395, 'MVA_rating': 88.37, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '569'},
        {'fromNode': '465.0', 'toNode': '488.0', 'Resistance': 0.02496, 'Reactance': 0.03802, 'Conductance': 0, 'Susceptance': 0.00491, 'MVA_rating': 100.27, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '570'},
        {'fromNode': '465.0', 'toNode': '529.0', 'Resistance': 0.0062, 'Reactance': 0.01562, 'Conductance': 0, 'Susceptance': 0.00277, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '571'},
        {'fromNode': '466.0', 'toNode': '467.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '572'},
        {'fromNode': '466.0', 'toNode': '535.0', 'Resistance': 0.00463, 'Reactance': 0.01446, 'Conductance': 0, 'Susceptance': 0.00163, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '573'},
        {'fromNode': '468.0', 'toNode': '516.0', 'Resistance': 0.0014, 'Reactance': 0.00471, 'Conductance': 0, 'Susceptance': 0.00048, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '574'},
        {'fromNode': '468.0', 'toNode': '517.0', 'Resistance': 0.00331, 'Reactance': 0.01066, 'Conductance': 0, 'Susceptance': 0.0011, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '575'},
        {'fromNode': '470.0', 'toNode': '471.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '576'},
        {'fromNode': '470.0', 'toNode': '479.0', 'Resistance': 0.0105, 'Reactance': 0.02983, 'Conductance': 0, 'Susceptance': 0.00402, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '577'},
        {'fromNode': '471.0', 'toNode': '532.0', 'Resistance': 0.01909, 'Reactance': 0.05413, 'Conductance': 0, 'Susceptance': 0.00729, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '578'},
        {'fromNode': '472.0', 'toNode': '473.0', 'Resistance': 0.00438, 'Reactance': 0.0086, 'Conductance': 0, 'Susceptance': 0.00086, 'MVA_rating': 47.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '579'},
        {'fromNode': '473.0', 'toNode': '500.0', 'Resistance': 0.00628, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.00213, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '580'},
        {'fromNode': '475.0', 'toNode': '513.0', 'Resistance': 0.00579, 'Reactance': 0.01446, 'Conductance': 0, 'Susceptance': 0.00162, 'MVA_rating': 65.52, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '581'},
        {'fromNode': '476.0', 'toNode': '485.0', 'Resistance': 0.00769, 'Reactance': 0.02711, 'Conductance': 0, 'Susceptance': 0.00317, 'MVA_rating': 85.94, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '582'},
        {'fromNode': '476.0', 'toNode': '510.0', 'Resistance': 0.01025, 'Reactance': 0.03198, 'Conductance': 0, 'Susceptance': 0.00354, 'MVA_rating': 112.8, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '583'},
        {'fromNode': '476.0', 'toNode': '511.0', 'Resistance': 0.01025, 'Reactance': 0.03198, 'Conductance': 0, 'Susceptance': 0.00354, 'MVA_rating': 138.36, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '584'},
        {'fromNode': '477.0', 'toNode': '478.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '585'},
        {'fromNode': '478.0', 'toNode': '535.0', 'Resistance': 0.00091, 'Reactance': 0.00256, 'Conductance': 0, 'Susceptance': 0.00033, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '586'},
        {'fromNode': '479.0', 'toNode': '480.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '587'},
        {'fromNode': '479.0', 'toNode': '489.0', 'Resistance': 0.00628, 'Reactance': 0.01959, 'Conductance': 0, 'Susceptance': 0.00217, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '588'},
        {'fromNode': '479.0', 'toNode': '525.0', 'Resistance': 0.00083, 'Reactance': 0.00248, 'Conductance': 0, 'Susceptance': 0.00027, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '589'},
        {'fromNode': '480.0', 'toNode': '490.0', 'Resistance': 0.0062, 'Reactance': 0.01942, 'Conductance': 0, 'Susceptance': 0.00215, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '590'},
        {'fromNode': '480.0', 'toNode': '496.0', 'Resistance': 0.01587, 'Reactance': 0.05182, 'Conductance': 0, 'Susceptance': 0.00527, 'MVA_rating': 80.94, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '591'},
        {'fromNode': '480.0', 'toNode': '528.0', 'Resistance': 0.00124, 'Reactance': 0.00355, 'Conductance': 0, 'Susceptance': 0.00045, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '592'},
        {'fromNode': '480.0', 'toNode': '530.0', 'Resistance': 0.00198, 'Reactance': 0.00529, 'Conductance': 0, 'Susceptance': 0.00071, 'MVA_rating': 50.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '593'},
        {'fromNode': '481.0', 'toNode': '482.0', 'Resistance': 0.00223, 'Reactance': 0.00562, 'Conductance': 0, 'Susceptance': 0.0006, 'MVA_rating': 50.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '594'},
        {'fromNode': '482.0', 'toNode': '514.0', 'Resistance': 0.01471, 'Reactance': 0.03025, 'Conductance': 0, 'Susceptance': 0.00563, 'MVA_rating': 92.04, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '595'},
        {'fromNode': '483.0', 'toNode': '484.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '596'},
        {'fromNode': '484.0', 'toNode': '495.0', 'Resistance': 0.01488, 'Reactance': 0.02661, 'Conductance': 0, 'Susceptance': 0.00302, 'MVA_rating': 170.72, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '597'},
        {'fromNode': '485.0', 'toNode': '486.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '598'},
        {'fromNode': '487.0', 'toNode': '488.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '599'},
        {'fromNode': '487.0', 'toNode': '495.0', 'Resistance': 0.00215, 'Reactance': 0.00603, 'Conductance': 0, 'Susceptance': 0.00063, 'MVA_rating': 98.36, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '600'},
        {'fromNode': '491.0', 'toNode': '492.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '601'},
        {'fromNode': '493.0', 'toNode': '494.0', 'Resistance': 0.00231, 'Reactance': 0.00694, 'Conductance': 0, 'Susceptance': 0.00072, 'MVA_rating': 63.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '602'},
        {'fromNode': '495.0', 'toNode': '496.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '603'},
        {'fromNode': '497.0', 'toNode': '498.0', 'Resistance': 0.00603, 'Reactance': 0.01744, 'Conductance': 0, 'Susceptance': 0.0023, 'MVA_rating': 105.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '604'},
        {'fromNode': '499.0', 'toNode': '500.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '605'},
        {'fromNode': '501.0', 'toNode': '502.0', 'Resistance': 0.00124, 'Reactance': 0.00364, 'Conductance': 0, 'Susceptance': 0.00047, 'MVA_rating': 57.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '606'},
        {'fromNode': '502.0', 'toNode': '523.0', 'Resistance': 0.0038, 'Reactance': 0.01215, 'Conductance': 0, 'Susceptance': 0.00125, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '607'},
        {'fromNode': '503.0', 'toNode': '504.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '608'},
        {'fromNode': '505.0', 'toNode': '506.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '609'},
        {'fromNode': '505.0', 'toNode': '507.0', 'Resistance': 0.01248, 'Reactance': 0.03545, 'Conductance': 0, 'Susceptance': 0.00474, 'MVA_rating': 86.66, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '610'},
        {'fromNode': '507.0', 'toNode': '512.0', 'Resistance': 0.00769, 'Reactance': 0.01934, 'Conductance': 0, 'Susceptance': 0.00211, 'MVA_rating': 64.98, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '611'},
        {'fromNode': '508.0', 'toNode': '510.0', 'Resistance': 0.00521, 'Reactance': 0.00884, 'Conductance': 0, 'Susceptance': 0.00089, 'MVA_rating': 47.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '612'},
        {'fromNode': '509.0', 'toNode': '511.0', 'Resistance': 0.00521, 'Reactance': 0.00884, 'Conductance': 0, 'Susceptance': 0.00089, 'MVA_rating': 47.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '613'},
        {'fromNode': '512.0', 'toNode': '513.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '614'},
        {'fromNode': '514.0', 'toNode': '520.0', 'Resistance': 0.00372, 'Reactance': 0.01231, 'Conductance': 0, 'Susceptance': 0.00124, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '615'},
        {'fromNode': '515.0', 'toNode': '516.0', 'Resistance': 0.0005, 'Reactance': 0.0014, 'Conductance': 0, 'Susceptance': 0.00017, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '616'},
        {'fromNode': '518.0', 'toNode': '519.0', 'Resistance': 0.00074, 'Reactance': 0.00231, 'Conductance': 0, 'Susceptance': 0.00026, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '617'},
        {'fromNode': '522.0', 'toNode': '523.0', 'Resistance': 8e-05, 'Reactance': 0.00017, 'Conductance': 0, 'Susceptance': 2e-05, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '618'},
        {'fromNode': '524.0', 'toNode': '525.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '619'},
        {'fromNode': '524.0', 'toNode': '526.0', 'Resistance': 0.00149, 'Reactance': 0.00446, 'Conductance': 0, 'Susceptance': 0.00057, 'MVA_rating': 62.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '620'},
        {'fromNode': '526.0', 'toNode': '527.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '621'},
        {'fromNode': '528.0', 'toNode': '529.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '622'},
        {'fromNode': '530.0', 'toNode': '531.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '623'},
        {'fromNode': '533.0', 'toNode': '534.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '624'},
        {'fromNode': '533.0', 'toNode': '535.0', 'Resistance': 0.0, 'Reactance': 8e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 75.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '625'},
        {'fromNode': '536.0', 'toNode': '545.0', 'Resistance': 0.0012, 'Reactance': 0.0089, 'Conductance': 0, 'Susceptance': 0.01512, 'MVA_rating': 461.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '626'},
        {'fromNode': '579.0', 'toNode': '536.0', 'Resistance': 0.00064, 'Reactance': 0.04304, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 250.0, 'kV_base': 345.0, 'm': 1.00985, 'shift': 0.0, 'Line_id': '627'},
        {'fromNode': '537.0', 'toNode': '546.0', 'Resistance': 0.0012, 'Reactance': 0.0089, 'Conductance': 0, 'Susceptance': 0.01513, 'MVA_rating': 461.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '628'},
        {'fromNode': '538.0', 'toNode': '539.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '629'},
        {'fromNode': '538.0', 'toNode': '540.0', 'Resistance': 0.00287, 'Reactance': 0.01671, 'Conductance': 0, 'Susceptance': 0.02467, 'MVA_rating': 295.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '630'},
        {'fromNode': '538.0', 'toNode': '543.0', 'Resistance': 0.00457, 'Reactance': 0.03351, 'Conductance': 0, 'Susceptance': 0.05245, 'MVA_rating': 317.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '631'},
        {'fromNode': '539.0', 'toNode': '545.0', 'Resistance': 0.00924, 'Reactance': 0.05052, 'Conductance': 0, 'Susceptance': 0.07698, 'MVA_rating': 367.86, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '632'},
        {'fromNode': '539.0', 'toNode': '550.0', 'Resistance': 0.00494, 'Reactance': 0.0269, 'Conductance': 0, 'Susceptance': 0.04225, 'MVA_rating': 250.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '633'},
        {'fromNode': '573.0', 'toNode': '539.0', 'Resistance': 0.00165, 'Reactance': 0.06901, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 205.87, 'kV_base': 345.0, 'm': 1.03303, 'shift': 0.0, 'Line_id': '634'},
        {'fromNode': '540.0', 'toNode': '541.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '635'},
        {'fromNode': '540.0', 'toNode': '546.0', 'Resistance': 0.00585, 'Reactance': 0.04027, 'Conductance': 0, 'Susceptance': 0.07161, 'MVA_rating': 463.05, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '636'},
        {'fromNode': '541.0', 'toNode': '545.0', 'Resistance': 0.00585, 'Reactance': 0.04027, 'Conductance': 0, 'Susceptance': 0.07161, 'MVA_rating': 400.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '637'},
        {'fromNode': '541.0', 'toNode': '560.0', 'Resistance': 0.01068, 'Reactance': 0.0475, 'Conductance': 0, 'Susceptance': 0.09495, 'MVA_rating': 382.19, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '638'},
        {'fromNode': '578.0', 'toNode': '541.0', 'Resistance': 0.00165, 'Reactance': 0.06901, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.03658, 'shift': 0.0, 'Line_id': '639'},
        {'fromNode': '542.0', 'toNode': '543.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '640'},
        {'fromNode': '542.0', 'toNode': '546.0', 'Resistance': 0.00399, 'Reactance': 0.02901, 'Conductance': 0, 'Susceptance': 0.04808, 'MVA_rating': 525.91, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '641'},
        {'fromNode': '580.0', 'toNode': '542.0', 'Resistance': 0.00157, 'Reactance': 0.06798, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.01194, 'shift': 0.0, 'Line_id': '642'},
        {'fromNode': '581.0', 'toNode': '543.0', 'Resistance': 0.00165, 'Reactance': 0.06901, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 160.0, 'kV_base': 345.0, 'm': 1.01633, 'shift': 0.0, 'Line_id': '643'},
        {'fromNode': '544.0', 'toNode': '547.0', 'Resistance': 0.00358, 'Reactance': 0.04122, 'Conductance': 0, 'Susceptance': 1.1338, 'MVA_rating': 1154.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '644'},
        {'fromNode': '545.0', 'toNode': '547.0', 'Resistance': 0.00036, 'Reactance': 0.02519, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.07313, 'shift': 0.0, 'Line_id': '645'},
        {'fromNode': '545.0', 'toNode': '554.0', 'Resistance': 0.00919, 'Reactance': 0.06614, 'Conductance': 0, 'Susceptance': 0.10451, 'MVA_rating': 375.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '646'},
        {'fromNode': '546.0', 'toNode': '548.0', 'Resistance': 0.00036, 'Reactance': 0.02519, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.07313, 'shift': 0.0, 'Line_id': '647'},
        {'fromNode': '546.0', 'toNode': '555.0', 'Resistance': 0.00822, 'Reactance': 0.05767, 'Conductance': 0, 'Susceptance': 0.0982, 'MVA_rating': 400.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '648'},
        {'fromNode': '547.0', 'toNode': '548.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '649'},
        {'fromNode': '547.0', 'toNode': '549.0', 'Resistance': 0.00097, 'Reactance': 0.01052, 'Conductance': 0, 'Susceptance': 0.28395, 'MVA_rating': 970.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '650'},
        {'fromNode': '547.0', 'toNode': '559.0', 'Resistance': 0.00234, 'Reactance': 0.02677, 'Conductance': 0, 'Susceptance': 0.74653, 'MVA_rating': 1154.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '651'},
        {'fromNode': '548.0', 'toNode': '557.0', 'Resistance': 0.00137, 'Reactance': 0.01569, 'Conductance': 0, 'Susceptance': 0.43744, 'MVA_rating': 1154.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '652'},
        {'fromNode': '570.0', 'toNode': '548.0', 'Resistance': 0.00079, 'Reactance': 0.03678, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 426.0, 'kV_base': 500.0, 'm': 0.91667, 'shift': 0.0, 'Line_id': '653'},
        {'fromNode': '549.0', 'toNode': '556.0', 'Resistance': 0.00097, 'Reactance': 0.01052, 'Conductance': 0, 'Susceptance': 0.28395, 'MVA_rating': 970.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '654'},
        {'fromNode': '550.0', 'toNode': '551.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '655'},
        {'fromNode': '552.0', 'toNode': '553.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '656'},
        {'fromNode': '552.0', 'toNode': '554.0', 'Resistance': 0.00136, 'Reactance': 0.0099, 'Conductance': 0, 'Susceptance': 0.01586, 'MVA_rating': 400.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '657'},
        {'fromNode': '554.0', 'toNode': '555.0', 'Resistance': 2e-05, 'Reactance': 0.00021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 876.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '658'},
        {'fromNode': '554.0', 'toNode': '556.0', 'Resistance': 0.00075, 'Reactance': 0.02445, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 500.0, 'm': 1.039, 'shift': 0.0, 'Line_id': '659'},
        {'fromNode': '554.0', 'toNode': '558.0', 'Resistance': 0.0039, 'Reactance': 0.02847, 'Conductance': 0, 'Susceptance': 0.04498, 'MVA_rating': 462.42, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '660'},
        {'fromNode': '555.0', 'toNode': '557.0', 'Resistance': 0.0004, 'Reactance': 0.0253, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 500.0, 'kV_base': 500.0, 'm': 1.0457, 'shift': 0.0, 'Line_id': '661'},
        {'fromNode': '556.0', 'toNode': '557.0', 'Resistance': 1e-05, 'Reactance': 6e-05, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1593.0, 'kV_base': 500.0, 'm': 1, 'shift': 0, 'Line_id': '662'},
        {'fromNode': '561.0', 'toNode': '564.0', 'Resistance': 0.00686, 'Reactance': 0.02215, 'Conductance': 0, 'Susceptance': 0.00229, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '663'},
        {'fromNode': '561.0', 'toNode': '588.0', 'Resistance': 0.02058, 'Reactance': 0.06694, 'Conductance': 0, 'Susceptance': 0.00691, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '664'},
        {'fromNode': '562.0', 'toNode': '573.0', 'Resistance': 0.01446, 'Reactance': 0.04802, 'Conductance': 0, 'Susceptance': 0.00469, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '665'},
        {'fromNode': '563.0', 'toNode': '564.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '666'},
        {'fromNode': '563.0', 'toNode': '579.0', 'Resistance': 0.01372, 'Reactance': 0.04446, 'Conductance': 0, 'Susceptance': 0.00461, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '667'},
        {'fromNode': '565.0', 'toNode': '577.0', 'Resistance': 0.00355, 'Reactance': 0.01157, 'Conductance': 0, 'Susceptance': 0.00121, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '668'},
        {'fromNode': '566.0', 'toNode': '567.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '669'},
        {'fromNode': '566.0', 'toNode': '568.0', 'Resistance': 0.00788, 'Reactance': 0.05315, 'Conductance': 0, 'Susceptance': 0.00589, 'MVA_rating': 196.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '670'},
        {'fromNode': '567.0', 'toNode': '576.0', 'Resistance': 0.0044, 'Reactance': 0.03039, 'Conductance': 0, 'Susceptance': 0.00342, 'MVA_rating': 208.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '671'},
        {'fromNode': '568.0', 'toNode': '569.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 708.43, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '672'},
        {'fromNode': '570.0', 'toNode': '568.0', 'Resistance': 0.00101, 'Reactance': 0.04727, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 496.5, 'kV_base': 138.0, 'm': 0.91667, 'shift': 0.0, 'Line_id': '673'},
        {'fromNode': '568.0', 'toNode': '586.0', 'Resistance': 0.00098, 'Reactance': 0.00423, 'Conductance': 0, 'Susceptance': 0.02594, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '674'},
        {'fromNode': '571.0', 'toNode': '575.0', 'Resistance': 0.00256, 'Reactance': 0.02273, 'Conductance': 0, 'Susceptance': 0.00199, 'MVA_rating': 208.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '675'},
        {'fromNode': '571.0', 'toNode': '579.0', 'Resistance': 0.00174, 'Reactance': 0.01207, 'Conductance': 0, 'Susceptance': 0.00134, 'MVA_rating': 208.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '676'},
        {'fromNode': '572.0', 'toNode': '586.0', 'Resistance': 0.00017, 'Reactance': 0.00074, 'Conductance': 0, 'Susceptance': 0.00757, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '677'},
        {'fromNode': '574.0', 'toNode': '579.0', 'Resistance': 1e-05, 'Reactance': 0.00017, 'Conductance': 0, 'Susceptance': 0.00075, 'MVA_rating': 95.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '678'},
        {'fromNode': '575.0', 'toNode': '576.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '679'},
        {'fromNode': '577.0', 'toNode': '584.0', 'Resistance': 0.02595, 'Reactance': 0.08512, 'Conductance': 0, 'Susceptance': 0.0086, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '680'},
        {'fromNode': '579.0', 'toNode': '583.0', 'Resistance': 0.03529, 'Reactance': 0.13702, 'Conductance': 0, 'Susceptance': 0.01374, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '681'},
        {'fromNode': '580.0', 'toNode': '582.0', 'Resistance': 0.00488, 'Reactance': 0.01587, 'Conductance': 0, 'Susceptance': 0.00164, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '682'},
        {'fromNode': '580.0', 'toNode': '585.0', 'Resistance': 0.01289, 'Reactance': 0.04289, 'Conductance': 0, 'Susceptance': 0.0042, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '683'},
        {'fromNode': '582.0', 'toNode': '583.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '684'},
        {'fromNode': '584.0', 'toNode': '585.0', 'Resistance': 0.01628, 'Reactance': 0.05397, 'Conductance': 0, 'Susceptance': 0.00529, 'MVA_rating': 90.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '685'},
        {'fromNode': '587.0', 'toNode': '588.0', 'Resistance': 8e-05, 'Reactance': 0.00083, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 438.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '686'}
    ]
    lines_AC = pd.DataFrame(lines_AC_data)

    nodes_DC_data = [
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '1.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '2.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '3.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '4.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '5.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '6.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'P', 'Voltage_0': 1.0, 'Power_Gained': 0, 'Power_load': 0.0, 'kV_base': 320.0, 'Node_id': '7.0', 'Umin': 0.9, 'Umax': 1.1, 'x_coord': None, 'y_coord': None, 'PZ': None}
    ]
    nodes_DC = pd.DataFrame(nodes_DC_data)

    lines_DC_data = [
        {'fromNode': '1.0', 'toNode': '2.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '1'},
        {'fromNode': '1.0', 'toNode': '3.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '2'},
        {'fromNode': '2.0', 'toNode': '3.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '3'},
        {'fromNode': '1.0', 'toNode': '6.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '4'},
        {'fromNode': '6.0', 'toNode': '7.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '5'},
        {'fromNode': '7.0', 'toNode': '4.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '6'},
        {'fromNode': '4.0', 'toNode': '5.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '7'},
        {'fromNode': '5.0', 'toNode': '3.0', 'Resistance': 0.052, 'MW_rating': 1000.0, 'kV_base': 320.0, 'Length_km': 1, 'Mono_Bi_polar': 'b', 'Line_id': '8'}
    ]
    lines_DC = pd.DataFrame(lines_DC_data)

    Converters_ACDC_data = [
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '73.0', 'DC_node': '1.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '1', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '191.0', 'DC_node': '2.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '2', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '537.0', 'DC_node': '3.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '3', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '424.0', 'DC_node': '4.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '4', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '551.0', 'DC_node': '5.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '5', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '71.0', 'DC_node': '6.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '6', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1},
        {'AC_type': 'PQ', 'DC_type': 'P', 'AC_node': '8.0', 'DC_node': '7.0', 'P_AC': -0.6, 'Q_AC': -0.4, 'P_DC': -0.586274, 'T_R': 0.01, 'T_X': 0.01, 'PR_R': 0.01, 'PR_X': 0.01, 'Filter': 0.01, 'Droop': 0.005, 'AC_kV_base': 320.0, 'MVA_rating': 600.0, 'Nconverter': 1, 'pol': 1, 'Conv_id': '7', 'lossa': 1.103, 'lossb': 0.887, 'losscrect': 2.885, 'losscinv': 2.885, 'Ucmin': 0.9, 'Ucmax': 1.1}
    ]
    Converters_ACDC = pd.DataFrame(Converters_ACDC_data)

    
    # Create the grid
    [grid, res] = pyf.Create_grid_from_data(S_base, nodes_AC, lines_AC, nodes_DC, lines_DC, Converters_ACDC, data_in_pu=True)
    
    # Assign Price Zones to Nodes
    for index, row in nodes_AC.iterrows():
        node_name = nodes_AC.at[index, 'Node_id']
        price_zone = nodes_AC.at[index, 'PZ']
        ACDC = 'AC'
        if price_zone is not None:
            pyf.assign_nodeToPrice_Zone(grid, node_name, ACDC, price_zone)
    
    for index, row in nodes_DC.iterrows():
        node_name = nodes_DC.at[index, 'Node_id']
        price_zone = nodes_DC.at[index, 'PZ']
        ACDC = 'DC'
        if price_zone is not None:
            pyf.assign_nodeToPrice_Zone(grid, node_name, ACDC, price_zone)
    
    # Add Generators
    pyf.add_gen(grid, '4.0', '2', price_zone_link=False, lf=40.064589, qf=0.0, MWmax=281.74, MWmin=172.18, MVArmax=279.47, MVArmin=-349.34, PsetMW=226.96, QsetMVA=-34.935)
    pyf.add_gen(grid, '10.0', '4', price_zone_link=False, lf=48.118382, qf=0.0, MWmax=221.78999999999996, MWmin=97.5, MVArmax=20.51, MVArmin=-25.64, PsetMW=159.645, QsetMVA=-2.565)
    pyf.add_gen(grid, '15.0', '5', price_zone_link=False, lf=58.288367, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=105.0, MVArmin=-62.0, PsetMW=-10.5, QsetMVA=21.5)
    pyf.add_gen(grid, '15.0', '6', price_zone_link=False, lf=37.736514, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=105.0, MVArmin=-62.0, PsetMW=-10.5, QsetMVA=21.5)
    pyf.add_gen(grid, '15.0', '7', price_zone_link=False, lf=13.819973, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=130.0, MVArmin=-75.0, PsetMW=-10.5, QsetMVA=27.500000000000004)
    pyf.add_gen(grid, '16.0', '8', price_zone_link=False, lf=47.620376, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=160.0, MVArmin=-100.0, PsetMW=-10.5, QsetMVA=30.0)
    pyf.add_gen(grid, '19.0', '9', price_zone_link=False, lf=56.39552, qf=0.0, MWmax=55.00000000000001, MWmin=26.0, MVArmax=35.0, MVArmin=-165.0, PsetMW=40.5, QsetMVA=-65.0)
    pyf.add_gen(grid, '22.0', '11', price_zone_link=False, lf=57.684387, qf=0.0, MWmax=285.0, MWmin=0.0, MVArmax=184.0, MVArmin=-180.0, PsetMW=142.5, QsetMVA=2.0)
    pyf.add_gen(grid, '29.0', '12', price_zone_link=False, lf=118.024289, qf=0.0, MWmax=18.0, MWmin=0.0, MVArmax=178.0, MVArmin=-177.0, PsetMW=9.0, QsetMVA=0.5)
    pyf.add_gen(grid, '30.0', '13', price_zone_link=False, lf=52.837524, qf=0.0, MWmax=22.0, MWmin=0.0, MVArmax=179.0, MVArmin=-180.0, PsetMW=11.0, QsetMVA=-0.5)
    pyf.add_gen(grid, '37.0', '14', price_zone_link=False, lf=33.544263, qf=0.0, MWmax=296.0, MWmin=0.0, MVArmax=186.0, MVArmin=-168.0, PsetMW=148.0, QsetMVA=9.0)
    pyf.add_gen(grid, '44.0', '15', price_zone_link=False, lf=26.97166, qf=0.0, MWmax=269.0, MWmin=1.0, MVArmax=178.0, MVArmin=-169.0, PsetMW=135.0, QsetMVA=4.5)
    pyf.add_gen(grid, '45.0', '16', price_zone_link=False, lf=52.152489, qf=0.0, MWmax=15.0, MWmin=0.0, MVArmax=188.0, MVArmin=-166.0, PsetMW=7.5, QsetMVA=11.0)
    pyf.add_gen(grid, '49.0', '17', price_zone_link=False, lf=49.296905, qf=0.0, MWmax=285.0, MWmin=0.0, MVArmax=191.0, MVArmin=-180.0, PsetMW=142.5, QsetMVA=5.5)
    pyf.add_gen(grid, '54.0', '18', price_zone_link=False, lf=29.096904, qf=0.0, MWmax=258.0, MWmin=0.0, MVArmax=189.0, MVArmin=-169.0, PsetMW=129.0, QsetMVA=10.0)
    pyf.add_gen(grid, '58.0', '20', price_zone_link=False, lf=42.612273, qf=0.0, MWmax=297.0, MWmin=0.0, MVArmax=198.0, MVArmin=-180.0, PsetMW=148.5, QsetMVA=9.0)
    pyf.add_gen(grid, '60.0', '21', price_zone_link=False, lf=49.412887, qf=0.0, MWmax=278.0, MWmin=0.0, MVArmax=179.0, MVArmin=-184.0, PsetMW=139.0, QsetMVA=-2.5)
    pyf.add_gen(grid, '69.0', '23', price_zone_link=False, lf=58.563848, qf=0.0, MWmax=281.74, MWmin=172.18, MVArmax=282.12, MVArmin=-352.66, PsetMW=226.96, QsetMVA=-35.27)
    pyf.add_gen(grid, '75.0', '25', price_zone_link=False, lf=42.146702, qf=0.0, MWmax=221.78999999999996, MWmin=78.08, MVArmax=41.51, MVArmin=-51.89, PsetMW=149.935, QsetMVA=-5.19)
    pyf.add_gen(grid, '80.0', '26', price_zone_link=False, lf=20.599273, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=105.0, MVArmin=-62.0, PsetMW=-10.5, QsetMVA=21.5)
    pyf.add_gen(grid, '80.0', '27', price_zone_link=False, lf=37.607163, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=105.0, MVArmin=-62.0, PsetMW=-10.5, QsetMVA=21.5)
    pyf.add_gen(grid, '80.0', '28', price_zone_link=False, lf=53.297971, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=130.0, MVArmin=-75.0, PsetMW=-10.5, QsetMVA=27.500000000000004)
    pyf.add_gen(grid, '81.0', '29', price_zone_link=False, lf=33.176582, qf=0.0, MWmax=179.0, MWmin=-200.0, MVArmax=160.0, MVArmin=-100.0, PsetMW=-10.5, QsetMVA=30.0)
    pyf.add_gen(grid, '84.0', '30', price_zone_link=False, lf=55.036018, qf=0.0, MWmax=55.00000000000001, MWmin=26.0, MVArmax=35.0, MVArmin=-175.0, PsetMW=40.5, QsetMVA=-70.0)
    pyf.add_gen(grid, '87.0', '32', price_zone_link=False, lf=54.880885, qf=0.0, MWmax=256.0, MWmin=0.0, MVArmax=197.0, MVArmin=-178.0, PsetMW=128.0, QsetMVA=9.5)
    pyf.add_gen(grid, '94.0', '33', price_zone_link=False, lf=43.052651, qf=0.0, MWmax=18.0, MWmin=0.0, MVArmax=179.0, MVArmin=-170.0, PsetMW=9.0, QsetMVA=4.5)
    pyf.add_gen(grid, '95.0', '34', price_zone_link=False, lf=153.861358, qf=0.0, MWmax=22.0, MWmin=0.0, MVArmax=199.0, MVArmin=-184.0, PsetMW=11.0, QsetMVA=7.5)
    pyf.add_gen(grid, '102.0', '35', price_zone_link=False, lf=52.885636, qf=0.0, MWmax=268.0, MWmin=0.0, MVArmax=182.0, MVArmin=-166.0, PsetMW=134.0, QsetMVA=8.0)
    pyf.add_gen(grid, '109.0', '36', price_zone_link=False, lf=33.376343, qf=0.0, MWmax=268.0, MWmin=1.0, MVArmax=191.0, MVArmin=-184.0, PsetMW=134.5, QsetMVA=3.5000000000000004)
    pyf.add_gen(grid, '110.0', '37', price_zone_link=False, lf=110.993979, qf=0.0, MWmax=15.0, MWmin=0.0, MVArmax=191.0, MVArmin=-176.0, PsetMW=7.5, QsetMVA=7.5)
    pyf.add_gen(grid, '114.0', '38', price_zone_link=False, lf=8.625968, qf=0.0, MWmax=298.0, MWmin=0.0, MVArmax=183.0, MVArmin=-171.0, PsetMW=149.0, QsetMVA=6.0)
    pyf.add_gen(grid, '119.0', '39', price_zone_link=False, lf=44.162061, qf=0.0, MWmax=283.0, MWmin=0.0, MVArmax=192.0, MVArmin=-167.0, PsetMW=141.5, QsetMVA=12.5)
    pyf.add_gen(grid, '123.0', '41', price_zone_link=False, lf=63.307485, qf=0.0, MWmax=264.0, MWmin=0.0, MVArmax=200.0, MVArmin=-172.0, PsetMW=132.0, QsetMVA=14.000000000000002)
    pyf.add_gen(grid, '125.0', '42', price_zone_link=False, lf=42.682479, qf=0.0, MWmax=252.99999999999997, MWmin=0.0, MVArmax=196.0, MVArmin=-167.0, PsetMW=126.49999999999999, QsetMVA=14.499999999999998)
    pyf.add_gen(grid, '133.0', '44', price_zone_link=False, lf=40.384949, qf=0.0, MWmax=393.52, MWmin=60.69, MVArmax=202.93, MVArmin=-202.93, PsetMW=227.105, QsetMVA=0.0)
    pyf.add_gen(grid, '138.0', '46', price_zone_link=False, lf=63.07322, qf=0.0, MWmax=33.73, MWmin=10.47, MVArmax=12.48, MVArmin=-12.48, PsetMW=22.1, QsetMVA=0.0)
    pyf.add_gen(grid, '152.0', '49', price_zone_link=False, lf=41.178786, qf=0.0, MWmax=41.0, MWmin=0.0, MVArmax=192.0, MVArmin=-171.0, PsetMW=20.5, QsetMVA=10.5)
    pyf.add_gen(grid, '153.0', '50', price_zone_link=False, lf=120.513617, qf=0.0, MWmax=25.0, MWmin=0.0, MVArmax=23.48, MVArmin=-178.0, PsetMW=12.5, QsetMVA=-77.26)
    pyf.add_gen(grid, '153.0', '51', price_zone_link=False, lf=154.618982, qf=0.0, MWmax=35.0, MWmin=0.0, MVArmax=32.44, MVArmin=-170.0, PsetMW=17.5, QsetMVA=-68.78)
    pyf.add_gen(grid, '172.0', '58', price_zone_link=False, lf=50.151077, qf=0.0, MWmax=276.0, MWmin=0.0, MVArmax=181.0, MVArmin=-184.0, PsetMW=138.0, QsetMVA=-1.5)
    pyf.add_gen(grid, '178.0', '59', price_zone_link=False, lf=53.25994, qf=0.0, MWmax=18.0, MWmin=0.0, MVArmax=185.0, MVArmin=-183.0, PsetMW=9.0, QsetMVA=1.0)
    pyf.add_gen(grid, '183.0', '61', price_zone_link=False, lf=25.090289, qf=0.0, MWmax=204.99999999999997, MWmin=135.0, MVArmax=138.75, MVArmin=-21.0, PsetMW=170.0, QsetMVA=58.875)
    pyf.add_gen(grid, '184.0', '64', price_zone_link=False, lf=40.400123, qf=0.0, MWmax=231.99999999999997, MWmin=100.0, MVArmax=119.0, MVArmin=-31.0, PsetMW=166.0, QsetMVA=44.0)
    pyf.add_gen(grid, '185.0', '65', price_zone_link=False, lf=51.622451, qf=0.0, MWmax=250.43, MWmin=90.0, MVArmax=127.49999999999999, MVArmin=-15.0, PsetMW=170.215, QsetMVA=56.25)
    pyf.add_gen(grid, '186.0', '67', price_zone_link=False, lf=30.330811, qf=0.0, MWmax=250.0, MWmin=90.0, MVArmax=127.49999999999999, MVArmin=-15.0, PsetMW=170.0, QsetMVA=56.25)
    pyf.add_gen(grid, '187.0', '68', price_zone_link=False, lf=143.785, qf=0.0, MWmax=1241.44, MWmin=607.86, MVArmax=72.29, MVArmin=-90.36, PsetMW=924.65, QsetMVA=-9.035)
    pyf.add_gen(grid, '199.0', '69', price_zone_link=False, lf=40.323533, qf=0.0, MWmax=222.00000000000003, MWmin=90.0, MVArmax=117.0, MVArmin=-19.0, PsetMW=156.0, QsetMVA=49.0)
    pyf.add_gen(grid, '205.0', '70', price_zone_link=False, lf=22.246249, qf=0.0, MWmax=297.0, MWmin=2.08, MVArmax=193.0, MVArmin=-171.0, PsetMW=149.54, QsetMVA=11.0)
    pyf.add_gen(grid, '210.0', '73', price_zone_link=False, lf=34.013952, qf=0.0, MWmax=258.0, MWmin=2.5, MVArmax=194.0, MVArmin=-168.0, PsetMW=130.25, QsetMVA=13.0)
    pyf.add_gen(grid, '222.0', '74', price_zone_link=False, lf=46.514194, qf=0.0, MWmax=26.0, MWmin=0.0, MVArmax=192.0, MVArmin=-177.0, PsetMW=13.0, QsetMVA=7.5)
    pyf.add_gen(grid, '226.0', '76', price_zone_link=False, lf=49.146742, qf=0.0, MWmax=261.0, MWmin=0.0, MVArmax=198.0, MVArmin=-171.0, PsetMW=130.5, QsetMVA=13.5)
    pyf.add_gen(grid, '227.0', '77', price_zone_link=False, lf=27.095592, qf=0.0, MWmax=50.0, MWmin=0.0, MVArmax=188.0, MVArmin=-172.0, PsetMW=25.0, QsetMVA=8.0)
    pyf.add_gen(grid, '230.0', '78', price_zone_link=False, lf=51.369408, qf=0.0, MWmax=254.0, MWmin=0.0, MVArmax=179.0, MVArmin=-183.0, PsetMW=127.0, QsetMVA=-2.0)
    pyf.add_gen(grid, '271.0', '82', price_zone_link=False, lf=57.188964, qf=0.0, MWmax=50.41, MWmin=16.56, MVArmax=13.209999999999999, MVArmin=-16.51, PsetMW=33.485, QsetMVA=-1.6500000000000001)
    pyf.add_gen(grid, '275.0', '83', price_zone_link=False, lf=34.036118, qf=0.0, MWmax=146.28, MWmin=89.84, MVArmax=27.49, MVArmin=-27.49, PsetMW=118.06, QsetMVA=0.0)
    pyf.add_gen(grid, '278.0', '84', price_zone_link=False, lf=61.591476, qf=0.0, MWmax=26.700000000000003, MWmin=9.75, MVArmax=24.5, MVArmin=-24.5, PsetMW=18.225, QsetMVA=0.0)
    pyf.add_gen(grid, '281.0', '87', price_zone_link=False, lf=42.642911, qf=0.0, MWmax=64.19, MWmin=19.3, MVArmax=198.0, MVArmin=-8.55, PsetMW=41.745, QsetMVA=94.725)
    pyf.add_gen(grid, '296.0', '88', price_zone_link=False, lf=7.493344, qf=0.0, MWmax=825.3, MWmin=334.33, MVArmax=101.0, MVArmin=-99.0, PsetMW=579.815, QsetMVA=1.0)
    pyf.add_gen(grid, '296.0', '89', price_zone_link=False, lf=41.48542, qf=0.0, MWmax=846.64, MWmin=343.14, MVArmax=101.0, MVArmin=-99.0, PsetMW=594.89, QsetMVA=1.0)
    pyf.add_gen(grid, '326.0', '94', price_zone_link=False, lf=63.292777, qf=0.0, MWmax=41.4, MWmin=22.0, MVArmax=180.0, MVArmin=-4.69, PsetMW=31.7, QsetMVA=87.655)
    pyf.add_gen(grid, '361.0', '103', price_zone_link=False, lf=43.651445, qf=0.0, MWmax=280.0, MWmin=0.0, MVArmax=199.0, MVArmin=-180.0, PsetMW=140.0, QsetMVA=9.5)
    pyf.add_gen(grid, '361.0', '104', price_zone_link=False, lf=120.987489, qf=0.0, MWmax=19.75, MWmin=4.75, MVArmax=198.0, MVArmin=-2.54, PsetMW=12.25, QsetMVA=97.73)
    pyf.add_gen(grid, '362.0', '106', price_zone_link=False, lf=55.499552, qf=0.0, MWmax=92.8, MWmin=64.83, MVArmax=197.0, MVArmin=-7.37, PsetMW=78.815, QsetMVA=94.815)
    pyf.add_gen(grid, '365.0', '110', price_zone_link=False, lf=56.112153, qf=0.0, MWmax=273.0, MWmin=5.13, MVArmax=195.0, MVArmin=-1.52, PsetMW=139.065, QsetMVA=96.74)
    pyf.add_gen(grid, '366.0', '111', price_zone_link=False, lf=52.581937, qf=0.0, MWmax=196.9, MWmin=131.27, MVArmax=26.69, MVArmin=-26.69, PsetMW=164.085, QsetMVA=0.0)
    pyf.add_gen(grid, '368.0', '112', price_zone_link=False, lf=43.905001, qf=0.0, MWmax=87.21, MWmin=60.17, MVArmax=199.0, MVArmin=-2.66, PsetMW=73.69, QsetMVA=98.17)
    pyf.add_gen(grid, '385.0', '119', price_zone_link=False, lf=51.918332, qf=0.0, MWmax=47.26, MWmin=26.890000000000004, MVArmax=14.790000000000001, MVArmin=-14.790000000000001, PsetMW=37.075, QsetMVA=0.0)
    pyf.add_gen(grid, '390.0', '122', price_zone_link=False, lf=33.388227, qf=0.0, MWmax=697.71, MWmin=465.14, MVArmax=33.19, MVArmin=-33.19, PsetMW=581.425, QsetMVA=0.0)
    pyf.add_gen(grid, '395.0', '123', price_zone_link=False, lf=47.99747, qf=0.0, MWmax=321.61, MWmin=214.41, MVArmax=34.72, MVArmin=-34.72, PsetMW=268.01, QsetMVA=0.0)
    pyf.add_gen(grid, '411.0', '129', price_zone_link=False, lf=30.733464, qf=0.0, MWmax=286.0, MWmin=0.0, MVArmax=190.0, MVArmin=-172.0, PsetMW=143.0, QsetMVA=9.0)
    pyf.add_gen(grid, '414.0', '130', price_zone_link=False, lf=26.034783, qf=0.0, MWmax=431.17, MWmin=287.44, MVArmax=53.15, MVArmin=-53.15, PsetMW=359.305, QsetMVA=0.0)
    pyf.add_gen(grid, '422.0', '131', price_zone_link=False, lf=39.923184, qf=0.0, MWmax=65.27, MWmin=35.69, MVArmax=18.48, MVArmin=-23.1, PsetMW=50.47999999999999, QsetMVA=-2.31)
    pyf.add_gen(grid, '423.0', '132', price_zone_link=False, lf=38.147179, qf=0.0, MWmax=143.75, MWmin=70.0, MVArmax=73.0, MVArmin=-1.0, PsetMW=106.87500000000001, QsetMVA=36.0)
    pyf.add_gen(grid, '424.0', '133', price_zone_link=False, lf=31.559684, qf=0.0, MWmax=143.75, MWmin=70.0, MVArmax=73.0, MVArmin=-1.0, PsetMW=106.87500000000001, QsetMVA=36.0)
    pyf.add_gen(grid, '440.0', '135', price_zone_link=False, lf=57.049979, qf=0.0, MWmax=137.5, MWmin=32.0, MVArmax=110.00000000000001, MVArmin=-40.0, PsetMW=84.75, QsetMVA=35.0)
    pyf.add_gen(grid, '455.0', '140', price_zone_link=False, lf=114.489771, qf=0.0, MWmax=31.25, MWmin=10.0, MVArmax=179.0, MVArmin=-168.0, PsetMW=20.625, QsetMVA=5.5)
    pyf.add_gen(grid, '455.0', '141', price_zone_link=False, lf=38.56928, qf=0.0, MWmax=25.0, MWmin=10.0, MVArmax=192.0, MVArmin=-182.0, PsetMW=17.5, QsetMVA=5.0)
    pyf.add_gen(grid, '475.0', '144', price_zone_link=False, lf=19.090609, qf=0.0, MWmax=120.0, MWmin=70.0, MVArmax=73.0, MVArmin=-1.0, PsetMW=95.0, QsetMVA=36.0)
    pyf.add_gen(grid, '475.0', '145', price_zone_link=False, lf=21.358502, qf=0.0, MWmax=120.0, MWmin=58.32999999999999, MVArmax=73.0, MVArmin=-1.0, PsetMW=89.165, QsetMVA=36.0)
    pyf.add_gen(grid, '476.0', '146', price_zone_link=False, lf=21.594677, qf=0.0, MWmax=254.99, MWmin=100.12, MVArmax=73.0, MVArmin=-1.25, PsetMW=177.555, QsetMVA=35.875)
    pyf.add_gen(grid, '514.0', '150', price_zone_link=False, lf=47.463637, qf=0.0, MWmax=79.66, MWmin=2.64, MVArmax=87.18, MVArmin=-87.18, PsetMW=41.15, QsetMVA=0.0)
    pyf.add_gen(grid, '545.0', '153', price_zone_link=False, lf=42.525074, qf=0.0, MWmax=431.25, MWmin=240.0, MVArmax=190.0, MVArmin=-23.75, PsetMW=335.625, QsetMVA=83.125)
    pyf.add_gen(grid, '546.0', '154', price_zone_link=False, lf=47.717223, qf=0.0, MWmax=462.5, MWmin=240.0, MVArmax=190.0, MVArmin=-23.75, PsetMW=351.25, QsetMVA=83.125)
    pyf.add_gen(grid, '546.0', '155', price_zone_link=False, lf=34.681788, qf=0.0, MWmax=370.0, MWmin=240.0, MVArmax=190.0, MVArmin=-21.84, PsetMW=305.0, QsetMVA=84.08)
    pyf.add_gen(grid, '547.0', '156', price_zone_link=False, lf=50.370228, qf=0.0, MWmax=1090.99, MWmin=353.78, MVArmax=190.0, MVArmin=-19.0, PsetMW=722.385, QsetMVA=85.5)
    pyf.add_gen(grid, '547.0', '157', price_zone_link=False, lf=17.068923, qf=0.0, MWmax=1090.99, MWmin=353.78, MVArmax=190.0, MVArmin=-19.0, PsetMW=722.385, QsetMVA=85.5)
    pyf.add_gen(grid, '547.0', '158', price_zone_link=False, lf=26.291695, qf=0.0, MWmax=1090.99, MWmin=353.78, MVArmax=190.0, MVArmin=-19.0, PsetMW=722.385, QsetMVA=85.5)
    pyf.add_gen(grid, '548.0', '159', price_zone_link=False, lf=17.800404, qf=0.0, MWmax=462.5, MWmin=240.0, MVArmax=227.40999999999997, MVArmin=-19.0, PsetMW=351.25, QsetMVA=104.205)
    pyf.add_gen(grid, '548.0', '160', price_zone_link=False, lf=23.872239, qf=0.0, MWmax=425.0, MWmin=240.0, MVArmax=190.0, MVArmin=-19.0, PsetMW=332.5, QsetMVA=85.5)
    pyf.add_gen(grid, '548.0', '161', price_zone_link=False, lf=32.137621, qf=0.0, MWmax=425.0, MWmin=240.0, MVArmax=190.0, MVArmin=-19.0, PsetMW=332.5, QsetMVA=85.5)
    pyf.add_gen(grid, '557.0', '162', price_zone_link=False, lf=35.386296, qf=0.0, MWmax=156.95, MWmin=83.9, MVArmax=133.54, MVArmin=-133.54, PsetMW=120.42500000000001, QsetMVA=0.0)
    pyf.add_gen(grid, '560.0', '163', price_zone_link=False, lf=38.893759, qf=0.0, MWmax=137.5, MWmin=91.67, MVArmax=106.25, MVArmin=-15.0, PsetMW=114.58500000000001, QsetMVA=45.625)
    pyf.add_gen(grid, '570.0', '165', price_zone_link=False, lf=40.314179, qf=0.0, MWmax=437.5, MWmin=240.0, MVArmax=237.5, MVArmin=-19.0, PsetMW=338.75, QsetMVA=109.25)
    pyf.add_gen(grid, '574.0', '166', price_zone_link=False, lf=126.957574, qf=0.0, MWmax=30.0, MWmin=0.0, MVArmax=179.0, MVArmin=-179.0, PsetMW=15.0, QsetMVA=0.0)
    pyf.add_gen(grid, '580.0', '167', price_zone_link=False, lf=18.663234, qf=0.0, MWmax=261.0, MWmin=0.0, MVArmax=178.0, MVArmin=-182.0, PsetMW=130.5, QsetMVA=-2.0)
    
    
    # Add Renewable Source Zones

    
    # Add Renewable Sources

    
    # Return the grid
    return grid,res
